﻿using System;
using LoongCAD.Util;
using LoongCAD.UI.Component;

namespace BeamColumnWeld107
{
    public class ParamsHandler : LoongCAD.STC.Plugins.ParamsHandlerBase
    {
        /// <summary>
        /// 属性空值全局总表
        /// </summary>
        protected static DObject g_emptyValues = null;

        /// <summary>
        /// 属性内置默认值全局总表
        /// </summary>
        protected static DObject g_builtinDefaults = null;

        /// <summary>
        /// 默认构造函数
        /// </summary>
        /// <remarks><b>此方法需要复制到其他组件代码中使用</b></remarks>
        public ParamsHandler()
            : base(ParamsHandler.EmptyValues, ParamsHandler.BuiltinDefautls)
        {

        }
        /// <summary>
        /// 组件公用属性空值表
        /// </summary>
        public static DObject EmptyValues
        {
            get
            {
                if (g_emptyValues == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_emptyValues;
            }
        }
        /// <summary>
        /// 组件公用内置默认值接口
        /// </summary>
        public static DObject BuiltinDefautls
        {
            get
            {
                if (g_builtinDefaults == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_builtinDefaults;
            }
        }
        /// <summary>
        /// 初始化默认值
        /// </summary>
        /// <param name="defaultConfig">从默认值配置加载的默认值</param>
        public override LoongCAD.Util.DObject InitializeDefaultValues(DObject defaultConfig)
        {
            m_defaultValues = new DObject();
            m_defaultValues.CopyFrom(defaultConfig);

            var emptyValues = ParamsHandler.EmptyValues;
            var builtinDefaults = ParamsHandler.BuiltinDefautls;
            if (emptyValues == null || builtinDefaults == null)
            {
                Console.WriteLine("ParamsHandler.InitializeDefaultValues: [Warning] EmptyValue/BuiltinDefaults is null!");
                return m_defaultValues;
            }

            this.TryCompleteDObjectValues(m_defaultValues, builtinDefaults, emptyValues);
            return m_defaultValues;
        }
        /// <summary>
        /// 初始化组件所有实例公用的属性空值、属性内置默认值
        /// </summary>
        public static void IntializeEmptyDefaults()
        {
            g_emptyValues = new DObject();
            g_builtinDefaults = new DObject();

            //图形页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_PARTS_GAP, double.MaxValue, 2.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_BEAM_CUT_TYPE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_BEAM_CUT_X, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_BEAM_CUT_Y, double.MaxValue, 0.0);

            //加劲肋页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_TOP_NS_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 10.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_TOP_FS_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_BOTTOM_NS_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_BOTTOM_FS_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_WEB_GAP, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_TOP_NS_OFFSET, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_TOP_FS_OFFSET, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_BOTTOM_NS_OFFSET, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_BOTTOM_FS_OFFSET, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_DIR, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_ENABLED, 0, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CHAMFER_ENABLED, 0, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_INSIDE_CHAMFER_TYPE, 0, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_INSIDE_CHAMFER_X, double.MaxValue, 10.5);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_INSIDE_CHAMFER_Y, double.MaxValue, 10.5);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CHAMFER_TYPE, 0, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CHAMFER_X, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CHAMFER_Y, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_FLANGE_GAP_X, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_FLANGE_GAP_Y, double.MaxValue, 0.0);

            //梁切割
            g_emptyValues.CopyFrom(BeamCutPage.EmptyValues);
            g_builtinDefaults.CopyFrom(BeamCutPage.BuiltinDefaults);

            //槽口页
            g_emptyValues.CopyFrom(BeamNotchPage.EmptyValues);
            g_builtinDefaults.CopyFrom(BeamNotchPage.BuiltinDefaults);

            //双板页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_PLATE_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 10.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_CREATION_MODE, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_PLATE_EDGE_SHAPE, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_PLATE_CUT, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_PLATE_BEVEL_ANGLE, double.MaxValue, 45.0); // In degree
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_DIAMETER, double.MaxValue, 16.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_SLOT_WIDTH, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_SLOT_HEIGHT, double.MaxValue, 16.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_TOP_MARGIN, double.MaxValue, 48.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_BOTTOM_MARGIN, double.MaxValue, 48.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_LEFT_MARGIN, double.MaxValue, 48.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_RIGHT_MARGIN, double.MaxValue, 48.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_FLANGE_GAP, double.MaxValue, double.MaxValue); //auto
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_BOTTOM_GAP, double.MaxValue, 48.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_HORI_COUNT, int.MaxValue, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_VERT_COUNT, int.MaxValue, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_HORI_STRIDE, new double[] { }, new double[] { 48.0 });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_VERT_STRIDE, new double[] { }, new double[] { 48.0 });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_PLATE_CUT_LENGTH, double.MaxValue, 10.5);

            // 焊缝
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length - 1; ++i)
            {
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WELD_PROPS_LIST[i], WeldPropertyPanel.EmptyValue, new DObject {
                    { WeldPropertyPanel.ParamNames.SizeUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeUp, 5 },
                    { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                    { WeldPropertyPanel.ParamNames.SizeDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeDown, 5 },
                    { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                    { WeldPropertyPanel.ParamNames.EdgeAround, 1 }, // Edge
                    { WeldPropertyPanel.ParamNames.MadeType, 2 }, // Workshop
                    { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                    { WeldPropertyPanel.ParamNames.RefText, "" }
                });
            }
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WELD_PROPS_LIST[7], WeldPropertyPanel.EmptyValue, new DObject {
                    { WeldPropertyPanel.ParamNames.SizeUp, 10.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeUp, 5 },
                    { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                    { WeldPropertyPanel.ParamNames.SizeDown, 10.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeDown, 5 },
                    { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                    { WeldPropertyPanel.ParamNames.EdgeAround, 1 }, // Edge
                    { WeldPropertyPanel.ParamNames.MadeType, 2 }, // Workshop
                    { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                    { WeldPropertyPanel.ParamNames.RefText, "" }
                });
        }
    }
}
