﻿using System;
using System.Windows.Forms;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;
using System.Collections.Generic;

namespace BeamColumnWeld107
{
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {
        private DObject m_rootUIProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;
        private bool m_stiffChamfer = true;
        private bool m_stiffInsideX = true;
        private bool m_stiffInsideY = true;
        private bool m_stiffX = true;
        private bool m_stiffY = true;
        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostIntialize();
        }
        public override ComponentPlugin ReferencePlugin => m_refPlugin;
        private void OnPostIntialize()
        {
            // 设置窗口标题
            this.Text = "梁柱焊接节点(107)";
            foreach (TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }

            //初始化图片选项卡
            this.cbo_beam_cut_type.ClearItems();
            this.cbo_beam_cut_type.AddIntegerItems(0, 0, 5);
            this.cbo_beam_cut_type.UpdateItems();

            this.cbo_stiff_dir.ClearItems();
            this.cbo_stiff_dir.AddIntegerItems(0, 0, 3);
            this.cbo_stiff_dir.UpdateItems();

            this.cbo_stiff_enabled.ClearItems();
            this.cbo_stiff_enabled.AddIntegerItems(0, 0, 3);
            this.cbo_stiff_enabled.UpdateItems();

            this.cbo_stiff_chamfer_enabled.ClearItems();
            this.cbo_stiff_chamfer_enabled.AddIntegerItems(0, 0, 3);
            this.cbo_stiff_chamfer_enabled.UpdateItems();

            this.cbo_double_plate_cut.ClearItems();
            this.cbo_double_plate_cut.AddIntegerItems(0, 0, 3);
            this.cbo_double_plate_cut.UpdateItems();
        }
        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null)
            {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }
        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked)
            {
                retProps.CopyFrom(m_rootUIProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }
        /// <summary>
        /// 当前组件需要重写此方法，并刷新螺栓基础面板
        /// </summary>
        /// <param name="root"></param>
        public override void ClearUI(Control root = null)
        {
            base.ClearUI(root);
        }
        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }
        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_rootUIProps.CopyFrom(srcProps);
        }
        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }
        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_rootUIProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_rootUIProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_rootUIProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_rootUIProps;
            return comp.UIProps;
        }
        private void btn_weld_props_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_rootUIProps, null);
        }
        private void cbo_beam_cut_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            bool enabledX = false;
            bool enabledY = false;
            if(cbo_beam_cut_type.SelectedIndex == 2)
            {
                enabledX = true;
                enabledY = true;
            }else if(cbo_beam_cut_type.SelectedIndex > 2)
            {
                enabledX = true;
            }
            tb_beam_cut_x.Enabled = enabledX;
            act_beam_cut_x.Enabled = enabledX;
            tb_beam_cut_y.Enabled = enabledY;
            act_beam_cut_y.Enabled = enabledY;
        }
        private void cbo_stiff_chamfer_enabled_SelectedIndexChanged(object sender, EventArgs e)
        {
            m_stiffChamfer = true;
            if (cbo_stiff_chamfer_enabled.SelectedIndex == 1)
                m_stiffChamfer = false;
            cbo_stiff_inside_chamfer_type.Enabled = m_stiffChamfer;
            act_stiff_inside_chamfer_type.Enabled = m_stiffChamfer;
            cbo_stiff_chamfer_type.Enabled = m_stiffChamfer;
            act_stiff_chamfer_type.Enabled = m_stiffChamfer;
            AdjustChamferLengthEnabled();
            SetChamferLengthEnabled();
            AdjustInsideChamferLengthEnabled();
            SetInsideChamferLengthEnabled();
        }
        private void cbo_stiff_chamfer_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            AdjustChamferLengthEnabled();
            SetChamferLengthEnabled();
        }
        private void cbo_stiff_inside_chamfer_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            AdjustInsideChamferLengthEnabled();
            SetInsideChamferLengthEnabled();
        }
        private void AdjustInsideChamferLengthEnabled()
        {
            if (!m_stiffChamfer)
            {
                m_stiffInsideX = false;
                m_stiffInsideY = false;
                return;
            }
            m_stiffInsideX = true;
            m_stiffInsideY = true;
            if (cbo_stiff_inside_chamfer_type.SelectedIndex == 1)
            {
                m_stiffInsideX = false;
                m_stiffInsideY = false;
            }
            if (cbo_stiff_inside_chamfer_type.SelectedIndex > 2)
                m_stiffInsideY = false;
        }
        private void SetInsideChamferLengthEnabled()
        {
            tb_stiff_inside_chamfer_x.Enabled = m_stiffInsideX;
            act_stiff_inside_chamfer_x.Enabled = m_stiffInsideX;
            tb_stiff_inside_chamfer_y.Enabled = m_stiffInsideY;
            act_stiff_inside_chamfer_y.Enabled = m_stiffInsideY;
        }
        private void AdjustChamferLengthEnabled()
        {
            if (!m_stiffChamfer)
            {
                m_stiffX = false;
                m_stiffY = false;
                return;
            }
            m_stiffX = true;
            m_stiffY = true;
            if (cbo_stiff_chamfer_type.SelectedIndex == 1)
            {
                m_stiffX = false;
                m_stiffY = false;
            }
            if (cbo_stiff_chamfer_type.SelectedIndex > 2)
                m_stiffY = false;
        }
        private void SetChamferLengthEnabled()
        {
            tb_stiff_chamfer_x.Enabled = m_stiffX;
            act_stiff_chamfer_x.Enabled = m_stiffX;
            tb_stiff_chamfer_y.Enabled = m_stiffY;
            act_stiff_chamfer_y.Enabled = m_stiffY;
        }
    }
}
