﻿using System;
using LoongCAD.STC.Object3D;
using LoongCAD.STC;
using LoongCAD.Geometry;

namespace BeamColumnWeld107
{
    public class PlateTools
    {
        /// <summary>
        /// 设置板折角
        /// </summary>
        public static void SetChamfer(ChamferCtrlPoint point, int type, double lenX, double lenY, PartBase retPlate)
        {
            if (retPlate == null)
                return;
            Define.CornerCutType chamferType;
            switch (type)
            {
                case 1:
                    chamferType = Define.CornerCutType.CORNER_CUT_NONE;
                    break;
                case 3:
                    chamferType = Define.CornerCutType.CORNER_CUT_ROUND;
                    break;
                case 4:
                    chamferType = Define.CornerCutType.CORNER_CUT_INV_ROUND;
                    break;
                case 0:
                default:
                    chamferType = Define.CornerCutType.CORNER_CUT_LINE;
                    break;
            }
            if (point != null)
            {
                point.CutX = lenX;
                point.CutY = lenY;
                point.CutType = chamferType;
                point.Flush();
                retPlate.Flush();
            }
        }
        /// <summary>
        /// 在给定平面内旋转一个三维向量指定的角度。
        /// </summary>
        /// <param name="vector">要旋转的向量。</param>
        /// <param name="planeNormal">定义旋转平面的法线向量。</param>
        /// <param name="angleDegrees">向量在平面内旋转的角度，单位为度。</param>
        /// <returns>旋转后的向量。</returns>
        public static Vec3 RotateVectorInPlane(Vec3 vector, Vec3 planeNormal, double angleDegrees)
        {
            // 将平面法线标准化，确保它是单位向量
            Vec3 normalizedPlaneNormal = planeNormal.Normalize();

            // 将角度从度转换为弧度
            double angleRadians = Math.PI / 180 * angleDegrees;

            // 根据罗德里格斯旋转公式计算旋转的各个分量
            Vec3 rotatedVector = vector * Math.Cos(angleRadians) // 向量与余弦角度的乘积
                                    + normalizedPlaneNormal.Cross(vector) * Math.Sin(angleRadians) // 法线与向量的叉乘与正弦角度的乘积
                                    + normalizedPlaneNormal * normalizedPlaneNormal.Dot(vector) * (1 - Math.Cos(angleRadians)); // 法线与法线与向量点乘的乘积，再乘以1减去余弦角度
            return rotatedVector; // 返回旋转后的向量
        }
    }
}
