﻿using System;
using LoongCAD.Util;
using LoongCAD.UI.Component;

namespace BeamSideEndPlate209
{
    public class ParamsHandler : LoongCAD.STC.Plugins.ParamsHandlerBase
    {
        /// <summary>
        /// 属性空值全局总表
        /// </summary>
        protected static DObject g_emptyValues = null;

        /// <summary>
        /// 属性内置默认值全局总表
        /// </summary>
        protected static DObject g_builtinDefaults = null;

        /// <summary>
        /// 默认构造函数
        /// </summary>
        /// <remarks><b>此方法需要复制到其他组件代码中使用</b></remarks>
        public ParamsHandler()
            : base(ParamsHandler.EmptyValues, ParamsHandler.BuiltinDefautls)
        {

        }

        /// <summary>
        /// 组件公用属性空值表
        /// </summary>
        public static DObject EmptyValues
        {
            get
            {
                if (g_emptyValues == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_emptyValues;
            }
        }

        /// <summary>
        /// 组件公用内置默认值接口
        /// </summary>
        public static DObject BuiltinDefautls
        {
            get
            {
                if (g_builtinDefaults == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_builtinDefaults;
            }
        }

        /// <summary>
        /// 初始化默认值
        /// </summary>
        /// <param name="defaultConfig">从默认值配置加载的默认值</param>
        public override LoongCAD.Util.DObject InitializeDefaultValues(DObject defaultConfig)
        {
            m_defaultValues = new DObject();
            m_defaultValues.CopyFrom(defaultConfig);

            var emptyValues = ParamsHandler.EmptyValues;
            var builtinDefaults = ParamsHandler.BuiltinDefautls;
            if (emptyValues == null || builtinDefaults == null)
            {
                Console.WriteLine("ParamsHandler.InitializeDefaultValues: [Warning] EmptyValue/BuiltinDefaults is null!");
                return m_defaultValues;
            }

            this.TryCompleteDObjectValues(m_defaultValues, builtinDefaults, emptyValues);
            return m_defaultValues;
        }

        /// <summary>
        /// 初始化组件所有实例公用的属性空值、属性内置默认值
        /// </summary>
        public static void IntializeEmptyDefaults()
        {
            g_emptyValues = new DObject();
            g_builtinDefaults = new DObject();

            //图形页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_FRONT_TO_MAIN_BOUND_UP_GAP, double.MaxValue, double.MaxValue);   //5.0
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_FRONT_TO_MAIN_BOUND_DOWN_GAP, double.MaxValue, double.MaxValue); //5.0
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_END_TO_SUB_BOUND_UP_GAP, double.MaxValue, double.MaxValue);      //10.0
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_END_TO_SUB_BOUND_DOWN_GAP, double.MaxValue, double.MaxValue);    //10.0
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_FRONT_TO_END_GAP, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_STIFF_TO_MAIN_GAP, double.MaxValue, 0.0);

            //零件页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PART_END_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 10.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PART_FRONT_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 10.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PART_CONNECT_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 25.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PART_PAD1_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 0.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PART_PAD1_COUNT, int.MaxValue, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PART_PAD2_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 0.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PART_PAD2_COUNT, int.MaxValue, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PART_PAD3_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 0.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PART_PAD3_COUNT, int.MaxValue, 1);

            //参数页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PARAM_PAD_BOLT_HORI_MARGIN, double.MaxValue, 30.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PARAM_PAD_BOLT_VERT_MARGIN, double.MaxValue, 30.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PARAM_PAD_SHAPE, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PARAM_TOLERANCE, double.MaxValue, 1.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PARAM_BOLT_DIR, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PARAM_STIFF_ENABLED, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PARAM_CHAMFER_TYPE, 0, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PARAM_CHAMFER_X, double.MaxValue, 15.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PARAM_CHAMFER_Y, double.MaxValue, 15.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PARAM_GAP_SIZE, double.MaxValue, 0.0);

            //螺栓页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_BASIC_PROPS, BoltBasicPropertyPanel.EmptyValue, (DObject)new object[] {
                BoltBasicPropertyPanel.ParamNames.BoltStandard, "TS10.9",
                BoltBasicPropertyPanel.ParamNames.BoltSize, 20.0,
                BoltBasicPropertyPanel.ParamNames.ThreadInMtrl, 2, // true
                BoltBasicPropertyPanel.ParamNames.HoleTol, double.MaxValue,
                BoltBasicPropertyPanel.ParamNames.MadeType0, 0, // Workshop
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HOLE_PROPS, BoltHolePanel.EmptyValue, (DObject)new object[] {
                BoltHolePanel.ParamNames.HoleX, 0.0,
                BoltHolePanel.ParamNames.HoleY, 0.0,
                BoltHolePanel.ParamNames.HoleType, 1,
                BoltHolePanel.ParamNames.RotSlot, 3,
                BoltHolePanel.ParamNames.SlotIn, 4
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_CONTENT_PROPS, null, (DObject)new object[]
            {
                BoltContentPanel.ParamNames.PartWasherUp, false,
                BoltContentPanel.ParamNames.PartWasherDown0, false,
                BoltContentPanel.ParamNames.PartWasherDown1, true,
                BoltContentPanel.ParamNames.PartNut0, true,
                BoltContentPanel.ParamNames.PartNut1, false,
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_EXTEND_PROPS, BoltExtendPanel.EmptyValue, (DObject)new object[]
            {
                BoltExtendPanel.ParamNames.ExtLength, 0.0,
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SHAPE, 0, 1); // Array
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_IGNORE, new int[] { }, null);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HORI_LEFT_GAP, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HORI_RIGHT_GAP, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_UP_GAP, double.MaxValue, 80.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_DOWN_GAP, double.MaxValue, 80.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_OFFSET_MODE, 0, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_OFFSET, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HORI_OFFSET_MODE, 0, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HORI_OFFSET, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HORI_COUNT, int.MaxValue, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HORI_STRIDE, new double[] { }, new double[] { 80.0 });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_COUNT, int.MaxValue, 3);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_STRIDE, new double[] { }, new double[] { 60.0 });

            //洞页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_NUMBER, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_STANDARD, 0, 7);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_UP_DIAMETER, double.MaxValue, double.MaxValue); //default:20.0
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_UP_HORI_GAP, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_UP_VERT_GAP, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_DOWN_DIAMETER, double.MaxValue, double.MaxValue); //default:20.0
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_DOWN_HORI_GAP, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HOLE_DOWN_VERT_GAP, double.MaxValue, double.MaxValue);

            //焊缝页
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WELD_PROPS_LIST[i], WeldPropertyPanel.EmptyValue, new DObject {
                    { WeldPropertyPanel.ParamNames.SizeUp, 5.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeUp, 11 },
                    { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                    { WeldPropertyPanel.ParamNames.SizeDown, 5.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeDown, 11 },
                    { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                    { WeldPropertyPanel.ParamNames.EdgeAround, 1 }, // Edge
                    { WeldPropertyPanel.ParamNames.MadeType, 2 }, // Workshop
                    { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                    { WeldPropertyPanel.ParamNames.RefText, "" }
                });
            }
        }
    }
}
