﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;

namespace BeamSideEndPlate209
{
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {
        private DObject m_rootUIProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;

        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostIntialize();
        }

        public override ComponentPlugin ReferencePlugin => m_refPlugin;

        private void OnPostIntialize()
        {
            // 设置窗口标题
            this.Text = "梁梁侧边端板(209)";

            // 螺栓标准变化之后调用螺栓内容的自适应方法
            this.panel_bolt_basic_property.BoltStandardChanged += this.boltContentPanel.AdaptBoltContents;

            foreach (TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }

            //初始化图片选项卡
            cbo_pad_shape.ClearItems();
            cbo_pad_shape.AddIntegerItems(0, 0, 4);
            cbo_pad_shape.UpdateItems();

            cbo_bolt_dir.ClearItems();
            cbo_bolt_dir.AddIntegerItems(0, 0, 3);
            cbo_bolt_dir.UpdateItems();

            cbo_stiff_enabled.ClearItems();
            cbo_stiff_enabled.AddIntegerItems(0, 0, 3);
            cbo_stiff_enabled.UpdateItems();

            cbo_hole_number.ClearItems();
            cbo_hole_number.AddIntegerItems(0, 0, 5);
            cbo_hole_number.UpdateItems();

            //初始化文字选项卡
            this.cbo_bolt_vert_offset_mode.ClearItems();
            this.cbo_bolt_vert_offset_mode.AddItem(0, "默认");
            this.cbo_bolt_vert_offset_mode.AddItem(1, "顶端");
            this.cbo_bolt_vert_offset_mode.AddItem(2, "中间");
            this.cbo_bolt_vert_offset_mode.AddItem(3, "底端");
            this.cbo_bolt_vert_offset_mode.UpdateItems();
            this.cbo_bolt_vert_offset_mode.SelectedKey = 0;

            this.cbo_bolt_hori_offset_mode.ClearItems();
            this.cbo_bolt_hori_offset_mode.AddItem(0, "默认");
            this.cbo_bolt_hori_offset_mode.AddItem(1, "左");
            this.cbo_bolt_hori_offset_mode.AddItem(2, "中间");
            this.cbo_bolt_hori_offset_mode.AddItem(3, "右");
            this.cbo_bolt_hori_offset_mode.UpdateItems();
            this.cbo_bolt_hori_offset_mode.SelectedKey = 0;

            this.cbo_hole_standard.ClearItems();
            this.cbo_hole_standard.AddItem(0, "默认值[TS10.9]");
            this.cbo_hole_standard.AddItem(1, "A");
            this.cbo_hole_standard.AddItem(2, "B");
            this.cbo_hole_standard.AddItem(3, "HS4.6");
            this.cbo_hole_standard.AddItem(4, "HS8.8");
            this.cbo_hole_standard.AddItem(5, "HS10.9");
            this.cbo_hole_standard.AddItem(6, "TS8.8");
            this.cbo_hole_standard.AddItem(7, "TS10.9");
            this.cbo_hole_standard.UpdateItems();
            this.cbo_hole_standard.SelectedKey = 0;
        }

        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null)
            {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }

        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked)
            {
                retProps.CopyFrom(m_rootUIProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }

        /// <summary>
        /// 当前组件需要重写此方法，并刷新螺栓基础面板
        /// </summary>
        /// <param name="root"></param>
        public override void ClearUI(Control root = null)
        {
            // 刷新螺栓基础面板
            this.panel_bolt_basic_property.RefreshControls();
            base.ClearUI(root);
        }

        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }

        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_rootUIProps.CopyFrom(srcProps);
        }

        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }

        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_rootUIProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_rootUIProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_rootUIProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_rootUIProps;
            return comp.UIProps;
        }

        private void btn_weld_props_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_rootUIProps, null);
        }

        private void cbo_pad_shape_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cbo_pad_shape.SelectedIndex < 2)
            {
                //孔基于节点的螺栓组
                chb_tolerance.Enabled = false;
                tb_tolerance.Enabled = false;
            }
            else
            {
                //带槽口的指状垫板
                chb_tolerance.Enabled = true;
                tb_tolerance.Enabled = true;
            }
        }

        private void cbo_chamfer_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            var index = cbo_chamfer_type.SelectedIndex;
            //默认为线折角
            bool xStatus = true;
            bool yStatus = true;
            if(index == 1)
            {
                //无折角
                xStatus = false;
                yStatus = false;
            }else if(index > 2)
            {
                //凸弧或凹弧折角
                yStatus = false;
            }

            tb_chamfer_x.Enabled = xStatus;
            chb_chamfer_x.Enabled = xStatus;
            tb_chamfer_y.Enabled = yStatus;
            chb_chamfer_y.Enabled = yStatus;
        }

        private void cbo_hole_number_SelectedIndexChanged(object sender, EventArgs e)
        {
            var index = cbo_hole_number.SelectedIndex;
            //默认状态为上侧孔与下侧孔均开启
            bool upStatus = true;
            bool downStatus = true;
            if(index < 2)
            {
                //无孔
                upStatus = false;
                downStatus = false;
            }else if(index == 2)
            {
                //1个孔且为上侧孔
                downStatus = false;
            }

            //下侧孔
            tb_hole_down_diameter.Enabled = downStatus;
            chb_hole_down_diameter.Enabled = downStatus;
            tb_hole_down_hori_gap.Enabled = downStatus;
            chb_hole_down_hori_gap.Enabled = downStatus;
            tb_hole_down_vert_gap.Enabled = downStatus;
            chb_hole_down_vert_gap.Enabled = downStatus;

            //上侧孔
            tb_hole_up_diameter.Enabled = upStatus;
            chb_hole_up_diameter.Enabled = upStatus;
            tb_hole_up_hori_gap.Enabled = upStatus;
            chb_hole_up_hori_gap.Enabled = upStatus;
            tb_hole_up_vert_gap.Enabled = upStatus;
            chb_hole_up_vert_gap.Enabled = upStatus;
        }
    }
}
