﻿using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System.Linq;

namespace BeamSideEndPlate209
{
    public class WeldHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler;
        private List<WeldProperties> m_weldProps = new List<WeldProperties>();

        public WeldHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;

            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                m_weldProps.Add(new WeldProperties());
            }
        }

        public void Execute()
        {
            if (!ParseProps())
                return;

            //焊接端板
            this.WeldEndPlate();

            //焊接前板
            this.WeldFrontPlate();

            //焊接连接板
            this.WeldConnPlate();

            //焊接加劲肋
            this.WeldStiffPlate();
        }

        private bool ParseProps()
        {
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }

            return true;
        }

        private void WeldEndPlate()
        {
            var endPlate = m_context.EndPlate;
            if (endPlate == null)
                return;

            Vec3 axisX = m_context.SubPlanes.TopBound.Normal.Normalize();
            Vec3 axisY = m_context.SubPlanes.BackBound.Normal.Normalize();
            Vec3 axisZ = axisX.Cross(axisY);
            var origin = m_context.SubAlignPlane.Intersect(
                m_context.SubPlanes.TopBound, m_context.SubPlanes.BackBound
                );
            //焊缝1
            do
            {
                if (m_weldProps[0].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;

                Weld curWeld = m_context.TargetComp.CreateAutoWeld(m_context.SubPart,
                    endPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, origin);
                curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_X;
                m_weldProps[0].ApplyToWeld(curWeld);
            } while (false);

            //焊缝2
            do
            {
                if (m_weldProps[1].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;

                Weld curWeld = m_context.TargetComp.CreateAutoWeld(m_context.SubPart,
                    endPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, origin);
                curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NX;
                m_weldProps[1].ApplyToWeld(curWeld);
            } while (false);

            //焊缝3
            do
            {
                if (m_weldProps[2].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;

                Weld curWeld = m_context.TargetComp.CreateAutoWeld(m_context.SubPart,
                    endPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, origin);
                curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NY;
                m_weldProps[2].ApplyToWeld(curWeld);
            } while (false);
        }

        private void WeldFrontPlate()
        {
            var frontPlate = m_context.FrontPlate;
            if (frontPlate == null)
                return;

            Vec3 axisX = m_context.MainTopPlane.Normal.Normalize();
            Vec3 axisY = m_context.MainLine;
            Vec3 axisZ = axisX.Cross(axisY);
            var origin = frontPlate.CtrlPoints.First().WorldPos;
            //焊缝4
            do
            {
                if (m_weldProps[3].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;

                Weld curWeld = m_context.TargetComp.CreateAutoWeld(m_context.MainPart,
                    frontPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, origin);
                curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_X;
                m_weldProps[3].ApplyToWeld(curWeld);
            } while (false);

            //焊缝5
            do
            {
                if (m_weldProps[4].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;

                Weld curWeld = m_context.TargetComp.CreateAutoWeld(m_context.MainPart,
                    frontPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, origin);
                curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NX;
                m_weldProps[4].ApplyToWeld(curWeld);
            } while (false);
        }

        private void WeldConnPlate()
        {
            var frontPlate = m_context.FrontPlate;
            var connPlate = m_context.ConnPlate;
            if (frontPlate == null || connPlate == null)
                return;

            Vec3 axisX = m_context.MainNearPlane.Normal.Normalize();
            Vec3 axisY = m_context.MainTopPlane.Normal.Normalize();
            Vec3 axisZ = axisX.Cross(axisY);
            var origin = connPlate.CtrlPoints.First().WorldPos;
            //焊缝6
            do
            {
                if (m_weldProps[5].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;

                Weld curWeld = m_context.TargetComp.CreateAutoWeld(m_context.MainPart,
                    connPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, origin);
                curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_Y;
                m_weldProps[5].ApplyToWeld(curWeld);
            } while (false);

            //焊缝7
            do
            {
                if (m_weldProps[6].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;

                Weld curWeld = m_context.TargetComp.CreateAutoWeld(m_context.MainPart,
                    connPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, origin);
                curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NY;
                m_weldProps[6].ApplyToWeld(curWeld);
            } while (false);

            //焊缝8
            do
            {
                if (m_weldProps[7].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;

                Weld curWeld = m_context.TargetComp.CreateAutoWeld(m_context.MainPart,
                    connPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, origin);
                curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NX;
                m_weldProps[7].ApplyToWeld(curWeld);
            } while (false);

            //焊缝9
            do
            {
                if (m_weldProps[8].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;

                Weld curWeld = m_context.TargetComp.CreateAutoWeld(frontPlate,
                    connPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, origin);
                curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_X;
                m_weldProps[8].ApplyToWeld(curWeld);
            } while (false);
        }

        private void WeldStiffPlate()
        {
            var stiffPlate = m_context.StiffPlate;
            if (stiffPlate == null)
                return;

            Vec3 axisX = -m_context.MainNearPlane.Normal.Normalize();
            Vec3 axisY = m_context.MainTopPlane.Normal.Normalize();
            Vec3 axisZ = axisX.Cross(axisY);
            var origin = stiffPlate.CtrlPoints.First().WorldPos;
            //焊缝6
            do
            {
                if (m_weldProps[5].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;

                Weld curWeld = m_context.TargetComp.CreateAutoWeld(m_context.MainPart,
                    stiffPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, origin);
                curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_Y;
                m_weldProps[5].ApplyToWeld(curWeld);
            } while (false);

            //焊缝7
            do
            {
                if (m_weldProps[6].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;

                Weld curWeld = m_context.TargetComp.CreateAutoWeld(m_context.MainPart,
                    stiffPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, origin);
                curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NY;
                m_weldProps[6].ApplyToWeld(curWeld);
            } while (false);

            //焊缝8
            do
            {
                if (m_weldProps[7].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;

                Weld curWeld = m_context.TargetComp.CreateAutoWeld(m_context.MainPart,
                    stiffPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, origin);
                curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NX;
                m_weldProps[7].ApplyToWeld(curWeld);
            } while (false);
        }
    }
}
