﻿using System;
using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace BoltedGusset_11_.Main
{
    public class ParamsHandler:ParamsHandlerBase
    {
        
        private static readonly string NoStartPostfix = "1";
        /// <summary>
        /// 属性空值全局总表
        /// </summary>
        /// <remarks><b>此静态变量可直接复制到其他组件代码中使用</b></remarks>
        private static DObject g_emptyValues;
        /// <summary>
        /// 属性内置默认值全局总表
        /// </summary>
        /// <remarks><b>此静态变量可直接复制到其他组件代码中使用</b></remarks>
        private static DObject g_builtinDefaults;
        
        /// <summary>
        /// 默认构造函数
        /// </summary>
        /// <remarks><b>此方法需要复制到其他组件代码中使用</b></remarks>
        public ParamsHandler() : base(EmptyValues, BuiltinDefaults) { }
        private static void AddEmptyDefaultValue(string paramName, object emptyValue, object defaultValue) {
            AddEmptyDefaultValue(g_emptyValues,g_builtinDefaults,paramName,emptyValue,defaultValue);
        }

        private static void InitializeEmptyDefaults()
        {
            g_emptyValues = new DObject();
            g_builtinDefaults = new DObject();
            #region  图形标签页  
            AddEmptyDefaultValue(ParamNames.PIC_GAP_BETWEEN_GUSSET_PLATE_AND_MAIN_PART, double.MaxValue, 0.0d);
            AddEmptyDefaultValue(ParamNames.PIC_DISTANCE_BETWEEN_MAIN_PART_AND_BRACE, double.MaxValue, 20.0d);
            AddEmptyDefaultValue(ParamNames.PIC_MIN_DISTANCE_BETWEEN_THE_BRACES, double.MaxValue, 20.0d);
            AddEmptyDefaultValue(ParamNames.PIC_THE_GUSSET_PLATE_CORNER_ANGLE_TOP, double.MaxValue, double.MaxValue);//自适应
            AddEmptyDefaultValue(ParamNames.PIC_THE_GUSSET_PLATE_CORNER_ANGLE_BOTTOM, double.MaxValue, double.MaxValue);//自适应
            AddEmptyDefaultValue(ParamNames.PIC_LENGTH_EDGE_OF_THE_GUSSET_PLATE_TOP_INSIDE, double.MaxValue, double.MaxValue);//自适应
            AddEmptyDefaultValue(ParamNames.PIC_LENGTH_EDGE_OF_THE_GUSSET_PLATE_TOP_OUTSIDE, double.MaxValue, double.MaxValue);//自适应
            AddEmptyDefaultValue(ParamNames.PIC_LENGTH_EDGE_OF_THE_GUSSET_PLATE_BOTTOM_INSIDE, double.MaxValue, double.MaxValue);//自适应
            AddEmptyDefaultValue(ParamNames.PIC_LENGTH_EDGE_OF_THE_GUSSET_PLATE_BOTTOM_OUTSIDE, double.MaxValue, double.MaxValue);//自适应
            AddEmptyDefaultValue(ParamNames.PIC_TOP_DISTANCE_BETWEEN_CONNECTION_PART_AND_GUSSET_PLATE_EDGE, double.MaxValue, double.MaxValue);//自适应
            #endregion
            #region 节点板标签页
            AddEmptyDefaultValue(ParamNames.GUSSET_GUSSET_PLATE, PlatePropertyPanel.EmptyValue,
                (DObject)new object[]{
                    PlatePropertyPanel.ParamNames.Thickness,20.0d,
                    PlatePropertyPanel.ParamNames.Width,0.0d,
                    PlatePropertyPanel.ParamNames.Heigth,0.0d,
                    PlatePropertyPanel.ParamNames.NoPrefix,"",
                    PlatePropertyPanel.ParamNames.NoStartPostfix,NoStartPostfix,
                    PlatePropertyPanel.ParamNames.Material,"Q235B",
                    PlatePropertyPanel.ParamNames.Name,"节点板",
            });
            AddEmptyDefaultValue(ParamNames.GUSSET_CONNECTION_PLATE, PlatePropertyPanel.EmptyValue,
                (DObject)new object[]{
                    PlatePropertyPanel.ParamNames.Thickness,20.0d,
                    PlatePropertyPanel.ParamNames.Width,double.MaxValue,//检查有无填写
                    PlatePropertyPanel.ParamNames.Heigth,0.0d,//关闭的属性
                    PlatePropertyPanel.ParamNames.NoPrefix,"",
                    PlatePropertyPanel.ParamNames.NoStartPostfix,NoStartPostfix,
                    PlatePropertyPanel.ParamNames.Material,"Q235B",
                    PlatePropertyPanel.ParamNames.Name,"连接板",
            });
            AddEmptyDefaultValue(ParamNames.GUSSET_L_PROFILE_PLATE, BeamPropertyPanel.EmptyValue,
                (DObject)new object[]{
                    BeamPropertyPanel.ParamNames.NoPrefix,"",
                    BeamPropertyPanel.ParamNames.NoStartPostfix,NoStartPostfix,
                    BeamPropertyPanel.ParamNames.Material,"Q235B",
                    BeamPropertyPanel.ParamNames.Name,"板",
                    BeamPropertyPanel.ParamNames.Profile,"BLL200*100*10",
            });
            AddEmptyDefaultValue(ParamNames.GUSSET_GUSSET_PLATE_Y_DIRECTION_OFFSET, double.MaxValue, 0.0d);
            AddEmptyDefaultValue(ParamNames.GUSSET_GUSSET_PLATE_Z_DIRECTION_OFFSET, double.MaxValue, 0.0d);
            AddEmptyDefaultValue(ParamNames.GUSSET_GUSSET_PLATE_SHAPE_SELECT, 0, 0);
            AddEmptyDefaultValue(ParamNames.GUSSET_GUSSET_PLATE_POSITION_ON_BRACE_SELECT, 0, 2);
            AddEmptyDefaultValue(ParamNames.GUSSET_SELECT_CONNECTION_OR_CLIP_ANGLES, 0, 1);
            AddEmptyDefaultValue(ParamNames.GUSSET_SELECT_CLIP_ANGLE_ORIENTATION, 0, 1);
            AddEmptyDefaultValue(ParamNames.GUSSET_MAIN_PART_EDGE_EXTENSION_DISTANCE, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(ParamNames.GUSSET_TOP_BOTTOM_EDGE_EXTENSION_DISTANCE, double.MaxValue, double.MaxValue);
            #endregion
            #region 节点板螺栓标签页
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_BOLT_BASIC_PROPERTIES, BoltBasicPropertyPanel.EmptyValue,
                (DObject)new object[] {
                    BoltBasicPropertyPanel.ParamNames.BoltStandard,"TS10.9",
                    BoltBasicPropertyPanel.ParamNames.BoltSize,20.0d,
                    BoltBasicPropertyPanel.ParamNames.ThreadInMtrl,1,
                    BoltBasicPropertyPanel.ParamNames.MadeType0,1,
                    BoltBasicPropertyPanel.ParamNames.HoleTol,double.MaxValue,
                });
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_SLOTTED_HOLES, BoltHolePanel.EmptyValue,
                (DObject)new object[] {
                    BoltHolePanel.ParamNames.HoleX,0.0d,
                    BoltHolePanel.ParamNames.HoleY,0.0d,
                    BoltHolePanel.ParamNames.HoleType,2,
                    BoltHolePanel.ParamNames.RotSlot,3,
                    BoltHolePanel.ParamNames.SlotIn,4,
                });
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_BOLT_ASSEMBLY, null,
                (DObject)new object[] {
                      BoltContentPanel.ParamNames.PartWasherUp,false,
                      BoltContentPanel.ParamNames.PartWasherDown0,false,
                      BoltContentPanel.ParamNames.PartWasherDown1,true,
                      BoltContentPanel.ParamNames.PartNut0,true,
                      BoltContentPanel.ParamNames.PartNut1,false,
                });
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_BOLT_LENGTH_INCREASE, BoltExtendPanel.EmptyValue,
                (DObject)new object[] {
                    BoltExtendPanel.ParamNames.ExtLength,0.0d,
                });
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_STAGGERING_OF_BOLTS, 0, 1);
            //螺栓布局
            #region 水平方向相关
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_HORIZONTAL_BOLTS_NUMBER, int.MaxValue, 2);
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_HORIZONTAL_BOLTS_SPACING, new double[] { }, new double[] { });
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_HORIZONTAL_BOLTS_TOP_EDGE_DIST, double.MaxValue, 40.0d);
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_HORIZONTAL_BOLTS_BOTTOM_EDGE_DIST, double.MaxValue, 40.0d);
            #endregion
            #region 垂直方向相关
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_VERTICAL_BOLTS_NUMBER, int.MaxValue, 1);
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_VERTICAL_BOLTS_SPACING, new double[] { }, new double[] { });
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_VERTICAL_BOLTS_NEAR_EDGE_DIST, double.MaxValue, 40.0d);//自适应
            #endregion
            #region 延长模式下螺栓配置

            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_EXTENDED_VERTICAL_BOLTS_TOP_EDGE_DIST, double.MaxValue, double.MaxValue);//自适应
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_EXTENDED_VERTICAL_BOLTS_NUMBER, int.MaxValue, 2);
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_EXTENDED_VERTICAL_BOLTS_SPACING, new double[]{}, new double[]{});

            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_EXTENDED_HORIZONTAL_BOLTS_FAR_EDGE_DIST, double.MaxValue, double.MaxValue);//自适应
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_EXTENDED_HORIZONTAL_BOLTS_NUMBER, int.MaxValue, 2);
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_EXTENDED_HORIZONTAL_BOLTS_SPACING, new double[]{}, new double[]{});

            #endregion
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_SELECT_LINKED_PART_BOLTS_MADE_TYPE,int.MaxValue,1);
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_SELECT_MAIN_PART_BOLTS_MADE_TYPE,int.MaxValue,1);
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_MAIN_PART_BOLTS_CENTER_EDGE_DIST, double.MaxValue, 40.0d);
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_MAIN_PART_BOLTS_NUMBER, int.MaxValue,1);
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_MAIN_PART_BOLTS_SPACING, new double[]{},new double[]{});
            
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_SELECT_BOLTS_ON_GUSSET_PLATE,int.MaxValue,1);
            AddEmptyDefaultValue(ParamNames.GUSSET_BOLTS_SELECT_CLIP_ANGLE_ATTACHMENT_TYPE,int.MaxValue,1);

            #endregion
            #region  支柱螺栓1 
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_BOLT_BASIC_PROPERTIES,BoltBasicPropertyPanel.EmptyValue,
                (DObject)new object[] {
                    BoltBasicPropertyPanel.ParamNames.BoltStandard,"TS10.9",
                    BoltBasicPropertyPanel.ParamNames.BoltSize,20.0d,
                    BoltBasicPropertyPanel.ParamNames.ThreadInMtrl,1,
                    BoltBasicPropertyPanel.ParamNames.MadeType0,1,
                    BoltBasicPropertyPanel.ParamNames.HoleTol,double.MaxValue,
                });
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_SLOTTED_HOLES,BoltHolePanel.EmptyValue,
                (DObject)new object[] {
                    BoltHolePanel.ParamNames.HoleX,0.0d,
                    BoltHolePanel.ParamNames.HoleY,0.0d,
                    BoltHolePanel.ParamNames.HoleType,2,
                    BoltHolePanel.ParamNames.RotSlot,3,
                    BoltHolePanel.ParamNames.SlotIn,4,
                });
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_BOLT_ASSEMBLY,null,
                (DObject) new object[] {
                      BoltContentPanel.ParamNames.PartWasherUp,false, 
                      BoltContentPanel.ParamNames.PartWasherDown0,false, 
                      BoltContentPanel.ParamNames.PartWasherDown1,true, 
                      BoltContentPanel.ParamNames.PartNut0,true, 
                      BoltContentPanel.ParamNames.PartNut1,false, 
                });
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_BOLT_LENGTH_INCREASE,BoltExtendPanel.EmptyValue,
                (DObject)new object[] {
                    BoltExtendPanel.ParamNames.ExtLength,0.0d,
                });
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_STAGGERING_OF_BOLTS,0,1);
            //TODO 水平螺栓数目存在支撑截面的自适应，先考虑L型截面的支撑
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_HORIZONTAL_BOLTS_NUMBER,int.MaxValue,1);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_HORIZONTAL_BOLTS_SPACING,new double[]{},new double[]{});
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_VERTICAL_BOLTS_NUMBER,int.MaxValue,2);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_VERTICAL_BOLTS_SPACING,new double[]{},new double[]{});
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_VERTICAL_BOLTS_NEAR_EDGE_DIST,double.MaxValue,40.0d);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_VERTICAL_BOLTS_FAR_EDGE_DIST,double.MaxValue,40.0d);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_VERTICAL_BOLTS_SELECT_MEASURE_METHOD,int.MaxValue,0);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_VERTICAL_BOLTS_MEASURE_OFFSET,double.MaxValue,0.0d);
            #endregion
            #region 支柱螺栓2 
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_BOLT_BASIC_PROPERTIES,BoltBasicPropertyPanel.EmptyValue,
                (DObject)new object[] {
                    BoltBasicPropertyPanel.ParamNames.BoltStandard,"TS10.9",
                    BoltBasicPropertyPanel.ParamNames.BoltSize,20.0d,
                    BoltBasicPropertyPanel.ParamNames.ThreadInMtrl,1,
                    BoltBasicPropertyPanel.ParamNames.MadeType0,1,
                    BoltBasicPropertyPanel.ParamNames.HoleTol,double.MaxValue,
                });
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_SLOTTED_HOLES,BoltHolePanel.EmptyValue,
                (DObject)new object[] {
                    BoltHolePanel.ParamNames.HoleX,0.0d,
                    BoltHolePanel.ParamNames.HoleY,0.0d,
                    BoltHolePanel.ParamNames.HoleType,2,
                    BoltHolePanel.ParamNames.RotSlot,3,
                    BoltHolePanel.ParamNames.SlotIn,4,
                });
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_BOLT_ASSEMBLY,null,
                (DObject) new object[] {
                      BoltContentPanel.ParamNames.PartWasherUp,false, 
                      BoltContentPanel.ParamNames.PartWasherDown0,false, 
                      BoltContentPanel.ParamNames.PartWasherDown1,true, 
                      BoltContentPanel.ParamNames.PartNut0,true, 
                      BoltContentPanel.ParamNames.PartNut1,false, 
                });
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_BOLT_LENGTH_INCREASE,BoltExtendPanel.EmptyValue,
                (DObject)new object[] {
                    BoltExtendPanel.ParamNames.ExtLength,0.0d,
                });
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_STAGGERING_OF_BOLTS,0,1);
            //TODO 水平螺栓数目存在支撑截面的自适应，先考虑L型截面的支撑
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_HORIZONTAL_BOLTS_NUMBER,int.MaxValue,1);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_HORIZONTAL_BOLTS_SPACING,new double[]{},new double[]{});
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_VERTICAL_BOLTS_NUMBER,int.MaxValue,2);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_VERTICAL_BOLTS_SPACING,new double[]{},new double[]{});
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_VERTICAL_BOLTS_NEAR_EDGE_DIST,double.MaxValue,40.0d);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_VERTICAL_BOLTS_FAR_EDGE_DIST,double.MaxValue,40.0d);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_VERTICAL_BOLTS_SELECT_MEASURE_METHOD,int.MaxValue,0);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_VERTICAL_BOLTS_MEASURE_OFFSET,double.MaxValue,0.0d);
            #endregion
            #region 支柱螺栓3 
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_BOLT_BASIC_PROPERTIES,BoltBasicPropertyPanel.EmptyValue,
                (DObject)new object[] {
                    BoltBasicPropertyPanel.ParamNames.BoltStandard,"TS10.9",
                    BoltBasicPropertyPanel.ParamNames.BoltSize,20.0d,
                    BoltBasicPropertyPanel.ParamNames.ThreadInMtrl,1,
                    BoltBasicPropertyPanel.ParamNames.MadeType0,1,
                    BoltBasicPropertyPanel.ParamNames.HoleTol,double.MaxValue,
                });
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_SLOTTED_HOLES,BoltHolePanel.EmptyValue,
                (DObject)new object[] {
                    BoltHolePanel.ParamNames.HoleX,0.0d,
                    BoltHolePanel.ParamNames.HoleY,0.0d,
                    BoltHolePanel.ParamNames.HoleType,2,
                    BoltHolePanel.ParamNames.RotSlot,3,
                    BoltHolePanel.ParamNames.SlotIn,4,
                });
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_BOLT_ASSEMBLY,null,
                (DObject) new object[] {
                      BoltContentPanel.ParamNames.PartWasherUp,false, 
                      BoltContentPanel.ParamNames.PartWasherDown0,false, 
                      BoltContentPanel.ParamNames.PartWasherDown1,true, 
                      BoltContentPanel.ParamNames.PartNut0,true, 
                      BoltContentPanel.ParamNames.PartNut1,false, 
                });
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_BOLT_LENGTH_INCREASE,BoltExtendPanel.EmptyValue,
                (DObject)new object[] {
                    BoltExtendPanel.ParamNames.ExtLength,0.0d,
                });
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_STAGGERING_OF_BOLTS,0,1);
            //TODO 水平螺栓数目存在支撑截面的自适应，先考虑L型截面的支撑
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_HORIZONTAL_BOLTS_NUMBER,int.MaxValue,1);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_HORIZONTAL_BOLTS_SPACING,new double[]{},new double[]{});
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_VERTICAL_BOLTS_NUMBER,int.MaxValue,2);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_VERTICAL_BOLTS_SPACING,new double[]{},new double[]{});
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_VERTICAL_BOLTS_NEAR_EDGE_DIST,double.MaxValue,40.0d);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_VERTICAL_BOLTS_FAR_EDGE_DIST,double.MaxValue,40.0d);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_VERTICAL_BOLTS_SELECT_MEASURE_METHOD,int.MaxValue,0);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_VERTICAL_BOLTS_MEASURE_OFFSET,double.MaxValue,0.0d);
            #endregion

            #region 额外控制 
            AddEmptyDefaultValue(ParamNames.PIC_BRACE_1_ALIGN_DIST,double.MaxValue,0.0d);
            AddEmptyDefaultValue(ParamNames.PIC_BRACE_2_ALIGN_DIST,double.MaxValue,0.0d);
            AddEmptyDefaultValue(ParamNames.PIC_BRACE_3_ALIGN_DIST,double.MaxValue,0.0d);
            
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_1_SELECT_WELD_ON_GUSSET_PLATE,0,1);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_2_SELECT_WELD_ON_GUSSET_PLATE,0,1);
            AddEmptyDefaultValue(ParamNames.BRACE_BOLTS_3_SELECT_WELD_ON_GUSSET_PLATE,0,1);
            #endregion
            
            #region 焊缝
            //需要自适应的默认值值也用空值表示
            //焊缝
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                AddEmptyDefaultValue(ParamNames.WELD_PROPS_LIST[i], WeldPropertyPanel.EmptyValue, new DObject {
                    { WeldPropertyPanel.ParamNames.SizeUp, 6.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeUp, 11 }, // Fillet
                    { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                    { WeldPropertyPanel.ParamNames.SizeDown, 6.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeDown, 11 }, // Fillet
                    { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                    { WeldPropertyPanel.ParamNames.EdgeAround, 1 }, // Edge
                    { WeldPropertyPanel.ParamNames.MadeType, 2 }, // Workshop
                    { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                    { WeldPropertyPanel.ParamNames.RefText, "" }
                });
            }
            #endregion
        }
        
        /// <summary>
        /// 组件公用属性空值表
        /// </summary>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public static DObject EmptyValues
        {
            get
            {
                if(g_emptyValues == null)
                {
                    InitializeEmptyDefaults();
                }
                return g_emptyValues;
            }
        }

        /// <summary>
        /// 组件公用内置默认值接口
        /// </summary>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public static DObject BuiltinDefaults
        {
            get
            {
                if (g_builtinDefaults == null)
                {
                    InitializeEmptyDefaults();
                }
                return g_builtinDefaults;
            }
        }

        /// <summary>
        /// 初始化默认值
        /// </summary>
        /// <param name="defaultConfig">从默认值配置加载的默认值</param>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks> 
        public override DObject InitializeDefaultValues(DObject defaultConfig)
        {
            m_defaultValues = new DObject();
            m_defaultValues.CopyFrom(defaultConfig);

            var emptyValues = EmptyValues;
            var builtinDefaults = BuiltinDefaults;
            if(emptyValues == null || builtinDefaults == null)
            {
                Console.WriteLine(@"ParamsHandler.InitializeDefaultValues:[Warning] EmptyValue/BuiltinDefault is null!");
                return m_defaultValues;
            }
            TryCompleteDObjectValues(m_defaultValues, builtinDefaults, emptyValues);
            return m_defaultValues;
        }

    }
}
