﻿using System;
using System.Collections.Generic;
using BoltedGusset_11_.Main;
using BoltedGusset_11_.Tool;
using LoongCAD.STC.Object3D;

namespace BoltedGusset_11_.Running
{
    
    public class BuildBraceBolts : RunningBaseClass
    {
        public BuildBraceBolts(ComponentInstanceContext context, ParserSubPart parserSubPart, ParserBraceBolts parserBraceBolts, BuildGussetPlate buildGussetPlate) : base(context)
        {
            ParserSubPart = parserSubPart;
            ParserBraceBolts = parserBraceBolts;
            BuildGussetPlate = buildGussetPlate;
        }

        private readonly ParserSubPart ParserSubPart;
        private readonly ParserBraceBolts ParserBraceBolts;
        private readonly BuildGussetPlate BuildGussetPlate;

        private List<BoltGroup> BoltGroups { get; set; } = new List<BoltGroup>();


        public override bool Execute()
        {
            if (BuildGussetPlate?.GussetPlate == null) {
                Console.WriteLine(@"GussetPlate is null");
                return false;
            }
            
            var subs = ParserSubPart.SubData;
            for (var i = 0; i < subs.Count; ++i)
            {
                var index = i % 3;
                if (!BuildBoltsWith(subs[i], index)) {
                    Console.WriteLine(@"create bolts fail");
                    return true;
                }
            }
            return true;
        }


        /// <summary>
        /// 创建螺栓组
        /// </summary>
        /// <param name="subData"></param>
        /// <param name="index"></param>
        private bool BuildBoltsWith(SubData subData, int index)
        {
            var param = ParserBraceBolts.ParamDataList[index];
            //数目为0时，不创建支撑处的螺栓
            if (param.HorBoltsNum == 0 || param.VerBoltsNum == 0) {
                return true;
            }
            var plate = BuildGussetPlate.GussetPlate;
            var firstCtrlPoint = PointHelper.Midpoint(subData.TopFarAlignPoint, subData.BottomFarAlignPoint);
            var lastCtrlPoint = PointHelper.Move(firstCtrlPoint, subData.NearToFarLine, 100);
            var target = m_context.TargetComp;
            var boltGroup = target.CreateBoltGroup
                (plate, subData.Beam, firstCtrlPoint, lastCtrlPoint, true);
            if (boltGroup == null) {
                return false;
            }
            //收面板控制的螺栓分组属性。
            ParserBraceBolts.SetUpBoltGroup(boltGroup,index);
            //单个支撑对应的特殊螺栓属性值
            //boltGroup.CutLength = (m_context.BuildGussetPlate.AllParamData.Thickness + subData.Thickness) * 2 + 1;
            boltGroup.CutLength = (m_pub.GussetPlatePro.Thickness + subData.Thickness) * 2 + 1;
            boltGroup.Refresh();
            BoltGroups.Add(boltGroup);
            return true;
        }
    }
}