﻿using System;
using System.Collections.Generic;
using System.Linq;
using BoltedGusset_11_.Main;
using BoltedGusset_11_.Tool;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;

namespace BoltedGusset_11_.Running
{
    public class BuildGussetPlateBolts: RunningBaseClass
    {
        private Vec3 m_btmBuildP;
        private Vec3 m_topBuildP;

        public BuildGussetPlateBolts(ComponentInstanceContext context, ParserSubPart parserSubPart, ParserGussetBolts parserGussetBolts, BuildGussetPlate buildGussetPlate, BuildConnPlate buildConnPlate, BuildLProfilePlate buildLProfilePlate) : base(context)
        {
            ParserSubPart = parserSubPart;
            ParserGussetBolts = parserGussetBolts;
            BuildGussetPlate = buildGussetPlate;
            BuildConnPlate = buildConnPlate;
            BuildLProfilePlate = buildLProfilePlate;
        }
        
        private readonly ParserSubPart ParserSubPart;
        private readonly ParserGussetBolts ParserGussetBolts;
        private readonly BuildGussetPlate BuildGussetPlate;
        private readonly BuildConnPlate BuildConnPlate;
        private readonly BuildLProfilePlate BuildLProfilePlate;
        
        /// <summary>
        /// 创建好的螺栓组实例
        /// </summary>
        private BoltGroup ConnPlateBolt { get; set; }


        /// <summary>
        /// 能否创建连接板侧螺栓
        /// </summary>
        private bool m_bCanBuild1;
        /// <summary>
        /// 能否创建主零件螺栓
        /// </summary>
        private bool m_bCanBuild2;
        private BoltGroup m_bolt1;
        private BoltGroup m_bolt2;


        public override bool Prepare()
        {
            var select =  m_pub.SelectClipAngleAttachmentType;
            m_bCanBuild1 = select == 0 || select == 1 || select == 3;
            m_bCanBuild2 = select == 0 || select == 1 || select == 2;
            return base.Prepare();
        }

        public override bool Execute()
        {

            var contPlate = BuildConnPlate;
            var lProPlate = BuildLProfilePlate;
            var gstBolt = ParserGussetBolts;
            m_btmBuildP = contPlate.m_btmBuildP;
            m_topBuildP = contPlate.m_topBuildP;

            var target = m_context.TargetComp;
            var gstPlate = BuildGussetPlate.GussetPlate;

            var main = contPlate.ConnectionPlate;
            var sub = gstPlate;
            var startP = m_btmBuildP;
            var endP = m_topBuildP;
            if (gstBolt.AllParamData.HorBoltsNum == 0 || gstBolt.AllParamData.VerBoltsNum == 0) {
                m_bCanBuild1 = false;
            }
            if (gstBolt.AllParamData.HorBoltsNum == 0 || gstBolt.AllParamData.MainPartBoltsNum == 0) {
                m_bCanBuild2 = false;
            }

            if (m_bCanBuild1)
            {
                if (contPlate.m_bCanBuildSide1)
                {
                    ConnPlateBolt =  target.CreateBoltGroup(main,sub,startP,endP,true);
                    if (ConnPlateBolt == null) {
                        Console.WriteLine(@"create bolts fail");
                        return false;
                    }
                    HandleBolt();
                }else if (lProPlate.m_bCanBuild1 || lProPlate.m_bCanBuild2)
                {
                    var subs = new List<PartBase>();
                    if(lProPlate.m_bCanBuild1)
                        subs.Add(lProPlate.LProfilePlate1);
                    if(lProPlate.m_bCanBuild2)
                        subs.Add(lProPlate.LProfilePlate2);

                    if (subs.Count == 1)
                    {
                        ConnPlateBolt =  target.CreateBoltGroup
                            (subs.First(),gstPlate,startP,endP,true);
                    }
                    else
                    {
                        ConnPlateBolt =  target.CreateBoltGroup
                            (gstPlate,subs.ToArray(),startP,endP,true);
                    }
                    if (ConnPlateBolt == null) {
                        Console.WriteLine(@"create bolts fail");
                        return false;
                    }
                    HandleBolt();
                }
            }

            if (m_bCanBuild2)
            {
                var subPart = ParserSubPart;
                startP = m_topBuildP;
                endP = m_btmBuildP;
                var boltData = new BoltsData
                {
                    RotationSide = Define.ObjectSide.SIDE_BOTTOM,
                    HeadOffsetX = gstBolt.AllParamData.MainPartCenterEdgeDist,
                    ArrayStrideX = gstBolt.ConnPlateBoltsData.ArrayStrideX.ToArray(),
                };
                var spaceY = BoltsHelp.AdaptBoltSpacing
                    (gstBolt.AllParamData.MainPartBoltsNum, gstBolt.AllParamData.MainPartBoltsSpace, 60);
                boltData.ArrayStrideY = spaceY.ToArray();
                var distOff = spaceY.Sum() / 2 + gstBolt.AllParamData.MainPartCenterEdgeDist; 
                if (lProPlate.m_bCanBuild1)
                {
                    var axisX = endP - startP;
                    var refPl1 = subPart.FarBuildPlane;
                    var axisY = refPl1.Normal;
                    boltData.m_workAxis = new Mat43(axisX,axisY,axisX.Cross(axisY),Vec3.ZERO);
                    PointHelper.MoveToRef(ref startP,refPl1);
                    PointHelper.MoveToRef(ref endP,refPl1);
                    PointHelper.MoveRef(ref startP,refPl1.Normal,distOff);
                    PointHelper.MoveRef(ref endP,refPl1.Normal,distOff);
                    PointHelper.MoveToRef(ref startP,subPart.FaceBuildPlane);
                    PointHelper.MoveToRef(ref endP,subPart.FaceBuildPlane);
                    m_bolt1= target.CreateBoltGroup(m_context.MainPart,lProPlate.LProfilePlate1,startP, endP, true);
                    if (m_bolt1 == null) {
                        Console.WriteLine(@"create bolts fail");
                        return false;
                    }
                    boltData.SetUpWith(m_bolt1);
                    gstBolt.AllParamData.BoltsConfig.FillAll(m_bolt1);
                    m_bolt1.MadeType = gstBolt.AllParamData.BoltsConfig.m_basicProps.MadeType1;
                    m_bolt1.Refresh();
                }
                startP = m_topBuildP;
                endP = m_btmBuildP;
                if (lProPlate.m_bCanBuild2)
                {
                    var axisX = endP - startP;
                    var refPl2 = subPart.NearBuildPlane;
                    var axisY = refPl2.Normal;
                    boltData.m_workAxis = new Mat43(axisX,axisY,axisX.Cross(axisY),Vec3.ZERO);
                    PointHelper.MoveToRef(ref startP,refPl2);
                    PointHelper.MoveToRef(ref endP,refPl2);
                    PointHelper.MoveRef(ref startP,refPl2.Normal,distOff);
                    PointHelper.MoveRef(ref endP,refPl2.Normal,distOff);
                    PointHelper.MoveToRef(ref startP,subPart.FaceBuildPlane);
                    PointHelper.MoveToRef(ref endP,subPart.FaceBuildPlane);
                    m_bolt2= target.CreateBoltGroup(m_context.MainPart,lProPlate.LProfilePlate2,startP, endP, true);
                    if (m_bolt2 == null) {
                        Console.WriteLine(@"create bolts fail");
                        return false;
                    }

                    boltData.RotationSide = Define.ObjectSide.SIDE_TOP;
                    boltData.SetUpWith(m_bolt2);
                    gstBolt.AllParamData.BoltsConfig.FillAll(m_bolt2);
                    m_bolt2.MadeType = gstBolt.AllParamData.BoltsConfig.m_basicProps.MadeType1;
                    m_bolt2.Refresh();
                }
            }
            
            return true;
        }

        private void HandleBolt()
        {
            var parser = ParserGussetBolts;
            parser.ConnPlateBoltsData.SetUpWith(ConnPlateBolt);
            parser.AllParamData.BoltsConfig.FillAll(ConnPlateBolt);
            ConnPlateBolt.HeadOffsetX = parser.AllParamData.HorTopEdgeDist; 
            ConnPlateBolt.Refresh();
        }
    }
}