﻿using System.Collections.Generic;
using System.Linq;
using BoltedGusset_11_.Main;
using BoltedGusset_11_.Tool;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;

namespace BoltedGusset_11_.Running
{
    public class BuildWeld : RunningBaseClass
    {
        public BuildWeld(ComponentInstanceContext context) : base(context)
        {
            m_context.BuildWeld = this;
        }

        /// <summary>
        /// 存储焊缝数据
        /// </summary>
        public List<WeldProperties> m_weldProps = new List<WeldProperties>();
        
        public override bool Init()
        {
            if (!InitWeld()) return true;
            return base.Init();
        }

        private bool InitWeld()
        {
            for(int i=0;i<ParamNames.WELD_PROPS_LIST.Length;++i)
            {
                m_weldProps.Add(new WeldProperties());
            }

            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                var weldProps = m_context.ParamsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }
            return true;
        }


        public override bool Prepare()
        {
            return base.Prepare();
        }

        public override bool Execute()
        {
            return base.Execute();
        }

        public void BuildWeldForBraceOnGussetPlateWeld(SteelPlate gussetPlate,ParserSubPart subPart)
        {
            if(gussetPlate == null){return;}
            var handler = m_context.ParamsHandler;
            var bTurnOnWeld = new List<bool>()
            {
                handler.GetIntValue(ParamNames.BRACE_BOLTS_1_SELECT_WELD_ON_GUSSET_PLATE) == 1,
                handler.GetIntValue(ParamNames.BRACE_BOLTS_2_SELECT_WELD_ON_GUSSET_PLATE) == 1,
                handler.GetIntValue(ParamNames.BRACE_BOLTS_3_SELECT_WELD_ON_GUSSET_PLATE) == 1,
            };
            var subs = subPart.SubDataSort;
            for (var i = 0; i < subs.Count; i++)
            {
                if (bTurnOnWeld[i % 3])
                {
                    CreateWeldForBraceOnGussetPlateWeld(gussetPlate,subs[i],subPart.FarBuildPlane,subPart.NearBuildPlane);
                }
            }

        }
        private void CreateWeldForBraceOnGussetPlateWeld(SteelPlate gussetPlate, SubData sub,Plane farBuildPlane,Plane nearBuildPlane)
        {
            var target = m_context.TargetComp;
            //NOTE 不适用自动焊缝，需要四周手动计算焊缝，或者后续增加四周焊缝
            //landingPl需要次零件最靠近节点板的前后面
            var midPl = PlaneHelper.GetMidPlane(sub.FrontPlane, sub.BackPlane);
            if(midPl == null){return;}
            var refPt = midPl.Intersect(sub.NearAlignPlane, sub.TopPlane);

            var dist1 = nearBuildPlane.Dist(refPt);
            var dist2 = farBuildPlane.Dist(refPt);
            var landingPl = dist2>=dist1 ?new Plane(nearBuildPlane):new Plane(farBuildPlane);

            var pls = new List<Plane>()
            {
                sub.NearAlignPlane,
                sub.TopPlane,
                sub.FarAlignPlane,
                sub.BottomPlane,
            };

            var pts = PlaneHelper.GetLandingPoint(landingPl, pls);
            pts.Add(pts.First());
            
            var weld = target.CreatePolyWeld(gussetPlate, sub.Beam,pts.ToArray(), true);
            m_weldProps[1].ApplyToWeld(weld);
            weld.Refresh();
        }


        public void BuildWeldForGussetPlate(Plane topMainPlane, Plane btmMainPlane, SteelPlate gussetPlate,
            ParserSubPart subPart)
        {
            if (m_pub.SelectConnectionPlateOrLProfile == 0 || m_pub.SelectConnectionPlateOrLProfile == 1)
            {
                if(subPart.m_bExtensionMode)
                    return;
                var landingPl = subPart.FaceBuildPlane;
                var points = PlaneHelper.GetLandingPoint(landingPl, new List<Plane>()
                {
                    topMainPlane,subPart.NearBuildPlane,
                    btmMainPlane,subPart.FarBuildPlane,
                });
                points.Add(points.First());
            
                var weld = m_context.TargetComp.CreatePolyWeld(m_context.MainPart, gussetPlate, points.ToArray(), true);
                m_weldProps[0].ApplyToWeld(weld);
            }
        }

        public void BuildWeldForConnPlate(ParserSubPart subPart, BuildGussetPlate buildGussetPlate,
            SteelBeam connectionPlate, Vec3 btmBuildP,
            Vec3 topBuildP
            , double thickness, double width, double height)
        {
            var target = m_context.TargetComp;
            var select =  m_pub.SelectClipAngleAttachmentType;
            var nearPlane = subPart.NearBuildPlane.Dist(topBuildP) > subPart.FarBuildPlane.Dist(topBuildP)
                ? subPart.FarBuildPlane : subPart.NearBuildPlane;
            var faceOffPlane = subPart.FaceBuildPlane.Move(subPart.AllParamData.OffSetForFaceBuildPlane);
            //连接板和主零件之间的焊缝
            var pointsL1 = new List<Vec3>();
            var topNearFaceP = PointHelper.MoveTo(topBuildP, new List<Plane>() { nearPlane, subPart.FaceBuildPlane });
            var btmNearFaceP = PointHelper.MoveTo(btmBuildP, new List<Plane>() { nearPlane, subPart.FaceBuildPlane });
            pointsL1.Add(topNearFaceP);
            pointsL1.Add(PointHelper.Move(topNearFaceP,nearPlane.Normal,thickness));
            pointsL1.Add(PointHelper.Move(btmNearFaceP,nearPlane.Normal,thickness));
            pointsL1.Add(btmNearFaceP);
            pointsL1.Add(topNearFaceP);
            var weld1 = target.CreatePolyWeld(m_context.MainPart,
                connectionPlate,pointsL1.ToArray(),true);
            m_weldProps[0].ApplyToWeld(weld1);
            
            if(select == 0 || select == 1 || select == 3) 
                return;
            
            var topNearFaceOffP= PointHelper.MoveTo(topBuildP, new List<Plane>(){nearPlane,faceOffPlane});
            //连接板和节点板之间的焊缝
            var pointsL2 = new List<Vec3>();
            pointsL2.Add(topNearFaceOffP);
            var tmpPoint = PointHelper.MoveTo(topBuildP, nearPlane);
            PointHelper.MoveRef(ref tmpPoint,faceOffPlane.Normal,width/2);
            pointsL2.Add(tmpPoint);
            tmpPoint = PointHelper.MoveTo(btmBuildP, nearPlane);
            PointHelper.MoveRef(ref tmpPoint,faceOffPlane.Normal,width/2);
            pointsL2.Add(tmpPoint);
            var btmNearFaceOffP= PointHelper.MoveTo(btmBuildP, new List<Plane>(){nearPlane,faceOffPlane});
            pointsL2.Add(btmNearFaceOffP);
            pointsL2.Add(topNearFaceOffP);
            
            var weld2 = target.CreatePolyWeld(buildGussetPlate.GussetPlate,connectionPlate,pointsL2.ToArray(),true);
            m_weldProps[3].ApplyToWeld(weld2);
        }

        /// <summary>
        /// 给L型截面创建焊缝
        /// </summary>
        /// <param name="parserSubPart"></param>
        /// <param name="buildGussetPlate"></param>
        /// <param name="lProfilePlate"></param>
        /// <param name="topBuildP"></param>
        /// <param name="btmBuildP"></param>
        /// <param name="dir"></param>
        public void BuildWeldForLProfile(ParserSubPart parserSubPart, BuildGussetPlate buildGussetPlate,
            SteelBeam lProfilePlate, Vec3 topBuildP,
            Vec3 btmBuildP, Vec3 dir)
        {
            var select =  m_pub.SelectClipAngleAttachmentType;
            var bGstPlate = select == 2 || select == 4;
            var bMain = select == 3 || select == 4;
            if(!bGstPlate && !bMain) return;
            if (lProfilePlate.Profile.GetType() == typeof(LProfile))
            {
                //对于别的截面等待自动焊缝进行处理
                if(!(lProfilePlate.Profile is LProfile )) return;
                var beamPlanes = BeamPlanes.CreateFromBeam(lProfilePlate);
                if(beamPlanes == null) return;
                //以BLL200*100*5为例
                var target = m_context.TargetComp;
                var subPart = parserSubPart;
                var gstPart = buildGussetPlate;

                var gstNearBuildPl = new Plane(gstPart.m_buildPointsInFacePlane.First(),subPart.AxisX);
                var typeList = new List<EBeamPlaneType>() {
                    EBeamPlaneType.FrontBound, EBeamPlaneType.BackBound,
                    EBeamPlaneType.TopBound, EBeamPlaneType.BottomBound,
                };
                if (!PlaneHelper.CheckTypeList(beamPlanes, typeList)) {
                    return;
                }
                var gstFarBuildPl = PlaneHelper.GetSameNormalPlane(beamPlanes,typeList,subPart.AxisX);
                var gstLandingPl = PlaneHelper.GetSameNormalPlane(beamPlanes, typeList, -dir);
                var gstTopPlane = new Plane(topBuildP, subPart.AxisY);
                var gstBtmPlane = new Plane(btmBuildP, subPart.AxisY);

                var mainLandingPl = new Plane(subPart.FaceBuildPlane);
                var mainTopPlane = new Plane(topBuildP, subPart.AxisY);
                var mainBtmPlane = new Plane(btmBuildP, subPart.AxisY);
                var mainNearBuildPl = new Plane(gstLandingPl);
                var mainFarBuildPl = PlaneHelper.GetSameNormalPlane(beamPlanes,typeList,-gstLandingPl.Normal);
                

                if (bGstPlate)
                {
                    //连接板和节点板之间的焊缝
                    var gstPointL = PlaneHelper.GetLandingPoint(gstLandingPl, new List<Plane>()
                    {
                        gstNearBuildPl,gstTopPlane,gstFarBuildPl,gstBtmPlane,
                    });
                    gstPointL.Add(gstPointL.First());
                    var weldGst = target.CreatePolyWeld(gstPart.GussetPlate,lProfilePlate,gstPointL.ToArray(),true);
                    m_weldProps[3].ApplyToWeld(weldGst);
                }

                if (bMain)
                {
                    //连接件和主零件之间的焊缝
                    var mainPointL = PlaneHelper.GetLandingPoint(mainLandingPl, new List<Plane>()
                    {
                        mainNearBuildPl,mainTopPlane,mainFarBuildPl,mainBtmPlane,
                    });
                    mainPointL.Add(mainPointL.First());
                    var weldMain = target.CreatePolyWeld(m_context.MainPart,lProfilePlate,mainPointL.ToArray(),true);
                    m_weldProps[0].ApplyToWeld(weldMain);
                }
            }
        }
    }
}