﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ColdRolledOverlap1
{
    public class BoltLocation
    {
        //默认值
        public double length;
        public double gap1;
        public double gap2;
        public int count;
        public List<double> stride;
        public List<double> defaultStride;
        double MAX = int.MaxValue - 1; //数值小于MAX看作有效值
        public bool centerMode = false; //无center无side时是否居中

        /// <summary>
        /// 设置默认的间距
        /// </summary>
        /// <param name="gap1"></param>
        /// <param name="gap2"></param>
        /// <param name="count"></param>
        /// <param name="stride"></param>
        public BoltLocation(double gap1, double gap2, int count, double[] stride)
        {
            this.gap1 = gap1;
            this.gap2 = gap2;
            this.count = count;
            this.defaultStride = new List<double>(stride); //这里的stride必须有值
            this.stride = ParseStrideInput(stride);
        }
        public BoltLocation() { }

        /// <summary>
        /// 设置间距
        /// </summary>
        /// <param name="count"></param>
        /// <param name="stride"></param>
        public void SetStride(int count, double[] stride)
        {
            if (count < MAX) //一个有效的输入(count可以是0)
            {
                this.count = count;
            }
            else
            {
                this.count = 2;
            }
            this.stride = ParseStrideInput(stride);
        }

        /// <summary>
        /// 处理输入的螺栓间距
        /// </summary>
        /// <param name="count">螺栓数量</param>
        /// <param name="originArray">从UI中获取的DoubleArray</param>
        /// <returns></returns>
        public List<double> ParseStrideInput(double[] originArray)
        {
            if (count > MAX)
                return new List<double>();
            if (originArray == null || originArray.Length == 0)
                originArray = this.defaultStride.ToArray();
            List<double> list = new List<double>();
            for (int i = 0; i < count - 1; ++i)
            {
                list.Add(originArray[Math.Min(i, originArray.Length - 1)]);
            }
            return list;
        }

        /// <summary>
        /// 输入UI中获取到的值，获得位置参数
        /// </summary>
        public void SetBoltLocation(double gap1, double gap2, int count, double[] stride, double length)
        {
            SetStride(count, stride);
            if (length > MAX) //未设置板参数，定位按默认值和设置的值来
            {
                if (gap1 < MAX)
                    this.gap1 = gap1;
                if (gap2 < MAX)
                    this.gap2 = gap2;
                this.length = this.gap1 + this.stride.Sum() + this.gap2;
                return;
            }
            this.length = length; //输入了一个有效length
            if (gap2 < MAX) //有side往side靠
            {
                if (gap1 < MAX && this.stride.Count > 0 && stride.Length == 0) //有center也有side，但是不输入间距
                {
                    double boltGap = (length - gap1 - gap2) / this.stride.Count; //自动布置螺栓间距
                    for (int i = 0; i < this.stride.Count; i++)
                        this.stride[i] = boltGap;
                    this.gap1 = gap1;
                    this.gap2 = length - this.stride.Sum() - gap1;
                    return;
                }
                else if(gap1 < MAX && this.stride.Count > 0 && stride.Length != 0 && centerMode) //全都输入了
                {
                    this.gap1 = length / 2 - this.stride.Sum() / 2; //居中
                    this.gap2 = this.gap1;
                    return;
                }
                this.gap1 = length - this.stride.Sum() - gap2; //有side无center，center由side决定
                this.gap2 = gap2;
                return;
            }
            if (gap1 < MAX && gap2 > MAX) //有center无side往center方向靠
            {
                this.gap1 = gap1;
                this.gap2 = length - this.stride.Sum() - gap1; //有center无side，side由center决定
                return;
            }
            if (gap1 > MAX && gap2 > MAX) //无center无side
            {
                if (centerMode)
                {
                    this.gap1 = length / 2 - this.stride.Sum() / 2; //居中
                    this.gap2 = this.gap1;
                }
                else
                    this.gap2 = length - this.stride.Sum() - this.gap1; //往center方向靠
                return;
            }
        }

        /// <summary>
        /// 用于自适应以中心定位的螺栓
        /// </summary>
        /// <param name="gap1"></param>
        /// <param name="gap2"></param>
        /// <param name="count"></param>
        /// <param name="stride"></param>
        public void SetCenterBoltLoaction(double gap1, double gap2, int count, double[] stride, double centerOffset)
        {
            if (count < MAX) //一个有效的输入(count可以是0)
            {
                this.count = count;
                this.stride = ParseStrideInput(stride);
            }
            if (gap1 < MAX)
                this.gap1 = gap1;
            else
                this.gap1 = centerOffset - this.stride.Sum() / 2;
            if (gap2 < MAX)
                this.gap2 = gap2;
            this.length = this.gap1 + this.stride.Sum() + this.gap2;
        }
    }
}
