﻿using System;
using System.Linq;
using System.Text;
using System.Collections.Generic;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.Util;

namespace ColdRolledOverlap1
{
    public class BoltParameter
    {
        public enum SoltInMode
        {
            NONE = 0,
            BEAM = 1,
            PLATE = 2,
            BOTH = 3,
        }

        public Vec3 BoltPos = Vec3.ZERO;
        public double BoltSizeX = 0.0;
        public double BoltSizeY = 0.0;
        public Vec3 BoltDirX = Vec3.AXIS_X;
        public Vec3 BoltDirY = Vec3.AXIS_Y;
        public double BoltTopMargin = 0.0;
        public double BoltBottomMargin = 0.0;
        public double BoltRightMargin = 0.0;

        public int HoriCount = 0;
        public int VertCount = 0;
        public List<double> StrideX = new List<double>();
        public List<double> StrideY = new List<double>();
        public List<int> BoltIgnores = new List<int>();
        public int BoltOrientation = 0;
        public List<Vec3> CornerPoints = new List<Vec3>();

        public SoltInMode SlotIn = SoltInMode.NONE;

        public bool PartWasherUp = false;
        public bool PartWasherDown0 = false;
        public bool PartWasherDown1 = false;
        public bool PartNut0 = false;
        public bool PartNut1 = false;

        public Define.PartMadeType MadeType0 = Define.PartMadeType.PART_MADE_SITE; //不变量
        public Define.PartMadeType MadeType1 = Define.PartMadeType.PART_MADE_WORKSHOP;



        //基础属性
        public string BoltStandard = "TS10.9";
        public double BoltSize = 20.0;
        public double HoleTol = 0.0;
        public bool ThreadInMaterial = false;
        public Define.PartMadeType MadeType = Define.PartMadeType.PART_MADE_SITE;

        //螺栓孔设置
        public double HoleX = 0.0;
        public double HoleY = 0.0;
        public Define.BoltHoleType HoleType = Define.BoltHoleType.BOLT_HOLE_OVERSIZED;
        public Define.BoltRotateSlot RotateSlot = Define.BoltRotateSlot.BOLT_SLOT_PARALLEL;

        //螺栓垫片(螺母)设置
        public uint SectionFlags = 196742; //螺栓部件开关标志位 

        //其它
        public Define.ObjectSide RotationSide = Define.ObjectSide.SIDE_TOP;
        public Define.BoltGroupShape BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_NONE;
        public double CutLength = 200.0;
        public double ExtLength = 0.0;


        //位置属性




        public BoltParameter()
        {

        }

        public void UpdateCornerPoints()
        {
            this.CornerPoints.Clear();
            this.CornerPoints.Add(this.BoltPos);
            this.CornerPoints.Add(this.BoltPos + this.BoltDirX * this.BoltSizeX);
            this.CornerPoints.Add(this.BoltPos + this.BoltDirY * this.BoltSizeY);
            this.CornerPoints.Add(this.CornerPoints[1] + this.BoltDirY * this.BoltSizeY);
        }

        public void SetShape(int shapeIndex)
        {
            switch (shapeIndex)
            {
                case 2:
                    BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_X_ODD;
                    break;
                case 3:
                    BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_X_EVEN;
                    break;
                case 4:
                    BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_Y_ODD;
                    break;
                case 5:
                    BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_Y_EVEN;
                    break;
                case 1:
                default:
                    BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ARRAY;
                    break;
            }
        }
        public void SetOrient(int orientIndex)
        {
            switch (orientIndex)
            {
                case 2:
                    RotationSide = Define.ObjectSide.SIDE_BOTTOM;
                    break;
                case 3:
                    RotationSide = Define.ObjectSide.SIDE_TOP;
                    break;
                case 1:
                default:
                    RotationSide = Define.ObjectSide.SIDE_TOP;
                    break;
            }
        }

        public void CopyFromBasicProps(DObject basicProps)
        {
            if (basicProps == null)
                return;

            this.BoltStandard = basicProps.GetString("bolt_standard");
            this.BoltSize = basicProps.GetDouble("bolt_size");
            this.HoleTol = basicProps.GetDouble("hole_tol");
            this.ThreadInMaterial = basicProps.GetInt("thread_in_mtrl") == 2;
            this.MadeType0 = this.ParseMadeType(basicProps.GetInt("made_type0"));
            this.MadeType1 = this.ParseMadeType(basicProps.GetInt("made_type1"));

            this.MadeType = this.ParseMadeType(basicProps.GetInt("made_type0"));

            if (this.HoleTol == double.MaxValue)
            {
                if (this.BoltSize >= 27.0)
                    this.HoleTol = 3.0;
                else if (this.BoltSize >= 16.0)
                    this.HoleTol = 2.0;
                else
                    this.HoleTol = 1.5;
            }
        }

        public void CopyFromHoleProps(DObject holeProps)
        {
            if (holeProps == null)
                return;

            this.HoleX = holeProps.GetDouble("hole_x");
            this.HoleY = holeProps.GetDouble("hole_y");
            // 孔类型
            {
                int comboIndex = holeProps.GetInt("hole_type");
                switch (comboIndex)
                {
                    case 1:
                        this.HoleType = Define.BoltHoleType.BOLT_HOLE_SLOTTED;
                        break;
                    case 2:
                    default:
                        this.HoleType = Define.BoltHoleType.BOLT_HOLE_OVERSIZED;
                        break;
                }
            }
            //
            {
                int comboIndex = holeProps.GetInt("rot_slot");
                switch (comboIndex)
                {
                    case 0:
                        this.RotateSlot = Define.BoltRotateSlot.BOLT_SLOT_ODD;
                        break;
                    case 1:
                        this.RotateSlot = Define.BoltRotateSlot.BOLT_SLOT_EVEN;
                        break;
                    case 2:
                    default:
                        this.RotateSlot = Define.BoltRotateSlot.BOLT_SLOT_PARALLEL;
                        break;
                }
            }
            //
            {
                int comboIndex = holeProps.GetInt("slot_in");
                switch (comboIndex)
                {
                    case 0:
                        this.SlotIn = SoltInMode.NONE;
                        break;
                    case 1:
                        this.SlotIn = SoltInMode.BEAM;
                        break;
                    case 2:
                        this.SlotIn = SoltInMode.PLATE;
                        break;
                    case 3:
                    default:
                        this.SlotIn = SoltInMode.BOTH;
                        break;
                }
            }
        }

        public void CopyFromContentProps(DObject contentProps)
        {
            if (contentProps == null)
                return;
            PartWasherUp = contentProps.GetBool("part_washer_up");
            PartWasherDown0 = contentProps.GetBool("part_washer_down0");
            PartWasherDown1 = contentProps.GetBool("part_washer_down1");
            PartNut0 = contentProps.GetBool("part_nut0");
            PartNut1 = contentProps.GetBool("part_nut1");
            uint sectFlags = 0;
            {
                switch (this.SlotIn)
                {
                    case BoltParameter.SoltInMode.BEAM:
                        sectFlags = sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE0;
                        break;
                    case BoltParameter.SoltInMode.PLATE:
                        {
                            sectFlags = sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE1;
                            sectFlags = sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE2;
                        }
                        break;
                    case BoltParameter.SoltInMode.BOTH:
                        {
                            sectFlags = sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE0;
                            sectFlags = sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE1;
                            sectFlags = sectFlags | Define.BOLT_SECT_OVERSIZED_HOLE2;
                        }
                        break;
                    default:
                        break;
                }
                if (this.PartWasherUp)
                {
                    sectFlags = sectFlags | Define.BOLT_SECT_WASHER_UP;
                }
                if (this.PartWasherDown0)
                {
                    sectFlags = sectFlags | Define.BOLT_SECT_WASHER_DOWN0;
                }
                if (this.PartWasherDown1)
                {
                    sectFlags = sectFlags | Define.BOLT_SECT_WASHER_DOWN1;
                }
                if (this.PartNut0)
                {
                    sectFlags = sectFlags | Define.BOLT_SECT_NUT0;
                }
                if (this.PartNut1)
                {
                    sectFlags = sectFlags | Define.BOLT_SECT_NUT1;
                }
                SectionFlags = sectFlags;
            }
        }

        public void CopyFromExtendProps(DObject extendProps)
        {
            if (extendProps == null)
                return;
            this.ExtLength = extendProps.GetDouble("ext_length");
        }

        private Define.PartMadeType ParseMadeType(int comboIndex)
        {
            switch (comboIndex)
            {
                case 2:
                    return Define.PartMadeType.PART_MADE_WORKSHOP;
                case 1:
                default:
                    return Define.PartMadeType.PART_MADE_SITE;
            }
        }
    
        /// <summary>
        /// 处理输入的螺栓间距
        /// </summary>
        /// <param name="count">螺栓数量</param>
        /// <param name="originArray">从UI中获取的DoubleArray</param>
        /// <returns></returns>
        public static List<double> ParseStrideInput(int count, double[] originArray)
        {
            if (originArray == null || originArray.Length == 0 || count == 0)
                return new List<double>();

            List<double> list = new List<double>();
            for (int i = 0; i < count - 1; ++i)
            {
                list.Add(originArray[Math.Min(i, originArray.Length - 1)]);
            }
            return list;
        }




    }
}
