﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;

namespace ColdRolledOverlap1
{
    public class ComponentInstanceContext
    {
        public static Plane DefaultPlane = new Plane(Vec3.ZERO + int.MaxValue * Vec3.AXIS_X, Vec3.AXIS_Z);
        private ConnectionComponent m_comp;
        private Beam m_mainPart = null; //屋面梁（椽子）
        private Beam m_subPart1 = null; //左侧檩条
        private Beam m_subPart2 = null; //右侧檩条
        private Beam m_fristSelectPart = null;
        private BeamPlanes m_mainPlanes;
        private BeamPlanes m_sub1Planes;
        private BeamPlanes m_sub2Planes;

        private Plane m_createPlane; //节点的主要创建面，在这个面上创建了夹板、拉条、拉条板（反向）
        private Plane m_createPlane2; //主要创建面是自动判断的，如果用户选择相反方向，就替换成这个面并调换主次零件

        //主零件（椽子）
        private Plane m_mainFrontWeb;
        private Plane m_mainBackWeb;
        private Plane m_mainTopBound;
        private Plane m_mainBottomBound;
        private Plane m_mainFrontBound;
        private Plane m_mainBackBound;
        private Plane m_mainBottomFlangeInside;
        private Plane m_mainTopFlangeInside;
        private Plane m_mainCenterWebPlane; 

        //次零件1（左侧檩条），次零件可以只有一个（1或2）
        private Plane m_sub1FrontWeb;
        private Plane m_sub1BackWeb;
        private Plane m_sub1TopBound;
        private Plane m_sub1BottomBound;
        private Plane m_sub1FrontBound;
        private Plane m_sub1BackBound;
        private Plane m_sub1NearCenterPlane;
        private Plane m_sub1AwayCenterPlane;
        //次零件2（右侧檩条）
        private Plane m_sub2FrontWeb;
        private Plane m_sub2BackWeb;
        private Plane m_sub2TopBound;
        private Plane m_sub2BottomBound;
        private Plane m_sub2FrontBound;
        private Plane m_sub2BackBound;
        private Plane m_sub2NearCenterPlane;
        private Plane m_sub2AwayCenterPlane;

        private BoltLocation m_cleatPurlinBoltHoriLocation; //檩条对齐面相对于节点中心的偏移，也是檩条螺栓的水平中心相对于节点中心的偏移
        private BoltLocation m_cleatPurlinBoltVertLocation; //檩条螺栓的垂直中心相对于MainTopFlange平面的偏移

        //
        private Vec3[] m_stayBoltPointLeft;
        private Vec3[] m_stayBoltPointRight;
        private SteelBeam m_cleat = null, m_leftStay = null, m_rightStay = null;
        private SteelPlate m_leftStiff = null, m_rightStiff = null, m_leftTopStiff = null, m_rightTopStiff = null;
        private double m_gap;
        private Plane m_alignCenterPlane;
        private Plane m_cleatCreatePlane;
        //
        private bool m_isOverlap;
        private Plane m_overlapCenterPlane;
        private Plane m_subBottomPlane;
        private double m_overlapHoriGap;
        private double m_overlapVertGap;
        private double m_overlapVertMarin;
        private double m_mainSubGap; //主梁与次零件的间距
        private bool[] m_overlapHoleFlag = { true, true, true, true}; //左上、左下、右上、右下的螺栓孔的开关
        private bool m_isSub1First = true; //左侧的是subPart1，可能不是第一个点击的
        private bool m_isSub1Closer = true; //subPart1是否离主零件更近
        //
        private double m_mainHeight = 0;
        private double m_mainFlangeThickness = 0;
        private double m_mainWebThickness = 0;
        private double m_mainWebFlangeWidth = 0;
        private double m_sub1WebFlangeWidth = 0;
        private double m_sub2WebFlangeWidth = 0;
        private double m_sub1WebThickness = 0;
        private double m_sub2WebThickness = 0;
        //主梁、檩条1、(檩条2)共同确定了一个空间中的方向
        public Vec3 m_leftOrient; //右方向
        public Vec3 m_topOrient; //顶方向

        ComponentInstanceContext()
        {

        }

        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (!compContext.Initialize(comp))
            {
                compContext.Clear();
                return null;
            }
            return compContext;
        }

        public bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;
            m_comp = comp;
            m_mainPart = comp.Main as Beam;
            if (comp.Subs.Length > 0)
            {
                m_subPart1 = comp.Subs.First<TFObject>() as Beam;
                m_fristSelectPart = m_subPart1;
            }
            if (comp.Subs.Length > 1)
            {
                m_subPart2 = comp.Subs[1] as Beam;
            }
            if (m_mainPart == null || m_subPart1 == null)
                return false;

            if (!ParseOrient()) //确定檩条上下方向、确定主梁上下边界面
                return false;

            /*
             * 确定创建面
             */
            Plane sub1CreatePlane1 = null, sub1CreatePlane2 = null;
            ParseCreatePlane(m_sub1Planes, ref sub1CreatePlane1, ref sub1CreatePlane2, m_subPart1.Profile.Type);
/*            Plane sub2CreatePlane1 = null, sub2CreatePlane2 = null; //暂时不用次檩条的创建面
            if (m_sub2Planes != null)
                ParseCreatePlane(m_sub2Planes, ref sub2CreatePlane1, ref sub2CreatePlane2);*/
            m_createPlane = sub1CreatePlane1;
            m_createPlane2 = sub1CreatePlane2;


            /*
             * 主梁参数
             */
            //前腹板面
            if (m_mainPlanes.FrontWeb == null)
                m_mainFrontWeb = m_mainFrontBound.Clone();
            else
                m_mainFrontWeb = m_mainPlanes.FrontWeb.Clone();
            //后腹板面
            if (m_mainPlanes.BackWeb == null)
                m_mainBackWeb = m_mainBackBound.Clone();
            else
                m_mainBackWeb = m_mainPlanes.BackWeb.Clone();
            //上翼缘内侧
            if (m_mainPlanes.TopFlangeInside == null)
                m_mainTopFlangeInside = m_mainTopBound.Invert();
            else
                m_mainTopFlangeInside = m_mainPlanes.TopFlangeInside.Clone();
            //下翼缘内侧
            if (m_mainPlanes.BottomFlangeInside == null)
                m_mainBottomFlangeInside = m_mainBottomBound.Invert();
            else
                m_mainBottomFlangeInside = m_mainPlanes.BottomFlangeInside.Clone();
            m_cleatCreatePlane = m_mainTopBound.Clone();
            //若旋转了，需改变腹板面、内翼缘面（影响拉条板创建）
            if (!IsParallel(m_mainFrontWeb, m_topOrient)) //腹板面与上方向垂直时，重新设置腹板面和翼缘面
            {
                //夹板底部创建面(选取朝上的腹板面)
                m_cleatCreatePlane = m_mainFrontWeb.Normal.Dot(m_topOrient) > 0 ? m_mainFrontWeb.Clone() : m_mainBackWeb.Clone(); 
                //腹板面
                m_mainFrontWeb = m_mainFrontBound.Clone();
                m_mainBackWeb = m_mainBackBound.Clone();
                //内翼缘面
                m_mainTopFlangeInside = m_mainTopBound.Invert();
                m_mainBottomFlangeInside = m_mainBottomBound.Invert();
            }
            m_mainHeight = m_mainTopBound.Dist(m_mainBottomBound.Origin);
            m_mainWebFlangeWidth = GetWebFlageWidth(m_mainFrontBound, m_mainBackBound, m_mainFrontWeb, m_mainBackWeb);
            if (m_mainTopFlangeInside.Normal.Dot(m_topOrient) > 0)
                Swap(ref m_mainTopFlangeInside, ref m_mainBottomFlangeInside);
            m_mainFlangeThickness = m_mainTopBound.Dist(m_mainTopFlangeInside.Origin);
            if (m_mainFlangeThickness < 1e-6)
                m_mainFlangeThickness = 20;
            m_mainCenterWebPlane = m_mainFrontBound.Move(-m_mainFrontBound.Dist(m_mainBackBound.Origin) / 2);


            /*
             * 确定近心面
             */
            m_leftOrient = m_createPlane.Normal.Cross(m_mainTopBound.Normal); //由创建面和翼缘上边界面确定的节点左侧
            if(m_sub1Planes != null)
            {
                double headCenterDistance = m_sub1Planes.HeadBound.Dist(m_mainCenterWebPlane.Origin);
                double tailCenterDistance = m_sub1Planes.TailBound.Dist(m_mainCenterWebPlane.Origin);
                if ( headCenterDistance < tailCenterDistance) //Head离center更近
                    m_sub1NearCenterPlane = m_sub1Planes.HeadBound;
                else if (headCenterDistance > tailCenterDistance)
                    m_sub1NearCenterPlane = m_sub1Planes.TailBound;
                else //间距相同，选择向左侧对齐（左侧为近心面）
                    m_sub1NearCenterPlane = m_sub1Planes.HeadBound.Normal.Dot(m_leftOrient) > 0 ? m_sub1Planes.HeadBound : m_sub1Planes.TailBound;
            }
            if (m_sub2Planes != null)
            {
                double headCenterDistance = m_sub2Planes.HeadBound.Dist(m_mainCenterWebPlane.Origin);
                double tailCenterDistance = m_sub2Planes.TailBound.Dist(m_mainCenterWebPlane.Origin);
                if (headCenterDistance < tailCenterDistance) //subPart2的近心面
                    m_sub2NearCenterPlane = m_sub2Planes.HeadBound;
                else if (headCenterDistance > tailCenterDistance)
                    m_sub2NearCenterPlane = m_sub2Planes.TailBound;
                else
                    m_sub2NearCenterPlane = m_sub2Planes.HeadBound.Normal.Dot(m_leftOrient) > 0 ? m_sub2Planes.HeadBound : m_sub2Planes.TailBound;
            }

            /*
             * 左为sub1右为sub2，实际若不是这个顺序要互换
             */
            if (m_sub1NearCenterPlane.Normal.Dot(m_leftOrient) > 0) //sub1的近心面应朝向右侧
            {
                m_isSub1First = false;
                SwapSub1Sub2();
            }
            SetPlanes();


            /*
             * 檩条参数
             */
            if(m_subPart1 != null)
            {
                if (!IsParallel(m_sub1FrontWeb, m_topOrient)) //腹板面与上方向垂直时，重新设置间距，为翼缘螺栓孔创建做准备
                {
                    m_sub1WebFlangeWidth = m_sub1FrontBound.Dist(m_sub1BackBound.Origin);
                }
                else
                {
                    m_sub1WebFlangeWidth = GetWebFlageWidth(m_sub1FrontBound, m_sub1BackBound, m_sub1FrontWeb, m_sub1BackWeb);
                    m_sub1WebThickness = m_sub1FrontWeb.Dist(m_sub1BackWeb.Origin);
                }
            }
            if(m_subPart2 != null)
            {
                if (!IsParallel(m_sub2FrontWeb, m_topOrient))
                {
                    m_sub2WebFlangeWidth = m_sub2FrontBound.Dist(m_sub2BackBound.Origin);
                }
                else
                {
                    m_sub2WebFlangeWidth = GetWebFlageWidth(m_sub2FrontBound, m_sub2BackBound, m_sub2FrontWeb, m_sub2BackWeb);
                    m_sub2WebThickness = m_sub2FrontWeb.Dist(m_sub2BackWeb.Origin);
                }
            }


            /*
             * 檩条1和檩条2哪个离主梁更近
             */
            if (m_subPart1 == null)
            {
                m_isSub1Closer = false;
                m_mainSubGap = m_cleatCreatePlane.Dist(m_sub2BottomBound.Origin);
                m_subBottomPlane = m_sub2BottomBound;
            }
            else if (m_subPart2 == null)
            {
                m_isSub1Closer = true;
                m_mainSubGap = m_cleatCreatePlane.Dist(m_sub1BottomBound.Origin);
                m_subBottomPlane = m_sub1BottomBound;
            }
            else
            {
                double dist1 = m_cleatCreatePlane.Dist(m_sub1BottomBound.Origin);
                double dist2 = m_cleatCreatePlane.Dist(m_sub2BottomBound.Origin);
                if (dist1 <= dist2)
                {
                    m_isSub1Closer = true;
                    m_mainSubGap = dist1;
                    m_subBottomPlane = m_sub1BottomBound;
                }
                else
                {
                    m_isSub1Closer = false;
                    m_mainSubGap = dist2;
                    m_subBottomPlane = m_sub2BottomBound;
                }
            }

            return true;
        }

        //腹板到边界面的距离
        public double GetWebFlageWidth(Plane frontBound, Plane backBound, Plane frontWeb, Plane backweb)
        {
            double gap;
            double d1 = frontBound.Dist(frontWeb.Origin);
            double d2 = backBound.Dist(backweb.Origin);
            gap = d1 > d2 ? d1 : d2;
            if (gap < 1e-4)
                gap = 70;
            return gap;
        }

        public bool ParseOrient()
        {
            /*
             * 先获取零件的边界面
             */
            //主
            m_mainPlanes = BeamPlanes.CreateFromBeam(m_mainPart);
            m_mainTopBound = m_mainPlanes.TopBound.Clone();
            m_mainBottomBound = m_mainPlanes.BottomBound.Clone();
            m_mainFrontBound = m_mainPlanes.FrontBound.Clone();
            m_mainBackBound = m_mainPlanes.BackBound.Clone();
            //次1
            m_sub1Planes = BeamPlanes.CreateFromBeam(m_subPart1);
            m_sub1TopBound = m_sub1Planes.TopBound.Clone();
            m_sub1BottomBound = m_sub1Planes.BottomBound.Clone();
            m_sub1FrontBound = m_sub1Planes.FrontBound.Clone();
            m_sub1BackBound = m_sub1Planes.BackBound.Clone();
            m_sub1FrontWeb = m_sub1Planes.FrontWeb.Clone();
            m_sub1BackWeb = m_sub1Planes.BackWeb.Clone();
            //次2
            if (m_subPart2 != null)
            {
                m_sub2Planes = BeamPlanes.CreateFromBeam(m_subPart2);
                m_sub2TopBound = m_sub2Planes.TopBound.Clone();
                m_sub2BottomBound = m_sub2Planes.BottomBound.Clone();
                m_sub2FrontBound = m_sub2Planes.FrontBound.Clone();
                m_sub2BackBound = m_sub2Planes.BackBound.Clone();
                m_sub2FrontWeb = m_sub2Planes.FrontWeb.Clone();
                m_sub2BackWeb = m_sub2Planes.BackWeb.Clone();
            }

            /*
             * 确定上方向并互换边界面
             */
            //主梁上下边界面
            if (!IsVertical(m_sub1Planes.HeadBound, m_mainTopBound)) //上下与左右互换
            {
                Swap(ref m_mainTopBound, ref m_mainFrontBound);
                Swap(ref m_mainBottomBound, ref m_mainBackBound);
            }
            //次梁上下边界面
            if (!IsVertical(m_mainPlanes.HeadBound, m_sub1TopBound))
            {
                Swap(ref m_sub1TopBound, ref m_sub1FrontBound);
                Swap(ref m_sub1BottomBound, ref m_sub1BackBound);
            }
            if (m_subPart2 != null && !IsVertical(m_mainPlanes.HeadBound, m_sub2TopBound))
            {
                Swap(ref m_sub2TopBound, ref m_sub2FrontBound);
                Swap(ref m_sub2BottomBound, ref m_sub2BackBound);
            }
            //确定上方向（主梁上下边界面）
            Plane subTopBottomCenterPlane = m_sub1TopBound.Move(-m_sub1TopBound.Dist(m_sub1BottomBound.Origin) / 2);
            if (m_mainBottomBound.Dist(subTopBottomCenterPlane.Origin) < m_mainTopBound.Dist(subTopBottomCenterPlane.Origin))
            {
                Swap(ref m_mainTopBound, ref m_mainBottomBound); //上下面反了，换位置
            }
            m_topOrient = m_mainTopBound.Normal; //上方向

            return true;
        }

        public void ParseCreatePlane(BeamPlanes beamPlanes, ref Plane p1, ref Plane p2, String profile)
        {
            /*
             * 原理：
             * 1.获取上下外翼缘面、上下内翼缘面、腹板面共6个面（如果存在），将与上方向平行的面筛选出来。
             * 2.将符合条件的面分成两组，一组为主梁head方向，一组为tail方向，两组面各自做相同处理
             * 3.①若是翼缘面平行于上方向，就取这组面的外翼缘面（dist=0），若取该方向的内翼缘面(dist=1)
             *   ②若是腹板面平行于上方向，两组面各自都只有一个面
             * 4.自动判断应该创建在哪一侧，这个只针对3.2的情况。取dist小的做创建面（Z单独处理）
             */
            //与top方向平行的平面
            List<Plane> horiPlaneList = new List<Plane>();
            List<double> distList = new List<double>();
            if (beamPlanes.TopFlange != null && IsParallel(beamPlanes.TopFlange, m_topOrient))
            {
                horiPlaneList.Add(beamPlanes.TopFlange);
                distList.Add(0);
            }
            if (beamPlanes.TopFlangeInside != null && IsParallel(beamPlanes.TopFlangeInside, m_topOrient))
            {
                horiPlaneList.Add(beamPlanes.TopFlangeInside);
                distList.Add(1);
            }
            if (beamPlanes.BottomFlange != null && IsParallel(beamPlanes.BottomFlange, m_topOrient))
            {
                horiPlaneList.Add(beamPlanes.BottomFlange);
                distList.Add(0);
            }
            if (beamPlanes.BottomFlangeInside != null && IsParallel(beamPlanes.BottomFlangeInside, m_topOrient))
            {
                horiPlaneList.Add(beamPlanes.BottomFlangeInside);
                distList.Add(1);
            }
            if (beamPlanes.FrontWeb != null && IsParallel(beamPlanes.FrontWeb, m_topOrient))
            {
                horiPlaneList.Add(beamPlanes.FrontWeb);
                distList.Add(beamPlanes.FrontWeb.Dist(beamPlanes.FrontBound.Origin));
            }
            if (beamPlanes.BackWeb != null && IsParallel(beamPlanes.BackWeb, m_topOrient))
            {
                horiPlaneList.Add(beamPlanes.BackWeb);
                distList.Add(beamPlanes.BackWeb.Dist(beamPlanes.BackBound.Origin));
            }

            //离外侧更近的面
            Vec3 headOrient = m_mainPlanes.HeadBound.Normal;
            Vec3 tailOrient = m_mainPlanes.TailBound.Normal;
            Plane createPlane1 = null; //head方向
            double dist1 = double.MaxValue;
            Plane createPlane2 = null; //tail方向
            double dist2 = double.MaxValue;
            for (int i = 0; i < horiPlaneList.Count; i++)
            {
                if (horiPlaneList[i].Normal.Dot(headOrient) > 0) //当前面为head方向
                {
                    if (createPlane1 == null || distList[i] < dist1)
                    {
                        createPlane1 = horiPlaneList[i];
                        dist1 = distList[i];
                    }
                }
                else //当前面为tail方向
                {
                    if (createPlane2 == null || distList[i] < dist2)
                    {
                        createPlane2 = horiPlaneList[i];
                        dist2 = distList[i];
                    }
                }
            }

            //远近判断(自动确定创建面)
            p1 = createPlane1.Clone();
            p2 = createPlane2.Clone();
            if (profile.Equals("Z") || profile.Equals("ZZ")) //碰到了Z/ZZ截面，单独处理
            {
                Vec3 createOrient = beamPlanes.BackBound.Normal;
                if (beamPlanes.TopBound.Normal.Dot(m_topOrient) < 0) //Top朝下了，说明檩条转了180度
                    createOrient = beamPlanes.FrontBound.Normal;
                if(createPlane2.Normal.Dot(createOrient) > 0) //createPlane2方向才是对的
                    Swap(ref p1, ref p2);
            }
            else if (dist2 < dist1)
                Swap(ref p1, ref p2);
        }

        public void SwapSub1Sub2() //1、2位置相反（应为左1右2）或用户手动更改创建面时，需进行调换
        {
            //交换左右零件
            Beam tempBeam = m_subPart1;
            m_subPart1 = m_subPart2;
            m_subPart2 = tempBeam;

            //交换左右平面
            BeamPlanes tempPlanes = m_sub1Planes;
            m_sub1Planes = m_sub2Planes;
            m_sub2Planes = tempPlanes;

            //左右檩条参数
            double tempH = m_sub1WebFlangeWidth;
            m_sub1WebFlangeWidth = m_sub2WebFlangeWidth;
            m_sub2WebFlangeWidth = tempH;

            //交换基础面
            Swap(ref m_sub1FrontWeb, ref m_sub2FrontWeb);
            Swap(ref m_sub1BackWeb, ref m_sub2BackWeb);
            Swap(ref m_sub1TopBound, ref m_sub2TopBound);
            Swap(ref m_sub1BottomBound, ref m_sub2BottomBound);
            Swap(ref m_sub1FrontBound, ref m_sub2FrontBound);
            Swap(ref m_sub1BackBound, ref m_sub2BackBound);
            Swap(ref m_sub1NearCenterPlane, ref m_sub2NearCenterPlane);
            Swap(ref m_sub1AwayCenterPlane, ref m_sub1AwayCenterPlane);
            Swap(ref m_mainFrontWeb, ref m_mainBackWeb);
            Swap(ref m_mainFrontBound, ref m_mainBackBound);
            m_mainCenterWebPlane.InvertThis();
            m_isSub1Closer = !m_isSub1Closer;

            
        }

        public bool IsParallel(Plane p1, Plane p2)
        {
            return p1.Normal.Cross(p2.Normal).Dist(Vec3.ZERO) < 1e-3;
        }
        public bool IsParallel(Plane p1, Vec3 vec)
        {
            return Math.Abs(p1.Normal.Dot(vec)) < 1e-3;
        }
        public bool IsVertical(Plane p1, Plane p2)
        {
            return Math.Abs(p1.Normal.Dot(p2.Normal)) < 1e-3;
        }
        public bool IsVertical(Plane p1, Vec3 vec)
        {
            return p1.Normal.Cross(vec).Dist(Vec3.ZERO) < 1e-3;
        }

        public void SetPlanes() //设置基础面
        {
            Vec3 purlinFront = m_createPlane.Normal;
            //主梁前后面
            if (m_mainCenterWebPlane.Normal.Dot(m_leftOrient) > 0) //center因该朝右
            {
                Swap(ref m_mainFrontWeb, ref m_mainBackWeb);
                Swap(ref m_mainFrontBound, ref m_mainBackBound);
                m_mainCenterWebPlane.InvertThis();
            }
            //主梁上下内翼缘面
            if (m_mainTopFlangeInside.Normal.Dot(m_topOrient) > 0)
            {
                Swap(ref m_mainTopFlangeInside,ref m_mainBottomFlangeInside);
            }
            //左檩条
            if (m_subPart1 != null)
            {
                //上下面
                if (m_sub1TopBound.Normal.Dot(m_mainTopBound.Normal) < 0) //檩条上下面必须和主梁方向一致
                {
                    Swap(ref m_sub1TopBound, ref m_sub1BottomBound);
                }
                //HeadTail方向
                if (m_sub1FrontBound.Normal.Dot(purlinFront) < 0)
                {
                    Swap(ref m_sub1FrontWeb, ref m_sub1BackWeb);
                    Swap(ref m_sub1FrontBound, ref m_sub1BackBound);
                }
            }
            //右檩条
            if (m_subPart2 != null)
            {
                //
                if (m_sub2TopBound.Normal.Dot(m_mainTopBound.Normal) < 0)
                {
                    Swap(ref m_sub2TopBound, ref m_sub2BottomBound);
                }
                //
                if (m_sub2FrontBound.Normal.Dot(purlinFront) < 0)
                {
                    Swap(ref m_sub2FrontWeb, ref m_sub2BackWeb);
                    Swap(ref m_sub2FrontBound, ref m_sub2BackBound);
                }
            }



        }

        public void Swap(ref Plane p1, ref Plane p2)
        {
            Plane p = p1;
            p1 = p2;
            p2 = p;
        }

        public void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_subPart1 = null;
            m_subPart2 = null;
            m_mainPlanes = null;
            m_sub1Planes = null;
            m_sub2Planes = null;
        }
        
        public void test(Plane p)
        {
            Comp.CreateQuad(p, 4000, 4000, true);
        }
        public void test(Vec3 p)
        {
            Comp.CreatePoint(p, true);
        }
        public void test(Vec3 v1, Vec3 v2)
        {
            Comp.CreateLine(v1, v2, 1000, Color.Red, true);
        }


        public ConnectionComponent Comp { get => m_comp; set => m_comp = value; }
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public Beam SubPart1 { get => m_subPart1; set => m_subPart1 = value; }
        public Beam SubPart2 { get => m_subPart2; set => m_subPart2 = value; }
        public Beam FristSelectPart { get => m_fristSelectPart; set => m_fristSelectPart = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public BeamPlanes Sub1Planes { get => m_sub1Planes; set => m_sub1Planes = value; }
        public BeamPlanes Sub2Planes { get => m_sub2Planes; set => m_sub2Planes = value; }
        public Plane Sub1NearCenterPlane { get => m_sub1NearCenterPlane; set => m_sub1NearCenterPlane = value; }
        public Plane Sub2NearCenterPlane { get => m_sub2NearCenterPlane; set => m_sub2NearCenterPlane = value; }

        /// <summary>
        /// 腹板中心面（平行于腹板，在腹板中心）
        /// </summary>
        public Plane MainCenterWebPlane { get => m_mainCenterWebPlane; set => m_mainCenterWebPlane = value; }

        /// <summary>
        /// 对齐SubPart1和SubPart2的中心面，方向朝右
        /// </summary>
        public Plane AlignCenterPlane { get => m_alignCenterPlane; set => m_alignCenterPlane = value; }

        /// <summary>
        /// 重叠时的中心面
        /// </summary>
        public Plane OverlapCenterPlane { get => m_overlapCenterPlane; set => m_overlapCenterPlane = value; }
        public Plane Sub1AwayCenterPlane { get => m_sub1AwayCenterPlane; set => m_sub1AwayCenterPlane = value; }
        public Plane Sub2AwayCenterPlane { get => m_sub2AwayCenterPlane; set => m_sub2AwayCenterPlane = value; }

        /// <summary>
        /// 节点的主要创建面，在这个面上创建了夹板、拉条、拉条板（反向）
        /// </summary>
        public Plane CreatePlane { get => m_createPlane; set => m_createPlane = value; }
        public Plane CreatePlane2 { get => m_createPlane2; set => m_createPlane2 = value; }
        public Plane CleatCreatePlane { get => m_cleatCreatePlane; set => m_cleatCreatePlane = value; }
        public Plane MainTopBound { get => m_mainTopBound; set => m_mainTopBound = value; }
        public Plane MainTopFlangeInside { get => m_mainTopFlangeInside; set => m_mainTopFlangeInside = value; }
        public Plane MainBottomFlangeInside { get => m_mainBottomFlangeInside; set => m_mainBottomFlangeInside = value; }
        public Plane MainBottomBound { get => m_mainBottomBound; set => m_mainBottomBound = value; }
        /// <summary>
        /// 离主梁最近的檩条下边界面
        /// </summary>
        public Plane SubBottomPlane { get => m_subBottomPlane; set => m_subBottomPlane = value; }
        public Plane Sub1BottomBound { get => m_sub1BottomBound; set => m_sub1BottomBound = value; }
        public Plane Sub2BottomBound { get => m_sub2BottomBound; set => m_sub2BottomBound = value; }
        public Plane Sub1TopBound { get => m_sub1TopBound; set => m_sub1TopBound = value; }
        public Plane Sub2TopBound { get => m_sub2TopBound; set => m_sub2TopBound = value; }
        public Plane Sub1FrontBound { get => m_sub1FrontBound; set => m_sub1FrontBound = value; }
        public Plane Sub2FrontBound { get => m_sub2FrontBound; set => m_sub2FrontBound = value; }
        public Plane Sub1BackBound { get => m_sub1BackBound; set => m_sub1BackBound = value; }
        public Plane Sub2BackBound { get => m_sub2BackBound; set => m_sub2BackBound = value; }
        public double Sub1WebThickness { get => m_sub1WebThickness; set => m_sub1WebThickness = value; }
        public double Sub2WebThickness { get => m_sub2WebThickness; set => m_sub2WebThickness = value; }
        public Plane MainFrontWeb { get => m_mainFrontWeb; set => m_mainFrontWeb = value; }
        public Plane MainBackWeb { get => m_mainBackWeb; set => m_mainBackWeb = value; }
        public Plane MainFrontBound { get => m_mainFrontBound; set => m_mainFrontBound = value; }
        public Plane MainBackBound { get => m_mainBackBound; set => m_mainBackBound = value; }
        public BoltLocation CleatPurlinBoltHoriLocation { get => m_cleatPurlinBoltHoriLocation; set => m_cleatPurlinBoltHoriLocation = value; }
        public BoltLocation CleatPurlinBoltVertLocation { get => m_cleatPurlinBoltVertLocation; set => m_cleatPurlinBoltVertLocation = value; }

        /// <summary>
        /// 左拉条的两个螺栓定位点，第一个在下侧，第二个在上侧
        /// </summary>
        public Vec3[] StayBoltPointLeft { get => m_stayBoltPointLeft; set => m_stayBoltPointLeft = value; }
        public Vec3[] StayBoltPointRight { get => m_stayBoltPointRight; set => m_stayBoltPointRight = value; }
        public SteelBeam Cleat { get => m_cleat; set => m_cleat = value; }
        //左拉条
        public SteelBeam LeftStay { get => m_leftStay; set => m_leftStay = value; }

        //右拉条
        public SteelBeam RightStay { get => m_rightStay; set => m_rightStay = value; }

        /// <summary>
        /// 左侧加劲肋，与左拉条和主梁连接
        /// </summary>
        public SteelPlate LeftStiff { get => m_leftStiff; set => m_leftStiff = value; }


        /// <summary>
        /// 右侧加劲肋，与右拉条和主梁连接
        /// </summary>
        public SteelPlate RightStiff { get => m_rightStiff; set => m_rightStiff = value; }
        public SteelPlate LeftTopStiff { get => m_leftTopStiff; set => m_leftTopStiff = value; }
        public SteelPlate RightTopStiff { get => m_rightTopStiff; set => m_rightTopStiff = value; }

        //主次零件之间的间距
        public double Gap { get => m_gap; set => m_gap = value; }

        /// <summary>
        /// 是否重叠
        /// </summary>
        public bool IsOverlap { get => m_isOverlap; set => m_isOverlap = value; }
        public double OverlapHoriGap { get => m_overlapHoriGap; set => m_overlapHoriGap = value; }
        public double OverlapVertGap { get => m_overlapVertGap; set => m_overlapVertGap = value; }
        public double OverlapVertMarin { get => m_overlapVertMarin; set => m_overlapVertMarin = value; }
        public double MainSubGap { get => m_mainSubGap; set => m_mainSubGap = value; }
        public double MainHeight { get => m_mainHeight; set => m_mainHeight = value; }
        public double MainFlangeThickness { get => m_mainFlangeThickness; set => m_mainFlangeThickness = value; }
        public double MainWebThickness { get => m_mainWebThickness; set => m_mainWebThickness = value; }
        public double MainWebFlangeWidth { get => m_mainWebFlangeWidth; set => m_mainWebFlangeWidth = value; }
        public double Sub1WebFlangeWidth { get => m_sub1WebFlangeWidth; set => m_sub1WebFlangeWidth = value; }
        public double Sub2WebFlangeWidth { get => m_sub2WebFlangeWidth; set => m_sub2WebFlangeWidth = value; }
        public bool[] OverlapHoleFlag { get => m_overlapHoleFlag; set => m_overlapHoleFlag = value; }
        public bool IsSub1First { get => m_isSub1First; set => m_isSub1First = value; }
        public bool IsSub1Closer { get => m_isSub1Closer; set => m_isSub1Closer = value; }

    }
}
