﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.Util;

namespace ColdRolledOverlap1
{
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {

        private DObject m_rootUIProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;
        public override ComponentPlugin ReferencePlugin => m_refPlugin;

        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostIntialize();
        }

        private void OnPostIntialize()
        {
            foreach (TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }
            //
            this.tb_cleat_mode.ClearItems();
            this.tb_cleat_mode.AddIntegerItems(0, 0, 5);
            this.tb_cleat_mode.UpdateItems();

            this.tb_create_orient.ClearItems();
            this.tb_create_orient.AddIntegerItems(0, 0, 2);
            this.tb_create_orient.UpdateItems();

            this.tb_cleat_rafter_connect_mode.ClearItems();
            this.tb_cleat_rafter_connect_mode.AddIntegerItems(0, 0, 3);
            this.tb_cleat_rafter_connect_mode.UpdateItems();

            this.tb_stay_stiff_chamfer_type.ClearItems();
            this.tb_stay_stiff_chamfer_type.AddIntegerItems(0, 0, 5);
            this.tb_stay_stiff_chamfer_type.UpdateItems();

            this.tb_stay_place.ClearItems();
            this.tb_stay_place.AddIntegerItems(0, 0, 7);
            this.tb_stay_place.UpdateItems();

            this.tb_stay_vert_location_bolt.ClearItems();
            this.tb_stay_vert_location_bolt.AddIntegerItems(0, 0, 4);
            this.tb_stay_vert_location_bolt.UpdateItems();

            this.tb_purin_web_hole.ClearItems();
            this.tb_purin_web_hole.AddIntegerItems(0, 0, 3);
            this.tb_purin_web_hole.UpdateItems(); 

            this.tb_overlap_mode.ClearItems();
            this.tb_overlap_mode.AddIntegerItems(0, 0, 3);
            this.tb_overlap_mode.UpdateItems();

            this.tb_overlap_bolt_place.ClearItems();
            this.tb_overlap_bolt_place.AddIntegerItems(0, 0, 3);
            this.tb_overlap_bolt_place.UpdateItems();

            this.tb_overlap_bolt_hole.ClearItems();
            this.tb_overlap_bolt_hole.AddIntegerItems(0, 0, 3);
            this.tb_overlap_bolt_hole.UpdateItems();

            this.tb_cleat_notch_open.ClearItems();
            this.tb_cleat_notch_open.AddIntegerItems(0, 0, 3);
            this.tb_cleat_notch_open.UpdateItems();
            this.tb_cleat_notch_open.SelectedKey = 0;

            this.tb_stay_top_stiff_open.ClearItems();
            this.tb_stay_top_stiff_open.AddIntegerItems(0, 0, 3);
            this.tb_stay_top_stiff_open.UpdateItems();
            this.tb_stay_top_stiff_open.SelectedKey = 0;

            //默认关闭夹板加劲肋
            SetStiffEnable(false);
            SetVertStiffEnable(false);

            //默认焊接夹板与椽子
            this.tb_cleat_rafter_bolt_gap_hori.Enabled = false;
            this.tb_cleat_rafter_bolt_count_vert.Enabled = false;
            this.tb_cleat_rafter_bolt_stride_vert.Enabled = false;
            this.tb_cleat_rafter_bolt_inside_margin_vert.Enabled = false;
            //this.tb_create_plane_offset.Enabled = false;
            this.act_cleat_rafter_bolt_gap_hori.Enabled = false;
            this.act_cleat_rafter_bolt_count_vert.Enabled = false;
            this.act_cleat_rafter_bolt_inside_margin_vert.Enabled = false;
            //this.act_create_plane_offset.Enabled = false;

            //默认不折角拉条板
            this.tb_stay_stiff_chamfer_x.Enabled = false;
            this.tb_stay_stiff_chamfer_y.Enabled = false;
            this.act_stay_stiff_chamfer_x.Enabled = false;
            this.act_stay_stiff_chamfer_y.Enabled = false;

            //
            this.tb_cleat_rafter_bolt_basic_props.BoltStandardChanged += this.tb_cleat_rafter_bolt_content_props.AdaptBoltContents;
            this.tb_cleat_bolt_basic_props.BoltStandardChanged += this.tb_cleat_bolt_content_props.AdaptBoltContents;
        }

        public override void ClearUI(Control root = null)
        {
            // 刷新螺栓基础面板
            this.tb_cleat_bolt_basic_props.RefreshControls();
            this.tb_cleat_rafter_bolt_basic_props.RefreshControls();
            this.tb_stay_purlin_bolt_basic_props.RefreshControls();
            this.tb_stay_rafter_bolt_basic_props.RefreshControls();
            base.ClearUI(root);
        }

        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            // 螺栓垫片设置
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null)
            {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }

        private void tb_cleat_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_cleat_mode.SelectedIntKey;
            switch (select)
            {
                case 3:
                    SetStiffEnable(true);
                    SetVertStiffEnable(false);
                    break;
                case 4:
                    SetStiffEnable(false);
                    SetVertStiffEnable(true);
                    break;
                default:
                    SetStiffEnable(false);
                    SetVertStiffEnable(false);
                    break;

            }
        }

        //L形夹板时，加劲肋参数是否为灰色
        private void SetStiffEnable(bool status)
        {
            this.tb_cleat_stiff_top_chamfer_x.Enabled = status;
            this.tb_cleat_stiff_top_chamfer_y.Enabled = status;
            this.tb_cleat_stiff_bottom_outside_chamfer_x.Enabled = status;
            this.tb_cleat_stiff_bottom_outside_chamfer_y.Enabled = status;
            this.tb_cleat_stiff_bottom_inside_chamfer_x.Enabled = status;
            this.tb_cleat_stiff_bottom_inside_chamfer_y.Enabled = status;
            this.act_cleat_stiff_top_chamfer_x.Enabled = status;
            this.act_cleat_stiff_top_chamfer_y.Enabled = status;
            this.act_cleat_stiff_bottom_outside_chamfer_x.Enabled = status;
            this.act_cleat_stiff_bottom_outside_chamfer_y.Enabled = status;
            this.act_cleat_stiff_bottom_inside_chamfer_x.Enabled = status;
            this.act_cleat_stiff_bottom_inside_chamfer_y.Enabled = status;
        }

        //板形截面夹板时，加劲肋参数是否为灰色
        private void SetVertStiffEnable(bool status)
        {
            this.tb_cleat_stiff_margin_top.Enabled = status;
            this.tb_cleat_stiff_margin_inside.Enabled = status;
            this.act_cleat_stiff_margin_top.Enabled = status;
            this.act_cleat_stiff_margin_inside.Enabled = status;
        }

        private void tb_cleat_rafter_connect_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_cleat_rafter_connect_mode.SelectedIntKey;
            if (select == 1)
            {
                this.tb_cleat_rafter_bolt_gap_hori.Enabled = true;
                this.tb_cleat_rafter_bolt_count_vert.Enabled = true;
                this.tb_cleat_rafter_bolt_stride_vert.Enabled = true;
                this.tb_cleat_rafter_bolt_inside_margin_vert.Enabled = true;
                this.act_cleat_rafter_bolt_gap_hori.Enabled = true;
                this.act_cleat_rafter_bolt_count_vert.Enabled = true;
                this.act_cleat_rafter_bolt_inside_margin_vert.Enabled = true;
                //this.tb_create_plane_offset.Enabled = true;
                //this.act_create_plane_offset.Enabled = true;
                tb_cleat_rafter_bolt_basic_props.Enabled = true;
                tb_cleat_rafter_bolt_content_props.Enabled = true;
            }
            else
            {
                this.tb_cleat_rafter_bolt_gap_hori.Enabled = false;
                this.tb_cleat_rafter_bolt_count_vert.Enabled = false;
                this.tb_cleat_rafter_bolt_stride_vert.Enabled = false;
                this.tb_cleat_rafter_bolt_inside_margin_vert.Enabled = false;
                this.act_cleat_rafter_bolt_gap_hori.Enabled = false;
                this.act_cleat_rafter_bolt_count_vert.Enabled = false;
                this.act_cleat_rafter_bolt_inside_margin_vert.Enabled = false;
                //this.tb_create_plane_offset.Enabled = false;
                //this.act_create_plane_offset.Enabled = false;
                tb_cleat_rafter_bolt_basic_props.Enabled = false;
                tb_cleat_rafter_bolt_content_props.Enabled = false;
            }
        }

        private void tb_stay_stiff_chamfer_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_stay_stiff_chamfer_type.SelectedIntKey;
            switch (select)
            {
                case 2:
                    this.tb_stay_stiff_chamfer_x.Enabled = true;
                    this.tb_stay_stiff_chamfer_y.Enabled = true;
                    this.act_stay_stiff_chamfer_x.Enabled = true;
                    this.act_stay_stiff_chamfer_y.Enabled = true;
                    break;
                case 3:
                    this.tb_stay_stiff_chamfer_x.Enabled = true;
                    this.tb_stay_stiff_chamfer_y.Enabled = false;
                    this.act_stay_stiff_chamfer_x.Enabled = true;
                    this.act_stay_stiff_chamfer_y.Enabled = false;
                    break;
                case 4:
                    this.tb_stay_stiff_chamfer_x.Enabled = true;
                    this.tb_stay_stiff_chamfer_y.Enabled = false;
                    this.act_stay_stiff_chamfer_x.Enabled = true;
                    this.act_stay_stiff_chamfer_y.Enabled = false;
                    break;
                default:
                    this.tb_stay_stiff_chamfer_x.Enabled = false;
                    this.tb_stay_stiff_chamfer_y.Enabled = false;
                    this.act_stay_stiff_chamfer_x.Enabled = false;
                    this.act_stay_stiff_chamfer_y.Enabled = false;
                    break;
            }
        }


        private void tb_cleat_notch_open_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_cleat_notch_open.SelectedIntKey;
            if (select == 1) //开启开槽
                EnableNotch(true);
            else
                EnableNotch(false);
        }

        private void EnableNotch(bool b)
        {
            this.act_notch_margin_center.Enabled = b;
            this.tb_notch_margin_center.Enabled = b;
            this.act_notch_margin_side.Enabled = b;
            this.tb_notch_margin_side.Enabled = b;
        }


        private void btn_weld_props_Click_1(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_rootUIProps, null);
        }

        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_rootUIProps.CopyFrom(srcProps);
        }

        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }

        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_rootUIProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_rootUIProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_rootUIProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_rootUIProps;

            return comp.UIProps;
        }


        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked)
            {
                retProps.CopyFrom(m_rootUIProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }


    }
}
