﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Util;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Plugins;
using LoongCAD.STC.Object3D;

namespace ColdRolledOverlap1
{
    class BoltHandler
    {
        private ComponentInstanceContext m_context = null; 
        private ParamsHandler m_paramsHandler = null;
        private DObject[] stayPurlinBoltProps;
        private DObject[] stayRafterBoltProps;
        private DObject[] cleatBoltProps;
        private List<double> stayPurlinBoltStride;
        private List<double> stayRafterBoltStride;
        //重叠搭接时的螺栓参数
        private bool isOpenHole; //是否在对应位置打孔
        private bool isWebBolt; //在腹板上加螺栓还是在翼缘上加螺栓
        private Mat43 workAxis;

        public BoltHandler(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {

            //工作坐标系设置
            Vec3 webx = m_context.MainTopBound.Normal;
            Vec3 weby = m_context.MainBackBound.Normal;
            workAxis = new Mat43(webx, weby, webx.Cross(weby), Vec3.ZERO); //左中右共用一个工作坐标系 

            /*
             * 夹板与檩条的螺栓
             */
            //螺栓属性获取
            DObject purlinBoltbasicProps = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_BOLT_BASIC_PROPS);
            DObject purlinBoltholeProps = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_BOLT_HOLE_PROPS);
            DObject purlinBoltcontentProps = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_BOLT_CONTENT_PROPS);
            DObject purlinBoltextendProps = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_BOLT_EXTEND_PROPS);
            cleatBoltProps = new DObject[] { purlinBoltbasicProps, purlinBoltholeProps, purlinBoltcontentProps, purlinBoltextendProps };
            if (m_context.CleatPurlinBoltHoriLocation != null && m_context.CleatPurlinBoltVertLocation != null 
                && m_context.CleatPurlinBoltHoriLocation.count > 0 && m_context.CleatPurlinBoltVertLocation.count > 0)
            {
                List<double> horiStride = m_context.CleatPurlinBoltHoriLocation.stride; //螺栓水平方向总是居中的，所以只需要排列间距
                if(m_context.SubPart1 != null && m_context.SubPart2 != null)
                {
                    if (!m_context.IsOverlap) //正常情况，螺栓分2个区间创建：左檩条、右檩条
                    {
                        List<double> horiStrideRight, horiStrideLeft; //两组螺栓的间距数组
                        double horiOffsetLeft = 0, horiOffsetRight = 0; //每一组螺栓的最右侧的那颗螺栓的坐标
                        double detect = horiStride.Sum() / 2; //当前坐标
                        int i = 0; //右侧螺栓间距数组大小（右侧螺栓数量 - 1）
                        if(horiStride.Sum() > 0) //数量为0时只创建一个
                        {
                            while (true) //不断比较右侧螺栓的左边缘与哪个次零件更近
                            {
                                double leftDistance = Math.Abs(detect - (-m_context.Gap / 2)); //与左零件(SubPart1)边缘(NearCenter面)的距离
                                double rightDistance = Math.Abs(detect - m_context.Gap / 2); //与右零件边缘的距离
                                if (leftDistance < rightDistance || i == horiStride.Count)//如果是最后一个间距或离左侧更近，就不用再向左了
                                {
                                    detect += horiStride[--i]; //回退并结束探测
                                    break;
                                }
                                else
                                    detect -= horiStride[i++]; //若rightDistance>=leftDistance，继续向左探测
                            }
                        }
                        horiStrideRight = horiStride.GetRange(0, i); //
                        if (i + 1 < horiStride.Count)
                            horiStrideLeft = horiStride.GetRange(i + 1, horiStride.Count - i - 1); //有一个水平间距用来容纳子零件之间的间距了，所以要排除掉
                        else
                            horiStrideLeft = new List<double>();
                        //左侧夹板螺栓创建
                        if (i >= 0 && i < horiStride.Count)
                            horiOffsetLeft = detect - horiStride[i];
                        CreateCleatBoltGroup(horiOffsetLeft, horiStrideLeft, new PartBase[] { m_context.SubPart1 });
                        //右侧夹板螺栓创建
                        horiOffsetRight = detect + horiStrideRight.Sum(); //移到最右侧
                        CreateCleatBoltGroup(horiOffsetRight, horiStrideRight, new PartBase[] { m_context.SubPart2 });
                    }
                    else //檩条重叠时的情况，螺栓分3个区间创建：左侧、右侧、重叠区域
                    {
                        double detect = horiStride.Sum() / 2; //当前坐标
                        double left = m_context.AlignCenterPlane.SignedDist(m_context.Sub2NearCenterPlane.Origin); //分割左侧与重叠区域的点坐标
                        double right = m_context.AlignCenterPlane.SignedDist(m_context.Sub1NearCenterPlane.Origin);
                        double leftOffset = double.MaxValue, rightOffset = double.MaxValue, middleOffset = double.MaxValue;
                        List<double> leftStride = new List<double>();
                        List<double> middleStride = new List<double>();
                        List<double> rightStride = new List<double>();
                        List<double> boltPointLocation = new List<double>() { detect };
                        for (int i = 0; i < horiStride.Count; i++)
                        {
                            detect -= horiStride[i];
                            boltPointLocation.Add(detect);
                        }
                        int index = 0;
                        for (; index < boltPointLocation.Count && boltPointLocation[index] >= right; index++)
                        {
                            rightOffset = boltPointLocation[index];
                            if (index + 1 < boltPointLocation.Count && boltPointLocation[index + 1] >= right)
                                rightStride.Add(horiStride[index]);
                        }
                        for (; index < boltPointLocation.Count && boltPointLocation[index] > left; index++)
                        {
                            middleOffset = boltPointLocation[index];
                            if (index + 1 < boltPointLocation.Count && boltPointLocation[index + 1] >= left)
                                middleStride.Add(horiStride[index]);
                        }
                        for (; index < boltPointLocation.Count; index++)
                        {
                            leftOffset = boltPointLocation[index];
                            if (index + 1 < boltPointLocation.Count)
                                leftStride.Add(horiStride[index]);
                        }
                        if (rightOffset != double.MaxValue)
                        {
                            rightOffset += rightStride.Sum();
                            CreateCleatBoltGroup(rightOffset, rightStride, new PartBase[] { m_context.SubPart2 });
                        }
                        if (middleOffset != double.MaxValue)
                        {
                            middleOffset += middleStride.Sum();
                            CreateCleatBoltGroup(middleOffset, middleStride, new PartBase[] { m_context.SubPart1, m_context.SubPart2 });
                        }
                        if (leftOffset != double.MaxValue)
                        {
                            leftOffset += leftStride.Sum();
                            CreateCleatBoltGroup(leftOffset, leftStride, new PartBase[] { m_context.SubPart1 });
                        }

                    }
                }
                else
                {
                    PartBase part = m_context.FristSelectPart;
                    CreateCleatBoltGroup(horiStride.Sum() / 2, horiStride, new PartBase[] { part });
                }
            }

            /*
             * 夹板与椽子的螺栓
             */
            if (m_context.CleatPurlinBoltHoriLocation != null && m_context.CleatPurlinBoltHoriLocation.count > 0)
            {
                //螺栓属性获取
                DObject cleatRafterBoltbasicProps = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_RAFTER_BOLT_BASIC_PROPS);
                DObject cleatRafterBoltcontentProps = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_RAFTER_BOLT_CONTENT_PROPS);
                DObject cleatRafterBoltholeProps = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_BOLT_HOLE_PROPS);
                DObject cleatRafterBoltextendProps = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_BOLT_EXTEND_PROPS);
                DObject[] cleatRafterBoltProps = { cleatRafterBoltbasicProps, cleatRafterBoltholeProps, cleatRafterBoltcontentProps, cleatRafterBoltextendProps };
                //间距数组
                int vertCount = m_paramsHandler.GetIntValue(ParamNames.CLEAT_RAFTER_BOLT_COUNT_VERT);
                double[] temVertStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.CLEAT_RAFTER_BOLT_STRIDE_VERT);
                double horiGap = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_RAFTER_BOLT_GAP_HORI); //两个螺栓的水平间距(后续可以改进)
                if(horiGap == double.MaxValue) //自适应
                {
                    double length = m_context.CleatPurlinBoltHoriLocation.length;
                    double mainFlangeWidth = m_context.MainFrontBound.Dist(m_context.MainBackBound.Origin);
                    if (length > mainFlangeWidth)
                        length = mainFlangeWidth;
                    horiGap = Math.Abs(length - 2 * 40); //40是固定的边距
                }
                List<double> vertStride = BoltParameter.ParseStrideInput(vertCount, temVertStride);
                List<double> horiStride = new List<double>{ horiGap };
                List<double>[] strides = { vertStride, horiStride };
                //螺栓组始末点
                double bottomOffset = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_RAFTER_BOLT_INSIDE_MARGIN_VERT);
                Vec3 start = m_context.AlignCenterPlane.Intersect(m_context.CreatePlane.Move(bottomOffset), m_context.CleatCreatePlane);
                Vec3 end = m_context.AlignCenterPlane.Intersect(m_context.CreatePlane.Move(bottomOffset + 50), m_context.CleatCreatePlane);
                Vec3 []points = { start , end };
                //工作坐标系
                Vec3 x1 = m_context.CreatePlane.Normal;
                Vec3 y1 = m_context.MainFrontBound.Normal;
                Mat43 workAxis1 = new Mat43(x1, y1, x1.Cross(y1), Vec3.ZERO);
                //创建螺栓组
                int connectMode = m_paramsHandler.GetIntValue(ParamNames.CLEAT_RAFTER_CONNECT_MODE);
                PartBase[] parts = { m_context.Cleat, m_context.MainPart};
                if(connectMode == 1 && vertCount > 0) //勾选螺栓时才创建
                    CreateBoltGroup(workAxis1, points, strides, cleatRafterBoltProps, parts, 200);
            }

            /*
             * 拉条螺栓
             */
            {
                //檩拉条螺栓属性
                DObject basicPropsTop = m_paramsHandler.GetDObjectValue(ParamNames.STAY_PURLIN_BOLT_BASIC_PROPS);
                DObject holeProps = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_BOLT_HOLE_PROPS);
                DObject contentProps = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_BOLT_CONTENT_PROPS);
                DObject extendProps = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_BOLT_EXTEND_PROPS);
                stayPurlinBoltProps = new DObject[]{ basicPropsTop, holeProps, contentProps, extendProps };
                int topCount = m_paramsHandler.GetIntValue(ParamNames.STAY_PURLIN_BOLT_COUNT);
                double[] topStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.STAY_PURLIN_BOLT_STRIDE);
                if(topCount > 0)
                    stayPurlinBoltStride = BoltParameter.ParseStrideInput(topCount, topStride);
                //椽子拉条螺栓属性
                DObject basicPropsBottom = m_paramsHandler.GetDObjectValue(ParamNames.STAY_RAFTER_BOLT_BASIC_PROPS);
                stayRafterBoltProps = new DObject[]{ basicPropsBottom, holeProps, contentProps, extendProps };
                int bottomCount = m_paramsHandler.GetIntValue(ParamNames.STAY_STIFF_BOLT_COUNT);
                double[] bottomStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.STAY_STIFF_BOLT_STRIDE);
                if (bottomCount > 0)
                    stayRafterBoltStride = BoltParameter.ParseStrideInput(bottomCount, bottomStride);
                //左侧拉条螺栓
                PartBase leftPart = m_context.SubPart1;
                if (m_context.SubPart1 == null)
                    leftPart = m_context.SubPart2;
                if (m_context.LeftTopStiff != null)
                    leftPart = m_context.LeftTopStiff;
                CreateStayBoltGroup(m_context.StayBoltPointLeft, m_context.LeftStay, leftPart, m_context.LeftStiff);
                //右侧拉条螺栓
                PartBase rightPart = m_context.SubPart2;
                if (m_context.SubPart2 == null)
                    rightPart = m_context.SubPart1;
                if (m_context.RightTopStiff != null)
                    rightPart = m_context.RightTopStiff;
                CreateStayBoltGroup(m_context.StayBoltPointRight, m_context.RightStay, rightPart, m_context.RightStiff);
            }

            /*
             * 腹板和翼缘螺栓(孔)参数解析
             */
            isOpenHole = m_paramsHandler.GetIntValue(ParamNames.OVERLAP_BOLT_HOLE) == 2; //重叠时的螺栓孔开关
            isWebBolt = m_paramsHandler.GetIntValue(ParamNames.OVERLAP_BOLT_PLACE) == 2;
            int webHole = m_paramsHandler.GetIntValue(ParamNames.PURIN_WEB_HOLE); //正常时的螺栓孔开关
            Plane leftPlane = m_context.OverlapCenterPlane.Move(-m_context.OverlapHoriGap / 2);
            Plane rightPlane = m_context.OverlapCenterPlane.Move(m_context.OverlapHoriGap / 2);
            Plane bottomPlane = m_context.SubBottomPlane.Invert().Move(m_context.OverlapVertMarin); //腹板下侧螺栓孔的垂直位置
            Plane topPlane = bottomPlane.Move(m_context.OverlapVertGap); //腹板上侧螺栓孔（如果有）的垂直位置

            /*
             * 腹板螺栓（孔）
             */
            if (m_context.IsOverlap)
            {
                //重叠时有4个螺栓（孔）的位置点：左上、左下、右上、右下
                Vec3 leftTopPoint = topPlane.Intersect(leftPlane, m_context.CreatePlane); //4个螺栓位置
                Vec3 leftBottomPoint = bottomPlane.Intersect(leftPlane, m_context.CreatePlane);
                Vec3 rightTopPoint = topPlane.Intersect(rightPlane, m_context.CreatePlane);
                Vec3 rightBottomPoint = bottomPlane.Intersect(rightPlane, m_context.CreatePlane);
                Vec3 centerTopPoint = topPlane.Intersect(m_context.OverlapCenterPlane, m_context.CreatePlane);
                Vec3 centerBottomPoint = bottomPlane.Intersect(m_context.OverlapCenterPlane, m_context.CreatePlane);
                bool[] boltFlag = m_context.OverlapHoleFlag; //四处螺栓是否创建
                //螺栓孔的创建
                PartBase[] parts = { m_context.SubPart1, m_context.SubPart2 };
                if (!isWebBolt && isOpenHole) //下侧的两个螺栓孔只有选择翼缘螺栓且开孔打开时才出现
                {
                    if(boltFlag[1]) //左下
                        CreateOneBoltHole(workAxis, leftBottomPoint, cleatBoltProps, parts);
                    if (boltFlag[3]) //右下
                        CreateOneBoltHole(workAxis, rightBottomPoint, cleatBoltProps, parts);
                }
                //螺栓的创建
                if (!isWebBolt)
                {
                    boltFlag[1] = false; //关闭腹板下侧螺栓
                    boltFlag[3] = false;
                }
                List<double> horiStride = new List<double>() { m_context.OverlapHoriGap };
                List<double> vertStride = new List<double>() { m_context.OverlapVertGap };
                int boltCount = 0; //4个螺栓位置中有几个有螺栓
                for(int i = 0; i < 4; i++)
                {
                    if (boltFlag[i])
                        boltCount++;
                }
                if (boltCount == 4) //一次4个螺栓
                {
                    Vec3[] points = { centerBottomPoint, centerTopPoint };
                    List<double>[] strides = { vertStride, horiStride };
                    CreateBoltGroup(workAxis, points, strides, cleatBoltProps, parts, 200);
                }
                else if(boltCount == 3 || boltCount == 2)
                {
                    Vec3[] leftPoints = {  leftBottomPoint, leftTopPoint };
                    Vec3[] rightPoints = { rightBottomPoint, rightTopPoint };
                    if(boltFlag[0] && boltFlag[1])
                    {
                        List<double>[] strides = { vertStride, null };
                        CreateBoltGroup(workAxis, leftPoints, strides, cleatBoltProps, parts, 200);
                        CreateOneOverlapBolt(new bool[] {boltFlag[3], boltFlag[2]}, rightPoints);
                    }
                    else if(boltFlag[2] && boltFlag[3])
                    {
                        List<double>[] strides = { vertStride, null };
                        CreateBoltGroup(workAxis, rightPoints, strides, cleatBoltProps, parts, 200);
                        CreateOneOverlapBolt(new bool[] { boltFlag[1], boltFlag[0] },leftPoints);
                    }
                    else if(boltFlag[0])//水平的两个螺栓是分开创建的
                        CreateOneOverlapBolt(new bool[] { boltFlag[0], boltFlag[2] }, new Vec3[] { leftTopPoint, rightTopPoint });
                    else
                        CreateOneOverlapBolt(new bool[] { boltFlag[1], boltFlag[3] }, new Vec3[] { leftBottomPoint, rightBottomPoint });
                }
                else
                    CreateOneOverlapBolt(boltFlag, new Vec3[] { leftTopPoint, leftBottomPoint, rightTopPoint, rightBottomPoint });
            }
            if (webHole == 2)
            {
                if (m_context.StayBoltPointLeft != null)
                {
                    PartBase[] parts;
                    Plane centetPlane;
                    if (m_context.SubPart1 == null)
                    {
                        parts = new PartBase[] { null, m_context.SubPart2 };
                        centetPlane = m_context.Sub2TopBound.Move(-m_context.Sub2TopBound.Dist(m_context.Sub2BottomBound.Origin) / 2);
                    }
                    else
                    {
                        parts = new PartBase[] { null, m_context.SubPart1 };
                        centetPlane = m_context.Sub1TopBound.Move(-m_context.Sub1TopBound.Dist(m_context.Sub1BottomBound.Origin) / 2);
                    }
                    Vec3 boltPoint = m_context.StayBoltPointLeft[1]; //拉条上侧螺栓位置点
                    Vec3 holePoint = SymmetryPoint(centetPlane, boltPoint);
                    if (boltPoint.Dist(holePoint) > 50)
                        CreateOneBoltHole(workAxis, holePoint, cleatBoltProps, parts);
                }
                if (m_context.StayBoltPointRight != null)
                {
                    PartBase[] parts;
                    Plane centetPlane;
                    if (m_context.SubPart2 == null)
                    {
                        parts = new PartBase[] { null, m_context.SubPart1 };
                        centetPlane = m_context.Sub1TopBound.Move(-m_context.Sub1TopBound.Dist(m_context.Sub1BottomBound.Origin) / 2);
                    }
                    else
                    {
                        parts = new PartBase[] { null, m_context.SubPart2 };
                        centetPlane = m_context.Sub2TopBound.Move(-m_context.Sub2TopBound.Dist(m_context.Sub2BottomBound.Origin) / 2);
                    }
                    Vec3 boltPoint = m_context.StayBoltPointRight[1]; //拉条上侧螺栓位置点
                    Vec3 holePoint = SymmetryPoint(centetPlane, boltPoint);
                    if (boltPoint.Dist(holePoint) > 50)
                        CreateOneBoltHole(workAxis, holePoint, cleatBoltProps, parts);
                }
            }
            if (!m_context.IsOverlap && isOpenHole && (m_context.SubPart1 == null || m_context.SubPart2 == null)) //这里可能有问题，翼缘孔开关影响了腹板孔
            {
                Plane sidePlane;
                PartBase subPart;
                double boltMargin = m_paramsHandler.GetDoubleValue(ParamNames.OVERLAP_BOLT_MARGIN);
                if (boltMargin == double.MaxValue)
                    boltMargin = 20;
                if (m_context.SubPart1 != null)
                {
                    subPart = m_context.SubPart1;
                    sidePlane = m_context.Sub1NearCenterPlane.Move(-boltMargin);
                }
                else
                {
                    subPart = m_context.SubPart2;
                    sidePlane = m_context.Sub2NearCenterPlane.Move(-boltMargin);
                }
                Vec3 topPoint = topPlane.Intersect(sidePlane, m_context.CreatePlane);
                Vec3 bottomPoint = bottomPlane.Intersect(sidePlane, m_context.CreatePlane);
                double[] emptyArray = { };
                double[] strideX = { m_context.CleatPurlinBoltVertLocation.stride.Sum() };
                CreateBoltHole(workAxis, bottomPoint, topPoint, cleatBoltProps, new PartBase[] { subPart }, strideX, emptyArray);
            }

            /*
             * 翼缘螺栓（孔）
             */
            Vec3 flangex = m_context.CreatePlane.Invert().Normal;
            Vec3 flangey = m_context.OverlapCenterPlane.Invert().Normal;
            Mat43 flangeWorkAxis = new Mat43(flangex, flangey, flangex.Cross(flangey), Vec3.ZERO); //上下螺栓（孔）共用一个工作坐标系 
            Plane[] auxVertPlane1 = ParseFlangeBoltHolePlace(m_context.SubPart1, m_context.Sub1FrontBound, m_context.Sub1BackBound, m_context.Sub1WebFlangeWidth, m_context.Sub1WebThickness);
            Plane[] auxVertPlane2 = ParseFlangeBoltHolePlace(m_context.SubPart2, m_context.Sub2FrontBound, m_context.Sub2BackBound, m_context.Sub2WebFlangeWidth, m_context.Sub2WebThickness);
            if (m_context.IsOverlap) //重叠时加螺栓孔
            {
                if (isOpenHole)
                {
                    PartBase[] parts1 = { m_context.SubPart1 };
                    CreateFlangeBoltHole(flangeWorkAxis, leftPlane, isWebBolt, auxVertPlane1, parts1); //子零件1
                    PartBase[] parts2 = { m_context.SubPart2 };
                    CreateFlangeBoltHole(flangeWorkAxis, rightPlane, isWebBolt, auxVertPlane2, parts2); //子零件2
                }
                if(!isWebBolt && auxVertPlane1[1] != null) //下翼缘螺栓
                {
                    Vec3 end = auxVertPlane1[1].Move(20).Intersect(m_context.OverlapCenterPlane, m_context.Sub1BottomBound);
                    Vec3 start = auxVertPlane1[1].Intersect(m_context.OverlapCenterPlane, m_context.Sub1BottomBound);
                    Vec3[] points = { start, end };
                    List<double> vertStride = new List<double>();
                    List<double> horiStride = new List<double>() { m_context.OverlapHoriGap };
                    List<double>[] strides = { vertStride, horiStride };
                    PartBase[] parts = { m_context.SubPart1, m_context.SubPart2 };
                    CreateBoltGroup(flangeWorkAxis, points, strides, cleatBoltProps, parts, 100);
                }
            }
            else if(isOpenHole)//正常情况下加螺栓孔（此时翼缘螺栓孔和腹板螺栓孔分开开关）
            {
                double boltHoleMargin = m_paramsHandler.GetDoubleValue(ParamNames.OVERLAP_BOLT_MARGIN);
                if (boltHoleMargin == double.MaxValue)
                    boltHoleMargin = 20;
                if(m_context.SubPart1 != null)
                {
                    PartBase[] parts1 = { null, m_context.SubPart1};
                    Plane sidePlane1 = m_context.Sub1NearCenterPlane.Move(-boltHoleMargin);
                    CreateFlangeBoltHole(flangeWorkAxis, sidePlane1, true, auxVertPlane1, parts1);
                }
                if (m_context.SubPart2 != null)
                {
                    PartBase[] parts2 = { null, m_context.SubPart2 };
                    Plane sidePlane2 = m_context.Sub2NearCenterPlane.Move(-boltHoleMargin);
                    CreateFlangeBoltHole(flangeWorkAxis, sidePlane2, true, auxVertPlane2, parts2);
                }
            }
            
        }

        public void CreateOneOverlapBolt(bool[] isCreate, Vec3[] points)
        {
            PartBase[] parts = { m_context.SubPart1, m_context.SubPart2 };
            for (int i = 0; i < isCreate.Length; i++)
            {
                if (isCreate[i])
                {
                    Vec3 start = points[i];
                    Vec3 end = start + workAxis.X;
                    CreateBoltGroup(workAxis, new Vec3[]{ start, end}, null, cleatBoltProps, parts, 200);
                }
            }
        }

        public void CreateFlangeBoltHole(Mat43 workAxis, Plane sidePlane, bool isCreateBottomHole, Plane[] auxVertPlane, PartBase[] parts)
        {
            Vec3 topHolePlace, bottomHolePlace;
            if (m_context.IsSub1Closer)
            {
                topHolePlace = m_context.Sub1TopBound.Intersect(sidePlane, auxVertPlane[0]);
                bottomHolePlace = m_context.Sub1BottomBound.Intersect(sidePlane, auxVertPlane[1]);
            }
            else
            {
                topHolePlace = m_context.Sub2TopBound.Intersect(sidePlane, auxVertPlane[0]);
                bottomHolePlace = m_context.Sub2BottomBound.Intersect(sidePlane, auxVertPlane[1]);
            }
            if(topHolePlace.IsValid())
                CreateOneBoltHole(workAxis, topHolePlace, cleatBoltProps, parts);
            if (isCreateBottomHole && bottomHolePlace.IsValid()) //是否创建底部螺栓孔
                CreateOneBoltHole(workAxis, bottomHolePlace, cleatBoltProps, parts);
        }

        public Plane[] ParseFlangeBoltHolePlace(Beam subPart, Plane frontBound, Plane backBound, double webFlangeWidth, double webThickness)
        {
            if (subPart == null)
                return null;
            Profile profile = subPart.Profile;
            String type = profile.Type;
            Plane[] auxVertPlane = new Plane[2]; //返回上侧螺栓孔和下侧螺栓孔的CreatePlane方向的定位面
            Vec3 frontOrient = m_context.CreatePlane.Normal; //第1个创建面的方向是Z形截面顶端的延申方向（L形截面底端延申方向）
            if (m_context.CreatePlane == m_context.CreatePlane2) //手动改变了创建面，取反方向，得到自动CreatePlane面方向
                frontOrient *= -1;
            if (type.Equals("Z") || type.Equals("ZZ"))
            {
                webFlangeWidth += webThickness;
                Plane centerPlane1 = frontBound.Move(-webFlangeWidth / 2);
                Plane centerPlane2 = backBound.Move(-webFlangeWidth / 2);
                auxVertPlane[0] = centerPlane1;
                auxVertPlane[1] = centerPlane2;
                if (centerPlane1.Normal.Dot(frontOrient) < 0)
                    m_context.Swap(ref auxVertPlane[0], ref auxVertPlane[1]);
            }
            else
            {
                double width = frontBound.Dist(backBound.Origin);
                Plane centerPlane = frontBound.Move(-width / 2);
                auxVertPlane[0] = centerPlane;
                auxVertPlane[1] = centerPlane;
            }
            if (type.Equals("L")) //分4个方向处理
            {
                BeamPlanes beamPlanes = BeamPlanes.CreateFromBeam(subPart);
                Plane beamTopPlane = beamPlanes.TopBound; //L梁（檩条自身的顶面）
                Plane beamFrontPlane = beamPlanes.FrontBound;
                Vec3 topOrient = m_context.MainTopBound.Normal; //节点的上方向
                if(m_context.IsVertical(beamTopPlane, topOrient))
                {
                    if (topOrient.Dot(beamTopPlane.Normal) > 0)
                        auxVertPlane[0] = null;
                    else
                        auxVertPlane[1] = null;
                }
                else if(m_context.IsVertical(beamFrontPlane, topOrient))
                {
                    if (beamFrontPlane.Normal.Dot(topOrient) > 0)
                        auxVertPlane[0] = null;
                    else
                        auxVertPlane[1] = null;
                }
            }
            return auxVertPlane;
        }

        public void CreateCleatBoltGroup(double horiOffset, List<double> horiStride, PartBase[] subParts)
        {
            if (horiStride == null || m_context.CleatPurlinBoltVertLocation == null)
                return;
            //位置获取
            double marginBottom = m_context.CleatPurlinBoltVertLocation.gap1; //最下侧螺栓与主零件上侧间距
            List<double> vertStride = m_context.CleatPurlinBoltVertLocation.stride; //垂直方向排列间距
            //创建螺栓
            PartBase[] cleatSubPart;
            if (subParts.Length == 1)
                cleatSubPart = new PartBase[]{ m_context.Cleat, subParts[0] };
            else
                cleatSubPart = new PartBase[] { m_context.Cleat, subParts[0] , subParts[1]};
            double horiCenterOffet = horiOffset - horiStride.Sum() / 2;
            Plane auxiliaryBottomPlane = m_context.CleatCreatePlane.Move(marginBottom); //这三个面用于确定螺栓创建点位置
            Plane auxiliarySidePlane = m_context.AlignCenterPlane.Move(horiCenterOffet);
            Plane auxiliaryFrontPlane = m_context.CreatePlane;
            Vec3 startPos = auxiliaryBottomPlane.Intersect(auxiliarySidePlane, auxiliaryFrontPlane);
            Vec3 endPos = auxiliaryBottomPlane.Move(20).Intersect(auxiliarySidePlane, auxiliaryFrontPlane);
            Vec3[] points = { startPos, endPos };
            List<double>[] strides = { vertStride, horiStride };
            CreateBoltGroup(workAxis, points, strides, cleatBoltProps, cleatSubPart, 200);
        }

        public void CreateStayBoltGroup(Vec3[] createPoints, SteelBeam stay, PartBase subPart, SteelPlate stiff)
        {
            if (createPoints == null)
                return;
            Vec3 moveNormal = m_context.CreatePlane.Normal.Normalize() * 10;
            Vec3 bottomPoint = createPoints[0] + moveNormal;
            Vec3 topPoint = createPoints[1] + moveNormal;
            //上侧螺栓组
            if(stay != null && stayPurlinBoltStride != null)
            {
                PartBase[] parts = { subPart, stay};
                Vec3 x = bottomPoint - topPoint;
                Vec3 z = m_context.CreatePlane.Normal;
                Vec3 y = z.Cross(x);
                Mat43 workAxis = new Mat43(x, y, z, Vec3.ZERO);
                Vec3[] points = { topPoint, topPoint + x };
                List<double>[] strides = { stayPurlinBoltStride, new List<double>() };
                CreateBoltGroup(workAxis, points, strides, stayPurlinBoltProps, parts, 200);
            }
            //下侧螺栓组
            if (stiff != null && stayRafterBoltStride != null)
            {
                PartBase[] parts = { stiff, stay };
                Vec3 x = topPoint - bottomPoint;
                Vec3 z = m_context.CreatePlane.Normal;
                Vec3 y = z.Cross(x);
                Mat43 workAxis = new Mat43(x, y, z, Vec3.ZERO);
                Vec3[] points = { bottomPoint, bottomPoint + x };
                List<double>[] strides = { stayRafterBoltStride, new List<double>() };
                if (stay != null) //创建下螺栓组
                    CreateBoltGroup(workAxis, points, strides, stayRafterBoltProps, parts, 200);
                else //此处可能有问题，加劲肋上有个孔
                    CreateOneBoltHole(workAxis, bottomPoint, stayRafterBoltProps, new PartBase[] { stiff });
            }
        }

        public void CreateBoltGroup(Mat43 workAxis, Vec3[] points, List<double>[] strides, DObject[] props, PartBase[] parts, double cutLength)
        {
            //是否创建
            if (parts[0] == null || parts[1] == null)
                return;


            //参数解析
            BoltParameter boltParameter = new BoltParameter();
            boltParameter.CopyFromBasicProps(props[0]);
            boltParameter.CopyFromHoleProps(props[1]);
            boltParameter.CopyFromContentProps(props[2]);
            boltParameter.CopyFromExtendProps(props[3]);

            //螺栓位置设置
            BoltGroup boltGroup;
            if (parts.Length == 2)
                boltGroup = m_context.Comp.CreateBoltGroup(parts[0], parts[1], points[0], points[1], true);
            else
            {
                PartBase[] subParts = { parts[1], parts[2] }; //最多只有两个次零件
                boltGroup = m_context.Comp.CreateBoltGroup(parts[0], subParts, points[0], points[1], true);
            }
            boltGroup.WorkAxis = workAxis;
            double[] emptyArray = { };
            if(strides == null)
            {
                boltGroup.ArrayStrideX = emptyArray;
                boltGroup.ArrayStrideY = emptyArray;
            }
            else
            {
                if (strides[0] != null)
                    boltGroup.ArrayStrideX = strides[0].ToArray(); //X方向间距
                else
                    boltGroup.ArrayStrideX = emptyArray;
                if (strides[1] != null)
                    boltGroup.ArrayStrideY = strides[1].ToArray();
                else
                    boltGroup.ArrayStrideY = emptyArray;
            }
            //基础属性
            boltGroup.BoltStandard = boltParameter.BoltStandard;
            boltGroup.BoltSize = boltParameter.BoltSize;
            boltGroup.HoleTol = boltParameter.HoleTol; //孔容差范围
            boltGroup.ThreadInMtrl = boltParameter.ThreadInMaterial; //是否包含螺纹 
            boltGroup.MadeType = boltParameter.MadeType; //螺栓制作类型(工厂/工地) 

            //螺栓孔设置
            boltGroup.HoleX = boltParameter.HoleX; //孔X方向扩大尺寸
            boltGroup.HoleY = boltParameter.HoleY;
            boltGroup.HoleType = boltParameter.HoleType; //孔类型
            boltGroup.RotateSlot = boltParameter.RotateSlot; //孔开槽旋转方式 

            //螺栓垫片(螺母)设置
            boltGroup.SectionFlags = boltParameter.SectionFlags; //螺栓部件开关标志位

            //其它
            boltGroup.CutLength = cutLength; //切割长度
            boltGroup.ExtLength = boltParameter.ExtLength; // 附加长度 
            boltGroup.RotationSide = boltParameter.RotationSide; //螺栓朝向

            //解决旋转后位置错误的问题
            boltGroup.Rotation = 0.0;
            boltGroup.HeadOffsetX = 0.0;
            boltGroup.HeadOffsetY = 0.0;
            boltGroup.HeadOffsetZ = 0.0;
            boltGroup.TailOffsetX = 0.0;
            boltGroup.TailOffsetY = 0.0;
            boltGroup.TailOffsetZ = 0.0;
            boltGroup.VerticalOffset = 0.0;
            boltGroup.DepthOffset = 0.0;


            boltGroup.Refresh();
        }
    
        public void CreateOneBoltHole(Mat43 workAxis, Vec3 start, DObject[] props, PartBase[] parts)
        {
            double[] emptyArray = { };
            Vec3 end = start + workAxis.X;
            CreateBoltHole(workAxis, start, end, props, parts, emptyArray, emptyArray);
        }

        public void CreateBoltHole(Mat43 workAxis, Vec3 start, Vec3 end, DObject[] props, PartBase[] parts, double[] strideX, double[] strideY)
        {
            //是否创建
            if (parts[0] == null && parts[1] == null)
                return;

            //参数解析
            BoltParameter boltParameter = new BoltParameter();
            boltParameter.CopyFromBasicProps(props[0]);
            boltParameter.CopyFromHoleProps(props[1]);
            boltParameter.CopyFromContentProps(props[2]);
            boltParameter.CopyFromExtendProps(props[3]);

            //螺栓位置设置
            BoltGroup boltGroup;
            if (parts.Length == 1)
                boltGroup = m_context.Comp.CreateBoltHoles(null, parts[0], start, end, true);
            else
                boltGroup = m_context.Comp.CreateBoltHoles(parts[0], parts[1], start, end, true);
            boltGroup.WorkAxis = workAxis;
            boltGroup.ArrayStrideX = strideX; //X方向间距
            boltGroup.ArrayStrideY = strideY;

            //基础属性
            boltGroup.BoltStandard = boltParameter.BoltStandard;
            boltGroup.BoltSize = boltParameter.BoltSize;
            boltGroup.HoleTol = boltParameter.HoleTol; //孔容差范围
            boltGroup.ThreadInMtrl = boltParameter.ThreadInMaterial; //是否包含螺纹 
            boltGroup.MadeType = boltParameter.MadeType; //螺栓制作类型(工厂/工地) 

            //螺栓孔设置
            boltGroup.HoleX = boltParameter.HoleX; //孔X方向扩大尺寸
            boltGroup.HoleY = boltParameter.HoleY;
            boltGroup.HoleType = boltParameter.HoleType; //孔类型
            boltGroup.RotateSlot = boltParameter.RotateSlot; //孔开槽旋转方式 

            //螺栓垫片(螺母)设置
            //boltGroup.SectionFlags = boltParameter.SectionFlags; //螺栓部件开关标志位

            //其它
            boltGroup.CutLength = 100; //切割长度
            boltGroup.ExtLength = boltParameter.ExtLength; // 附加长度 
            boltGroup.RotationSide = boltParameter.RotationSide; //螺栓朝向

            //解决旋转后位置错误的问题
            boltGroup.Rotation = 0.0;
            boltGroup.HeadOffsetX = 0.0;
            boltGroup.HeadOffsetY = 0.0;
            boltGroup.HeadOffsetZ = 0.0;
            boltGroup.TailOffsetX = 0.0;
            boltGroup.TailOffsetY = 0.0;
            boltGroup.TailOffsetZ = 0.0;
            boltGroup.VerticalOffset = 0.0;
            boltGroup.DepthOffset = 0.0;

            boltGroup.Refresh();
        }

        public Vec3 SymmetryPoint(Plane plane, Vec3 point)
        {
            double dist = plane.SignedDist(point);
            point = point - 2 * dist * plane.Normal.Normalize();
            return point;
        }
    }
}
