﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;
using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;

namespace ColdRolledOverlap1
{
    class CleatHandler
    {
        private ComponentInstanceContext m_context = null; 
        private ParamsHandler m_paramsHandler = null;
        private BeamProperties cleatProperties = new BeamProperties();
        private double length = double.MaxValue; //夹板梁的Tail与Head间距(随翻转变化)
        private double width = double.MaxValue; //夹板截面的宽度
        private double height = double.MaxValue; //夹板的Top与Bottom间距(随翻转变化)
        private double thickness = 10.0;
        BoltLocation boltLocationHori, boltLocationVert;
        private bool isRotate; //夹板是否翻转
        private String type;

        public CleatHandler(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }

        public bool PraseLocation()
        {
            /*
             * 解析夹板截面输入
             */
            DObject profileD = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_PROFILE);
            cleatProperties.SetProps(profileD);
            Profile profile = STC.ConfigMgr.ProfileMgr.GetProfile(cleatProperties.Profile); //获取截面对象，大的类型如C、L、CC、Plate
            type = profile.Type;
            cleatProperties.Profile = ParseProfileType(cleatProperties.Profile); //获取截面类型，并存入创建截面。如：BL、PL、HI
            if (cleatProperties.Profile != "")
            {
                height = profile.ShapeHeight;
                width = profile.ShapeWidth;
                if (type == "C")
                {
                    CProfile cProfile = profile as CProfile;
                    thickness = cProfile.PlateThickness;
                }
                else if(type == "L")
                {
                    LProfile lProfile = profile as LProfile;
                    thickness = lProfile.PlateThickness;
                }
                else if(type == "Plate")
                {
                    PlateProfile plateProfile = profile as PlateProfile;
                    thickness = plateProfile.Thickness;
                }
                else //暂时不支持的截面类型
                {
                    thickness = 10;
                }
            }
            else
            {
                type = "L"; //默认为L型截面
                cleatProperties.Profile = "L";
                width = 100;
            }


            /*
             * 图形界面所设高度和宽度
             */
            double inputHeight = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_HEIGHT);
            double inputLength = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_WIDTH);
            if (inputHeight != double.MaxValue)
                height = inputHeight;
            if (inputLength != double.MaxValue)
                length = inputLength;

            /*
             * 夹板是否翻转
             */
            int cleatMode = m_paramsHandler.GetIntValue(ParamNames.CLEAT_MODE);
            isRotate = cleatMode == 2 || (type != "Plate" && cleatMode == 4);
            if (isRotate)
                Util.Swap(ref height, ref length);

            /*
             * 螺栓确定的高度
             */
            double marginTop = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_BOLT_PURLIN_MARGIN_TOP);
            double marginBottom = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_BOLT_PURLIN_MARGIN_BOTTOM);
            int countVert = m_paramsHandler.GetIntValue(ParamNames.CLEAT_BOLT_PURLIN_COUNT_VERT);
            double[] strideVert = m_paramsHandler.GetDoubleArrayValue(ParamNames.CLEAT_BOLT_PURLIN_STRIDE_VERT);
            //螺栓中心距夹板底面（创建面）距离
            double boltCenterHeight = m_context.MainSubGap;
            if(m_context.IsSub1Closer)
                boltCenterHeight += m_context.Sub1TopBound.Dist(m_context.Sub1BottomBound.Origin) / 2;
            else
                boltCenterHeight += m_context.Sub2TopBound.Dist(m_context.Sub2BottomBound.Origin) / 2;
            //前面已经定了高度，螺栓位置依据板位置定
            if(height != double.MaxValue)
            {
                boltLocationVert = new BoltLocation();
                boltLocationVert.defaultStride = new List<double>() { 60.0 };
                boltLocationVert.SetStride(countVert, strideVert);
                boltLocationVert.gap1 = boltCenterHeight - boltLocationVert.stride.Sum() / 2;
                boltLocationVert.gap2 = height - boltCenterHeight - boltLocationVert.stride.Sum() / 2;
                boltLocationVert.SetBoltLocation(marginBottom, marginTop, countVert, strideVert, height);
                if (countVert == 0)
                    boltLocationVert = null;
            }
            //前面没有定高度，板高度依据螺栓定
            else
            {
                boltLocationVert = new BoltLocation(40, 40, 2, new double[] { 60 });
                boltLocationVert.SetCenterBoltLoaction(marginBottom, marginTop, countVert, strideVert, boltCenterHeight);
                height = boltLocationVert.length;
            }
            m_context.CleatPurlinBoltVertLocation = boltLocationVert;

            /*
             * 螺栓确定的长度
             */
            double marginLeft = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_BOLT_PURLIN_MARGIN_LEFT);
            double marginRight = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_BOLT_PURLIN_MARGIN_RIGHT);
            int countHori = m_paramsHandler.GetIntValue(ParamNames.CLEAT_BOLT_PURLIN_COUNT_HORI);
            double[] strideHori = m_paramsHandler.GetDoubleArrayValue(ParamNames.CLEAT_BOLT_PURLIN_STRIDE_HORI);
            boltLocationHori = new BoltLocation(40, 40, 2, new double[] { 60 });
            boltLocationHori.centerMode = true;
            boltLocationHori.SetBoltLocation(marginLeft, marginRight, countHori, strideHori, length);
            length = boltLocationHori.length;
            m_context.CleatPurlinBoltHoriLocation = boltLocationHori;


            /*
             * 将参数存入截面字符串中(夹板不翻转的情况，若有翻转在创建时拼截面类型)
             */
            //正常情况
            if (isRotate)
                Util.Swap(ref length, ref height);
            if (type == "Plate")
                cleatProperties.Profile +=  thickness + "*" + height;
            else
                cleatProperties.Profile += height + "*" + width + "*" + thickness;

            return true;
        }


        public void Execute()
        {
            //创建面的偏移
            double createOffset = m_paramsHandler.GetDoubleValue(ParamNames.CREATE_PLANE_OFFSET); 
            m_context.CreatePlane.MoveThis(createOffset);
            //位置解析
            if (!PraseLocation())
                return;

            /*
             * 添加夹板
             */
            Plane lengthCenterPlane, heightCenterPlane;
            Plane widthCenterPlane = m_context.CreatePlane.Move(width / 2);
            //正常朝向
            Vec3 cy = m_context.CreatePlane.Normal;
            Vec3 cz = m_context.MainTopBound.Normal;
            Vec3 cx = cy.Cross(cz);
            if (!isRotate) //板截面且选择垂直加劲肋时不翻转夹板
            {
                lengthCenterPlane = m_context.AlignCenterPlane;
                heightCenterPlane = m_context.CleatCreatePlane.Move(height / 2);
            }
            //L截面时选择垂直加劲肋时要翻转夹板，选翻转选项也要翻转夹板
            else
            {
                lengthCenterPlane = m_context.CleatCreatePlane.Move(length / 2);
                heightCenterPlane = m_context.AlignCenterPlane;
                Util.Swap(ref cx, ref cz);
            }
            //
            Vec3 cstart = lengthCenterPlane.Move(-length / 2).Intersect(heightCenterPlane, widthCenterPlane);
            Vec3 cend = lengthCenterPlane.Move(length / 2).Intersect(heightCenterPlane, widthCenterPlane);
            SteelBeam cleat = m_context.Comp.CreateSteelBeam(cleatProperties.Profile, cstart, cend, true);
            //
            Mat43 workAxis = new Mat43(cx, cy, cz, Vec3.ZERO);
            cleat.WorkAxis = workAxis;
            //
            cleat.RotationSide = Define.ObjectSide.SIDE_TOP;
            cleat.Rotation = 0;
            cleatProperties.ApplyToPart(cleat);
            cleat.Flush();
            m_context.Cleat = cleat;

            /*
             * 添加加劲肋
             */
            int cleatMode = m_paramsHandler.GetIntValue(ParamNames.CLEAT_MODE);
            //加劲肋属性
            PlateProperties stiffProperties = new PlateProperties();
            DObject pro = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_STIFF_PROPS);
            stiffProperties.SetProps(pro);
            //加劲肋焊缝属性
            WeldProperties weldParams = new WeldProperties();
            DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[4]);
            weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
            weldParams.SetProps(weldProps);
            //加劲肋侧边和创建面
            double thickness = stiffProperties.Thickness;
            Plane zCtrlPlane = m_context.AlignCenterPlane.Move(-thickness / 2);
            Plane insidePlane = m_context.CreatePlane.Move(this.thickness); //内侧边
            //三角形的加劲肋
            if (cleatMode == 3 && stiffProperties.Thickness > 0)
            {
                if (type == "Plate")
                {
                    this.thickness = 0; //板截面的夹板底边就是MainTopBound
                    width = height; //这里原本是按L形截面的边缘定位的，现在只能设置一个固定值，有待改进
                }
                Plane bottomPlane = m_context.CleatCreatePlane.Move(this.thickness); //直角边——底边
                Vec3 acutePointBottom = bottomPlane.Intersect(zCtrlPlane, m_context.CreatePlane.Move(width)); //底部锐角点
                Vec3 acutePointTop = insidePlane.Intersect(zCtrlPlane, bottomPlane.Move(height - this.thickness)); //顶部锐角点
                Vec3 rightPoint = insidePlane.Intersect(zCtrlPlane, bottomPlane); //底部直角点
                Vec3 x = m_context.CreatePlane.Normal.Normalize(); //chamferX方向单位向量，用于移动点
                Vec3 y = m_context.MainTopBound.Normal.Normalize();
                //折角处理（其实是创建多边形板）
                List<Vec3> pointList = new List<Vec3>();
                List<Vec3> sideWeldPointList = new List<Vec3>();
                List<Vec3> bottomWeldPointList = new List<Vec3>();
                List<Vec3> allWeldPointList = new List<Vec3>(); 
                //获取上侧的2（1）个点
                {
                    double cutX = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_STIFF_TOP_CHAMFER_X);
                    double cutY = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_STIFF_TOP_CHAMFER_Y);
                    if (cutX < 0)
                        cutX = 0;
                    Vec3 insidePoint = acutePointTop - y * cutY; //加劲肋折角之后上侧的直角点
                    if (cutX != 0)
                    {
                        pointList.Add(insidePoint);
                    }
                    pointList.Add(acutePointTop + x * cutX - y * cutY);
                    Plane comparePlane = m_context.CleatCreatePlane.Move(height); //比较是否超过边界并存入焊接点列表
                    if (comparePlane.SignedDist(insidePoint) > 0)
                        insidePoint = StiffenerWeld.ProjectPoint(comparePlane, insidePoint);
                    sideWeldPointList.Add(insidePoint);
                }
                //获取下侧内侧（直角处）的2（1）个点
                {
                    double cutX = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_STIFF_BOTTOM_OUTSIDE_CHAMFER_X);
                    double cutY = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_STIFF_BOTTOM_OUTSIDE_CHAMFER_Y);
                    if (cutY < 0)
                        cutY = 0;
                    pointList.Add(acutePointBottom - x * cutX + y * cutY);
                    Vec3 insidePoint = acutePointBottom - x * cutX;
                    if (cutY != 0)
                    {
                        pointList.Add(insidePoint);
                    }
                    Plane comparePlane = m_context.CreatePlane.Move(width); 
                    if (comparePlane.SignedDist(insidePoint) > 0)
                        insidePoint = StiffenerWeld.ProjectPoint(comparePlane, insidePoint);
                    bottomWeldPointList.Add(insidePoint);
                }
                //获取下侧外侧（直角处）的2（1）个点
                {
                    double cutX = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_STIFF_BOTTOM_INSIDE_CHAMFER_X);
                    double cutY = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_STIFF_BOTTOM_INSIDE_CHAMFER_Y);
                    if (cutX < 0)
                        cutX = 0;
                    if (cutY < 0)
                        cutY = 0;
                    if (cutX != 0 && cutY != 0)
                    {
                        pointList.Add(rightPoint + x * cutX);
                        pointList.Add(rightPoint + y * cutY);
                        sideWeldPointList.Add(rightPoint + y * cutY);
                        bottomWeldPointList.Add(rightPoint + x * cutX);
                    }
                    else
                    {
                        pointList.Add(rightPoint); //普通折角
                        allWeldPointList.AddRange(sideWeldPointList);
                        allWeldPointList.Add(rightPoint);
                        allWeldPointList.AddRange(bottomWeldPointList);
                    }
                }
                //创建加劲肋
                SteelPlate stiff = CreateStiffByPoints(pointList.ToArray(), thickness);
                stiffProperties.ApplyToPart(stiff);
                stiff.Flush();
                //直角未折角，创建整体焊缝（侧边和底部形成一个整体）
                if (allWeldPointList.Count != 0)
                {
                    GetAnotherPlaneWeldPoint(ref allWeldPointList, m_context.AlignCenterPlane.Normal, thickness);
                    Weld weld = m_context.Comp.CreatePolyWeld(stiff, m_context.Cleat, allWeldPointList.ToArray(), true);
                    weldParams.ApplyToWeld(weld);
                    weld.Flush();
                }
                //直角折角了，分别创建底边和侧边的多边形焊缝
                else
                { 
                    GetAnotherPlaneWeldPoint(ref sideWeldPointList, m_context.AlignCenterPlane.Normal, thickness);
                    Weld weld1 = m_context.Comp.CreatePolyWeld(stiff, m_context.Cleat, sideWeldPointList.ToArray(), true);
                    weldParams.ApplyToWeld(weld1);
                    weld1.Flush();
                    GetAnotherPlaneWeldPoint(ref bottomWeldPointList, m_context.AlignCenterPlane.Normal, thickness);
                    Weld weld2 = m_context.Comp.CreatePolyWeld(stiff, m_context.Cleat, bottomWeldPointList.ToArray(), true);
                    weldParams.ApplyToWeld(weld2);
                    weld2.Flush();
                }
            }
            //长方形加劲肋
            double marginInside = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_STIFF_MARGIN_INSIDE);
            if (cleatMode == 4 && stiffProperties.Thickness > 0 && marginInside != 0)
            {
                //创建加劲肋
                double marginTop = m_paramsHandler.GetDoubleValue(ParamNames.CLEAT_STIFF_MARGIN_TOP);
                Plane bottomPlane = m_context.CleatCreatePlane;
                double stiffHeight = height;
                if(type != "Plate")
                    stiffHeight = length;
                Plane topPlane = bottomPlane.Move(stiffHeight - marginTop);
                Plane outSidePlane = insidePlane.Move(marginInside);
                SteelPlate stiff = m_context.Comp.CreateSteelPlate(zCtrlPlane, thickness, new Plane[]{ topPlane, outSidePlane, bottomPlane, insidePlane}, true);
                stiffProperties.ApplyToPart(stiff);
                stiff.Flush();
                //添加焊缝
                Weld cleatStiffWeld = m_context.Comp.CreateAutoWeld(m_context.Cleat, stiff, Define.SpaceAxis.AXIS_X, true);
                weldParams.ApplyToWeld(cleatStiffWeld);
                cleatStiffWeld.Flush();
                Weld beamStiffWeld = m_context.Comp.CreateAutoWeld(m_context.MainPart, stiff, Define.SpaceAxis.AXIS_X, true);
                weldParams.ApplyToWeld(beamStiffWeld);
                beamStiffWeld.Flush();
            }

            CreateCleatCutter();
        }

        //创建夹板切割（I、H截面檩条时有效）
        private void CreateCleatCutter()
        {
            int cutMode = m_paramsHandler.GetIntValue(ParamNames.CLEAT_NOTCH_OPEN);
            if (cutMode != 1)
                return;
            double length = this.length, height = this.height;
            if (isRotate)
                Util.Swap(ref length, ref height);
            double marginSide = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_MARGIN_SIDE);
            double marginCenter = m_paramsHandler.GetDoubleValue(ParamNames.NOTCH_MARGIN_CENTER);
            Plane rightSidePlane = m_context.AlignCenterPlane.Move(length / 2 + marginSide);
            Plane leftSidePlane = m_context.AlignCenterPlane.Invert().Move(length / 2 + marginSide);
            Plane frontSidePlane = m_context.CreatePlane.Move(-marginCenter);
            Plane backSidePlane = frontSidePlane.Move(width);
            Plane bottomPlane = m_context.CleatCreatePlane.Clone();
            Plane topPlane = bottomPlane.Move(height);
            Plane[] sidePlanes = { frontSidePlane, leftSidePlane, backSidePlane, rightSidePlane };
            if (m_context.SubPart1 != null)
                m_context.Comp.CreatePlateCutter(m_context.SubPart1, bottomPlane, topPlane, sidePlanes, true);
            if (m_context.SubPart2 != null)
                m_context.Comp.CreatePlateCutter(m_context.SubPart2, bottomPlane, topPlane, sidePlanes, true);
        }

        private void GetAnotherPlaneWeldPoint(ref List<Vec3> list, Vec3 orient, double thickness)
        {
            Vec3 move = orient.Normalize() * thickness;
            List<Vec3> re = new List<Vec3>();
            re.AddRange(list);
            for (int i = 0; i < list.Count; i++)
                list[i] += move;
            re.Reverse();
            re.AddRange(list);
            re.Add(re[0]);
            list = re;
        }

        public SteelPlate CreateStiffByPoints(Vec3[] points, double thickness)
        {
            Vec3 v1 = points[1] - points[0];
            Vec3 v2 = points[2] - points[1];
            Vec3 createOrient = v1.Cross(v2);
            Plane zCtrlPlane = new Plane(points[0], createOrient);
            List<Plane> sidePlaneList = new List<Plane>();
            for (int i = 0; i < points.Length; i++)
            {
                Vec3 normal = createOrient.Cross(points[(i + 1) % points.Length] - points[i]);
                Plane sidePlane = new Plane(points[i], normal);
                sidePlaneList.Add(sidePlane);
            }
            Plane[] sidePlanes = sidePlaneList.ToArray();
            return m_context.Comp.CreateSteelPlate(zCtrlPlane, thickness, sidePlanes, true);
        }

        public String ParseProfileType(String profileString)
        {
            for(int i = 0; i < profileString.Length; i++)
            {
                char c = profileString[i];
                if (c >= '0' && c <= '9')
                    return profileString.Substring(0, i);
            }
            return "";
        }
    }
}
