﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;
using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;

namespace ColdRolledOverlap1
{
    class CutHandler
    {
        private ComponentInstanceContext m_context = null; //公共参数、方法
        private ParamsHandler m_paramsHandler = null; //包含UI里的参数

        public CutHandler(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            //创建面的选择，默认是自动在空的那一侧生成的
            int orientMode = m_paramsHandler.GetIntValue(ParamNames.CREATE_ORIENT); 
            if(orientMode == 1)
            {
                m_context.CreatePlane = m_context.CreatePlane2;
                m_context.m_leftOrient *= -1;
                m_context.SwapSub1Sub2();
                m_context.SetPlanes();
            }
            //檩条的对齐
            double gap = m_paramsHandler.GetDoubleValue(ParamNames.PURLIN_ALIGN_GAP); //檩条间距
            double offset = m_paramsHandler.GetDoubleValue(ParamNames.PURLIN_ALIGN_PLANE_OFFSET);
            double alignLength; //檩条对齐面相对于对齐中心的偏移
            if (m_context.SubPart1 == null || m_context.SubPart2 == null)
                alignLength = m_paramsHandler.GetDoubleValue(ParamNames.PURLIN_ALIGN_MARGIN); //只有一个子零件，向边缘对齐
            else
                alignLength = gap; //有两个子零件，向中心对齐
            Plane centerPlane = m_context.MainCenterWebPlane.Move(offset); //对齐中心面
            m_context.AlignCenterPlane = centerPlane; //此时螺栓创建面就是对齐中心面
            m_context.Gap = gap * 2;
            //重叠情况下重新计算对齐面
            bool isOverlap = m_paramsHandler.GetIntValue(ParamNames.OVERLAP_MODE) == 1; //是否重叠
            if (m_context.SubPart1 == null || m_context.SubPart2 == null)
                isOverlap = false;
            if(isOverlap)
            {
                m_context.IsOverlap = true;
                offset = m_paramsHandler.GetDoubleValue(ParamNames.OVERLAP_OFFSET);
                double boltGap = m_paramsHandler.GetDoubleValue(ParamNames.OVERLAP_BOLT_GAP);
                double boltMargin = m_paramsHandler.GetDoubleValue(ParamNames.OVERLAP_BOLT_MARGIN);
                if (boltMargin == double.MaxValue)
                    boltMargin = 40;
                alignLength = -(boltGap / 2 + boltMargin);
                GetAlignLength(ref alignLength);
                centerPlane = m_context.MainCenterWebPlane.Move(offset); //此时需要修改对齐中心面
                m_context.OverlapCenterPlane = centerPlane; //重叠中心面是新的对齐中心面
            }
            else
            {
                m_context.OverlapCenterPlane = centerPlane;
            }
            //SubPart1
            if (m_context.SubPart1 != null && alignLength < int.MaxValue)
            {
                GetAlignLength(ref alignLength);
                Plane alignPlane = centerPlane.Invert().Move(alignLength);
                if (m_context.SubPart2 == null) //只有次零件1，对齐面为中心面向外移动alignLength
                    alignPlane = m_context.MainCenterWebPlane.Invert().Move(-alignLength);
                m_context.Comp.CreateAlignPlane(m_context.SubPart1, alignPlane, true);
                m_context.Sub1NearCenterPlane = alignPlane.Invert();
            }
            //subpart2
            if (m_context.SubPart2 != null && alignLength < int.MaxValue)
            {
                GetAlignLength(ref alignLength);
                Plane alignPlane = centerPlane.Move(alignLength);
                if (m_context.SubPart1 == null)
                    alignPlane = m_context.MainCenterWebPlane.Move(-alignLength);
                m_context.Comp.CreateAlignPlane(m_context.SubPart2, alignPlane, true);
                m_context.Sub2NearCenterPlane = alignPlane.Invert();
            }

        }

        public void GetAlignLength(ref double length)
        {
            double length1 = double.MaxValue, length2 = double.MaxValue;
            if (m_context.SubPart1 != null)
                length1 = m_context.Sub1Planes.HeadBound.Dist(m_context.Sub1Planes.TailBound.Origin);
            if (m_context.SubPart2 != null)
                length2 = m_context.Sub2Planes.HeadBound.Dist(m_context.Sub2Planes.TailBound.Origin);
            double minLen = length1 < length2 ? length1 : length2;
            if (Math.Abs(length) > minLen)
                length = length < 0 ? -minLen /2 : minLen / 2;
            
        }
    }
}
