﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;
using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;

namespace ColdRolledOverlap1
{
    class StayHandler{
        private ComponentInstanceContext m_context = null; //公共参数、方法
        private ParamsHandler m_paramsHandler = null; //包含UI里的参数
        private BeamProperties stayProps;
        private double stayWidth, stayHeight;

        public StayHandler(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            /*
             * 添加连接在拉条上的加劲肋（拉条板）
             */
            StayOpenMode stayMode = (StayOpenMode)m_paramsHandler.GetIntValue(ParamNames.STAY_PLACE);
            bool isCreateLeftStiff = stayMode != StayOpenMode.NONE && stayMode != StayOpenMode.RIGHT; //是否创建加劲肋
            bool isCreateRightStiff = stayMode != StayOpenMode.NONE && stayMode != StayOpenMode.LEFT;
            bool isCreateLeftStay = stayMode == StayOpenMode.BOTH || stayMode == StayOpenMode.LEFT || stayMode == StayOpenMode.LEFT_ALL_STIFF; //是否创建拉条
            bool isCreateRightStay = stayMode == StayOpenMode.BOTH || stayMode == StayOpenMode.RIGHT || stayMode == StayOpenMode.RIGHT_ALL_STIFF;
            //确定宽高
            DObject pro = m_paramsHandler.GetDObjectValue(ParamNames.STAY_STIFF_PROPS);
            PlateProperties stiffProps = new PlateProperties();
            stiffProps.SetProps(pro);
            //加劲肋长宽
            double topStiffWidth = m_paramsHandler.GetDoubleValue(ParamNames.STAY_TOP_STIFF_WIDTH);
            double topStiffHeight = m_paramsHandler.GetDoubleValue(ParamNames.STAY_TOP_STIFF_HEIGHT);
            Util.SetDefaultValue(ref stiffProps.Width, m_context.MainWebFlangeWidth); //默认长宽为腹板面和边缘面的间距
            Util.SetDefaultValue(ref stiffProps.Height, stiffProps.Width);
            double heightOffset = m_paramsHandler.GetDoubleValue(ParamNames.STAY_STIFF_VERT_OFFSET);
            //折角
            double cutX = m_paramsHandler.GetDoubleValue(ParamNames.STAY_STIFF_CHAMFER_X);
            double cutY = m_paramsHandler.GetDoubleValue(ParamNames.STAY_STIFF_CHAMFER_Y);
            int chamferType = m_paramsHandler.GetIntValue(ParamNames.STAY_STIFF_CHAMFER_TYPE);
            Define.CornerCutType cutType;
            switch (chamferType)
            {
                case 2: 
                    cutType = Define.CornerCutType.CORNER_CUT_LINE;
                    break;
                case 3:
                    cutType = Define.CornerCutType.CORNER_CUT_ROUND;
                    cutY = cutX;
                    break;
                case 4:
                    cutType = Define.CornerCutType.CORNER_CUT_INV_ROUND;
                    cutY = cutX;
                    break;
                default:
                    cutType = Define.CornerCutType.CORNER_CUT_NONE;
                    break;
            }
            //确定位置
            Plane bottomSidePlane = m_context.MainBottomFlangeInside.Move(heightOffset);
            //创建下侧加劲肋
            if (isCreateLeftStiff) //创建左侧加劲肋
            {
                SteelPlate leftStiff = CreateBottomSitff(stiffProps, bottomSidePlane, m_context.MainBackWeb, cutX, cutY, cutType);
                m_context.LeftStiff = leftStiff;
            }
            if (isCreateRightStiff) //创建右侧加劲肋
            {
                SteelPlate rightStiff = CreateBottomSitff(stiffProps, bottomSidePlane, m_context.MainFrontWeb, cutX, cutY, cutType);
                m_context.RightStiff = rightStiff;
            }

            /*
             * 添加拉条，先定位边缘螺栓，再确定拉条长度
             */
            if (m_context.CleatPurlinBoltVertLocation == null)
                return;
            //底部螺栓位置
            double bottomHoriOffset = m_paramsHandler.GetDoubleValue(ParamNames.STAY_BOTTOM_BOLT_HORI_OFFSET);
            double bottomVertOffset = m_paramsHandler.GetDoubleValue(ParamNames.STAY_BOTTOM_BOLT_VERT_OFFSET);
            Util.SetDefaultValue(ref bottomHoriOffset, stiffProps.Width / 2);
            Util.SetDefaultValue(ref bottomVertOffset, stiffProps.Height / 2);
            bottomVertOffset += m_paramsHandler.GetDoubleValue(ParamNames.STAY_STIFF_VERT_OFFSET); //相对于主梁翼缘的偏移
            //顶部螺栓位置
            double roltAngle = m_paramsHandler.GetDoubleValue(ParamNames.STAY_ANGLE);
            double topVertOffset = m_paramsHandler.GetDoubleValue(ParamNames.STAY_TOP_BOLT_VERT_OFFSET);
            double topHoriOffsetLeft = m_paramsHandler.GetDoubleValue(ParamNames.STAY_TOP_BOLT_HORI_OFFSET_LEFT);
            double topHoriOffsetRight = m_paramsHandler.GetDoubleValue(ParamNames.STAY_TOP_BOLT_HORI_OFFSET_RIGHT);
            //重叠的情况
            double overlapHoriGap = m_paramsHandler.GetDoubleValue(ParamNames.OVERLAP_BOLT_GAP);
            double overlapVertGap = m_paramsHandler.GetDoubleValue(ParamNames.OVERLAP_BOLT_VERT_GAP);
            double overlapOffset = m_paramsHandler.GetDoubleValue(ParamNames.OVERLAP_OFFSET); ; //重叠中心面相对于腹板中心面的偏移
            //顶部加劲肋
            TopSiffOpen topStiffOpen = (TopSiffOpen)m_paramsHandler.GetIntValue(ParamNames.STAY_TOP_STIFF_OPEN);
            if (topStiffOpen == TopSiffOpen.OPEN)
                Util.SetDefaultValue(ref topVertOffset, -topStiffHeight / 2);
            //
            double defaultTopVertOffset = 10; //是按对齐檩条夹板螺栓来算的
            StayTopBoltPlace alignMode = (StayTopBoltPlace)m_paramsHandler.GetIntValue(ParamNames.STAY_TOP_BOLT_VERT_ALIGN_MODE); //拉条和加劲肋的开关
            switch (alignMode)
            {
                case StayTopBoltPlace.ALIGN_CLEAT_TOP_BOLT: //上
                    defaultTopVertOffset = m_context.CleatPurlinBoltVertLocation.gap1 + m_context.CleatPurlinBoltVertLocation.stride.Sum();
                    break;
                case StayTopBoltPlace.ALIGN_CLEAT_MIDDLE_BOLT: //中
                    defaultTopVertOffset = m_context.CleatPurlinBoltVertLocation.gap1 + m_context.CleatPurlinBoltVertLocation.stride.Sum() / 2;
                    break;
                case StayTopBoltPlace.ALIGN_CLEAT_BOTTOM_BOLT: //下
                    defaultTopVertOffset = m_context.CleatPurlinBoltVertLocation.gap1;
                    break;
            }
            //
            double upDownOffset = m_context.MainHeight - m_context.MainFlangeThickness; //上下拉条螺栓垂直偏移的参考位置不同，下侧螺栓加上该值后与上侧一致
            double defaultTopHoriOffset = m_context.CleatPurlinBoltVertLocation.gap1 + upDownOffset; //默认水平偏移是依据默认垂直偏移确定的
            if (topVertOffset == double.MaxValue && roltAngle != double.MaxValue) //topVertOffset无效且设置了角度时,角度才有效
            {
                defaultTopHoriOffset = defaultTopVertOffset + upDownOffset - bottomVertOffset;
                roltAngle = roltAngle * Math.PI / 180;
                topHoriOffsetLeft = defaultTopHoriOffset * Math.Tan(roltAngle); //defaultTopHoriOffset是等腰直角三角形的一个直角边
                topHoriOffsetRight = topHoriOffsetLeft; //与左侧一样
                topVertOffset = defaultTopVertOffset;
            }
            else
            {
                if (topHoriOffsetLeft == double.MaxValue) //水平间距优先级：手动设置>默认
                {
                    topHoriOffsetLeft = defaultTopHoriOffset;
                    if (m_context.IsOverlap)
                        topHoriOffsetLeft = overlapHoriGap / 2 - overlapOffset;
                }
                if (topHoriOffsetRight == double.MaxValue)
                {
                    topHoriOffsetRight = defaultTopHoriOffset;
                    if (m_context.IsOverlap)
                        topHoriOffsetRight = overlapHoriGap / 2 + overlapOffset;
                }
                if (topVertOffset == double.MaxValue)
                    topVertOffset = defaultTopVertOffset;
                else
                    topVertOffset += m_context.MainSubGap;
            }
            double topVertOffsetLeft = topVertOffset, topVertOffsetRight = topVertOffset;
            //重叠情况下重新计算位置
            {
                double vertMarginTop = m_paramsHandler.GetDoubleValue(ParamNames.OVERLAP_BOLT_MARGIN_TOP);
                double vertMarginBottom = m_paramsHandler.GetDoubleValue(ParamNames.OVERLAP_BOLT_MARGIN_BOTTOM);
                double mainSubDist = m_context.MainSubGap;
                double subHeight;
                if (m_context.IsSub1Closer)
                    subHeight = m_context.Sub1TopBound.Dist(m_context.Sub1BottomBound.Origin); //檩条高度
                else
                    subHeight = m_context.Sub2TopBound.Dist(m_context.Sub2BottomBound.Origin);
                if (vertMarginBottom == double.MaxValue)
                    vertMarginBottom = m_context.CleatPurlinBoltVertLocation.gap1 - mainSubDist; //默认对齐夹板螺栓的下侧
                if (overlapVertGap == double.MaxValue)
                {
                    if (vertMarginTop != double.MaxValue)
                        overlapVertGap = subHeight - vertMarginBottom - vertMarginTop;
                    else
                        overlapVertGap = m_context.CleatPurlinBoltVertLocation.stride.Sum();
                }
                m_context.OverlapVertMarin = vertMarginBottom;
                m_context.OverlapVertGap = overlapVertGap;
                m_context.OverlapHoriGap = overlapHoriGap;
                //计算4个重叠螺栓的位置(垂直方向) ,重叠时默认间距与螺栓页设置有关
                if (m_context.IsOverlap)
                {
                    //临界距离
                    DObject overlapBoltProps = m_paramsHandler.GetDObjectValue(ParamNames.CLEAT_BOLT_BASIC_PROPS);
                    DObject stayBoltProps = m_paramsHandler.GetDObjectValue(ParamNames.STAY_PURLIN_BOLT_BASIC_PROPS);
                    double radius1 = overlapBoltProps.GetDouble("bolt_size");
                    double radius2 = stayBoltProps.GetDouble("bolt_size");
                    double minDistance = radius1 + radius2;
                    //左上
                    {
                        double horiGap = Math.Abs(topHoriOffsetLeft - (overlapHoriGap / 2 - overlapOffset));
                        double vertGap = Math.Abs(topVertOffsetLeft - (vertMarginBottom + overlapVertGap + mainSubDist));
                        if (Math.Sqrt(horiGap * horiGap + vertGap * vertGap) <= minDistance && isCreateLeftStay)
                        {
                            m_context.OverlapHoleFlag[0] = false;
                            topHoriOffsetLeft = overlapHoriGap / 2 - overlapOffset;
                            topVertOffsetLeft = vertMarginBottom + overlapVertGap + mainSubDist;
                        }
                    }
                    //左下
                    {
                        double horiGap = Math.Abs(topHoriOffsetLeft - (overlapHoriGap / 2 - overlapOffset));
                        double vertGap = Math.Abs(topVertOffsetLeft - (vertMarginBottom + mainSubDist));
                        if (Math.Sqrt(horiGap * horiGap + vertGap * vertGap) <= minDistance && isCreateLeftStay)
                        {
                            m_context.OverlapHoleFlag[1] = false;
                            topHoriOffsetLeft = overlapHoriGap / 2 - overlapOffset;
                            topVertOffsetLeft = vertMarginBottom + mainSubDist;
                        }
                    }
                    //右上
                    {
                        double horiGap = Math.Abs(topHoriOffsetRight - (overlapHoriGap / 2 + overlapOffset));
                        double vertGap = Math.Abs(topVertOffsetRight - (vertMarginBottom + overlapVertGap + mainSubDist));
                        if (Math.Sqrt(horiGap * horiGap + vertGap * vertGap) <= minDistance && isCreateRightStay)
                        {
                            m_context.OverlapHoleFlag[2] = false;
                            topHoriOffsetRight = overlapHoriGap / 2 + overlapOffset;
                            topVertOffsetRight = vertMarginBottom + overlapVertGap + mainSubDist;
                        }
                    }
                    //右下
                    {
                        double horiGap = Math.Abs(topHoriOffsetRight - (overlapHoriGap / 2 + overlapOffset));
                        double vertGap = Math.Abs(topVertOffsetRight - (vertMarginBottom + +mainSubDist));
                        if (Math.Sqrt(horiGap * horiGap + vertGap * vertGap) <= minDistance && isCreateRightStay)
                        {
                            m_context.OverlapHoleFlag[3] = false;
                            topHoriOffsetRight = overlapHoriGap / 2 + overlapOffset;
                            topVertOffsetRight = vertMarginBottom + mainSubDist;
                        }
                    }
                }
            }
            //拉条创建
            stayProps = new BeamProperties();
            DObject dStayProp = m_paramsHandler.GetDObjectValue(ParamNames.STAY_PROFILE);
            stayProps.SetProps(dStayProp);
            Profile profile = STC.ConfigMgr.ProfileMgr.GetProfile(stayProps.Profile);
            stayWidth = profile.ShapeWidth;
            stayHeight = profile.ShapeHeight;
            double bottomMargin = m_paramsHandler.GetDoubleValue(ParamNames.STAY_BOLT_BOTTOM_MARGIN);
            double topMargin = m_paramsHandler.GetDoubleValue(ParamNames.STAY_BOLT_TOP_MARGIN);
            //左拉条
            {
                double sideMargin = m_paramsHandler.GetDoubleValue(ParamNames.STAY_BOLT_SIDE_MARGIN_LEFT);
                Vec3[] boltPoint = null;
                double[] boltLocation = { topHoriOffsetLeft, topVertOffsetLeft, bottomHoriOffset, bottomVertOffset};
                double[] boltMargin = { sideMargin, bottomMargin, topMargin };
                SteelBeam stay = CreateStay(m_context.MainCenterWebPlane.Invert(), m_context.MainBackWeb, boltLocation, boltMargin, ref boltPoint, isCreateLeftStay);
                m_context.LeftStay = stay;
                m_context.StayBoltPointLeft = boltPoint;
            }
            //右拉条
            {
                double sideMargin = m_paramsHandler.GetDoubleValue(ParamNames.STAY_BOLT_SIDE_MARGIN_RIGHT);
                Vec3[] boltPoint = null;
                double[] boltLocation = { topHoriOffsetRight, topVertOffsetRight, bottomHoriOffset, bottomVertOffset };
                double[] boltMargin = { sideMargin, bottomMargin, topMargin };
                SteelBeam stay = CreateStay(m_context.MainCenterWebPlane, m_context.MainFrontWeb, boltLocation, boltMargin, ref boltPoint, isCreateRightStay);
                m_context.StayBoltPointRight = boltPoint;
                m_context.RightStay = stay;
            }



            //创建上侧加劲肋
            PlateProperties topStiffProps = new PlateProperties();
            topStiffProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.STAY_TOP_STIFF_PROPS));
            if (isCreateLeftStiff && topStiffOpen == TopSiffOpen.OPEN) //创建左侧加劲肋
            {
                Plane stiffTopPlane = m_context.Sub1BottomBound;
                if (stiffTopPlane == null)
                    stiffTopPlane = m_context.Sub2BottomBound;
                SteelPlate leftStiff = CreateTopSitff(topStiffProps, stiffTopPlane.Clone(), topStiffWidth, topStiffHeight, m_context.StayBoltPointLeft[1]);
                m_context.LeftTopStiff = leftStiff;
            }
            if (isCreateRightStiff && topStiffOpen == TopSiffOpen.OPEN) //创建右侧加劲肋
            {
                Plane stiffTopPlane = m_context.Sub2BottomBound;
                if (stiffTopPlane == null)
                    stiffTopPlane = m_context.Sub1BottomBound;
                SteelPlate rightStiff = CreateTopSitff(topStiffProps, stiffTopPlane.Clone(), topStiffWidth, topStiffHeight, m_context.StayBoltPointRight[1]);
                m_context.RightTopStiff = rightStiff;
            }
        }

        public SteelBeam CreateStay(Plane horiCenterPlane, Plane sidePlane, double[] boltOffset, double[] boltMargin, ref Vec3[] boltPoint, bool isCreate)
        {
            double topHoriOffset = boltOffset[0];
            double topVertOffset = boltOffset[1];
            double bottomHoriOffset = boltOffset[2];
            double bottomVertOffset = boltOffset[3];
            double sideMargin = boltMargin[0]; //与侧边的边距
            double bottomMargin = boltMargin[1]; //与头的边距
            double topMargin = boltMargin[2]; //与尾的边距
            double webThickness = m_context.MainWebThickness;
            Plane createPlane = m_context.CreatePlane.Move(stayWidth / 2);
            Vec3 headPoint = sidePlane.Move(bottomHoriOffset).Intersect(createPlane, m_context.MainBottomFlangeInside.Move(bottomVertOffset));
            Vec3 tailPoint = horiCenterPlane.Move(topHoriOffset - webThickness / 2).Intersect(createPlane, m_context.CleatCreatePlane.Move(topVertOffset));
            {
                boltPoint = new Vec3[2]; //保存螺栓位置，创建螺栓时用
                boltPoint[0] = sidePlane.Move(bottomHoriOffset).Intersect(m_context.CreatePlane, m_context.MainBottomFlangeInside.Move(bottomVertOffset));
                boltPoint[1] = horiCenterPlane.Move(topHoriOffset - webThickness / 2).Intersect(m_context.CreatePlane, m_context.CleatCreatePlane.Move(topVertOffset));
            }
            Vec3 x = createPlane.Normal;
            Vec3 y = (tailPoint - headPoint).Normalize();
            Vec3 z = x.Cross(y);
            headPoint -= y * bottomMargin;
            tailPoint += y * topMargin; 
            if(z.Dot(sidePlane.Normal) < 0) //左侧
            {
                Vec3 temp = headPoint;
                headPoint = tailPoint;
                tailPoint = temp;
                y *= -1;
                z *= -1;
            }
            if (sideMargin != double.MaxValue)
            {
                z = z.Normalize();
                headPoint = headPoint + z * (sideMargin - stayHeight / 2);
                tailPoint = tailPoint + z * (sideMargin - stayHeight / 2);
            }
            if (!isCreate)
                return null;
            //
            SteelBeam stay = m_context.Comp.CreateSteelBeam(stayProps.Profile, headPoint, tailPoint, true);
            stayProps.ApplyToPart(stay);
            stay.WorkAxis = new Mat43(x, y, z, Vec3.ZERO);
            stay.RotationSide = Define.ObjectSide.SIDE_TOP;
            stay.Rotation = 0;
            stay.Flush();
            return stay;
        }
    
        public SteelPlate CreateBottomSitff(PlateProperties stiffProps, Plane bottomSidePlane, Plane insideSidePlane, double cutX, double cutY, Define.CornerCutType cutType)
        {
            Plane zCtrlPlane = m_context.CreatePlane.Invert();
            Plane topSidePlane = bottomSidePlane.Move(stiffProps.Height);
            Plane outsideSidePlane = insideSidePlane.Move(stiffProps.Width);
            Plane[] sidePlanes = { bottomSidePlane, insideSidePlane, topSidePlane, outsideSidePlane };
            SteelPlate stiff = m_context.Comp.CreateSteelPlate(zCtrlPlane, stiffProps.Thickness, sidePlanes, true);
            if (stiff == null)
                return null;
            stiffProps.ApplyToPart(stiff);
            stiff.Flush();
            ChamferControl chamfer = new ChamferControl(stiff, sidePlanes);
            chamfer.ChamferOnePoint(cutX, cutY, cutType, chamfer.comparePointList[1]);
            return stiff;
        }

        public SteelPlate CreateTopSitff(PlateProperties stiffProps, Plane topSidePlane, double width, double height, Vec3 boltPlace)
        {
            if (topSidePlane == null)
                return null;
            Plane zCtrlPlane = m_context.CreatePlane.Invert();
            Plane bottomSidePlane = topSidePlane.Move(height);
            Plane vertCenterPlane = new Plane(boltPlace, m_context.AlignCenterPlane.Normal);
            Plane leftSidePlane = vertCenterPlane.Move(-width / 2);
            Plane rightSideSidePlane = vertCenterPlane.Move(width / 2);
            Plane[] sidePlanes = { bottomSidePlane, rightSideSidePlane, topSidePlane, leftSidePlane };
            SteelPlate stiff = m_context.Comp.CreateSteelPlate(zCtrlPlane, stiffProps.Thickness, sidePlanes, true);
            if (stiff == null)
                return null;
            stiffProps.ApplyToPart(stiff);
            stiff.Flush();
            return stiff;
        }
    }
}
