﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;

namespace ColdRolledOverlap1
{
    class WeldHandler
    {
        private ComponentInstanceContext m_context = null; 
        private ParamsHandler m_paramsHandler = null;
        private List<WeldProperties> m_weldParams = new List<WeldProperties>();

        public WeldHandler(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                m_weldParams.Add(new WeldProperties());
            }
        }

        public void Execute()
        {
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldParams[i].SetProps(weldProps);
            }
            DObject stayStiffProps = m_paramsHandler.GetDObjectValue(ParamNames.STAY_STIFF_PROPS);
            double thickness = stayStiffProps.GetDouble("thickness");
            //左拉条加劲肋
            if(m_context.LeftStiff != null)
            {
                StiffenerWeld stiffWeld = new StiffenerWeld(m_context.Comp, m_context.MainPart, m_context.LeftStiff);
                if (m_context.MainBackWeb.Dist(m_context.MainBackBound.Origin) < 1e-4)
                    m_context.MainBackBound.MoveThis(100);
                List<Plane> surroundPlaneList = new List<Plane>() { m_context.MainBottomFlangeInside, m_context.MainBackWeb, m_context.MainTopFlangeInside, m_context.MainBackBound };
                List<WeldProperties> weldPropList = new List<WeldProperties>() { m_weldParams[1], m_weldParams[2], m_weldParams[0] };
                weldPropList.Add(null);
                stiffWeld.SetSurroundPlane(surroundPlaneList, weldPropList);
                stiffWeld.CreateSideWeld(m_context.CreatePlane, m_context.CreatePlane.Move(-thickness), 2);
            }
            //右拉条加劲肋
            if (m_context.RightStiff != null)
            {
                StiffenerWeld stiffWeld = new StiffenerWeld(m_context.Comp, m_context.MainPart, m_context.RightStiff);
                if (m_context.MainFrontWeb.Dist(m_context.MainFrontBound.Origin) < 1e-4)
                    m_context.MainFrontBound.MoveThis(100);
                List<Plane> surroundPlaneList = new List<Plane>() { m_context.MainBottomFlangeInside, m_context.MainFrontWeb, m_context.MainTopFlangeInside, m_context.MainFrontBound };
                List<WeldProperties> weldPropList = new List<WeldProperties>() { m_weldParams[1], m_weldParams[2], m_weldParams[0]};
                weldPropList.Add(null);
                stiffWeld.SetSurroundPlane(surroundPlaneList, weldPropList);
                stiffWeld.CreateSideWeld(m_context.CreatePlane, m_context.CreatePlane.Move(-thickness), 2);
            }
            //左拉条上侧加劲肋
            if (m_context.LeftTopStiff != null)
            {
                PartBase weldMainPart = m_context.SubPart1;
                if (weldMainPart == null)
                    weldMainPart = m_context.SubPart2;
                Mat43 workAxis = Mat43.CreateByYZ(m_context.CreatePlane.Normal, m_context.m_topOrient);
                Weld weld = m_context.Comp.CreateAutoWeld(weldMainPart, m_context.LeftTopStiff, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                m_weldParams[5].ApplyToWeld(weld);
                weld.WorkAxis = workAxis;
                weld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_Z;
                weld.Refresh();
            }
            //右拉条上侧加劲肋
            if (m_context.RightTopStiff != null)
            {
                PartBase weldMainPart = m_context.SubPart2;
                if (weldMainPart == null)
                    weldMainPart = m_context.SubPart1;
                Mat43 workAxis = Mat43.CreateByYZ(m_context.CreatePlane.Normal, m_context.m_topOrient);
                Weld weld = m_context.Comp.CreateAutoWeld(weldMainPart, m_context.RightTopStiff, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                m_weldParams[5].ApplyToWeld(weld);
                weld.WorkAxis = workAxis;
                weld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_Z;
                weld.Refresh();
            }
            //夹板和椽子焊接
            int connectMode = m_paramsHandler.GetIntValue(ParamNames.CLEAT_RAFTER_CONNECT_MODE);
            if(connectMode != 1)
            {
                Vec3 weldPoint = m_context.CreatePlane.Intersect(m_context.CleatCreatePlane, m_context.MainFrontWeb);
                //Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, m_context.Cleat, new Vec3[] { weldPoint }, true);

                Mat43 workAxis = m_context.Cleat.WorkAxis;
                Weld weld = m_context.Comp.CreateAutoWeld(m_context.MainPart, m_context.Cleat, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                Vec3 x = -m_context.CleatCreatePlane.Normal;
                Vec3 y = m_context.AlignCenterPlane.Normal;
                weld.WorkAxis = new Mat43(x, y, x.Cross(y), weldPoint);
                //weld.WorkAxis = workAxis;
                weld.EdgeOrAround = LoongCAD.STC.Define.WeldEdgeAround.WELD_AROUND;
                weld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_X;


                m_weldParams[3].ApplyToWeld(weld);
                //weld.Flush();
                weld.Refresh();
            }

        }
    }
}
