﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace ColumnSplice42
{
    public class ParamsHandler : LoongCAD.STC.Plugins.ParamsHandlerBase
    {
        /// <summary>
        /// 属性空值全局总表
        /// </summary>
        /// <remarks><b>此静态变量可直接复制到其他组件代码中使用</b></remarks>
        protected static DObject g_emptyValues = null;

        /// <summary>
        /// 属性内置默认值全局总表
        /// </summary>
        /// <remarks><b>此静态变量可直接复制到其他组件代码中使用</b></remarks>
        protected static DObject g_builtinDefaults = null;

        /// <summary>
        /// 默认构造函数
        /// </summary>
        /// <remarks><b>此方法需要复制到其他组件代码中使用</b></remarks>
        /// 
        public ParamsHandler()
            : base(ParamsHandler.EmptyValues, ParamsHandler.BuiltinDefautls)
        {

        }

        /// <summary>
        /// 组件公用属性空值表
        /// </summary>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public static DObject EmptyValues
        {
            get
            {
                if (g_emptyValues == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_emptyValues;
            }
        }

        /// <summary>
        /// 组件公用内置默认值接口
        /// </summary>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public static DObject BuiltinDefautls
        {
            get
            {
                if (g_builtinDefaults == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_builtinDefaults;
            }
        }

        /// <summary>
        /// 初始化默认值
        /// </summary>
        /// <param name="defaultConfig">从默认值配置加载的默认值</param>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public override LoongCAD.Util.DObject InitializeDefaultValues(DObject defaultConfig)
        {
            m_defaultValues = new DObject();
            m_defaultValues.CopyFrom(defaultConfig);

            var emptyValues = ParamsHandler.EmptyValues;
            var builtinDefaults = ParamsHandler.BuiltinDefautls;
            if (emptyValues == null || builtinDefaults == null)
            {
                Console.WriteLine("ParamsHandler.InitializeDefaultValues: [Warning] EmptyValue/BuiltinDefaults is null!");
                return m_defaultValues;
            }

            this.TryCompleteDObjectValues(m_defaultValues, builtinDefaults, emptyValues);
            return m_defaultValues;
        }

        public static void IntializeEmptyDefaults()
        {
            g_emptyValues = new DObject();
            g_builtinDefaults = new DObject();

            //图形
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_END_GAP, double.MaxValue, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_MAIN_GAP, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_SUB_GAP, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_SHEAR_PLATE_THICKNESS, double.MaxValue, double.MaxValue);

            //零件
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEB_PLATE_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                        PlatePropertyPanel.ParamNames.Thickness, 10.0,
                        PlatePropertyPanel.ParamNames.Width, double.MaxValue,
                        PlatePropertyPanel.ParamNames.Heigth, double.MaxValue,
                        PlatePropertyPanel.ParamNames.NoPrefix, "",
                        PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                        PlatePropertyPanel.ParamNames.AssNoPrefix, "",
                        PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                        PlatePropertyPanel.ParamNames.Material, "Q235B",
                        PlatePropertyPanel.ParamNames.Name, "板",
                        });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGE_PLATE_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                        PlatePropertyPanel.ParamNames.Thickness, 10.0,
                        PlatePropertyPanel.ParamNames.Width, double.MaxValue,
                        PlatePropertyPanel.ParamNames.Heigth, double.MaxValue,
                        PlatePropertyPanel.ParamNames.NoPrefix, "",
                        PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                        PlatePropertyPanel.ParamNames.AssNoPrefix, "",
                        PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                        PlatePropertyPanel.ParamNames.Material, "Q235B",
                        PlatePropertyPanel.ParamNames.Name, "板",
                        });

            //参数
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.ARG_HOLE_GAP_H, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.ARG_HOLE_GAP_V, double.MaxValue, 140.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.ARG_HOLE_R, double.MaxValue, 60.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.ARG_SHEAR_MODE, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.ARG_HOLE_MODE, 0, 1);

            //腹板螺栓
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEB_BOLT_BASIC_PROPS, BoltBasicPropertyPanel.EmptyValue, (DObject)new object[] {
                            BoltBasicPropertyPanel.ParamNames.BoltStandard, "TS10.9",
                            BoltBasicPropertyPanel.ParamNames.BoltSize, 20,
                            BoltBasicPropertyPanel.ParamNames.ThreadInMtrl, 1, // false
                            BoltBasicPropertyPanel.ParamNames.MadeType0, 1, // Site
                        });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEB_BOLT_HOLE_PROPS, BoltHolePanel.EmptyValue, (DObject)new object[] {
                            BoltHolePanel.ParamNames.HoleX, 0.0,
                            BoltHolePanel.ParamNames.HoleY, 0.0,
                            BoltHolePanel.ParamNames.HoleType, 2,
                            BoltHolePanel.ParamNames.RotSlot, 3,
                            BoltHolePanel.ParamNames.SlotIn, 4
                        });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEB_BOLT_CONTENT_PROPS, null, (DObject)new object[]
                        {
                            BoltContentPanel.ParamNames.PartWasherUp, false,
                            BoltContentPanel.ParamNames.PartWasherDown0, false,
                            BoltContentPanel.ParamNames.PartWasherDown1, true,
                            BoltContentPanel.ParamNames.PartNut0, true,
                            BoltContentPanel.ParamNames.PartNut1,false,
                        });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEB_BOLT_EXTEND_PROPS, BoltExtendPanel.EmptyValue, (DObject)new object[] {
                            BoltExtendPanel.ParamNames.ExtLength, 0.0,
                        });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEB_BOLT_SHAPE, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEB_BOLT_HORI_COUNT, int.MaxValue, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEB_BOLT_HORI_STRIDE, new double[] { }, new double[] {});
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEB_BOLT_LEFT_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEB_BOLT_RIGHT_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEBM_BOLT_VERT_COUNT, int.MaxValue, 3);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEBM_BOLT_VERT_STRIDE, new double[] { }, new double[] {});
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEBM_BOLT_TOP_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEBM_BOLT_BOTTOM_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEBM_BOLT_MADE_MODE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEBS_BOLT_VERT_COUNT, int.MaxValue, 3);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEBS_BOLT_VERT_STRIDE, new double[] { }, new double[] {  });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEBS_BOLT_TOP_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEBS_BOLT_BOTTOM_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WEBS_BOLT_MADE_MODE, 0, 0);


            //翼缘螺栓
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGE_BOLT_BASIC_PROPS, BoltBasicPropertyPanel.EmptyValue, (DObject)new object[] {
                            BoltBasicPropertyPanel.ParamNames.BoltStandard, "TS10.9",
                            BoltBasicPropertyPanel.ParamNames.BoltSize, 20,
                            BoltBasicPropertyPanel.ParamNames.ThreadInMtrl, 1, // false
                            BoltBasicPropertyPanel.ParamNames.MadeType0, 1, // Site
                        });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGE_BOLT_HOLE_PROPS, BoltHolePanel.EmptyValue, (DObject)new object[] {
                            BoltHolePanel.ParamNames.HoleX, 0.0,
                            BoltHolePanel.ParamNames.HoleY, 0.0,
                            BoltHolePanel.ParamNames.HoleType, 2,
                            BoltHolePanel.ParamNames.RotSlot, 3,
                            BoltHolePanel.ParamNames.SlotIn, 4
                        });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGE_BOLT_CONTENT_PROPS, null, (DObject)new object[]
                        {
                            BoltContentPanel.ParamNames.PartWasherUp, false,
                            BoltContentPanel.ParamNames.PartWasherDown0, false,
                            BoltContentPanel.ParamNames.PartWasherDown1, true,
                            BoltContentPanel.ParamNames.PartNut0, true,
                            BoltContentPanel.ParamNames.PartNut1, false,
                        });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGE_BOLT_EXTEND_PROPS, BoltExtendPanel.EmptyValue, (DObject)new object[] {
                            BoltExtendPanel.ParamNames.ExtLength, 0.0,
                        });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGE_BOLT_SHAPE, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGE_BOLT_HORI_COUNT, int.MaxValue, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGE_BOLT_HORI_STRIDE, new double[] { }, new double[] {});
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGE_BOLT_LEFT_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGE_BOLT_RIGHT_MARGIN, double.MaxValue, double.MaxValue);
                        // 纵向螺栓个数为自适应数量
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGEM_BOLT_VERT_COUNT, int.MaxValue, 3);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGEM_BOLT_VERT_STRIDE, new double[] { }, new double[] { });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGEM_BOLT_TOP_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGEM_BOLT_BOTTOM_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGES_BOLT_VERT_COUNT, int.MaxValue, 3);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGES_BOLT_VERT_STRIDE, new double[] { }, new double[] { });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGES_BOLT_TOP_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.FLANGES_BOLT_BOTTOM_MARGIN, double.MaxValue, double.MaxValue);
        }

       
    }
}
