﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.Util;

namespace ColumnSplice42
{
    public class WebBoltParameter : BoltParameter
    {
        public WebBoltParameter() { }

        private ParamsHandler paramsHandler;
        private ComponentInstanceContext m_context;


        public override BoltParameter Clone()
        {
            BoltParameter boltParameter = new WebBoltParameter();
            boltParameter.BoltStandard = this.BoltStandard;
            boltParameter.BoltSize = this.BoltSize;
            boltParameter.HoleTol = this.HoleTol;
            boltParameter.ThreadInMaterial = this.ThreadInMaterial;
            boltParameter.MadeType = this.MadeType;
            boltParameter.HoleX = this.HoleX;
            boltParameter.HoleY = this.HoleY;
            boltParameter.HoleType = this.HoleType;
            boltParameter.RotateSlot = this.RotateSlot;
            boltParameter.SectionFlags = this.SectionFlags;
            boltParameter.CutLength = this.CutLength;
            boltParameter.ExtLength = this.ExtLength;
            boltParameter.RotationSide = this.RotationSide;
            boltParameter.BoltShape = this.BoltShape;
            return boltParameter;
        }

        /*
         * 基础属性
         */
        public override void InitialBasic(ComponentInstanceContext compContext, ParamsHandler paramsHandler) 
        {
            DObject basicProps = paramsHandler.GetDObjectValue(ParamNames.WEB_BOLT_BASIC_PROPS);
            CopyFromBasicProps(basicProps);

            DObject holeProps = paramsHandler.GetDObjectValue(ParamNames.WEB_BOLT_HOLE_PROPS);
            CopyFromHoleProps(holeProps);

            DObject contentProps = paramsHandler.GetDObjectValue(ParamNames.WEB_BOLT_CONTENT_PROPS);
            CopyFromContentProps(contentProps);

            DObject extendProps = paramsHandler.GetDObjectValue(ParamNames.WEB_BOLT_EXTEND_PROPS);
            CopyFromExtendProps(extendProps);

            int shapeIndex = paramsHandler.GetIntValue(ParamNames.WEB_BOLT_SHAPE); 
            SetShape(shapeIndex);

            //int orientIndex = paramsHandler.GetIntValue(ParamNames.BOLT_WEB_ORIENTATION);//朝向设置，没有此选项
            //SetOrient(orientIndex);


            this.paramsHandler = paramsHandler;
            this.m_context = compContext;
        }


        /*
        * 位置属性
        */
        public override void InitalLocation(CommonBoltGroup bg)
        {
            CommonBoltLocation boltLocationVert, boltLocationHori1, boltLocationHori2;
            bg.SetHoriPlane(m_context.SubNearCenterPlane.Invert(), m_context.MainNearCenterPlane.Invert());
            //处理水平方向上的螺栓分布
            {
                double plateMargin1 = (m_context.WebWidth - m_context.Gap) / 2;
                double plateMargin2 = (m_context.WebWidth - m_context.Gap) / 2;
                if (m_context.MainWebMargin != double.MaxValue && m_context.SubWebMargin == double.MaxValue)
                {
                    plateMargin1 = m_context.MainWebMargin;
                    if(m_context.WebWidth - plateMargin1 - m_context.Gap > 0)
                        plateMargin2 = m_context.WebWidth - plateMargin1 - m_context.Gap;
                }
                if (m_context.MainWebMargin == double.MaxValue && m_context.SubWebMargin != double.MaxValue)
                {
                    plateMargin2 = m_context.SubWebMargin;
                    if (m_context.WebWidth - plateMargin2 - m_context.Gap > 0)
                        plateMargin1 = m_context.WebWidth - plateMargin2 - m_context.Gap;
                }
                if (m_context.MainWebMargin != double.MaxValue && m_context.SubWebMargin != double.MaxValue)
                {
                    plateMargin1 = m_context.MainWebMargin;
                    plateMargin2 = m_context.SubWebMargin; 
                }

                double centerGap1 = paramsHandler.GetDoubleValue(ParamNames.WEBM_BOLT_TOP_MARGIN);
                double sideGap1 = paramsHandler.GetDoubleValue(ParamNames.WEBM_BOLT_BOTTOM_MARGIN);
                int count1 = paramsHandler.GetIntValue(ParamNames.WEBM_BOLT_VERT_COUNT);
                double[] originStride1 = paramsHandler.GetDoubleArrayValue(ParamNames.WEBM_BOLT_VERT_STRIDE);
                boltLocationHori1 = new CommonBoltLocation(40, 40, 3, new double[] { 75.0 });
                boltLocationHori1.SetBoltLocation(centerGap1, sideGap1, count1, originStride1, plateMargin1);
                //
                double centerGap2 = paramsHandler.GetDoubleValue(ParamNames.WEBS_BOLT_BOTTOM_MARGIN);
                double sideGap2 = paramsHandler.GetDoubleValue(ParamNames.WEBS_BOLT_TOP_MARGIN);
                int count2 = paramsHandler.GetIntValue(ParamNames.WEBS_BOLT_VERT_COUNT);
                double[] originStride2 = paramsHandler.GetDoubleArrayValue(ParamNames.WEBS_BOLT_VERT_STRIDE);
                boltLocationHori2 = new CommonBoltLocation(40, 40, 3, new double[] { 75.0 });
                boltLocationHori2.SetBoltLocation(centerGap2, sideGap2, count2, originStride2, plateMargin2);
                //
                bg.SetHoriLocationPropsSingle(boltLocationHori1.gap1, boltLocationHori1.stride, boltLocationHori1.gap2, 2);
                m_context.MainWebMargin = bg.TotalLengthHori;
                bg.SetHoriLocationPropsSingle(boltLocationHori2.gap1, boltLocationHori2.stride, boltLocationHori2.gap2, 1);
                m_context.SubWebMargin = bg.TotalLengthHori - m_context.MainWebMargin;
                m_context.WebWidth = bg.TotalLengthHori + m_context.gap;
            }
            //处理垂直方向上的螺栓分布
            {
                double centerGap = paramsHandler.GetDoubleValue(ParamNames.WEB_BOLT_LEFT_MARGIN);
                double sideGap = paramsHandler.GetDoubleValue(ParamNames.WEB_BOLT_RIGHT_MARGIN);
                int count = paramsHandler.GetIntValue(ParamNames.WEB_BOLT_HORI_COUNT);
                double[] originStride = paramsHandler.GetDoubleArrayValue(ParamNames.WEB_BOLT_HORI_STRIDE);
                boltLocationVert = new CommonBoltLocation(40, 40, 1, new double[] { 50.0 });
                boltLocationVert.SetBoltLocation(centerGap, sideGap, count, originStride, m_context.WebHeight); //自动排列螺栓
                //
                bg.SetVertLocationProps(boltLocationVert.gap1, boltLocationVert.stride, boltLocationVert.gap2, CommonBoltLocation.BoltLocationMode.MODE3);
                m_context.WebHeight = bg.TotalLengthVert;
            }
            bg.SetVertPlane(m_context.MainHCenter.Move(-m_context.WebHeight / 2), null);
            //数量为0删除螺栓
            if (boltLocationVert.count == 0)
            {
                bg.boltFlags[0] = false;
                bg.boltFlags[1] = false;
            }
            if (boltLocationHori1.count == 0)
                bg.boltFlags[0] = false;
            if (boltLocationHori2.count == 0)
                bg.boltFlags[1] = false;
        }

    }
}
