﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace ColumnSplice42
{
    class BoltsHandler
    {
        private ComponentInstanceContext m_context = null; //公共参数、方法
        private ParamsHandler m_paramsHandler = null; //包含UI里的参数

        public BoltsHandler(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }
        public void Execute()
        {
            //螺栓组主零件
            PartBase mainPart = m_context.MainPart;
            PartBase subPart = m_context.SubPart;
            PartBase[] main = { mainPart, subPart }; //分别以主梁和次梁作为螺栓的主零件

            //处理车间螺栓选项
            BoltParameter topMainBoltParameter = m_context.FlangeBoltParameter.Clone();
            BoltParameter topSubBoltParameter = m_context.FlangeBoltParameter.Clone();
            BoltParameter webMainBoltParameter = m_context.WebBoltParameter.Clone();
            BoltParameter webSubBoltParameter = m_context.WebBoltParameter.Clone();
            BoltParameter bottomMainBoltParameter = m_context.FlangeBoltParameter.Clone();
            BoltParameter bottomSubBoltParameter = m_context.FlangeBoltParameter.Clone();
            int webMadeModeMain = m_paramsHandler.GetIntValue(ParamNames.WEBM_BOLT_MADE_MODE);
            int webMadeModeSub = m_paramsHandler.GetIntValue(ParamNames.WEBS_BOLT_MADE_MODE);
            if (webMadeModeMain == 2)
            {
                topMainBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
                webMainBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
                bottomMainBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
            }
            if (webMadeModeSub == 2)
            {
                topSubBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
                webSubBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
                bottomSubBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
            }


            //上翼缘螺栓
            m_context.TopBoltGroup.Initial(m_context.Comp, m_context.m_topFlangePlateCreate.outerPlane);
            m_context.TopBoltGroup.SetBoltParameters(topMainBoltParameter, topSubBoltParameter);
            m_context.TopBoltGroup.AddMainParts(main);
            m_context.TopBoltGroup.AddSubParts(m_context.m_topFlangePlateCreate.mainParts, m_context.m_topFlangePlateCreate.subParts);
            m_context.TopBoltGroup.Execute(0b1100);

            //腹板螺栓
            m_context.WebBoltGroup.Initial(m_context.Comp, m_context.m_frontWebPlateCreate.outerPlane);
            m_context.WebBoltGroup.SetBoltParameters(webMainBoltParameter, webSubBoltParameter);
            m_context.WebBoltGroup.AddMainParts(main);
            m_context.WebBoltGroup.AddSubParts(m_context.m_frontWebPlateCreate.mainParts, m_context.m_frontWebPlateCreate.subParts);
            m_context.WebBoltGroup.AddSubParts(m_context.m_backWebPlateCreate.mainParts, m_context.m_backWebPlateCreate.subParts);
            m_context.WebBoltGroup.Execute(0b1100);

            //下翼缘螺栓
            m_context.BottomBoltGroup.Initial(m_context.Comp, m_context.m_bottomFlangePlateCreate.outerPlane);
            m_context.BottomBoltGroup.SetBoltParameters(bottomMainBoltParameter, bottomSubBoltParameter);
            m_context.BottomBoltGroup.AddMainParts(main);
            m_context.BottomBoltGroup.AddSubParts(m_context.m_bottomFlangePlateCreate.mainParts, m_context.m_bottomFlangePlateCreate.subParts);
            m_context.BottomBoltGroup.Execute(0b1100);

        }

    }
}
