﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ColumnSplice42
{
    public class CommonBoltLocation
    {
        //默认值
        public double length;
        public double gap1;
        public double gap2;
        public int count;
        public List<double> stride;
        public List<double> defaultStride;
        double MAX = int.MaxValue - 1; //数值小于MAX看作有效值

        /// <summary>
        /// 设置默认的间距
        /// </summary>
        /// <param name="gap1"></param>
        /// <param name="gap2"></param>
        /// <param name="count"></param>
        /// <param name="stride"></param>
        public CommonBoltLocation(double gap1, double gap2, int count, double[] stride)
        {
            this.gap1 = gap1;
            this.gap2 = gap2;
            this.count = count;
            this.defaultStride = new List<double>(stride); //这里的stride必须有值
            this.stride = ParseStrideInput(stride);
        }

        /// <summary>
        /// 设置总长度的默认值
        /// </summary>
        /// <param name="length"></param>
        public void SetLength(double length)
        {
            this.length = length;
        }

        /// <summary>
        /// 处理输入的螺栓间距
        /// </summary>
        /// <param name="count">螺栓数量</param>
        /// <param name="originArray">从UI中获取的DoubleArray</param>
        /// <returns></returns>
        public List<double> ParseStrideInput(double[] originArray)
        {
            if (count > MAX)
                return new List<double>();
            if (originArray == null || originArray.Length == 0)
                originArray = this.defaultStride.ToArray();
            List<double> list = new List<double>();
            for (int i = 0; i < count - 1; ++i)
            {
                list.Add(originArray[Math.Min(i, originArray.Length - 1)]);
            }
            return list;
        }

        /// <summary>
        /// 设置螺栓位置的3种方式
        /// </summary>
        public enum BoltLocationMode
        {
            /// <summary>
            /// [内侧螺栓距中心（或断开面）距离 + 间距数组 + 外侧螺栓距外侧距离] x 2
            /// </summary>
            MODE1 = 1,

            /// <summary>
            /// 两个相邻内侧螺栓的距离 + [间距数组 + 外侧螺栓距外侧距离] x 2
            /// </summary>
            MODE2 = 2,

            /// <summary>
            /// 一侧的外侧螺栓距外侧距离 + 间距数组 + 另一侧的外侧螺栓距外侧距离
            /// </summary>
            MODE3 = 3
        }

        /// <summary>
        /// 输入UI中获取到的值，获得位置参数
        /// </summary>
        public void SetBoltLocation(double gap1, double gap2, int count, double[] stride, double length)
        {
            if (count < MAX) //一个有效的输入(count可以是0)
            {
                this.count = count;
                this.stride = ParseStrideInput(stride);
            }
            if(length > MAX) //未设置板参数，定位按默认值和设置的值来
            {
                if (gap1 < MAX)
                    this.gap1 = gap1;
                if (gap2 < MAX)
                    this.gap2 = gap2;
                return;
            }
            if (gap2 < MAX) //有side往side靠
            {
                if (gap1 < MAX && this.stride.Count > 0 && stride.Length == 0) //有center也有side，但是不输入间距
                {
                    double boltGap = (length - gap1 - gap2) / this.stride.Count; //自动布置螺栓间距
                    for (int i = 0; i < this.stride.Count; i++)
                        this.stride[i] = boltGap;
                    this.gap1 = length / 2 - this.stride.Sum() / 2;
                    this.gap2 = this.gap1;
                    return;
                }
                this.gap1 = length - this.stride.Sum() - gap2; //有side无center，center由side决定
                this.gap2 = gap2;
                return;
            }
            if (gap1 < MAX && gap2 > MAX) //有center无side往center方向靠
            {
                this.gap1 = gap1;
                this.gap2 = length - this.stride.Sum() - gap1; //有center无side，side由center决定
                return;
            }
            if (gap1 > MAX && gap2 > MAX) //无center无side就居中
            {
                this.gap1 = length / 2 - this.stride.Sum() / 2;
                this.gap2 = this.gap1;
                return;
            }
        }
    }
}
