﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;
using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;

namespace ColumnSplice42
{
    class PlateHandler
    {
        private ComponentInstanceContext m_context = null; //公共参数、方法
        private ParamsHandler m_paramsHandler = null; //包含UI里的参数

        private PlateProperties m_flangeProps = new PlateProperties();
        private PlateProperties m_webProps = new PlateProperties();

        public PlateHandler(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            // 对齐两个梁
            m_context.gap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_END_GAP);
            Plane mainAlignPlane = m_context.MainNearCenterPlane.Invert().Move(m_context.gap / 2); //都是对齐到主梁近心面
            Plane subAlignPlane = m_context.MainNearCenterPlane.Move(m_context.gap / 2);
            m_context.Comp.CreateAlignPlane(m_context.MainPart, mainAlignPlane, true);
            m_context.Comp.CreateAlignPlane(m_context.SubPart, subAlignPlane, true);
            m_context.MainNearCenterPlane = mainAlignPlane.Invert();
            m_context.SubNearCenterPlane = subAlignPlane.Invert();

            // 获取连接板参数
            //翼缘连接板
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.FLANGE_PLATE_PROPS);
                m_flangeProps.SetProps(plateProps);
                m_context.FlangeWidth = m_flangeProps.Height;
                m_context.FlangeHeight =  m_flangeProps.Width;
            }
            //腹板连接板
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.WEB_PLATE_PROPS);
                m_webProps.SetProps(plateProps);
                m_context.WebWidth = m_webProps.Height;
                m_context.WebHeight = m_webProps.Width;

                m_context.MainWebMargin = m_paramsHandler.GetDoubleValue(ParamNames.PIC_MAIN_GAP);
                m_context.SubWebMargin = m_paramsHandler.GetDoubleValue(ParamNames.PIC_SUB_GAP);
            }

            // 更新连接板参数(通过螺栓)
            m_context.FlangeBoltParameter.InitialBasic(m_context, m_paramsHandler);
            m_context.FlangeBoltParameter.InitalLocation(m_context.TopBoltGroup);
            m_context.FlangeBoltParameter.InitalLocation(m_context.BottomBoltGroup);
            m_context.WebBoltParameter.InitialBasic(m_context, m_paramsHandler);
            m_context.WebBoltParameter.InitalLocation(m_context.WebBoltGroup);

            // 添加连接板和垫板
            double underBoardThickness = m_paramsHandler.GetDoubleValue(ParamNames.PIC_SHEAR_PLATE_THICKNESS);
            PlateProperties ubPlateProperties = new PlateProperties();
            ubPlateProperties.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.FLANGE_PLATE_PROPS));
            ubPlateProperties.Thickness = underBoardThickness;
            int webPlatePlaceMode = m_paramsHandler.GetIntValue(ParamNames.ARG_SHEAR_MODE);//腹板开关
            // 上翼缘
            {
                CommonPlateCreate plateCreate = m_context.m_topFlangePlateCreate;

                Plane front = m_context.MainWebCenter.Move(m_context.FlangeHeight / 2);
                Plane back = m_context.MainWebCenter.Move(-m_context.FlangeHeight / 2).Invert();
                Plane tail = m_context.MainNearCenterPlane.Invert().Move(m_context.MainFlangeMargin);
                Plane head = m_context.SubNearCenterPlane.Invert().Move(m_context.SubFlangeMargin);

                plateCreate.SetCtrlPlane(m_context.MainTopFlange, m_context.SubTopFlange);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetCenterPlane(m_context.MainNearCenterPlane, m_context.SubNearCenterPlane);
                plateCreate.SetPlateProperties(m_flangeProps, ubPlateProperties);
                plateCreate.Execute();

            }
            // 前腹板
            {
                CommonPlateCreate plateCreate = m_context.m_frontWebPlateCreate;

                Plane front = m_context.MainHCenter.Move(m_context.WebHeight / 2);
                Plane back = m_context.MainHCenter.Move(-m_context.WebHeight / 2).Invert();
                Plane head = m_context.MainNearCenterPlane.Move(m_context.Gap / 2 + m_context.WebWidth / 2);//远离主零件
                Plane tail = m_context.SubNearCenterPlane.Move(m_context.Gap / 2 + m_context.WebWidth / 2);
                //如果在图形界面有设置，重新定位
                if (m_context.MainWebMargin != double.MaxValue)
                    tail = m_context.MainNearCenterPlane.Invert().Move(m_context.MainWebMargin);
                if (m_context.SubWebMargin != double.MaxValue)
                    head = m_context.SubNearCenterPlane.Invert().Move(m_context.SubWebMargin);

                plateCreate.SetCtrlPlane(m_context.MainFrontWeb, m_context.SubFrontWeb);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetCenterPlane(m_context.MainNearCenterPlane, m_context.SubNearCenterPlane);
                plateCreate.SetPlateProperties(m_webProps, ubPlateProperties);
                if (webPlatePlaceMode == 2)
                    plateCreate.thickness = 0;
                plateCreate.Execute();
            }
            // 后腹板
            {
                CommonPlateCreate plateCreate = m_context.m_backWebPlateCreate;

                Plane front = m_context.m_frontWebPlateCreate.frontPlane;
                Plane back = m_context.m_frontWebPlateCreate.backPlane;
                Plane head = m_context.m_frontWebPlateCreate.headPlane;
                Plane tail = m_context.m_frontWebPlateCreate.tailPlane;

                plateCreate.SetCtrlPlane(m_context.MainBackWeb, m_context.SubBackWeb);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetCenterPlane(m_context.MainNearCenterPlane, m_context.SubNearCenterPlane);
                plateCreate.SetPlateProperties(m_webProps, ubPlateProperties);
                if (webPlatePlaceMode == 3)
                    plateCreate.thickness = 0;
                plateCreate.Execute();
            }
            // 下翼缘
            {
                CommonPlateCreate plateCreate = m_context.m_bottomFlangePlateCreate;

                Plane front = m_context.m_topFlangePlateCreate.frontPlane;
                Plane back = m_context.m_topFlangePlateCreate.backPlane;
                Plane head = m_context.m_topFlangePlateCreate.headPlane;
                Plane tail = m_context.m_topFlangePlateCreate.tailPlane;

                plateCreate.SetCtrlPlane(m_context.MainBottomFlange, m_context.SubBottomFlange);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetCenterPlane(m_context.MainNearCenterPlane, m_context.SubNearCenterPlane);
                plateCreate.SetPlateProperties(m_flangeProps, ubPlateProperties);
                plateCreate.Execute();
            }
        }

    }
}
