﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;

namespace ColumnWithStiffeners182
{

    /// <summary>
    /// 组件上下文类，该类对象用来存储组件公用变量、属性
    /// </summary>
    public class ComponentInstanceContext
    {

#if false
        public const string RUNTIME_CFG_NAME = "$RUNTIME$";

        public const string DEFAULT_CFG_NAME = "$DEF$";
#endif

        private LoongCAD.STC.Object3D.ConnectionComponent m_comp;

#if false
        private LoongCAD.Util.DObject m_compProps = null;

        private LoongCAD.Util.DObject m_defaultValues = null;
#endif

        private LoongCAD.STC.Object3D.Beam m_mainPart = null;

        private LoongCAD.STC.Object3D.Beam m_subPart = null;

        private LoongCAD.Geometry.Vec3 m_mainDirection = LoongCAD.Geometry.Vec3.ZERO;

        private LoongCAD.Geometry.Vec3 m_subDirection = LoongCAD.Geometry.Vec3.ZERO;

        private LoongCAD.Geometry.Vec3 m_mainUpDirection = LoongCAD.Geometry.Vec3.AXIS_Z;

        private BeamPlanes m_mainPlanes;

        private BeamPlanes m_subPlanes;

        private LoongCAD.Geometry.Plane m_mainNearBound = null;

        private LoongCAD.Geometry.Plane m_mainNearWeb = null;

        private LoongCAD.Geometry.Plane m_mainFarWeb = null;

        private LoongCAD.Geometry.Plane m_mainFarBound = null;

        private LoongCAD.Geometry.Plane m_subNearBound = null;

        private LoongCAD.Geometry.Plane m_mainJointPlane = null;

        private LoongCAD.Geometry.Plane m_mainJointBackside = null;

        private LoongCAD.Geometry.Plane m_subAlignPlane = null;

        private double m_mainJointThickness = 0.0;

        private double m_subWebThickness = 0.0;

        private double m_subTopThickness = 0.0;

        private double m_subBottomThickness = 0.0;

        private double m_mainJointMargin = 0.0;

        private List<PartBase> m_haunchPlates = new List<PartBase>();

        private List<PartBase> m_stiffPlates = new List<PartBase>();

        private Dictionary<string, PartBase> m_stiffMap = new Dictionary<string, PartBase>();

        private Dictionary<string, StiffPlanes> m_stiffPlanesMap = new Dictionary<string, StiffPlanes>();

        private List<PartBase> m_shearPlates = new List<PartBase>();

        private List<Plane> m_shearWeldPlanes = new List<Plane>();

        private Plane m_shearFrontPlane = null;

        private Plane m_shearBackPlane = null;

        private PartBase m_nearDoublePlate;
        private PartBase m_farDoublePlate;

        private bool m_isMainRotate = false;

        private bool m_isSubRotate = false;

        private StiffPlanes m_topNearStiffPlanes, m_bottomNearStiffPlanes;

        private List<Plane> m_topWeldPlacePlane = new List<Plane>(), m_bottomWeldPlacePlane = new List<Plane>();

        ComponentInstanceContext()
        {

        }

        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (!compContext.Initialize(comp)) {
                compContext.Clear();
                return null;
            }

            return compContext;
        }

        public ConnectionComponent Comp { get => m_comp; set => m_comp = value; }
#if false
        public DObject CompProps { get => m_compProps; set => m_compProps = value; }
        public LoongCAD.Util.DObject DefaultValues { get => m_defaultValues; set => m_defaultValues = value; }
#endif
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public Beam SubPart { get => m_subPart; set => m_subPart = value; }
        public Vec3 MainDirection { get => m_mainDirection; set => m_mainDirection = value; }
        public Vec3 SubDirection { get => m_subDirection; set => m_subDirection = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public BeamPlanes SubPlanes { get => m_subPlanes; set => m_subPlanes = value; }
        public Plane MainNearBound { get => m_mainNearBound; set => m_mainNearBound = value; }
        public Plane MainFarBound { get => m_mainFarBound; set => m_mainFarBound = value; }
        public Plane MainNearWeb { get => m_mainNearWeb; set => m_mainNearWeb = value; }
        public Plane MainFarWeb { get => m_mainFarWeb; set => m_mainFarWeb = value; }
        public Plane SubNearBound { get => m_subNearBound; set => m_subNearBound = value; }
        public Plane MainJointPlane { get => m_mainJointPlane; set => m_mainJointPlane = value; }
        public Plane MainJointBackside { get => m_mainJointBackside; set => m_mainJointBackside = value; }
        public Plane SubAlignPlane { get => m_subAlignPlane; set => m_subAlignPlane = value; }
        public double MainJointThickness { get => m_mainJointThickness; set => m_mainJointThickness = value; }
        public double SubWebThickness { get => m_subWebThickness; set => m_subWebThickness = value; }
        public double SubTopThickness { get => m_subTopThickness; set => m_subTopThickness = value; }
        public double SubBottomThickness { get => m_subBottomThickness; set => m_subBottomThickness = value; }
        public double MainJointMargin { get => m_mainJointMargin; set => m_mainJointMargin = value; }
        public List<PartBase> HaunchPlates { get => m_haunchPlates; }
        public List<PartBase> StiffPlates { get => m_stiffPlates; set => m_stiffPlates = value; }
        public List<PartBase> ShearPlates { get => m_shearPlates; }

        /// <summary>
        /// 连接板用于创建焊缝的参考侧面，此列表与ShearPlates属性的连接板一一对应
        /// </summary>
        public List<Plane> ShearWeldPlanes { get => m_shearWeldPlanes; }

        /// <summary>
        /// 柱和次梁直接焊接时，切割体会把焊缝截断
        /// </summary>
        public List<Plane> TopWeldPlacePlane { get => m_topWeldPlacePlane; set => m_topWeldPlacePlane = value; }
        public List<Plane> BottomWeldPlacePlane { get => m_bottomWeldPlacePlane; set => m_bottomWeldPlacePlane = value; }

        public Plane ShearFrontPlane { get => m_shearFrontPlane; set => m_shearFrontPlane = value; }
        public Plane ShearBackPlane { get => m_shearBackPlane; set => m_shearBackPlane = value; }
        public Dictionary<string, PartBase> StiffMap { get => m_stiffMap; set => m_stiffMap = value; }
        public Dictionary<string, StiffPlanes> StiffPlanesMap { get => m_stiffPlanesMap; set => m_stiffPlanesMap = value; }
        public PartBase NearDoublePlate { get => m_nearDoublePlate; set => m_nearDoublePlate = value; }
        public PartBase FarDoublePlate { get => m_farDoublePlate; set => m_farDoublePlate = value; }
        public Vec3 MainUpDirection { get => m_mainUpDirection; set => m_mainUpDirection = value; }

        /// <summary>
        /// 图形页中柱腹板面朝右，此值表示柱相较于标准情况是否旋转了
        /// </summary>
        public bool IsMainRotate { get => m_isMainRotate; set => m_isMainRotate = value; }
        /// <summary>
        /// 图形页中次梁腹板面朝前，此值表示梁相较于标准情况是否旋转了
        /// </summary>
        public bool IsSubRotate { get => m_isSubRotate; set => m_isSubRotate = value; }
        public StiffPlanes TopNearStiffPlanes { get => m_topNearStiffPlanes; set => m_topNearStiffPlanes = value; }
        public StiffPlanes BottomNearStiffPlanes { get => m_bottomNearStiffPlanes; set => m_bottomNearStiffPlanes = value; }

        public bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;

            Beam mainPart = comp.Main as Beam;
            Beam subPart = comp.Subs.Length > 0 ? comp.Subs.First<TFObject>() as Beam : null;
            if (mainPart == null || subPart == null)
                return false;

            m_comp = comp;
            m_mainPart = mainPart;
            m_subPart = subPart;
            m_mainPlanes = BeamPlanes.CreateFromBeam(m_mainPart);
            m_subPlanes = BeamPlanes.CreateFromBeam(m_subPart);
            this.ParsePlanes();

            m_mainDirection = m_mainPart.WorldMatrix.Z.Along(Vec3.AXIS_Z);
            m_subDirection = m_subPart.WorldMatrix.Z.Along(m_comp.WorldMatrix.X);
            if (m_mainJointPlane != null && m_mainJointBackside != null) {
                m_mainJointThickness = m_mainJointPlane.Dist(m_mainJointBackside.Origin);
            }
            if (m_subPlanes.FrontWeb != null && m_subPlanes.FrontWebInside != null) {
                m_subWebThickness = m_subPlanes.FrontWeb.Dist(m_subPlanes.BackWeb.Origin);
            }
            if (m_subPlanes.TopFlange != null && m_subPlanes.TopFlangeInside != null) {
                m_subTopThickness = m_subPlanes.TopFlange.Dist(m_subPlanes.TopFlangeInside.Origin);
            }
            if (m_subPlanes.BottomFlange != null && m_subPlanes.BottomFlangeInside != null) {
                m_subBottomThickness = m_subPlanes.BottomFlange.Dist(m_subPlanes.BottomFlangeInside.Origin);
            }
            return true;
        }

        public void ParseAwaySidePlane(
            Plane posPlane, Plane negPlane, out Plane nsPlane, out Plane fsPlane)
        {
            //
            nsPlane = negPlane.Clone();
            fsPlane = posPlane.Clone();
            Vec3 nsNormal = nsPlane.Normal;

            //
            Vec3 xCompDir = this.Comp.WorldMatrix.X.Normalize();
            Vec3 yCompDir = this.Comp.WorldMatrix.Y.Normalize();
            Vec3 zCompDir = this.Comp.WorldMatrix.Z.Normalize();

            Vec3 nsOrient = nsNormal;
            nsOrient = nsOrient.Along(zCompDir);
            nsOrient = nsOrient.Along(yCompDir);
            nsOrient = nsOrient.Along(xCompDir);
            if (nsOrient.Dot(nsNormal) < 0)
                Util.Swap(ref nsPlane, ref fsPlane);
#if false
            // 基于组件X方向确定远近腹板面
            double maxDot = xCompDir.Dot(posNormal);
            {
                if (maxDot > 0) {
                    nsPlane = negPlane;
                    fsPlane = posPlane;
                } else {
                    nsPlane = posPlane;
                    fsPlane = negPlane;
                }
                maxDot = Math.Abs(maxDot);
            }
            // 基于组件Y方向确定远近腹板面
            do {
                double curDot = yCompDir.Dot(posNormal);
                if (Math.Abs(curDot) < maxDot) {
                    break;
                }
                maxDot = Math.Abs(maxDot);
                if (curDot > 0) {
                    nsPlane = negPlane;
                    fsPlane = posPlane;
                } else {
                    nsPlane = posPlane;
                    fsPlane = negPlane;
                }
            } while (false);
            // 基于组件Z方向确定远近腹板面
            do {
                double curDot = zCompDir.Dot(posNormal);
                if (Math.Abs(curDot) < maxDot) {
                    break;
                }
                maxDot = Math.Abs(maxDot);
                if (curDot > 0) {
                    nsPlane = negPlane;
                    fsPlane = posPlane;
                } else {
                    nsPlane = posPlane;
                    fsPlane = negPlane;
                }
            } while (false);
#endif
        }

        private void ParsePlanes()
        {
            // 组件X轴正方向永远朝向次零件
            Vec3 xCompDir = m_comp.WorldMatrix.X.Normalize();

            // 解析主零件长度方向的上方向
            if (!m_mainPart.WorldMatrix.Z.IsParallelTo(Vec3.AXIS_Z)) {
                m_mainUpDirection = m_mainPart.WorldMatrix.Z.Along(Vec3.AXIS_Z);
            } else {
                m_mainUpDirection = m_mainPart.WorldMatrix.Z;
            }

            // 解析主零件相对次零件的远近侧边界面
            {
                Vec3 frontDir = this.MainPlanes.FrontBound.Normal.Normalize();
                Vec3 backDir = this.MainPlanes.BackBound.Normal.Normalize();
                Vec3 topDir = this.MainPlanes.TopBound.Normal.Normalize();
                Vec3 bottomDir = this.MainPlanes.BottomBound.Normal.Normalize();
                double maxDot = 0.0;
                // 前面
                {
                    double curDot = frontDir.Dot(xCompDir);
                    if (curDot > maxDot)
                    {
                        m_mainNearBound = this.MainPlanes.FrontBound.Clone();
                        m_mainFarBound = this.MainPlanes.BackBound.Clone();
                        m_mainNearWeb = this.MainPlanes.FrontWeb.Clone();
                        m_mainFarWeb = this.MainPlanes.BackWeb.Clone();
                        m_mainJointPlane = this.MainPlanes.FrontWeb;
                        m_mainJointBackside = this.MainPlanes.FrontWebInside;
                        maxDot = curDot;
                    }
                }
                //  后面
                {
                    double curDot = backDir.Dot(xCompDir);
                    if (curDot > maxDot)
                    {
                        m_mainNearBound = this.MainPlanes.BackBound.Clone();
                        m_mainFarBound = this.MainPlanes.FrontBound.Clone();
                        m_mainNearWeb = this.MainPlanes.BackWeb.Clone();
                        m_mainFarWeb = this.MainPlanes.FrontWeb.Clone();
                        m_mainJointPlane = this.MainPlanes.BackWeb;
                        m_mainJointBackside = this.MainPlanes.BackWebInside;
                        maxDot = curDot;
                    }
                }
                // 顶面
                {
                    double curDot = topDir.Dot(xCompDir);
                    if (curDot > maxDot)
                    {
                        m_mainNearBound = this.MainPlanes.TopBound.Clone();
                        m_mainFarBound = this.MainPlanes.BottomBound.Clone();
                        m_mainJointPlane = this.MainPlanes.TopFlange;
                        m_mainJointBackside = this.MainPlanes.TopFlangeInside;
                        maxDot = curDot;
                        m_isMainRotate = true;
                    }
                }
                // 底面
                {
                    double curDot = bottomDir.Dot(xCompDir);
                    if (curDot > maxDot)
                    {
                        m_mainNearBound = this.MainPlanes.BottomBound.Clone();
                        m_mainFarBound = this.MainPlanes.TopBound.Clone();
                        m_mainJointPlane = this.MainPlanes.BottomFlange;
                        m_mainJointBackside = this.MainPlanes.BottomFlangeInside;
                        maxDot = curDot;
                        m_isMainRotate = true;
                    }
                }

                //
                if (m_mainJointPlane != null) {
                    m_mainJointPlane = m_mainJointPlane.Clone();
                }

            }

            // 解析次零件相对主零件的近侧边界面
            {
                if (m_subPlanes.HeadBound.Normal.Dot(xCompDir) < m_subPlanes.TailBound.Normal.Dot(xCompDir)) {
                    m_subNearBound = m_subPlanes.HeadBound;
                } else {
                    m_subNearBound = m_subPlanes.TailBound;
                }
            }


            /*
             * 设置次梁包围面
             */
            //
            Vec3 subWebOrient = m_subPlanes.FrontWeb.Normal.Normalize();
            Vec3 mainDir = m_mainPlanes.HeadBound.Normal.Normalize();
            subWebOrient = subWebOrient.Along(mainDir);
            if (mainDir.Dot(subWebOrient) > 0.707)
                m_isSubRotate = true;
        }

        public void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_subPart = null;
            m_mainPlanes = null;
            m_subPlanes = null;

            m_mainNearBound = null;
            m_mainFarBound = null;
        }


        public void test(Plane p, double width = 4000)
        {
            Comp.CreateQuad(p, width, width, true);
        }
        public void test(Vec3 p)
        {
            Comp.CreatePoint(p, true);
        }
        public void test(Vec3 v1, Vec3 v2)
        {
            Comp.CreateLine(v1, v2, 0, Color.Red, true);
        }
        public void test(Mat43 mat, double len = 1000)
        {
            Comp.CreatePoint(mat.Pos, true);
            Comp.CreateLine(mat.Pos, mat.Pos + len / 2 * mat.X.Normalize(), 0, Color.Red, true);
            Comp.CreateLine(mat.Pos, mat.Pos + len / 2 * mat.Y.Normalize(), 0, Color.Green, true);
            Comp.CreateLine(mat.Pos, mat.Pos + len / 2 * mat.Z.Normalize(), 0, Color.Blue, true);
        }

    }

}
