﻿using LoongCAD.UI.Component;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;

namespace ColumnWithStiffeners182
{
    public class ParamsHandler : LoongCAD.STC.Plugins.ParamsHandlerBase
    {

        /// <summary>
        /// 属性空值全局总表
        /// </summary>
        /// <remarks><b>此静态变量可直接复制到其他组件代码中使用</b></remarks>
        protected static DObject g_emptyValues = null;

        /// <summary>
        /// 属性内置默认值全局总表
        /// </summary>
        /// <remarks><b>此静态变量可直接复制到其他组件代码中使用</b></remarks>
        protected static DObject g_builtinDefaults = null;

        /// <summary>
        /// 默认构造函数
        /// </summary>
        /// <remarks><b>此方法需要复制到其他组件代码中使用</b></remarks>
        public ParamsHandler()
            : base(ParamsHandler.EmptyValues, ParamsHandler.BuiltinDefautls)
        {

        }

        /// <summary>
        /// 组件公用属性空值表
        /// </summary>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public static DObject EmptyValues
        {
            get
            {
                if (g_emptyValues == null) {
                    IntializeEmptyDefaults();
                }
                return g_emptyValues;
            }
        }

        /// <summary>
        /// 组件公用内置默认值接口
        /// </summary>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public static DObject BuiltinDefautls
        {
            get
            {
                if (g_builtinDefaults == null) {
                    IntializeEmptyDefaults();
                }
                return g_builtinDefaults;
            }
        }

        /// <summary>
        /// 初始化默认值
        /// </summary>
        /// <param name="defaultConfig">从默认值配置加载的默认值</param>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public override LoongCAD.Util.DObject InitializeDefaultValues(DObject defaultConfig)
        {
            m_defaultValues = new DObject();
            m_defaultValues.CopyFrom(defaultConfig);

            var emptyValues = ParamsHandler.EmptyValues;
            var builtinDefaults = ParamsHandler.BuiltinDefautls;
            if (emptyValues == null || builtinDefaults == null) {
                Console.WriteLine("ParamsHandler.InitializeDefaultValues: [Warning] EmptyValue/BuiltinDefaults is null!");
                return m_defaultValues;
            }

            this.TryCompleteDObjectValues(m_defaultValues, builtinDefaults, emptyValues);
            return m_defaultValues;
        }
        
        /// <summary>
        /// 初始化组件所有实例公用的属性空值、属性内置默认值
        /// </summary>
        /// <remarks><b>每个组件需要分别实现此方法</b></remarks>
        public static void IntializeEmptyDefaults()
        {
            g_emptyValues = new DObject();
            g_builtinDefaults = new DObject();

            // 槽口页
            g_emptyValues.CopyFrom(BeamNotchPage.EmptyValues);
            g_builtinDefaults.CopyFrom(BeamNotchPage.BuiltinDefaults);

            // 梁切割页
            g_emptyValues.CopyFrom(BeamCutPage.EmptyValues);
            g_builtinDefaults.CopyFrom(BeamCutPage.BuiltinDefaults);

            // 图形页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_END_CUT, 0, 3); // Bevel
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_FLANGE_CUT, 0, 1); // Bevel
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_END_GAP, double.MaxValue, 20.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_FLANGE_GAP, double.MaxValue, 20.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_MARGIN_COLUMN, double.MaxValue, 0d);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WELD_READY_MODE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_SHAPE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BEAM_ALIGN_PLANE, 0, 0);

            // 板页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                    PlatePropertyPanel.ParamNames.Thickness, 12.0,
                    PlatePropertyPanel.ParamNames.NoPrefix, "",
                    PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                    PlatePropertyPanel.ParamNames.AssNoPrefix, "",
                    PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                    PlatePropertyPanel.ParamNames.Material, "Q235B",
                    PlatePropertyPanel.ParamNames.Name, "板",
                });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CHAMFER_TYPE, 0, 2); // Line
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CHAMFER_X, double.MaxValue, 10.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CHAMFER_Y, double.MaxValue, 10.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CHAMFER_TYPE2, 0, 2); // Line
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_OUTSIDE_CHAMFER_CUT, double.MaxValue, 10.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CREATION_MODE, 0, 1); // Auto
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_ORIENTATION, 0, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_GAP, double.MaxValue, 0d);

            // 螺栓
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_BASIC_PROPS, BoltBasicPropertyPanel.EmptyValue, (DObject)new object[] {
                    BoltBasicPropertyPanel.ParamNames.BoltStandard, "TS10.9",
                    BoltBasicPropertyPanel.ParamNames.BoltSize, 20,
                    BoltBasicPropertyPanel.ParamNames.ThreadInMtrl, 1, // false
                    BoltBasicPropertyPanel.ParamNames.MadeType0, 0, // Workshop
                });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HOLE_PROPS, BoltHolePanel.EmptyValue, (DObject)new object[] {
                BoltHolePanel.ParamNames.HoleX, 0.0,
                BoltHolePanel.ParamNames.HoleY, 0.0,
                BoltHolePanel.ParamNames.HoleType, 2,
                BoltHolePanel.ParamNames.RotSlot, 3,
                BoltHolePanel.ParamNames.SlotIn, 4
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_CONTENT_PROPS, null, (DObject)new object[]
            {
                BoltContentPanel.ParamNames.PartWasherUp, false,
                BoltContentPanel.ParamNames.PartWasherDown0, false,
                BoltContentPanel.ParamNames.PartWasherDown1, true,
                BoltContentPanel.ParamNames.PartNut0, true,
                BoltContentPanel.ParamNames.PartNut1, false,
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_EXTEND_PROPS, BoltExtendPanel.EmptyValue, (DObject)new object[]
            {
                BoltExtendPanel.ParamNames.ExtLength, 0.0,
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SHAPE, 0, 1); // Array
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_ARRANGE_MODE, 0, 3);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HORI_COUNT, int.MaxValue, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HORI_STRIDE, new double[] { }, new double[] { 50.0 });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_COUNT, int.MaxValue, null);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_STRIDE, new double[] { }, new double[] { 50.0 });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HORI_OFFSET, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_OFFSET, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_OFFSET_MODE, 0, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_TOP_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_BOTTOM_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_RIGHT_MARGIN, double.MaxValue, double.MaxValue);

            // 加劲肋
            {
                string[] stiffPropNames = new string[]
                {
                    ParamNames.STIFF_TOP_NS_PROPS, ParamNames.STIFF_TOP_FS_PROPS,
                    ParamNames.STIFF_BOTTOM_NS_PROPS, ParamNames.STIFF_BOTTOM_FS_PROPS,
                };
                double thickness = 12;
                foreach (var propName in stiffPropNames) {
                    AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, propName, PlatePropertyPanel.EmptyValue, (DObject)new object[]
                    {
                        PlatePropertyPanel.ParamNames.Thickness, thickness,
                        PlatePropertyPanel.ParamNames.NoPrefix, "",
                        PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                        PlatePropertyPanel.ParamNames.Material, "Q235B",
                        PlatePropertyPanel.ParamNames.Name, "板",
                    });
                    thickness = double.MaxValue;
                }

                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CREATION, 0, 3); // Enable stiffeners
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_ORIENTATION, 0, 1); // Parallel to sub part.
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_SHAPE, 0, 3); // With chamfer
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CHAMFER_TYPE_INSIDE, 0, 2);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CHAMFER_X_INSIDE, double.MaxValue, 10.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CHAMFER_Y_INSIDE, double.MaxValue, 10.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CHAMFER_TYPE_OUTSIDE, 0, 2);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CHAMFER_X_OUTSIDE, double.MaxValue, 10.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CHAMFER_Y_OUTSIDE, double.MaxValue, 10.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_WEB_GAP, double.MaxValue, 0.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_TOP_NS_OFFSET, double.MaxValue, 0.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_TOP_FS_OFFSET, double.MaxValue, 0.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_BOTTOM_NS_OFFSET, double.MaxValue, 0.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_BOTTOM_FS_OFFSET, double.MaxValue, 0.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_NS_FLANGE_GAP_X, double.MaxValue, 0.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_NS_FLANGE_GAP_Y, double.MaxValue, 0.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_FS_FLANGE_GAP_X, double.MaxValue, 0.0);
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_FS_FLANGE_GAP_Y, double.MaxValue, 0.0);
            }

            // 腋板
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HAUNCH_TOP_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[]
            {
                PlatePropertyPanel.ParamNames.NoPrefix, "",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "板",
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HAUNCH_BOTTOM_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[]
            {
                PlatePropertyPanel.ParamNames.NoPrefix, "",
                    PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                    PlatePropertyPanel.ParamNames.Material, "Q235B",
                    PlatePropertyPanel.ParamNames.Name, "板",
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HAUNCH_CREATION, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HAUNCH_TOP_CHAMFER_X, double.MaxValue, 10.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HAUNCH_TOP_CHAMFER_Y, double.MaxValue, 10.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HAUNCH_BOTTOM_CHAMFER_X, double.MaxValue, 10.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.HAUNCH_BOTTOM_CHAMFER_Y, double.MaxValue, 10.0);

            // 双板
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_PLATE_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 10.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "板",
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_CREATION_MODE, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_PLATE_EDGE_SHAPE, 0, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_PLATE_BEVEL_ANGLE, double.MaxValue, 45.0); // In degree
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_DIAMETER, double.MaxValue, 16.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_SLOT_WIDTH, double.MaxValue, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_SLOT_HEIGHT, double.MaxValue, 16);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_TOP_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_BOTTOM_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_LEFT_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_RIGHT_MARGIN, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_BOTTOM_GAP, double.MaxValue, 50.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_FLANGE_GAP, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_HORI_COUNT, int.MaxValue, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_VERT_COUNT, int.MaxValue, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_HORI_STRIDE, new double[] { }, new double[] { });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.DOUBLE_HOLE_VERT_STRIDE, new double[] { }, new double[] { 50.0 });

            // 焊缝
            for (int i=0; i<ParamNames.WELD_PROPS_LIST.Length; ++i) {
                int madeType = 2; //工厂
                if (i == 2 || i == 7)
                    madeType = 1; //工地
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WELD_PROPS_LIST[i], WeldPropertyPanel.EmptyValue, new DObject {
                    { WeldPropertyPanel.ParamNames.SizeUp, 6.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeUp, 5 }, 
                    { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                    { WeldPropertyPanel.ParamNames.SizeDown, 6.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeDown, 5 }, 
                    { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                    { WeldPropertyPanel.ParamNames.EdgeAround, 1 }, // Edge
                    { WeldPropertyPanel.ParamNames.MadeType, madeType }, // 
                    { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                    { WeldPropertyPanel.ParamNames.RefText, "" }
                });
            }
        }

    }
}
