﻿using LoongCAD.STC.Object3D;
using LoongCAD.Geometry;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Util;
using LoongCAD.STC;

namespace ColumnWithStiffeners182
{
    public static class PlateHelper
    {

        public static void SetChamfer(
            PartBase srcPlate, Plane nearPlane,
            Define.CornerCutType chamferType, double chamferX, double chamferY)
        {
            if (srcPlate == null)
                return;

            ChamferCtrlPoint[] targetPoints = FindNearCtrlPoints(srcPlate, nearPlane);
            if (targetPoints != null) {
                for (int i=0; i<targetPoints.Length; ++i) {
                    targetPoints[i].CutType = chamferType;
                    targetPoints[i].CutX = chamferX;
                    targetPoints[i].CutY = chamferY;
                }
            }
        }

        public static void SetChamfer(
            PartBase srcPlate, Plane nearPlane0, Plane nearPlane1,
            Define.CornerCutType chamferType, double chamferX, double chamferY)
        {
            if (srcPlate == null)
                return;

            ChamferCtrlPoint targetPoint = FindNearCtrlPoint(srcPlate, nearPlane0, nearPlane1);
            if (targetPoint != null) {
                targetPoint.CutType = chamferType;
                targetPoint.CutX = chamferX;
                targetPoint.CutY = chamferY;
            }
        }

        public static ChamferCtrlPoint FindNearCtrlPoint(
            PartBase srcPlate, Plane nearPlane0, Plane nearPlane1,
            double eps = 1e-3)
        {
            if (srcPlate == null || nearPlane0 == null || nearPlane1 == null)
                return null;
            CtrlPoint[] pts = srcPlate.CtrlPoints;
            if (pts == null)
                return null;

            ChamferCtrlPoint retPoint = null;
            for (int idx = 0; idx < pts.Length; ++idx) {
                ChamferCtrlPoint curPoint = pts[idx] as ChamferCtrlPoint;
                if (nearPlane0.Dist(curPoint.WorldPos) < eps &&
                    nearPlane1.Dist(curPoint.WorldPos) < eps) {
                    retPoint = curPoint;
                    break;
                }
            }

            return retPoint;
        }

        public static ChamferCtrlPoint[] FindNearCtrlPoints(
            PartBase srcPlate, Plane nearPlane, double eps = 1e-3)
        {
            List<ChamferCtrlPoint> retPoints = new List<ChamferCtrlPoint>();
            if (srcPlate == null || nearPlane == null)
                return retPoints.ToArray();

            CtrlPoint[] pts = srcPlate.CtrlPoints;
            if (pts == null)
                return null;
            for (int idx = 0; idx < pts.Length; ++idx) {
                ChamferCtrlPoint curPoint = pts[idx] as ChamferCtrlPoint;
                if (curPoint == null)
                    continue;
                if (nearPlane.Dist(curPoint.WorldPos) < eps) {
                    retPoints.Add(curPoint);
                }
            }

            return retPoints.ToArray();
        }

        public static Vec3[] FindEdgePoints(PartBase srcPlate, Plane nearPlane, double eps = 1e-3)
        {
            ChamferCtrlPoint[] ctrlPoints = PlateHelper.FindNearCtrlPoints(srcPlate, nearPlane);
            if (ctrlPoints == null || ctrlPoints.Length < 2) {
                return null;
            }
            List<Vec3> edgePoints = new List<Vec3>();
            for (int i = 0; i < ctrlPoints.Length; ++i) {
                edgePoints.Add(ctrlPoints[i].WorldPos);
            }
            return edgePoints.ToArray();
        }

    }

}
