﻿using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ColumnWithStiffeners182
{

    /// <summary>
    /// 节点组件通用主窗口，其他组件可参考此类实现其主窗口
    /// </summary>
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {

        private DObject m_weldProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;
        //
        private bool m_isOpenStiff;
        private bool m_isChamferStiff;

        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostIntialize();
        }

        public override ComponentPlugin ReferencePlugin => m_refPlugin;

        private void OnPostIntialize()
        {
            // 设置窗口标题
            this.Text = "带加劲肋凸形连接板(101)";

            // 初始化螺栓纵向偏移模式控件
            {
                this.cbo_bolt_vert_offset_mode.ClearItems();
                this.cbo_bolt_vert_offset_mode.AddItem(0, "默认");
                this.cbo_bolt_vert_offset_mode.AddItem(1, "顶部");
                this.cbo_bolt_vert_offset_mode.AddItem(2, "中间");
                this.cbo_bolt_vert_offset_mode.AddItem(3, "底部");
                this.cbo_bolt_vert_offset_mode.UpdateItems();
                this.cbo_bolt_vert_offset_mode.SelectedKey = 0;
            }

            // 焊接坡口准备
            this.tb_weld_ready_mode.ClearItems();
            this.tb_weld_ready_mode.AddItem(0, "默认");
            this.tb_weld_ready_mode.AddItem(1, "开");
            this.tb_weld_ready_mode.AddItem(2, "关");
            this.tb_weld_ready_mode.UpdateItems();
            this.tb_weld_ready_mode.SelectedKey = 0;

            this.cbo_plate_chamfer_type2.ClearItems();
            this.cbo_plate_chamfer_type2.AddIntegerItems(0, 0, 4);
            this.cbo_plate_chamfer_type2.UpdateItems();
            this.cbo_plate_chamfer_type2.SelectedKey = 0;

            this.shearOrientationCombo.ClearItems();
            this.shearOrientationCombo.AddIntegerItems(0, 0, 4);
            this.shearOrientationCombo.UpdateItems();
            this.shearOrientationCombo.SelectedKey = 0;

            this.tb_plate_shape.ClearItems();
            this.tb_plate_shape.AddIntegerItems(0, 0, 2);
            this.tb_plate_shape.UpdateItems();
            this.tb_plate_shape.SelectedKey = 0;

            this.tb_beam_align_plane.ClearItems();
            this.tb_beam_align_plane.AddIntegerItems(0, 0, 2);
            this.tb_beam_align_plane.UpdateItems();
            this.tb_beam_align_plane.SelectedKey = 0;

            // 螺栓标准变化之后调用螺栓内容的自适应方法
            this.panel_bolt_basic_property.BoltStandardChanged += this.panel_bolt_content.AdaptBoltContents;

            foreach (TabPage curPage in this.tab_main.TabPages) {
                curPage.AutoScroll = true;
            }
        }

        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null) {
                m_weldProps.Clear();
                m_weldProps.CopyFrom(paras);
            }
        }

        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked) {
                retProps.CopyFrom(m_weldProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }

        /// <summary>
        /// 当前组件需要重写此方法，并刷新螺栓基础面板
        /// </summary>
        /// <param name="root"></param>
        public override void ClearUI(Control root = null)
        {
            // 刷新螺栓基础面板
            this.panel_bolt_basic_property.RefreshControls();
            base.ClearUI(root);
        }

        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }

        private void btn_weld_props_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null) {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_weldProps, null);
        }

        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_weldProps.CopyFrom(srcProps);
        }

        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }

        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_weldProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_weldProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_weldProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_weldProps;

            return comp.UIProps;
        }

        //接头板的两处折角
        private void chamferComboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (this.cbo_plate_chamfer_type1.SelectedIndex) {
                case 0: // Default
                case 2: { // Line
                        this.act_plate_chamfer_x.Enabled = true;
                        this.tb_plate_chamfer_x.Enabled = true;

                        this.act_plate_chamfer_y.Enabled = true;
                        this.tb_plate_chamfer_y.Enabled = true;
                    } break;
                case 3: // Round
                case 4: { // InvRound
                        this.act_plate_chamfer_x.Enabled = true;
                        this.tb_plate_chamfer_x.Enabled = true;

                        this.act_plate_chamfer_y.Enabled = false;
                        this.tb_plate_chamfer_y.Enabled = false;
                    } break;
                case 1:
                default: { // None
                        this.act_plate_chamfer_x.Enabled = false;
                        this.tb_plate_chamfer_x.Enabled = false;

                        this.act_plate_chamfer_y.Enabled = false;
                        this.tb_plate_chamfer_y.Enabled = false;
                    }
                    break;
            }
        }
        private void cbo_plate_chamfer_type2_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cbo_plate_chamfer_type2.SelectedIndex)
            {
                case 1:
                    act_plate_outside_chamfer_cut.Enabled = false;
                    tb_plate_outside_chamfer_cut.Enabled = false;
                    break;
                default:
                    act_plate_outside_chamfer_cut.Enabled = true;
                    tb_plate_outside_chamfer_cut.Enabled = true;
                    break;
            }
        }

        //图形页板形状选择
        private void tb_plate_shape_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (tb_plate_shape.SelectedIndex)
            {
                case 0:
                    SetNearEnable(true);
                    break;
                case 1:
                    SetNearEnable(false);
                    break;
            }
        }
        private void SetNearEnable(bool b)
        {
            this.act_plate_margin_column.Enabled = b;
            this.tb_plate_margin_column.Enabled = b;
            this.act_stay_stiff_chamfer_type.Enabled = b;
            this.cbo_plate_chamfer_type2.Enabled = b;
            this.act_plate_outside_chamfer_cut.Enabled = b;
            this.tb_plate_outside_chamfer_cut.Enabled = b;
            this.act_beam_align_plane.Enabled = !b;
            this.tb_beam_align_plane.Enabled = !b;
            //
            UpdateNsStiffEnable();
            this.panel_stiff_top_ns_plate_props.Enabled = b;
            this.platePropertyPanel3.Enabled = b;
        }

        //加劲肋边距是否变灰
        private void SetFlangeGapEnable(bool b)
        {
            this.act_stiff_ns_flange_gap_x.Enabled = b;
            this.tb_stiff_ns_flange_gap_x.Enabled = b;
            this.tb_stiff_ns_flange_gap_y.Enabled = b;
            this.act_stiff_ns_flange_gap_y.Enabled = b;
        }
        //近侧加劲肋是否变灰
        private void UpdateNsStiffEnable()
        {
            bool isOpenStiff = cbo_stiff_creation_mode.SelectedIndex == (int)StiffCreationMode.OPEN
                || cbo_stiff_creation_mode.SelectedIndex == (int)StiffCreationMode.AUTO
                || cbo_stiff_creation_mode.SelectedIndex == (int)StiffCreationMode.DEFAULT; //默认开启加劲肋
            bool b = tb_plate_shape.SelectedIndex == 0 && isOpenStiff;
            this.act_stiff_top_ns_offset.Enabled = b;
            this.tb_stiff_top_ns_offset.Enabled = b;
            this.act_stiff_bottom_ns_offset.Enabled = b;
            this.tb_stiff_bottom_ns_offset.Enabled = b;
        }
        //远侧加劲肋是否变灰
        private void SetFsStiffEnable(bool b)
        {
            this.act_stiff_top_fs_offset.Enabled = b;
            this.tb_stiff_top_fs_offset.Enabled = b;
            this.act_stiff_bottom_fs_offset.Enabled = b;
            this.tb_stiff_bottom_fs_offset.Enabled = b;
            this.act_stiff_web_gap.Enabled = b;
            this.tb_stiff_web_gap.Enabled = b;
        }

        //加劲肋是否开启
        private void cbo_stiff_creation_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            m_isOpenStiff = cbo_stiff_creation_mode.SelectedIndex == (int)StiffCreationMode.OPEN
                || cbo_stiff_creation_mode.SelectedIndex == (int)StiffCreationMode.AUTO
                || cbo_stiff_creation_mode.SelectedIndex == (int)StiffCreationMode.DEFAULT; //默认开启加劲肋
            SetFlangeGapEnable(m_isOpenStiff);
            SetFsStiffEnable(m_isOpenStiff);
            UpdateNsStiffEnable();
            //折角是否开启
            UpdateStiffInsideChamfer();
            UpdateStiffOutsideChamfer();
        }

        //加劲肋是否折角
        private void cbo_stiff_shape_SelectedIndexChanged(object sender, EventArgs e)
        {
            m_isChamferStiff = cbo_stiff_shape.SelectedIndex == (int)StiffShape.OPEN_CHAMFER
                || cbo_stiff_shape.SelectedIndex == (int)StiffShape.AUTO
                || cbo_stiff_shape.SelectedIndex == (int)StiffShape.DEFAULT; //默认开启折角
            UpdateStiffInsideChamfer();
            UpdateStiffOutsideChamfer();
        }

        //加劲肋内侧折角
        private void UpdateStiffInsideChamfer()
        {
            bool b1 = m_isOpenStiff; //默认开启加劲肋
            bool b2 = m_isChamferStiff; //默认开启折角
            //cut_x是否变灰
            {
                bool b3 = cbo_stiff_chamfer_type_inside.SelectedIndex != (int)ChamferType.NONE;
                if (b1 && b2 && b3)
                {
                    tb_stiff_chamfer_x_inside.Enabled = true;
                    act_stiff_chamfer_x_inside.Enabled = true;
                }
                else
                {
                    tb_stiff_chamfer_x_inside.Enabled = false;
                    act_stiff_chamfer_x_inside.Enabled = false;
                }
            }
            //cut_y是否变灰
            {
                bool b3 = cbo_stiff_chamfer_type_inside.SelectedIndex == (int)ChamferType.DEFAULT
                    || cbo_stiff_chamfer_type_inside.SelectedIndex == (int)ChamferType.LINE;
                if (b1 && b2 && b3)
                {
                    tb_stiff_chamfer_y_inside.Enabled = true;
                    act_stiff_chamfer_y_inside.Enabled = true;
                }
                else
                {
                    tb_stiff_chamfer_y_inside.Enabled = false;
                    act_stiff_chamfer_y_inside.Enabled = false;
                }
            }
            //type是否变灰
            {
                if (m_isOpenStiff && m_isChamferStiff)
                    cbo_stiff_chamfer_type_inside.Enabled = true;
                else
                    cbo_stiff_chamfer_type_inside.Enabled = false;
            }
        }
        private void cbo_stiff_chamfer_type_inside_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateStiffInsideChamfer();
        }

        //加劲肋外侧折角
        private void UpdateStiffOutsideChamfer()
        {
            bool b1 = m_isOpenStiff; //默认开启加劲肋
            bool b2 = m_isChamferStiff; //默认开启折角
            //cut_x是否变灰
            {
                bool b3 = cbo_stiff_chamfer_type_outside.SelectedIndex != (int)ChamferType.NONE;
                if (b1 && b2 && b3)
                {
                    tb_stiff_chamfer_x_outside.Enabled = true;
                    act_stiff_chamfer_x_outside.Enabled = true;
                }
                else
                {
                    tb_stiff_chamfer_x_outside.Enabled = false;
                    act_stiff_chamfer_x_outside.Enabled = false;
                }
            }
            //cut_y是否变灰
            {
                bool b3 = cbo_stiff_chamfer_type_outside.SelectedIndex == (int)ChamferType.DEFAULT
                    || cbo_stiff_chamfer_type_outside.SelectedIndex == (int)ChamferType.LINE;
                if (b1 && b2 && b3)
                {
                    tb_stiff_chamfer_y_outside.Enabled = true;
                    act_stiff_chamfer_y_outside.Enabled = true;
                }
                else
                {
                    tb_stiff_chamfer_y_outside.Enabled = false;
                    act_stiff_chamfer_y_outside.Enabled = false;
                }
            }
            //type是否变灰
            {
                if (m_isOpenStiff && m_isChamferStiff)
                    cbo_stiff_chamfer_type_outside.Enabled = true;
                else
                    cbo_stiff_chamfer_type_outside.Enabled = false;
            }
        }
        private void cbo_stiff_chamfer_type_outside_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateStiffOutsideChamfer();
        }
    }
}
