﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace ColumnWithStiffeners182
{
    public class BoltGroupHandler
    {
        private Vec3 start, end;
        private Mat43 workAxis;
        private BoltGroup boltGroup;
        private PlateHandler plateHandler;
        private ComponentInstanceContext m_context;

        //初始化起始点、工作坐标系、要删除的螺栓索引
        public BoltGroupHandler(PlateHandler plateHandler, ComponentInstanceContext m_context)
        {
            this.plateHandler = plateHandler;
            this.m_context = m_context;
            Plane startPlane; //start所在边面
            Plane boltCenterPlane = plateHandler.boltCenterPlane.Clone();
            if (plateHandler.boltArrangeMode == BoltArrangeMode.PARALLEL_SUB) //与次零件平行
            {
                startPlane = plateHandler.slopeAlignPlane.Move(plateHandler.boltLocationHori.margin2); //在此基础上再移动一定边距
            }
            else //与主零件平行或交叉模式
            {
                startPlane = plateHandler.vertAlignPlane.Move(plateHandler.boltLocationHori.margin2);
                if (plateHandler.boltArrangeMode == BoltArrangeMode.MIX)
                    boltCenterPlane.MoveThis(plateHandler.boltLocationVert.stride.Sum() / 2); //重新定位x轴位置，此时这个面是螺栓下侧面，不是中间面
                boltCenterPlane = Util.VerticalPlane(startPlane, boltCenterPlane); //boltCenterPlane是水平的，需转换
            }
            Plane endPlane = startPlane.Move(plateHandler.boltLocationHori.length); //end所在边面根据start面求得
            start = startPlane.Intersect(plateHandler.subFrontWeb, boltCenterPlane);
            end = endPlane.Intersect(plateHandler.subFrontWeb, boltCenterPlane);
            //
            Vec3 xDir = (end - start);//.Normalize();
            Vec3 zDir = plateHandler.subFrontWeb.Normal;
            workAxis = new Mat43(xDir, zDir.Cross(xDir), zDir, Vec3.ZERO);
            //
            boltGroup = CreateBoltGroup(true);
        }

        public List<Tuple<Vec3, int>> GetBoltPoints()
        {
            List<Tuple<Vec3, int>> boltPoints = new List<Tuple<Vec3, int>>();
            foreach (Bolt bolt in boltGroup.Bolts)
            {
                var boltInfo = new Tuple<Vec3, int>(bolt.WorldMatrix.Pos, bolt.ShapeIndex);
                boltPoints.Add(boltInfo);
            }
            return boltPoints;
        }

        private void SetStride(BoltGroup boltGroup, List<int> ignoreList, bool[][] deleteMatrix)
        {
            BoltLocation boltLocationHori = plateHandler.boltLocationHori, boltLocationVert = plateHandler.boltLocationVert;
            if (plateHandler.boltArrangeMode == BoltArrangeMode.MIX)
            {
                //求各个点的坐标
                List<double> xList = new List<double>(), yList = new List<double>();
                List<double> yStartLocationList = new List<double>();
                { //确定每一列第一个点的纵坐标
                    double yLocation = 0;
                    for (int x = 0; ;)
                    {
                        yStartLocationList.Add(yLocation);
                        if (x == boltLocationHori.stride.Count)
                            break;
                        yLocation += boltLocationHori.stride[x++] * Math.Tan(plateHandler.offsetAngle);
                    }
                }
                {
                    double xLocation = 0;
                    for (int x = 0; ;) //遍历列
                    {
                        double yLocation = yStartLocationList[x];
                        for (int y = 0; ;) //遍历行
                        {
                            if (deleteMatrix == null || !deleteMatrix[y + 1][x + 1]) //该螺栓是否被删除
                            {
                                xList.Add(xLocation);
                                yList.Add(yLocation);
                            }
                            if (y == boltLocationVert.stride.Count)
                                break;
                            yLocation += boltLocationVert.stride[y++] / Math.Cos(plateHandler.offsetAngle); //去掉除号即为另一种算法
                        }
                        if (x == boltLocationHori.stride.Count)
                            break;
                        xLocation += boltLocationHori.stride[x++];//移动至后一列
                    }
                }
                //
                plateHandler.boltParameter.BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_LIST;
                boltGroup.ListX = xList.ToArray();
                boltGroup.ListY = yList.ToArray();
            }
            else
            {
                boltGroup.ArrayStrideX = boltLocationHori.stride.ToArray();
                boltGroup.ArrayStrideY = boltLocationVert.stride.ToArray();
                if (ignoreList != null)
                    boltGroup.IgnoreIndices = ignoreList.ToArray();
            }
        }
        public BoltGroup CreateBoltGroup(bool isSetStride, List<PartBase> subPartList = null)
        {
            if (subPartList == null)
            {
                subPartList = new List<PartBase>();
                if (plateHandler.backPlate != null)
                    subPartList.Add(plateHandler.backPlate);
                subPartList.Add(plateHandler.m_context.SubPart);
            }
            PartBase mainPart = plateHandler.frontPlate;
            if (mainPart == null)
            {
                mainPart = subPartList[0];
                subPartList.RemoveAt(0);
            }
            BoltGroup boltGroup = plateHandler.m_context.Comp.
                CreateBoltGroup(mainPart, subPartList.ToArray(), start, end, true);
            boltGroup.WorkAxis = workAxis;
            if (isSetStride)
                SetStride(boltGroup, null, null);
            plateHandler.boltParameter.FlushBoltGroup(boltGroup);
            boltGroup.CutLength = m_context.SubPlanes.FrontWeb.Dist(m_context.SubPlanes.BackWeb.Origin) * 2 + 50;
            return boltGroup;
        }
        public void CreateBoltGroup(List<int> ignoreList, PartBase subpart1 = null, PartBase subpart2 = null)
        {
            if (this.boltGroup == null)
            {
                List<PartBase> subPartList = new List<PartBase>();
                if (subpart1 != null)
                    subPartList.Add(subpart1);
                if (subpart2 != null)
                    subPartList.Add(subpart2);
                boltGroup = CreateBoltGroup(false, subPartList);
            }
            int xCount = plateHandler.boltLocationHori.count, yCount = plateHandler.boltLocationVert.count;
            //腋板造成的忽略螺栓
            bool[][] deleteMatrix = null;
            Util.IngoreListToMatrix(ignoreList, yCount, xCount, ref deleteMatrix);
            deleteMatrix = Util.TMatrix(deleteMatrix); //BOLT_GROUPSHAPE_LIST返回的索引按y方向排列，此处有问题(删除此语句需改变deleteMatrix[y + 1][x + 1]中x,y顺序)
            //手动忽略的螺栓
            int[] oriIgnores = plateHandler.m_paramsHandler.GetIntArrayValue(ParamNames.BOLT_IGNORE); //要忽略的螺栓（手动填入的）
            int[] ignores = Util.ReverseIngoreList(oriIgnores, xCount, yCount);
            List<int> manulIgnoreList = new List<int>(ignores);
            Util.IngoreListToMatrix(manulIgnoreList, xCount, yCount, ref deleteMatrix);
            //设置总的忽略螺栓
            ignoreList.AddRange(manulIgnoreList);
            SetStride(boltGroup, ignoreList, deleteMatrix);
            boltGroup.Refresh();
            boltGroup = null;
        }
    }
}
