﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace ColumnWithStiffeners182
{
    public class PlateHandler 
    {
        public ComponentInstanceContext m_context = null;
        public ParamsHandler m_paramsHandler = null;
        public PlateProperties plateProps = new PlateProperties();
        public Plane stiffTopInsidePlane, stiffBottomInsidePlane; //加劲肋的上下内翼缘面
        public Plane subFrontWeb, subBackWeb, subFrontBound, subBackBound;
        public Plane subTopFlange, subBottomFlange, subTopFlangeInside, subBottomFlangeInside;
        public Plane plateLeftPlane, plateRightPlane; //板左侧、右侧边缘侧面
        public Plane plateTopPlane, plateBottomPlane; //板的上下侧面
        public BoltLocation boltLocationHori, boltLocationVert;
        public Vec3 xCompDir, yCompDir, zCompDir;
        public double offsetAngle;
        public double middlePlaneOffset, mainSubGap; //定位板边距
        public SteelPlate frontPlate, backPlate, topHaunch, bottomHaunch;
        public bool isOpenFrontPlate, isOpenBackPlate;
        public BoltArrangeMode boltArrangeMode;
        public ConnPlateShape plateShape;
        public Define.CornerCutType insideChamferType, outsideChamferType;
        public double insideCutX, insideCutY, outsideCut;
        public BoltParameter boltParameter = new BoltParameter();
        public Plane boltCenterPlane; //螺栓起始点都在这个面上
        public Plane vertAlignPlane, slopeAlignPlane; //垂直和斜着的对齐面
        public List<int> topBoltIndexList = new List<int>(), middleBoltIndexList = new List<int>(), bottomBoltIndexList = new List<int>();//上腋板、次梁、下腋板螺栓索引
        public BoltGroupHandler haunchBoltGroup;

        public PlateHandler() { }
        public PlateHandler(ComponentInstanceContext ctx, ParamsHandler paramsHandler)
        {
            m_context = ctx;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            /*
             * 分4种情况：
             * 1.柱腹板面朝右（图形页的右侧），次梁腹板面朝前（朝向屏幕外）。（一般情况）
             * 2.柱腹板面朝右，次梁腹板面朝上（节点上方向，图形页上侧）
             * 3.柱腹板面朝前，次梁腹板面超前
             * 4.柱腹板面朝前，次梁腹板面朝上
             */

            //参数处理
            if (!ParseParameters())
                return;

            //计算板的边面、创建面
            ParsePlanes();

            //依据主次零件的不同相对位置创建板
            if (!m_context.IsMainRotate) //柱腹板面朝右
            {
                CreatePlate1();
            }
            else //柱腹板面朝前
            {
                CreatePlate2();
            }
            //添加属性
            bool isCreatePlate = false;
            if (frontPlate != null)
            {
                isCreatePlate = true;
                plateProps.ApplyToPart(frontPlate);
                frontPlate.Flush();
            }
            if (backPlate != null)
            {
                isCreatePlate = true;
                plateProps.ApplyToPart(backPlate);
                backPlate.Flush();
            }
            //是否成功创建
            if (!isCreatePlate)
                return;

            //创建腋板
            haunchBoltGroup = new BoltGroupHandler(this, m_context);
            CreateHaunchs();

            //添加切割体
            if (frontPlate != null)
                CreateCutter(subFrontWeb, subFrontBound);
            if (backPlate != null)
                CreateCutter(subBackWeb, subBackBound);

            //创建螺栓
            {
                List<int> middleIgnoreList = new List<int>();
                middleIgnoreList.AddRange(topBoltIndexList);
                middleIgnoreList.AddRange(bottomBoltIndexList);
                haunchBoltGroup.CreateBoltGroup(middleIgnoreList, m_context.SubPart);
            }
            if (topBoltIndexList.Count() > 0)
            {
                List<int> topIgnoreList = new List<int>();
                topIgnoreList.AddRange(middleBoltIndexList);
                topIgnoreList.AddRange(bottomBoltIndexList);
                haunchBoltGroup.CreateBoltGroup(topIgnoreList, backPlate, topHaunch);
            }
            if (bottomBoltIndexList.Count() > 0)
            {
                List<int> bottomIgnoreList = new List<int>();
                bottomIgnoreList.AddRange(middleBoltIndexList);
                bottomIgnoreList.AddRange(topBoltIndexList);
                haunchBoltGroup.CreateBoltGroup(bottomIgnoreList, backPlate, bottomHaunch);
            }
        }

        public void CreatePlate1()
        {
            Plane plateTopPlaneInside = stiffTopInsidePlane; //上侧较高侧边面
            Plane plateBottomPlaneInside = stiffBottomInsidePlane; //下侧更低的边面
            //如果是次梁腹板面朝上的情况
            if (m_context.IsSubRotate)
            {
                plateTopPlaneInside = m_context.MainPlanes.TopFlangeInside?.Clone(); //上侧较高侧边面
                plateBottomPlaneInside = m_context.MainPlanes.BottomFlangeInside?.Clone(); //下侧更低的边面
                if (plateTopPlaneInside.Normal.Dot(plateTopPlane.Normal) > 0)
                    Util.Swap(ref plateTopPlaneInside, ref plateBottomPlaneInside);
            }
            if (plateProps.Height != double.MaxValue)
                plateBottomPlaneInside = plateTopPlaneInside.Invert().Move(plateProps.Height);
            //Plane plateMiddlePlane = Util.VerticalPlane(subFrontWeb, m_context.MainNearBound);
            Plane plateMiddlePlane = m_context.MainNearBound;
            double middleOffset = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_MARGIN_COLUMN);
            plateMiddlePlane.MoveThis(-middleOffset);
            //解决连接板嵌进柱的问题
            Plane frontLeftPlane = MoveInsidePlane(subFrontWeb, subFrontWeb.Move(plateProps.Thickness), plateLeftPlane);
            Plane frontMiddlePlane = MoveInsidePlane(subFrontWeb, subFrontWeb.Move(plateProps.Thickness), plateMiddlePlane);
            Plane frontRightPlane = MoveInsidePlane(subFrontWeb, subFrontWeb.Move(plateProps.Thickness), plateRightPlane);
            Plane backLeftPlane = MoveInsidePlane(subBackWeb, subBackWeb.Move(plateProps.Thickness), plateLeftPlane);
            Plane backMiddlePlane = MoveInsidePlane(subBackWeb, subBackWeb.Move(plateProps.Thickness), plateMiddlePlane);
            Plane backRightPlane = MoveInsidePlane(subBackWeb, subBackWeb.Move(plateProps.Thickness), plateRightPlane);
            //根据接头板形状选择边面
            Plane[] frontSidePlanes, backSidePlanes;
            if (plateShape == ConnPlateShape.SQUARE) //接头板是方形，改变边面
            {
                frontSidePlanes = new Plane[] { frontLeftPlane, plateTopPlane, frontRightPlane, plateBottomPlane };
                backSidePlanes = new Plane[] { backLeftPlane, plateTopPlane, backRightPlane, plateBottomPlane };
            }
            else
            {
                frontSidePlanes = new Plane[] { frontLeftPlane, plateTopPlaneInside, frontMiddlePlane, plateTopPlane,
                                       frontRightPlane, plateBottomPlane, frontMiddlePlane, plateBottomPlaneInside};
                backSidePlanes = new Plane[] { backLeftPlane, plateTopPlaneInside, backMiddlePlane, plateTopPlane,
                                       backRightPlane, plateBottomPlane, backMiddlePlane, plateBottomPlaneInside};
            }
            if (isOpenFrontPlate)
            {
                frontPlate = m_context.Comp.CreateSteelPlate(subFrontWeb, plateProps.Thickness, frontSidePlanes, true);
                m_context.ShearPlates.Add(frontPlate);
                ChamferControl chamferControl = new ChamferControl(frontPlate, frontSidePlanes);
                chamferControl.ChamferOnePoint(insideCutX, insideCutY, insideChamferType, 0);
                chamferControl.ChamferOnePoint(insideCutY, insideCutX, insideChamferType, 1);
                if (plateShape == ConnPlateShape.CONVEX) //只有凸形接头板需要对凹进去的部分折角
                    chamferControl.ChamferPoints(outsideCut, outsideCut, outsideChamferType, new int[] { 3, 6 });
            }
            if (isOpenBackPlate)
            {
                backPlate = m_context.Comp.CreateSteelPlate(subBackWeb, plateProps.Thickness, backSidePlanes, true);
                m_context.ShearPlates.Add(backPlate);
                ChamferControl chamferControl = new ChamferControl(backPlate, backSidePlanes);
                chamferControl.ChamferOnePoint(insideCutX, insideCutY, insideChamferType, 0);
                chamferControl.ChamferOnePoint(insideCutY, insideCutX, insideChamferType, 1);
                if (plateShape == ConnPlateShape.CONVEX)
                    chamferControl.ChamferPoints(outsideCut, outsideCut, outsideChamferType, new int[] { 3, 6 });
            }
        }

        public void CreatePlate2()
        {
            Plane[] sidePlanes = { m_context.MainNearBound, plateTopPlane, plateRightPlane, plateBottomPlane};
            if (isOpenFrontPlate)
            {
                frontPlate = m_context.Comp.CreateSteelPlate(subFrontWeb, plateProps.Thickness, sidePlanes, true);
                m_context.ShearPlates.Add(frontPlate);
                ChamferControl chamferControl = new ChamferControl(frontPlate, sidePlanes);
                chamferControl.ChamferOnePoint(insideCutX, insideCutY, insideChamferType, 0);
                chamferControl.ChamferOnePoint(insideCutY, insideCutX, insideChamferType, 1);
            }
            if (isOpenBackPlate)
            {
                backPlate = m_context.Comp.CreateSteelPlate(subBackWeb, plateProps.Thickness, sidePlanes, true);
                m_context.ShearPlates.Add(backPlate);
                ChamferControl chamferControl = new ChamferControl(backPlate, sidePlanes);
                chamferControl.ChamferOnePoint(insideCutX, insideCutY, insideChamferType, 0);
                chamferControl.ChamferOnePoint(insideCutY, insideCutX, insideChamferType, 1);
            }
        }

        public void CreateHaunchs()
        {
            HaunchOpen haunchOpen = (HaunchOpen)m_paramsHandler.GetIntValue(ParamNames.HAUNCH_CREATION);
            //板边缘点
            Vec3 leftTopPoint = m_context.SubAlignPlane.Intersect(plateTopPlane, subFrontWeb);
            Vec3 leftBottomPoint = m_context.SubAlignPlane.Intersect(plateBottomPlane, subFrontWeb);
            Vec3 rightTopPoint = plateRightPlane.Intersect(plateTopPlane, subFrontWeb);
            Vec3 rightBottomPoint = plateRightPlane.Intersect(plateBottomPlane, subFrontWeb);
            //边缘点和次梁翼缘距离
            double leftTopDistance = subTopFlange.SignedDist(leftTopPoint);
            double rightTopDistance = subTopFlange.SignedDist(rightTopPoint);
            double leftBottomDistance = subBottomFlange.SignedDist(leftBottomPoint);
            double rightBottomDistance = subBottomFlange.SignedDist(rightBottomPoint);
            //记录切割体是否切割了次梁
            if (leftTopDistance > 0 && isOpenFrontPlate)
                m_context.TopWeldPlacePlane.Add(subFrontWeb);
            if (rightTopDistance > 0 && isOpenBackPlate)
                m_context.TopWeldPlacePlane.Add(subBackWeb);
            if (leftBottomDistance > 0 && isOpenFrontPlate)
                m_context.BottomWeldPlacePlane.Add(subFrontWeb);
            if (rightBottomDistance > 0 && isOpenBackPlate)
                m_context.BottomWeldPlacePlane.Add(subBackWeb);
            if (haunchOpen == HaunchOpen.CLOSE)
                return;
            //默认宽高
            double defaultWidthTop, defaultHeightTop, defaultWidthBottom, defaultHeightBottom;
            {
                defaultHeightTop = Math.Max(leftTopDistance, rightTopDistance);//边缘点和翼缘交点确定默认宽度
                defaultHeightBottom = Math.Max(leftBottomDistance, rightBottomDistance);
                defaultWidthTop = Util.PlaneLineIntersect(subTopFlange, leftTopPoint, leftBottomPoint)
                    .Dist(Util.PlaneLineIntersect(subTopFlange, rightTopPoint, rightBottomPoint));
                defaultWidthBottom = Util.PlaneLineIntersect(subBottomFlange, leftTopPoint, leftBottomPoint)
                    .Dist(Util.PlaneLineIntersect(subBottomFlange, rightTopPoint, rightBottomPoint));
                if (defaultHeightTop < 50)
                    defaultHeightTop = 50;
                if (defaultHeightBottom < 50)
                    defaultHeightBottom = 50;
            }
            //螺栓上侧是否超过次梁边缘（自动时是否开启腋板）
            List<Tuple<Vec3, int>> boltPoints = haunchBoltGroup.GetBoltPoints();
            foreach (var boltInfo in boltPoints)
            {
                Vec3 boltLocation = boltInfo.Item1;
                int boltIndex = boltInfo.Item2 + 1;
                if (subTopFlange.SignedDist(boltLocation) > 0)
                {
                    topBoltIndexList.Add(boltIndex);
                }
                else if (subBottomFlange.SignedDist(boltLocation) > 0)
                {
                    bottomBoltIndexList.Add(boltIndex);
                }
                else
                {
                    middleBoltIndexList.Add(boltIndex);
                }
            }
            bool isAutoOpenTop = topBoltIndexList.Count() > 0;
            bool isAutoOpenBottom = bottomBoltIndexList.Count() > 0;
            //折角参数
            double topCutX = m_paramsHandler.GetDoubleValue(ParamNames.HAUNCH_TOP_CHAMFER_X);
            double topCutY = m_paramsHandler.GetDoubleValue(ParamNames.HAUNCH_TOP_CHAMFER_Y);
            double bottomCutX = m_paramsHandler.GetDoubleValue(ParamNames.HAUNCH_BOTTOM_CHAMFER_X);
            double bottomCutY = m_paramsHandler.GetDoubleValue(ParamNames.HAUNCH_BOTTOM_CHAMFER_Y);
            //创建腋板
            bool isOpenTop = (isAutoOpenTop && haunchOpen == HaunchOpen.AUTO) || haunchOpen == HaunchOpen.OPEN;
            bool isOpenBottom = (isAutoOpenBottom && haunchOpen == HaunchOpen.AUTO) || haunchOpen == HaunchOpen.OPEN;
            if (isOpenTop)
            {
                topHaunch = CreateHaunch(ParamNames.HAUNCH_TOP_PROPS, subTopFlange, 
                    defaultWidthTop, defaultHeightTop, topCutX, topCutY);
            }
            if (isOpenBottom)
            {
                bottomHaunch = CreateHaunch(ParamNames.HAUNCH_BOTTOM_PROPS, subBottomFlange, 
                    defaultWidthBottom, defaultHeightBottom, bottomCutX, bottomCutY);
            }
            //
            m_context.HaunchPlates.Add(topHaunch);
            m_context.HaunchPlates.Add(bottomHaunch);




        }
        private SteelPlate CreateHaunch(String propsName, Plane bottPlane, double defaultWidth, double defaultHeight, double cutX, double cutY)
        {
            PlateProperties haunchProps = new PlateProperties();
            DObject haunchPropsD = m_paramsHandler.GetDObjectValue(propsName);
            haunchProps.SetProps(haunchPropsD);
            Util.Swap(ref haunchProps.Width, ref haunchProps.Height); //为了与TK一致
            Plane subWebCenterPlane = Util.CenterPlane(subFrontWeb, subBackWeb);
            HaunchHelper haunch = new HaunchHelper(m_context.SubAlignPlane, bottPlane, subWebCenterPlane, haunchProps);
            haunch.isRightPlaneVertical = true;
            haunch.SetDefaultSize(defaultWidth, defaultHeight);
            haunch.SetOutsideChamfer(cutX, cutY, Define.CornerCutType.CORNER_CUT_LINE);
            haunch.CreateHaunch(m_context.Comp);
            return haunch.haunch;
        }

        public bool ParseParameters()
        {
            //板参数
            PlateOpenMode plateOpenMode = (PlateOpenMode)m_paramsHandler.GetIntValue(ParamNames.PLATE_CREATION_MODE); //板开关
            isOpenFrontPlate = plateOpenMode == PlateOpenMode.AUTO || plateOpenMode == PlateOpenMode.FRONT || plateOpenMode == PlateOpenMode.BOTH;
            isOpenBackPlate = plateOpenMode == PlateOpenMode.BACK || plateOpenMode == PlateOpenMode.BOTH;
            DObject platePropsD = m_paramsHandler.GetDObjectValue(ParamNames.PLATE_PROPS);
            plateProps.SetProps(platePropsD);
            middlePlaneOffset = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_MARGIN_COLUMN); //向柱内侧偏移
            mainSubGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_END_GAP);
            if (!Util.PlatePropIsValid(plateProps))
                return false;

            //板折角
            int insideChamferMode = m_paramsHandler.GetIntValue(ParamNames.PLATE_CHAMFER_TYPE);
            insideCutX = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_CHAMFER_X);
            insideCutY = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_CHAMFER_Y);
            ChamferControl.SetCutParameter(ref insideCutX, ref insideCutY, insideChamferMode, out insideChamferType);
            int outsideChamferMode = m_paramsHandler.GetIntValue(ParamNames.PLATE_CHAMFER_TYPE2);
            outsideCut = m_paramsHandler.GetDoubleValue(ParamNames.PLATE_OUTSIDE_CHAMFER_CUT);
            ChamferControl.SetCutParameter(ref outsideCut, ref outsideCut, outsideChamferMode, out outsideChamferType);

            //确定螺栓垂直方向参数
            double[] defaultStride = { 60.0 };
            double boltMarginTop = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_TOP_MARGIN);
            double boltMarginBottom = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_BOTTOM_MARGIN);
            int vCount = m_paramsHandler.GetIntValue(ParamNames.BOLT_VERT_COUNT);
            double[] vStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_VERT_STRIDE);
            double height = double.MaxValue; //板高度
            if (m_context.IsMainRotate) //柱翼缘朝向次梁时高度参数才会影响螺栓
                height = plateProps.Height;
            boltLocationVert = new BoltLocation(40, 40, 3, defaultStride);
            boltLocationVert.SetBoltLocation(boltMarginBottom, boltMarginTop, vCount, vStride, height);
            boltLocationVert.stride.Reverse(); //与TK保持一致

            //确定螺栓水平方向参数
            double boltMarginRight = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_RIGHT_MARGIN);
            double boltMarginLeft = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_HORI_OFFSET);
            int hCount = m_paramsHandler.GetIntValue(ParamNames.BOLT_HORI_COUNT);
            double[] hStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_HORI_STRIDE);
            boltLocationHori = new BoltLocation(40, 40, 3, defaultStride);
            double width = plateProps.Width - middlePlaneOffset - mainSubGap;
            boltLocationHori.SetBoltLocation(boltMarginRight, boltMarginLeft, hCount, hStride, width);

            //螺栓参数解析
            DObject basicProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_BASIC_PROPS);
            DObject holeProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_HOLE_PROPS);
            DObject contentProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_CONTENT_PROPS);
            DObject extendProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_EXTEND_PROPS);
            int shapeIndex = m_paramsHandler.GetIntValue(ParamNames.BOLT_SHAPE);
            boltParameter.CopyFromBasicProps(basicProps);
            boltParameter.CopyFromHoleProps(holeProps);
            boltParameter.CopyFromContentProps(contentProps);
            boltParameter.CopyFromExtendProps(extendProps);
            boltParameter.SetShape(shapeIndex);

            //螺栓排布
            boltArrangeMode = (BoltArrangeMode)m_paramsHandler.GetIntValue(ParamNames.BOLT_ARRANGE_MODE);

            //板形状
            plateShape = (ConnPlateShape)m_paramsHandler.GetIntValue(ParamNames.PLATE_SHAPE);

            return true;
        }

        public void ParsePlanes()
        {
            //节点坐标系
            xCompDir = m_context.Comp.WorldMatrix.X.Normalize();
            yCompDir = m_context.Comp.WorldMatrix.Y.Normalize();
            zCompDir = m_context.Comp.WorldMatrix.Z.Normalize();

            //次梁前后面
            subFrontWeb = m_context.SubPlanes.FrontWeb.Clone();
            subBackWeb = m_context.SubPlanes.BackWeb.Clone();
            subFrontBound = m_context.SubPlanes.FrontBound.Clone();
            subBackBound = m_context.SubPlanes.BackBound.Clone();
            if (IsReverseByMatrix(subFrontWeb.Normal, zCompDir, -yCompDir))
            {
                Util.Swap(ref subFrontWeb, ref subBackWeb);
                Util.Swap(ref subFrontBound, ref subBackBound);
            }

            //次梁顶底面
            subTopFlange = m_context.SubPlanes.TopBound.Clone();
            subBottomFlange = m_context.SubPlanes.BottomBound.Clone();
            subTopFlangeInside = m_context.SubPlanes.TopFlangeInside?.Clone();
            subBottomFlangeInside = m_context.SubPlanes.BottomFlangeInside?.Clone();
            if (IsReverseByMatrix(subTopFlange.Normal, yCompDir, zCompDir))
            {
                Util.Swap(ref subTopFlange, ref subBottomFlange);
                Util.Swap(ref subTopFlangeInside, ref subBottomFlangeInside);
            }
            Util.SetDefaultValue(ref subTopFlangeInside, subTopFlange.Invert());
            Util.SetDefaultValue(ref subBottomFlangeInside, subBottomFlange.Invert());

            //对齐面
            Plane alignTopBottomPlane = offsetAngle > 0 ? subBottomFlange : subTopFlange;
            vertAlignPlane = new Plane(m_context.SubAlignPlane.Intersect(alignTopBottomPlane, subFrontWeb), m_context.MainNearBound.Normal);
            slopeAlignPlane = Util.VerticalPlane(alignTopBottomPlane, m_context.SubAlignPlane);

            //次梁向上（下）偏移的角度
            offsetAngle = Util.VectorAngle(m_context.SubDirection, vertAlignPlane.Normal);
            if (m_context.SubDirection.Dot(zCompDir) < 0) //次梁向上偏转还是向下偏转
                offsetAngle = -offsetAngle;

            //接头板创建上、下侧面（默认平行于次梁）
            BoltOffsetMode boltPlace = (BoltOffsetMode)m_paramsHandler.GetIntValue(ParamNames.BOLT_VERT_OFFSET_MODE);
            double vertOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_VERT_OFFSET);
            switch (boltPlace)
            {
                case BoltOffsetMode.TOP:
                    if (vertOffset == double.MaxValue)
                        vertOffset = subTopFlange.Dist(subTopFlangeInside.Origin) + 10;
                    else
                        vertOffset -= boltLocationVert.margin2;
                    plateTopPlane = subTopFlange.Move(-vertOffset);
                    plateBottomPlane = plateTopPlane.Invert().Move(boltLocationVert.length);
                    break;
                case BoltOffsetMode.MIDDLE:
                    Plane centerPlane = Util.CenterPlane(subTopFlange, subBottomFlange);
                    Util.SetDefaultValue(ref vertOffset, 0);
                    centerPlane.MoveThis(vertOffset);
                    plateTopPlane = centerPlane.Move(boltLocationVert.CenterMargin2);
                    plateBottomPlane = centerPlane.Invert().Move(boltLocationVert.CenterMargin1);
                    break;
                case BoltOffsetMode.BOTTOM:
                    if (vertOffset == double.MaxValue)
                        vertOffset = subBottomFlange.Dist(subBottomFlangeInside.Origin) + 10;
                    else
                        vertOffset -= boltLocationVert.margin1;
                    plateBottomPlane = subBottomFlange.Move(-vertOffset);
                    plateTopPlane = plateBottomPlane.Invert().Move(boltLocationVert.length);
                    break;
                default:
                    break;
            }

            //接头板外侧面
            plateRightPlane = vertAlignPlane.Move(boltLocationHori.length);

            //根据螺栓排列方式和板方向不同重新计算板边面
            boltCenterPlane = plateBottomPlane.Move(-boltLocationVert.CenterMargin1); //保存用于定位螺栓的中心面
            PlateOrient plateOrientMode = (PlateOrient)m_paramsHandler.GetIntValue(ParamNames.PLATE_ORIENTATION);
            if (plateOrientMode == PlateOrient.AUTO || plateOrientMode == PlateOrient.PARALLEL_SUB)
            {
                if (boltArrangeMode == BoltArrangeMode.PARALLEL_SUB)
                {
                    Plane endPlane = slopeAlignPlane.Move(boltLocationHori.length); //螺栓右侧边界面（加上了边距后的边界面）
                    Vec3 bottomIntersect = endPlane.Intersect(subFrontWeb, plateBottomPlane);//与板上、下边缘的交点
                    Vec3 topIntersect = endPlane.Intersect(subFrontWeb, plateTopPlane); //选取离得较远的作为面创建点
                    Vec3 p = offsetAngle > 0 ? bottomIntersect : topIntersect;
                    plateRightPlane = new Plane(p, m_context.SubAlignPlane.Normal) ;
                }
                if (boltArrangeMode == BoltArrangeMode.PARALLEL_MAIN)
                {
                    double topMove = boltLocationVert.stride.Sum() / 2 * Math.Cos(offsetAngle);//移动topMove到达螺栓组最靠近sutopflange的螺栓
                    double k = Math.Abs(Math.Tan(offsetAngle));
                    double bottomMove = (k * boltLocationHori.StrideSum + boltLocationVert.StrideSum / 2) / Math.Sqrt(k * k + 1);
                    if (offsetAngle < 0)
                        Util.Swap(ref topMove, ref bottomMove);
                    plateTopPlane = boltCenterPlane.Invert().Move(topMove + boltLocationVert.margin2);
                    plateBottomPlane = boltCenterPlane.Move(bottomMove + boltLocationVert.margin1);
                }
            }
            else //板末端垂直于柱
            {
                Plane boltLeftPlane, boltRightPlane;//螺栓上下左右边面(不包含边距)
                Plane boltTopPlane = boltCenterPlane.Invert().Move(boltLocationVert.StrideSum / 2);
                Plane boltBottomPlane = boltCenterPlane.Move(boltLocationVert.StrideSum / 2);
                switch (boltArrangeMode)
                {
                    case BoltArrangeMode.AUTO:
                    case BoltArrangeMode.MIX:
                        boltRightPlane = vertAlignPlane.Move(boltLocationHori.length - boltLocationHori.margin1);
                        boltLeftPlane = vertAlignPlane.Move(boltLocationHori.margin2);
                        SetTopBottomPlane(boltTopPlane, boltBottomPlane, boltLeftPlane, boltRightPlane);
                        break;
                    case BoltArrangeMode.PARALLEL_SUB:
                        boltRightPlane = slopeAlignPlane.Move(boltLocationHori.length - boltLocationHori.margin1);
                        boltLeftPlane = slopeAlignPlane.Move(boltLocationHori.margin2);
                        plateRightPlane = Util.OrientPlane(offsetAngle > 0?boltBottomPlane:boltTopPlane, boltRightPlane, xCompDir).Move(boltLocationHori.margin1);
                        SetTopBottomPlane(boltTopPlane, boltBottomPlane, boltLeftPlane, boltRightPlane);
                        break;
                    case BoltArrangeMode.PARALLEL_MAIN:
                        boltCenterPlane = Util.VerticalPlane(vertAlignPlane, this.boltCenterPlane);//变为水平的螺栓中心面，法线朝下
                        plateTopPlane = boltCenterPlane.Invert().Move(boltLocationVert.CenterMargin2);
                        plateBottomPlane = boltCenterPlane.Move(boltLocationVert.CenterMargin1);
                        break;
                }
            }

            //接头板内侧面
            if (m_context.IsMainRotate)
            {
                plateLeftPlane = m_context.MainNearBound.Clone();
            }
            else
            {
                plateLeftPlane = m_context.MainNearWeb.Clone();
                if (m_context.NearDoublePlate != null)
                {
                    DObject doublePlateProps = m_paramsHandler.GetDObjectValue(ParamNames.DOUBLE_PLATE_PROPS);
                    double thickness = (double)doublePlateProps.Get("thickness");
                    plateLeftPlane.MoveThis(thickness);
                }
            }
            m_context.ShearWeldPlanes.Add(plateLeftPlane);

            //加劲肋内侧面
            Util.SetDefaultValue(ref stiffTopInsidePlane, m_context.TopNearStiffPlanes?.ZBottomPlane, subTopFlange.Invert());
            Util.SetDefaultValue(ref stiffBottomInsidePlane, m_context.BottomNearStiffPlanes?.ZBottomPlane, subBottomFlange.Invert());
        }

        public void CreateCutter(Plane bottomPlane, Plane topPlane)
        {
            Plane rightPlane = plateRightPlane.Clone();
            double flangeCutGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_FLANGE_GAP);
            rightPlane.MoveThis(flangeCutGap);
            Plane[] cutSidePlanes = { m_context.SubAlignPlane, plateTopPlane, rightPlane, plateBottomPlane };
            m_context.Comp.CreatePlateCutter(m_context.SubPart, bottomPlane, topPlane, cutSidePlanes, true);
        }

        private bool IsReverseByMatrix(Vec3 v, params Vec3[] orientList)
        {
            Vec3 ov = v;
            for (int i = 0; i < orientList.Length; i++)
                v = v.Along(orientList[i]);
            return ov.Dot(v) < 0;
        }

        private void SetTopBottomPlane(Plane boltTopPlane, Plane boltBottomPlane, Plane boltLeftPlane, Plane boltRightPlane)
        {
            if (offsetAngle < 0) //次梁向上偏转时使用螺栓右边面重新定位螺栓上侧面，使用螺栓左边面重新定位螺栓下侧面
                Util.Swap(ref boltRightPlane, ref boltLeftPlane); //反之互换左右边面
            boltTopPlane = Util.OrientPlane(boltRightPlane, boltTopPlane, zCompDir);//让topplane变为水平
            boltBottomPlane = Util.OrientPlane(boltLeftPlane, boltBottomPlane, -zCompDir);
            plateTopPlane = boltTopPlane.Move(boltLocationVert.margin2);
            plateBottomPlane = boltBottomPlane.Move(boltLocationVert.margin1);
        }
    
        //梁向左右偏移，会导致连接板与柱碰撞
        private Plane MoveInsidePlane(Plane sidePlane1, Plane sidePlane2, Plane curPlane)
        {
            Vec3 orient = m_context.MainNearBound.Normal;
            if (curPlane.Normal.IsVerticalTo(sidePlane1.Normal))
                return curPlane;
            Plane movePlane1 = Util.VerticalPlane(sidePlane1, sidePlane1.Intersect(curPlane), orient);
            Plane movePlane2 = Util.VerticalPlane(sidePlane2, sidePlane2.Intersect(curPlane), orient);
            Plane movePlane = movePlane1.SignedDist(movePlane2.Origin) > 0 ? movePlane2 : movePlane1;
            Util.Along(ref movePlane, curPlane.Normal);
            return movePlane;
        }
    }
}
