﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ColumnWithStiffeners182
{
    public class StiffPlanes
    {

        public const string TOP_NS_KEY = "top_ns";
        public const string TOP_FS_KEY = "top_fs";
        public const string BOTTOM_NS_KEY = "bottom_ns";
        public const string BOTTOM_FS_KEY = "bottom_fs";

        public Plane ZTopPlane = null;
        public Plane ZBottomPlane = null;

        // 0 - 与下翼板连接的侧面
        // 1 - 与腹板连接的侧面
        // 2 - 与上翼板连接的面
        // 3 - 与翼板侧面重合的面
        // 根据0、2面计算出的侧面距离为板高度
        // 根据1、3面计算出的侧面距离为板宽度
        public List<Plane> RefPlanes = new List<Plane>();
        public List<double> RefPlaneOffsets = new List<double>();

        // 根据上下平面角度、加劲板属性计算的最终侧面
        // 0 - 与下翼板连接的侧面
        // 1 - 与腹板连接的侧面
        // 2 - 与上翼板连接的面
        // 3 - 与翼板侧面重合的面
        public List<Plane> SidePlanes = new List<Plane>();

        public string StiffKey = "";

        public StiffPlanes()
        {

        }

        public void ParseSidePlanes(int stiffOrient, PlateProperties plateProps)
        {
            this.SidePlanes.Clear();
            if (plateProps.Width <= 0 || plateProps.Height <= 0)
                return;

            //
            Vec3 zTopNormal = this.ZTopPlane.Normal.Normalize();
            List<Vec3> refNormals = new List<Vec3>();
            for (int i=0; i<this.RefPlanes.Count; ++i) {
                Vec3 curNormal = this.RefPlanes[i].Normal;
                if (curNormal.IsParallelTo(zTopNormal)) {
                    return;
                }
                refNormals.Add(curNormal.Normalize());
            }

            // 计算紧贴主零件各板的加劲肋侧面
            {
                for (int idx = 0; idx < this.RefPlanes.Count; ++idx)
                {
                    Vec3 pt0;
                    if (zTopNormal.Dot(refNormals[idx]) >= 0.0)
                    {
                        pt0 = this.RefPlanes[idx].Intersect(
                            this.ZTopPlane, this.RefPlanes[(idx + 1) % this.RefPlanes.Count]);
                    }
                    else
                    {
                        pt0 = this.RefPlanes[idx].Intersect(
                            this.ZBottomPlane, this.RefPlanes[(idx + 1) % this.RefPlanes.Count]);
                    }
                    if (pt0 == Vec3.MAX)
                        break;
                    Vec3 sideNormal = zTopNormal.Cross(refNormals[idx].Cross(zTopNormal));
                    Plane curPlane = new Plane(pt0, sideNormal);
                    SidePlanes.Add(curPlane);
                }
            }
            if (SidePlanes.Count != this.RefPlanes.Count)
                return;

            // 根据指定宽度调整侧面3
            if (plateProps.Width != double.MaxValue) {
                double oriWidth = this.SidePlanes[1].Dist(this.SidePlanes[3].Origin);
                double curOffset = plateProps.Width - oriWidth;
                this.SidePlanes[3].MoveThis(curOffset);
            }

            // 根据指定高度调整侧面0、2
            if (plateProps.Height != double.MaxValue) {
                double oriHeight = this.SidePlanes[0].Dist(this.SidePlanes[2].Origin);
                double curOffset = (plateProps.Height - oriHeight) * 0.5;
                this.SidePlanes[0].MoveThis(curOffset);
                this.SidePlanes[2].MoveThis(curOffset);
            }

            // 应用每个侧面的偏移
            for (int idx = 0; idx < this.SidePlanes.Count; ++idx) {
                Vec3 refDir = refNormals[idx];
                this.SidePlanes[idx].MoveThis(-this.RefPlaneOffsets[idx]);
            }

            // 检查对立侧面距离是否有效
            if (this.SidePlanes[0].Dist(this.SidePlanes[2].Origin) == 0 ||
                this.SidePlanes[1].Dist(this.SidePlanes[3].Origin) == 0) {
                this.SidePlanes.Clear();
            }
        }

    }

}
