﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ColumnWithStiffeners186
{
    
    public class BoltParameter
    {

        public enum SoltInMode
        {
            NONE = 0,
            BEAM = 1,
            PLATE = 2,
            BOTH = 3,
        }

        public Vec3 BoltPos = Vec3.ZERO;
        public double BoltSizeX = 0.0;
        public double BoltSizeY = 0.0;
        public Vec3 BoltDirX = Vec3.AXIS_X;
        public Vec3 BoltDirY = Vec3.AXIS_Y;
        public double BoltTopMargin = 0.0;
        public double BoltBottomMargin = 0.0;
        public double BoltRightMargin = 0.0;

        public Define.BoltGroupShape BoltShape = Define.BoltGroupShape.BOLT_GROUPSHAPE_NONE;
        public int HoriCount = 0;
        public int VertCount = 0;
        public List<double> StrideX = new List<double>();
        public List<double> StrideY = new List<double>();
        public List<int> BoltIgnores = new List<int>();
        public int BoltOrientation = 0;
        public List<Vec3> CornerPoints = new List<Vec3>();

        public string BoltStandard;
        public double BoltSize = 0.0;
        public double HoleTol = 0.0;
        public bool ThreadInMaterial = false;
        public Define.PartMadeType MadeType0 = Define.PartMadeType.PART_MADE_SITE;
        public Define.PartMadeType MadeType1 = Define.PartMadeType.PART_MADE_SITE;

        public Define.BoltHoleType HoleType = Define.BoltHoleType.BOLT_HOLE_OVERSIZED;
        public double HoleX = 0.0;
        public double HoleY = 0.0;
        public Define.BoltRotateSlot RotSlot = 0;
        public SoltInMode SlotIn = SoltInMode.NONE;

        public bool PartWasherUp = false;
        public bool PartWasherDown0 = false;
        public bool PartWasherDown1 = false;
        public bool PartNut0 = false;
        public bool PartNut1 = false;

        public double ExtLength = 0.0;

        public BoltParameter()
        {

        }

        public void UpdateCornerPoints()
        {
            this.CornerPoints.Clear();
            this.CornerPoints.Add(this.BoltPos);
            this.CornerPoints.Add(this.BoltPos + this.BoltDirX * this.BoltSizeX);
            this.CornerPoints.Add(this.BoltPos + this.BoltDirY * this.BoltSizeY);
            this.CornerPoints.Add(this.CornerPoints[1] + this.BoltDirY * this.BoltSizeY);
        }

        public void CopyFromBasicProps(DObject basicProps)
        {
            if (basicProps == null)
                return;

            this.BoltStandard = basicProps.GetString("bolt_standard");
            this.BoltSize = basicProps.GetDouble("bolt_size");
            this.HoleTol = basicProps.GetDouble("hole_tol");
            this.ThreadInMaterial = basicProps.GetInt("thread_in_mtrl") == 2;
            this.MadeType0 = this.ParseMadeType(basicProps.GetInt("made_type0"));
            this.MadeType1 = this.ParseMadeType(basicProps.GetInt("made_type1"));

            /// FIXME: 自适应孔容差范围，有待转移到底层选项中
            if (this.HoleTol == double.MaxValue) {
                if (this.BoltSize >= 27.0)
                    this.HoleTol = 3.0;
                else if (this.BoltSize >= 16.0)
                    this.HoleTol = 2.0;
                else
                    this.HoleTol = 1.5;
            }
        }

        public void CopyFromHoleProps(DObject holeProps)
        {
            if (holeProps == null)
                return;

            this.HoleX = holeProps.GetDouble("hole_x");
            this.HoleY = holeProps.GetDouble("hole_y");
            // 孔类型
            {
                int comboIndex = holeProps.GetInt("hole_type");
                switch (comboIndex) {
                case 1:
                    this.HoleType = Define.BoltHoleType.BOLT_HOLE_SLOTTED;
                    break;
                case 2:
                default:
                    this.HoleType = Define.BoltHoleType.BOLT_HOLE_OVERSIZED;
                    break;
                }
            }
            //
            {
                int comboIndex = holeProps.GetInt("rot_slot");
                switch (comboIndex) {
                case 0:
                    this.RotSlot = Define.BoltRotateSlot.BOLT_SLOT_ODD;
                    break;
                case 1:
                    this.RotSlot = Define.BoltRotateSlot.BOLT_SLOT_EVEN;
                    break;
                case 2:
                default:
                    this.RotSlot = Define.BoltRotateSlot.BOLT_SLOT_PARALLEL;
                    break;
                }
            }
            //
            {
                int comboIndex = holeProps.GetInt("slot_in");
                switch (comboIndex) {
                case 0:
                    this.SlotIn = SoltInMode.NONE;
                    break;
                case 1:
                    this.SlotIn = SoltInMode.BEAM;
                    break;
                case 2:
                    this.SlotIn = SoltInMode.PLATE;
                    break;
                case 3:
                default:
                    this.SlotIn = SoltInMode.BOTH;
                    break;
                }
            }
        }

        public void CopyFromContentProps(DObject contentProps)
        {
            if (contentProps == null)
                return;
            this.PartWasherUp = contentProps.GetBool("part_washer_up");
            this.PartWasherDown0 = contentProps.GetBool("part_washer_down0");
            this.PartWasherDown1 = contentProps.GetBool("part_washer_down1");
            this.PartNut0 = contentProps.GetBool("part_nut0");
            this.PartNut1 = contentProps.GetBool("part_nut1");
        }

        public void CopyFromExtendProps(DObject extendProps)
        {
            if (extendProps == null)
                return;
            this.ExtLength = extendProps.GetDouble("ext_length");
        }

        private Define.PartMadeType ParseMadeType(int comboIndex)
        {
            switch (comboIndex) {
            case 2:
                return Define.PartMadeType.PART_MADE_WORKSHOP;
            case 1:
            default:
                return Define.PartMadeType.PART_MADE_SITE;
            }
        }

    }

}
