﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ColumnWithStiffeners186
{

    /// <summary>
    /// 组件上下文类，该类对象用来存储组件公用变量、属性
    /// </summary>
    public class ComponentInstanceContext
    {

#if false
        public const string RUNTIME_CFG_NAME = "$RUNTIME$";

        public const string DEFAULT_CFG_NAME = "$DEF$";
#endif

        private LoongCAD.STC.Object3D.ConnectionComponent m_comp;

#if false
        private LoongCAD.Util.DObject m_compProps = null;

        private LoongCAD.Util.DObject m_defaultValues = null;
#endif

        private LoongCAD.STC.Object3D.Beam m_mainPart = null;

        private LoongCAD.STC.Object3D.Beam m_subPart = null;

        private LoongCAD.Geometry.Vec3 m_mainDirection = LoongCAD.Geometry.Vec3.ZERO;

        private LoongCAD.Geometry.Vec3 m_subDirection = LoongCAD.Geometry.Vec3.ZERO;

        private LoongCAD.Geometry.Vec3 m_mainUpDirection = LoongCAD.Geometry.Vec3.AXIS_Z;

        private BeamPlanes m_mainPlanes;

        private BeamPlanes m_subPlanes;

        private LoongCAD.Geometry.Plane m_mainNearBound = null;

        private LoongCAD.Geometry.Plane m_mainFarBound = null;

        private LoongCAD.Geometry.Plane m_subNearBound = null;

        private LoongCAD.Geometry.Plane m_mainJointPlane = null;

        private LoongCAD.Geometry.Plane m_mainJointBackside = null;

        private LoongCAD.Geometry.Plane m_subAlignPlane = null;

        private double m_mainJointThickness = 0.0;

        private double m_subWebThickness = 0.0;

        private double m_subTopThickness = 0.0;

        private double m_subBottomThickness = 0.0;

        private double m_mainJointMargin = 0.0;

        private List<PartBase> m_haunchPlates = new List<PartBase>();

        private List<PartBase> m_stiffPlates = new List<PartBase>();

        private Dictionary<string, PartBase> m_stiffMap = new Dictionary<string, PartBase>();

        private List<PartBase> m_shearPlates = new List<PartBase>();

        private List<Plane> m_shearWeldPlanes = new List<Plane>();

        private Plane m_shearFrontPlane = null;

        private Plane m_shearBackPlane = null;

        private List<PartBase> m_doublePlates = new List<PartBase>();

        ComponentInstanceContext()
        {

        }

        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (!compContext.Initialize(comp)) {
                compContext.Clear();
                return null;
            }

            return compContext;
        }

        public ConnectionComponent Comp { get => m_comp; set => m_comp = value; }
#if false
        public DObject CompProps { get => m_compProps; set => m_compProps = value; }
        public LoongCAD.Util.DObject DefaultValues { get => m_defaultValues; set => m_defaultValues = value; }
#endif
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public Beam SubPart { get => m_subPart; set => m_subPart = value; }
        public Vec3 MainDirection { get => m_mainDirection; set => m_mainDirection = value; }
        public Vec3 SubDirection { get => m_subDirection; set => m_subDirection = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public BeamPlanes SubPlanes { get => m_subPlanes; set => m_subPlanes = value; }
        public Plane MainNearBound { get => m_mainNearBound; set => m_mainNearBound = value; }
        public Plane MainFarBound { get => m_mainFarBound; set => m_mainFarBound = value; }
        public Plane SubNearBound { get => m_subNearBound; set => m_subNearBound = value; }
        public Plane MainJointPlane { get => m_mainJointPlane; set => m_mainJointPlane = value; }
        public Plane MainJointBackside { get => m_mainJointBackside; set => m_mainJointBackside = value; }
        public Plane SubAlignPlane { get => m_subAlignPlane; set => m_subAlignPlane = value; }
        public double MainJointThickness { get => m_mainJointThickness; set => m_mainJointThickness = value; }
        public double SubWebThickness { get => m_subWebThickness; set => m_subWebThickness = value; }
        public double SubTopThickness { get => m_subTopThickness; set => m_subTopThickness = value; }
        public double SubBottomThickness { get => m_subBottomThickness; set => m_subBottomThickness = value; }
        public double MainJointMargin { get => m_mainJointMargin; set => m_mainJointMargin = value; }
        public List<PartBase> HaunchPlates { get => m_haunchPlates; }
        public List<PartBase> StiffPlates { get => m_stiffPlates; set => m_stiffPlates = value; }
        public List<PartBase> ShearPlates { get => m_shearPlates; }

        /// <summary>
        /// 连接板用于创建焊缝的参考侧面，此列表与ShearPlates属性的连接板一一对应
        /// </summary>
        public List<Plane> ShearWeldPlanes { get => m_shearWeldPlanes; }

        public Plane ShearFrontPlane { get => m_shearFrontPlane; set => m_shearFrontPlane = value; }
        public Plane ShearBackPlane { get => m_shearBackPlane; set => m_shearBackPlane = value; }
        public Dictionary<string, PartBase> StiffMap { get => m_stiffMap; set => m_stiffMap = value; }
        public List<PartBase> DoublePlates { get => m_doublePlates; set => m_doublePlates = value; }
        public Vec3 MainUpDirection { get => m_mainUpDirection; set => m_mainUpDirection = value; }

        public bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;

            Beam mainPart = comp.Main as Beam;
            Beam subPart = comp.Subs.Length > 0 ? comp.Subs.First<TFObject>() as Beam : null;
            if (mainPart == null || subPart == null)
                return false;

            m_comp = comp;
            m_mainPart = mainPart;
            m_subPart = subPart;
            m_mainPlanes = BeamPlanes.CreateFromBeam(m_mainPart);
            m_subPlanes = BeamPlanes.CreateFromBeam(m_subPart);
            this.ParsePlanes();

            m_mainDirection = m_mainPart.WorldMatrix.Z.Along(Vec3.AXIS_Z);
            m_subDirection = m_subPart.WorldMatrix.Z.Along(m_comp.WorldMatrix.X);
            if (m_mainJointPlane != null && m_mainJointBackside != null) {
                m_mainJointThickness = m_mainJointPlane.Dist(m_mainJointBackside.Origin);
            }
            if (m_subPlanes.FrontWeb != null && m_subPlanes.FrontWebInside != null) {
                m_subWebThickness = m_subPlanes.FrontWeb.Dist(m_subPlanes.BackWeb.Origin);
            }
            if (m_subPlanes.TopFlange != null && m_subPlanes.TopFlangeInside != null) {
                m_subTopThickness = m_subPlanes.TopFlange.Dist(m_subPlanes.TopFlangeInside.Origin);
            }
            if (m_subPlanes.BottomFlange != null && m_subPlanes.BottomFlangeInside != null) {
                m_subBottomThickness = m_subPlanes.BottomFlange.Dist(m_subPlanes.BottomFlangeInside.Origin);
            }
            return true;
        }

#if false
        public static void VerifyDoubleValue(DObject rootProps, double errorValue = double.MaxValue, bool bEnableAssert = true)
        {
            if (rootProps == null)
                return;
            foreach (var curItem in rootProps) {
                if (curItem.Value is double) {
                    if (object.Equals(curItem.Value, errorValue)) {
                        string errorMsg = "无效的属性值:" + curItem.Key.ToString();
                        if (bEnableAssert) {
                            System.Diagnostics.Debug.Assert(false, errorMsg);
                        } else {
                            System.Console.WriteLine("***********ComponentInstanceContext.VerifyDoubleValue:" + errorMsg + "\n");
                        }
                    }
                    continue;
                }
                else if (curItem.Value is DObject) {
                    VerifyDoubleValue(curItem.Value as DObject, errorValue);
                }
            }
        }
#endif

#if false
        public int GetIntegerValue(string propName, int emptyValue = 0)
        {
            int propVal = emptyValue;
            if (this.CompProps.ContainsKey(propName)) {
                propVal = this.CompProps.GetInt(propName);
            }
            return DefaultsHandler.TryGetInteger(this.DefaultValues, propName, propVal, emptyValue);
        }

        public int[] GetIntegerArrayValue(string propName)
        {
            int[] propVal = null;
            if (this.CompProps.ContainsKey(propName)) {
                propVal = this.CompProps.GetIntArr(propName);
            }
            if (propVal == null || propVal.Length == 0 &&
                this.DefaultValues.ContainsKey(propName)) {
                propVal = this.DefaultValues.GetIntArr(propName);
            }
            return propVal;
        }

        public double GetDoubleValue(string propName, double emptyValue = double.MaxValue)
        {
            double propVal = emptyValue;
            if (this.CompProps.ContainsKey(propName)) {
                propVal = this.CompProps.GetDouble(propName);
            }
            return DefaultsHandler.TryGetDouble(this.DefaultValues, propName, propVal, emptyValue);
        }

        public double[] GetDoubleArrayValue(string propName)
        {
            double[] propVal = null;
            if (this.CompProps.ContainsKey(propName)) {
                propVal = this.CompProps.GetDoubleArr(propName);
            }
            if (propVal == null || propVal.Length == 0 &&
                this.DefaultValues.ContainsKey(propName)) {
                propVal = this.DefaultValues.GetDoubleArr(propName);
            }
            return propVal;
        }

        public DObject GetDObjectValue(string propName, DObject emptyValue)
        {
            DObject retValue = this.CompProps.GetDObject(propName, new DObject());
            if (retValue == null) {
                retValue = new DObject();
            }

            // 应用默认值
            DObject subDefaults = this.DefaultValues.GetDObject(propName, null);
            if (subDefaults == null)
                return retValue;
            if (emptyValue != null && emptyValue.Count > 0) {
                foreach (var kv in subDefaults) {
                    if (!retValue.ContainsKey(kv.Key)) {
                        retValue[kv.Key] = kv.Value;
                        continue;
                    }
                    if (!emptyValue.ContainsKey(kv.Key)) {
                        continue;
                    }
                    if (object.Equals(retValue.Get(kv.Key), emptyValue.Get(kv.Key))) {
                        retValue[kv.Key] = kv.Value;
                    }
                }
            } else {
                foreach (var kv in subDefaults) {
                    if (!retValue.ContainsKey(kv.Key)) {
                        retValue[kv.Key] = kv.Value;
                        continue;
                    }
                }
            }

            return retValue;
        }
#endif

        public void ParseAwaySidePlane(
            Plane posPlane, Plane negPlane, out Plane nsPlane, out Plane fsPlane)
        {
            //
            nsPlane = negPlane;
            fsPlane = posPlane;
            Vec3 posNormal = posPlane.Normal;
            Vec3 negNormal = negPlane.Normal;

            //
            Vec3 xCompDir = this.Comp.WorldMatrix.X.Normalize();
            Vec3 yCompDir = this.Comp.WorldMatrix.Y.Normalize();
            Vec3 zCompDir = this.Comp.WorldMatrix.Z.Normalize();
            // 基于组件X方向确定远近腹板面
            double maxDot = xCompDir.Dot(posNormal);
            {
                if (maxDot > 0) {
                    nsPlane = negPlane;
                    fsPlane = posPlane;
                } else {
                    nsPlane = posPlane;
                    fsPlane = negPlane;
                }
                maxDot = Math.Abs(maxDot);
            }
            // 基于组件Y方向确定远近腹板面
            do {
                double curDot = yCompDir.Dot(posNormal);
                if (Math.Abs(curDot) < maxDot) {
                    break;
                }
                maxDot = Math.Abs(maxDot);
                if (curDot > 0) {
                    nsPlane = negPlane;
                    fsPlane = posPlane;
                } else {
                    nsPlane = posPlane;
                    fsPlane = negPlane;
                }
            } while (false);
            // 基于组件Z方向确定远近腹板面
            do {
                double curDot = zCompDir.Dot(posNormal);
                if (Math.Abs(curDot) < maxDot) {
                    break;
                }
                maxDot = Math.Abs(maxDot);
                if (curDot > 0) {
                    nsPlane = negPlane;
                    fsPlane = posPlane;
                } else {
                    nsPlane = posPlane;
                    fsPlane = negPlane;
                }
            } while (false);
        }

        private void ParsePlanes()
        {
            // 组件X轴正方向永远朝向次零件
            Vec3 xCompDir = m_comp.WorldMatrix.X.Normalize();

            // 解析主零件长度方向的上方向
            if (!m_mainPart.WorldMatrix.Z.IsParallelTo(Vec3.AXIS_Z)) {
                m_mainUpDirection = m_mainPart.WorldMatrix.Z.Along(Vec3.AXIS_Z);
            } else {
                m_mainUpDirection = m_mainPart.WorldMatrix.Z;
            }

            // 解析主零件相对次零件的远近侧边界面
            {
                Vec3 frontDir = this.MainPlanes.FrontBound.Normal.Normalize();
                Vec3 backDir = this.MainPlanes.BackBound.Normal.Normalize();
                Vec3 topDir = this.MainPlanes.TopBound.Normal.Normalize();
                Vec3 bottomDir = this.MainPlanes.BottomBound.Normal.Normalize();
                double maxDot = 0.0;
                // 前面
                {
                    double curDot = frontDir.Dot(xCompDir);
                    if (curDot > maxDot)
                    {
                        m_mainNearBound = this.MainPlanes.FrontBound.Clone();
                        m_mainFarBound = this.MainPlanes.BackBound.Clone();
                        m_mainJointPlane = this.MainPlanes.FrontWeb;
                        m_mainJointBackside = this.MainPlanes.FrontWebInside;
                        maxDot = curDot;
                    }
                }
                //  后面
                {
                    double curDot = backDir.Dot(xCompDir);
                    if (curDot > maxDot)
                    {
                        m_mainNearBound = this.MainPlanes.BackBound.Clone();
                        m_mainFarBound = this.MainPlanes.FrontBound.Clone();
                        m_mainJointPlane = this.MainPlanes.BackWeb;
                        m_mainJointBackside = this.MainPlanes.BackWebInside;
                        maxDot = curDot;
                    }
                }
                // 顶面
                {
                    double curDot = topDir.Dot(xCompDir);
                    if (curDot > maxDot)
                    {
                        m_mainNearBound = this.MainPlanes.TopBound.Clone();
                        m_mainFarBound = this.MainPlanes.BottomBound.Clone();
                        m_mainJointPlane = this.MainPlanes.TopFlange;
                        m_mainJointBackside = this.MainPlanes.TopFlangeInside;
                        maxDot = curDot;
                    }
                }
                // 底面
                {
                    double curDot = bottomDir.Dot(xCompDir);
                    if (curDot > maxDot)
                    {
                        m_mainNearBound = this.MainPlanes.BottomBound.Clone();
                        m_mainFarBound = this.MainPlanes.TopBound.Clone();
                        m_mainJointPlane = this.MainPlanes.BottomFlange;
                        m_mainJointBackside = this.MainPlanes.BottomFlangeInside;
                        maxDot = curDot;
                    }
                }

                //
                if (m_mainJointPlane != null) {
                    m_mainJointPlane = m_mainJointPlane.Clone();
                }
            }

            // 解析次零件相对主零件的近侧边界面
            {
                if (m_subPlanes.HeadBound.Normal.Dot(xCompDir) < m_subPlanes.TailBound.Normal.Dot(xCompDir)) {
                    m_subNearBound = m_subPlanes.HeadBound;
                } else {
                    m_subNearBound = m_subPlanes.TailBound;
                }
            }
        }

        public void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_subPart = null;
            m_mainPlanes = null;
            m_subPlanes = null;

            m_mainNearBound = null;
            m_mainFarBound = null;
        }

    }

}
