﻿using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ColumnWithStiffeners186
{
    public class PluginMain : ComponentPlugin
    {

        private readonly Guid m_nameGuid = new Guid(System.Reflection.Assembly.GetExecutingAssembly().GetName().Name);

        private Form_PluginMain m_propsForm = null;

        public PluginMain()
        {
             
        }

        public override Guid Guid => m_nameGuid;

        /// <summary>
        /// 执行组件过程，必需实现此方法
        /// </summary>
        /// <param name="compInst"></param>
        /// <param name="defaultId"></param>
        /// <returns></returns>
        public override bool OnInstanceRun(ComponentBase compInst, long defaultId)
        {
            if (compInst == null || !(compInst is ConnectionComponent))
                return false;

            // 1. 获取组件实例的属性
            DObject compProps = compInst.UIProps;
            if (compProps == null) {
                compProps = new DObject();
            }

            // 2. 初始化参数处理器，此过程包含以下操作:
            // * 初始化参数值
            // * 初始化参数空值表
            // * 初始化参数内置默认值表
            // * 组合参数默认值
            // 其他组件可直接复制此段代码
            ParamsHandler paramsHandler = new ParamsHandler();
            paramsHandler.ParamValues = compProps;
            paramsHandler.InitializeDefaultValues(this.GetDefaultsData(defaultId));

            // 3. 初始化组件上下文，此过程执行以下操作
            var compContext = ComponentInstanceContext.CreateInstance(compInst as ConnectionComponent);
            if (compContext == null)
                return false;

            // 4. 执行组件的每一个功能
            {
                EndHandler endHandler = new EndHandler(compContext, paramsHandler);
                endHandler.Execute();

                NotchHandler notchHandler = new NotchHandler(compContext, paramsHandler);
                notchHandler.Execute();
                
                DoubleHandler doubleHandler = new DoubleHandler(compContext, paramsHandler);
                doubleHandler.Execute();
                
                ShearHandler shearHandler = new ShearHandler(compContext, paramsHandler);
                shearHandler.Execute();
                
                StiffHandler stiffHandler = new StiffHandler(compContext, paramsHandler);
                stiffHandler.Execute();
                
                WeldHandler weldHandler = new WeldHandler(compContext, paramsHandler);
                weldHandler.Execute();
            }
            return true;
        }

        /// <summary>
        /// 显示组件的属性窗，必须实现此方法
        /// </summary>
        /// <param name="ownerForm">组件窗口所属主窗口</param>
        /// <returns>是否显示成功</returns>
        /// <remarks><b>其他组件可直接复制此方法</b></remarks>
        public override bool OnShowPluginPropsForm(Form ownerForm)
        {
            if (m_propsForm == null) {
                m_propsForm = new Form_PluginMain(this);
                m_propsForm.Owner = ownerForm;
            }

            m_propsForm.ShowPluginProps();
            return true;
        }

        /// <summary>
        /// 显示组件实例的属性窗，必须实现此方法
        /// </summary>
        /// <param name="instObj">组件实例对象</param>
        /// <param name="defId">组件默认值ID</param>
        /// <param name="ownerForm">组件窗口所属主窗口</param>
        /// <returns>是否显示成功</returns>
        /// <remarks><b>其他组件可直接复制此方法</b></remarks>
        public override bool OnShowInstancePropsForm(ComponentBase instObj, long defId, Form ownerForm)
        {
            if (instObj == null)
                return false;
            if (m_propsForm == null) {
                m_propsForm = new Form_PluginMain(this);
                m_propsForm.Owner = ownerForm;
            }

            m_propsForm.ShowInstanceProps(instObj,
                new ParamsHandler().InitializeDefaultValues(this.GetDefaultsData(defId)));
            return true;
        }

    }

}
