﻿using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ColumnWithStiffeners186
{

    /// <summary>
    /// 节点组件通用主窗口，其他组件可参考此类实现其主窗口
    /// </summary>
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {

        private DObject m_weldProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;

        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostInitialize();
        }

        public override ComponentPlugin ReferencePlugin => m_refPlugin;

        private void OnPostInitialize()
        {
            // 设置窗口标题
            this.Text = "有加劲肋的柱(186)";

            // 初始化螺栓纵向偏移模式控件
            {
                this.cbo_bolt_vert_offset_mode.ClearItems();
                this.cbo_bolt_vert_offset_mode.AddItem(0, "默认");
                this.cbo_bolt_vert_offset_mode.AddItem(1, "顶部");
                this.cbo_bolt_vert_offset_mode.AddItem(2, "中间");
                this.cbo_bolt_vert_offset_mode.AddItem(3, "底部");
                this.cbo_bolt_vert_offset_mode.UpdateItems();
                this.cbo_bolt_vert_offset_mode.SelectedKey = 0;
            }

            // 螺栓标准变化之后调用螺栓内容的自适应方法
            this.panel_bolt_basic_property.BoltStandardChanged += this.panel_bolt_content.AdaptBoltContents;

            foreach (TabPage curPage in this.tab_main.TabPages) {
                curPage.AutoScroll = true;
            }
        }

        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null) {
                m_weldProps.Clear();
#if false
                m_weldProps.CopyFrom(paras);
#else
                if (paras != null) {
                    List<string> weldNames = Form_WeldParameter.ParamNames;
                    foreach (var curParamName in weldNames) {
                        if (paras.ContainsKey(curParamName)) {
                            m_weldProps[curParamName] = paras[curParamName];
                        }
                    }
                }
#endif
            }
        }

        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked) {
                retProps.CopyFrom(m_weldProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }

        /// <summary>
        /// 当前组件需要重写此方法，并刷新螺栓基础面板
        /// </summary>
        /// <param name="root"></param>
        public override void ClearUI(Control root = null)
        {
            // 刷新螺栓基础面板
            this.panel_bolt_basic_property.RefreshControls();
            base.ClearUI(root);
        }

        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }

        private void btn_weld_props_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null) {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_weldProps, null);
        }

        protected override void OnClickCheckBox(object sender, EventArgs e)
        {
            System.Windows.Forms.CheckBox[] userChecks = new System.Windows.Forms.CheckBox[] { this.act_weld_props };
            this.SwitchControlChecks(null, userChecks);
        }

        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_weldProps.CopyFrom(srcProps);
        }

        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }

        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_weldProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_weldProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_weldProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_weldProps;

            return comp.UIProps;
        }

        private void chamferComboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (this.cbo_plate_chamfer_type.SelectedIndex) {
            case 0: // Default
            case 2: { // Line
                this.act_plate_chamfer_x.Enabled = true;
                this.tb_plate_chamfer_x.Enabled = true;

                    this.act_plate_chamfer_y.Enabled = true;
                    this.tb_plate_chamfer_y.Enabled = true;
                } break;
            case 3: // Round
            case 4: { // InvRound
                    this.act_plate_chamfer_x.Enabled = true;
                    this.tb_plate_chamfer_x.Enabled = true;

                    this.act_plate_chamfer_y.Enabled = false;
                    this.tb_plate_chamfer_y.Enabled = false;
                } break;
            case 1:
            default: { // None
                    this.act_plate_chamfer_x.Enabled = false;
                    this.tb_plate_chamfer_x.Enabled = false;

                    this.act_plate_chamfer_y.Enabled = false;
                    this.tb_plate_chamfer_y.Enabled = false;
                }
                break;
            }
        }

    }
}
