﻿using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ColumnWithStiffeners186
{
    
    public class WeldHandler
    {

        private ComponentInstanceContext m_context;

        private ParamsHandler m_paramsHandler = null;

        private List<WeldProperties> m_weldProps = new List<WeldProperties>();

        public WeldHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;

            for (int i=0; i<ParamNames.WELD_PROPS_LIST.Length; ++i) {
                m_weldProps.Add(new WeldProperties());
            }
        }

        public void Execute()
        {
            if (!this.ParseProps())
                return;

            this.CreateWelds();
        }

        private bool ParseProps()
        {
            for (int i=0; i<ParamNames.WELD_PROPS_LIST.Length; ++i) {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }
            return true;
        }

        private void CreateWelds()
        {
            //
            Plane nsMainWeb, fsMainWeb;
            m_context.ParseAwaySidePlane(
                    m_context.MainPlanes.FrontWeb, m_context.MainPlanes.BackWeb,
                    out nsMainWeb, out fsMainWeb);
            int weldIndex = 0;

            // 加劲肋与主零件之间的焊接
            {
                // Weld1, top_ns -> main.web
                do {
                    if (m_weldProps[0].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                        break;
                    string plateKey = StiffPlanes.TOP_NS_KEY;
                    if (!m_context.StiffMap.ContainsKey(plateKey)) {
                        continue;
                    }
                    PartBase curPlate = m_context.StiffMap[plateKey];
                    Vec3[] weldPoints = PlateHelper.FindEdgePoints(curPlate, nsMainWeb);
                    if (weldPoints == null)
                        break;
                    Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                        ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                        m_context.MainPart, curPlate, weldPoints.ToArray());
                    m_weldProps[0].ApplyToWeld(curWeld);
                } while (false);

                // Weld2, bottom_ns -> main.web
                do {
                    if (m_weldProps[1].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                        break;
                    string plateKey = StiffPlanes.BOTTOM_NS_KEY;
                    if (!m_context.StiffMap.ContainsKey(plateKey)) {
                        continue;
                    }
                    PartBase curPlate = m_context.StiffMap[plateKey];
                    Vec3[] weldPoints = PlateHelper.FindEdgePoints(curPlate, nsMainWeb);
                    if (weldPoints == null)
                        break;
                    Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                        ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                        m_context.MainPart, curPlate, weldPoints.ToArray());
                    m_weldProps[1].ApplyToWeld(curWeld);
                } while (false);

                // Weld3, fs_top -> main.web
                do {
                    if (m_weldProps[2].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                        break;
                    string plateKey = StiffPlanes.TOP_FS_KEY;
                    if (!m_context.StiffMap.ContainsKey(plateKey)) {
                        continue;
                    }
                    PartBase curPlate = m_context.StiffMap[plateKey];
                    Vec3[] weldPoints = PlateHelper.FindEdgePoints(curPlate, fsMainWeb);
                    if (weldPoints == null)
                        break;
                    Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                        ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                        m_context.MainPart, curPlate, weldPoints.ToArray());
                    m_weldProps[2].ApplyToWeld(curWeld);
                } while (false);

                // Weld4, fs_bottom -> main.web
                do {
                    if (m_weldProps[3].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                        break;
                    string plateKey = StiffPlanes.BOTTOM_FS_KEY;
                    if (!m_context.StiffMap.ContainsKey(plateKey)) {
                        continue;
                    }
                    PartBase curPlate = m_context.StiffMap[plateKey];
                    Vec3[] weldPoints = PlateHelper.FindEdgePoints(curPlate, fsMainWeb);
                    if (weldPoints == null)
                        break;
                    Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                        ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                        m_context.MainPart, curPlate, weldPoints.ToArray());
                    m_weldProps[3].ApplyToWeld(curWeld);
                } while (false);

                // weld 7, fs_top -> main.flange
                do {
                    if (m_weldProps[6].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                        break;
                    string plateKey = StiffPlanes.TOP_FS_KEY;
                    if (!m_context.StiffMap.ContainsKey(plateKey)) {
                        continue;
                    }
                    PartBase curPlate = m_context.StiffMap[plateKey];
                    // 上翼板处焊接
                    {
                        Vec3[] weldPoints = PlateHelper.FindEdgePoints(
                            curPlate, m_context.MainPlanes.TopFlangeInside);
                        if (weldPoints == null)
                            break;
                        Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                            ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                            m_context.MainPart, curPlate, weldPoints.ToArray());
                        m_weldProps[6].ApplyToWeld(curWeld);
                    }
                    // 下翼板处焊接
                    {
                        Vec3[] weldPoints = PlateHelper.FindEdgePoints(
                            curPlate, m_context.MainPlanes.BottomFlangeInside);
                        if (weldPoints == null)
                            break;
                        Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                            ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                            m_context.MainPart, curPlate, weldPoints.ToArray());
                        m_weldProps[6].ApplyToWeld(curWeld);
                    }
                } while (false);

                // weld 8, ns_bottom -> main.flange
                do {
                    if (m_weldProps[7].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                        break;
                    string plateKey = StiffPlanes.BOTTOM_NS_KEY;
                    if (!m_context.StiffMap.ContainsKey(plateKey)) {
                        continue;
                    }
                    PartBase curPlate = m_context.StiffMap[plateKey];
                    // 上翼板处焊接
                    {
                        Vec3[] weldPoints = PlateHelper.FindEdgePoints(
                            curPlate, m_context.MainPlanes.TopFlangeInside);
                        if (weldPoints == null)
                            break;
                        Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                            ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                            m_context.MainPart, curPlate, weldPoints.ToArray());
                        m_weldProps[7].ApplyToWeld(curWeld);
                    }
                    // 下翼板处焊接
                    {
                        Vec3[] weldPoints = PlateHelper.FindEdgePoints(
                            curPlate, m_context.MainPlanes.BottomFlangeInside);
                        if (weldPoints == null)
                            break;
                        Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                            ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                            m_context.MainPart, curPlate, weldPoints.ToArray());
                        m_weldProps[7].ApplyToWeld(curWeld);
                    }
                } while (false);

                // weld 9, fs_bottom -> main.flange
                do {
                    if (m_weldProps[8].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                        break;
                    string plateKey = StiffPlanes.BOTTOM_FS_KEY;
                    if (!m_context.StiffMap.ContainsKey(plateKey)) {
                        continue;
                    }
                    PartBase curPlate = m_context.StiffMap[plateKey];
                    // 上翼板处焊接
                    {
                        Vec3[] weldPoints = PlateHelper.FindEdgePoints(
                            curPlate, m_context.MainPlanes.TopFlangeInside);
                        if (weldPoints == null)
                            break;
                        Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                            ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                            m_context.MainPart, curPlate, weldPoints.ToArray());
                        m_weldProps[8].ApplyToWeld(curWeld);
                    }
                    // 下翼板处焊接
                    {
                        Vec3[] weldPoints = PlateHelper.FindEdgePoints(
                            curPlate, m_context.MainPlanes.BottomFlangeInside);
                        if (weldPoints == null)
                            break;
                        Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                            ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                            m_context.MainPart, curPlate, weldPoints.ToArray());
                        m_weldProps[8].ApplyToWeld(curWeld);
                    }
                } while (false);

                // weld 10, ns_top -> main.flange
                do {
                    if (m_weldProps[9].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                        break;
                    string plateKey = StiffPlanes.TOP_NS_KEY;
                    if (!m_context.StiffMap.ContainsKey(plateKey)) {
                        continue;
                    }
                    PartBase curPlate = m_context.StiffMap[plateKey];
                    // 上翼板处焊接
                    {
                        Vec3[] weldPoints = PlateHelper.FindEdgePoints(
                            curPlate, m_context.MainPlanes.TopFlangeInside);
                        if (weldPoints == null)
                            break;
                        Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                            ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                            m_context.MainPart, curPlate, weldPoints.ToArray());
                        m_weldProps[9].ApplyToWeld(curWeld);
                    }
                    // 下翼板处焊接
                    {
                        Vec3[] weldPoints = PlateHelper.FindEdgePoints(
                            curPlate, m_context.MainPlanes.BottomFlangeInside);
                        if (weldPoints == null)
                            break;
                        Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                            ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                            m_context.MainPart, curPlate, weldPoints.ToArray());
                        m_weldProps[9].ApplyToWeld(curWeld);
                    }
                } while (false);
            }

            // 双板与主零件之间的焊接, weld 4
            do {
                if (m_weldProps[3].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
                if (m_context.DoublePlates.Count == 0)
                    break;

                foreach (var curPlate in m_context.DoublePlates) {
                    if (curPlate == null)
                        continue;

                    var partCtrls = curPlate.CtrlPoints;
                    if (partCtrls == null || partCtrls.Length < 2)
                        continue;
                    List<Vec3> weldPoints = new List<Vec3>();
                    for (int i=0; i<partCtrls.Length; ++i) {
                        weldPoints.Add(partCtrls[i].WorldPos);
                    }
                    Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                        ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                        m_context.MainPart, curPlate, weldPoints.ToArray());
                    m_weldProps[3].ApplyToWeld(curWeld);
                }
            } while (false);

            // 连接板与主零件之间的焊接，weld 5
            do {
                if (m_weldProps[4].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
#if false
                foreach (var curPlate in m_context.ShearPlates) {
                    Vec3[] weldPoints = PlateHelper.FindEdgePoints(curPlate, m_context.MainJointPlane.Move(m_context.MainJointMargin));
                    if (weldPoints == null)
                        break;
                    Weld curWeld = m_context.Comp.CreatePolyWeld(m_context.MainPart, curPlate, weldPoints.ToArray(), true);
                    m_weldProps[4].ApplyToWeld(curWeld);
                }
#else
                for (int i = 0; i < m_context.ShearPlates.Count; ++i) {
                    var curPlate = m_context.ShearPlates[i];
                    Vec3[] weldPoints = PlateHelper.FindEdgePoints(curPlate, m_context.ShearWeldPlanes[i]);
                    if (weldPoints == null)
                        break;
                    Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                        ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                        m_context.MainPart, curPlate, weldPoints.ToArray());
                    m_weldProps[4].ApplyToWeld(curWeld);
                }
#endif
            } while (false);

            // 腋板与次零件之间的焊接, weld 6
            do {
                if (m_weldProps[5].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
                if (m_context.HaunchPlates.Count == 0)
                    break;
                foreach (var curPlate in m_context.HaunchPlates) {
                    if (curPlate == null)
                        continue;

                    //
                    Vec3 partCenter = Vec3.ZERO;
                    var partCtrls = curPlate.CtrlPoints;
                    if (partCtrls == null || partCtrls.Length < 0)
                        continue;
                    for (int i=0; i<partCtrls.Length; ++i) {
                        partCenter += partCtrls[i].WorldPos;
                    }
                    partCenter /= (double)partCtrls.Length;

                    //
                    Plane nearPlane = m_context.SubPlanes.TopFlange;
                    if (nearPlane.SignedDist(partCenter) < 0.0) {
                        nearPlane = m_context.SubPlanes.BottomFlange;
                    }
                    Vec3[] weldPoints = PlateHelper.FindEdgePoints(curPlate, nearPlane);
                    if (weldPoints == null)
                        continue;
                    Weld curWeld = m_context.Comp.GetOrCreatePolyWeld(
                        ObjectUsage.WELD_PREFIX + (weldIndex++).ToString(),
                        m_context.SubPart, curPlate, weldPoints.ToArray());
                    m_weldProps[5].ApplyToWeld(curWeld);
                }
            } while (false);
        }

    }

}
