﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace ConnectingPlates14
{
    class EarPlateHandler
    {
        private ComponentInstanceContext m_context = null;
        private ParamsHandler m_paramsHandler = null;
        //
        Plane zCtrlPlane;
        Plane topPlane;
        Plane bottomPlane;
        PlateProperties plateProperties = new PlateProperties();
        List<double> horiStrideList, vertStrideList;
        Plane boltCreateTopPlane; //螺栓创建的起点所在平面
        double boltCreateMarginSide; //螺栓创建的起点距侧面的距离
        double boltSize;

        public EarPlateHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            /*
             * 位置参数、板属性
             */
            double marignTop = m_paramsHandler.GetDoubleValue(ParamNames.PIC_A_EAR_MARGIN); //耳板上端距主端板上边距 
            double marginSide = m_paramsHandler.GetDoubleValue(ParamNames.PIC_A_EAR_MARGIN_SIDE); //耳板创建面距主端板距离
            double mainEndPlateThickness = (double)m_paramsHandler.GetDObjectValue(ParamNames.MAIN_PLATE_PROPS).Get("thickness");
            plateProperties.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.EAR_PLATE_PROPS));
            Util.SetDefaultValue(ref marignTop, 38.0);
            if (plateProperties.Thickness <= 0 || plateProperties.Width <= 0 || plateProperties.Height <= 0)
                return;

            /*
             * 创建面
             */
            zCtrlPlane = m_context.MainCtrlPlane.Move(mainEndPlateThickness - marginSide).Invert();
            topPlane = m_context.MainSidePlanes[1].Move(-marignTop);
            bottomPlane = topPlane.Invert().Move(plateProperties.Height);
            //前侧耳板
            Plane frontPlane1 = m_context.MainSidePlanes[0].Move(plateProperties.Width);
            Plane backPlane1 = m_context.MainSidePlanes[0].Invert();
            Plane[] sidePlanesFront = { topPlane , frontPlane1, bottomPlane, backPlane1};
            //后侧耳板
            Plane frontPlane2 = m_context.MainSidePlanes[2].Move(plateProperties.Width);
            Plane backPlane2 = m_context.MainSidePlanes[2].Invert();
            Plane[] sidePlanesBack = { topPlane, frontPlane2, bottomPlane, backPlane2 };


            /*
             * 创建
             */
            //是否开启耳板
            EarPlaceMode frontBackChecked = (EarPlaceMode)m_paramsHandler.GetIntValue(ParamNames.EAR_PLATE_PLACE);
            //螺栓间距数组
            int horiCount = m_paramsHandler.GetIntValue(ParamNames.EAR_PLATE_BOLT_HORI_COUNT);
            int vertCount = m_paramsHandler.GetIntValue(ParamNames.EAR_PLATE_BOLT_VERT_COUNT);
            double[] horiStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.EAR_PLATE_BOLT_HORI_STRIDE);
            double[] vertStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.EAR_PLATE_BOLT_VERT_STRIDE);
            horiStrideList = new BoltLocation().SetStride(horiCount, horiStride);
            vertStrideList = new BoltLocation().SetStride(vertCount, vertStride);
            //创建点的定位
            double boltMarginTop = m_paramsHandler.GetDoubleValue(ParamNames.EAR_PLATE_BOLT_H_MARGIN);
            double boltMarginSide = m_paramsHandler.GetDoubleValue(ParamNames.EAR_PLATE_BOLT_V_MARGIN);
            boltCreateTopPlane = topPlane.Move(-boltMarginTop);
            boltCreateMarginSide = boltMarginSide + horiStrideList.Sum() / 2;
            //螺栓大小
            boltSize = m_paramsHandler.GetDoubleValue(ParamNames.EAR_PLATE_BOLT_SIZE);
            do
            {
                if (frontBackChecked == EarPlaceMode.RIGHT_OPEN)
                    break;
                SteelPlate sp = m_context.Comp.CreateSteelPlate(zCtrlPlane, plateProperties.Thickness, sidePlanesFront, true);
                m_context.FrontEarPlate = sp;
                if (sp == null || horiCount <= 0 || vertCount <= 0)
                    break;
                CreateBoltHoles(sidePlanesFront, sp);
            } while (false);
            //
            do
            {
                if (frontBackChecked == EarPlaceMode.LEFT_OPEN)
                    break;
                SteelPlate sp = m_context.Comp.CreateSteelPlate(zCtrlPlane, plateProperties.Thickness, sidePlanesBack, true);
                m_context.BackEarPlate = sp;
                if (sp == null || horiCount <= 0 || vertCount <= 0)
                    break;
                CreateBoltHoles(sidePlanesBack, sp);
            } while (false);
        }

        public void CreateBoltHoles(Plane[] sidePlanes, SteelPlate sp)
        {
            Plane frontPlane = sidePlanes[1];
            //
            Vec3 x = frontPlane.Normal; 
            Vec3 y = bottomPlane.Normal; 
            Mat43 workAsix = new Mat43(x, y, x.Cross(y), Vec3.ZERO);
            //
            Vec3 start = boltCreateTopPlane.Intersect(frontPlane.Move(-boltCreateMarginSide), zCtrlPlane);
            Vec3 end = bottomPlane.Intersect(frontPlane.Move(-boltCreateMarginSide), zCtrlPlane);
            //
            BoltGroup boltGroup = m_context.Comp.CreateBoltHoles(null, sp, start, end, true);
            boltGroup.BoltSize = this.boltSize;
            boltGroup.ArrayStrideX = vertStrideList.ToArray();
            boltGroup.ArrayStrideY = horiStrideList.ToArray();
            boltGroup.WorkAxis = workAsix;
            boltGroup.CutLength = 2 * plateProperties.Thickness + 10;
            boltGroup.RotationSide = Define.ObjectSide.SIDE_TOP;
            //解决旋转后位置错误的问题
            boltGroup.Rotation = 0.0;
            boltGroup.HeadOffsetX = 0.0;
            boltGroup.HeadOffsetY = 0.0;
            boltGroup.HeadOffsetZ = 0.0;
            boltGroup.TailOffsetX = 0.0;
            boltGroup.TailOffsetY = 0.0;
            boltGroup.TailOffsetZ = 0.0;
            boltGroup.VerticalOffset = 0.0;
            boltGroup.DepthOffset = 0.0;
            boltGroup.Refresh();
        }
    }
} 
