﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace ConnectingPlates14
{
    class PlateHandler
    {
        private ComponentInstanceContext m_context = null; 
        private ParamsHandler m_paramsHandler = null;
        //板参数
        PlateProperties mainEndPlateProps = new PlateProperties(); //主端板
        PlateProperties subEndPlateProps = new PlateProperties();
        PlateProperties shimPlate1Props = new PlateProperties(); //衬板1(垫板、装配板)属性
        PlateProperties shimPlate2Props = new PlateProperties();
        PlateProperties shimPlate3Props = new PlateProperties();
        int shimPlate1Count; //衬板1数量
        int shimPlate2Count; //衬板2数量
        int shimPlate3Count; //衬板3数量
        double gap;

        public PlateHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            /*
             * 解析板参数
             */
            ParsePlateParameter();

            /*
             * 对齐主梁和次梁
             */
            AlignBeam();

            /*
             * 端板位置解析
             */
            //次梁高度参数
            double subEndPlateMarginTop = m_paramsHandler.GetDoubleValue(ParamNames.PIC_B_TOP_MARGIN); //端板边缘与梁的距离
            double subEndPlateMarginBottom = m_paramsHandler.GetDoubleValue(ParamNames.PIC_B_BOTTOM_MARGIN);
            double subEndPlateHeight = subEndPlateProps.Height;
            //主梁高度参数
            double mainEndPlateMarginTop = m_paramsHandler.GetDoubleValue(ParamNames.PIC_A_TOP_MARGIN);
            double mainEndPlateMarginBottom = m_paramsHandler.GetDoubleValue(ParamNames.PIC_A_BOTTOM_MARGIN);
            double mainEndPlateHeight = mainEndPlateProps.Height;
            //次梁宽度参数
            double subEndPlateWidth = subEndPlateProps.Width;
            //主梁宽度参数
            double mainEndPlateWidth = mainEndPlateProps.Width;
            //次端板和螺栓的垂直定位
            {
                //默认值
                BoltLocation vertLocation = new BoltLocation(); //垂直方向上的螺栓定位
                vertLocation.SetDefaultStride(2, new double[] { 60.0 });
                //螺栓参数
                double boltMarginBottom = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_BOTTOM_MARGIN); //底部螺栓与端板边缘的距离
                double boltMarginTop = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_TOP_MARGIN);
                double[] vertStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_VERT_STRIDE);
                int vertCount = m_paramsHandler.GetIntValue(ParamNames.BOLT_VERT_COUNT);
                double vertOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_VERT_OFFSET); //螺栓的垂直偏移
                //若设置板高度，重新定位
                if (subEndPlateHeight != double.MaxValue)
                {
                    //两个边距没填，上下边距由板高度决定
                    if (subEndPlateMarginTop == double.MaxValue && subEndPlateMarginBottom == double.MaxValue)
                    {
                        subEndPlateMarginBottom = (subEndPlateHeight - m_context.SubHeight) / 2;
                        subEndPlateMarginTop = (subEndPlateHeight - m_context.SubHeight) / 2;
                    }
                    //填了上边距，就由上边距定位（不管下边距是否填写）
                    else if (subEndPlateMarginTop != double.MaxValue)
                    {
                        subEndPlateMarginBottom = subEndPlateHeight - m_context.SubHeight - subEndPlateMarginTop;
                    }
                    //只填了下边距，按下边距定位
                    else
                    {
                        subEndPlateMarginTop = subEndPlateHeight - m_context.SubHeight - subEndPlateMarginBottom;
                    }
                }
                //计算螺栓边距
                vertLocation.SetCenterBoltLoaction(boltMarginBottom, boltMarginTop, vertCount, vertStride, m_context.SubHeight / 2 + vertOffset,
                    m_context.SubHeight, subEndPlateMarginTop, subEndPlateMarginBottom, 10);
                //两个边距没填，由螺栓(垂直方向上居中次梁)确定高度
                if (subEndPlateMarginTop == double.MaxValue && subEndPlateMarginBottom == double.MaxValue)
                {
                    subEndPlateMarginBottom = vertLocation.margin1 + vertLocation.stride.Sum() / 2 - (m_context.SubHeight / 2 + vertOffset);
                    subEndPlateMarginTop = vertLocation.margin2 + vertLocation.stride.Sum() / 2 - (m_context.SubHeight / 2 - vertOffset);
                }
                //填了端板距上翼缘的距离，下边距由螺栓参数确定
                else if (subEndPlateMarginTop != double.MaxValue && subEndPlateMarginBottom == double.MaxValue)
                {
                    subEndPlateMarginBottom = vertLocation.length - m_context.SubHeight - subEndPlateMarginTop;
                }
                //填了端板距下翼缘的距离，上边距由螺栓参数确定
                else if (subEndPlateMarginTop == double.MaxValue && subEndPlateMarginBottom != double.MaxValue)
                {
                    subEndPlateMarginTop = vertLocation.length - m_context.SubHeight - subEndPlateMarginBottom;
                }
                //
                vertLocation.stride.Reverse();
                m_context.EndPlateBoltLocationVert = vertLocation;
            }
            //主端板垂直定位
            {
                //若设置板高度，重新定位
                if (mainEndPlateHeight != double.MaxValue)
                {
                    //两个边距没填，上下边距由板高度决定
                    if (mainEndPlateMarginTop == double.MaxValue && mainEndPlateMarginBottom == double.MaxValue)
                    {
                        mainEndPlateMarginBottom = (mainEndPlateHeight - m_context.MainHeight) / 2;
                        mainEndPlateMarginTop = (mainEndPlateHeight - m_context.MainHeight) / 2;
                    }
                    //填了上边距，就由上边距定位（不管下边距是否填写）
                    else if (mainEndPlateMarginTop != double.MaxValue)
                    {
                        mainEndPlateMarginBottom = mainEndPlateHeight - m_context.MainHeight - mainEndPlateMarginTop;
                    }
                    //只填了下边距，按下边距定位
                    else
                    {
                        mainEndPlateMarginTop = mainEndPlateHeight - m_context.MainHeight - mainEndPlateMarginBottom;
                    }
                }
                //主梁端板高度跟随次梁端板高度
                Util.SetDefaultValue(ref mainEndPlateMarginTop, subEndPlateMarginTop);
                Util.SetDefaultValue(ref mainEndPlateMarginBottom, subEndPlateMarginBottom);
            }
            //次端板和螺栓的水平定位（螺栓居中放置）
            BoltLocation horiLocation; //水平方向上的螺栓定位
            {
                //默认值
                double defaultBoltMargin = 40.0;
                double[] defaultBoltStride = new double[] { 80.0 };
                int defaultBoltCount = 2;
                //螺栓参数
                double boltMarginLeft = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_LEFT_MARGIN);
                double boltMarginRight = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_RIGHT_MARGIN);
                double[] horiStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_HORI_STRIDE);
                int horiCount = m_paramsHandler.GetIntValue(ParamNames.BOLT_HORI_COUNT);
                //未设置板宽度
                horiLocation = new BoltLocation(defaultBoltMargin, defaultBoltMargin, defaultBoltCount, defaultBoltStride);
                horiLocation.centerMode = true;
                horiLocation.SetBoltLocation(boltMarginRight, boltMarginLeft, horiCount, horiStride, subEndPlateWidth);
                m_context.EndPlateBoltLocationHori = horiLocation;
            }


            /*
             * 端板创建
             */
            double horiOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_HORI_OFFSET); //板的水平偏移（由螺栓中心偏移引起）
            Vec3 subTopCenterPoint = m_context.SubTopBound.Intersect(m_context.SubNearCenterPlane, m_context.SubWebCenterPlane);
            Plane vertCenterPlane = new Plane(subTopCenterPoint, m_context.PlateFrontOrient).Move(-horiOffset); //端板定位面（法线朝前）
            //主
            do
            {
                Util.SetDefaultValue(ref mainEndPlateWidth, horiLocation.length);
                Plane auxTopPlane = Util.VerticalPlane(m_context.MainCtrlPlane, m_context.MainNearCenterPlane.Intersect(m_context.MainTopBound), m_context.PlateTopOrient);
                Plane auxBottomPlane = Util.VerticalPlane(m_context.MainCtrlPlane, m_context.MainNearCenterPlane.Intersect(m_context.MainBottomBound), -m_context.PlateTopOrient);
                Plane frontPlane = vertCenterPlane.Move(mainEndPlateWidth / 2);
                Plane backPlane = vertCenterPlane.Move(-mainEndPlateWidth / 2).Invert();
                Plane topPlane = auxTopPlane.Move(mainEndPlateMarginTop);
                Plane bottomPlane = auxBottomPlane.Move(mainEndPlateMarginBottom);
                Plane[] sidePlanes = { frontPlane, topPlane, backPlane, bottomPlane };
                if (!Util.PlatePropIsValid(mainEndPlateProps))
                    break;
                SteelPlate sp = m_context.Comp.CreateSteelPlate(m_context.MainCtrlPlane, mainEndPlateProps.Thickness, sidePlanes, true);
                mainEndPlateProps.ApplyToPart(sp);
                sp.Flush();
                m_context.MainEndPlate = sp;
                m_context.MainSidePlanes = sidePlanes;
                m_context.MainEndPlateThickness = mainEndPlateProps.Thickness;
            } while (false);
            Plane[] subEndPlateSidePlanes; //创建次梁端板的边面
            //次
            {
                Plane auxTopPlane = Util.VerticalPlane(m_context.SubCtrlPlane, m_context.SubNearCenterPlane.Intersect(m_context.SubTopBound), m_context.PlateTopOrient);
                Plane auxBottomPlane = Util.VerticalPlane(m_context.SubCtrlPlane, m_context.SubNearCenterPlane.Intersect(m_context.SubBottomBound), -m_context.PlateTopOrient);
                Plane frontPlane = vertCenterPlane.Move(horiLocation.stride.Sum() / 2 + horiLocation.margin1);
                Plane backPlane = vertCenterPlane.Move(-horiLocation.stride.Sum() / 2 - horiLocation.margin2).Invert();
                Plane topPlane = auxTopPlane.Move(subEndPlateMarginTop);
                Plane bottomPlane = auxBottomPlane.Move(subEndPlateMarginBottom);
                subEndPlateSidePlanes = new Plane[] { frontPlane, topPlane, backPlane, bottomPlane };
                if (subEndPlateProps.Thickness > 0 && subEndPlateProps.Width > 0 && subEndPlateProps.Height > 0)
                {
                    SteelPlate subEndPlate = m_context.Comp.CreateSteelPlate(m_context.SubCtrlPlane, subEndPlateProps.Thickness, subEndPlateSidePlanes, true);
                    subEndPlateProps.ApplyToPart(subEndPlate);
                    subEndPlate.Flush();
                    m_context.SubEndPlate = subEndPlate;
                }
                m_context.SubSidePlanes = subEndPlateSidePlanes;
                m_context.SubEndPlateThickness = subEndPlateProps.Thickness;
            }


            /*
             * 衬板创建
             */
            Plane shimCtrlPlane = m_context.MainCtrlPlane.Move(mainEndPlateProps.Thickness);
            Plane[] shimSidePlaneList = m_context.CopyPlaneList(subEndPlateSidePlanes);
            double shimMarginTopBottom = m_paramsHandler.GetDoubleValue(ParamNames.PAD_BOLT_BOTTOM_MARGIN); //顶底方向的边距
            double shimMarginFrontBack = m_paramsHandler.GetDoubleValue(ParamNames.PAD_BOLT_RIGHT_MARGIN); //前后方向的边距
            if(shimMarginTopBottom != double.MaxValue)
            {
                shimSidePlaneList[1].MoveThis(-m_context.EndPlateBoltLocationVert.margin2 + shimMarginTopBottom);
                shimSidePlaneList[3].MoveThis(-m_context.EndPlateBoltLocationVert.margin1 + shimMarginTopBottom);
            }
            if (shimMarginFrontBack != double.MaxValue)
            {
                shimSidePlaneList[0].MoveThis(-m_context.EndPlateBoltLocationHori.margin1 + shimMarginFrontBack);
                shimSidePlaneList[2].MoveThis(-m_context.EndPlateBoltLocationHori.margin2 + shimMarginFrontBack);
            }
            m_context.ShimSidePlanes = shimSidePlaneList;
            //衬板组1
            List<SteelPlate> shim1List = CreateMulitPlate(ref shimCtrlPlane, shimSidePlaneList, shimPlate1Props.Thickness, shimPlate1Count, shimPlate1Props);
            m_context.ShimPlateList.AddRange(shim1List);
            //衬板组2
            List<SteelPlate> shim2List = CreateMulitPlate(ref shimCtrlPlane, shimSidePlaneList, shimPlate2Props.Thickness, shimPlate2Count, shimPlate2Props);
            m_context.ShimPlateList.AddRange(shim2List);
            //衬板组3
            List<SteelPlate> shim3List = CreateMulitPlate(ref shimCtrlPlane, shimSidePlaneList, shimPlate3Props.Thickness, shimPlate3Count, shimPlate3Props);
            m_context.ShimPlateList.AddRange(shim3List);


            /*
             * 加劲肋创建
             */
            CreateHaunchPlates();
            /*
            double cutY = m_paramsHandler.GetDoubleValue(ParamNames.PIC_STIFF_GAP); //看作折角，在Y方向有切割
            Util.SetDefaultValue(ref cutY, 25.0);
            //主
            do
            {
                PlateProperties stiffPlateProps = new PlateProperties(); //加劲肋属性
                stiffPlateProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.MAIN_STIFF_PLATE_PROPS));
                if (!Util.PlatePropIsValid(stiffPlateProps))
                    break;
                Line centerLine = m_context.MainNearCenterPlane.Intersect(m_context.MainWebCenterPlane); //腹板中心线
                Plane centerPlane = Util.VerticalPlane(m_context.MainNearCenterPlane, centerLine, Vec3.AXIS_X); //加劲肋中心面（过主梁中心）
                SteelPlate mainStiff = CreateStiff(centerPlane, m_context.MainCtrlPlane, m_context.MainBottomBound, m_context.MainSidePlanes[3], cutY, stiffPlateProps);
                m_context.MainStiff = mainStiff;
            } while (false);
            //次
            do
            {
                PlateProperties stiffPlateProps = new PlateProperties(); //加劲肋属性
                stiffPlateProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.SUB_STIFF_PLATE_PROPS));
                if (!Util.PlatePropIsValid(stiffPlateProps))
                    break;
                Line centerLine = m_context.SubNearCenterPlane.Intersect(m_context.SubWebCenterPlane);
                Plane centerPlane = Util.VerticalPlane(m_context.SubNearCenterPlane, centerLine, Vec3.AXIS_X);
                SteelPlate subStiff = CreateStiff(centerPlane, m_context.SubCtrlPlane, m_context.SubBottomBound, m_context.SubSidePlanes[3], cutY, stiffPlateProps);
                m_context.SubStiff = subStiff;
            } while (false);
            */
        }

        //创建多块衬板并返回零件
        public List<SteelPlate> CreateMulitPlate(ref Plane zCtrlPlane, Plane[] sidePlanes, double thickness, double num, PlateProperties properties)
        {
            List<SteelPlate> re = new List<SteelPlate>();
            if (num == 0 || thickness == 0)
                return re;
            for (int i = 1; i <= num; i++)
            {
                SteelPlate steelPlate = m_context.Comp.CreateSteelPlate(zCtrlPlane, thickness, sidePlanes, true);
                properties.ApplyToPart(steelPlate);
                steelPlate.Flush();
                re.Add(steelPlate); //添加到零件组里
                zCtrlPlane.MoveThis(thickness);
            }
            return re;
        }

        public SteelPlate CreateStiff(Plane centerPlane, Plane insideSidePlane, Plane topSidePlane, Plane bottomPlane, double cutY, PlateProperties props)
        {
            //加劲肋有四个侧面（上、右、下、左），下侧面是斜着的，垂直方向上的两个面分内侧和外侧（可能没有），水平方向的顶面是梁的下翼缘面
            Plane ctrlPlane = centerPlane.Move(-props.Thickness / 2);
            Plane outsideSidePlane = insideSidePlane.Invert().Move(props.Width);
            //确定下侧斜面 
            Plane auxBottomPlane = bottomPlane.Clone(); //下侧辅助面（下包围面，法向量朝正下方）
            if (props.Height != double.MaxValue)
            {
                Plane auxTopPlane1 = Util.VerticalPlane(insideSidePlane, insideSidePlane.Intersect(topSidePlane), topSidePlane.Normal); //移动的起始面（内侧）
                auxBottomPlane = auxTopPlane1.Move(props.Height);
            }
            Plane auxBottomPlane1 = auxBottomPlane.Move(-cutY); 
            //边面
            Vec3 topOutsidePoint = topSidePlane.Intersect(outsideSidePlane, centerPlane);
            Plane[] sidePlanes;
            if (auxBottomPlane1.SignedDist(topOutsidePoint) < 0) //加劲肋为梯形
            {
                Line topLine = outsideSidePlane.Intersect(auxBottomPlane1);
                Line bottomLine = insideSidePlane.Intersect(auxBottomPlane);
                Plane bottomSidePlane = Util.CreatePlaneBy2Line(topLine, bottomLine);
                sidePlanes = new Plane[] { topSidePlane, outsideSidePlane, bottomSidePlane, insideSidePlane };
            }
            else //加劲肋为三角形
            {
                Line topLine = outsideSidePlane.Intersect(topSidePlane);
                Line bottomLine = insideSidePlane.Intersect(auxBottomPlane);
                Plane bottomSidePlane = Util.CreatePlaneBy2Line(topLine, bottomLine);
                sidePlanes = new Plane[] { topSidePlane, bottomSidePlane, insideSidePlane };
            }
            //创建
            SteelPlate sp = m_context.Comp.CreateSteelPlate(ctrlPlane, props.Thickness, sidePlanes, true);
            if(sp != null)
            {
                props.ApplyToPart(sp);
                sp.Flush();
            }
            return sp;
        }

        //对齐主梁和次梁
        public void AlignBeam()
        {
            double shimPlate1Tickness = shimPlate1Props.Thickness; //衬板1厚度
            double shimPlate2Tickness = shimPlate2Props.Thickness; //衬板2厚度
            double shimPlate3Tickness = shimPlate3Props.Thickness; //衬板3厚度
            //使用次梁参数计算是否焊接到端板上
            double dist = m_context.SubHeight * Math.Sin(Util.VectorAngle(m_context.PlateTopOrient, m_context.MainTopBound.Normal));
            double minDist = m_paramsHandler.GetDoubleValue(ParamNames.MIN_GAP);
            bool isRotate = dist < minDist;
            bool isTopOffset = m_context.MainNearCenterPlane.Normal.Dot(m_context.PlateTopOrient) <= 0; //是否向上偏移
            //主
            {
                double endPlateThickness = mainEndPlateProps.Thickness; //主梁端板厚度
                Plane alignPlane = m_context.AlignPlane.Move(-endPlateThickness - gap / 2).Invert();
                m_context.MainCtrlPlane = alignPlane.Invert();
                if(isRotate)
                {
                    if (isTopOffset)
                    {
                        Line line = m_context.MainTopBound.Intersect(alignPlane);
                        alignPlane = Util.VerticalPlane(m_context.MainTopBound, line, alignPlane.Normal);
                    }
                    else
                    {
                        Line line = m_context.MainBottomBound.Intersect(alignPlane);
                        alignPlane = Util.VerticalPlane(m_context.MainBottomBound, line, alignPlane.Normal);
                    }
                }
                m_context.MainNearCenterPlane = alignPlane.Invert();
                m_context.Comp.CreateAlignPlane(m_context.MainPart, alignPlane, true);
            }
            //次
            {
                double endPlateThickness = subEndPlateProps.Thickness; //次梁端板厚度
                Plane alignPlane = m_context.AlignPlane.Move(gap / 2 + shimPlate1Count * shimPlate1Tickness + shimPlate2Count * shimPlate2Tickness + shimPlate3Count * shimPlate3Tickness + endPlateThickness);
                m_context.SubCtrlPlane = alignPlane.Invert();
                if (isRotate)
                {
                    if (isTopOffset)
                    {
                        Line line = m_context.SubTopBound.Intersect(alignPlane);
                        alignPlane = Util.VerticalPlane(m_context.SubTopBound, line, alignPlane.Normal);
                    }
                    else
                    {
                        Line line = m_context.SubBottomBound.Intersect(alignPlane);
                        alignPlane = Util.VerticalPlane(m_context.SubBottomBound, line, alignPlane.Normal);
                    }
                }
                m_context.SubNearCenterPlane = alignPlane.Invert();
                m_context.Comp.CreateAlignPlane(m_context.SubPart, alignPlane, true);
                
            }

            /*
             * 板创建面
             */
            Vec3 subTopCenterPoint = m_context.SubTopBound.Intersect(m_context.SubNearCenterPlane, m_context.SubWebCenterPlane);
            m_context.PlateTopPlane = new Plane(subTopCenterPoint, m_context.PlateTopOrient);
            Vec3 subBottomCenterPoint = m_context.SubBottomBound.Intersect(m_context.SubNearCenterPlane, m_context.SubWebCenterPlane);
            m_context.PlateBottomPlane = new Plane(subBottomCenterPoint, -m_context.PlateTopOrient);
        }

        //解析板参数
        public void ParsePlateParameter()
        {
            //端板属性
            mainEndPlateProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.MAIN_PLATE_PROPS));
            subEndPlateProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.SUB_PLATE_PROPS));
            //衬板属性
            shimPlate1Props.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.PAD_1_PLATE_PROPS));
            shimPlate2Props.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.PAD_2_PLATE_PROPS));
            shimPlate3Props.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.PAD_3_PLATE_PROPS));
            //衬板数量
            shimPlate1Count = m_paramsHandler.GetIntValue(ParamNames.PAD_1_PLATE_NUM); //衬板1数量
            shimPlate2Count = m_paramsHandler.GetIntValue(ParamNames.PAD_2_PLATE_NUM); //衬板2数量
            shimPlate3Count = m_paramsHandler.GetIntValue(ParamNames.PAD_3_PLATE_NUM); //衬板3数量
            //梁间距
            gap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_SARKING_GAP);
        }

        //创建腋板——（已改名为上、下加劲板）
        public void CreateHaunchPlates()
        {
            //
            m_context.HaunchDict.Add("main_haunch", new List<HaunchHelper>());
            m_context.HaunchDict.Add("sub_haunch", new List<HaunchHelper>());

            /*
             * 加劲板属性
             */
            PlateProperties topStiffProps = new PlateProperties();
            PlateProperties bottomStiffProps = new PlateProperties();
            PlateProperties webStiffProps = new PlateProperties();
            WeldProperties weldProps = new WeldProperties();
            topStiffProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.TOP_HAUNCH_PLATES_PROPS));
            bottomStiffProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.BOTTOM_HAUNCH_PLATES_PROPS));
            SetPropsFollow(bottomStiffProps, topStiffProps);
            webStiffProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.WEB_STIFF_PROPS));
            weldProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[7]));
            int webStiffCount = m_paramsHandler.GetIntValue(ParamNames.MAIN_WEB_STIFF_COUNT); //次梁腋板属性
            double[] webStiffStride = (double[])m_paramsHandler.GetDoubleArrayValue(ParamNames.MAIN_WEB_STIFF_STRIDE).Clone();
            double webStiffMarginTop = m_paramsHandler.GetDoubleValue(ParamNames.MAIN_WEB_STIFF_MARGIN_TOP);
            double mainHeight = m_context.SubTopBound.Dist(m_context.SubBottomBound.Origin);
            Util.SetDefaultValue(ref webStiffMarginTop, 60); //默认2个腋板，翼缘边距60，stride自适应
            Util.SetDefaultValue(ref webStiffStride[0], (mainHeight - 2 * webStiffMarginTop) / (webStiffCount - 1));
            List<double> stiffStride = BoltParameter.ParseStrideInput(webStiffCount, webStiffStride);

            /*
             * 折角参数
             */
            HaunchChamferProps mainFlangeChamferProps = new HaunchChamferProps(m_paramsHandler, "main", "");
            HaunchChamferProps mainWebChamferProps = new HaunchChamferProps(m_paramsHandler, "main", "web");
            HaunchChamferProps subFlangeChamferProps = new HaunchChamferProps(m_paramsHandler, "sub", ""); 
            HaunchChamferProps subWebChamferProps = new HaunchChamferProps(m_paramsHandler, "sub", "web");
            subFlangeChamferProps.SetFollowValue(mainFlangeChamferProps);
            mainWebChamferProps.SetFollowValue(mainFlangeChamferProps);
            subWebChamferProps.SetFollowValue(mainWebChamferProps);
            double inCutX, inCutY, outCutX, outCutY;
            StiffShape stiffShape;

            /*
             * 主加劲板（靠近柱一侧）
             */
            //翼缘加劲板
            {
                inCutX = mainFlangeChamferProps.insideCutX;
                inCutY = mainFlangeChamferProps.insideCutY;
                outCutX = mainFlangeChamferProps.outsideCutX;
                outCutY = mainFlangeChamferProps.outsideCutY;
                stiffShape = mainFlangeChamferProps.stiffShape;
                //加劲板创建面
                Line centerLine = m_context.MainNearCenterPlane.Intersect(m_context.MainWebCenterPlane); //腹板中心线
                Plane centerPlane = Util.VerticalPlane(m_context.MainNearCenterPlane, centerLine, Vec3.AXIS_X); //加劲肋中心面（过主梁中心）
                //上加劲板
                {
                    HaunchHelper haunch = new HaunchHelper(m_context.MainCtrlPlane.Invert(), m_context.MainTopBound, centerPlane, Util.Copy(topStiffProps));
                    haunch.SetFollowPlane(m_context.MainSidePlanes[1]);
                    haunch.SetOutsideChamfer(outCutX, outCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    haunch.SetInsideChamfer(inCutX, inCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch.isTrianlge = true;
                    haunch.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["main_haunch"].Add(haunch);
                }
                //下加劲板
                {
                    HaunchHelper haunch = new HaunchHelper(m_context.MainCtrlPlane.Invert(), m_context.MainBottomBound, centerPlane, Util.Copy(bottomStiffProps));
                    haunch.SetFollowPlane(m_context.MainSidePlanes[3]);
                    haunch.SetOutsideChamfer(outCutX, outCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    haunch.SetInsideChamfer(inCutX, inCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch.isTrianlge = true;
                    haunch.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["main_haunch"].Add(haunch);
                }
            }
            //腹板加劲板
            do
            {
                inCutX = mainWebChamferProps.insideCutX;
                inCutY = mainWebChamferProps.insideCutY;
                outCutX = mainWebChamferProps.outsideCutX;
                outCutY = mainWebChamferProps.outsideCutY;
                stiffShape = mainWebChamferProps.stiffShape;
                Plane webStiffCtrlPlane = m_context.MainTopBound.Invert().Move(webStiffMarginTop);
                if (!Util.PlatePropIsValid(webStiffProps) || webStiffCount <= 0)
                    break;
                for (int i = 0; ; i++)
                {
                    //前侧
                    HaunchHelper haunch1 = new HaunchHelper(m_context.MainCtrlPlane.Invert(), m_context.MainFrontWeb, webStiffCtrlPlane, Util.Copy(webStiffProps));
                    //Plane bottomPlane = m_context.IsMainRotate ? endPlateSidePlanes[1] : endPlateSidePlanes[0];
                    haunch1.SetFollowPlane(m_context.MainSidePlanes[0]);
                    haunch1.SetOutsideChamfer(outCutX, outCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    haunch1.SetInsideChamfer(inCutX, inCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch1.isTrianlge = true;
                    haunch1.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["main_haunch"].Add(haunch1);
                    //后侧
                    HaunchHelper haunch2 = new HaunchHelper(m_context.MainCtrlPlane.Invert(), m_context.MainBackWeb, webStiffCtrlPlane, webStiffProps);
                    //bottomPlane = m_context.IsMainRotate ? endPlateSidePlanes[3] : endPlateSidePlanes[2];
                    haunch2.SetFollowPlane(m_context.MainSidePlanes[2]);
                    haunch2.SetOutsideChamfer(outCutX, outCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    haunch2.SetInsideChamfer(inCutX, inCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch2.isTrianlge = true;
                    haunch2.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["main_haunch"].Add(haunch2);
                    //
                    if (i == stiffStride.Count)
                        break;
                    webStiffCtrlPlane.MoveThis(stiffStride[i]);
                }
            } while (false);
#if false
            do
            {
                int mainWebStiffCount = m_paramsHandler.GetIntValue(ParamNames.MAIN_WEB_STIFF_COUNT);
                Util.SetDefaultValue(ref mainWebStiffCount, webStiffCount); //跟随右侧数目
                double[] mainWebStiffStride = (double[])m_paramsHandler.GetDoubleArrayValue(ParamNames.MAIN_WEB_STIFF_STRIDE).Clone();
                double mainWebStiffMarginTop = m_paramsHandler.GetDoubleValue(ParamNames.MAIN_WEB_STIFF_MARGIN_TOP);
                Plane webStiffCtrlPlane;
                List<double> mainWebStiffStrideList;
                //未旋转时可跟随右侧
                if (!m_context.IsSubRotate)
                {
                    //跟随右侧间距
                    Util.SetDefaultValue(ref mainWebStiffStride[0], stiffStride[0]);
                    mainWebStiffStrideList = BoltParameter.ParseStrideInput(mainWebStiffCount, mainWebStiffStride);
                    //边距设置
                    if (mainWebStiffMarginTop != double.MaxValue)//设置了边距
                        webStiffCtrlPlane = rightStiffTopBound.Invert().Move(mainWebStiffMarginTop);
                    else  //左侧未设置边距，默认跟随右侧加劲板
                        webStiffCtrlPlane = Util.VerticalPlane(ctrlPlane, m_context.SubEndPlateCtrlPlane.Intersect(m_context.SubTopPlane), -m_context.TopOrient).Move(webStiffMarginTop);
                }
                //旋转了需自适应
                else
                {
                    //自适应边距
                    Util.SetDefaultValue(ref mainWebStiffMarginTop, 60);
                    webStiffCtrlPlane = rightStiffTopBound.Invert().Move(mainWebStiffMarginTop);
                    //自适应间距
                    subHeight = m_context.SubTopPlane.Dist(m_context.SubBottomPlane.Origin);
                    Util.SetDefaultValue(ref mainWebStiffStride[0], subHeight - 2 * mainWebStiffMarginTop);
                    mainWebStiffStrideList = BoltParameter.ParseStrideInput(mainWebStiffCount, mainWebStiffStride);
                }
                if (!Util.PlatePropIsValid(webStiffProps) || webStiffCount <= 0)
                    break;
                //添加多跟腹板加劲板
                for (int i = 0; ; i++)
                {
                    //前侧
                    HaunchHelper haunch1 = new HaunchHelper(ctrlPlane, webStiffFrontPlane, webStiffCtrlPlane, Util.Copy(webStiffProps));
                    haunch1.SetFollowPlane(endPlateSidePlanes[0]);
                    haunch1.SetOutsideChamfer(flangeOutsideCutX, flangeOutsideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    haunch1.SetInsideChamfer(flangeInsideCutX, flangeInsideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch1.isTrianlge = true;
                    haunch1.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["main_web_haunch"].Add(haunch1);
                    //后侧
                    HaunchHelper haunch2 = new HaunchHelper(ctrlPlane, webStiffBackPlane, webStiffCtrlPlane, Util.Copy(webStiffProps));
                    haunch2.SetFollowPlane(endPlateSidePlanes[2]);
                    haunch2.SetOutsideChamfer(flangeOutsideCutX, flangeOutsideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    haunch2.SetInsideChamfer(flangeInsideCutX, flangeInsideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch2.isTrianlge = true;
                    haunch2.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["main_web_haunch"].Add(haunch2);
                    //
                    if (i == mainWebStiffStrideList.Count)
                        break;
                    webStiffCtrlPlane.MoveThis(mainWebStiffStrideList[i]);
                }
            } while (false);
#endif

            /*
             * 次加劲板（靠近梁一侧）
             */
            //翼缘加劲板
            {
                inCutX = subFlangeChamferProps.insideCutX;
                inCutY = subFlangeChamferProps.insideCutY;
                outCutX = subFlangeChamferProps.outsideCutX;
                outCutY = subFlangeChamferProps.outsideCutY;
                stiffShape = subFlangeChamferProps.stiffShape;
                //加劲板创建面
                Line centerLine = m_context.SubNearCenterPlane.Intersect(m_context.SubWebCenterPlane); //腹板中心线
                Plane subCenterPlane = Util.VerticalPlane(m_context.SubNearCenterPlane, centerLine, Vec3.AXIS_X); //加劲肋中心面（过主梁中心）
                //上加劲板
                {
                    HaunchHelper haunch = new HaunchHelper(m_context.SubCtrlPlane.Invert(), m_context.SubTopBound, subCenterPlane, Util.Copy(topStiffProps));
                    //Plane topPlane = m_context.IsSubRotate ? endPlateSidePlanes[2] : endPlateSidePlanes[1];
                    haunch.SetFollowPlane(m_context.SubSidePlanes[1]);
                    haunch.SetOutsideChamfer(outCutX, outCutY, Define.CornerCutType.CORNER_CUT_LINE); //TODO 梁和板斜放的情况
                    haunch.SetInsideChamfer(inCutX, inCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch.isTrianlge = true;
                    haunch.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["sub_haunch"].Add(haunch);
                }
                //下加劲板
                {
                    HaunchHelper haunch = new HaunchHelper(m_context.SubCtrlPlane.Invert(), m_context.SubBottomBound, subCenterPlane, Util.Copy(bottomStiffProps));
                    //Plane bottomPlane = m_context.IsSubRotate ? endPlateSidePlanes[0] : endPlateSidePlanes[3];
                    haunch.SetFollowPlane(m_context.SubSidePlanes[3]);
                    haunch.SetOutsideChamfer(outCutX, outCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    haunch.SetInsideChamfer(inCutX, inCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch.isTrianlge = true;
                    haunch.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["sub_haunch"].Add(haunch);
                }
            }
            //腹板加劲板
            do
            {
                inCutX = subWebChamferProps.insideCutX;
                inCutY = subWebChamferProps.insideCutY;
                outCutX = subWebChamferProps.outsideCutX;
                outCutY = subWebChamferProps.outsideCutY;
                stiffShape = subWebChamferProps.stiffShape;
                //排列
                int subWebStiffCount = m_paramsHandler.GetIntValue(ParamNames.SUB_WEB_STIFF_COUNT);
                double[] subWebStiffStride = (double[])m_paramsHandler.GetDoubleArrayValue(ParamNames.SUB_WEB_STIFF_STRIDE).Clone();
                double subWebStiffMarginTop = m_paramsHandler.GetDoubleValue(ParamNames.SUB_WEB_STIFF_MARGIN_TOP);
                Plane webStiffCtrlPlane;
                List<double> subWebStiffStrideList;
                //跟随主加劲
                Util.SetDefaultValue(ref subWebStiffMarginTop, webStiffMarginTop);
                Util.SetDefaultValue(ref subWebStiffCount, webStiffCount);
                Util.SetDefaultValue(ref subWebStiffStride[0], (m_context.SubTopBound.Dist(m_context.SubBottomBound.Origin) //SubHeight
                    - 2 * subWebStiffMarginTop) / (subWebStiffCount - 1));
                subWebStiffStrideList = BoltParameter.ParseStrideInput(subWebStiffCount, subWebStiffStride);
                webStiffCtrlPlane = m_context.SubTopBound.Invert().Move(subWebStiffMarginTop);
                //
                if (!Util.PlatePropIsValid(webStiffProps) || webStiffCount <= 0)
                    break;
                for (int i = 0; ; i++)
                {
                    //前侧
                    HaunchHelper haunch1 = new HaunchHelper(m_context.SubCtrlPlane.Invert(), m_context.SubFrontWeb, webStiffCtrlPlane, Util.Copy(webStiffProps));
                    //Plane bottomPlane = m_context.IsSubRotate ? endPlateSidePlanes[1] : endPlateSidePlanes[0];
                    haunch1.SetFollowPlane(m_context.SubSidePlanes[0]);
                    haunch1.SetOutsideChamfer(outCutX, outCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    haunch1.SetInsideChamfer(inCutX, inCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch1.isTrianlge = true;
                    haunch1.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["sub_haunch"].Add(haunch1);
                    //后侧
                    HaunchHelper haunch2 = new HaunchHelper(m_context.SubCtrlPlane.Invert(), m_context.SubBackWeb, webStiffCtrlPlane, webStiffProps);
                    //bottomPlane = m_context.IsSubRotate ? endPlateSidePlanes[3] : endPlateSidePlanes[2];
                    haunch2.SetFollowPlane(m_context.SubSidePlanes[2]);
                    haunch2.SetOutsideChamfer(outCutX, outCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    haunch2.SetInsideChamfer(inCutX, inCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch2.isTrianlge = true;
                    haunch2.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["sub_haunch"].Add(haunch2);
                    //
                    if (i == subWebStiffStrideList.Count)
                        break;
                    webStiffCtrlPlane.MoveThis(subWebStiffStrideList[i]);
                }
            } while (false);
        }

        class HaunchChamferProps
        {
            public double insideCutX, insideCutY, outsideCutX, outsideCutY;
            public StiffShape stiffShape;
            public HaunchChamferProps(ParamsHandler paramsHandler, string mainOrSub, string flangeOrWeb)
            {
                if (!flangeOrWeb.Equals(""))
                    flangeOrWeb += "_";
                string prefix = mainOrSub + "_" + flangeOrWeb + "haunch_";
                insideCutX = paramsHandler.GetDoubleValue(prefix + "inside_chamfer_cutx");
                insideCutY = paramsHandler.GetDoubleValue(prefix + "inside_chamfer_cuty");
                outsideCutX = paramsHandler.GetDoubleValue(prefix + "outside_chamfer_cutx");
                outsideCutY = paramsHandler.GetDoubleValue(prefix + "outside_chamfer_cuty");
                stiffShape = (StiffShape)paramsHandler.GetIntValue(prefix + "shape");
            }
            public void SetFollowValue(HaunchChamferProps props)
            {
                Util.SetDefaultValue(ref insideCutX, props.insideCutX);
                Util.SetDefaultValue(ref insideCutY, props.insideCutY);
                Util.SetDefaultValue(ref outsideCutX, props.outsideCutX);
                Util.SetDefaultValue(ref outsideCutY, props.outsideCutY);
                if (stiffShape == StiffShape.DEFAULT)
                    stiffShape = props.stiffShape;
            }
        }

        private void SetPropsFollow(PlateProperties props1, PlateProperties props2)
        {
            Util.SetDefaultValue(ref props1.Thickness, props2.Thickness);
            Util.SetDefaultValue(ref props1.Width, props2.Width);
            Util.SetDefaultValue(ref props1.Height, props2.Height);
        }

    }
}
