﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;

namespace ConnectingPlates14
{
    class WeldHandler
    {
        private ComponentInstanceContext m_context = null;
        private ParamsHandler m_paramsHandler = null;
        private List<WeldProperties> m_weldParams = new List<WeldProperties>();

        public WeldHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
            ParseWeldProps();
        }

        public void Execute()
        {
            //主端板和主梁焊接
            WeldMainEndPlate();

            //次端板和次梁焊接
            WeldSubEndPlate();

            /*
             * 7.主、次端板和耳板焊接
             */
            do
            {
                if (m_context.MainEndPlate == null)
                    break;
                if (m_context.FrontEarPlate != null)
                {
                    Weld weld = m_context.Comp.CreateAutoWeld(m_context.MainEndPlate, m_context.FrontEarPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_Y, true); 
                    Vec3 x = m_context.SubSidePlanes[2].Normal; 
                    Vec3 y = m_context.SubCtrlPlane.Normal;
                    weld.WorkAxis = new Mat43(x, y, x.Cross(y), Vec3.ZERO);
                    m_weldParams[6].ApplyToWeld(weld);
                    weld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_X;
                    weld.EdgeOrAround = LoongCAD.STC.Define.WeldEdgeAround.WELD_AROUND;
                    weld.Refresh();
                }
                if (m_context.BackEarPlate != null)
                {
                    Weld weld = m_context.Comp.CreateAutoWeld(m_context.MainEndPlate, m_context.BackEarPlate, LoongCAD.STC.Define.SpaceAxis.AXIS_Y, true);
                    Vec3 x = m_context.SubSidePlanes[0].Normal;
                    Vec3 y = m_context.MainCtrlPlane.Normal;
                    weld.WorkAxis = new Mat43(x, y, x.Cross(y), Vec3.ZERO);
                    m_weldParams[6].ApplyToWeld(weld);
                    weld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_X;
                    weld.EdgeOrAround = LoongCAD.STC.Define.WeldEdgeAround.WELD_AROUND;
                    weld.Refresh();
                }
            } while (false);

            /*
             * 8.加劲板与端板焊接（2条）× 4 （主上、主下、次上、次下4个腋板）
             */
            //主端板和主梁上的腋板焊接
            foreach (var haunch in m_context.HaunchDict["main_haunch"])
            {
                haunch.CreateWeld(m_weldParams[7], m_context.MainPart, m_context.MainEndPlate);
            }
            //次端板和次梁上的腋板焊接
            foreach (var haunch in m_context.HaunchDict["sub_haunch"])
            {
                haunch.CreateWeld(m_weldParams[7], m_context.SubPart, m_context.SubEndPlate);
            }

            m_context.MainPart.PreBuilt.Main = m_context.MainPart;
            m_context.SubPart.PreBuilt.Main = m_context.SubPart;

            /*
             * 衬板构件归属
             */
            do
            {
                ShimWeldMode shimWeld = (ShimWeldMode)m_paramsHandler.GetIntValue(ParamNames.SHIM_WELD_MODE);
                List<long> shimIdList = new List<long>();
                foreach (var plate in m_context.ShimPlateList)
                {
                    shimIdList.Add(plate.Id);
                }
                if (shimIdList.Count == 0 || shimWeld == ShimWeldMode.NONE)
                    break;
                if (shimWeld == ShimWeldMode.WELD_ON_MAIN) //连接到主梁上
                {
                    m_context.MainPart.PreBuilt.AddSubObjects(shimIdList.ToArray());
                }
                if (shimWeld == ShimWeldMode.WELD_ON_SUB) //连接到次梁上
                {
                    m_context.SubPart.PreBuilt.AddSubObjects(shimIdList.ToArray());
                }
            } while (false);
            m_context.MainPart.PreBuilt.Refresh();
            m_context.SubPart.PreBuilt.Refresh();
        }

        public void WeldSubEndPlate()
        {
            /*
             * 1.次梁上翼缘和次端板焊接
             */
            Vec3 frontTopPoint = m_context.SubCtrlPlane.Intersect(m_context.SubSidePlanes[0], m_context.SubSidePlanes[1]);
            Vec3 backTopPoint = m_context.SubCtrlPlane.Intersect(m_context.SubSidePlanes[2], m_context.SubSidePlanes[1]);
            Vec3 frontBottomPoint = m_context.SubCtrlPlane.Intersect(m_context.SubSidePlanes[0], m_context.SubSidePlanes[3]);
            Vec3 backBottomPoint = m_context.SubCtrlPlane.Intersect(m_context.SubSidePlanes[2], m_context.SubSidePlanes[3]);
            do
            {
                //
                Vec3 start = GetOutPoint(m_context.SubTopBound, frontTopPoint, frontBottomPoint);
                Vec3 end = GetOutPoint(m_context.SubTopBound, backTopPoint, backBottomPoint);
                //
                start = GetOutPoint(m_context.SubFrontBound, start, end);
                end = GetOutPoint(m_context.SubBackBound, end, start);
                if (m_context.SubTopFlangeInside.SignedDist(start) > 0 || m_context.SubTopFlangeInside.SignedDist(end) > 0 || m_context.SubNearCenterPlane.Dist(start) > 1e-3)
                    break;
                //
                Weld weld = m_context.Comp.CreatePolyWeld(m_context.SubPart, m_context.SubEndPlate, new Vec3[] { start, end }, true);
                m_weldParams[0].ApplyToWeld(weld);
                weld.Refresh();
            } while (false);

            /*
             * 2.次梁下翼缘和次端板焊接
             */
            do
            {
                //
                Vec3 start = GetOutPoint(m_context.SubBottomBound, frontBottomPoint, frontTopPoint);
                Vec3 end = GetOutPoint(m_context.SubBottomBound, backBottomPoint, backTopPoint);
                //
                start = GetOutPoint(m_context.SubFrontBound, start, end);
                end = GetOutPoint(m_context.SubBackBound, end, start);
                if (m_context.SubBottomFlangeInside.SignedDist(start) > 0 || m_context.SubBottomFlangeInside.SignedDist(end) > 0 || m_context.SubNearCenterPlane.Dist(start) > 1e-3)
                    break;
                //
                Weld weld = m_context.Comp.CreatePolyWeld(m_context.SubPart, m_context.SubEndPlate, new Vec3[] { start, end }, true);
                m_weldParams[1].ApplyToWeld(weld);
                weld.Refresh();
            } while (false);


            /*
             * 3.次梁腹板和次端板焊接
             */
            do
            {
                //
                Vec3 start = Util.PlaneLineIntersect(m_context.SubFrontWeb, frontTopPoint, backTopPoint);
                Vec3 end = Util.PlaneLineIntersect(m_context.SubFrontWeb, frontBottomPoint, backBottomPoint);
                //
                start = GetOutPoint(m_context.SubTopFlangeInside.Invert(), start, end);
                end = GetOutPoint(m_context.SubBottomFlangeInside.Invert(), end, start);
                if (m_context.SubNearCenterPlane.Dist(start) > 1e-3 || m_context.SubNearCenterPlane.Dist(end) > 1e-3) //旋转了，间距过大焊接不上
                    break;
                //
                Weld weld = m_context.Comp.CreatePolyWeld(m_context.SubPart, m_context.SubEndPlate, new Vec3[] { start, end }, true);
                m_weldParams[2].ApplyToWeld(weld);
                weld.Refresh();
            } while (false);
        }

        public void WeldMainEndPlate()
        {
            /*
             * 4.主梁上翼缘和主端板焊接
             */
            Vec3 frontTopPoint = m_context.MainCtrlPlane.Intersect(m_context.MainSidePlanes[0], m_context.MainSidePlanes[1]);
            Vec3 backTopPoint = m_context.MainCtrlPlane.Intersect(m_context.MainSidePlanes[2], m_context.MainSidePlanes[1]);
            Vec3 frontBottomPoint = m_context.MainCtrlPlane.Intersect(m_context.MainSidePlanes[0], m_context.MainSidePlanes[3]);
            Vec3 backBottomPoint = m_context.MainCtrlPlane.Intersect(m_context.MainSidePlanes[2], m_context.MainSidePlanes[3]);
            do
            {
                //
                Vec3 start = GetOutPoint(m_context.MainTopBound, frontTopPoint, frontBottomPoint);
                Vec3 end = GetOutPoint(m_context.MainTopBound, backTopPoint, backBottomPoint);
                //
                start = GetOutPoint(m_context.MainFrontBound, start, end);
                end = GetOutPoint(m_context.MainBackBound, end, start);
                if (m_context.MainTopFlangeInside.SignedDist(start) > 0 || m_context.MainTopFlangeInside.SignedDist(end) > 0 || m_context.MainNearCenterPlane.Dist(start) > 1e-3)
                    break;
                //
                Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, m_context.MainEndPlate, new Vec3[] { start, end }, true);
                m_weldParams[3].ApplyToWeld(weld);
                weld.Refresh();
            } while (false);

            /*
             * 5.主梁下翼缘和主端板焊接
             */
            do
            {
                //
                Vec3 start = GetOutPoint(m_context.MainBottomBound, frontBottomPoint, frontTopPoint);
                Vec3 end = GetOutPoint(m_context.MainBottomBound, backBottomPoint, backTopPoint);
                //
                start = GetOutPoint(m_context.MainFrontBound, start, end);
                end = GetOutPoint(m_context.MainBackBound, end, start);
                if (m_context.MainBottomFlangeInside.SignedDist(start) > 0 || m_context.MainBottomFlangeInside.SignedDist(end) > 0 || m_context.MainNearCenterPlane.Dist(start) > 1e-3)
                    break;
                //
                Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, m_context.MainEndPlate, new Vec3[] { start, end }, true);
                m_weldParams[4].ApplyToWeld(weld);
                weld.Refresh();
            } while (false);

            /*
             * 6.主梁腹板和主端板焊接
             */
            do
            {
                //
                Vec3 start = Util.PlaneLineIntersect(m_context.MainFrontWeb, frontTopPoint, backTopPoint);
                Vec3 end = Util.PlaneLineIntersect(m_context.MainFrontWeb, frontBottomPoint, backBottomPoint);
                //
                start = GetOutPoint(m_context.MainTopFlangeInside.Invert(), start, end);
                end = GetOutPoint(m_context.MainBottomFlangeInside.Invert(), end, start);
                if (m_context.MainNearCenterPlane.Dist(start) > 1e-3 || m_context.MainNearCenterPlane.Dist(end) > 1e-3)
                    break;
                //
                Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, m_context.MainEndPlate, new Vec3[] { start, end }, true);
                m_weldParams[5].ApplyToWeld(weld);
                weld.Refresh();
            } while (false);
        }

        public void ParseWeldProps()
        {
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                m_weldParams.Add(new WeldProperties());
            }
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldParams[i].SetProps(weldProps);
            }
        }

        //输入梁的边缘平面，返回焊接创建点
        public Vec3 GetOutPoint(Plane beamPlane1, Plane beamPlane2, Plane beamPlane3, Plane platePlane1, Plane platePlane2)
        {
            Plane plane1 = GetOutPlane(beamPlane1, platePlane1);
            Plane plane2 = GetOutPlane(beamPlane2, platePlane2);
            //m_context.test(plane2);
            Vec3 point = beamPlane3.Intersect(plane1, plane2);
            return point;
        }

        //输入梁的边缘平面和板顶点，点在内部就返回，在外部求交点
        public Vec3 GetOutPoint(Plane beamPlane, Vec3 createPoint, Vec3 auxPoint)
        {
            if (beamPlane.SignedDist(createPoint) > 0)
                return Util.PlaneLineIntersect(beamPlane, createPoint, auxPoint);
            return createPoint;
        }

        //输入梁的平面和板边缘面，返回创建点所在面
        public Plane GetOutPlane(Plane beamPlane, Plane platePlane)
        {
            double d = beamPlane.SignedDist(platePlane.Origin);
            if (beamPlane.SignedDist(platePlane.Origin) > 0)
                return beamPlane;
            return platePlane;
        }

    }
}
