﻿using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;

namespace CornerBoltedGusset57.Common
{
    /// <summary>
    /// 节点板边缘附近的支撑类型
    /// </summary>
    public enum BraceSpacingType
    {
        FIRST,
        BETWEEN,
        LAST
    }
    /// <summary>
    /// 支撑截面类型
    /// </summary>
    public enum BraceProfileType
    {
        L,
        T,
        I,
        C,
        U,
        CC,
        RECTHOLLOW,
        BAN,
        HQ,
        WELDBOX,
        OTHER
    }
    public class BraceData
    {
        public Beam Beam;
        public BeamPlanes BeamPlanes;
        /// <summary>
        /// 支撑截面类型
        /// </summary>
        public BraceProfileType ProfileType;
        /// <summary>
        /// 支撑从次零件方向出发，较近的面
        /// </summary>
        public Plane TopPlane;
        /// <summary>
        /// 支撑从次零件方向出发，较远的面
        /// </summary>
        public Plane BottomPlane;
        /// <summary>
        /// 支撑离主零件较远的面
        /// </summary>
        public Plane FarPlane;
        /// <summary>
        /// 调整后支撑的构建方向
        /// </summary>
        public Vec3 braceLine;
        /// <summary>
        /// 支撑的控制点中离主零件较远的控制点
        /// </summary>
        public Vec3 farPoint;
        /// <summary>
        /// 支撑构件节点板的第一控制点
        /// </summary>
        public Vec3 TopPoint;
        /// <summary>
        /// 支撑构建节点板的第二控制点
        /// </summary>
        public Vec3 BottomPoint;
        /// <summary>
        /// 支撑构建节点板的对齐面
        /// </summary>
        public Plane AlignPlane;
        /// <summary>
        /// 节点板所在支撑的边界面
        /// </summary>
        public Plane GussetBorderPlane;
        /// <summary>
        /// 支撑的选取序号
        /// </summary>
        public int ClickNumber;
        /// <summary>
        /// 支撑排序后的顺序编号
        /// </summary>
        public int SortNumber;
        /// <summary>
        /// 支撑沿着Z轴正方向面
        /// 规定Z轴方向为主零件构建方向与主零件面向支撑面法线的叉积
        /// </summary>
        public Plane FrontPlane;
        /// <summary>
        /// 支撑沿着Z轴负方向面
        /// </summary>
        public Plane BackPlane;
    }
}
