﻿using System;
using System.Windows.Forms;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;
using CornerBoltedGusset57.Common;
using System.Collections.Generic;

namespace CornerBoltedGusset57
{
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {
        private DObject m_rootUIProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;

        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostIntialize();
        }
        public override ComponentPlugin ReferencePlugin => m_refPlugin;
        private void OnPostIntialize()
        {
            // 设置窗口标题
            this.Text = "角部支撑连接节点(301)";

            foreach (TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }

            // 螺栓标准变化之后调用螺栓内容的自适应方法
            this.panel_bolt1_basic_property.BoltStandardChanged += this.boltContentPanel1.AdaptBoltContents;
            this.panel_bolt2_basic_property.BoltStandardChanged += this.boltContentPanel2.AdaptBoltContents;
            this.panel_bolt3_basic_property.BoltStandardChanged += this.boltContentPanel3.AdaptBoltContents;

            // 初始化螺栓纵向偏移模式控件
            this.cbo_bolt1_vert_offset_mode.ClearItems();
            this.cbo_bolt1_vert_offset_mode.AddItem(0, "默认");
            this.cbo_bolt1_vert_offset_mode.AddItem(1, "顶部");
            this.cbo_bolt1_vert_offset_mode.AddItem(2, "中间");
            this.cbo_bolt1_vert_offset_mode.AddItem(3, "底部");
            this.cbo_bolt1_vert_offset_mode.UpdateItems();
            this.cbo_bolt1_vert_offset_mode.SelectedKey = 0;
  
            this.cbo_bolt2_vert_offset_mode.ClearItems();
            this.cbo_bolt2_vert_offset_mode.AddItem(0, "默认");
            this.cbo_bolt2_vert_offset_mode.AddItem(1, "顶部");
            this.cbo_bolt2_vert_offset_mode.AddItem(2, "中间");
            this.cbo_bolt2_vert_offset_mode.AddItem(3, "底部");
            this.cbo_bolt2_vert_offset_mode.UpdateItems();
            this.cbo_bolt2_vert_offset_mode.SelectedKey = 0;

            this.cbo_bolt3_vert_offset_mode.ClearItems();
            this.cbo_bolt3_vert_offset_mode.AddItem(0, "默认");
            this.cbo_bolt3_vert_offset_mode.AddItem(1, "顶部");
            this.cbo_bolt3_vert_offset_mode.AddItem(2, "中间");
            this.cbo_bolt3_vert_offset_mode.AddItem(3, "底部");
            this.cbo_bolt3_vert_offset_mode.UpdateItems();
            this.cbo_bolt3_vert_offset_mode.SelectedKey = 0;

            //初始化图片选项卡
            this.gusset_plate_position_brace.ClearItems();
            this.gusset_plate_position_brace.AddIntegerItems(0, 0, 5);
            this.gusset_plate_position_brace.UpdateItems();
            
            this.gusset_plate_shape.ClearItems();
            this.gusset_plate_shape.AddIntegerItems(0, 0, 4);
            this.gusset_plate_shape.UpdateItems();
           
            this.gusset_brace_notch.ClearItems();
            this.gusset_brace_notch.AddIntegerItems(0, 0, 4);
            this.gusset_brace_notch.UpdateItems();
            
            this.gusset_plate_edge_shape_secondary_part_last_brace.ClearItems();
            this.gusset_plate_edge_shape_secondary_part_last_brace.AddIntegerItems(0, 0, 5);
            this.gusset_plate_edge_shape_secondary_part_last_brace.UpdateItems();
           
            this.gusset_plate_edge_shape_main_part_first_brace.ClearItems();
            this.gusset_plate_edge_shape_main_part_first_brace.AddIntegerItems(0, 0, 5);
            this.gusset_plate_edge_shape_main_part_first_brace.UpdateItems();
          
            this.gusset_plate_fitting.ClearItems();
            this.gusset_plate_fitting.AddIntegerItems(0, 0, 3);
            this.gusset_plate_fitting.UpdateItems();

            this.gusset_welding.ClearItems();
            this.gusset_welding.AddIntegerItems(0, 0, 4);
            this.gusset_welding.UpdateItems();
        }

        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null)
            {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }
        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked)
            {
                retProps.CopyFrom(m_rootUIProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }
        /// <summary>
        /// 当前组件需要重写此方法，并刷新螺栓基础面板
        /// </summary>
        /// <param name="root"></param>
        public override void ClearUI(Control root = null)
        {
            // 刷新螺栓基础面板
            this.panel_bolt1_basic_property.RefreshControls();
            this.panel_bolt2_basic_property.RefreshControls();
            this.panel_bolt3_basic_property.RefreshControls();
            base.ClearUI(root);
        }
        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }
        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_rootUIProps.CopyFrom(srcProps);
        }
        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }
        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_rootUIProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_rootUIProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_rootUIProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_rootUIProps;
            return comp.UIProps;
        }
        private void btn_weld_props_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_rootUIProps, null);
        }

        private void cbo_plate_main_chamfer_SelectedIndexChanged(object sender, EventArgs e)
        {
            bool flagX, flagY;
            if(cbo_plate_main_chamfer.SelectedIndex == 1)
            {
                flagX = false;
                flagY = false;
            }
            else if(cbo_plate_main_chamfer.SelectedIndex == 3 || cbo_plate_main_chamfer.SelectedIndex == 4)
            {
                flagX = true;
                flagY = false;
            }
            else
            {
                flagX = true;
                flagY = true;
            }

            tb_plate_main_chamferX.Enabled = flagX;
            act_plate_main_chamferX.Enabled = flagX;
            tb_plate_main_chamferY.Enabled = flagY;
            act_plate_main_chamferY.Enabled = flagY;
        }

        private void cbo_plate_sub_chamfer_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            bool flagX, flagY;
            if (cbo_plate_sub_chamfer_type.SelectedIndex == 1)
            {
                flagX = false;
                flagY = false;
            }
            else if (cbo_plate_sub_chamfer_type.SelectedIndex == 3 || cbo_plate_sub_chamfer_type.SelectedIndex == 4)
            {
                flagX = true;
                flagY = false;
            }
            else
            {
                flagX = true;
                flagY = true;
            }

            tb_plate_sub_chamferX.Enabled = flagX;
            act_plate_sub_chamferX.Enabled = flagX;
            tb_plate_sub_chamferY.Enabled = flagY;
            act_plate_sub_chamferY.Enabled = flagY;
        }
    }
}
