﻿using CornerBoltedGusset57.Common;
using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.STC;
using System;

namespace CornerBoltedGusset57.Handlers
{
    public class GraphicHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;

        private PartBase retPlate;
        private PlateProperties m_plateProps = new PlateProperties();
        private List<BraceData> m_braceData = new List<BraceData>();

        private Plane m_mainFaceBracePlane;
        private Plane m_subFaceBracePlane;
        private Vec3 m_subLine;
        private Plane m_baseGussetPlane;
        private List<Vec3> CtrlPoints = new List<Vec3>();
        private List<Vec3> points = new List<Vec3>();

        //图形页面参数
        private double m_gussetPlateEdgeGap = double.MaxValue;
        private double s_gussetPlateEdgeGap = double.MaxValue;
        private double m_firstBraceGussetPlateCornerAngle = double.MaxValue;
        private double m_lastBraceGussetPlateCornerAngle = double.MaxValue;

        //节点板页面参数
        private int m_plateShape = 0;
        private int m_subGussetShapeType = 0;
        private int m_mainGussetShapeType = 0;
        private int m_gussetFitting = 0;

        public GraphicHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public bool Execute()
        {
            //初始化上下文参数
            m_subLine = m_context.SubLine;
            m_braceData = m_context.BraceData;
            m_mainFaceBracePlane = m_context.MainFaceBracePlane;
            m_subFaceBracePlane = m_context.SubFaceBracePlane;
            m_baseGussetPlane = m_context.BaseGussetPlane;

            if (!ParseProps())
                return false;
            //执行对齐操作
            foreach (var item in m_braceData)
                m_context.TargetComp.CreateAlignPlane(item.Beam, item.AlignPlane, true);
            if(!CreateGussetPlate())
                return false;
            if (!CreateBolts(m_braceData.ToArray(), retPlate))
                return false;

            HandlerChamfer();
            return true;
        }
        //解析属性
        private bool ParseProps()
        {
            DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.GUSSET_PLATE_PROPS);
            if (plateProps == null)
                return false;
            m_plateProps.SetProps(plateProps);
            m_context.Thickness = m_plateProps.Thickness;

            m_gussetPlateEdgeGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_GUSSET_PLATE_EDGE_GAP_MAIN_PART);
            s_gussetPlateEdgeGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_GUSSET_PLATE_EDGE_GAP_SECONDARY_PART);
            m_lastBraceGussetPlateCornerAngle = m_paramsHandler.GetDoubleValue(ParamNames.PIC_GUSSET_PLATE_CORNER_ANGLE_LAST_BRACE);
            m_firstBraceGussetPlateCornerAngle = m_paramsHandler.GetDoubleValue(ParamNames.PIC_GUSSET_PLATE_CORNER_ANGLE_FIRST_BRACE);

            m_plateShape = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_SHAPE);
            m_subGussetShapeType = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_EDGE_SHAPE_SECONDARY_PART_LAST_BRACE);
            m_mainGussetShapeType = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_EDGE_SHAPE_MAIN_PART_FIRST_BRACE);
            m_gussetFitting = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_FITTING);

            //自适应角度
            if (m_lastBraceGussetPlateCornerAngle == double.MaxValue)
            {
                m_lastBraceGussetPlateCornerAngle = AdjustSubGussetEdgeShape();
            }

            if (m_firstBraceGussetPlateCornerAngle == double.MaxValue)
            {
                m_firstBraceGussetPlateCornerAngle = AdjustMainGussetEdgeShape();
            }
            return true;
        }
        //调整次零件附近节点板形状
        private double AdjustSubGussetEdgeShape()
        {
            double angle = 0.0;
            var normal = m_braceData.First().TopPlane.Normal.Normalize();
            double cosine;
            switch (m_subGussetShapeType)
            {
                case 2:
                    cosine = normal.Dot(m_subLine);
                    angle = Math.Acos(cosine) * 180 / Math.PI; //将弧度转换为角度
                    break;
                case 3:
                    angle = 90.0;
                    break;
                case 4:
                    cosine = normal.Dot(m_mainFaceBracePlane.Normal.Normalize());
                    angle = Math.Acos(cosine) * 180 / Math.PI;
                    break;
                case 0:
                default:
                    break;
            }
            return angle;
        }
        //调整主零件附近节点板形状
        private double AdjustMainGussetEdgeShape()
        {
            double angle = 0.0;
            var normal = m_braceData.Last().BottomPlane.Normal.Normalize();
            double cosine;
            switch (m_mainGussetShapeType)
            {
                case 1:
                    break;
                case 3:
                    angle = 90.0;
                    break;
                case 4:
                    cosine = normal.Dot(m_subFaceBracePlane.Normal.Normalize());
                    angle = Math.Acos(cosine) * 180 / Math.PI;
                    break;
                case 0:
                default:
                    cosine = Math.Abs(normal.Dot(m_context.MainLine));
                    angle = Math.Acos(cosine) * 180 / Math.PI;
                    break;
            }
            return angle;
        }
        //创建节点板
        private bool CreateGussetPlate()
        {
            var LastBrace = m_braceData.Last();
            
            //控制点1
            var point1 = AdjustGussetFitting()[0];
            //控制点2
            var point2 = AdjustGussetFitting()[1];
            //控制点3
            var point3 = GetRotatePlane(m_braceData.Last().BottomPlane.Normal.Normalize(),
                       -m_firstBraceGussetPlateCornerAngle, LastBrace.BottomPoint)
                       .Intersect(m_mainFaceBracePlane.Move(m_gussetPlateEdgeGap), m_baseGussetPlane);

            //创建方形节点板
            if (m_plateShape == 2)
            {
                CtrlPoints.Add(point1);
                double max = GetMaxpointDist(m_braceData, m_mainFaceBracePlane);
                var b = m_plateProps.Width;
                var h = m_plateProps.Height;
                if (h == 0 || b == 0)
                    return false;

                if (h != double.MaxValue)
                    point3 = point1 + (point3 - point1).Normalize() * h;

                if (b == double.MaxValue)
                    b = max;

                var pointX = point3 + m_mainFaceBracePlane.Normal.Normalize() * b;

                var plane = new Plane(pointX, m_mainFaceBracePlane.Normal);
                var pointY = plane.Intersect(m_baseGussetPlane, m_subFaceBracePlane);

                //存储焊接点
                m_context.GussetWeldPoints.Add(point1);
                m_context.GussetWeldPoints.Add(point1);
                m_context.GussetWeldPoints.Add(pointY);
                m_context.GussetWeldPoints.Add(point3);

                CtrlPoints.Add(pointY);
                CtrlPoints.Add(pointX);
                CtrlPoints.Add(point3);
                m_context.SquarePoints = CtrlPoints;
                retPlate = m_context.TargetComp.CreateSteelPlate(AdjustGussetPlateDepth(m_baseGussetPlane, CtrlPoints), 
                    m_plateProps.Thickness, Define.ObjectSide.SIDE_LEFT, true);
            }
            else
            {
                //创建凸形节点板
                CtrlPoints.Add(point1);
                if(!CtrlPoints.Contains(point2))
                    CtrlPoints.Add(point2);
                foreach (var item in m_braceData)
                {
                    CtrlPoints.Add(item.TopPoint);
                    CtrlPoints.Add(item.BottomPoint);
                }
                CtrlPoints.Add(point3);

                //存储焊接点
                m_context.GussetWeldPoints.Add(point1);
                //处理倒角 复制了一份焊接点 分别向倒角的X和Y方向移动
                m_context.GussetWeldPoints.Add(point1);
                if (point1.Equals(point2))
                    m_context.GussetWeldPoints.Add(point3);
                else
                    m_context.GussetWeldPoints.Add(point2);
                m_context.GussetWeldPoints.Add(point3);
                retPlate = m_context.TargetComp.CreateSteelPlate(AdjustGussetPlateDepth(m_baseGussetPlane, CtrlPoints), 
                    m_plateProps.Thickness, Define.ObjectSide.SIDE_LEFT, true);
            }
            
            if (retPlate == null)
                return false;
            // 根据组件通用属性设置板的级别
            retPlate.Class = m_context.TargetComp.CompClass;
            m_plateProps.ApplyToPart(retPlate);

            m_context.RetPlate = retPlate;
            return true;
        }
        /// <summary>
        /// 得到节点板控制点到指定面距离的最大值
        /// </summary>
        /// <param name="braces"></param>
        /// <param name="plane"></param>
        /// <returns></returns>
        private double GetMaxpointDist(List<BraceData> braces, Plane plane)
        {
            double max = -1.0;
            foreach (var brace in braces)
            {
                var dist1 = plane.Dist(brace.TopPoint);
                var dist2 = plane.Dist(brace.BottomPoint);

                var temp = Math.Max(dist1, dist2);
                if (temp > max)
                    max = temp;
            }
            return max;
        }
        //调整节点板是否接合次梁
        private Vec3[] AdjustGussetFitting()
        {
            List<Vec3> points = new List<Vec3>();
            var FirstBrace = m_braceData.First();
            //主零件 次零件 节点板的交点
            var point1 = m_mainFaceBracePlane.Move(m_gussetPlateEdgeGap)
                .Intersect(m_baseGussetPlane, m_subFaceBracePlane.Move(s_gussetPlateEdgeGap));
            //次零件 节点板的交点
            var point2 = GetRotatePlane(FirstBrace.TopPlane.Normal.Normalize(),
                        m_lastBraceGussetPlateCornerAngle, FirstBrace.TopPoint)
                        .Intersect(m_subFaceBracePlane.Move(s_gussetPlateEdgeGap), m_baseGussetPlane);
            //次零件旁支撑上侧节点板和主零件的交点
            var point = GetRotatePlane(FirstBrace.TopPlane.Normal.Normalize(),
                        m_lastBraceGussetPlateCornerAngle, FirstBrace.TopPoint)
                        .Intersect(m_mainFaceBracePlane.Move(m_gussetPlateEdgeGap), m_baseGussetPlane);
            var line = point2 - point1;
            points.Add(point1);
            points.Add(point2);
            bool flag = line.Normalize().Dot(m_subLine) < 0;
            switch (m_gussetFitting)
            {
                case 2:
                    if (flag)
                    {
                        points.Clear();
                        points.Add(point1);
                        points.Add(point1 + m_context.SubLine * 5);
                    }
                    break;
                case 0:
                default:
                    if (flag)
                    {
                        points.Clear();
                        //此时取点为次零件旁支撑上侧节点板和主零件的交点
                        points.Add(point);
                        points.Add(point);
                    }
                    break;
            }
            return points.ToArray();
        }
        /// <summary>
        /// 对平面按照给定的角度进行旋转
        /// </summary>
        /// <param name="vector">平面的法线</param>
        /// <param name="angle">旋转角度</param>
        /// <param name="point">面上的一点</param>
        private Plane GetRotatePlane(Vec3 vector, double angle, Vec3 point)
        {
            Vec3 axis = m_subFaceBracePlane.Normal.Normalize().Cross(m_subLine.Normalize());
            Vec3 normal = GenericTools.RotateVectorInPlane(vector, -axis, angle);
            Plane plane = new Plane(point, normal);
            return plane;
        }
        //调整节点板深度
        private Vec3[] AdjustGussetPlateDepth(Plane plane, List<Vec3> list)
        {
            Vec3 axis = m_subFaceBracePlane.Normal.Normalize().Cross(m_subLine.Normalize());
            //记录错误情况翻转次数
            int count = 0;
            for(int i = 0; i < m_braceData.Count - 1; i++)
            {
                var bottomPoint = m_braceData[i].BottomPoint;
                var topPoint = m_braceData[i + 1].TopPoint;
                var topPlane = m_braceData[i + 1].TopPlane;
                double dist1 = topPlane.Dist(bottomPoint);
                double dist2 = topPlane.Dist(topPoint);
                if (dist1 < dist2)
                    count++;
            }
            if(count % 2 == 1)
                axis = -axis;

            if (plane.Normal.Dot(-axis) > 0)
            {
                points = list;
                return list.ToArray();
            }

            var newList = new List<Vec3>();
            newList.Add(list.First());
            for (int i = list.Count - 1; i >= 1; i--)
                newList.Add(list[i]);
            points = newList;
            return newList.ToArray();
        }
        //创建螺栓组
        private bool CreateBolts(BraceData[] braces, PartBase retPlate)
        {
            BoltParse boltParse = new BoltParse(m_context, m_paramsHandler);
            if (!boltParse.Execute()) return false;
            List<BoltParse.ParamData> BoltParams = boltParse.ParamDataMap;
            int len = BoltParams.Count;
            var target = m_context.TargetComp;
            for (int i = 0; i < braces.Length; i++)
            {
                var brace = braces[i];
                BoltParse.ParamData boltGroup = BoltParams[brace.ClickNumber % len];
                if (boltGroup.HorBoltsSpace == null || boltGroup.VerBoltsSpace == null)
                    continue;

                Vec3 bolt_Y = brace.BottomPlane.Normal.Normalize();
                Vec3 bolt_X = brace.braceLine.Normalize();
                Vec3 bolt_Z = bolt_X.Cross(bolt_Y);
                var topPoint = brace.GussetBorderPlane.Intersect(m_baseGussetPlane, brace.TopPlane);
                var bottomPoint = brace.GussetBorderPlane.Intersect(m_baseGussetPlane, brace.BottomPlane);
                Vec3 posStart = (topPoint + bottomPoint) / 2;
                Vec3 posEnd = posStart + bolt_X * 1000;
                BoltGroup bolts = target.CreateBoltGroup(brace.Beam, retPlate, posStart, posEnd, true);
                if (bolts == null)
                    return false;
                bolts.WorkAxis = new Mat43(bolt_X, bolt_Y, bolt_Z, Vec3.ZERO);
                boltGroup.BoltsCfg.FillBasicProps(bolts, true);
                boltGroup.BoltsCfg.FillHoleProps(bolts);
                boltGroup.BoltsCfg.FillContentProps(bolts);
                boltGroup.BoltsCfg.FillExtendProps(bolts);
                boltGroup.BoltsCfg.FillShape(bolts);
                bolts.ArrayStrideX = boltGroup.HorBoltsSpace;
                bolts.ArrayStrideY = boltGroup.VerBoltsSpace;
                bolts.HeadOffsetX = -(boltGroup.VerFarEdgeDist + GenericTools.SumArray(bolts.ArrayStrideX));
                bolts.RotationSide = Define.ObjectSide.SIDE_TOP;
                bolts.Rotation = 0;

                //计算纵向偏移距离
                var dist = brace.TopPlane.Dist(brace.BottomPlane.Origin);
                var NonMiddle = GenericTools.SumArray(bolts.ArrayStrideY);
                double vertOffset;
                if (GussetPlateTools.ParseBraceProfileType(brace.Beam) == BraceProfileType.L)
                {
                    //L型
                    vertOffset = GussetPlateTools.AdjustLVertOffset(boltGroup.VerMeasureMethod,
                        dist, NonMiddle, boltGroup.VerMeasureOffset);
                } else
                {
                    //其他类型
                    vertOffset = GussetPlateTools.AdjustOtherVertOffset(boltGroup.VerMeasureMethod,
                    dist, NonMiddle, boltGroup.VerMeasureOffset);
                }
                bolts.VerticalOffset = vertOffset;
                //计算切割长度
                if (m_context.EnableNotch || m_context.EnableSpecialNotch)
                {
                    //开启槽口的情况
                    var CutLength = brace.BackPlane.Dist(brace.FrontPlane.Origin);
                    bolts.CutLength = CutLength + 25.0;
                }else
                {
                    var CutLength = GussetPlateTools.GetSectionThickness(brace.ProfileType, brace.BeamPlanes, m_baseGussetPlane);
                    bolts.CutLength = 2 * (CutLength + m_plateProps.Thickness) + 1.0;
                }
                bolts.Refresh();
            }
            return true;
        }
        private void HandlerChamfer()
        {
            int m_chamferType = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_MAIN_CHAMFER_TYPE);
            var m_chamferX = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_PLATE_MAIN_CHAMFERX);
            var m_chamferY = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_PLATE_MAIN_CHAMFERY);
            setMainChamfer(m_chamferX, m_chamferY, m_chamferType);

            int s_chamferType = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_SUB_CHAMFER_TYPE);
            var s_chamferX = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_PLATE_SUB_CHAMFERX);
            var s_chamferY = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_PLATE_SUB_CHAMFERY);
            setSubChamfer(s_chamferX, s_chamferY, s_chamferType);
        }
        private void setMainChamfer(double chamferX, double chamferY, int type)
        {
            var dirY = (CtrlPoints[1] - CtrlPoints[0]).Normalize();
            var dirX = (CtrlPoints.Last() - CtrlPoints[0]).Normalize();
            Define.CornerCutType chamferType;
            switch (type)
            {
                case 1:
                    chamferType = Define.CornerCutType.CORNER_CUT_NONE;
                    break;
                case 3:
                    chamferType = Define.CornerCutType.CORNER_CUT_ROUND;
                    
                    break;
                case 4:
                    chamferType = Define.CornerCutType.CORNER_CUT_INV_ROUND;
                    m_context.GussetWeldPoints[0] += dirX * chamferX;
                    m_context.GussetWeldPoints[1] += dirY * chamferX;
                    break;
                case 0:
                default:
                    chamferType = Define.CornerCutType.CORNER_CUT_LINE;
                    if (points[1].Equals(CtrlPoints[1]))
                    {
                        m_context.GussetWeldPoints[0] += dirX * chamferX;
                        m_context.GussetWeldPoints[1] += dirY * chamferY;
                    }
                    else
                    {
                        m_context.GussetWeldPoints[0] += dirX * chamferY;
                        m_context.GussetWeldPoints[1] += dirY * chamferX;
                    }
                    break;
            }
            ChamferCtrlPoint chamferPoint = retPlate.CtrlPoints[0] as ChamferCtrlPoint;
            if (chamferPoint != null)
            {
                chamferPoint.CutX = chamferX;
                chamferPoint.CutY = chamferY;
                chamferPoint.CutType = chamferType;
                chamferPoint.Flush();
                retPlate.Flush();
            }
        }
        private void setSubChamfer(double chamferX, double chamferY, int type)
        {
            //零件焊接方向
            var dir1 = (CtrlPoints[0] - CtrlPoints[1]).Normalize();
            var dir2 = (CtrlPoints[0] - CtrlPoints.Last()).Normalize();
            Define.CornerCutType chamferType;

            //加劲肋建立方向
            var dirStiff1 = (CtrlPoints[2] - CtrlPoints[1]).Normalize();
            m_context.Stiffener1.Add(CtrlPoints.Last());
            m_context.Stiffener1.Add(m_braceData.Last().BottomPoint);

            //节点板远离次零件的情况
            bool stiffFlag = false;
            var dirStiff2 = (m_braceData.Last().BottomPoint - CtrlPoints.Last()).Normalize();
            if (m_context.GussetWeldPoints[2].Equals(m_context.GussetWeldPoints[3]))
            {
                stiffFlag = true;
                m_context.Stiffener2.Add(CtrlPoints[0]);
                m_context.Stiffener2.Add(m_braceData.First().TopPoint);
                dirStiff2 = (CtrlPoints[1] - CtrlPoints[0]).Normalize();
            }
            else
            {
                m_context.Stiffener2.Add(CtrlPoints[1]);
                m_context.Stiffener2.Add(m_braceData.First().TopPoint);
            }
               
            switch (type)
            {
                case 1:
                    m_context.Stiffener1.Add(m_context.GussetWeldPoints[3]);
                    if (stiffFlag)
                        m_context.Stiffener2.Add(CtrlPoints[0]);
                    else
                        m_context.Stiffener2.Add(m_context.GussetWeldPoints[2]);
                    chamferType = Define.CornerCutType.CORNER_CUT_NONE;
                    break;
                case 3:
                    m_context.Stiffener1.Add(m_context.GussetWeldPoints[3]);
                    if (stiffFlag)
                        m_context.Stiffener2.Add(CtrlPoints[0]);
                    else
                        m_context.Stiffener2.Add(m_context.GussetWeldPoints[2]);
                    chamferType = Define.CornerCutType.CORNER_CUT_ROUND;
                    break;
                case 4:
                    m_context.Stiffener1.Add(m_context.GussetWeldPoints[3] + dirStiff2 * chamferX);
                    if (stiffFlag)
                        m_context.Stiffener2.Add(CtrlPoints[0] + dirStiff2 * chamferX);
                    else
                    {
                        m_context.Stiffener2.Add(m_context.GussetWeldPoints[2] + dirStiff1 * chamferX);
                        m_context.GussetWeldPoints[2] += dir1 * chamferX;
                        m_context.GussetWeldPoints[3] += dir2 * chamferX;
                    }
                    chamferType = Define.CornerCutType.CORNER_CUT_INV_ROUND;
                    break;
                case 0:
                default:
                    chamferType = Define.CornerCutType.CORNER_CUT_LINE;
                    double X = chamferX;
                    double Y = chamferY;
                    if (points[1].Equals(CtrlPoints[1]))
                    {
                        X = chamferY;
                        Y = chamferX;
                    }

                    m_context.Stiffener1.Add(m_context.GussetWeldPoints[3] + dirStiff2 * Y);
                    if (stiffFlag)
                        m_context.Stiffener2.Add(CtrlPoints[0] + dirStiff1 * X);
                    else
                    {
                        m_context.Stiffener2.Add(m_context.GussetWeldPoints[2] + dirStiff1 * X);
                        m_context.GussetWeldPoints[2] += dir1 * Y;
                        m_context.GussetWeldPoints[3] += dir2 * X;
                    }    
                    break;
            }
            ChamferCtrlPoint chamferPoint1 = retPlate.CtrlPoints[1] as ChamferCtrlPoint;
            if (chamferPoint1 != null)
            {
                chamferPoint1.CutX = chamferX;
                chamferPoint1.CutY = chamferY;
                chamferPoint1.CutType = chamferType;
                chamferPoint1.Flush();
                retPlate.Flush();
            }
            ChamferCtrlPoint chamferPoint2 = retPlate.CtrlPoints.Last() as ChamferCtrlPoint;
            if (chamferPoint2 != null)
            {
                chamferPoint2.CutX = chamferX;
                chamferPoint2.CutY = chamferY;
                chamferPoint2.CutType = chamferType;
                chamferPoint2.Flush();
                retPlate.Flush();
            }
        }
    }
}
