﻿using CornerBoltedGusset57.Common;
using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using LoongCAD.STC;
using System;

namespace CornerBoltedGusset57.Handlers
{
    public class StiffenerHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;

        private Vec3 m_axisZ = Vec3.ZERO;

        //上下文属性
        private List<Vec3> m_stiff1 = new List<Vec3>();
        private List<Vec3> m_stiff2 = new List<Vec3>();

        private double m_plateDistanceToMain = double.MaxValue;
        private double m_plateDistanceToSub = double.MaxValue;

        //加劲肋页参数
        //加劲肋1
        private PlateProperties m_stiffProps1 = new PlateProperties();
        private double m_stiffLength1 = double.MaxValue;
        private double m_stiffEdge1 = double.MaxValue;
        private double m_stiffWidth1 = double.MaxValue;
        private double m_bottomPlateLength1 = double.MaxValue;
        private double m_skewPartLength1 = double.MaxValue;
        private double m_disToCenterLine1 = double.MaxValue;
        private double m_skewPartVertLength1 = double.MaxValue;

        //加劲肋2
        private PlateProperties m_stiffProps2 = new PlateProperties();
        private double m_stiffLength2 = double.MaxValue;
        private double m_stiffEdge2 = double.MaxValue;
        private double m_stiffWidth2 = double.MaxValue;
        private double m_bottomPlateLength2 = double.MaxValue;
        private double m_skewPartLength2 = double.MaxValue;
        private double m_disToCenterLine2 = double.MaxValue;
        private double m_skewPartVertLength2 = double.MaxValue;

        public StiffenerHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public bool Execute()
        {
            //定义Z轴
            m_axisZ = m_context.BaseGussetPlane.Normal.Normalize();

            m_stiff1 = m_context.Stiffener1;
            m_stiff2 = m_context.Stiffener2;
            if (!ParseProps())
                return false;

            this.CreateStiff1();
            this.CreateStiff2();

            return true;
        }
        private bool ParseProps()
        {
            //节点板属性
            if (m_stiff1.Count < 1 || m_stiff2.Count < 1)
                return false;

            m_plateDistanceToMain = m_paramsHandler.GetDoubleValue(ParamNames.PIC_GUSSET_PLATE_EDGE_GAP_MAIN_PART);
            m_plateDistanceToSub = m_paramsHandler.GetDoubleValue(ParamNames.PIC_GUSSET_PLATE_EDGE_GAP_SECONDARY_PART);

            //加劲肋1属性
            DObject stiff1Props = m_paramsHandler.GetDObjectValue(ParamNames.STIFFENER_PROPS1);
            if (stiff1Props == null)
                return false;
            m_stiffProps1.SetProps(stiff1Props);
            m_stiffLength1 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_LENGTH1);
            var plateLength1 = (m_stiff1[1] - m_stiff1[0]).Length; //节点板底部长度
            m_stiffEdge1 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_EDGE_GUSSET_PLATE_EDGE_DISTANCE1);
            if (m_stiffEdge1 >= plateLength1 && m_stiffEdge1 != double.MaxValue)
                return false;
            if(m_stiffLength1 == double.MaxValue)
            {
                m_stiffLength1 = m_stiffEdge1 == double.MaxValue ? 200.0 : plateLength1 - m_stiffEdge1;
            }

            m_stiffWidth1 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_WIDTH1);

            //自适应加劲肋长度
            m_bottomPlateLength1 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_BASE_LENGTH1);
            m_skewPartLength1 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_SKEW_PART_LENGTH1);
            if(m_bottomPlateLength1 != double.MaxValue && m_skewPartLength1 != double.MaxValue)
                m_stiffLength1 = m_bottomPlateLength1 + m_skewPartLength1;
            if (m_bottomPlateLength1 != double.MaxValue && m_skewPartLength1 == double.MaxValue)
                m_stiffLength1 = Math.Max(m_bottomPlateLength1, m_stiffLength1);
            if(m_bottomPlateLength1 == double.MaxValue)
            {
                m_bottomPlateLength1 =  20.0;
            }

            //自适应距加劲肋中心线的距离
            m_disToCenterLine1 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_CENTER_LINE_DISTANCE1);
            m_skewPartVertLength1 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_BASE_DISTANCE_SKEW_PART1);
            if(m_disToCenterLine1 == double.MaxValue && m_skewPartVertLength1 != double.MaxValue)
            {
                m_disToCenterLine1 = m_stiffWidth1 / 2 - m_skewPartVertLength1;
                m_skewPartVertLength1 = 0.0;
            }
            if(m_disToCenterLine1 == double.MaxValue && m_skewPartVertLength1 == double.MaxValue)
            {
                m_disToCenterLine1 = 20.0;
            }
            if (m_skewPartVertLength1 == double.MaxValue)
                m_skewPartVertLength1 = 0.0;

            //加劲肋2属性
            DObject stiff2Props = m_paramsHandler.GetDObjectValue(ParamNames.STIFFENER_PROPS2);
            if (stiff2Props == null)
                return false;
            m_stiffProps2.SetProps(stiff2Props);
            m_stiffLength2 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_LENGTH2);
            var plateLength2 = (m_stiff2[1] - m_stiff2[0]).Length; //节点板顶部长度
            m_stiffEdge2 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_EDGE_GUSSET_PLATE_EDGE_DISTANCE2);
            if (m_stiffEdge2 >= plateLength2 && m_stiffEdge2 != double.MaxValue)
                return false;
            if (m_stiffLength2 == double.MaxValue)
            {
                m_stiffLength2 = m_stiffEdge2 == double.MaxValue ? 200.0 : plateLength2 - m_stiffEdge2;
            }

            m_stiffWidth2 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_WIDTH2);

            //自适应加劲肋长度
            m_bottomPlateLength2 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_BASE_LENGTH2);
            m_skewPartLength2 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_SKEW_PART_LENGTH2);
            if (m_bottomPlateLength2 != double.MaxValue && m_skewPartLength2 != double.MaxValue)
                m_stiffLength2 = m_bottomPlateLength2 + m_skewPartLength2;
            if (m_bottomPlateLength2 != double.MaxValue && m_skewPartLength2 == double.MaxValue)
                m_stiffLength2 = Math.Max(m_bottomPlateLength2, m_stiffLength2);
            if (m_bottomPlateLength2 == double.MaxValue)
            {
                m_bottomPlateLength2 = 20.0;
            }

            //自适应距加劲肋中心线的距离
            m_disToCenterLine2 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_CENTER_LINE_DISTANCE2);
            m_skewPartVertLength2 = m_paramsHandler.GetDoubleValue(ParamNames.STIFFENER_BASE_DISTANCE_SKEW_PART2);
            if (m_disToCenterLine2 == double.MaxValue && m_skewPartVertLength2 != double.MaxValue)
            {
                m_disToCenterLine2 = m_stiffWidth2 / 2 - m_skewPartVertLength2;
                m_skewPartVertLength2 = 0.0;
            }
            if (m_disToCenterLine2 == double.MaxValue && m_skewPartVertLength2 == double.MaxValue)
            {
                m_disToCenterLine2 = 20.0;
            }
            if (m_skewPartVertLength2 == double.MaxValue)
                m_skewPartVertLength2 = 0.0;

            return true;
        }
        //创建加劲肋1
        private void CreateStiff1()
        {
            if (m_stiffProps1.Thickness <= 0)
                return;
            Vec3 horiOffset1 = (m_stiff1[1] - m_stiff1[0]).Normalize();
            Vec3[] Stiff1Points = new Vec3[6];
            Stiff1Points[0] = m_stiff1[0] + m_axisZ * (m_stiffWidth1 + m_context.Thickness) / 2;
            Stiff1Points[1] = m_stiff1[0] + horiOffset1 * m_bottomPlateLength1
                              + m_axisZ * (m_context.Thickness / 2 + m_stiffWidth1 / 2 + m_skewPartVertLength1);
            double value;
            if(m_skewPartLength1 != double.MaxValue)
                value = m_bottomPlateLength1 + m_skewPartLength1;
            else
                value = m_bottomPlateLength1;
            Stiff1Points[2] = m_stiff1[0] + m_axisZ * (m_context.Thickness / 2 + m_disToCenterLine1)
                              + horiOffset1 * (value > 200 ? 200 : m_stiffLength1);
            //与2对称
            Stiff1Points[3] = Stiff1Points[2] - m_axisZ * 2 * m_disToCenterLine1;
            //与1对称
            Stiff1Points[4] = Stiff1Points[1] - m_axisZ * 2 * (m_skewPartVertLength1 + m_stiffWidth1 / 2);
            //与0对称
            Stiff1Points[5] = Stiff1Points[0] - m_axisZ * m_stiffWidth1;

            //调整深度
            var StiffPoints = AdjustGussetPlateDepth(m_axisZ, Stiff1Points);
            PartBase Stiffener =  m_context.TargetComp.CreateSteelPlate(StiffPoints, m_stiffProps1.Thickness, Define.ObjectSide.SIDE_LEFT, true);
            m_stiffProps1.ApplyToPart(Stiffener);

            //加劲肋与节点板之间的焊缝
            var len = (m_stiff1[2] - m_stiff1[0]).Length;
            m_context.TargetComp.CreatePolyWeld(m_context.RetPlate, Stiffener, new Vec3[] {
                m_stiff1[2],
                m_stiff1[2] + horiOffset1 * (value > 200 ? 200 : m_stiffLength1 - len)
            }, true);

            m_context.TargetComp.CreatePolyWeld(m_context.RetPlate, Stiffener, new Vec3[] {
                m_stiff1[2] + m_axisZ * m_context.Thickness,
                m_stiff1[2] + horiOffset1 * (value > 200 ? 200 : m_stiffLength1 - len) 
                                 + m_axisZ * m_context.Thickness
            }, true);

            //加劲肋与主零件之间的焊缝
            if (m_plateDistanceToMain > 0.0)
                return;
            m_context.TargetComp.CreatePolyWeld(m_context.MainPart, Stiffener, new Vec3[] {
                Stiff1Points[0],
                Stiff1Points[5]
            }, true);
            m_context.TargetComp.CreateAlignPlane(Stiffener, m_context.MainFaceBracePlane, true);
        }
        //创建加劲肋2
        private void CreateStiff2()
        {
            if (m_stiffProps2.Thickness <= 0)
                return;
            Vec3 horiOffset2 = (m_stiff2[1] - m_stiff2[0]).Normalize();
            Vec3[] Stiff2Points = new Vec3[6];
            Stiff2Points[0] = m_stiff2[0] - m_axisZ * (m_stiffWidth2 - m_context.Thickness) / 2;
            Stiff2Points[1] = m_stiff2[0] + horiOffset2 * m_bottomPlateLength2
                              - m_axisZ * (-m_context.Thickness / 2 + m_stiffWidth2 / 2 + m_skewPartVertLength2);

            double value;
            if (m_skewPartLength2 != double.MaxValue)
                value = m_bottomPlateLength2 + m_skewPartLength2;
            else
                value = m_bottomPlateLength2;
            Stiff2Points[2] = m_stiff2[0] - m_axisZ * (-m_context.Thickness / 2 + m_disToCenterLine2)
                              + horiOffset2 * (value > 200 ? 200 : m_stiffLength2);
            //与2对称
            Stiff2Points[3] = Stiff2Points[2] + m_axisZ * 2 * m_disToCenterLine2;
            //与1对称
            Stiff2Points[4] = Stiff2Points[1] + m_axisZ * 2 * (m_stiffWidth2 / 2 + m_skewPartVertLength2);
            //与0对称
            Stiff2Points[5] = Stiff2Points[0] + m_axisZ * m_stiffWidth2;

            //调整深度
            var StiffPoints = AdjustGussetPlateDepth(m_axisZ, Stiff2Points);
            PartBase Stiffener = m_context.TargetComp.CreateSteelPlate(StiffPoints, m_stiffProps2.Thickness, Define.ObjectSide.SIDE_LEFT, true);
            if (Stiffener == null)
                return;
            m_stiffProps2.ApplyToPart(Stiffener);

            //加劲肋与节点板之间的焊缝
            var len = (m_stiff2[2] - m_stiff2[0]).Length;
            m_context.TargetComp.CreatePolyWeld(m_context.RetPlate, Stiffener, new Vec3[] {
                m_stiff2[2],
                m_stiff2[2] + horiOffset2 * (value > 200 ? 200 : m_stiffLength2 - len)
            }, true);

            m_context.TargetComp.CreatePolyWeld(m_context.RetPlate, Stiffener, new Vec3[] {
                m_stiff2[2] + m_axisZ * m_context.Thickness,
                m_stiff2[2] + horiOffset2 * (value > 200 ? 200 : m_stiffLength2 - len)
                                 + m_axisZ * m_context.Thickness
            }, true);

            //加劲肋与次零件之间的焊缝
            if (m_plateDistanceToSub > 0.0)
                return;
            m_context.TargetComp.CreatePolyWeld(m_context.SubPart, m_context.RetPlate, new Vec3[] {
                Stiff2Points[0],
                Stiff2Points[5]
            }, true);
            m_context.TargetComp.CreateAlignPlane(Stiffener, m_context.SubFaceBracePlane, true);
        }
        //调整加劲肋深度
        private Vec3[] AdjustGussetPlateDepth(Vec3 m_axis, Vec3[] points)
        {
            Vec3 axis = m_context.SubFaceBracePlane.Normal.Normalize().Cross(m_context.SubLine);
            if(m_axis.Dot(-axis) > 0)
            {
                return points;
            }
            else
            {
                List<Vec3> list = new List<Vec3>();
                for(int i = points.Length - 1; i >= 0; i--)
                {
                    list.Add(points[i]);
                }
                return list.ToArray();
            }
        }
    }
}
