﻿using System;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using TargetComponentType = LoongCAD.STC.Object3D.CustomComponent;

namespace CustomCompExampl
{
    /// <summary>
    /// 组件上下文类，该类对象用来存储组件公用变量、属性
    /// </summary>
    public class ComponentInstanceContext
    {
        /// <summary>
        /// 细部组件实例
        /// </summary>
        public TargetComponentType TargetComp { get; private set; }
        public ParamsHandler ParamsHandler { get;private set; }


        #region 组件内部自定义 
        public PartBase Part0 { get; private set; }
        public PartBase Part1 { get; private set; }
        public Vec3 CtrlPt { get; private set; }
        #endregion

        public static ComponentInstanceContext CreatInstance(TargetComponentType comp,ParamsHandler handler)
        {
            var compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp,handler)) 
                return compContext;
            compContext.Clear();
            return null;
        }
        
        //具体的组件实例
        private bool Initialize(TargetComponentType comp,ParamsHandler handler)
        {
            if (!InitCustomProps(comp, handler)) {return false;}
            return true;
        }
        private bool InitCustomProps(TargetComponentType comp, ParamsHandler handler)
        {

            if (handler == null) {
                return false;
            }
            ParamsHandler = handler;
            if (comp == null) {
                return false;
            }
            //把传过来的组件，放到上下文
            TargetComp = comp;
            var data = TargetComp.CustomProps;
            
            if (data == null) {
                return false;
            }
            
            if (!data.ContainsKey(CreateStairController.CTRL_COMP_0)) { return false; }
            if (!data.ContainsKey(CreateStairController.CTRL_COMP_1)) { return false; }
            if (TargetComp.CtrlPoints == null) { return false;}
            if (TargetComp.CtrlPoints.Length < 1) { return false;}

            CtrlPt = TargetComp.CtrlPoints[0].WorldPos;

            var id0 = data.GetLong(CreateStairController.CTRL_COMP_0);
            var id1 = data.GetLong(CreateStairController.CTRL_COMP_1);

            var scene = STC.Document.FocusedScene;

            if (scene.GetObject(id0) is PartBase part0) { Part0 = part0; }
            else { return false;}

            if (scene.GetObject(id1) is PartBase part1) { Part1 = part1; }
            else { return false;}

#if DEBUG
            Console.WriteLine("pos: "+ CtrlPt);
            Console.WriteLine("Part0 ProfileName: "+ Part0.ProfileName);
            Console.WriteLine("Part1 ProfileName: "+ Part1.ProfileName);
#endif

            return true;
        }


        private void Clear()
        {
        }

    }
}