﻿using System;
using System.Windows.Forms;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
namespace CustomCompExampl
{
    public class CreateStairController : CustomCompCreateController
    {
        public CreateStairController(ComponentPlugin plugin) : base(plugin)
        {


        }

        /// <summary>
        /// 选中的一号零件
        /// </summary>
        public static readonly string CTRL_COMP_0 = "ctrl_comp_0";
        /// <summary>
        /// 选中的二号零件
        /// </summary>
        public static readonly string CTRL_COMP_1 = "ctrl_comp_1";
        /// <summary>
        /// 选中的控制点
        /// </summary>
        public static readonly string CTRL_POS_0 = "ctrl_pos_0";


        private Vec3 m_pos0;
        private long m_IdComp0;
        private long m_IdComp1;

        private long m_CurId;

        int m_state;


        //携带一个CustomCompCreateInfo

        public override void OnActive(DObject paras)
        {
            base.OnActive(paras);

            this.CreateInfo.UserData = new DObject();
            SwitchStep0();
        }

        private bool GetCurId(long[] ids)
        {
            if (ids?.Length > 0)
            {
                m_CurId = ids[0];
                return true;
            }
            else
            {
                m_CurId = 0;
                return false;
            }
        }


        private void SwitchStep0()
        {
            this.BeginPick(Define.PICK_PART, Define.PICK_PART, Define.PICK_NONE);
            this.SetTip("开始创建：选择第一个零件");
            this.SetCursor(LoongCAD.STC.Define.MouseCursor.CURSOR_HAND);
            m_state = 0;
            m_IdComp0 = 0;
            m_IdComp1 = 0;
        }

        private void SwitchStep1(long id)
        {
            this.BeginPick(Define.PICK_PART, Define.PICK_PART, Define.PICK_NONE);
            this.SetCursor(LoongCAD.STC.Define.MouseCursor.CURSOR_HAND);
            this.SetTip("选择第二个零件");
            m_IdComp0 = id;
            m_IdComp1 = 0;
            m_state = 1;
        }

        private void SwitchStep2(long id)
        {
            this.SetTip("选择一个控制点");
            this.SetCursor(LoongCAD.STC.Define.MouseCursor.CURSOR_CROSS);
            BeginSnapPoint();
            m_IdComp1 = id;
            m_state = 2;
        }

        public override void OnMouseClick(MouseButtons button, int x, int y, bool ctrl, bool alt, bool shift)
        {
            if (m_state == 0)
            {
                var ids = this.GetPickedObjects(ctrl, alt, shift);
                if (GetCurId(ids))
                {
                    SwitchStep1(m_CurId);
                }
            }
            else if (m_state == 1)
            {
                var ids = this.GetPickedObjects(ctrl, alt, shift);
                if (GetCurId(ids))
                {
                    SwitchStep2(m_CurId);
                }
            }
            base.OnMouseClick(button, x, y, ctrl, alt, shift);
        }



        public override bool IsObjectFocused(long id)
        {
            if (id == m_IdComp0 || id == m_IdComp1) { return true;}
            return false;
        }

        public override void OnDeactive()
        {
            m_state = 0;
            m_IdComp0 = 0;
            m_IdComp1 = 0;
            this.CreateInfo.UserData = null;
            base.OnDeactive();
        }

        public override void OnSnapPoint(Vec3 pt)
        {
            base.OnSnapPoint(pt);
            if(m_state == 2)
            {
                m_pos0 = pt;
                this.CreateInfo.UserData = new DObject();
                this.CreateInfo.UserData[CTRL_POS_0] = m_pos0;
                //获取选中的零件
                this.CreateInfo.UserData[CTRL_COMP_0] = m_IdComp0;
                this.CreateInfo.UserData[CTRL_COMP_1] = m_IdComp1;
                m_state = 0;
                this.CompleteCreation();
                this.Quit();
            }
        }


        public override void OnSnapRevert()
        {
            if(m_state == 1)
            {
                SwitchStep0();
            }else if (m_state == 2)
            {
                SwitchStep1(m_IdComp0);
            }
            base.OnSnapRevert();
        }

    }
}
