﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.Util;
namespace CustomCompExampl
{
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_CustomComponentProps
    {
        private PluginMain m_plugin;
        public PluginMain Plugin { get => m_plugin; set => m_plugin = value; }
        private DObject m_weldPropsBatch = new DObject();
        
        private DObject m_rootUIProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        public override ComponentPlugin ReferencePlugin => m_refPlugin;
        
        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostInitialize();
            DisposeOnClose = false;
        }

        private void OnPostInitialize()
        {
            //配置UI面板的自动缩放
            foreach (TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }
        }
        
        #region 辅助类函数
        private void AddItems(KeyValueComboBox comboBox,List<string> msg)
        {
            comboBox.ClearItems();
            for(var i = 0; i < msg.Count; ++i)
            {
                comboBox.AddItem(i, msg[i]);
            }
            comboBox.UpdateItems();
        }
        private void AddItems(ImageComboBox imageComboBox,int num)
        {
            imageComboBox.ClearItems();
            for(var i = 0; i< num; ++i) {
                imageComboBox.AddItem(i,i);
            }
            imageComboBox.UpdateItems();
        }
        #endregion

        #region 焊缝相关 
        private Form_WeldParameter m_weldForm = null;
        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null) {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }
        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_rootUIProps.CopyFrom(srcProps);
        }
        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }
        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_rootUIProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_rootUIProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                       ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_rootUIProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_rootUIProps;

            return comp.UIProps;
        }
        private void btn_weld_props_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_rootUIProps, null);
        }

        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked) {
                retProps.CopyFrom(m_rootUIProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }

        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            //base.UpdatePlaceholders(compProps, defaultValues);
            this.SetPlaceholders(defaultValues);
        }
        #endregion

    }
}
