﻿using System;
using LoongCAD.UI.Component;
using LoongCAD.Util;
namespace CustomCompExampl
{
    public class ParamsHandler : LoongCAD.STC.Plugins.ParamsHandlerBase
    {
        private static readonly string NoStartPostfix = "1";
        /// <summary>
        /// 属性空值全局总表
        /// </summary>
        /// <remarks><b>此静态变量可直接复制到其他组件代码中使用</b></remarks>
        protected static DObject g_emptyValues = null;

        /// <summary>
        /// 属性内置默认值全局总表
        /// </summary>
        /// <remarks><b>此静态变量可直接复制到其他组件代码中使用</b></remarks>
        protected static DObject g_builtinDefaults = null;
        
        /// <summary>
        /// 默认构造函数 
        /// </summary>
        public ParamsHandler()
        :base(EmptyValues,BuiltinDefaults)
        {}

        private static void AddEmptyDefaultValue(string paramName, object emptyValue, object defaultValue)
        {
            AddEmptyDefaultValue(g_emptyValues,g_builtinDefaults,paramName,emptyValue,defaultValue);
        }

        /// <summary>
        /// 初始化组件所有实例公用的属性空值、属性内置默认值
        /// </summary>
        /// <remarks><b>每个组件需要分别实现此方法</b></remarks>
        private static void InitializeEmptyDefaults()
        {
            g_emptyValues = new DObject();
            g_builtinDefaults = new DObject();

            #region 焊缝 
            for(int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                AddEmptyDefaultValue(ParamNames.WELD_PROPS_LIST[i], WeldPropertyPanel.EmptyValue, new DObject
                {
                    { WeldPropertyPanel.ParamNames.SizeUp, 6.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeUp, 11 }, // Fillet
                    { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                    { WeldPropertyPanel.ParamNames.SizeDown, 6.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeDown, 11 }, // Fillet
                    { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                    { WeldPropertyPanel.ParamNames.EdgeAround, 1 }, // Edge
                    { WeldPropertyPanel.ParamNames.MadeType, 2 }, // Workshop
                    { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                    { WeldPropertyPanel.ParamNames.RefText, "" }
                });
            }
            #endregion

        }

        /// <summary>
        /// 组件公用属性空值表
        /// </summary>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public static DObject EmptyValues
        {
            get
            {
                if(g_emptyValues == null)
                {
                    InitializeEmptyDefaults();
                }
                return g_emptyValues;
            }
        }

        /// <summary>
        /// 组件公用内置默认值接口
        /// </summary>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks>
        public static DObject BuiltinDefaults
        {
            get
            {
                if (g_builtinDefaults == null)
                {
                    InitializeEmptyDefaults();
                }
                return g_builtinDefaults;
            }
        }

        /// <summary>
        /// 初始化默认值
        /// </summary>
        /// <param name="defaultConfig">从默认值配置加载的默认值</param>
        /// <remarks><b>此方法可直接复制到其他组件代码中使用</b></remarks> 
        public override DObject InitializeDefaultValues(DObject defaultConfig)
        {
            m_defaultValues = new DObject();
            m_defaultValues.CopyFrom(defaultConfig);

            var emptyValues = ParamsHandler.EmptyValues;
            var builtinDefaults = ParamsHandler.BuiltinDefaults;
            if(emptyValues == null || builtinDefaults == null)
            {
                Console.WriteLine(@"ParamsHandler.InitializeDefaultValues:[Warning] EmptyValue/BuiltinDefault is null!");
                return m_defaultValues;
            }
            this.TryCompleteDObjectValues(m_defaultValues, builtinDefaults, emptyValues);
            return m_defaultValues;
        }

    }
}
