﻿using System;
using System.Windows.Forms;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using TargetComponentType = LoongCAD.STC.Object3D.CustomComponent;

namespace CustomCompExampl
{
    public class PluginMain : CustomCompPlugin
    {
        /// <summary>
        /// 用于开启组件定义基于代码，而非基于软件
        /// </summary>
        public override bool StandAlone => true;
        
        public override Guid Guid => new Guid("9c920083-ec6f-4143-9a97-b92167b99598");
        public override int CompNumber => 0;

        public override string CompName => "示例";

        public override string CompKeyword => "示例custom";

        public override string CompDescription => "";

        public override string CompShortDescription => "";

        public override string CompGroup => "";

        public override System.Drawing.Image Thumbnail => null;

        public override System.Drawing.Image ButtonImage => null;

        public override System.Drawing.Image SelectedButtonImage => null;

        public override Controller CreateController => new CreateStairController(this);


        public PluginMain() { }

        private Form_PluginMain m_propsForm = null;
        private Form_PluginMain m_defaultForm;

        public override bool OnInstanceCreate(ComponentBase instObj, CompCreateInfo cinfo)
        {
            //处理相对坐标
            var data = cinfo.UserData;
            //设置组件图标位置
            var ctrlPos0 = data.GetVec3(CreateStairController.CTRL_POS_0);
            instObj.ObjectWorldMatrix = new Mat43(Vec3.AXIS_X, Vec3.AXIS_Y, Vec3.AXIS_Z, ctrlPos0); 
            instObj.SetWorldCtrlPoints(new []{ctrlPos0});
            instObj.CustomProps[CreateStairController.CTRL_COMP_0] = data.GetLong(CreateStairController.CTRL_COMP_0);
            instObj.CustomProps[CreateStairController.CTRL_COMP_1] = data.GetLong(CreateStairController.CTRL_COMP_1);
            
            return base.OnInstanceCreate(instObj, cinfo);
        }

        public override bool OnInstanceRun(ComponentBase compInst, long defaultId)
        {
            if (!(compInst is TargetComponentType inst))
                return false;
            
            //1. 获取组件实例的属性
            DObject compProps = inst.UIProps;
            if (compProps == null)
            {
                compProps = new DObject();
            }
            //2. 初始化参数处理器，此过程包含以下操作：
            //* 初始化参数值
            //* 初始化参数空值表
            //* 初始化参数内置默认值表
            //* 组合参数默认值
            //其他代码可以直接复制该段代码
            ParamsHandler paramsHandler = new ParamsHandler();
            paramsHandler.ParamValues = compProps;
            paramsHandler.InitializeDefaultValues(this.GetDefaultsData(defaultId));
            
            var compContext = ComponentInstanceContext.CreatInstance(inst,paramsHandler);
            if (compContext == null) {
                return false;
            }

            return true;
        }

        
        public override bool OnShowInstancePropsForm(ComponentBase instObj, long defId, Form ownerForm)
        {
            if (instObj == null)
                return false;
            if (m_propsForm == null) {
                m_propsForm = new Form_PluginMain(this);
                m_propsForm.Owner = ownerForm;
            }

            m_propsForm.ShowInstanceProps(instObj,
                new ParamsHandler().InitializeDefaultValues(this.GetDefaultsData(defId)));
            return true;
        }


        /// <summary>
        /// 显示组件的属性窗，必须实现此方法
        /// </summary>
        /// <param name="ownerForm">组件窗口所属主窗口</param>
        /// <returns>是否显示成功</returns>
        /// <remarks><b>其他组件可直接复制此方法</b></remarks>
        public override bool OnShowPluginPropsForm(Form ownerForm)
        {
            if (m_propsForm == null) {
                m_propsForm = new Form_PluginMain(this);
                m_propsForm.Owner = ownerForm;
            }

            m_propsForm.ShowPluginProps();
            return true;
        }
        
    }
}
