﻿using System.Linq;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Geometry;
using System;

namespace DiagonalJoint53
{
    public class ComponentInstanceContext
    {
        ///<summary>
        /// 目标组件的类型
        ///</summary>
        private ConnectionComponent m_comp;

        private Beam m_mainPart = null;

        private Vec3 m_mainLine;

        private BeamPlanes m_mainPlanes;

        private Plane m_mainFrontPlane;

        private Plane m_mainBackPlane;

        private Plane m_mainLeftPlane;

        private Plane m_mainRightPlane;

        private Vec3 m_axisX;

        private Vec3 m_axisY;

        private Vec3 m_axisZ;
        //开发组件需要的上下文
        public ConnectionComponent TargetComp { get => m_comp; set => m_comp = value; }
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public Vec3 MainLine { get => m_mainLine; set => m_mainLine = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public Plane MainFrontPlane { get => m_mainFrontPlane; set => m_mainFrontPlane = value; }
        public Plane MainBackPlane { get => m_mainBackPlane; set => m_mainBackPlane = value; }
        public Plane MainLeftPlane { get => m_mainLeftPlane; set => m_mainLeftPlane = value; }
        public Plane MainRightPlane { get => m_mainRightPlane; set => m_mainRightPlane = value; }
        public SubData UpSub { get; set; }
        public SubData DownSub { get; set; }
        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp))
                return compContext;
            compContext.Clear();
            return null;
        }
        private bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;
            Beam mainPart = comp.Main as Beam;
            if (mainPart == null || comp.Subs == null || comp.Subs.Length != 2)
                return false;
            if (!PrepareMainLine(mainPart))
                return false;

            m_comp = comp;
            m_mainPart = mainPart;
            m_mainPlanes = BeamPlanes.CreateFromBeam(mainPart);

            if (!PrepareMainSidePlanes(m_mainPlanes))
                return false;

            return true;
        }
        /// <summary>
        /// 准备主零件由起点到终点的线
        /// </summary>
        private bool PrepareMainLine(Beam mainPart)
        {
            var line = mainPart.CtrlPoints?.Last()?.WorldPos -
                       mainPart.CtrlPoints?.First()?.WorldPos;
            if (!line.HasValue)
                return false;
            m_mainLine = line.Value.Normalize();
            return true;
        }
        /// <summary>
        /// 准备主零件的侧面
        /// </summary>
        private bool PrepareMainSidePlanes(BeamPlanes mainPlanes)
        {
            if (mainPlanes == null)
                return false;
            //建立坐标系
            m_axisY = m_mainLine;
            m_axisX = m_comp.LocalMatrix.X;
            m_axisZ = m_axisX.Cross(m_axisY);

            var planes = new Plane[]
            {
                mainPlanes.TopBound,
                mainPlanes.BackBound,
                mainPlanes.BottomBound,
                mainPlanes.FrontBound
            };

            var dot1 = Math.Abs(planes[0].Normal.Normalize().Dot(m_axisZ));
            var dot2 = Math.Abs(planes[1].Normal.Normalize().Dot(m_axisZ));
            bool flag = false;
            if (dot1 > dot2)
                flag = true;

            Plane[] Zplanes, XPlanes;
            switch (flag)
            {
                case true:
                    Zplanes = ParsePlanes(planes[0], planes[2], m_axisZ);
                    XPlanes = ParsePlanes(planes[1], planes[3], m_axisX);
                    break;
                default:
                    Zplanes = ParsePlanes(planes[1], planes[3], m_axisZ);
                    XPlanes = ParsePlanes(planes[0], planes[2], m_axisX);
                    break;
            }
            m_mainFrontPlane = Zplanes[0];
            m_mainBackPlane = Zplanes[1];
            m_mainLeftPlane = XPlanes[0];
            m_mainRightPlane = XPlanes[1];
            return true;
        }
        private Plane[] ParsePlanes(Plane plane1, Plane plane2, Vec3 line)
        {
            if (plane1.Normal.Dot(line) > 0)
                return new Plane[] { plane1, plane2 };
            return new Plane[] { plane2, plane1 };
        }
        private void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_mainPlanes = null;
            m_mainBackPlane = null;
            m_mainFrontPlane = null;
            m_mainLeftPlane = null;
            m_mainRightPlane = null;
        }
    }
}
