﻿using System;
using LoongCAD.Util;
using LoongCAD.UI.Component;

namespace DiagonalJoint53
{
    public class ParamsHandler : LoongCAD.STC.Plugins.ParamsHandlerBase
    {
        /// <summary>
        /// 属性空值全局总表
        /// </summary>
        protected static DObject g_emptyValues = null;

        /// <summary>
        /// 属性内置默认值全局总表
        /// </summary>
        protected static DObject g_builtinDefaults = null;

        /// <summary>
        /// 默认构造函数
        /// </summary>
        /// <remarks><b>此方法需要复制到其他组件代码中使用</b></remarks>
        public ParamsHandler()
            : base(ParamsHandler.EmptyValues, ParamsHandler.BuiltinDefautls)
        {

        }
        /// <summary>
        /// 组件公用属性空值表
        /// </summary>
        public static DObject EmptyValues
        {
            get
            {
                if (g_emptyValues == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_emptyValues;
            }
        }
        /// <summary>
        /// 组件公用内置默认值接口
        /// </summary>
        public static DObject BuiltinDefautls
        {
            get
            {
                if (g_builtinDefaults == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_builtinDefaults;
            }
        }
        /// <summary>
        /// 初始化默认值
        /// </summary>
        /// <param name="defaultConfig">从默认值配置加载的默认值</param>
        public override LoongCAD.Util.DObject InitializeDefaultValues(DObject defaultConfig)
        {
            m_defaultValues = new DObject();
            m_defaultValues.CopyFrom(defaultConfig);

            var emptyValues = ParamsHandler.EmptyValues;
            var builtinDefaults = ParamsHandler.BuiltinDefautls;
            if (emptyValues == null || builtinDefaults == null)
            {
                Console.WriteLine("ParamsHandler.InitializeDefaultValues: [Warning] EmptyValue/BuiltinDefaults is null!");
                return m_defaultValues;
            }

            this.TryCompleteDObjectValues(m_defaultValues, builtinDefaults, emptyValues);
            return m_defaultValues;
        }
        /// <summary>
        /// 初始化组件所有实例公用的属性空值、属性内置默认值
        /// </summary>
        public static void IntializeEmptyDefaults()
        {
            g_emptyValues = new DObject();
            g_builtinDefaults = new DObject();

            //图形页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_DIST_PLATE_UP_EDGE_INTERSECTION, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_DIST_PLATE_DOWN_EDGE_INTERSECTION, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_UP_PARTS_ALIGN_DIST, double.MaxValue, double.MaxValue); //auto
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_DOWN_PARTS_ALIGN_DIST, double.MaxValue, double.MaxValue); //auto
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_PLATE_TOLERANCE, double.MaxValue, 10.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_UP_GAP, double.MaxValue, 20.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_DOWN_GAP, double.MaxValue, 20.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_PLATE_LENGTH_CONTROL, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_PLATE_CHAMFER_CONTROL, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_LEFT_UP_CHAMFER, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_LEFT_DOWN_CHAMFER, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_RIFHT_UP_CHAMFER, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_RIGHT_DOWN_CHAMFER, double.MaxValue, double.MaxValue);

            //零件页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PARTS_CONNECT_PLATE_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 20.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "板" 
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PARTS_SPACER_PLATE_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 10.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "板"
            });

            //参数页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PARAM_PLATE_POSITION, 0, 0);

            //螺栓页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SUB_BASIC_PROPS, BoltBasicPropertyPanel.EmptyValue, (DObject)new object[] {
                BoltBasicPropertyPanel.ParamNames.BoltStandard, "TS10.9",
                BoltBasicPropertyPanel.ParamNames.BoltSize, 20.0,
                BoltBasicPropertyPanel.ParamNames.ThreadInMtrl, 2, // true
                BoltBasicPropertyPanel.ParamNames.HoleTol, double.MaxValue,
                BoltBasicPropertyPanel.ParamNames.MadeType0, 0, // Workshop
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_MAIN_BASIC_PROPS, BoltBasicPropertyPanel.EmptyValue, (DObject)new object[] {
                BoltBasicPropertyPanel.ParamNames.BoltStandard, "TS10.9",
                BoltBasicPropertyPanel.ParamNames.BoltSize, 20.0,
                BoltBasicPropertyPanel.ParamNames.ThreadInMtrl, 2, // true
                BoltBasicPropertyPanel.ParamNames.HoleTol, double.MaxValue,
                BoltBasicPropertyPanel.ParamNames.MadeType0, 0, // Workshop
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HOLE_PROPS, BoltHolePanel.EmptyValue, (DObject)new object[] {
                BoltHolePanel.ParamNames.HoleX, 0.0,
                BoltHolePanel.ParamNames.HoleY, 0.0,
                BoltHolePanel.ParamNames.HoleType, 1,
                BoltHolePanel.ParamNames.RotSlot, 3,
                BoltHolePanel.ParamNames.SlotIn, 4
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_CONTENT_PROPS, null, (DObject)new object[]
            {
                BoltContentPanel.ParamNames.PartWasherUp, false,
                BoltContentPanel.ParamNames.PartWasherDown0, false,
                BoltContentPanel.ParamNames.PartWasherDown1, true,
                BoltContentPanel.ParamNames.PartNut0, true,
                BoltContentPanel.ParamNames.PartNut1, false,
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_EXTEND_PROPS, BoltExtendPanel.EmptyValue, (DObject)new object[]
            {
                BoltExtendPanel.ParamNames.ExtLength, 0.0,
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SHAPE, 0, 0); // Array
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SUB_HORI_COUNT, int.MaxValue, 3);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SUB_HORI_STRIDE, new double[] { }, new double[] { 120.0 });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SUB_VERT_COUNT, int.MaxValue, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SUB_VERT_STRIDE, new double[] { }, new double[] { 120.0 });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_ALIGN_DIST, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_PLATE_DIST, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SUB_OFFSET, double.MaxValue, 0.0);

            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_MAIN_HORI_COUNT, int.MaxValue, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_MAIN_HORI_STRIDE, new double[] { }, new double[] { 120.0 });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_MAIN_VERT_COUNT, int.MaxValue, 3);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_MAIN_VERT_STRIDE, new double[] { }, new double[] { 120.0 });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HORI_OFFSET, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_OFFSET, double.MaxValue, 0.0);
        }
    }
}
