﻿using System.Windows.Forms;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;

namespace DiagonalJoint53
{
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {
        private DObject m_rootUIProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostInitialize();
        }
        public override ComponentPlugin ReferencePlugin => m_refPlugin;
        private void OnPostInitialize()
        {
            // 设置窗口标题
            this.Text = "支撑交叉节点(304)";

            //初始化图片选项卡
            this.cbo_plate_position.ClearItems();
            this.cbo_plate_position.AddIntegerItems(0, 0, 4);
            this.cbo_plate_position.UpdateItems();

            //初始化文字选项卡
            this.cbo_plate_length_control.ClearItems();
            this.cbo_plate_length_control.AddItem(0, "默认");
            this.cbo_plate_length_control.AddItem(1, "斜长");
            this.cbo_plate_length_control.AddItem(2, "间隙");
            this.cbo_plate_length_control.UpdateItems();
            this.cbo_plate_length_control.SelectedKey = 0;

            this.cbo_plate_chamfer_control.ClearItems();
            this.cbo_plate_chamfer_control.AddItem(0, "默认");
            this.cbo_plate_chamfer_control.AddItem(1, "折点");
            this.cbo_plate_chamfer_control.AddItem(2, "折角");
            this.cbo_plate_chamfer_control.UpdateItems();
            this.cbo_plate_chamfer_control.SelectedKey = 0;

            // 螺栓标准变化之后调用螺栓内容的自适应方法
            this.panel_sub_basic_props.BoltStandardChanged += this.boltContentPanel.AdaptBoltContents;

            foreach (TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }
        }
        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null)
            {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }
        /// <summary>
        /// 当前组件需要重写此方法，并刷新螺栓基础面板
        /// </summary>
        /// <param name="root"></param>
        public override void ClearUI(Control root = null)
        {
            // 刷新螺栓基础面板
            this.panel_main_basic_props.RefreshControls();
            this.panel_sub_basic_props.RefreshControls();
            base.ClearUI(root);
        }
        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }

        private void cbo_plate_length_control_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            bool flag1, flag2;
            if(cbo_plate_length_control.SelectedIndex == 2)
            {
                flag1 = false;
                flag2 = true;
            }
            else
            {
                flag1 = true;
                flag2 = false;
            }
            act_tb_pic_up_parts_align_dist.Enabled = flag1;
            tb_pic_up_parts_align_dist.Enabled = flag1;
            act_pic_down_parts_align_dist.Enabled = flag1;
            tb_pic_down_parts_align_dist.Enabled = flag1;

            act_up_gap.Enabled = flag2;
            tb_up_gap.Enabled = flag2;
            act_down_gap.Enabled = flag2;
            tb_down_gap.Enabled = flag2;
        }

        private void cbo_plate_chamfer_control_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            bool flag1, flag2;
            if (cbo_plate_chamfer_control.SelectedIndex == 2)
            {
                flag1 = false;
                flag2 = true;
            }
            else
            {
                flag1 = true;
                flag2 = false;
            }

            act_pic_dist_plate_up_edge_intersection.Enabled = flag1;
            tb_pic_dist_plate_up_edge_intersection.Enabled = flag1;
            act_pic_dist_plate_down_edge_intersection.Enabled = flag1;
            tb_dist_plate_down_edge_intersection.Enabled = flag1;

            act_leftDown_chamfer.Enabled = flag2;
            tb_leftDown_chamfer.Enabled = flag2;
            act_leftUp_chamfer.Enabled = flag2;
            tb_leftUp_chamfer.Enabled = flag2;
            act_rightDown_chamfer.Enabled = flag2;
            tb_rightDown_chamfer.Enabled = flag2;
            act_rightUp_chamfer.Enabled = flag2;
            tb_rightUp_chamfer.Enabled = flag2;
        }
    }
}
