﻿using System.Collections.Generic;
using System.Linq;

namespace DiagonalJoint53
{
    public class BoltParse
    {
        private ParamsHandler m_paramsHandler = null;
        private BoltData m_boltData = new BoltData();

        public BoltParse(ParamsHandler paramsHandler)
        {
            m_paramsHandler = paramsHandler;
        }
        public BoltData GetBoltData()
        {
            if (!InitParamData())
                return null;
            return m_boltData;
        }
        //螺栓属性
        private bool InitParamData()
        {
            m_boltData.BoltsCfg.CopyFromBasicProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_SUB_BASIC_PROPS));
            m_boltData.BoltsCfg.CopyFromHoleProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_HOLE_PROPS));
            m_boltData.BoltsCfg.CopyFromContentProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_CONTENT_PROPS));
            m_boltData.BoltsCfg.CopyFromExtendProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_EXTEND_PROPS));
            m_boltData.BoltsCfg.CopyFromShapeProps
                (m_paramsHandler.GetIntValue(ParamNames.BOLT_SHAPE));

            m_boltData.SubHoriBoltsNum = m_paramsHandler.GetIntValue(ParamNames.BOLT_SUB_HORI_COUNT);
            m_boltData.SubVertBoltsNum = m_paramsHandler.GetIntValue(ParamNames.BOLT_SUB_VERT_COUNT);
            var subArrayX = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_SUB_HORI_STRIDE);
            var subArrayY = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_SUB_VERT_STRIDE);
            m_boltData.SubHoriBoltsStride = HandlerSpacing(subArrayX, m_boltData.SubHoriBoltsNum);
            m_boltData.SubVertBoltsStride = HandlerSpacing(subArrayY, m_boltData.SubVertBoltsNum);
            m_boltData.AlignDist = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_ALIGN_DIST);
            m_boltData.PlateDist = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_PLATE_DIST);
            m_boltData.SubOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_SUB_OFFSET);

            m_boltData.MainHoriBoltsNum = m_paramsHandler.GetIntValue(ParamNames.BOLT_MAIN_HORI_COUNT);
            m_boltData.MainVertBoltsNum = m_paramsHandler.GetIntValue(ParamNames.BOLT_MAIN_VERT_COUNT);
            var mainArrayX = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_MAIN_HORI_STRIDE);
            var mainArrayY = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_MAIN_VERT_STRIDE);
            m_boltData.MainHoriBoltsStride = HandlerSpacing(mainArrayX, m_boltData.MainHoriBoltsNum);
            m_boltData.MainVertBoltsStride = HandlerSpacing(mainArrayY, m_boltData.MainVertBoltsNum);
            m_boltData.HoriOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_HORI_OFFSET);
            m_boltData.VertOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_VERT_OFFSET);

            return true;
        }
        /// <summary>
        /// 根据螺栓数目调整螺栓间距的设置
        /// </summary>
        private double[] HandlerSpacing(double[] arr, int num)
        {
            if (num <= 0)
                return null;
            if (num == 1)
                return new double[] { 0 };
            var ret = new List<double>();
            if (arr.Length == 0)
                ret.AddRange(Enumerable.Repeat(120.0, num - 1));
            else
            {
                for (int i = 0; i < num - 1; i++)
                {
                    if (i > arr.Length - 1)
                        ret.Add(arr[arr.Length - 1]);
                    else
                        ret.Add(arr[i]);
                }
            }
            return ret.ToArray();
        }
    }
}
