﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.STC;
using System;

namespace DiagonalJoint53
{
    public class GraphicHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;

        private Plane m_mainFrontPlane;
        private Plane m_mainBackPlane;
        private Plane m_mainLeftPlane;
        private Plane m_mainRightPlane;
        private SubData m_upSub;
        private SubData m_downSub;

        private Vec3 m_axisX = Vec3.ZERO;
        private Vec3 m_axisY = Vec3.ZERO;
        private Vec3 m_axisZ = Vec3.ZERO;
        private Plane upAlignPlane;
        private Plane downAlignPlane;
        private PartBase frontConnectPlate;
        private PartBase backConnectPlate;
        //参数页
        private int m_platePos = 0;
        //零件页
        private PlateProperties m_ConnectPlateProps = new PlateProperties();
        private PlateProperties m_spacerPlateProps = new PlateProperties();
        //图形页
        private double m_upAlignLen = double.MaxValue;
        private double m_upPointLen = double.MaxValue;
        private double m_downAlignLen = double.MaxValue;
        private double m_downPointLen = double.MaxValue;
        private double m_plateHol = double.MaxValue;

        private double m_upGap = double.MaxValue;
        private double m_downGap = double.MaxValue;
        private int m_plateLenCtrl = 0;

        private int m_chamferCtrl = 0;
        private double m_leftUpChamfer = double.MaxValue;
        private double m_leftDownChamfer = double.MaxValue;
        private double m_rightUpChamfer = double.MaxValue;
        private double m_rightDownChamfer = double.MaxValue;

        //螺栓页数据
        private BoltData m_boltData = new BoltData();
        private double boltLen = double.MaxValue;

        public GraphicHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public bool Execute()
        {
            if (!ParseProps())
                return false;
            this.InitParams();
            if (!InitBoltData())
                return false;

            //构建连接板和隔板
            if (!ParseAlignPlane())
                return false;
            this.AdaptPlatePos();

            m_context.TargetComp.CreateAlignPlane(m_upSub.Beam, upAlignPlane, true);
            m_context.TargetComp.CreateAlignPlane(m_downSub.Beam, downAlignPlane, true);

            return true;
        }
        private bool ParseProps()
        {
            DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.PARTS_CONNECT_PLATE_PROPS);
            if (plateProps == null)
                return false;
            m_ConnectPlateProps.SetProps(plateProps);
            DObject props = m_paramsHandler.GetDObjectValue(ParamNames.PARTS_SPACER_PLATE_PROPS);
            if (props == null)
                return false;
            m_spacerPlateProps.SetProps(props);
            //板厚度不为零
            if (m_ConnectPlateProps.Thickness == 0)
                return false;

            m_platePos = m_paramsHandler.GetIntValue(ParamNames.PARAM_PLATE_POSITION);
            m_upAlignLen = m_paramsHandler.GetDoubleValue(ParamNames.PIC_UP_PARTS_ALIGN_DIST);
            m_upPointLen = m_paramsHandler.GetDoubleValue(ParamNames.PIC_DIST_PLATE_UP_EDGE_INTERSECTION);
            m_downAlignLen = m_paramsHandler.GetDoubleValue(ParamNames.PIC_DOWN_PARTS_ALIGN_DIST);
            m_downPointLen = m_paramsHandler.GetDoubleValue(ParamNames.PIC_DIST_PLATE_DOWN_EDGE_INTERSECTION);
            m_plateHol = m_paramsHandler.GetDoubleValue(ParamNames.PIC_PLATE_TOLERANCE);

            m_upGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_UP_GAP);
            m_downGap = m_paramsHandler.GetDoubleValue(ParamNames.PIC_DOWN_GAP);
            m_plateLenCtrl = m_paramsHandler.GetIntValue(ParamNames.PIC_PLATE_LENGTH_CONTROL);

            m_chamferCtrl = m_paramsHandler.GetIntValue(ParamNames.PIC_PLATE_CHAMFER_CONTROL);
            m_leftUpChamfer = m_paramsHandler.GetDoubleValue(ParamNames.PIC_LEFT_UP_CHAMFER);
            m_leftDownChamfer = m_paramsHandler.GetDoubleValue(ParamNames.PIC_LEFT_DOWN_CHAMFER);
            m_rightUpChamfer = m_paramsHandler.GetDoubleValue(ParamNames.PIC_RIFHT_UP_CHAMFER);
            m_rightDownChamfer = m_paramsHandler.GetDoubleValue(ParamNames.PIC_RIGHT_DOWN_CHAMFER);

            return true;
        }
        private void InitParams()
        {
            //坐标系
            m_axisY = m_context.MainLine;
            m_axisX = m_context.TargetComp.LocalMatrix.X;
            m_axisZ = m_axisX.Cross(m_axisY);

            m_mainFrontPlane = m_context.MainFrontPlane;
            m_mainBackPlane = m_context.MainBackPlane;
            m_mainLeftPlane = m_context.MainLeftPlane;
            m_mainRightPlane = m_context.MainRightPlane;
            m_upSub = m_context.UpSub;
            m_downSub = m_context.DownSub;
        }
        private bool ParseAlignPlane()
        {
            var dist = m_mainLeftPlane.Dist(m_mainRightPlane.Origin);
            if (m_upAlignLen == double.MaxValue || m_plateLenCtrl == 2)
            {
                var topPoint = m_mainLeftPlane.Move(m_upGap).Intersect(m_upSub.TopPlane, m_upSub.FrontPlane);
                var bottomPoint = m_mainLeftPlane.Move(m_upGap).Intersect(m_upSub.BottomPlane, m_upSub.FrontPlane);
                upAlignPlane = PartsTools.GetDistMinPlane(m_upSub.FarPoint,
                    new Plane(topPoint, m_upSub.BraceLine),
                    new Plane(bottomPoint, m_upSub.BraceLine)
                );
            }
            else
            {
                var len = m_upSub.TopPlane.Dist(m_upSub.BottomPlane.Origin);
                var point = m_upSub.TopPlane.Move(-len / 2)
                    .Intersect(m_mainFrontPlane, m_mainLeftPlane.Move(-dist / 2));
                upAlignPlane = new Plane(point, m_upSub.BraceLine).Move(m_upAlignLen);
            }

            if (m_downAlignLen == double.MaxValue || m_plateLenCtrl == 2)
            {
                var topPoint = m_mainRightPlane.Move(m_downGap).Intersect(m_downSub.TopPlane, m_downSub.FrontPlane);
                var bottomPoint = m_mainRightPlane.Move(m_downGap).Intersect(m_downSub.BottomPlane, m_downSub.FrontPlane);
                downAlignPlane = PartsTools.GetDistMinPlane(m_downSub.FarPoint,
                    new Plane(topPoint, m_downSub.BraceLine),
                    new Plane(bottomPoint, m_downSub.BraceLine)
                );
            }
            else
            {
                var len = m_downSub.TopPlane.Dist(m_downSub.BottomPlane.Origin);
                var point = m_downSub.TopPlane.Move(-len / 2)
                    .Intersect(m_mainFrontPlane, m_mainLeftPlane.Move(-dist / 2));
                downAlignPlane = new Plane(point, m_downSub.BraceLine).Move(m_downAlignLen);
            }

            return true;
        }
        private void AdaptPlatePos()
        {
            switch (m_platePos)
            {
                case 1:
                    CreateFrontPlate();
                    CreateSubBolts(m_mainFrontPlane, frontConnectPlate);
                    CreateMainBolts(m_mainFrontPlane, frontConnectPlate);
                    break;
                case 3:
                    CreateBackPlate();
                    CreateSubBolts(m_mainBackPlane, backConnectPlate);
                    CreateMainBolts(m_mainBackPlane, backConnectPlate);
                    break;
                case 0:
                default:
                    CreateBackPlate();
                    CreateSubBolts(m_mainBackPlane, backConnectPlate);
                    CreateMainBolts(m_mainBackPlane, backConnectPlate);

                    CreateFrontPlate();
                    CreateSubBolts(m_mainFrontPlane, frontConnectPlate);
                    CreateMainBolts(m_mainFrontPlane, frontConnectPlate);
                    break;
            }
        }
        private Vec3[] GetCtrlPoints(Plane plane)
        {
            var points = new List<Vec3>();
            //计算控制点
            var bottomPoint1 = m_upSub.BottomPlane.Move(m_plateHol)
                .Intersect(plane, upAlignPlane.Move(boltLen));
            var topPoint1 = m_upSub.TopPlane.Move(m_plateHol)
                .Intersect(plane, upAlignPlane.Move(boltLen));
            var topPoint2 = m_downSub.TopPlane.Move(m_plateHol)
                .Intersect(plane, downAlignPlane.Move(boltLen));
            var bottomPoint2 = m_downSub.BottomPlane.Move(m_plateHol)
                .Intersect(plane, downAlignPlane.Move(boltLen));

            points.Add(bottomPoint1);
            points.Add(topPoint1);
            if(m_chamferCtrl == 2)
            {
                if(m_leftUpChamfer != double.MaxValue)
                {
                    var leftTopLine = m_upSub.TopPlane.Normal.Normalize();
                    var upTopLine = PartsTools.RotateVectorInPlane(leftTopLine,
                        leftTopLine.Cross(m_upSub.BraceLine.Normalize()), m_leftUpChamfer);
                    var upTopPlane = new Plane(topPoint1, upTopLine);
                    points.Add(upTopPlane.Intersect(plane, m_mainLeftPlane));
                }
                
                if(m_rightUpChamfer != double.MaxValue)
                {
                    var rightTopLine = m_downSub.TopPlane.Normal.Normalize();
                    var downTopLine = PartsTools.RotateVectorInPlane(rightTopLine,
                        rightTopLine.Cross(m_downSub.BraceLine.Normalize()), m_rightUpChamfer);
                    var downTopPlane = new Plane(topPoint2, downTopLine);
                    points.Add(downTopPlane.Intersect(plane, m_mainRightPlane));
                }
            }
            else
            {
                //上对后点击的那一侧
                var middlePlane1 = new Plane(topPoint1, topPoint2 - topPoint1, m_axisZ);
                var topMiddle = middlePlane1.Intersect(m_mainRightPlane, plane);
                points.Add(topMiddle - m_axisY * m_upPointLen);
            }
            points.Add(topPoint2);
            points.Add(bottomPoint2);
            if(m_chamferCtrl == 2)
            {
                if(m_rightDownChamfer != double.MaxValue)
                {
                    var rightBottomLine = m_downSub.BottomPlane.Normal.Normalize();
                    var downBottomLine = PartsTools.RotateVectorInPlane(rightBottomLine,
                        rightBottomLine.Cross(m_downSub.BraceLine.Normalize()), m_rightDownChamfer);
                    var downBottomPlane = new Plane(bottomPoint2, downBottomLine);
                    points.Add(downBottomPlane.Intersect(plane, m_mainRightPlane));
                }
                
                if(m_leftDownChamfer != double.MaxValue)
                {
                    var leftBottomLine = m_upSub.BottomPlane.Normal.Normalize();
                    var upBottomLine = PartsTools.RotateVectorInPlane(leftBottomLine,
                        leftBottomLine.Cross(m_upSub.BraceLine.Normalize()), m_leftDownChamfer);
                    var upBottomPlane = new Plane(bottomPoint1, upBottomLine);
                    points.Add(upBottomPlane.Intersect(plane, m_mainLeftPlane));
                }
            }
            else
            {
                //下对先点击的那一侧
                var middlePlane2 = new Plane(bottomPoint1, bottomPoint2 - bottomPoint1, m_axisZ);
                var bottomMiddle = middlePlane2.Intersect(m_mainLeftPlane, plane);
                points.Add(bottomMiddle + m_axisY * m_downPointLen);
            }
            
            return AdjustGussetDepth(points, plane);
        }
        private void CreateFrontPlate()
        {
            frontConnectPlate = m_context.TargetComp.CreateSteelPlate(GetCtrlPoints(m_mainFrontPlane),
                m_ConnectPlateProps.Thickness, Define.ObjectSide.SIDE_LEFT, true);
            if (frontConnectPlate == null)
                return;
            frontConnectPlate.Class = m_context.TargetComp.CompClass;
            m_ConnectPlateProps.ApplyToPart(frontConnectPlate);
        }
        private void CreateBackPlate()
        {
            backConnectPlate = m_context.TargetComp.CreateSteelPlate(GetCtrlPoints(m_mainBackPlane),
                m_ConnectPlateProps.Thickness, Define.ObjectSide.SIDE_LEFT, true);
            if (backConnectPlate == null)
                return;
            backConnectPlate.Class = m_context.TargetComp.CompClass;
            m_ConnectPlateProps.ApplyToPart(backConnectPlate);
        }
        /// <summary>
        /// 创建隔板
        /// </summary>
        /// <param name="sub">次零件</param>
        /// <param name="basePlane">连接板参照面</param>
        /// <returns></returns>
        private List<PartBase> CreateSpacerPlate(SubData sub, Plane basePlane)
        {
            if (m_spacerPlateProps.Thickness == 0)
                return null;
            Plane plane;
            if (basePlane.Equals(m_mainBackPlane))
                plane = sub.BackPlane;
            else
                plane = sub.FrontPlane;
            var upDist = Math.Round(basePlane.SignedDist(plane.Origin));
            var upPlane = basePlane.Invert();
            if (upDist > 0)
            {
                upPlane.InvertThis();
                upPlane.MoveThis(m_ConnectPlateProps.Thickness);
                upDist = Math.Round(upPlane.SignedDist(plane.Move(-m_spacerPlateProps.Thickness).Origin));
            }

            if (Math.Abs(upDist) < 1e-6 || Math.Abs(upDist) > 90.0)
                return null;

            double symbol = upDist / Math.Abs(upDist);
            var Target = m_context.TargetComp;
            var RetPlates = new List<PartBase>();
            while (true)
            {
                if (Math.Abs(upDist) < m_spacerPlateProps.Thickness)
                    return RetPlates;
                var points = new List<Vec3>();
                points.Add(sub.AlignPlane.Intersect(sub.TopPlane, upPlane));
                points.Add(sub.AlignPlane.Intersect(sub.BottomPlane, upPlane));
                points.Add(sub.BorderPlane.Intersect(sub.BottomPlane, upPlane));
                points.Add(sub.BorderPlane.Intersect(sub.TopPlane, upPlane));
                var ctrlPoints = AdjustGussetDepth(points, upPlane);
                var retPlate = Target.CreateSteelPlate(ctrlPoints, 
                    m_spacerPlateProps.Thickness, Define.ObjectSide.SIDE_LEFT, true);
                if (retPlate == null)
                    continue;
                retPlate.Class = Target.CompClass;
                m_spacerPlateProps.ApplyToPart(retPlate);
                RetPlates.Add(retPlate);

                upDist += -symbol * m_spacerPlateProps.Thickness;
                upPlane.MoveThis(m_spacerPlateProps.Thickness);
            }
        }
        private bool InitBoltData()
        {
            BoltParse boltParse = new BoltParse(m_paramsHandler);
            var bolts = boltParse.GetBoltData();
            if (bolts == null)
                return false;
            m_boltData = bolts;
            boltLen = PartsTools.SumArray(bolts.SubHoriBoltsStride)
                + bolts.AlignDist + bolts.PlateDist;
            return true;
        }
        private bool CreateSubBolts(Plane plane, PartBase retPlate)
        {
            if (retPlate == null)
                return false;
            if (m_boltData.SubHoriBoltsStride == null || m_boltData.SubVertBoltsStride == null)
                return false;

            m_upSub.BorderPlane = upAlignPlane.Move(boltLen);
            m_upSub.AlignPlane = upAlignPlane;
            m_downSub.AlignPlane = downAlignPlane;
            m_downSub.BorderPlane = downAlignPlane.Move(boltLen);
            var m_Subs = new SubData[] { m_upSub, m_downSub };
            foreach (var sub in m_Subs)
            {
                var boltX = -sub.BraceLine.Normalize();
                var boltY = sub.TopPlane.Normal.Normalize();
                var boltZ = sub.FrontPlane.Normal.Normalize();
                var topPoint = sub.TopPlane.Intersect(sub.BorderPlane, plane);
                var bottomPoint = sub.BottomPlane.Intersect(sub.BorderPlane, plane);
                Vec3 startPos = topPoint / 2 + bottomPoint / 2 + boltY * m_boltData.SubOffset;
                Vec3 endPos = startPos + boltX * boltLen;

                //创建隔板
                var plates = CreateSpacerPlate(sub, plane);
                var parts = new List<PartBase>();
                parts.Add(retPlate);
                if (plates != null)
                    parts.AddRange(plates);
                BoltGroup bolts = m_context.TargetComp.CreateBoltGroup(sub.Beam, parts.ToArray(), startPos, endPos, true);
                if (bolts == null)
                    return false;

                bolts.WorkAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
                bolts.RotationSide = Define.ObjectSide.SIDE_TOP;
                bolts.Rotation = 0;
                m_boltData.BoltsCfg.FillBasicProps(bolts, true);
                m_boltData.BoltsCfg.FillHoleProps(bolts);
                m_boltData.BoltsCfg.FillContentProps(bolts);
                m_boltData.BoltsCfg.FillExtendProps(bolts);
                m_boltData.BoltsCfg.FillShape(bolts);
                bolts.ArrayStrideX = m_boltData.SubHoriBoltsStride;
                bolts.ArrayStrideY = m_boltData.SubVertBoltsStride;
                bolts.HeadOffsetX = m_boltData.PlateDist;
                bolts.VerticalOffset = 0;
                bolts.CutLength = 135.0;
                bolts.Refresh();
            }

            return true;
        }
        private bool CreateMainBolts(Plane plane, PartBase retPlate)
        {
            if (retPlate == null)
                return false;
            if (m_boltData.MainHoriBoltsStride == null || m_boltData.MainVertBoltsStride == null)
                return false;
            var dist = m_mainLeftPlane.Dist(m_mainRightPlane.Origin);
            var len = m_upSub.TopPlane.Dist(m_upSub.BottomPlane.Origin);
            var point = m_upSub.TopPlane.Move(-len / 2)
                .Intersect(plane, m_mainLeftPlane.Move(-dist / 2));
            var VertLen = PartsTools.SumArray(m_boltData.MainVertBoltsStride);

            Vec3 startPos = point - m_axisY * (VertLen / 2 - m_boltData.VertOffset)
                + m_mainLeftPlane.Normal.Normalize() * m_boltData.HoriOffset;
            if (VertLen == 0)
                VertLen = 100;
            Vec3 endPos = startPos + m_axisY * VertLen;
            var boltX = m_axisY;
            var boltY = m_mainLeftPlane.Normal.Normalize();
            var boltZ = m_mainFrontPlane.Normal.Normalize();
            BoltGroup bolts = m_context.TargetComp.CreateBoltGroup(m_context.MainPart, retPlate, startPos, endPos, true);
            if (bolts == null)
                return false;

            bolts.WorkAxis = new Mat43(boltX, boltY, boltZ, Vec3.ZERO);
            bolts.RotationSide = Define.ObjectSide.SIDE_TOP;
            bolts.Rotation = 0;

            BoltsConfig config = new BoltsConfig();
            config.CopyFromBasicProps(m_paramsHandler.GetDObjectValue(ParamNames.BOLT_MAIN_BASIC_PROPS));
            config.FillBasicProps(bolts);
            m_boltData.BoltsCfg.FillHoleProps(bolts);
            m_boltData.BoltsCfg.FillContentProps(bolts);
            m_boltData.BoltsCfg.FillExtendProps(bolts);
            m_boltData.BoltsCfg.FillShape(bolts);
            bolts.ArrayStrideX = m_boltData.MainVertBoltsStride;
            bolts.ArrayStrideY = m_boltData.MainHoriBoltsStride;
            bolts.VerticalOffset = 0;
            bolts.CutLength = 135.0;
            bolts.Refresh();
            return true;
        }
        /// <summary>
        /// 调整节点板深度
        /// </summary>
        private Vec3[] AdjustGussetDepth(List<Vec3> points, Plane plane)
        {
            var arr = points.ToArray();
            //计算创建的板的深度方向
            var axisX = (arr[2] - arr[1]).Normalize();
            var axisY = (arr[0] - arr[1]).Normalize();
            var axisZ = axisX.Cross(axisY);
            return axisZ.Dot(plane.Normal) > 0 ? points.ToArray() : points.Reverse<Vec3>().ToArray();
        }
    }
}
