﻿using LoongCAD.STC.Plugins;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using System.Linq;

namespace DiagonalJoint53
{
    public class PartsHandler
    {
        private ComponentInstanceContext m_context;
        private SubData[] m_subs = new SubData[2];
        private Vec3 m_axisY = Vec3.ZERO;
        private Vec3 m_axisZ = Vec3.ZERO;
        private Vec3 m_axisX = Vec3.ZERO;
        public PartsHandler(ComponentInstanceContext context)
        {
            m_context = context;
        }
        public bool Execute()
        {
            //建立坐标系
            m_axisY = m_context.MainLine;
            m_axisX = m_context.TargetComp.LocalMatrix.X;
            m_axisZ = m_axisX.Cross(m_axisY);

            //解析次零件
            if (!ParseSubData())
                return false;
            if (!DefineSubBuild())
                return false;
            if (!ParseSubFrontAndBackPlane())
                return false;
            if (!ParseSubTopAndBottomPlane())
                return false;

            m_context.UpSub = m_subs[0];
            m_context.DownSub = m_subs[1];

            return true;
        }
        /// <summary>
        /// 解析次零件数据信息
        /// </summary>
        private bool ParseSubData()
        {
            var len = m_context.TargetComp.Subs.Length;
            if (len < 2 || len > 2)
                return false;
            for (int i = 0; i < len; i++)
            {
                var item = m_context.TargetComp.Subs[i];
                if (item != null)
                {
                    Beam beam = item as Beam;
                    if (beam == null)
                        return false;
                    var sub = new SubData
                    {
                        Beam = beam,
                        BeamPlanes = BeamPlanes.CreateFromBeam(beam),
                    };
                    m_subs[i] = sub;
                }
            }
            return true;
        }
        /// <summary>
        /// 定义次零件构建方向
        /// </summary>
        private bool DefineSubBuild()
        {
            var linePoint = m_context.MainPart.CtrlPoints.First().WorldPos;
            foreach (var sub in m_subs)
            {
                var firstCtrlPoint = sub.Beam?.CtrlPoints?.First()?.WorldPos;
                var lastCtrlPoint = sub.Beam?.CtrlPoints?.Last()?.WorldPos;
                if (!firstCtrlPoint.HasValue || !lastCtrlPoint.HasValue)
                    return false;

                var dist1 = (firstCtrlPoint.Value - linePoint).Cross(m_axisY).Length;
                var dist2 = (lastCtrlPoint.Value - linePoint).Cross(m_axisY).Length;
                if (dist2 > dist1)
                {
                    sub.BraceLine = (lastCtrlPoint.Value - firstCtrlPoint.Value).Normalize();
                    sub.FarPoint = lastCtrlPoint.Value;
                }
                else
                {
                    sub.BraceLine = -(lastCtrlPoint.Value - firstCtrlPoint.Value).Normalize();
                    sub.FarPoint = firstCtrlPoint.Value;
                }
            }
            return true;
        }
        private bool ParseSubFrontAndBackPlane()
        {
            foreach(var sub in m_subs)
            {
                var planes = PartsTools.DotExtremeBoundPlanes(sub.BeamPlanes, m_axisZ, true);
                if (planes == null)
                    return false;

                sub.FrontPlane = planes[0];
                sub.BackPlane = planes[1];
            }

            return true;
        }
        private bool ParseSubTopAndBottomPlane()
        {
            foreach (var sub in m_subs)
            {
                var planes = PartsTools.DotExtremeBoundPlanes(sub.BeamPlanes, m_axisY, true);
                if (planes == null)
                    return false;

                sub.TopPlane = planes[1];
                sub.BottomPlane = planes[0];
            }

            return true;
        }
    }
}
