﻿using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Geometry;
using System.Linq;

namespace EndSealingPlate1002
{
    public class ComponentInstanceContext
    {
        ///<summary>
        /// 目标组件的类型
        ///</summary>
        private DetailComponent m_comp;

        private Beam m_mainPart = null;

        private Vec3 m_mainLine;

        private BeamPlanes m_mainPlanes;
        //开发组件需要的上下文
        public DetailComponent TargetComp { get => m_comp; set => m_comp = value; }
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public Vec3 MainLine { get => m_mainLine; set => m_mainLine = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public PartBase RetBeam { get; set; }
        public Vec3 X { get; set; }
        public Vec3 Y { get; set; }
        public Vec3 Z { get; set; }
        public Vec3 Pos0 { get; set; }
        public static ComponentInstanceContext CreateInstance(DetailComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp))
                return compContext;
            compContext.Clear();
            return null;
        }
        private bool Initialize(DetailComponent comp)
        {
            if (comp == null)
                return false;

            Beam mainPart = comp.Main as Beam;
            if (mainPart == null)
                return false;
            if (!PrepareMainLine(mainPart, comp))
                return false;

            m_comp = comp;
            m_mainPart = mainPart;
            m_mainPlanes = BeamPlanes.CreateFromBeam(mainPart);

            return true;
        }
        /// <summary>
        /// 准备主零件由起点到终点的线
        /// </summary>
        private bool PrepareMainLine(Beam mainPart, DetailComponent comp)
        {
            var point = comp.CtrlPoints[0].WorldPos;
            var line = mainPart.CtrlPoints?.Last()?.WorldPos -
                       mainPart.CtrlPoints?.First()?.WorldPos;
            if (!line.HasValue)
                return false;
            var plane = new Plane(point, line.Value);
            var dist1 = plane.Dist(mainPart.CtrlPoints.First().WorldPos);
            var dist2 = plane.Dist(mainPart.CtrlPoints.Last().WorldPos);
            m_mainLine = dist1 > dist2 ? -line.Value.Normalize() : line.Value.Normalize();
            return true;
        }
        private void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_mainPlanes = null;
        }
    }
}
