﻿using System;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace EndSealingPlate1002
{
    public class ParamsHandler : LoongCAD.STC.Plugins.ParamsHandlerBase
    {
        /// <summary>
        /// 属性空值全局总表
        /// </summary>
        protected static DObject g_emptyValues = null;

        /// <summary>
        /// 属性内置默认值全局总表
        /// </summary>
        protected static DObject g_builtinDefaults = null;

        /// <summary>
        /// 默认构造函数
        /// </summary>
        /// <remarks><b>此方法需要复制到其他组件代码中使用</b></remarks>
        public ParamsHandler()
            : base(ParamsHandler.EmptyValues, ParamsHandler.BuiltinDefautls)
        {

        }
        /// <summary>
        /// 组件公用属性空值表
        /// </summary>
        public static DObject EmptyValues
        {
            get
            {
                if (g_emptyValues == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_emptyValues;
            }
        }
        /// <summary>
        /// 组件公用内置默认值接口
        /// </summary>
        public static DObject BuiltinDefautls
        {
            get
            {
                if (g_builtinDefaults == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_builtinDefaults;
            }
        }
        /// <summary>
        /// 初始化默认值
        /// </summary>
        /// <param name="defaultConfig">从默认值配置加载的默认值</param>
        public override LoongCAD.Util.DObject InitializeDefaultValues(DObject defaultConfig)
        {
            m_defaultValues = new DObject();
            m_defaultValues.CopyFrom(defaultConfig);

            var emptyValues = ParamsHandler.EmptyValues;
            var builtinDefaults = ParamsHandler.BuiltinDefautls;
            if (emptyValues == null || builtinDefaults == null)
            {
                Console.WriteLine("ParamsHandler.InitializeDefaultValues: [Warning] EmptyValue/BuiltinDefaults is null!");
                return m_defaultValues;
            }

            this.TryCompleteDObjectValues(m_defaultValues, builtinDefaults, emptyValues);
            return m_defaultValues;
        }
        /// <summary>
        /// 初始化组件所有实例公用的属性空值、属性内置默认值
        /// </summary>
        public static void IntializeEmptyDefaults()
        {
            g_emptyValues = new DObject();
            g_builtinDefaults = new DObject();

            //图形页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_UP_GAP, double.MaxValue, 10.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_DOWN_GAP, double.MaxValue, 10.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_LEFT_GAP, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_RIGHT_GAP, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_PLATE_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 10.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "板",
            });
            //焊缝
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length - 1; ++i)
            {
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WELD_PROPS_LIST[i], WeldPropertyPanel.EmptyValue, new DObject {
                    { WeldPropertyPanel.ParamNames.SizeUp, 10.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeUp, 11}, // Fillet
                    { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                    { WeldPropertyPanel.ParamNames.SizeDown, 10.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeDown, 11 }, // Fillet
                    { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                    { WeldPropertyPanel.ParamNames.EdgeAround, 1 }, // Edge
                    { WeldPropertyPanel.ParamNames.MadeType, 2 }, // Workshop
                    { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                    { WeldPropertyPanel.ParamNames.RefText, "" }
                });
            }
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WELD_PROPS_LIST[2], WeldPropertyPanel.EmptyValue, new DObject {
                { WeldPropertyPanel.ParamNames.SizeUp, 8.0 },
                { WeldPropertyPanel.ParamNames.WeldTypeUp, 4}, // Fillet
                { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                { WeldPropertyPanel.ParamNames.SizeDown, 8.0 },
                { WeldPropertyPanel.ParamNames.WeldTypeDown, 4 }, // Fillet
                { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                { WeldPropertyPanel.ParamNames.EdgeAround, 1 }, // Edge
                { WeldPropertyPanel.ParamNames.MadeType, 2 }, // Workshop
                { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                { WeldPropertyPanel.ParamNames.RefText, "" }
            });
        }
    }
}
