﻿using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System;

namespace EndSealingPlate1002
{
    public class WeldHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler;
        private List<WeldProperties> m_weldProps = new List<WeldProperties>();
        public WeldHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;

            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                m_weldProps.Add(new WeldProperties());
            }
        }
        public void Execute()
        {
            if (!ParseProps())
                return;
            if (!CreateWelds())
                return;
        }
        private bool ParseProps()
        {
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }
            return true;
        }
        private bool CreateWelds()
        {
            var mainPart = m_context.MainPart;
            var retBeam = m_context.RetBeam;
            if (retBeam == null)
                return false;
            var axisX = m_context.X;
            var axisY = m_context.Y;
            var axisZ = m_context.Z;
            var pos0 = m_context.Pos0;
            //创建1,2焊缝
            {
                if (m_weldProps[0].SizeUp > 0 || m_weldProps[0].SizeDown > 0)
                {
                    Weld curWeld1 = m_context.TargetComp.CreateAutoWeld(mainPart, retBeam, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                    curWeld1.WorkAxis = new Mat43(axisX, axisY, axisZ, pos0);
                    curWeld1.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_Y;
                    m_weldProps[0].ApplyToWeld(curWeld1);
                }

                if (m_weldProps[1].SizeDown > 0 || m_weldProps[1].SizeUp > 0)
                {
                    Weld curWeld2 = m_context.TargetComp.CreateAutoWeld(mainPart, retBeam, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                    curWeld2.WorkAxis = new Mat43(axisX, axisY, axisZ, pos0);
                    curWeld2.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NY;
                    m_weldProps[1].ApplyToWeld(curWeld2);
                }
                
            }

            //创建3焊缝
            {
                if(m_weldProps[2].SizeUp > 0 || m_weldProps[2].SizeDown > 0)
                {
                    Weld curWeld3 = m_context.TargetComp.CreateAutoWeld(mainPart, retBeam, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                    curWeld3.WorkAxis = new Mat43(axisX, axisY, axisZ, pos0);
                    curWeld3.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NX;
                    m_weldProps[2].ApplyToWeld(curWeld3);
                }
            }
            return true;
        }
    }
}
