﻿using System.Collections.Generic;
using System.Linq;

namespace ExtendedTrapezoidalPlate103
{
    public class BoltData
    {
        public int VertOffsetMode { get; set; }
        public double VertOffset { get; set; }
        public double UpGap { get; set; }
        public double DownGap { get; set; }
        public double[] VertStride { get; set; }
        public int HoriCount { get; set; }
        public double[] HoriStride { get; set; }
        public int[] IgnoreBolt { get; set; }
        public double LeftGap { get; set; }
        public double RightGap { get; set; }
        public int BoltOrientation { get; set; }
        public BoltsConfig BoltsCfg { get; set; } = new BoltsConfig();
    }
    public class BoltParse
    {
        private ParamsHandler m_paramsHandler = null;
        private BoltData m_boltData = new BoltData();
        public BoltParse(ParamsHandler paramsHandler)
        {
            m_paramsHandler = paramsHandler;
        }
        public BoltData GetBoltParams()
        {
            if (!ParseProps())
                return null;
            return m_boltData;
        }
        private bool ParseProps()
        {
            m_boltData.BoltsCfg.CopyFromBasicProps
               (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_BASIC_PROPS));
            m_boltData.BoltsCfg.CopyFromHoleProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_HOLE_PROPS));
            m_boltData.BoltsCfg.CopyFromContentProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_CONTENT_PROPS));
            m_boltData.BoltsCfg.CopyFromExtendProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_EXTEND_PROPS));
            m_boltData.BoltsCfg.CopyFromShapeProps
                (m_paramsHandler.GetIntValue(ParamNames.BOLT_SHAPE));

            m_boltData.VertOffsetMode = m_paramsHandler.GetIntValue(ParamNames.BOLT_VERT_OFFSET_MODE);
            m_boltData.VertOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_VERT_OFFSET);
            m_boltData.UpGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_VERT_UP_GAP);
            m_boltData.DownGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_VERT_DOWN_GAP);
            m_boltData.LeftGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_HORI_LEFT_GAP);
            m_boltData.RightGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_HORI_RIGHT_GAP);
            m_boltData.BoltOrientation = m_paramsHandler.GetIntValue(ParamNames.BOLT_ORIENTATION);
            
            m_boltData.HoriCount = m_paramsHandler.GetIntValue(ParamNames.BOLT_HORI_COUNT);
            var arrY = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_HORI_STRIDE);
            m_boltData.HoriStride = HandlerStrideY(arrY, m_boltData.HoriCount);
            return true;
        }
        /// <summary>
        /// 根据螺栓数目调整螺栓Y方向间距
        /// </summary>
        private double[] HandlerStrideY(double[] arr, int num)
        {
            if (num <= 0)
                return null;
            if (num == 1)
                return new double[] { 0 };
            var ret = new List<double>();
            if (arr == null || arr.Length == 0)
            {
                ret.Add(60.0);
                if(num == 2)
                    return ret.ToArray();
                ret.Add(60.0);
                ret.AddRange(Enumerable.Repeat(76.2, num - 3));
                return ret.ToArray();
            }
            for (int i = 0; i < num - 1; i++)
            {
                if (i > arr.Length - 1)
                    ret.Add(arr[arr.Length - 1]);
                else
                    ret.Add(arr[i]);
            }
            return ret.ToArray();
        }
    }
}
