﻿using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System;

namespace ExtendedTrapezoidalPlate103
{
    public class WeldHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler;
        private List<WeldProperties> m_weldProps = new List<WeldProperties>();
        private List<PartBase> m_plates = new List<PartBase>();
        private PartBase[] m_stiffs = null;
        public WeldHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;

            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                m_weldProps.Add(new WeldProperties());
            }
        }
        public void Execute()
        {
            if (!ParseProps())
                return;
            CreatePlateWelds();

            //加劲肋焊缝
            if (m_context.Stiffs == null || m_context.Stiffs.Count < 1)
                return;
            m_stiffs = m_context.Stiffs.ToArray();
            //topNs 焊缝4 -- nearWeb 焊缝9 -- nearFlange farFlange
            CreateStiffWelds(m_stiffs[0], 3, 8);
            //topFs 焊缝2 -- farWeb 焊缝8 -- nearFlange farFlange
            CreateStiffWelds(m_stiffs[1], 1, 7);
            //bottomNs 焊缝3 -- nearWeb 焊缝7 -- nearFlange farFlange
            CreateStiffWelds(m_stiffs[2], 2, 6);
            //bottomNs 焊缝1 -- farWeb 焊缝10 -- nearFlange farFlange
            CreateStiffWelds(m_stiffs[3], 0, 9);
        }
        private bool ParseProps()
        {
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }
            m_plates = m_context.Plates;
            return true;
        }
        private void CreatePlateWelds()
        {
            if (m_plates == null || m_plates.Count < 1)
                return;
            //焊缝5
            do
            {
                if (m_weldProps[4].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
                foreach(var item in m_plates)
                {
                    var point1 = item.CtrlPoints[0].WorldPos;
                    var point2 = item.CtrlPoints[1].WorldPos;
                    Weld curWeld1 = m_context.TargetComp.CreatePolyWeld(m_context.MainPart,
                        item, new Vec3[] { point1, point2 }, true);
                    m_weldProps[4].ApplyToWeld(curWeld1);
                    var dir = item.LocalMatrix.Z.Normalize();
                    double thickness = GetPolyPlateThickness(item);
                    if (thickness == 0.0)
                        continue;
                    Weld curWeld2 = m_context.TargetComp.CreatePolyWeld(m_context.MainPart,
                        item, new Vec3[] { point1 + dir * thickness, point2 + dir * thickness }, true);
                    m_weldProps[4].ApplyToWeld(curWeld2);
                }
            } while (false);
            //焊缝6
            do
            {
                if (m_weldProps[5].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
                //todo 待自动焊缝完成
                /*foreach (var item in m_plates)
                {
                    var point = item.CtrlPoints.First().WorldPos;
                    Weld curWeld = m_context.TargetComp.CreateAutoWeld(m_context.SubPart,
                        item, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                    Vec3 axisX = (item.CtrlPoints[1].WorldPos - point).Normalize();
                    Vec3 axisY = m_context.SubLine;
                    Vec3 axisZ = item.LocalMatrix.Z.Normalize();
                    curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, point);
                    curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_X;
                    m_weldProps[5].ApplyToWeld(curWeld);
                }*/
                foreach(var item in m_plates)
                {
                    var points = item.CtrlPoints;
                    int len = points.Length;
                    if (len < 3)
                        continue;
                    for(int i = 2; i < len; i++)
                    {
                        Weld curWeld = m_context.TargetComp.CreatePolyWeld(m_context.SubPart,
                        item, new Vec3[] { points[i].WorldPos / 2 + points[(i + 1) % len].WorldPos / 2 }, true);
                        m_weldProps[5].ApplyToWeld(curWeld);
                    }
                }
            } while (false);
        }
        /// <summary>
        /// 创建加劲肋的焊缝
        /// </summary>
        /// <param name="stiff">加劲肋</param>
        /// <param name="position1">焊缝位置1</param>
        /// <param name="position2">焊缝位置2</param>
        private void CreateStiffWelds(PartBase stiff, int position1, int position2)
        {
            if (stiff == null)
                return;
            Vec3[] pos = new Vec3[stiff.CtrlPoints.Length];
            //不是矩形板
            if (pos.Length != 4)
                return;
            for (int i = 0; i < pos.Length; i++)
                pos[i] = stiff.CtrlPoints[i].WorldPos;
            var axisX = (pos[1] - pos[0]).Normalize();
            var axisY = (pos[3] - pos[0]).Normalize();
            var axisZ = stiff.LocalMatrix.Z.Normalize();
            double thickness = GetPolyPlateThickness(stiff);
            //焊缝position1 -- nearWeb
            do
            {
                if (m_weldProps[position1].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
                Weld curWeld;
                if (m_weldProps[position1].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_FILLET || 
                    m_weldProps[position1].TypeDown == LoongCAD.STC.Define.WeldType.WELD_TYPE_FILLET)
                {
                    curWeld = m_context.TargetComp.CreateAutoWeld(m_context.MainPart,
                        stiff, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                    curWeld.WorkAxis = new Mat43(axisX, axisY, axisZ, pos[0]);
                    curWeld.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NY;
                }
                else
                {
                    curWeld = m_context.TargetComp.CreatePolyWeld(m_context.MainPart,
                        stiff, new Vec3[] { pos[0] / 2 + pos[1] / 2 }, true);
                }
                m_weldProps[position1].ApplyToWeld(curWeld);

                //豁口
                if (thickness < 1e-3)
                    break;
                if (m_weldProps[position1].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_BEVEL_GROOVE_BEVEL_BUTT)
                {
                    double upAngle = m_weldProps[position1].GrooveAngleUp;
                    double upLen = m_weldProps[position1].SizeUp;
                    CreateCutter(axisY, axisZ, pos[0], pos[1], upAngle, upLen, stiff);
                }
                if (m_weldProps[position1].TypeDown == LoongCAD.STC.Define.WeldType.WELD_TYPE_BEVEL_GROOVE_BEVEL_BUTT)
                {
                    double downAngle = m_weldProps[position1].GrooveAngleDown;
                    double downLen = m_weldProps[position1].SizeDown;
                    CreateCutter(axisY, -axisZ, pos[0] + axisZ * thickness,
                        pos[1] + axisZ * thickness, downAngle, downLen, stiff);
                }
            } while (false);

            //焊缝position2 -- nearFlange farFlange
            do
            {
                if (m_weldProps[position2].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                    break;
                Weld curWeld1, curWeld2;
                if (m_weldProps[position2].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_FILLET ||
                    m_weldProps[position2].TypeDown == LoongCAD.STC.Define.WeldType.WELD_TYPE_FILLET)
                {
                    curWeld1 = m_context.TargetComp.CreateAutoWeld(m_context.MainPart,
                        stiff, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                    curWeld1.WorkAxis = new Mat43(axisX, axisY, axisZ, pos[0]);
                    curWeld1.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_NX;
                    
                    curWeld2 = m_context.TargetComp.CreateAutoWeld(m_context.MainPart,
                        stiff, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
                    curWeld2.WorkAxis = new Mat43(axisX, axisY, axisZ, pos[0]);
                    curWeld2.AutoWeldPosition = LoongCAD.STC.Define.SpaceAxis.AXIS_X;
                }
                else
                {
                    curWeld1 = m_context.TargetComp.CreatePolyWeld(m_context.MainPart, stiff,
                        new Vec3[] { pos[0] / 2 + pos[3] / 2 }, true);
                    curWeld2 = m_context.TargetComp.CreatePolyWeld(m_context.MainPart, stiff,
                        new Vec3[] { pos[1] / 2 + pos[2] / 2 }, true);
                }
                m_weldProps[position2].ApplyToWeld(curWeld1);
                m_weldProps[position2].ApplyToWeld(curWeld2);

                //豁口
                if (thickness < 1e-3)
                    break;
                if (m_weldProps[position2].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_BEVEL_GROOVE_BEVEL_BUTT)
                {
                    double upAngle = m_weldProps[position2].GrooveAngleUp;
                    double upLen = m_weldProps[position2].SizeUp;
                    CreateCutter(axisX, axisZ, pos[0], pos[3], upAngle, upLen, stiff);
                    CreateCutter(-axisX, axisZ, pos[1], pos[2], upAngle, upLen, stiff);
                }
                if (m_weldProps[position2].TypeDown == LoongCAD.STC.Define.WeldType.WELD_TYPE_BEVEL_GROOVE_BEVEL_BUTT)
                {
                    double downAngle = m_weldProps[position2].GrooveAngleDown;
                    double downLen = m_weldProps[position2].SizeDown;
                    CreateCutter(axisX, -axisZ, pos[0] + axisZ * thickness,
                        pos[3] + axisZ * thickness, downAngle, downLen, stiff);
                    CreateCutter(-axisX, -axisZ, pos[1] + axisZ * thickness,
                        pos[2] + axisZ * thickness, downAngle, downLen, stiff);
                }
            } while (false);
        }
        /// <summary>
        /// 创建豁口
        /// </summary>
        private void CreateCutter(Vec3 X, Vec3 Y, Vec3 pos0, Vec3 pos1, double angle, double len, PartBase plate)
        {
            if (angle <= 0 || len <= 0)
                return;
            double offsetX = len * Math.Tan(angle * Math.PI / 180);
            var point1 = pos0 + Y * len;
            var point2 = pos0 + X * offsetX;
            List<Plane> plsSide = new List<Plane>();
            var dir = (pos1 - pos0).Normalize();
            plsSide.Add(new Plane(point1, point2 - point1, dir));
            plsSide.Add(new Plane(pos0, -Y).Move(5.0));
            plsSide.Add(new Plane(pos0, -X).Move(5.0));
            Plane zTop = new Plane(pos0, -dir);
            Plane zBottom = new Plane(pos1, dir);
            var cutter = m_context.TargetComp.CreatePlateCutter(plate, zBottom, zTop, plsSide.ToArray(), true);
            if (cutter == null)
                return;
            cutter.IsWeldGroove = true;
            cutter.Flush();
        }
        private double GetPolyPlateThickness(PartBase plate)
        {
            if (plate == null)
                return 0.0;
            return Convert.ToDouble(plate.Profile.Name.Substring(2));
        }
    }
}
