﻿using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using System.Collections.Generic;
using System.Linq;
using System;

namespace HorizontalStiffener1017
{
    public class WeldHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler;
        private List<WeldProperties> m_weldProps = new List<WeldProperties>();

        public WeldHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;

            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                m_weldProps.Add(new WeldProperties());
            }
        }

        public void Execute()
        {
            if (!ParseProps())
                return;

            this.CreateWelds();
        }

        private bool ParseProps()
        {
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }

            return true;
        }

        private void CreateWelds()
        {
            var parts = m_context.Parts;
            if (parts.Count < 1)
                return;
            if (m_weldProps[0].TypeUp == LoongCAD.STC.Define.WeldType.WELD_TYPE_NONE)
                return;

            var chamferType = m_paramsHandler.GetIntValue(ParamNames.PART_PLATE_CHAMFER_TYPE);
            double chamferX = m_paramsHandler.GetDoubleValue(ParamNames.PART_PLATE_CHAMFER_X);
            double chamferY = m_paramsHandler.GetDoubleValue(ParamNames.PART_PLATE_CHAMFER_Y);
            if (chamferType == 1)
            {
                //无折角
                chamferY = 0d;
            }else if(chamferType > 2)
            {
                //凹弧或凸弧折角
                chamferY = chamferX;
            }

            foreach(var part in parts)
            {
                var dirX = part.LocalMatrix.Z.Normalize();
                var point1 = part.CtrlPoints[0].WorldPos;
                var point2 = part.CtrlPoints[1].WorldPos;
                var dirY = (point1 - point2).Normalize();

                Vec3 weldPoint1 = point1.Move(-dirY, chamferY);
                Vec3 weldPoint2 = point2.Move(dirY, chamferY);

                var thickness = this.GetPolyPlateThickness(part);

                Vec3[] weldPoints = new Vec3[]
                {
                    weldPoint1, weldPoint2, 
                    weldPoint2.Move(dirX, thickness),
                    weldPoint1.Move(dirX, thickness)
                };

                for(int i = 0; i < weldPoints.Length; i++)
                {
                    Weld curWeld = m_context.TargetComp.CreatePolyWeld(m_context.MainPart,
                        part, new Vec3[] {weldPoints[i], weldPoints[(i + 1) % weldPoints.Length] }, true);
                    m_weldProps[0].ApplyToWeld(curWeld);
                }
            }
        }

        private double GetPolyPlateThickness(PartBase plate)
        {
            if (plate == null)
                return 0.0;
            return Convert.ToDouble(plate.Profile.Name.Substring(2));
        }
    }
}
