﻿using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using System.Linq;
using LoongCAD.Geometry;

namespace InnerDiaphragm1059
{

    /// <summary>
    /// 组件上下文类，该类对象用来存储组件公用变量、属性
    /// </summary>
    public class ComponentInstanceContext
    {
        ///<summary>
        /// 目标组件的类型
        ///</summary>
        private DetailComponent m_comp;

        private Beam m_mainPart;

        private BeamPlanes m_mainPlanes;

        private Vec3 m_mainLine;

        private Vec3[] m_weldPoints = null;
        //开发组件需要的上下文
        public DetailComponent TargetComp{ get => m_comp; set => m_comp = value; }
        //组件的主要部分
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public Vec3 MainLine { get => m_mainLine; set => m_mainLine = value; }
        public PartBase RetPlate { get; set; }
        public Vec3[] WeldPoints { get => m_weldPoints; set => m_weldPoints = value; }
        public static ComponentInstanceContext CreateInstance(DetailComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp))
                return compContext;
            compContext.Clear();
            return null;
        }
        private bool Initialize(DetailComponent comp)
        {
            if (comp == null)
                return false;

            Beam mainPart = comp.Main as Beam;
            if (mainPart == null)
                return false;
            if (!PrepareMainLine(mainPart))
                return false;
            m_comp = comp;
            m_mainPart = mainPart;
            m_mainPlanes = BeamPlanes.CreateFromBeam(mainPart);

            return true;
        }
        /// <summary>
        /// 准备主零件由起点到终点的线
        /// </summary>
        private bool PrepareMainLine(Beam mainPart)
        {
            var line = mainPart.CtrlPoints?.Last()?.WorldPos -
                       mainPart.CtrlPoints?.First()?.WorldPos;
            if (!line.HasValue)
                return false;
            m_mainLine = line.Value.Normalize();
            return true;
        }
        private void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_mainPlanes = null;
        }
    }
}
