﻿using System;
using System.Windows.Forms;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using System.Collections.Generic;

namespace InnerDiaphragm1059
{
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_DetailComponentProps
    {
        private DObject m_rootUIProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;
        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostIntialize();
        }
        public override ComponentPlugin ReferencePlugin => m_refPlugin;
        private void OnPostIntialize()
        {
            // 设置窗口标题
            this.Text = "管内加劲板(409)";
            this.cbo_plate_position.ClearItems();
            this.cbo_plate_position.AddIntegerItems(0,0,4);
            this.cbo_plate_position.UpdateItems();

            foreach (TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }
        }
        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null)
            {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }
        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked)
            {
                retProps.CopyFrom(m_rootUIProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }
        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }
        private void btn_weld_props_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_rootUIProps, null);
        }
        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_rootUIProps.CopyFrom(srcProps);
        }
        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }
        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_rootUIProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_rootUIProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_rootUIProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_rootUIProps;
            return comp.UIProps;
        }

        private void cbo_plate_chamfer_SelectedIndexChanged(object sender, EventArgs e)
        {
            bool flag = true;
            if (cbo_plate_chamfer.SelectedIndex == 1)
                flag = false;
            act_plate_chamfer.Enabled = flag;
            tb_plate_chamfer.Enabled = flag;
        }
    }
}
