﻿using LoongCAD.UI;
using LoongCAD.Util;
using LoongCAD.UI.Component;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


namespace MultipleSiffeners1064
{
    public static class BuiltinDefaults
    {

        /// <summary>
        /// 内置默认值汇总，若用户未设置指定属性的默认值，则使用内置默认值
        /// </summary>
        private static DObject m_defaultProps = null;

        public static int GetInteger(string propName, int defValue = 0)
        {
            if (m_defaultProps == null)
            {
                Initialize();
            }

            return m_defaultProps.GetInt(propName, defValue);
        }

        public static int[] GetIntArray(string propName)
        {
            if (m_defaultProps == null)
            {
                Initialize();
            }

            return m_defaultProps.GetIntArr(propName);
        }

        public static double GetDouble(string propName, double defValue = 0)
        {
            if (m_defaultProps == null)
            {
                Initialize();
            }

            return m_defaultProps.GetDouble(propName, defValue);
        }

        public static double[] GetDoubleArray(string propName)
        {
            if (m_defaultProps == null)
            {
                Initialize();
            }

            return m_defaultProps.GetDoubleArr(propName);
        }

        public static DObject GetDObject(string propName, DObject defValue = null)
        {
            if (m_defaultProps == null)
            {
                Initialize();
            }

            return m_defaultProps.GetDObject(propName, defValue);
        }

        /// <summary>
        /// 初始化组件的内置默认值
        /// </summary>
        private static void Initialize()
        {
            m_defaultProps = new DObject();
            DObject newDefaults = new object[]
            {
                // 图形页
               ParamNames.PIC_STIFFENER_COUNT, 1,
               ParamNames.PIC_STIFFENER_STRIDE, new double[] {387.0 },

               // 参数
               ParamNames.ARG_LEFT_ROT, 0.0,
               ParamNames.ARG_RIGHT_ROT, 0.0,
               ParamNames.ARG_PLATES_GAP, 0.0,
               ParamNames.ARG_PLATES_POS, 1,
               ParamNames.ARG_START_POS, 1,

                // 板页
                //左板
                ParamNames.PART_LEFT_PLATE_PROPS, (DObject)new object[] {
                    PlatePropertyPanel.ParamNames.Thickness, 12.0,
                    PlatePropertyPanel.ParamNames.Width, 90.0,
                    PlatePropertyPanel.ParamNames.NoPrefix, "",
                    PlatePropertyPanel.ParamNames.NoStartPostfix, "1001",
                    PlatePropertyPanel.ParamNames.AssNoPrefix, "",
                    PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                    PlatePropertyPanel.ParamNames.Material, "Q235B",
                    PlatePropertyPanel.ParamNames.Name, "板",
                },

                ParamNames.STIFF_LEFT_PLATE_TYPE, 1, // Line
                ParamNames.STIFF_LEFT_PLATE_TOP_GAP, 0.0,
                ParamNames.STIFF_LEFT_PLATE_BOTTOM_GAP, 0.0,
                ParamNames.STIFF_LEFT_PLATE_INSIDE_GAP, 0.0,
                ParamNames.STIFF_LEFT_PLATE_OUTSIDE_GAP, 6.0,

                ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_TYPE, 2,
                ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_X, 15.0,
                ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_Y,15.0,

                ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_TYPE,1,
                ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_X, 0.0,
                ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_Y, 0.0,

                ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_TYPE, 2,
                ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_X, 15.0,
                ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_Y,15.0,

                ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_TYPE,1,
                ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_X, 0.0,
                ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_Y, 0.0,
                




                // 右板
                ParamNames.PART_RIGHT_PLATE_PROPS, (DObject)new object[] {
                    PlatePropertyPanel.ParamNames.Thickness, 12.0,
                    PlatePropertyPanel.ParamNames.Width, 90.0,
                    PlatePropertyPanel.ParamNames.NoPrefix, "",
                    PlatePropertyPanel.ParamNames.NoStartPostfix, "1001",
                    PlatePropertyPanel.ParamNames.AssNoPrefix, "",
                    PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                    PlatePropertyPanel.ParamNames.Material, "Q235B",
                    PlatePropertyPanel.ParamNames.Name, "板",
                },

                ParamNames.STIFF_RIGHT_PLATE_TYPE, 1, // Line
                ParamNames.STIFF_RIGHT_PLATE_TOP_GAP, 0.0,
                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_GAP, 0.0,
                ParamNames.STIFF_RIGHT_PLATE_INSIDE_GAP, 0.0,
                ParamNames.STIFF_RIGHT_PLATE_OUTSIDE_GAP, 6.0,

                ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_TYPE, 2,
                ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_X, 15.0,
                ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_Y,15.0,

                ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_TYPE,1,
                ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_X, 0.0,
                ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_Y, 0.0,

                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_TYPE, 2,
                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_X, 15.0,
                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_Y,15.0,

                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_TYPE,1,
                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_X, 0.0,
                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_Y, 0.0,
            };
            // 焊缝
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                newDefaults[ParamNames.WELD_PROPS_LIST[i]] = new DObject
                {
                    { WeldPropertyPanel.ParamNames.SizeUp, 6.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeUp, 11 }, // Fillet
                    { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                    { WeldPropertyPanel.ParamNames.SizeDown, 6.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeDown, 11 }, // Fillet
                    { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                    { WeldPropertyPanel.ParamNames.EdgeAround, 1 }, // Edge
                    { WeldPropertyPanel.ParamNames.MadeType, 2 }, // Workshop
                    { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                    { WeldPropertyPanel.ParamNames.RefText, "" }
                };
            }

            m_defaultProps.CopyFrom(newDefaults);
        
        } //Note

    }
}
