﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;

namespace MultipleSiffeners1064
{
    class ChamferControl
    {
        public List<double> cut_xList;
        public List<double> cut_yList;
        public List<Define.CornerCutType> cut_typeList;
        public CtrlPoint[] ctrlPointList; //需要进行折角的点
        public int[] controlList; //输出chamferCtrlPointList中的点对应哪一个cut_type，这个列表存着cut_typeList的对应下标
        public List<Vec3> comparePointList;
        public DetailComponent cmp;

        public void SetCutParameters(List<double> cut_xList, List<double> cut_yList, List<Define.CornerCutType> cut_typeList)
        {
            this.cut_xList = cut_xList;
            this.cut_yList = cut_yList;
            this.cut_typeList = cut_typeList;
        }

        public void SetChamferPlace(CtrlPoint[] ctrlPointList, List<Plane> nearPlaneList)
        {
            this.ctrlPointList = new CtrlPoint[nearPlaneList.Count];
            Vec3 v0 = ctrlPointList[0].WorldPos;
            Vec3 v1 = ctrlPointList[1].WorldPos;
            Vec3 v2 = ctrlPointList[2].WorldPos;
            Plane ctrlPlane = new Plane(v0, v1 - v0, v2 - v0);
            List<Vec3> comparePointList = new List<Vec3>(); //比较控制点与这些点的距离，确定正确的对应关系
            comparePointList.Add(ctrlPlane.Intersect(nearPlaneList[nearPlaneList.Count - 1], nearPlaneList[0])); //先确定第一个点
            for (int i = 1; i < nearPlaneList.Count; i++)
            {
                Vec3 point = ctrlPlane.Intersect(nearPlaneList[i - 1], nearPlaneList[i]);
                comparePointList.Add(point);
            }
            this.ctrlPointList = ctrlPointList;
            /*            for (int i = 0; i < nearPlaneList.Count; i++)
                        {
                            if (cmp != null)
                            {
                                cmp.CreatePoint(comparePointList[i], true);
                                cmp.CreateQuad(nearPlaneList[i], 1000, 1000, true);
                            }
                        }
            cmp.CreatePoint(ctrlPointList[2].WorldPos, true);*/
        }

        public void SetChamferPlace(CtrlPoint[] ctrlPointList, List<Vec3> comparePointList)
        {
            this.ctrlPointList = ctrlPointList;
            this.comparePointList = comparePointList;
        }

        public void Execute()
        {
            if (ctrlPointList.Length < 3)
                return;
            controlList = new int[comparePointList.Count];
            int[] mark = new int[comparePointList.Count];
            for (int i = 0; i < comparePointList.Count; i++)
            {
                Vec3 comparePoint = comparePointList[i];
                double distance = double.MaxValue;
                int pos = 0;
                for(int j = 0; j < ctrlPointList.Length; j++) //遍历得到最接近comparePoint的控制点
                {
                    if (mark[j] == 1)
                        continue;
                    double nowDistance = ctrlPointList[j].WorldPos.Dist(comparePoint);
                    if(nowDistance < distance)
                    {
                        distance = nowDistance;
                        pos = j;
                    }
                }
                controlList[pos] = i;
                mark[pos] = 1; //对已经记录位置的控制点做标记
            } 
            for(int i = 0; i < ctrlPointList.Length; i++)
            {
                int index = controlList[i];
                ChamferCtrlPoint chamferCtrlPoint = ctrlPointList[i] as ChamferCtrlPoint;
                chamferCtrlPoint.CutType = this.cut_typeList[index];
                chamferCtrlPoint.CutX = this.cut_xList[index];
                chamferCtrlPoint.CutY = this.cut_yList[index];
                chamferCtrlPoint.Flush();
            }
        }
    }
}
