﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace MultipleSiffeners1064
{

    public class ComponentInstanceContext
    {
        private LoongCAD.STC.Object3D.DetailComponent m_comp;
        public static Plane DefaultPlane = new Plane(Vec3.ZERO + int.MaxValue * Vec3.AXIS_X, Vec3.AXIS_Z);

        private LoongCAD.STC.Object3D.Beam m_mainPart = null;

        private BeamPlanes m_mainPlanes;

        private List<PartBase> m_leftStiffs = new List<PartBase>();

        private List<PartBase> m_rightStiffs = new List<PartBase>();

        private List<Vec3[]> m_pointListLeftTop = new List<Vec3[]>();  //左侧加劲肋上侧的控制点
        private List<Vec3[]> m_pointListLeftBottom = new List<Vec3[]>(); //左侧加劲肋下侧的控制点
        private List<Vec3[]> m_pointListRightTop = new List<Vec3[]>();
        private List<Vec3[]> m_pointListRightBottom = new List<Vec3[]>();

        private Plane m_frontBound;
        private Plane m_backBound;
        private Plane m_topBound;
        private Plane m_bottomBound;
        private Plane m_headBound;
        private Plane m_tailBound;
        private Plane m_frontWeb = DefaultPlane;
        private Plane m_backWeb = DefaultPlane;
        private Plane m_topFlangeInside = DefaultPlane;
        private Plane m_bottomFlangeInside = DefaultPlane;

        public Mat43 coordinatesLeft, coordinatesRight; //左侧和右侧的坐标系

        ComponentInstanceContext()
        {

        }

        public static ComponentInstanceContext CreateInstance(DetailComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (!compContext.Initialize(comp))
            {
                compContext.Clear();
                return null;
            }

            return compContext;
        }

        public DetailComponent Comp { get => m_comp; set => m_comp = value; }
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public List<PartBase> LeftStiffs { get => m_leftStiffs; set => m_leftStiffs = value; }
        public List<PartBase> RightStiffs { get => m_rightStiffs; set => m_rightStiffs = value; }
        public List<Vec3[]> PointListLeftTop { get => m_pointListLeftTop; set => m_pointListLeftTop = value; }
        public List<Vec3[]> PointListLeftBottom { get => m_pointListLeftBottom; set => m_pointListLeftBottom = value; }
        public List<Vec3[]> PointListRightTop { get => m_pointListRightTop; set => m_pointListRightTop = value; }
        public List<Vec3[]> PointListRightBottom { get => m_pointListRightBottom; set => m_pointListRightBottom = value; }
        public Plane FrontBoundPlane { get => m_frontBound; set => m_frontBound = value; }
        public Plane BackBoundPlane { get => m_backBound; set => m_backBound = value; }
        public Plane TopBoundPlane { get => m_topBound; set => m_topBound = value; }
        public Plane BottomBoundPlane { get => m_bottomBound; set => m_bottomBound = value; }
        public Plane HeadBoundPlane { get => m_headBound; set => m_headBound = value; }
        public Plane TailBoundPlane { get => m_tailBound; set => m_tailBound = value; }
        public Plane FrontWebPlane { get => m_frontWeb; set => m_frontWeb = value; }
        public Plane BackWebPlane { get => m_backWeb; set => m_backWeb = value; }
        public Plane TopFlangeInsidePlane { get => m_topFlangeInside; set => m_topFlangeInside = value; }
        public Plane BottomFlangeInsidePlane { get => m_bottomFlangeInside; set => m_bottomFlangeInside = value; }

        public bool Initialize(DetailComponent comp)
        {
            if (comp == null)
                return false;

            Beam mainPart = comp.Main as Beam;
            if (mainPart == null)
                return false;

            m_comp = comp;
            m_mainPart = mainPart;
            m_mainPlanes = BeamPlanes.CreateFromBeam(m_mainPart);

            //边界面
            m_frontBound = m_mainPlanes.FrontBound;
            m_backBound = m_mainPlanes.BackBound;
            m_topBound = m_mainPlanes.TopBound;
            m_bottomBound = m_mainPlanes.BottomBound;
            m_headBound = m_mainPlanes.HeadBound;
            m_tailBound = m_mainPlanes.TailBound;
            //翼缘面
            if (m_mainPlanes.TopFlangeInside != null)
                m_topFlangeInside = m_mainPlanes.TopFlangeInside;
            if (m_mainPlanes.BottomFlangeInside != null)
                m_bottomFlangeInside = m_mainPlanes.BottomFlangeInside;
            //腹板面
            if (m_mainPlanes.FrontWeb != null)
                m_frontWeb = m_mainPlanes.FrontWeb.Clone();
            if (m_mainPlanes.BackWeb.Dist(m_mainPlanes.BackBound.Origin) > 1e-6)
                m_backWeb = m_mainPlanes.BackWeb.Clone();

            return true;
        }

        public void Swap(ref Plane p1, ref Plane p2)
        {
            Plane p = p1;
            p1 = p2;
            p2 = p;
        }

        public void SwapFrontBack()
        {
            Swap(ref m_frontBound, ref m_backBound);
            Swap(ref m_frontWeb, ref m_backWeb);
        }

        public void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_mainPlanes = null;

        }

        public void test(Plane p)
        {
            Comp.CreateQuad(p, 4000, 4000, true);
        }
        public void test(Vec3 p)
        {
            Comp.CreatePoint(p, true);
        }
        public void test(Vec3 v1, Vec3 v2)
        {
            Comp.CreateLine(v1, v2, 0, Color.Red, true);
        }
        public void test(Mat43 mat)
        {
            double len = 1000;
            Comp.CreatePoint(mat.Pos, true);
            Comp.CreateLine(mat.Pos, mat.Pos + len / 2 * mat.X.Normalize(), 0, Color.Red, true);
            Comp.CreateLine(mat.Pos, mat.Pos + len / 2 * mat.Y.Normalize(), 0, Color.Green, true);
            Comp.CreateLine(mat.Pos, mat.Pos + len / 2 * mat.Z.Normalize(), 0, Color.Purple, true);
        }

    }
}
