﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.STC.Plugins;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;
using LoongCAD.UI.Component;

namespace MultipleSiffeners1064
{
    public class DefaultsHandler
    {

        private DefaultsHandler()
        {

        }

        public static DObject ParseDefaultValues(DObject defaultConfig)
        {
            DObject defaultValues = new DObject();
            defaultValues.CopyFrom(defaultConfig);
            DefaultsHandler.FillBuiltinDefaults(defaultValues);
            return defaultValues;
        }



        /// <summary>
        /// 选择默认值，只保留当前属性值是默认值的项
        /// </summary>
        /// <param name="compProps">组件当前属性，基于此变量判断哪些属性配置为默认值</param>
        /// <param name="defaultValues">组件默认值配置</param>
        /// <returns></returns>
        public static DObject SelectDefaultValues(DObject compProps, DObject defaultValues)
        {
            //
            DObject emptyValues = new object[]
            {
               
                // 图片
                ParamNames.PIC_STIFFENER_COUNT, int.MaxValue,
                ParamNames.PIC_STIFFENER_STRIDE, new double[] {},

                // 零件
                ParamNames.PART_LEFT_PLATE_PROPS, PlatePropertyPanel.EmptyValue,
                ParamNames.PART_RIGHT_PLATE_PROPS, PlatePropertyPanel.EmptyValue,

                //参数
                ParamNames.ARG_LEFT_ROT, double.MaxValue,
                ParamNames.ARG_RIGHT_ROT, double.MaxValue,
                ParamNames.ARG_START_POS, 0,
                ParamNames.ARG_PLATES_POS, 0,
                ParamNames.ARG_PLATES_GAP, double.MaxValue,

                //加劲肋
                // 左板
                ParamNames.STIFF_LEFT_PLATE_TOP_GAP, double.MaxValue,
                ParamNames.STIFF_LEFT_PLATE_BOTTOM_GAP, double.MaxValue,
                ParamNames.STIFF_LEFT_PLATE_INSIDE_GAP, double.MaxValue,
                ParamNames.STIFF_LEFT_PLATE_OUTSIDE_GAP, double.MaxValue,
                ParamNames.STIFF_LEFT_PLATE_TYPE, 0,

                ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_TYPE, 0,
                ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_X, double.MaxValue, 
                ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_Y, double.MaxValue,
                
                ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_TYPE, 0,
                ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_X, double.MaxValue,
                ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_Y, double.MaxValue,

                ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_TYPE, 0,
                ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_X, double.MaxValue,
                ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_Y, double.MaxValue,

                ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_TYPE, 0,
                ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_X, double.MaxValue,
                ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_Y, double.MaxValue,

                // 右板
                ParamNames.STIFF_RIGHT_PLATE_TOP_GAP, double.MaxValue,
                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_GAP, double.MaxValue,
                ParamNames.STIFF_RIGHT_PLATE_INSIDE_GAP, double.MaxValue,
                ParamNames.STIFF_RIGHT_PLATE_OUTSIDE_GAP, double.MaxValue,
                ParamNames.STIFF_RIGHT_PLATE_TYPE, 0,

                ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_TYPE, 0,
                ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_X, double.MaxValue,
                ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_Y, double.MaxValue,

                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_TYPE, 0,
                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_X, double.MaxValue,
                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_Y, double.MaxValue,

                ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_TYPE, 0,
                ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_X, double.MaxValue,
                ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_Y, double.MaxValue,

                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_TYPE, 0,
                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_X, double.MaxValue,
                ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_Y, double.MaxValue,

                //// 板页
                //ParamNames.PLATE_PROPS, PlatePropertyPanel.EmptyValue,
                //ParamNames.PLATE_CHAMFER_TYPE, 0,
                //ParamNames.PLATE_CHAMFER_X, double.MaxValue,
                //ParamNames.PLATE_CHAMFER_Y, double.MaxValue,
                //ParamNames.PLATE_CREATION_MODE, 0,
                //ParamNames.PLATE_ORIENTATION, 0,
                //ParamNames.PLATE_GAP, double.MaxValue,

                //// 螺栓
                //ParamNames.BOLT_BASIC_PROPS, BoltBasicPropertyPanel.EmptyValue,
                //ParamNames.BOLT_HOLE_PROPS, BoltHolePanel.EmptyValue,
                //ParamNames.BOLT_EXTEND_PROPS, BoltExtendPanel.EmptyValue,
                //ParamNames.BOLT_SHAPE, 0,
                //ParamNames.BOLT_ORIENTATION, 0,
                //ParamNames.BOLT_HORI_COUNT, int.MaxValue,
                //ParamNames.BOLT_VERT_COUNT, int.MaxValue,
                //ParamNames.BOLT_HORI_STRIDE, new double[] {},
                //ParamNames.BOLT_VERT_STRIDE, new double[] {},
                //ParamNames.BOLT_HORI_OFFSET, double.MaxValue,
                //ParamNames.BOLT_VERT_OFFSET, double.MaxValue,
                //ParamNames.BOLT_VERT_OFFSET_MODE, 0,
                //ParamNames.BOLT_TOP_MARGIN, double.MaxValue,
                //ParamNames.BOLT_BOTTOM_MARGIN, double.MaxValue,
                //ParamNames.BOLT_RIGHT_MARGIN, double.MaxValue,

                //// 加劲肋
                //ParamNames.STIFF_TOP_NS_PROPS, PlatePropertyPanel.EmptyValue,
                //ParamNames.STIFF_TOP_FS_PROPS, PlatePropertyPanel.EmptyValue,
                //ParamNames.STIFF_BOTTOM_NS_PROPS, PlatePropertyPanel.EmptyValue,
                //ParamNames.STIFF_BOTTOM_FS_PROPS, PlatePropertyPanel.EmptyValue,
                //ParamNames.STIFF_CREATION, 0,
                //ParamNames.STIFF_ORIENTATION, 0,
                //ParamNames.STIFF_SHAPE, 0,
                //ParamNames.STIFF_CHAMFER_TYPE, 0,
                //ParamNames.STIFF_CHAMFER_X, double.MaxValue,
                //ParamNames.STIFF_CHAMFER_Y, double.MaxValue,
                //ParamNames.STIFF_WEB_GAP, double.MaxValue,
                //ParamNames.STIFF_TOP_NS_OFFSET, double.MaxValue,
                //ParamNames.STIFF_TOP_FS_OFFSET, double.MaxValue,
                //ParamNames.STIFF_BOTTOM_NS_OFFSET, double.MaxValue,
                //ParamNames.STIFF_BOTTOM_FS_OFFSET, double.MaxValue,
                //ParamNames.STIFF_NS_FLANGE_GAP_X, double.MaxValue,
                //ParamNames.STIFF_NS_FLANGE_GAP_Y, double.MaxValue,
                //ParamNames.STIFF_FS_FLANGE_GAP_X, double.MaxValue,
                //ParamNames.STIFF_FS_FLANGE_GAP_Y, double.MaxValue,

                //// 腋板
                //ParamNames.HAUNCH_TOP_PROPS, PlatePropertyPanel.EmptyValue,
                //ParamNames.HAUNCH_BOTTOM_PROPS, PlatePropertyPanel.EmptyValue,
                //ParamNames.HAUNCH_CREATION, 0,

                //// 双板
                //ParamNames.DOUBLE_PLATE_PROPS, PlatePropertyPanel.EmptyValue,
                //ParamNames.DOUBLE_CREATION_MODE, 0,
                //ParamNames.DOUBLE_PLATE_EDGE_SHAPE, 0,
                //ParamNames.DOUBLE_PLATE_BEVEL_ANGLE, double.MaxValue,
                //ParamNames.DOUBLE_HOLE_DIAMETER, double.MaxValue,
                //ParamNames.DOUBLE_SLOT_WIDTH, double.MaxValue,
                //ParamNames.DOUBLE_SLOT_HEIGHT, double.MaxValue,
                //ParamNames.DOUBLE_FLANGE_GAP, double.MaxValue,
                //ParamNames.DOUBLE_HOLE_TOP_MARGIN, double.MaxValue,
                //ParamNames.DOUBLE_HOLE_BOTTOM_MARGIN, double.MaxValue,
                //ParamNames.DOUBLE_HOLE_LEFT_MARGIN, double.MaxValue,
                //ParamNames.DOUBLE_HOLE_RIGHT_MARGIN, double.MaxValue,
                //ParamNames.DOUBLE_BOTTOM_GAP, double.MaxValue,
                //ParamNames.DOUBLE_HOLE_HORI_STRIDE, new double[] {},
                //ParamNames.DOUBLE_HOLE_VERT_STRIDE, new double[] {},
            };

            // 焊缝
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                emptyValues[ParamNames.WELD_PROPS_LIST[i]] = new DObject
                {
                    { WeldPropertyPanel.ParamNames.SizeUp, double.MaxValue },
                    { WeldPropertyPanel.ParamNames.WeldTypeUp, 0 },
                    { WeldPropertyPanel.ParamNames.GrooveAngleUp, double.MaxValue},
                    { WeldPropertyPanel.ParamNames.WeldContourUp,  0 },
                    { WeldPropertyPanel.ParamNames.WeldFinishUp, 0 },
                    { WeldPropertyPanel.ParamNames.LengthUp, double.MaxValue },
                    { WeldPropertyPanel.ParamNames.PitchUp, double.MaxValue },
                    { WeldPropertyPanel.ParamNames.RootFaceUp, double.MaxValue },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatUp, double.MaxValue },
                    { WeldPropertyPanel.ParamNames.RootGapUp, double.MaxValue },

                    { WeldPropertyPanel.ParamNames.SizeDown, double.MaxValue },
                    { WeldPropertyPanel.ParamNames.WeldTypeDown, 0 },
                    { WeldPropertyPanel.ParamNames.GrooveAngleDown, double.MaxValue},
                    { WeldPropertyPanel.ParamNames.WeldContourDown,  0 },
                    { WeldPropertyPanel.ParamNames.WeldFinishDown, 0 },
                    { WeldPropertyPanel.ParamNames.LengthDown, double.MaxValue },
                    { WeldPropertyPanel.ParamNames.PitchDown, double.MaxValue },
                    { WeldPropertyPanel.ParamNames.RootFaceDown, double.MaxValue },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatDown, double.MaxValue },
                    { WeldPropertyPanel.ParamNames.RootGapDown, double.MaxValue },

                    { WeldPropertyPanel.ParamNames.EdgeAround, 0 },
                    { WeldPropertyPanel.ParamNames.MadeType, 0 },
                    { WeldPropertyPanel.ParamNames.StitchWeld, 0 },
                    { WeldPropertyPanel.ParamNames.RefText, "" }
                };
            }

            return SelectDefaultValues(compProps, defaultValues, emptyValues);
        }

        private static DObject SelectDefaultValues(DObject compProps, DObject defaultValues, DObject emptyValues)
        {
            DObject retProps = new DObject();
            if (defaultValues == null || emptyValues == null)
                return retProps;

            // 选择当前取默认值的属性项
            if (compProps == null || compProps.Count == 0)
            {
                foreach (var emptyItem in emptyValues)
                {
                    if (!defaultValues.ContainsKey(emptyItem.Key))
                    {
                        continue;
                    }
                    if (!compProps.ContainsKey(emptyItem.Key))
                    {
                        retProps[emptyItem.Key] = defaultValues[emptyItem.Key];
                        continue;
                    }
                    if (emptyItem.Value is DObject)
                    {
                        retProps[emptyItem.Key] = DefaultsHandler.SelectDefaultValues(
                            compProps.GetDObject(emptyItem.Key),
                            defaultValues.GetDObject(emptyItem.Key),
                            (DObject)emptyItem.Value);
                    }
                    else if (object.Equals(compProps.Get(emptyItem.Key), emptyItem.Value))
                    {
                        retProps[emptyItem.Key] = defaultValues[emptyItem.Key];
                    }
                }
            }
            else
            {
                foreach (var emptyItem in emptyValues)
                {
                    if (!defaultValues.ContainsKey(emptyItem.Key))
                    {
                        continue;
                    }
                    retProps[emptyItem.Key] = defaultValues[emptyItem.Key];
                }
            }

            return retProps;
        }

        //默认值引用

        private static void FillBuiltinDefaults(DObject targetDefautls)
        {
            DefaultsHandler.FillPictureDefaults(targetDefautls);
            DefaultsHandler.FillArgDefualts(targetDefautls);
            DefaultsHandler.FillPartDefaults(targetDefautls);
            DefaultsHandler.FillStiffDefualts(targetDefautls);
        }

        private static void FillPictureDefaults(DObject targetDefautls)
        {
            DefaultsHandler.TryFillInt(targetDefautls,ParamNames.PIC_STIFFENER_COUNT);
            DefaultsHandler.TryFillDoubleArray(targetDefautls,ParamNames.PIC_STIFFENER_STRIDE);
        
        }

        private static void FillPartDefaults(DObject targetDefaults)
        {
            DefaultsHandler.TryFillDObject(
                targetDefaults, ParamNames.PART_LEFT_PLATE_PROPS, PlatePropertyPanel.EmptyValue);
            DefaultsHandler.TryFillDObject(
               targetDefaults, ParamNames.PART_RIGHT_PLATE_PROPS, PlatePropertyPanel.EmptyValue);
        }

        private static void FillArgDefualts(DObject targetDefualts)
        {
            DefaultsHandler.TryFillDouble(targetDefualts,ParamNames.ARG_LEFT_ROT);
            DefaultsHandler.TryFillDouble(targetDefualts,ParamNames.ARG_RIGHT_ROT);
            DefaultsHandler.TryFillDouble(targetDefualts,ParamNames.ARG_PLATES_GAP);
            DefaultsHandler.TryFillInt(targetDefualts, ParamNames.ARG_PLATES_POS);
            DefaultsHandler.TryFillInt(targetDefualts,ParamNames.ARG_START_POS);
        }

        private static void FillStiffDefualts(DObject targetDefualts)
        {
            //左板
            DefaultsHandler.TryFillInt(targetDefualts,ParamNames.STIFF_LEFT_PLATE_TYPE);
            DefaultsHandler.TryFillDouble(targetDefualts,ParamNames.STIFF_LEFT_PLATE_TOP_GAP);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_LEFT_PLATE_INSIDE_GAP);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_LEFT_PLATE_OUTSIDE_GAP);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_LEFT_PLATE_BOTTOM_GAP);

            DefaultsHandler.TryFillInt(targetDefualts, ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_TYPE);
            DefaultsHandler.TryFillDouble(targetDefualts,ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_X);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_Y);

            DefaultsHandler.TryFillInt(targetDefualts, ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_TYPE);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_X);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_Y);

            DefaultsHandler.TryFillInt(targetDefualts, ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_TYPE);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_X);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_Y);

            DefaultsHandler.TryFillInt(targetDefualts, ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_TYPE);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_X);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_Y);

            // 右板
            DefaultsHandler.TryFillInt(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_TYPE);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_TOP_GAP);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_INSIDE_GAP);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_OUTSIDE_GAP);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_BOTTOM_GAP);

            DefaultsHandler.TryFillInt(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_TYPE);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_X);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_Y);

            DefaultsHandler.TryFillInt(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_TYPE);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_X);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_Y);

            DefaultsHandler.TryFillInt(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_TYPE);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_X);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_Y);

            DefaultsHandler.TryFillInt(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_TYPE);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_X);
            DefaultsHandler.TryFillDouble(targetDefualts, ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_Y);

        }

        public static int TryGetInteger(
            DObject defaultValues, string propName, int curVal, int defVal)
        {
            if (!defaultValues.ContainsKey(propName) || curVal != defVal)
                return curVal;

            return defaultValues.GetInt(propName);
        }

        public static double TryGetDouble(
            DObject defaultValues, string propName, double curVal, double defVal)
        {
            if (!defaultValues.ContainsKey(propName) || curVal != defVal)
                return curVal;

            return defaultValues.GetDouble(propName);
        }
        private static void TryFillInt(DObject targetDefaults, string paramName, int emptyValue = 0)
        {
            int paramVal = targetDefaults.GetInt(paramName, emptyValue);
            if (paramVal != emptyValue)
                return;
            targetDefaults[paramName] = BuiltinDefaults.GetInteger(paramName, emptyValue);
        }

        private static void TryFillDouble(DObject targetDefaults, string paramName, double emptyValue = double.MaxValue)
        {
            double paramVal = targetDefaults.GetDouble(paramName, emptyValue);
            if (paramVal != emptyValue)
                return;
            targetDefaults[paramName] = BuiltinDefaults.GetDouble(paramName, emptyValue);
        }

        private static void TryFillDoubleArray(DObject targetDefaults, string paramName)
        {
            double[] paramVal = targetDefaults.GetDoubleArr(paramName);
            if (paramVal == null || paramVal.Length == 0)
            {
                targetDefaults[paramName] = BuiltinDefaults.GetDoubleArray(paramName);
            }
        }

        private static void TryFillDoubleBatch(DObject targetDefaults, List<string> paramNames, double emptyValue = double.MaxValue)
        {
            for (int i = 0; i < paramNames.Count; ++i)
            {
                DefaultsHandler.TryFillDouble(targetDefaults, paramNames[i], emptyValue);
            }
        }

        private static void TryFillDObject(DObject targetDefaults, string paramName, DObject emptyValues)
        {
            DObject srcDefaults = BuiltinDefaults.GetDObject(paramName);
            if (srcDefaults == null)
                return;

            DObject subDefaults = targetDefaults.GetDObject(paramName, new DObject());
            if (emptyValues != null)
            {
                foreach (var kv in srcDefaults)
                {
                    if (!subDefaults.ContainsKey(kv.Key))
                    {
                        subDefaults[kv.Key] = kv.Value;
                        continue;
                    }
                    if (!emptyValues.ContainsKey(kv.Key))
                    {
                        continue;
                    }
                    if (subDefaults[kv.Key] == emptyValues[kv.Key])
                    {
                        subDefaults[kv.Key] = kv.Value;
                    }
                }
            }
            else
            {
                foreach (var kv in srcDefaults)
                {
                    if (!subDefaults.ContainsKey(kv.Key))
                    {
                        subDefaults[kv.Key] = kv.Value;
                        continue;
                    }
                }
            }

            targetDefaults[paramName] = subDefaults;
        }
        private static void TryFillString(
           DObject targetDefaults, string paramName, string defVal, string builtinDef)
        {
            string paramVal = targetDefaults.GetString(paramName, defVal);
            if (paramVal == defVal)
            {
                targetDefaults.Add(paramName, builtinDef);
            }
        }
    }
}
