﻿using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using LoongCAD.UI.Component;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;


namespace MultipleSiffeners1064
{
    public partial class Form_PluginMain : Form_DetailComponentProps
    {
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;

        private DObject m_rootUIProps = new DObject();


        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostIntialize();
        }


        public override ComponentPlugin ReferencePlugin => m_refPlugin;

        //[Browsable(false), DefaultValue(false)]

        private void OnPostIntialize()
        {
            // 设置窗口标题
            this.Text = "多重加劲板（406）";

            foreach (TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }

            this.tb_arg_start_pos.ClearItems();
            this.tb_arg_start_pos.AddIntegerItems(0, 0, 4);
            this.tb_arg_start_pos.UpdateItems();

            this.tb_arg_plates_pos.ClearItems();
            this.tb_arg_plates_pos.AddIntegerItems(0, 0, 4);
            this.tb_arg_plates_pos.UpdateItems();

            this.tb_stiff_left_plate_type.ClearItems();
            this.tb_stiff_left_plate_type.AddIntegerItems(0, 0, 6);
            this.tb_stiff_left_plate_type.UpdateItems();

            this.tb_stiff_right_plate_type.ClearItems();
            this.tb_stiff_right_plate_type.AddIntegerItems(0, 0, 6);
            this.tb_stiff_right_plate_type.UpdateItems();

            this.tb_stiff_left_plate_top_outside_type.ClearItems();
            this.tb_stiff_left_plate_top_outside_type.AddIntegerItems(0, 0, 6);
            this.tb_stiff_left_plate_top_outside_type.UpdateItems();

            this.tb_stiff_left_plate_top_inside_type.ClearItems();
            this.tb_stiff_left_plate_top_inside_type.AddIntegerItems(0, 0, 6);
            this.tb_stiff_left_plate_top_inside_type.UpdateItems();

            this.tb_stiff_left_plate_bottom_outside_type.ClearItems();
            this.tb_stiff_left_plate_bottom_outside_type.AddIntegerItems(0, 0, 6);
            this.tb_stiff_left_plate_bottom_outside_type.UpdateItems();

            this.tb_stiff_left_plate_bottom_inside_type.ClearItems();
            this.tb_stiff_left_plate_bottom_inside_type.AddIntegerItems(0, 0, 6);
            this.tb_stiff_left_plate_bottom_inside_type.UpdateItems();

            this.tb_stiff_right_plate_top_inside_type.ClearItems();
            this.tb_stiff_right_plate_top_inside_type.AddIntegerItems(0, 0, 6);
            this.tb_stiff_right_plate_top_inside_type.UpdateItems();

            this.tb_stiff_right_plate_top_outside_type.ClearItems();
            this.tb_stiff_right_plate_top_outside_type.AddIntegerItems(0, 0, 6);
            this.tb_stiff_right_plate_top_outside_type.UpdateItems();

            this.tb_stiff_right_plate_bottom_inside_type.ClearItems();
            this.tb_stiff_right_plate_bottom_inside_type.AddIntegerItems(0, 0, 6);
            this.tb_stiff_right_plate_bottom_inside_type.UpdateItems();

            this.tb_stiff_right_plate_bottom_outside_type.ClearItems();
            this.tb_stiff_right_plate_bottom_outside_type.AddIntegerItems(0, 0, 6);
            this.tb_stiff_right_plate_bottom_outside_type.UpdateItems();

        }

        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null)
            {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }

        /// <summary>
        /// 当前组件需要重写此方法，并刷新螺栓基础面板
        /// </summary>
        /// <param name="root"></param>
        public override void ClearUI(Control root = null)
        {
            base.ClearUI(root);
        }

        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }


        private void btn_weld_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }
            m_weldForm.Show(m_rootUIProps, null);
        }
        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_rootUIProps.CopyFrom(srcProps);
        }

        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }

        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_rootUIProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_rootUIProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_rootUIProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_rootUIProps;

            return comp.UIProps;
        }

        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked)
            {
                retProps.CopyFrom(m_rootUIProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }


        private void tb_stiff_left_plate_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_stiff_left_plate_type.SelectedIntKey;
            switch (select)
            {
                case 2: //靠上方形
                    this.act_stiff_left_plate_inside_gap.Enabled = false;
                    this.tb_stiff_left_plate_inside_gap.Enabled = false;
                    this.act_stiff_left_plate_outside_gap.Enabled = true;
                    this.tb_stiff_left_plate_outside_gap.Enabled = true;
                    this.act_stiff_left_plate_top_gap.Enabled = true;
                    this.tb_stiff_left_plate_top_gap.Enabled = true;
                    this.act_stiff_left_plate_bottom_gap.Enabled = true;
                    this.tb_stiff_left_plate_bottom_gap.Enabled = true;
                    break;
                case 3: //靠下方形
                    this.act_stiff_left_plate_inside_gap.Enabled = false;
                    this.tb_stiff_left_plate_inside_gap.Enabled = false;
                    this.act_stiff_left_plate_outside_gap.Enabled = true;
                    this.tb_stiff_left_plate_outside_gap.Enabled = true;
                    this.act_stiff_left_plate_top_gap.Enabled = true;
                    this.tb_stiff_left_plate_top_gap.Enabled = true;
                    this.act_stiff_left_plate_bottom_gap.Enabled = true;
                    this.tb_stiff_left_plate_bottom_gap.Enabled = true;
                    break;
                case 4: //三角
                    this.act_stiff_left_plate_inside_gap.Enabled = false;
                    this.tb_stiff_left_plate_inside_gap.Enabled = false;
                    this.act_stiff_left_plate_outside_gap.Enabled = true;
                    this.tb_stiff_left_plate_outside_gap.Enabled = true;
                    this.act_stiff_left_plate_top_gap.Enabled = false;
                    this.tb_stiff_left_plate_top_gap.Enabled = false;
                    this.act_stiff_left_plate_bottom_gap.Enabled = true;
                    this.tb_stiff_left_plate_bottom_gap.Enabled = true;
                    break;
                case 5: //无
                    this.act_stiff_left_plate_inside_gap.Enabled = false;
                    this.tb_stiff_left_plate_inside_gap.Enabled = false;
                    this.act_stiff_left_plate_outside_gap.Enabled = false;
                    this.tb_stiff_left_plate_outside_gap.Enabled = false;
                    this.act_stiff_left_plate_top_gap.Enabled = false;
                    this.tb_stiff_left_plate_top_gap.Enabled = false;
                    this.act_stiff_left_plate_bottom_gap.Enabled = false;
                    this.tb_stiff_left_plate_bottom_gap.Enabled = false;
                    break;
                default: //都有效
                    this.act_stiff_left_plate_inside_gap.Enabled = true;
                    this.tb_stiff_left_plate_inside_gap.Enabled = true;
                    this.act_stiff_left_plate_outside_gap.Enabled = true;
                    this.tb_stiff_left_plate_outside_gap.Enabled = true;
                    this.act_stiff_left_plate_top_gap.Enabled = true;
                    this.tb_stiff_left_plate_top_gap.Enabled = true;
                    this.act_stiff_left_plate_bottom_gap.Enabled = true;
                    this.tb_stiff_left_plate_bottom_gap.Enabled = true;
                    break;
            }
        }

        private void tb_stiff_right_plate_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_stiff_right_plate_type.SelectedIntKey;
            switch (select)
            {
                case 2: //靠上方形
                    this.act_stiff_right_plate_inside_gap.Enabled = false;
                    this.tb_stiff_right_plate_inside_gap.Enabled = false;
                    this.act_stiff_right_plate_outside_gap.Enabled = true;
                    this.tb_stiff_right_plate_outside_gap.Enabled = true;
                    this.act_stiff_right_plate_top_gap.Enabled = true;
                    this.tb_stiff_right_plate_top_gap.Enabled = true;
                    this.act_stiff_right_plate_bottom_gap.Enabled = true;
                    this.tb_stiff_right_plate_bottom_gap.Enabled = true;
                    break;
                case 3: //靠下方形
                    this.act_stiff_right_plate_inside_gap.Enabled = false;
                    this.tb_stiff_right_plate_inside_gap.Enabled = false;
                    this.act_stiff_right_plate_outside_gap.Enabled = true;
                    this.tb_stiff_right_plate_outside_gap.Enabled = true;
                    this.act_stiff_right_plate_top_gap.Enabled = true;
                    this.tb_stiff_right_plate_top_gap.Enabled = true;
                    this.act_stiff_right_plate_bottom_gap.Enabled = true;
                    this.tb_stiff_right_plate_bottom_gap.Enabled = true;
                    break;
                case 4: //三角
                    this.act_stiff_right_plate_inside_gap.Enabled = false;
                    this.tb_stiff_right_plate_inside_gap.Enabled = false;
                    this.act_stiff_right_plate_outside_gap.Enabled = true;
                    this.tb_stiff_right_plate_outside_gap.Enabled = true;
                    this.act_stiff_right_plate_top_gap.Enabled = false;
                    this.tb_stiff_right_plate_top_gap.Enabled = false;
                    this.act_stiff_right_plate_bottom_gap.Enabled = true;
                    this.tb_stiff_right_plate_bottom_gap.Enabled = true;
                    break;
                case 5: //无
                    this.act_stiff_right_plate_inside_gap.Enabled = false;
                    this.tb_stiff_right_plate_inside_gap.Enabled = false;
                    this.act_stiff_right_plate_outside_gap.Enabled = false;
                    this.tb_stiff_right_plate_outside_gap.Enabled = false;
                    this.act_stiff_right_plate_top_gap.Enabled = false;
                    this.tb_stiff_right_plate_top_gap.Enabled = false;
                    this.act_stiff_right_plate_bottom_gap.Enabled = false;
                    this.tb_stiff_right_plate_bottom_gap.Enabled = false;
                    break;
                default: //都有效
                    this.act_stiff_right_plate_inside_gap.Enabled = true;
                    this.tb_stiff_right_plate_inside_gap.Enabled = true;
                    this.act_stiff_right_plate_outside_gap.Enabled = true;
                    this.tb_stiff_right_plate_outside_gap.Enabled = true;
                    this.act_stiff_right_plate_top_gap.Enabled = true;
                    this.tb_stiff_right_plate_top_gap.Enabled = true;
                    this.act_stiff_right_plate_bottom_gap.Enabled = true;
                    this.tb_stiff_right_plate_bottom_gap.Enabled = true;
                    break;
            }
        }

        private void tb_stiff_left_plate_top_outside_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_stiff_left_plate_top_outside_type.SelectedIntKey;
            if(select == 3 || select == 4)
            {
                this.tb_stiff_left_plate_top_outside_x.Enabled = true;
                this.act_stiff_left_plate_top_outside_x.Enabled = true;
                this.tb_stiff_left_plate_top_outside_y.Enabled = false;
                this.act_stiff_left_plate_top_outside_y.Enabled = false;
            }
            else if(select == 0 || select == 1)
            {
                this.tb_stiff_left_plate_top_outside_x.Enabled = false;
                this.act_stiff_left_plate_top_outside_x.Enabled = false;
                this.tb_stiff_left_plate_top_outside_y.Enabled = false;
                this.act_stiff_left_plate_top_outside_y.Enabled = false;
            }
            else
            {
                this.tb_stiff_left_plate_top_outside_x.Enabled = true;
                this.act_stiff_left_plate_top_outside_x.Enabled = true;
                this.tb_stiff_left_plate_top_outside_y.Enabled = true;
                this.act_stiff_left_plate_top_outside_y.Enabled = true;
            }
        }

        private void tb_stiff_left_plate_top_inside_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_stiff_left_plate_top_inside_type.SelectedIntKey;
            if (select == 3 || select == 4)
            {
                this.tb_stiff_left_plate_top_inside_y.Enabled = false;
                this.act_stiff_left_plate_top_inside_y.Enabled = false;
                this.tb_stiff_left_plate_top_inside_x.Enabled = true;
                this.act_stiff_left_plate_top_inside_x.Enabled = true;
            }
            else if (select == 1)
            {
                this.tb_stiff_left_plate_top_inside_x.Enabled = false;
                this.act_stiff_left_plate_top_inside_x.Enabled = false;
                this.tb_stiff_left_plate_top_inside_y.Enabled = false;
                this.act_stiff_left_plate_top_inside_y.Enabled = false;
            }
            else
            {
                this.tb_stiff_left_plate_top_inside_x.Enabled = true;
                this.act_stiff_left_plate_top_inside_x.Enabled = true;
                this.tb_stiff_left_plate_top_inside_y.Enabled = true;
                this.act_stiff_left_plate_top_inside_y.Enabled = true;
            }
        }

        private void tb_stiff_left_plate_bottom_outside_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_stiff_left_plate_bottom_outside_type.SelectedIntKey;
            if (select == 3 || select == 4)
            {
                this.tb_stiff_left_plate_bottom_outside_y.Enabled = false;
                this.act_stiff_left_plate_bottom_outside_y.Enabled = false;
                this.tb_stiff_left_plate_bottom_outside_x.Enabled = true;
                this.act_stiff_left_plate_bottom_outside_x.Enabled = true;
            }
            else if (select == 0 || select == 1)
            {
                this.tb_stiff_left_plate_bottom_outside_x.Enabled = false;
                this.act_stiff_left_plate_bottom_outside_x.Enabled = false;
                this.tb_stiff_left_plate_bottom_outside_y.Enabled = false;
                this.act_stiff_left_plate_bottom_outside_y.Enabled = false;
            }
            else
            {
                this.tb_stiff_left_plate_bottom_outside_x.Enabled = true;
                this.act_stiff_left_plate_bottom_outside_x.Enabled = true;
                this.tb_stiff_left_plate_bottom_outside_y.Enabled = true;
                this.act_stiff_left_plate_bottom_outside_y.Enabled = true;
            }
        }

        private void tb_stiff_left_plate_bottom_inside_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_stiff_left_plate_bottom_inside_type.SelectedIntKey;
            if (select == 3 || select == 4)
            {
                this.tb_stiff_left_plate_bottom_inside_y.Enabled = false;
                this.act_stiff_left_plate_bottom_inside_y.Enabled = false;
                this.tb_stiff_left_plate_bottom_inside_x.Enabled = true;
                this.act_stiff_left_plate_bottom_inside_x.Enabled = true;
            }
            else if (select == 1)
            {
                this.tb_stiff_left_plate_bottom_inside_x.Enabled = false;
                this.act_stiff_left_plate_bottom_inside_x.Enabled = false;
                this.tb_stiff_left_plate_bottom_inside_y.Enabled = false;
                this.act_stiff_left_plate_bottom_inside_y.Enabled = false;
            }
            else
            {
                this.tb_stiff_left_plate_bottom_inside_x.Enabled = true;
                this.act_stiff_left_plate_bottom_inside_x.Enabled = true;
                this.tb_stiff_left_plate_bottom_inside_y.Enabled = true;
                this.act_stiff_left_plate_bottom_inside_y.Enabled = true;
            }
        }

        private void tb_stiff_right_plate_top_outside_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_stiff_right_plate_top_outside_type.SelectedIntKey;
            if (select == 3 || select == 4)
            {
                this.tb_stiff_right_plate_top_outside_y.Enabled = false;
                this.act_stiff_right_plate_top_outside_y.Enabled = false;
                this.tb_stiff_right_plate_top_outside_x.Enabled = true;
                this.act_stiff_right_plate_top_outside_x.Enabled = true;
            }
            else if (select == 0 || select == 1)
            {
                this.tb_stiff_right_plate_top_outside_x.Enabled = false;
                this.act_stiff_right_plate_top_outside_x.Enabled = false;
                this.tb_stiff_right_plate_top_outside_y.Enabled = false;
                this.act_stiff_right_plate_top_outside_y.Enabled = false;
            }
            else
            {
                this.tb_stiff_right_plate_top_outside_x.Enabled = true;
                this.act_stiff_right_plate_top_outside_x.Enabled = true;
                this.tb_stiff_right_plate_top_outside_y.Enabled = true;
                this.act_stiff_right_plate_top_outside_y.Enabled = true;
            }
        }

        private void tb_stiff_right_plate_top_inside_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_stiff_right_plate_top_inside_type.SelectedIntKey;
            if (select == 3 || select == 4)
            {
                this.tb_stiff_right_plate_top_inside_y.Enabled = false;
                this.act_stiff_right_plate_top_inside_y.Enabled = false;
                this.tb_stiff_right_plate_top_inside_x.Enabled = true;
                this.act_stiff_right_plate_top_inside_x.Enabled = true;
            }
            else if (select == 1)
            {
                this.tb_stiff_right_plate_top_inside_x.Enabled = false;
                this.act_stiff_right_plate_top_inside_x.Enabled = false;
                this.tb_stiff_right_plate_top_inside_y.Enabled = false;
                this.act_stiff_right_plate_top_inside_y.Enabled = false;
            }
            else
            {
                this.tb_stiff_right_plate_top_inside_x.Enabled = true;
                this.act_stiff_right_plate_top_inside_x.Enabled = true;
                this.tb_stiff_right_plate_top_inside_y.Enabled = true;
                this.act_stiff_right_plate_top_inside_y.Enabled = true;
            }
        }

        private void tb_stiff_right_plate_bottom_outside_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_stiff_right_plate_bottom_outside_type.SelectedIntKey;
            if (select == 3 || select == 4)
            {
                this.tb_stiff_right_plate_bottom_outside_y.Enabled = false;
                this.act_stiff_right_plate_bottom_outside_y.Enabled = false;
                this.tb_stiff_right_plate_bottom_outside_x.Enabled = true;
                this.act_stiff_right_plate_bottom_outside_x.Enabled = true;
            }
            else if (select == 0 || select == 1)
            {
                this.tb_stiff_right_plate_bottom_outside_x.Enabled = false;
                this.act_stiff_right_plate_bottom_outside_x.Enabled = false;
                this.tb_stiff_right_plate_bottom_outside_y.Enabled = false;
                this.act_stiff_right_plate_bottom_outside_y.Enabled = false;
            }
            else
            {
                this.tb_stiff_right_plate_bottom_outside_x.Enabled = true;
                this.act_stiff_right_plate_bottom_outside_x.Enabled = true;
                this.tb_stiff_right_plate_bottom_outside_y.Enabled = true;
                this.act_stiff_right_plate_bottom_outside_y.Enabled = true;
            }
        }

        private void tb_stiff_right_plate_bottom_inside_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_stiff_right_plate_bottom_inside_type.SelectedIntKey;
            if (select == 3 || select == 4)
            {
                this.tb_stiff_right_plate_bottom_inside_y.Enabled = false;
                this.act_stiff_right_plate_bottom_inside_y.Enabled = false;
                this.tb_stiff_right_plate_bottom_inside_x.Enabled = true;
                this.act_stiff_right_plate_bottom_inside_x.Enabled = true;
            }
            else if (select == 1)
            {
                this.tb_stiff_right_plate_bottom_inside_x.Enabled = false;
                this.act_stiff_right_plate_bottom_inside_x.Enabled = false;
                this.tb_stiff_right_plate_bottom_inside_y.Enabled = false;
                this.act_stiff_right_plate_bottom_inside_y.Enabled = false;
            }
            else
            {
                this.tb_stiff_right_plate_bottom_inside_x.Enabled = true;
                this.act_stiff_right_plate_bottom_inside_x.Enabled = true;
                this.tb_stiff_right_plate_bottom_inside_y.Enabled = true;
                this.act_stiff_right_plate_bottom_inside_y.Enabled = true;
            }
        }
    }
}
