﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;

namespace MultipleSiffeners1064
{
    class StiffHandler
    {
        private ComponentInstanceContext m_context = null;
        private ParamsHandler m_paramsHandler = null;
        Mat43 coordinatesLeft, coordinatesRight; //左侧和右侧的坐标系

        public StiffHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            // 图形页
            int count = m_paramsHandler.GetIntValue(ParamNames.PIC_STIFFENER_COUNT);
            double[] stiffStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.PIC_STIFFENER_STRIDE);

            //零件页
            DObject pro = m_paramsHandler.GetDObjectValue(ParamNames.PART_LEFT_PLATE_PROPS);
            PlateProperties platePropsLeft = new PlateProperties();
            platePropsLeft.SetProps(pro);
            double stiffThicknessLeft = platePropsLeft.Thickness ;
            double stiffWidthLeft = platePropsLeft.Width; //自适应
            pro = m_paramsHandler.GetDObjectValue(ParamNames.PART_RIGHT_PLATE_PROPS);
            PlateProperties platePropsRight = new PlateProperties();
            platePropsRight.SetProps(pro);
            double stiffThicknessRight = platePropsRight.Thickness;
            double stiffWidthRight = platePropsRight.Width;

            //参数页
            double roltAngleLeft = m_paramsHandler.GetDoubleValue(ParamNames.ARG_LEFT_ROT); //角度制
            if (Math.Abs(roltAngleLeft) > 60.0)
                roltAngleLeft = 0;
            roltAngleLeft *= Math.PI / 180; //转为弧度制
            double roltAngleRight = m_paramsHandler.GetDoubleValue(ParamNames.ARG_RIGHT_ROT);
            if (Math.Abs(roltAngleRight) > 60.0)
                roltAngleRight = 0;
            roltAngleRight *= -Math.PI / 180; //修改方向
            int arrangeMode = m_paramsHandler.GetIntValue(ParamNames.ARG_START_POS); //排列方向，默认居中
            double distanceWithRoltPoint = m_paramsHandler.GetDoubleValue(ParamNames.ARG_PLATES_GAP); //第一个加劲肋与旋转中心的间距
            int distanceMode = m_paramsHandler.GetIntValue(ParamNames.ARG_PLATES_POS); //第一个加劲肋的（中心、左边缘、右边缘）与旋转中心的间距

            //加劲肋页
            int stiffShapeModeLeft = m_paramsHandler.GetIntValue(ParamNames.STIFF_LEFT_PLATE_TYPE); //加劲肋形状
            double marginTopLeft = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_TOP_GAP); //加劲肋上侧与上翼缘内侧的间距
            double marginBottomLeft = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_BOTTOM_GAP); //加劲肋下侧与下翼缘内侧的间距
            double marginOutsideLeft = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_OUTSIDE_GAP); //加劲肋外侧与零件前侧的间距
            double marginInsideLeft = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_INSIDE_GAP); //加劲肋内侧与零件腹板的间距
            int stiffShapeModeRight = m_paramsHandler.GetIntValue(ParamNames.STIFF_RIGHT_PLATE_TYPE); //右侧的
            double marginTopRight = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_TOP_GAP);
            double marginBottomRight = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_BOTTOM_GAP);
            double marginOutsideRight = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_OUTSIDE_GAP);
            double marginInsideRight = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_INSIDE_GAP);

            //折角处理
            int chamferMode1Left = m_paramsHandler.GetIntValue(ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_TYPE); 
            double chamferX1Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_X), chamferY1Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_Y);
            int chamferMode2Left = m_paramsHandler.GetIntValue(ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_TYPE);
            double chamferX2Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_X), chamferY2Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_Y);
            int chamferMode3Left = m_paramsHandler.GetIntValue(ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_TYPE); 
            double chamferX3Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_X), chamferY3Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_Y);
            int chamferMode4Left = m_paramsHandler.GetIntValue(ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_TYPE);
            double chamferX4Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_X),chamferY4Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_Y);
            //
            int chamferMode1Right = m_paramsHandler.GetIntValue(ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_TYPE); //上——内
            double chamferX1Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_X), chamferY1Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_Y);
            int chamferMode2Right = m_paramsHandler.GetIntValue(ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_TYPE); //上——外
            double chamferX2Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_X), chamferY2Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_Y);
            int chamferMode3Right = m_paramsHandler.GetIntValue(ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_TYPE); //下——内
            double chamferX3Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_X), chamferY3Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_Y);
            int chamferMode4Right = m_paramsHandler.GetIntValue(ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_TYPE); //下——外
            double chamferX4Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_X), chamferY4Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_Y);


            /*
             * 说明：
             * 1. 每根加劲肋有3或4个控制点，延控制平面方向延申得到另外3或4个控制点（用这6或8个点作为焊缝输入）
             * 2. 分别为两侧的加劲肋组建立空间坐标系，其中：
             *    (1) x轴方向为坐标轴正方向(梁的方向Head=>Tail)
             *    (2) y轴方向为这一侧的腹板面方向
             *    (3) z轴方向为TopFlangePlane的方向
             *    (4) 原点为控制点在BottomFlangeInside和web交线上的投影点
             * 3. 四个控制点中，1号控制点为最靠近腹板面的点，宽度方向延申得到2号点，厚度方向延申得到3号点，对角为4号点
             */
/*
            Vec3 cp = m_context.Comp.CtrlPoints[0].WorldPos;
            m_context.test(m_context.Comp.LocalMatrix);*/

            //建立坐标系
            Vec3 ctrlPoint = m_context.Comp.CtrlPoints[0].WorldPos;
            Vec3 x; //x轴
            Vec3 headNormal = m_context.HeadBoundPlane.Normal;
            Vec3 tailNormal = m_context.TailBoundPlane.Normal;
            if (headNormal.Dot(Vec3.AXIS_X) > 0) //头向量与x轴夹角为正，排列方向取头向量
                x = headNormal.Normalize();
            else if (headNormal.Dot(Vec3.AXIS_X) < 0)
                x = tailNormal.Normalize();
            else //head面垂直于zy平面的情况
            {
                if (headNormal.Dot(Vec3.AXIS_Y) > 0)
                    x = headNormal.Normalize();
                else if (headNormal.Dot(Vec3.AXIS_Y) < 0)
                    x = tailNormal.Normalize(); 
                else //head面平行于xy面
                    x = Vec3.AXIS_Z;
            }
            Vec3 z = m_context.TopBoundPlane.Normal.Normalize(); //z轴
            //确定左右
            Vec3 left = x.Cross(z);
            if(left.Dot(m_context.FrontBoundPlane.Normal) < 0)
                m_context.SwapFrontBack();
            Plane bottomFlangeInside = m_context.BottomFlangeInsidePlane;
            Plane temp = new Plane(ctrlPoint, x); //临时面，过ctrlPoint将梁截断
            Plane hCenterPlane = m_context.BottomBoundPlane.Move(-m_context.TopBoundPlane.Dist(m_context.BottomBoundPlane.Origin) / 2);
            bool isParallel = m_context.TopFlangeInsidePlane.Normal.Cross(m_context.BottomFlangeInsidePlane.Normal) == Vec3.ZERO;
            //左侧坐标系
            Plane frontWebPlane = m_context.FrontWebPlane;
            Vec3 o = temp.Intersect(bottomFlangeInside, frontWebPlane);
            Vec3 tempPointLeft = m_context.FrontBoundPlane.Intersect(temp, bottomFlangeInside); //辅助点，用于确定y轴方向
            Vec3 y = (tempPointLeft - o).Normalize(); //左侧y轴
            coordinatesLeft = new Mat43(x, y, z, o);
            m_context.coordinatesLeft = coordinatesLeft;
            //右侧坐标系
            Plane backWebPlane = m_context.BackWebPlane;
            o = temp.Intersect(bottomFlangeInside, backWebPlane);
            Vec3 tempPointRight = m_context.BackBoundPlane.Intersect(temp, bottomFlangeInside);
            y = (tempPointRight - o).Normalize(); //右侧y轴
            coordinatesRight = new Mat43(x, y, z, o);
            m_context.coordinatesRight = coordinatesRight;

            //零件参数——腹板厚度
            double webThickness = frontWebPlane.Dist(backWebPlane.Origin);
            //零件参数——上下翼缘内表面之间的间距
            double flangeInsideDistance = m_context.TopFlangeInsidePlane.Dist(bottomFlangeInside.Origin);
            double flangeOutsideDistance = m_context.FrontBoundPlane.Dist(m_context.FrontWebPlane.Origin);
            //加劲肋参数——间距数组
            List<double> strideList = new List<double>(); 
            for (int i = 0; i < count - 1; ++i)
            {
                strideList.Add(stiffStride[Math.Min(i, stiffStride.Length - 1)]); //count-1个间距
            }
            
            //左侧
            do
            {
                if (stiffShapeModeLeft == 5 || frontWebPlane == ComponentInstanceContext.DefaultPlane || !Util.PlatePropIsValid(platePropsLeft))
                    break;
                double stiffTotalLengthLeft = strideList.Sum() + stiffThicknessLeft;
                //加劲肋参数——默认边距
                switch (stiffShapeModeLeft)
                {
                    case 2:
                        marginInsideLeft = 0; //重置变为灰色的选项
                        if (marginBottomLeft == double.MaxValue)
                            marginBottomLeft = flangeInsideDistance / 2;
                        break;
                    case 3:
                        marginInsideLeft = 0;
                        if (marginTopLeft == 0)
                            marginTopLeft = flangeInsideDistance / 2;
                        break;
                    case 4:
                        marginInsideLeft = 0;
                        marginTopLeft = 0;
                        if (marginBottomLeft == double.MaxValue)
                            marginBottomLeft = flangeInsideDistance - (flangeOutsideDistance - marginOutsideLeft);
                        break;
                }
                if (marginBottomLeft == double.MaxValue)
                    marginBottomLeft = 0;
                //加劲肋参数——默认宽度
                if (stiffWidthLeft == double.MaxValue) //板宽度未设置时，自适应。冲突时忽略marginOutsideLeft，保留板宽度和marginInsideLeft
                    stiffWidthLeft = m_context.FrontBoundPlane.Dist(frontWebPlane.Origin) - marginOutsideLeft - marginInsideLeft;
                //确定一组控制点位置(一组加劲肋的第一个)
                {
                    Vec3 point1Bottom, point2Bottom, point3Bottom, point4Bottom;
                    Vec3 point1Top, point2Top, point3Top, point4Top;
                    //确定1号点位置（下）
                    {
                        double xMove;
                        if (roltAngleLeft >= 0)
                        {
                            xMove = webThickness / 2 * Math.Tan(roltAngleLeft) + stiffThicknessLeft / 2 / Math.Cos(roltAngleLeft); //基础情况（默认distanceMode为0或1）
                        }
                        else
                        {
                            xMove = webThickness / 2 * Math.Tan(roltAngleLeft) - stiffThicknessLeft / 2 / Math.Cos(roltAngleLeft);
                        }
                        if (arrangeMode == 3) //第一个加劲肋向哪个方向移动是通过arrange确定的
                        {
                            xMove -= distanceWithRoltPoint; //加上旋转中心距旋转线间距
                        }
                        else
                        {
                            xMove += distanceWithRoltPoint;
                        }
                        switch (arrangeMode)
                        {
                            case 2:
                                if(distanceMode == 2)
                                    xMove += stiffThicknessLeft / 2 ;
                                if(distanceMode == 3)
                                    xMove -= stiffThicknessLeft / 2 ;
                                break;
                            case 3:
                                if(distanceMode == 2)
                                    xMove -= stiffThicknessLeft / 2 ;
                                if(distanceMode == 3)
                                    xMove += stiffThicknessLeft / 2 ;
                                break;
                            default: //加劲肋对称放置，控制点移向边缘
                                xMove += stiffTotalLengthLeft / 2 - stiffThicknessLeft / 2;
                                if (distanceMode == 2)
                                    xMove += stiffThicknessLeft / 2 ; 
                                if (distanceMode == 3)
                                    xMove -= stiffThicknessLeft / 2 ; 
                                break;
                        }
                        double yMove = marginInsideLeft;
                        double zMove = marginBottomLeft;
                        if (stiffShapeModeLeft == 2) //靠上的正方形加劲肋，需另外计算
                            zMove = flangeInsideDistance - marginTopLeft - marginBottomLeft;
                        point1Bottom = MovePoint(xMove, yMove, zMove, coordinatesLeft.Pos, coordinatesLeft);
                    }
                    //确定2号点位置（下）
                    {
                        double xMove = stiffWidthLeft * Math.Sin(roltAngleLeft);
                        double yMove = stiffWidthLeft * Math.Cos(roltAngleLeft);
                        point2Bottom = MovePoint(xMove, yMove, 0, point1Bottom, coordinatesLeft);
                    }
                    //确定3号点位置（下）
                    {
                        double xMove = stiffThicknessLeft * Math.Cos(roltAngleLeft);
                        double yMove = -stiffThicknessLeft * Math.Sin(roltAngleLeft);
                        if (roltAngleLeft >= 0)
                        {
                            xMove *= -1;
                            yMove *= -1;
                        }
                        point3Bottom = MovePoint(xMove, yMove, 0, point1Bottom, coordinatesLeft);
                    }
                    //确定4号点位置（下）
                    {
                        double xMove = stiffWidthLeft * Math.Sin(roltAngleLeft);
                        double yMove = stiffWidthLeft * Math.Cos(roltAngleLeft);
                        point4Bottom = MovePoint(xMove, yMove, 0, point3Bottom, coordinatesLeft);
                    }
                    //确定所有控制点位置（上）
                    {
                        double zMove = 0;
                        switch (stiffShapeModeLeft)
                        {
                            case 2: //靠上侧的正方形
                                zMove = marginBottomLeft;
                                break;
                            case 3: //靠下侧的正方形
                                zMove = marginTopLeft;
                                break;
                            case 4: //靠上侧的三角形
                                zMove = flangeInsideDistance - marginBottomLeft;
                                break;
                            default:
                                zMove = flangeInsideDistance - marginTopLeft - marginBottomLeft;
                                break;
                        }
                        point1Top = MovePoint(0, 0, zMove, point1Bottom, coordinatesLeft);
                        point2Top = MovePoint(0, 0, zMove, point2Bottom, coordinatesLeft);
                        point3Top = MovePoint(0, 0, zMove, point3Bottom, coordinatesLeft);
                        point4Top = MovePoint(0, 0, zMove, point4Bottom, coordinatesLeft);
                    }
                    if (stiffShapeModeLeft == 4)
                    {
                        point2Bottom = Vec3.ZERO;
                        point4Bottom = Vec3.ZERO;
                    }
                    m_context.PointListLeftBottom.Add(new Vec3[] { point1Bottom, point2Bottom, point3Bottom, point4Bottom });
                    //当上下翼缘内侧不平行时
                    if(!isParallel)
                    {
                        point1Top = SymmetryPoint(hCenterPlane, point1Bottom);
                        point2Top = SymmetryPoint(hCenterPlane, point2Bottom);
                        point3Top = SymmetryPoint(hCenterPlane, point3Bottom);
                        point4Top = SymmetryPoint(hCenterPlane, point4Bottom);
                    }
                    m_context.PointListLeftTop.Add(new Vec3[] { point1Top, point2Top, point3Top, point4Top });
                }
                //创建多组控制点
                {
                    double orient = arrangeMode == 2 ? 1 : -1;
                    Vec3[] topPoints = CloneVec3Array(m_context.PointListLeftTop[0]);
                    Vec3[] bottomPoints = CloneVec3Array(m_context.PointListLeftBottom[0]);
                    for (int i = 0; i < count - 1; i++)
                    {
                        double gap = strideList[i] * orient;
                        for (int j = 0; j < 4; j++)
                        {
                            topPoints[j] = MovePoint(gap, 0, 0, topPoints[j], coordinatesLeft);
                            bottomPoints[j] = MovePoint(gap, 0, 0, bottomPoints[j], coordinatesLeft);
                        }
                        m_context.PointListLeftTop.Add(topPoints);
                        m_context.PointListLeftBottom.Add(bottomPoints);
                        topPoints = CloneVec3Array(topPoints);
                        bottomPoints = CloneVec3Array(bottomPoints);
                    }
                }
                //创建加劲肋
                {
                    if (!isParallel && stiffShapeModeLeft != 0 && stiffShapeModeLeft != 1)
                        stiffShapeModeLeft = 1;
                    if (chamferMode1Left == 3 || chamferMode1Left == 4)
                        chamferY1Left = chamferX1Left;
                    if (chamferMode2Left == 3 || chamferMode2Left == 4)
                        chamferY2Left = chamferX2Left;
                    if (chamferMode3Left == 3 || chamferMode3Left == 4)
                        chamferY3Left = chamferX3Left;
                    if (chamferMode4Left == 3 || chamferMode4Left == 4)
                        chamferY4Left = chamferX4Left;
                    //angle < 0 时
                    List<int> cutTypes1 = new List<int> { chamferMode3Left, chamferMode4Left, chamferMode2Left, chamferMode1Left };
                    List<double> xCuts1 = new List<double> { chamferX3Left, chamferY4Left, chamferX2Left, chamferY1Left };
                    List<double> yCuts1 = new List<double> { chamferY3Left, chamferX4Left, chamferY2Left, chamferX1Left };
                    //angle > 0 时
                    List<int> cutTypes2 = new List<int> { chamferMode4Left, chamferMode3Left, chamferMode1Left, chamferMode2Left };
                    List<double> xCuts2 = new List<double> { chamferX4Left, chamferY3Left, chamferX1Left, chamferY2Left };
                    List<double> yCuts2 = new List<double> { chamferY4Left, chamferX3Left, chamferY1Left, chamferX2Left };
                    //三角形加劲肋单独处理
                    if (stiffShapeModeLeft == 4)
                    {
                        cutTypes1.RemoveAt(1); xCuts1.RemoveAt(1); yCuts1.RemoveAt(1);
                        cutTypes2 = new List<int> {chamferMode3Left, chamferMode2Left , chamferMode1Left};
                        xCuts2 = new List<double> {chamferY3Left, chamferY2Left, chamferX1Left };
                        yCuts2 = new List<double> {chamferX3Left, chamferX2Left, chamferY1Left };
                    }ChamferControl chamferControl1 = new ChamferControl();
                    List<Define.CornerCutType> cutTypeList1 = ConvertToCornerCutType(cutTypes1);
                    chamferControl1.SetCutParameters(xCuts1, yCuts1, cutTypeList1);
                    ChamferControl chamferControl2 = new ChamferControl();
                    List<Define.CornerCutType> cutTypeList2 = ConvertToCornerCutType(cutTypes2);
                    chamferControl2.SetCutParameters(xCuts2, yCuts2, cutTypeList2);

                    for (int i = 0; i < m_context.PointListLeftTop.Count; i++)
                    {
                        Vec3 topInsidePoint = m_context.PointListLeftTop[i][0];
                        Vec3 topOutsidePoint = m_context.PointListLeftTop[i][1];
                        Vec3 bottomInsidePoint = m_context.PointListLeftBottom[i][0];
                        Vec3 bottomOutsidePoint = m_context.PointListLeftBottom[i][1];
                        SteelPlate sp;
                        if (roltAngleLeft < 0)
                        {
                            Vec3[] pointList = new Vec3[] { bottomOutsidePoint, bottomInsidePoint, topInsidePoint, topOutsidePoint };
                            List<Vec3> comparePoints = new List<Vec3>() {bottomInsidePoint,  bottomOutsidePoint, topOutsidePoint, topInsidePoint };
                            if (stiffShapeModeLeft == 4)
                            {
                                pointList = new Vec3[] { bottomInsidePoint, topInsidePoint, topOutsidePoint };
                                comparePoints = new List<Vec3>() { bottomInsidePoint, topOutsidePoint, topInsidePoint };
                            }
                            sp = CreateStiffByPoints(pointList, stiffThicknessLeft);
                            platePropsLeft.ApplyToPart(sp);
                            chamferControl1.SetChamferPlace(sp.CtrlPoints, comparePoints);
                            chamferControl1.Execute();
                        }
                        else
                        {
                            Vec3[] pointList = new Vec3[] { bottomInsidePoint, bottomOutsidePoint, topOutsidePoint, topInsidePoint };
                            List<Vec3> comparePoints = new List<Vec3>() { bottomOutsidePoint, bottomInsidePoint, topInsidePoint, topOutsidePoint };
                            if (stiffShapeModeLeft == 4)
                            {
                                pointList = new Vec3[] { bottomInsidePoint, topOutsidePoint, topInsidePoint };
                                comparePoints = new List<Vec3>() {bottomInsidePoint, topOutsidePoint, topInsidePoint };
                            }
                            sp = CreateStiffByPoints(pointList, stiffThicknessLeft);
                            platePropsLeft.ApplyToPart(sp);
                            chamferControl2.SetChamferPlace(sp.CtrlPoints, comparePoints);
                            chamferControl2.Execute();
                        }
                        m_context.LeftStiffs.Add(sp);
                    }
                }

            } while (false);

            //右侧
            do
            {
                if (stiffShapeModeRight == 5 || backWebPlane == ComponentInstanceContext.DefaultPlane || !Util.PlatePropIsValid(platePropsRight))
                    break;
                double stiffTotalLengthRight = strideList.Sum() + stiffThicknessRight;
                //加劲肋参数（右）——默认边距
                switch (stiffShapeModeRight)
                {
                    case 2:
                        marginInsideRight = 0; //重置变为灰色的选项
                        if (marginBottomRight == double.MaxValue)
                            marginBottomRight = flangeInsideDistance / 2;
                        break;
                    case 3:
                        marginInsideRight = 0;
                        if (marginTopRight == 0)
                            marginTopRight = flangeInsideDistance / 2;
                        break;
                    case 4:
                        marginInsideRight = 0;
                        marginTopRight = 0;
                        if (marginBottomRight == double.MaxValue)
                            marginBottomRight = flangeInsideDistance - (flangeOutsideDistance - marginOutsideRight);
                        break;
                }
                if (marginBottomRight == double.MaxValue)
                    marginBottomRight = 0;
                //加劲肋参数（右）——默认宽度
                if (stiffWidthRight == double.MaxValue)
                    stiffWidthRight = m_context.BackBoundPlane.Dist(backWebPlane.Origin) - marginOutsideRight - marginInsideRight;
                //一根加劲肋的8（6）个点
                {
                    Vec3 point1Bottom, point2Bottom, point3Bottom, point4Bottom;
                    Vec3 point1Top, point2Top, point3Top, point4Top;
                    //确定1号点位置（下）
                    {
                        double xMove;
                        if (roltAngleRight >= 0)
                        {
                            xMove = webThickness / 2 * Math.Tan(roltAngleRight) + stiffThicknessRight / 2 / Math.Cos(roltAngleRight); //基础情况（默认distanceMode为0或1）
                        }
                        else
                        {
                            xMove = webThickness / 2 * Math.Tan(roltAngleRight) - stiffThicknessRight / 2 / Math.Cos(roltAngleRight);
                        }
                        if (arrangeMode == 3) //第一个加劲肋向哪个方向移动是通过arrange确定的
                        {
                            xMove -= distanceWithRoltPoint; //加上旋转中心距旋转线间距
                        }
                        else
                        {
                            xMove += distanceWithRoltPoint;
                        }
                        switch (arrangeMode)
                        {
                            case 2:
                                if (distanceMode == 2)
                                    xMove += stiffThicknessRight / 2;
                                if (distanceMode == 3)
                                    xMove -= stiffThicknessRight / 2;
                                break;
                            case 3:
                                if (distanceMode == 2)
                                    xMove -= stiffThicknessRight / 2;
                                if (distanceMode == 3)
                                    xMove += stiffThicknessRight / 2;
                                break;
                            default: //加劲肋对称放置，控制点移向边缘
                                xMove += stiffTotalLengthRight / 2 - stiffThicknessRight / 2;
                                if (distanceMode == 2)
                                    xMove += stiffThicknessRight / 2;
                                if (distanceMode == 3)
                                    xMove -= stiffThicknessRight / 2;
                                break;
                        }
                        double yMove = marginInsideRight;
                        double zMove = marginBottomRight;
                        if (stiffShapeModeRight == 2) //靠上的正方形加劲肋，需另外计算
                            zMove = flangeInsideDistance - marginTopRight - marginBottomRight;
                        point1Bottom = MovePoint(xMove, yMove, zMove, coordinatesRight.Pos, coordinatesRight);
                    }
                    //确定2号点位置（下）
                    {
                        double xMove = stiffWidthRight * Math.Sin(roltAngleRight);
                        double yMove = stiffWidthRight * Math.Cos(roltAngleRight);
                        point2Bottom = MovePoint(xMove, yMove, 0, point1Bottom, coordinatesRight);
                    }
                    //确定3号点位置（下）
                    {
                        double xMove = stiffThicknessRight * Math.Cos(roltAngleRight);
                        double yMove = -stiffThicknessRight * Math.Sin(roltAngleRight);
                        if(roltAngleRight >= 0)
                        {
                            xMove *= -1;
                            yMove *= -1;
                        }
                        point3Bottom = MovePoint(xMove, yMove, 0, point1Bottom, coordinatesRight);
                    }
                    //确定4号点位置（下）
                    {
                        double xMove = stiffWidthRight * Math.Sin(roltAngleRight);
                        double yMove = stiffWidthRight * Math.Cos(roltAngleRight);
                        point4Bottom = MovePoint(xMove, yMove, 0, point3Bottom, coordinatesRight);
                    }
                    //确定所有控制点位置（上）
                    {
                        double zMove;
                        switch (stiffShapeModeRight)
                        {
                            case 2: //靠上侧的正方形
                                zMove = marginBottomRight;
                                break;
                            case 3: //靠下侧的正方形
                                zMove = marginTopRight;
                                break;
                            case 4: //靠上侧的三角形
                                zMove = flangeInsideDistance - marginBottomRight;
                                break;
                            default:
                                zMove = flangeInsideDistance - marginTopRight - marginBottomRight;
                                break;
                        }
                        point1Top = MovePoint(0, 0, zMove, point1Bottom, coordinatesRight);
                        point2Top = MovePoint(0, 0, zMove, point2Bottom, coordinatesRight);
                        point3Top = MovePoint(0, 0, zMove, point3Bottom, coordinatesRight);
                        point4Top = MovePoint(0, 0, zMove, point4Bottom, coordinatesRight);
                    }
                    if (stiffShapeModeRight == 4)
                    {
                        point2Bottom = Vec3.ZERO;
                        point4Bottom = Vec3.ZERO;
                    }
                    //当上下翼缘内侧不平行时
                    if (!isParallel)
                    {
                        point1Top = SymmetryPoint(hCenterPlane, point1Bottom);
                        point2Top = SymmetryPoint(hCenterPlane, point2Bottom);
                        point3Top = SymmetryPoint(hCenterPlane, point3Bottom);
                        point4Top = SymmetryPoint(hCenterPlane, point4Bottom);
                    }
                    m_context.PointListRightBottom.Add(new Vec3[] { point1Bottom, point2Bottom, point3Bottom, point4Bottom });
                    m_context.PointListRightTop.Add(new Vec3[] { point1Top, point2Top, point3Top, point4Top });

                }
                //每个点
                {
                    double orient = arrangeMode == 2 ? 1 : -1;
                    Vec3[] topPoints = CloneVec3Array(m_context.PointListRightTop[0]);
                    Vec3[] bottomPoints = CloneVec3Array(m_context.PointListRightBottom[0]);
                    for (int i = 0; i < count - 1; i++)
                    {
                        double gap = strideList[i] * orient;
                        for (int j = 0; j < 4; j++)
                        {
                            topPoints[j] = MovePoint(gap, 0, 0, topPoints[j], coordinatesRight);
                            bottomPoints[j] = MovePoint(gap, 0, 0, bottomPoints[j], coordinatesRight);
                        }
                        m_context.PointListRightTop.Add(topPoints);
                        m_context.PointListRightBottom.Add(bottomPoints);
                        topPoints = CloneVec3Array(topPoints);
                        bottomPoints = CloneVec3Array(bottomPoints);
                    }
                }
                //创建加劲肋
                {
                    if (chamferMode1Right == 3 || chamferMode1Right == 4)
                        chamferY1Right = chamferX1Right;
                    if (chamferMode2Right == 3 || chamferMode2Right == 4)
                        chamferY2Right = chamferX2Right;
                    if (chamferMode3Right == 3 || chamferMode3Right == 4)
                        chamferY3Right = chamferX3Right;
                    if (chamferMode4Right == 3 || chamferMode4Right == 4)
                        chamferY4Right = chamferX4Right;
                    //angle >= 0 时
                    List<int> cutTypes1 = new List<int> { chamferMode3Right, chamferMode4Right, chamferMode2Right, chamferMode1Right };
                    List<double> xCuts1 = new List<double> { chamferX3Right, chamferY4Right, chamferX2Right, chamferY1Right };
                    List<double> yCuts1 = new List<double> { chamferY3Right, chamferX4Right, chamferY2Right, chamferX1Right };
                    //angle < 0 时
                    List<int> cutTypes2 = new List<int> { chamferMode4Right, chamferMode3Right, chamferMode1Right, chamferMode2Right };
                    List<double> xCuts2 = new List<double> { chamferX4Right, chamferY3Right, chamferX1Right, chamferY2Right };
                    List<double> yCuts2 = new List<double> { chamferY4Right, chamferX3Right, chamferY1Right, chamferX2Right };
                    //三角形加劲肋单独处理
                    if (stiffShapeModeRight == 4)
                    {
                        cutTypes1.RemoveAt(1); xCuts1.RemoveAt(1); yCuts1.RemoveAt(1);
                        cutTypes2 = new List<int> { chamferMode3Right, chamferMode2Right, chamferMode1Right };
                        xCuts2 = new List<double> { chamferY3Right, chamferY2Right, chamferX1Right };
                        yCuts2 = new List<double> { chamferX3Right, chamferX2Right, chamferY1Right };
                    }
                    //设置对应面以计算距离
                    List<Plane> nearPlaneList1 = new List<Plane>() {m_context.BottomFlangeInsidePlane, m_context.BackBoundPlane, m_context.TopBoundPlane, m_context.BackWebPlane };
                    List<Plane> nearPlaneList2 = new List<Plane>() { m_context.BottomFlangeInsidePlane, m_context.BackWebPlane, m_context.TopBoundPlane, m_context.BackBoundPlane };
                    ChamferControl chamferControl1 = new ChamferControl();
                    List<Define.CornerCutType> cutTypeList1 = ConvertToCornerCutType(cutTypes1);
                    chamferControl1.SetCutParameters(xCuts1, yCuts1, cutTypeList1);
                    ChamferControl chamferControl2 = new ChamferControl();
                    List<Define.CornerCutType> cutTypeList2 = ConvertToCornerCutType(cutTypes2);
                    chamferControl2.SetCutParameters(xCuts2, yCuts2, cutTypeList2);

                    for (int i = 0; i < m_context.PointListRightTop.Count; i++)
                    {
                        Vec3 topInsidePoint = m_context.PointListRightTop[i][0];
                        Vec3 topOutsidePoint = m_context.PointListRightTop[i][1];
                        Vec3 bottomInsidePoint = m_context.PointListRightBottom[i][0];
                        Vec3 bottomOutsidePoint = m_context.PointListRightBottom[i][1];
                        SteelPlate sp;
                        if (roltAngleRight >= 0)
                        {
                            Vec3[] pointList = new Vec3[] { bottomOutsidePoint, bottomInsidePoint, topInsidePoint, topOutsidePoint };
                            List<Vec3> comparePoints = new List<Vec3>() { bottomInsidePoint, bottomOutsidePoint, topOutsidePoint, topInsidePoint };
                            if (stiffShapeModeRight == 4)
                            {
                                pointList = new Vec3[] { bottomInsidePoint, topInsidePoint, topOutsidePoint };
                                comparePoints = new List<Vec3>() { bottomInsidePoint, topOutsidePoint, topInsidePoint };
                            }
                            sp = CreateStiffByPoints(pointList, stiffThicknessRight);
                            platePropsRight.ApplyToPart(sp);
                            chamferControl1.SetChamferPlace(sp.CtrlPoints, comparePoints);
                            chamferControl1.Execute();
                        }
                        else
                        {
                            Vec3[] pointList = new Vec3[] { bottomInsidePoint, bottomOutsidePoint, topOutsidePoint, topInsidePoint };
                            List<Vec3> comparePoints = new List<Vec3>() { bottomOutsidePoint, bottomInsidePoint, topInsidePoint, topOutsidePoint };
                            if (stiffShapeModeRight == 4)
                            {
                                pointList = new Vec3[] { bottomInsidePoint, topOutsidePoint, topInsidePoint };
                                comparePoints = new List<Vec3>() { bottomInsidePoint, topOutsidePoint, topInsidePoint };
                            }
                            sp = CreateStiffByPoints(pointList, stiffThicknessRight);
                            platePropsRight.ApplyToPart(sp);
                            chamferControl2.SetChamferPlace(sp.CtrlPoints, comparePoints);
                            chamferControl2.Execute();
                        }
                        m_context.RightStiffs.Add(sp);
                    }
                }
            } while (false);

        }

        public Vec3 SymmetryPoint(Plane plane, Vec3 point)
        {
            double dist = plane.SignedDist(point);
            point = point - 2 * dist * plane.Normal.Normalize();
            return point;
        }

        public Vec3 MovePoint(double x, double y, double z, Vec3 point, Mat43 mat)
        {
            point += mat.X * x;
            point += mat.Y * y;
            point += mat.Z * z;
            return point;
        }
    
        public Vec3[] CloneVec3Array(Vec3[] vec3s)
        {
            Vec3[] list = new Vec3[vec3s.Length];
            for (int i = 0; i < vec3s.Length; i++)
                list[i] = vec3s[i];
            return list;
        }
    
        public List<Define.CornerCutType> ConvertToCornerCutType(List<int> cutTypes)
        {
            List<Define.CornerCutType> cutTypeList = new List<Define.CornerCutType>();
            for (int j = 0; j < cutTypes.Count; j++)
            {
                Define.CornerCutType cutType = Define.CornerCutType.CORNER_CUT_NONE;
                switch (cutTypes[j])
                {
                    case 2: //斜角
                        cutType = Define.CornerCutType.CORNER_CUT_LINE;
                        break;
                    case 3: //向外圆角
                        cutType = Define.CornerCutType.CORNER_CUT_ROUND;
                        break;
                    case 4: //向内圆角
                        cutType = Define.CornerCutType.CORNER_CUT_INV_ROUND;
                        break;
                    case 5: //直角
                        cutType = Define.CornerCutType.CORNER_CUT_ORTHO;
                        break;
                    default: //什么也不做
                        break;
                }
                cutTypeList.Add(cutType);
            }
            return cutTypeList;
        }

        public SteelPlate CreateStiffByPoints(Vec3[] points, double thickness)
        {
            Vec3 v1 = points[1] - points[0];
            Vec3 v2 = points[2] - points[1];
            Vec3 createOrient = v1.Cross(v2);
            Plane zCtrlPlane = new Plane(points[0], createOrient);
            List<Plane> sidePlaneList = new List<Plane>();
            for(int i = 0; i < points.Length; i++)
            {
                Vec3 normal = createOrient.Cross(points[(i + 1) % points.Length] - points[i]);
                Plane sidePlane = new Plane(points[i], normal);
                sidePlaneList.Add(sidePlane);
            }
            Plane[] sidePlanes = sidePlaneList.ToArray();
            return m_context.Comp.CreateSteelPlate(zCtrlPlane, thickness, sidePlanes, true);
        }
    }
}
