﻿using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using LoongCAD.STC.Plugins;

namespace MultipleSiffeners1064
{
    class WeldHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;
        private List<WeldProperties> m_weldParams = new List<WeldProperties>();

        public WeldHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;

            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                m_weldParams.Add(new WeldProperties());
            }
        }

        public void Execute()
        {
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldParams[i].SetProps(weldProps);
            }

            this.CreateWeldsLeft();
            this.CreateWeldsRight();
        }


        private void CreateWeldsLeft()
        {
            int stiffShapeModeLeft = m_paramsHandler.GetIntValue(ParamNames.STIFF_LEFT_PLATE_TYPE); //加劲肋形状
            if (m_context.PointListLeftTop.Count == 0 || stiffShapeModeLeft == 5 || m_context.LeftStiffs.Count < 1)
                return;
            //折角处理
            int chamferMode1Left = m_paramsHandler.GetIntValue(ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_TYPE);
            double chamferX1Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_X), chamferY1Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_TOP_INSIDE_Y);
            int chamferMode2Left = m_paramsHandler.GetIntValue(ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_TYPE);
            double chamferX2Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_X), chamferY2Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_TOP_OUTSIDE_Y);
            int chamferMode3Left = m_paramsHandler.GetIntValue(ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_TYPE);
            double chamferX3Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_X), chamferY3Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_BOTTOM_INSIDE_Y);
            int chamferMode4Left = m_paramsHandler.GetIntValue(ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_TYPE);
            double chamferX4Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_X), chamferY4Left = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_LEFT_PLATE_BOTTOM_OUTSIDE_Y);
            //左侧
            for (int i = 0; i < m_context.PointListLeftTop.Count; i++)
            {
                Vec3[] topPoints = m_context.PointListLeftTop[i];
                Vec3[] bottomPoints = m_context.PointListLeftBottom[i];
                Vec3 bottomNearWebPoint = bottomPoints[0];
                Vec3 bottomAwayWebPoint = bottomPoints[1];
                Vec3 topNearWebPoint = topPoints[0];
                Vec3 topAwayWebPoint = topPoints[1];
                Plane temp = new Plane(topNearWebPoint, topAwayWebPoint - topNearWebPoint, bottomNearWebPoint - topNearWebPoint);//用于创建控制点
                //上侧的焊缝
                do
                {
                    //确定新的y方向为加劲肋的朝向
                    Vec3 y = m_context.PointListLeftTop[0][1] - m_context.PointListLeftTop[0][0];
                    m_context.coordinatesLeft.Y = y.Normalize();
                    if (m_context.TopFlangeInsidePlane.SignedDist(topNearWebPoint) > 1e-6)
                        break;
                    Vec3[] ctrlPoints = new Vec3[2] { topNearWebPoint, topAwayWebPoint };
                    if (chamferMode1Left != 1) //根据折角移动内侧焊接控制点
                        ctrlPoints[0] = MovePoint(0, chamferX1Left, 0, ctrlPoints[0], m_context.coordinatesLeft);
                    if(chamferMode2Left != 0 && chamferMode2Left != 1) //根据折角移动外侧焊接控制点
                        ctrlPoints[1] = MovePoint(0, -chamferX2Left, 0, ctrlPoints[1], m_context.coordinatesLeft);
                    if (m_context.FrontBoundPlane.SignedDist(ctrlPoints[1]) >= 0) //加劲肋边缘点在零件外部
                    {
                        Vec3 v = m_context.FrontBoundPlane.Intersect(temp, m_context.TopFlangeInsidePlane);
                        ctrlPoints[1] = v;
                    }
                    Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, m_context.LeftStiffs[i], ctrlPoints, true);
                    m_weldParams[0].ApplyToWeld(weld);
                    weld.Flush();
                } while (false);
                //下侧的焊缝
                do{
                    if (m_context.BottomFlangeInsidePlane.SignedDist(bottomNearWebPoint) > 1e-6)
                        break;
                    if (stiffShapeModeLeft == 4) //三角形不支持
                        break;
                    //确定新的y方向为加劲肋的朝向
                    Vec3 y = m_context.PointListLeftBottom[0][1] - m_context.PointListLeftBottom[0][0];
                    m_context.coordinatesLeft.Y = y.Normalize();
                    Vec3[] ctrlPoints = new Vec3[2] { bottomNearWebPoint, bottomAwayWebPoint };
                    if (chamferMode3Left != 1) //根据折角移动内侧焊接控制点
                        ctrlPoints[0] = MovePoint(0, chamferX3Left, 0, ctrlPoints[0], m_context.coordinatesLeft);
                    if (chamferMode4Left != 0 && chamferMode4Left != 1) //根据折角移动外侧焊接控制点
                        ctrlPoints[1] = MovePoint(0, -chamferX4Left, 0, ctrlPoints[1], m_context.coordinatesLeft);
                    if (m_context.FrontBoundPlane.SignedDist(ctrlPoints[1]) >= 0)
                    {
                        Vec3 v = m_context.FrontBoundPlane.Intersect(temp, m_context.BottomFlangeInsidePlane);
                        ctrlPoints[1] = v;
                    }
                    Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, m_context.LeftStiffs[i], ctrlPoints, true);
                    m_weldParams[1].ApplyToWeld(weld);
                    weld.Flush();
                } while (false) ;
                //腹板的焊缝
                {
                    Vec3[] ctrlPoints = { topNearWebPoint, bottomNearWebPoint };
                    if (m_context.FrontWebPlane.SignedDist(bottomNearWebPoint) < 1e-6)
                    {
                        if (chamferMode1Left == 3 || chamferMode1Left == 4) //根据折角移动上侧焊接控制点
                            ctrlPoints[0] = MovePoint(0, 0, -chamferX1Left, ctrlPoints[0], m_context.coordinatesLeft);
                        else if(chamferMode1Left != 1) //直角折角的情况
                            ctrlPoints[0] = MovePoint(0, 0, -chamferY1Left, ctrlPoints[0], m_context.coordinatesLeft);
                        if (chamferMode3Left == 3 || chamferMode3Left == 4)
                            ctrlPoints[1] = MovePoint(0, 0, chamferX3Left, ctrlPoints[1], m_context.coordinatesLeft);
                        else if (chamferMode3Left != 1)
                            ctrlPoints[1] = MovePoint(0, 0, chamferY3Left, ctrlPoints[1], m_context.coordinatesLeft);
                        Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, m_context.LeftStiffs[i], ctrlPoints, true);
                        m_weldParams[2].ApplyToWeld(weld);
                        weld.Flush();
                    }
                }

            }

        }

        private void CreateWeldsRight()
        {
            int stiffShapeModeRight = m_paramsHandler.GetIntValue(ParamNames.STIFF_RIGHT_PLATE_TYPE); //加劲肋形状
            if (m_context.PointListRightTop.Count == 0 || stiffShapeModeRight == 5 || m_context.RightStiffs.Count < 1)
                return;
            //折角处理
            int chamferMode1Right = m_paramsHandler.GetIntValue(ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_TYPE);
            double chamferX1Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_X), chamferY1Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_TOP_INSIDE_Y);
            int chamferMode2Right = m_paramsHandler.GetIntValue(ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_TYPE);
            double chamferX2Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_X), chamferY2Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_TOP_OUTSIDE_Y);
            int chamferMode3Right = m_paramsHandler.GetIntValue(ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_TYPE);
            double chamferX3Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_X), chamferY3Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_BOTTOM_INSIDE_Y);
            int chamferMode4Right = m_paramsHandler.GetIntValue(ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_TYPE);
            double chamferX4Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_X), chamferY4Right = m_paramsHandler.GetDoubleValue(ParamNames.STIFF_RIGHT_PLATE_BOTTOM_OUTSIDE_Y);
            for (int i = 0; i < m_context.PointListRightTop.Count; i++)
            {
                Vec3[] topPoints = m_context.PointListRightTop[i];
                Vec3[] bottomPoints = m_context.PointListRightBottom[i];
                Vec3 bottomNearWebPoint = bottomPoints[0];
                Vec3 bottomAwayWebPoint = bottomPoints[1];
                Vec3 topNearWebPoint = topPoints[0];
                Vec3 topAwayWebPoint = topPoints[1];
                Plane temp = new Plane(topNearWebPoint, topAwayWebPoint - topNearWebPoint, bottomNearWebPoint - topNearWebPoint);//用于创建控制点
                //上侧的焊缝
                do
                {
                    //确定新的y方向为加劲肋的朝向
                    Vec3 y = m_context.PointListRightTop[0][1] - m_context.PointListRightTop[0][0];
                    m_context.coordinatesRight.Y = y.Normalize();
                    if (m_context.TopFlangeInsidePlane.SignedDist(topNearWebPoint) > 1e-6)
                        break;
                    Vec3[] ctrlPoints = new Vec3[2] { topNearWebPoint, topAwayWebPoint };
                    if (chamferMode1Right != 1) //根据折角移动内侧焊接控制点
                        ctrlPoints[0] = MovePoint(0, chamferX1Right, 0, ctrlPoints[0], m_context.coordinatesRight);
                    if (chamferMode2Right != 0 && chamferMode2Right != 1) //根据折角移动外侧焊接控制点
                        ctrlPoints[1] = MovePoint(0, -chamferX2Right, 0, ctrlPoints[1], m_context.coordinatesRight);
                    if (m_context.BackBoundPlane.SignedDist(ctrlPoints[1]) >= 0) //加劲肋边缘点在零件外部
                    {
                        Vec3 v = m_context.BackBoundPlane.Intersect(temp, m_context.TopFlangeInsidePlane);
                        ctrlPoints[1] = v;
                    }
                    Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, m_context.RightStiffs[i], ctrlPoints, true);
                    m_weldParams[0].ApplyToWeld(weld);
                    weld.Flush();
                } while (false);
                //下侧的焊缝
                do
                {
                    if (m_context.BottomFlangeInsidePlane.SignedDist(bottomNearWebPoint) > 1e-6)
                        break;
                    if (stiffShapeModeRight == 4) //三角形不支持
                        break;
                    //确定新的y方向为加劲肋的朝向
                    Vec3 y = m_context.PointListRightBottom[0][1] - m_context.PointListRightBottom[0][0];
                    m_context.coordinatesRight.Y = y.Normalize();
                    Vec3[] ctrlPoints = new Vec3[2] { bottomNearWebPoint, bottomAwayWebPoint };
                    if (chamferMode3Right != 1) //根据折角移动内侧焊接控制点
                        ctrlPoints[0] = MovePoint(0, chamferX3Right, 0, ctrlPoints[0], m_context.coordinatesRight);
                    if (chamferMode4Right != 0 && chamferMode4Right != 1) //根据折角移动外侧焊接控制点
                        ctrlPoints[1] = MovePoint(0, -chamferX4Right, 0, ctrlPoints[1], m_context.coordinatesRight);
                    if (m_context.BackBoundPlane.SignedDist(ctrlPoints[1]) >= 0)
                    {
                        Vec3 v = m_context.BackBoundPlane.Intersect(temp, m_context.BottomFlangeInsidePlane);
                        ctrlPoints[1] = v;
                    }
                    Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, m_context.RightStiffs[i], ctrlPoints, true);
                    m_weldParams[1].ApplyToWeld(weld);
                    weld.Flush();
                } while (false);
                //腹板的焊缝
                {
                    Vec3[] ctrlPoints = { topNearWebPoint, bottomNearWebPoint };
                    if (m_context.BackWebPlane.SignedDist(bottomNearWebPoint) < 1e-6)
                    {
                        if (chamferMode1Right == 3 || chamferMode1Right == 4) //根据折角移动上侧焊接控制点
                            ctrlPoints[0] = MovePoint(0, 0, -chamferX1Right, ctrlPoints[0], m_context.coordinatesRight);
                        else if (chamferMode1Right != 1) //直角折角的情况
                            ctrlPoints[0] = MovePoint(0, 0, -chamferY1Right, ctrlPoints[0], m_context.coordinatesRight);
                        if (chamferMode3Right == 3 || chamferMode3Right == 4)
                            ctrlPoints[1] = MovePoint(0, 0, chamferX3Right, ctrlPoints[1], m_context.coordinatesRight);
                        else if (chamferMode3Right != 1)
                            ctrlPoints[1] = MovePoint(0, 0, chamferY3Right, ctrlPoints[1], m_context.coordinatesRight);
                        Weld weld = m_context.Comp.CreatePolyWeld(m_context.MainPart, m_context.RightStiffs[i], ctrlPoints, true);
                        m_weldParams[2].ApplyToWeld(weld);
                        weld.Flush();
                    }
                }

            }

        }

        public Vec3 MovePoint(double x, double y, double z, Vec3 point, Mat43 mat)
        {
            point += mat.X * x;
            point += mat.Y * y;
            point += mat.Z * z;
            return point;
        }
    }
}
