﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.Util;
using System.Diagnostics;
using System.IO;
using System.Linq.Expressions;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Threading.Tasks;


namespace ShortColumn133.ColumnSplice42
{
    public class FlangeBoltParameter : BoltParameter
    {
        private ParamsHandler paramsHandler;
        private SpliceContext m_context;
        private CommonBoltLocation boltLocationVert;
        private CommonBoltLocation boltLocationHori1;
        private CommonBoltLocation boltLocationHori2;

        public FlangeBoltParameter() { }

        public override BoltParameter Clone()
        {
            BoltParameter boltParameter = new FlangeBoltParameter();
            boltParameter.BoltStandard = this.BoltStandard;
            boltParameter.BoltSize = this.BoltSize;
            boltParameter.HoleTol = this.HoleTol; 
            boltParameter.ThreadInMaterial = this.ThreadInMaterial; 
            boltParameter.MadeType = this.MadeType; 
            boltParameter.HoleX = this.HoleX; 
            boltParameter.HoleY = this.HoleY;
            boltParameter.HoleType = this.HoleType;
            boltParameter.RotateSlot = this.RotateSlot; 
            boltParameter.SectionFlags = this.SectionFlags; 
            boltParameter.CutLength = this.CutLength; 
            boltParameter.ExtLength = this.ExtLength;
            boltParameter.RotationSide = this.RotationSide; 
            boltParameter.BoltShape = this.BoltShape;
            return boltParameter;
        }



        /*
         * 基础属性
         */
        public override void InitialBasic(SpliceContext compContext, ParamsHandler paramsHandler)
        {
            DObject basicProps = paramsHandler.GetDObjectValue(ParamNames.FLANGE_BOLT_BASIC_PROPS);
            CopyFromBasicProps(basicProps);

            DObject holeProps = paramsHandler.GetDObjectValue(ParamNames.FLANGE_BOLT_HOLE_PROPS);
            CopyFromHoleProps(holeProps);

            DObject contentProps = paramsHandler.GetDObjectValue(ParamNames.FLANGE_BOLT_CONTENT_PROPS);
            CopyFromContentProps(contentProps);

            DObject extendProps = paramsHandler.GetDObjectValue(ParamNames.FLANGE_BOLT_EXTEND_PROPS);
            CopyFromExtendProps(extendProps);

            int shapeIndex = paramsHandler.GetIntValue(ParamNames.FLANGE_BOLT_SHAPE);
            SetShape(shapeIndex);

            //int orientIndex = paramsHandler.GetIntValue(ParamNames.BOLT_FLANGE_ORIENTATION);
            //SetOrient(orientIndex);

            this.paramsHandler = paramsHandler;
            this.m_context = compContext;

            //处理水平（左右两侧）方向上的螺栓分布
            {
                double centerGap1 = paramsHandler.GetDoubleValue(ParamNames.FLANGE_BOLT_HORI_MARGIN_TOP);
                double sideGap1 = paramsHandler.GetDoubleValue(ParamNames.FLANGE_BOLT_HORI_MARGIN_BOTTOM);
                int count1 = paramsHandler.GetIntValue(ParamNames.FLANGE_BOLT_HORI_COUNT);
                double[] originStride1 = paramsHandler.GetDoubleArrayValue(ParamNames.FLANGE_BOLT_HORI_STRIDE);
                boltLocationHori1 = new CommonBoltLocation(40, 40, 3, new double[] { 75.0 });
                boltLocationHori1.SetBoltLocation(centerGap1, sideGap1, count1, originStride1, (m_context.FlangeWidth - m_context.Gap) / 2);
                double centerGap2 = paramsHandler.GetDoubleValue(ParamNames.FLANGE_BOLT_HORI_MARGIN_TOP);
                double sideGap2 = paramsHandler.GetDoubleValue(ParamNames.FLANGE_BOLT_HORI_MARGIN_BOTTOM);
                int count2 = paramsHandler.GetIntValue(ParamNames.FLANGE_BOLT_HORI_COUNT);
                double[] originStride2 = paramsHandler.GetDoubleArrayValue(ParamNames.FLANGE_BOLT_HORI_STRIDE);
                boltLocationHori2 = new CommonBoltLocation(40, 40, 3, new double[] { 75.0 });
                boltLocationHori2.SetBoltLocation(centerGap2, sideGap2, count2, originStride2, (m_context.FlangeWidth - m_context.Gap) / 2);
            }
            //处理垂直方向上的螺栓分布
            {
                double centerGap = paramsHandler.GetDoubleValue(ParamNames.FLANGE_BOLT_VERT_MARGIN_BOTTOM);
                double sideGap = paramsHandler.GetDoubleValue(ParamNames.FLANGE_BOLT_VERT_MARGIN_TOP);
                int count = paramsHandler.GetIntValue(ParamNames.FLANGE_BOLT_VERT_COUNT);
                double[] originStride = paramsHandler.GetDoubleArrayValue(ParamNames.FLANGE_BOLT_VERT_STRIDE);
                boltLocationVert = new CommonBoltLocation(40, 40, 1, new double[] { 50.0 });
                boltLocationVert.SetBoltLocation(centerGap, sideGap, count, originStride, m_context.FlangeHeight); //自动排列螺栓
            }
        }

        /*
        * 位置属性
        */
        public override void InitalLocation(CommonBoltGroup bg)
        {
            bg.SetHoriPlane(m_context.SubNearCenterPlane.Invert(), m_context.MainNearCenterPlane.Invert());
            {
                bg.SetHoriLocationPropsSingle(boltLocationHori1.gap1, boltLocationHori1.stride, boltLocationHori1.gap2, 1);
                m_context.SubFlangeMargin = bg.TotalLengthHori;
                bg.SetHoriLocationPropsSingle(boltLocationHori2.gap1, boltLocationHori2.stride, boltLocationHori2.gap2, 2);
                m_context.MainFlangeMargin = bg.TotalLengthHori - m_context.SubFlangeMargin;
                m_context.FlangeWidth = bg.TotalLengthHori + m_context.gap;
            }
            //处理垂直方向上的螺栓分布
            {
                bg.SetVertLocationProps(boltLocationVert.gap1, boltLocationVert.stride, boltLocationVert.gap2, CommonBoltLocation.BoltLocationMode.MODE3);
                m_context.FlangeHeight = bg.TotalLengthVert;
            }
            bg.SetVertPlane(m_context.MainWebCenter.Move(-boltLocationVert.stride.Sum() / 2 - boltLocationVert.gap1), null );
            //数量为0删除螺栓
            if(boltLocationVert.count == 0)
            {
                bg.boltFlags[0] = false;
                bg.boltFlags[1] = false;
            }
            if (boltLocationHori1.count == 0)
                bg.boltFlags[0] = false;
            if (boltLocationHori2.count == 0)
                bg.boltFlags[1] = false;
        }
    }
}
