﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace ShortColumn133.ColumnSplice42
{
    class BoltsHandler
    {
        private SpliceContext m_context = null; //公共参数、方法
        private ParamsHandler m_paramsHandler = null; //包含UI里的参数

        public BoltsHandler(SpliceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }
        public void Execute()
        {
            //螺栓组主零件
            PartBase topPlate = m_context.MainTopFlangePlate;
            PartBase bottomPlate = m_context.MainBottomFlangePlate;
            PartBase webPlate = m_context.MainWebPlate;
            PartBase subPart = m_context.SubPart;

            //处理车间螺栓选项
            BoltParameter topMainBoltParameter = m_context.FlangeBoltParameter.Clone();
            BoltParameter topSubBoltParameter = m_context.FlangeBoltParameter.Clone();
            BoltParameter webMainBoltParameter = m_context.WebBoltParameter.Clone();
            BoltParameter webSubBoltParameter = m_context.WebBoltParameter.Clone();
            BoltParameter bottomMainBoltParameter = m_context.FlangeBoltParameter.Clone();
            BoltParameter bottomSubBoltParameter = m_context.FlangeBoltParameter.Clone();
            int webMadeModeMain = m_paramsHandler.GetIntValue(ParamNames.WEB_BOLT_LEFT_MADE_MODE);
            int webMadeModeSub = m_paramsHandler.GetIntValue(ParamNames.WEB_BOLT_RIGHT_MADE_MODE);
            if (webMadeModeMain == 2)
            {
                topMainBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
                webMainBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
                bottomMainBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
            }
            if (webMadeModeSub == 2)
            {
                topSubBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
                webSubBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
                bottomSubBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
            }


            //上翼缘螺栓
            m_context.TopBoltGroup.Initial(m_context.Comp, m_context.TopTopPlateCreate.outerPlane);
            m_context.TopBoltGroup.SetBoltParameters(topMainBoltParameter, topSubBoltParameter);
            m_context.TopBoltGroup.AddMainParts(new PartBase[] { topPlate, subPart });
            m_context.TopBoltGroup.AddSubParts(m_context.TopFrontPlateCreate.mainParts, m_context.TopBackPlateCreate.mainParts, m_context.TopFrontPlateCreate.subParts, m_context.TopBackPlateCreate.subParts);
            m_context.TopBoltGroup.AddSubParts(m_context.TopTopPlateCreate.mainParts, m_context.TopTopPlateCreate.subParts);
            m_context.TopBoltGroup.Execute(0b1100);

            //腹板螺栓
            m_context.WebBoltGroup.Initial(m_context.Comp, m_context.WebFrontPlateCreate.outerPlane);
            m_context.WebBoltGroup.SetBoltParameters(webMainBoltParameter, webSubBoltParameter);
            m_context.WebBoltGroup.AddMainParts(new PartBase[] { webPlate, subPart });
            m_context.WebBoltGroup.AddSubParts(m_context.WebFrontPlateCreate.mainParts, m_context.WebFrontPlateCreate.subParts);
            m_context.WebBoltGroup.AddSubParts(m_context.WebBackPlateCreate.mainParts, m_context.WebBackPlateCreate.subParts);
            m_context.WebBoltGroup.Execute(0b1100);

            //下翼缘螺栓
            m_context.BottomBoltGroup.Initial(m_context.Comp, m_context.BottomBottomPlateCreate.outerPlane);
            m_context.BottomBoltGroup.SetBoltParameters(bottomMainBoltParameter, bottomSubBoltParameter);
            m_context.BottomBoltGroup.AddMainParts(new PartBase[] { bottomPlate, subPart });
            m_context.BottomBoltGroup.AddSubParts(m_context.BottomBackPlateCreate.mainParts, m_context.BottomFrontPlateCreate.mainParts, m_context.BottomBackPlateCreate.subParts, m_context.BottomFrontPlateCreate.subParts);
            m_context.BottomBoltGroup.AddSubParts(m_context.BottomBottomPlateCreate.mainParts, m_context.BottomBottomPlateCreate.subParts);
            m_context.BottomBoltGroup.Execute(0b1100);

        }

    }
}
