﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using System.Drawing;

namespace ShortColumn133.ColumnSplice42
{
    public class CommonBoltGroup
    {        
        private ConnectionComponent cmp = null;
        //创建螺栓时需要的次零件组
        private List<PartBase> mainParts1 = new List<PartBase>(); //主梁零件组(前)
        private List<PartBase> mainParts2 = new List<PartBase>(); //主梁零件组(后)
        private List<PartBase> subParts1 = new List<PartBase>(); //次梁零件组(前)
        private List<PartBase> subParts2 = new List<PartBase>(); //次梁零件组(后)
        //四个象限的螺栓参数（默认相同）
        private BoltParameter[] boltParameters;
        public bool[] boltFlags = new bool[] { true, true, true, true };
        //螺栓组主零件
        private PartBase[] main = null;

        //四个平面确定位置，逆时针为h1,v1,h2,v2，构成四个象限
        //      |      |
        //  ②  |  |   |  ①
        //------|--v1--|------
        //      |      |
        //  <-h2|      |h1->
        //------|--v2--|------
        //      |  |   |
        //  ③  |      |  ④
        //      |      |
        private Plane horiPlane1 = null; //法向量指向hori方向的中心平面1
        private Plane horiPlane2 = null; 
        public Plane vertPlane1 = null;//法向量指向vert方向的中心平面1
        public Plane vertPlane2 = null;
        //螺栓顶面(创建面)
        private Plane topPlane = null;

        public List<double> marginCenterHori = new List<double>(4); //与靠近中心的边缘的间距(水平)
        public List<List<double>> strideArrayHori = new List<List<double>>(4); //螺栓间距数组
        public List<double> marginCenterVert = new List<double>(4);
        public List<List<double>> strideArrayVert = new List<List<double>>(4);

        //通过螺栓间距计算出的，板在水平/垂直方向上的总长
        private double totalLengthHori = 0;
        private double totalLengthVert = 0;

        public CommonBoltGroup() 
        {
            for(int i = 0; i < 4; i++)
            {
                List<double> list = new List<double> {10 };
                marginCenterHori.Add(0);
                marginCenterVert.Add(0);
                strideArrayHori.Add(list);
                strideArrayVert.Add(list);
            }
        }

        /// <summary>
        /// 设置螺栓组创建位置
        /// </summary>
        /// <param name="cmp"></param>
        /// <param name="topPlane"></param>
        public void Initial(ConnectionComponent cmp, Plane topPlane)
        {
            this.cmp = cmp;
            this.topPlane = topPlane;
        }

        /// <summary>
        /// 设置螺栓组的参数
        /// </summary>
        public void SetBoltParameters(BoltParameter bp1, BoltParameter bp2, BoltParameter bp3, BoltParameter bp4)
        {
            boltParameters = new BoltParameter[4];
            boltParameters[0] = bp1;
            boltParameters[1] = bp2;
            boltParameters[2] = bp3;
            boltParameters[3] = bp4;
        }

        /// <summary>
        /// 设置螺栓组的参数
        /// </summary>
        /// <param name="mainBoltParameter">主零件螺栓参数</param>
        /// <param name="subBoltParameter">次零件螺栓参数</param>
        public void SetBoltParameters(BoltParameter mainBoltParameter, BoltParameter subBoltParameter)
        {
            SetBoltParameters(subBoltParameter, mainBoltParameter, mainBoltParameter, subBoltParameter);
        }

        /// <summary>
        /// 设置不同螺栓组的设置主零件
        /// </summary>
        public void AddMainParts(PartBase[] main)
        {
            this.main = main;
        }

        /// <summary>
        /// 设置不同螺栓组的次零件组
        /// </summary>
        /// <param name="mainParts1">主梁前侧次零件组</param>
        /// <param name="mainParts2">主梁后侧次零件组</param>
        /// <param name="subParts1">次梁前侧次零件组</param>
        /// <param name="subParts2">次梁后侧次零件组</param>
        public void AddSubParts(List<PartBase> mainParts1, List<PartBase> mainParts2, List<PartBase> subParts1, List<PartBase> subParts2)
        {
            if (mainParts1 != null)
                this.mainParts1.AddRange(mainParts1);
            if (mainParts2 != null)
                this.mainParts2.AddRange(mainParts2);
            if (subParts1 != null)
                this.subParts1.AddRange(subParts1);
            if (subParts2 != null)
                this.subParts2.AddRange(subParts2);
        }

        /// <summary>
        /// 设置不同螺栓组的次零件组（连接板的前侧与后侧未断开）
        /// </summary>
        /// <param name="mainParts">主梁次零件组</param>
        /// <param name="subParts">次梁次零件组</param>
        public void AddSubParts(List<PartBase> mainParts, List<PartBase> subParts)
        {
            if (mainParts != null)
            {
                this.mainParts1.AddRange(mainParts);
                this.mainParts2.AddRange(mainParts);
            }
            if (subParts != null)
            {
                this.subParts1.AddRange(subParts);
                this.subParts2.AddRange(subParts);
            }

        }


        /// <summary>
        /// 设置螺栓组在水平方向上的位置参数
        /// </summary>
        /// <param name="gapArray">间距数组</param>
        /// <param name="boltLocationMode">使用BoltLocationMode设置</param>
        public void SetHoriLocationProps(double gap1, List<double> gapArray, double gap2, CommonBoltLocation.BoltLocationMode boltLocationMode) 
        {
            int mode = (int)boltLocationMode;
            double totalLength = 0; //水平方向上的总长度
            double marginCenter = 0;
            if(mode == 1)
            {
                marginCenter = gap1;
                totalLength = (gap1 + gapArray.Sum() + gap2) * 2 + horiPlane1.Dist(horiPlane2.Origin);
            }
            if(mode == 2)
            {
                marginCenter = gap1 / 2 - horiPlane1.Dist(horiPlane2.Origin) / 2;
                totalLength = gap1 + (gapArray.Sum() + gap2) * 2;
            }
            //
            for (int i = 0; i < 4; i++)
            {
                marginCenterHori[i] = marginCenter;
                strideArrayHori[i] = gapArray;
            }
            if (mode == 3) //这种情况只有1,4两个象限
            {
                marginCenterHori[0] = gap1;
                marginCenterHori[3] = gap1; 
                totalLength = gapArray.Sum() + gap1 + gap2;
            }
            totalLengthHori = totalLength;
        }

        /// <summary>
        /// 单独设置各个象限的螺栓组在水平方向上的位置参数
        /// </summary>
        /// <param name="gap1">靠近内侧（中心）的螺栓离内侧边缘的间距</param>
        /// <param name="n">设置第几象限，取值1,2,3,4</param>
        /// <param name="gap2">靠近外侧（远离中心）的螺栓离外侧边缘的间距</param>
        public void SetHoriLocationPropsSingle(double gap1, List<double> gapArray, double gap2, int n)
        {
            if (n < 1 || n > 4)
                return;
            marginCenterHori[n - 1] = gap1;
            strideArrayHori[n - 1] = gapArray;
            totalLengthHori += gap1 + gapArray.Sum() + gap2;
        }

        /// <summary>
        /// 设置螺栓组在垂直方向上的位置参数
        /// </summary>
        /// <param name="gapArray">间距数组</param>
        /// <param name="boltLocationMode">使用BoltLocationMode设置</param>
        public void SetVertLocationProps(double gap1, List<double> gapArray, double gap2, CommonBoltLocation.BoltLocationMode boltLocationMode)
        {
            int mode = (int)boltLocationMode;
            double totalLength = 0; //水平方向上的总长度
            double marginCenter = 0;
            if (mode == 1)
            {
                marginCenter = gap1;
                totalLength = (gap1 + gapArray.Sum() + gap2) * 2 + vertPlane1.Dist(vertPlane2.Origin);
            }
            if (mode == 2)
            {
                marginCenter = gap1 / 2 - vertPlane1.Dist(vertPlane2.Origin) / 2;
                totalLength = gap1 + (gapArray.Sum() + gap2) * 2;
            }
            //
            for (int i = 0; i < 4; i++)
            {
                marginCenterVert[i] = marginCenter;
                strideArrayVert[i] = gapArray;
            }
            if (mode == 3) //这种情况只有1,2两个象限
            {
                marginCenterVert[0] = gap1;
                marginCenterVert[1] = gap1; 
                totalLength = gapArray.Sum() + gap1 + gap2;
            }
            totalLengthVert = totalLength;
        }

        /// <summary>
        /// 单独设置各个象限的螺栓组在水平方向上的位置参数
        /// </summary>
        /// <param name="gap1">靠近内侧（中心）的螺栓离内侧边缘的间距</param>
        /// <param name="n">设置第几象限，取值1,2,3,4</param>
        /// <param name="gap2">靠近外侧（远离中心）的螺栓离外侧边缘的间距</param>
        public void SetVertLocationPropsSingle(double gap1, List<double> gapArray, double gap2, int n)
        {
            if (n < 1 || n > 4)
                return;
            marginCenterVert[n - 1] = gap1;
            strideArrayVert[n - 1] = gapArray;
            totalLengthVert += gap1 + gapArray.Sum() + gap2;
        }

        /// <summary>
        /// 设置法向量为水平方向的平面
        /// </summary>
        /// <param name="p1">次零件靠近中心的边缘平面</param>
        /// <param name="p2">主零件靠近中心的边缘平面</param>
        public void SetHoriPlane(Plane p1, Plane p2)
        {
            horiPlane1 = p1;
            horiPlane2 = p2;
        }

        /// <summary>
        /// 设置法向量为垂直方向的平面
        /// </summary>
        /// <param name="p1">次零件靠近中心的边缘平面</param>
        /// <param name="p2">主零件靠近中心的边缘平面</param>
        public void SetVertPlane(Plane p1, Plane p2)
        {
            vertPlane1 = p1;
            vertPlane2 = p2;
        }

        /// <summary>
        /// 是否打开螺栓
        /// </summary>
        /// <param name="mode">从左至右代表第1至第4象限，mode=0b1111代表全部打开</param>
        public void Execute(int mode)
        {
            if (main == null)
                return;
            Plane px, py;

            int[] modes = {1, 1, 1, 1 };//是否打开螺栓
            for(int i =0; i < 4; i++)
                modes[i] = mode >> (3 - i) & 1;

            //第一象限
            if (subParts2.Count != 0 && modes[1] == 1 && boltFlags[0]) //只有当此螺栓组有子零件组且打开螺栓开关时才创建螺栓
            {
                px = horiPlane1;
                py = vertPlane1;
                PartBase[] p = subParts2.ToArray();
                CreateBolts(px, py, main[1], p, 1);
            }
            //第二象限
            if (mainParts2.Count != 0 && modes[0] == 1 && boltFlags[1])
            {
                px = horiPlane2;
                py = vertPlane1;
                PartBase[] p = mainParts2.ToArray();
                CreateBolts(px, py, main[0], p, 2);
            }
            //第三象限
            if (mainParts1.Count != 0 && modes[2] == 1 && boltFlags[2])
            {
                px = horiPlane2;
                py = vertPlane2;
                PartBase[] p = mainParts1.ToArray();
                CreateBolts(px, py, main[0], p, 3);
            }
            //第四象限
            if (subParts1.Count != 0 && modes[3] == 1 && boltFlags[3])
            {
                px = horiPlane1;
                py = vertPlane2;
                PartBase[] p = subParts1.ToArray();
                CreateBolts(px, py, main[1], p, 4);
            }
        }

        private void CreateBolts(Plane px, Plane py, PartBase main, PartBase[] subs, int n) //传入第几象限
        {
            if (topPlane == null)
                return;

            //螺栓位置设置
            Vec3 x = px.Normal;
            Vec3 y = py.Normal;
            Vec3 z = x.Cross(y);
            Vec3 o = topPlane.Intersect(px, py);  //螺栓工作坐标系的原点靠近正中心
            if (z.Dot(topPlane.Normal) < 0)
                z *= -1; // 二四象限的z轴方向不对
            Mat43 workAxis = new Mat43(x, y, z, o);
            Vec3 posStart = topPlane.Intersect(px.Move(marginCenterHori[n - 1]), py.Move(strideArrayVert[n - 1].Sum() / 2 + marginCenterVert[n - 1]));
            Vec3 posEnd = topPlane.Intersect(px.Move(marginCenterHori[n - 1] + 20), py.Move(strideArrayVert[n - 1].Sum() / 2 + marginCenterVert[n - 1]));

            var boltGroup = cmp.CreateBoltGroup(main, subs, posStart, posEnd, true);
            boltGroup.WorkAxis = workAxis;
            boltGroup.ArrayStrideX = strideArrayHori[n - 1].ToArray(); //X方向间距
            boltGroup.ArrayStrideY = strideArrayVert[n - 1].ToArray();

            BoltParameter boltParameter = boltParameters[n - 1];
            //基础属性
            boltGroup.BoltStandard = boltParameter.BoltStandard;
            boltGroup.BoltSize = boltParameter.BoltSize;
            boltGroup.HoleTol = boltParameter.HoleTol; //孔容差范围
            boltGroup.ThreadInMtrl = boltParameter.ThreadInMaterial; //是否包含螺纹 
            boltGroup.MadeType = boltParameter.MadeType; //螺栓制作类型(工厂/工地) 

            //螺栓孔设置
            boltGroup.HoleX = boltParameter.HoleX; //孔X方向扩大尺寸
            boltGroup.HoleY = boltParameter.HoleY;
            boltGroup.HoleType = boltParameter.HoleType; //孔类型
            boltGroup.RotateSlot = boltParameter.RotateSlot; //孔开槽旋转方式 

            //螺栓垫片(螺母)设置
            boltGroup.SectionFlags = boltParameter.SectionFlags; //螺栓部件开关标志位

            //其它
            boltGroup.CutLength = boltParameter.CutLength; //切割长度
            boltGroup.ExtLength = boltParameter.ExtLength; // 附加长度 
            boltGroup.RotationSide = boltParameter.RotationSide; //螺栓朝向
            boltGroup.Shape = boltParameter.BoltShape; //螺栓形状(排列方式) 


            //解决旋转后位置错误的问题
            boltGroup.Rotation = 0.0;
            boltGroup.HeadOffsetX = 0.0;
            boltGroup.HeadOffsetY = 0.0;
            boltGroup.HeadOffsetZ = 0.0;
            boltGroup.TailOffsetX = 0.0;
            boltGroup.TailOffsetY = 0.0;
            boltGroup.TailOffsetZ = 0.0;
            boltGroup.VerticalOffset = 0.0;
            boltGroup.DepthOffset = 0.0;

            boltGroup.Refresh();
        }

        public double TotalLengthHori{ set => totalLengthHori = value; get => totalLengthHori; }
        public double TotalLengthVert { set => totalLengthVert = value; get => totalLengthVert; }
    }
}
