﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace ShortColumn133.ColumnSplice42
{
    public class CommonPlateCreate
    {

        //输入值
        public List<PartBase> mainParts;
        public List<PartBase> subParts;

        public ConnectionComponent cmp;
        public Plane mainCtrlPlane, subCtrlPlane;
        public Plane frontPlane, backPlane, headPlane, tailPlane;

        public double underBoardThickness = double.MaxValue; //垫板厚度
        public double thickness = 0;

        public PartBase subPart;
        public SpliceContext m_context;

        public PlateProperties connPlateProps;
        public PlateProperties ubPlateProps;

        //输出值
        public Plane outerPlane;
        public Plane connCtrlPlane; //连接板的创建面 
        public Plane connFront;
        public Plane connBack;
        public Plane connHead;
        public Plane connTail;
        public Plane ubCtrlPlane; //垫板的创建面
        public Plane ubFront;
        public Plane ubBack;
        public Plane ubHead;
        public Plane ubTail;

        public double distance; //两个创建面的间距
        public double ubLength1 = 0, ubLength2 = 0, gap = 0;
        public bool isVaild = true;

        //为焊缝做准备
        public List<Plane> underPlanes = new List<Plane>();

        public CommonPlateCreate() { }

        public CommonPlateCreate(SpliceContext comp)
        {
            cmp = comp.Comp;
            subPart = comp.SubPart;
            m_context = comp;
        }

        public void SetPlateProperties(PlateProperties connPlateProps, PlateProperties ubPlateProps)
        {
            this.connPlateProps = connPlateProps;
            this.ubPlateProps = ubPlateProps;
            this.underBoardThickness = ubPlateProps.Thickness;
            this.thickness = connPlateProps.Thickness;
        }

        /// <summary>
        /// 设置板的生成面（主、次零件）
        /// </summary>
        public void SetCtrlPlane(Plane basePlane1, Plane basePlane2)
        {
            mainCtrlPlane = basePlane1.Clone();
            subCtrlPlane = basePlane2.Clone();
        }

        /// <summary>
        /// 设置边面
        /// </summary>
        public void SetSidePlane(Plane frontPlane, Plane backPlane, Plane headPlane, Plane tailPlane)
        {
            //假设是头尾方向的中间发生厚度变化
            this.frontPlane = frontPlane.Clone();
            this.backPlane = backPlane.Clone();
            this.headPlane = headPlane.Clone();
            this.tailPlane = tailPlane.Clone();
            if (frontPlane.Dist(backPlane.Origin) < 1e-4 || headPlane.Dist(tailPlane.Origin) < 1e-4)
                isVaild = false;
        }

        /// <summary>
        /// 设置间距和厚度
        /// </summary>
        /// <param name="gap">主、次梁之间的间距，会影响垫板的width</param>
        public void SetCenterPlane(Plane p1, Plane p2)
        {
            this.ubLength1 = p1.Dist(tailPlane.Origin);
            this.ubLength2 = p2.Dist(headPlane.Origin);
            this.gap = p1.Dist(p2.Origin);
        }

        /// <summary>
        /// 创建连接板和垫板
        /// </summary>
        public void Execute()
        {
            if (mainCtrlPlane == SpliceContext.DefaultPlane || subCtrlPlane == SpliceContext.DefaultPlane)
                return;

            mainParts = new List<PartBase>();
            subParts = new List<PartBase>();

            distance = mainCtrlPlane.SignedDist(subCtrlPlane.Origin);

            connCtrlPlane = mainCtrlPlane.Clone(); //连接板创建面
            connFront = frontPlane.Clone();
            connBack = backPlane.Clone();
            connHead = headPlane.Clone();
            connTail = tailPlane.Clone();

            ubCtrlPlane = mainCtrlPlane.Clone(); //垫板创建面
            ubFront = connFront.Clone();
            ubBack = connBack.Clone();
            ubHead = connHead.Clone(); //垫板头
            ubTail = connTail.Clone(); //垫板尾

            if (distance < -1e-6) //主零件在前，此时选用次零件做垫板创建面
            {
                ubCtrlPlane = subCtrlPlane.Clone();
                ubTail = ubHead.Invert().Move(ubLength2);
                outerPlane = mainCtrlPlane.Clone();
            }
            else if (distance > 1e-6)
            {
                connCtrlPlane = subCtrlPlane.Clone();
                ubHead = ubTail.Invert().Move(ubLength1);
                outerPlane = subCtrlPlane.Clone();
            }
            else
            {
                ubHead.MoveThis(-ubLength2);
                ubTail.MoveThis(-ubLength1); //这里没有自动垫板了，ubhead和ubtail表示中间空的一块区域
                outerPlane = mainCtrlPlane.Clone();
            }


            //添加连接板
            if (thickness != 0)
            {
                outerPlane.MoveThis(thickness);
                if (isVaild)
                {
                    PartBase p = cmp.CreateSteelPlate(connCtrlPlane, thickness,
                        new Plane[] { connHead, connFront, connTail, connBack }, true);
                    connPlateProps.ApplyToPart(p);
                    p.Flush();
                    mainParts.Add(p);
                    subParts.Add(p);
                }
            }

            //添加垫板——需要主次平面不重合、开关打开、连接板厚度不为0（TK的逻辑）
            if (Math.Abs(distance) > 1e-6 && underBoardThickness != 0 && thickness != 0 && isVaild)
            {
                List<PartBase> list = CreateMulitUnderBoardPlate(ubCtrlPlane, Math.Abs(distance),
                    new Plane[] { ubHead, ubFront, ubTail, ubBack }, true);
                if (distance < 0)
                    subParts.AddRange(list);
                else
                    mainParts.AddRange(list);
            }

        }

        //创建多块垫板
        public List<PartBase> CreateMulitUnderBoardPlate(Plane zCtrlPlane, double thickness, Plane[] sidePlanes, bool bRuntime)
        {
            List<PartBase> re = new List<PartBase>();
            Plane zPlane = zCtrlPlane.Clone();

            int split;
            //默认的垫板厚度
            if (underBoardThickness == double.MaxValue)
            {
                split = 1;
                if (thickness >= 30)
                    split = 5;
                else if (thickness >= 20)
                    split = 4;
                else if (thickness >= 12)
                    split = 3;
                else if (thickness >= 2)
                    split = 2;
                underBoardThickness = Math.Floor(thickness / split); //向下取整
            }
            else
            {
                split = (int)(thickness / underBoardThickness);
            }

            for (int i = 1; i <= split; i++)
            {
                underPlanes.Add(zPlane.Clone());
                SteelPlate steelPlate = cmp.CreateSteelPlate(zPlane, underBoardThickness, sidePlanes, bRuntime);
                connPlateProps.ApplyToPart(steelPlate);
                steelPlate.Flush();
                zPlane.MoveThis(underBoardThickness);
                re.Add(steelPlate); //添加到零件组里
            }

            return re;
        }
    }
}
