﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;
using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;

namespace ShortColumn133.ColumnSplice42
{
    class PlateHandler
    {
        private SpliceContext m_context = null; //公共参数、方法
        private ParamsHandler m_paramsHandler = null; //包含UI里的参数

        private PlateProperties m_flangeProps = new PlateProperties();
        private PlateProperties m_topFlangeInsideProps = new PlateProperties();
        private PlateProperties m_bottomFlangeInsideProps = new PlateProperties();
        private PlateProperties m_webProps = new PlateProperties();

        public PlateHandler(SpliceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            // 对齐两个梁
            m_context.gap = m_paramsHandler.GetDoubleValue(ParamNames.WEB_PLATE_BEAM_GAP);
            Plane subAlignPlane = m_context.MainNearCenterPlane.Move(m_context.gap);
            m_context.Comp.CreateAlignPlane(m_context.SubPart, subAlignPlane, true);
            m_context.SubNearCenterPlane = subAlignPlane.Invert();

            // 获取连接板参数
            //翼缘连接板
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.OUTSIDE_FLANGE_CONN_PLATE_PROPS);
                m_flangeProps.SetProps(plateProps);
                m_context.FlangeWidth = m_flangeProps.Height;
                m_context.FlangeHeight =  m_flangeProps.Width;
            }
            {
                m_topFlangeInsideProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.TOP_INSIDE_FLANGE_CONN_PLATE_PROPS));
                m_bottomFlangeInsideProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.BOTTOM_INSIDE_FLANGE_CONN_PLATE_PROPS));
                Util.Swap(ref m_topFlangeInsideProps.Width, ref m_topFlangeInsideProps.Height);
                Util.Swap(ref m_bottomFlangeInsideProps.Width, ref m_bottomFlangeInsideProps.Height);
            }
            //腹板连接板
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.WEB_CONN_PLATE_PROPS);
                m_webProps.SetProps(plateProps);
                m_context.WebWidth = m_webProps.Height;
                m_context.WebHeight = m_webProps.Width;

                m_context.MainWebMargin = m_paramsHandler.GetDoubleValue(ParamNames.WEB_CONN_PLATE_MARGIN_LEFT);
                m_context.SubWebMargin = m_paramsHandler.GetDoubleValue(ParamNames.WEB_CONN_PLATE_MARGIN_RIGHT);
            }

            // 更新连接板长宽参数(通过螺栓)
            m_context.FlangeBoltParameter.InitialBasic(m_context, m_paramsHandler);
            m_context.FlangeBoltParameter.InitalLocation(m_context.TopBoltGroup);
            m_context.FlangeBoltParameter.InitalLocation(m_context.BottomBoltGroup);
            m_context.WebBoltParameter.InitialBasic(m_context, m_paramsHandler);
            m_context.WebBoltParameter.InitalLocation(m_context.WebBoltGroup);
            {
                Util.SetDefaultValue(ref m_topFlangeInsideProps.Thickness, m_flangeProps.Thickness);
                Util.SetDefaultValue(ref m_topFlangeInsideProps.Width, m_context.MainFlangeMargin + m_context.SubFlangeMargin + m_context.gap);
                Util.SetDefaultValue(ref m_topFlangeInsideProps.Height, m_context.FlangeHeight / 2 - m_context.m_main_s / 2);
                Util.SetDefaultValue(ref m_bottomFlangeInsideProps.Thickness, m_flangeProps.Thickness);
                Util.SetDefaultValue(ref m_bottomFlangeInsideProps.Width, m_context.MainFlangeMargin + m_context.SubFlangeMargin + m_context.gap);
                Util.SetDefaultValue(ref m_bottomFlangeInsideProps.Height, m_context.FlangeHeight / 2 - m_context.m_main_s / 2);
            }

            // 添加连接板和垫板
            double underBoardThickness = m_paramsHandler.GetDoubleValue(ParamNames.SHIM_THICKNESS);
            PlateProperties ubPlateProperties = new PlateProperties();
            ubPlateProperties.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.TOP_FLANGE_PLATE_PROPS));
            ubPlateProperties.Thickness = underBoardThickness;
            WebOpenMode webPlatePlaceMode = (WebOpenMode)m_paramsHandler.GetIntValue(ParamNames.WEB_CONN_PLATE_OPEN_MODE);//腹板开关
            // 上翼缘
            {
                CommonPlateCreate plateCreate = m_context.TopTopPlateCreate;

                Plane back = m_context.TopBoltGroup.vertPlane1.Invert();
                Plane front = back.Invert().Move(m_context.FlangeHeight);
                Plane tail = m_context.MainNearCenterPlane.Invert().Move(m_context.MainFlangeMargin);
                Plane head = m_context.SubNearCenterPlane.Invert().Move(m_context.SubFlangeMargin);

                plateCreate.SetCtrlPlane(m_context.MainTopFlange, m_context.SubTopFlange);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetCenterPlane(m_context.MainNearCenterPlane, m_context.SubNearCenterPlane);
                plateCreate.SetPlateProperties(m_flangeProps, ubPlateProperties);
                plateCreate.Execute();

            }
            //上侧前侧内翼缘
            {
                CommonPlateCreate plateCreate = m_context.TopFrontPlateCreate;

                Plane front = m_context.MainFrontWeb.Move(m_topFlangeInsideProps.Height);
                Plane back = m_context.MainFrontWeb.Invert();
                Plane head = m_context.MainNearCenterPlane.Move(m_context.Gap / 2 + m_topFlangeInsideProps.Width / 2);//远离主零件
                Plane tail = m_context.SubNearCenterPlane.Move(m_context.Gap / 2 + m_topFlangeInsideProps.Width / 2);
                plateCreate.SetCtrlPlane(m_context.MainTopFlangeInside, m_context.SubTopFlangeInside);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetCenterPlane(m_context.MainNearCenterPlane, m_context.SubNearCenterPlane);
                plateCreate.SetPlateProperties(m_topFlangeInsideProps, ubPlateProperties);
                plateCreate.Execute();
            }
            //上侧后侧内翼缘
            {
                CommonPlateCreate plateCreate = m_context.TopBackPlateCreate;

                Plane front = Util.CreatePlaneBySymmetry(m_context.TopFrontPlateCreate.backPlane, m_context.MainWebCenter);
                Plane back = Util.CreatePlaneBySymmetry(m_context.TopFrontPlateCreate.frontPlane, m_context.MainWebCenter);
                Plane head = m_context.MainNearCenterPlane.Move(m_context.Gap / 2 + m_topFlangeInsideProps.Width / 2);//远离主零件
                Plane tail = m_context.SubNearCenterPlane.Move(m_context.Gap / 2 + m_topFlangeInsideProps.Width / 2);

                plateCreate.SetCtrlPlane(m_context.MainTopFlangeInside, m_context.SubTopFlangeInside);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetCenterPlane(m_context.MainNearCenterPlane, m_context.SubNearCenterPlane);
                plateCreate.SetPlateProperties(m_topFlangeInsideProps, ubPlateProperties);
                plateCreate.Execute();
            }
            // 前腹板
            {
                CommonPlateCreate plateCreate = m_context.WebFrontPlateCreate;

                Plane back = m_context.WebBoltGroup.vertPlane1.Invert();
                Plane front = back.Invert().Move(m_context.WebHeight);
                Plane head = m_context.MainNearCenterPlane.Move(m_context.Gap / 2 + m_context.WebWidth / 2);//远离主零件
                Plane tail = m_context.SubNearCenterPlane.Move(m_context.Gap / 2 + m_context.WebWidth / 2);
                //如果在图形界面有设置，重新定位
                if (m_context.MainWebMargin != double.MaxValue)
                    tail = m_context.MainNearCenterPlane.Invert().Move(m_context.MainWebMargin);
                if (m_context.SubWebMargin != double.MaxValue)
                    head = m_context.SubNearCenterPlane.Invert().Move(m_context.SubWebMargin);
                plateCreate.SetCtrlPlane(m_context.MainFrontWeb, m_context.SubFrontWeb);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetCenterPlane(m_context.MainNearCenterPlane, m_context.SubNearCenterPlane);
                plateCreate.SetPlateProperties(m_webProps, ubPlateProperties);
                if (webPlatePlaceMode == WebOpenMode.LEFT)
                    plateCreate.thickness = 0;
                plateCreate.Execute();
            }
            // 后腹板
            {
                CommonPlateCreate plateCreate = m_context.WebBackPlateCreate;

                Plane front = m_context.WebFrontPlateCreate.frontPlane;
                Plane back = m_context.WebFrontPlateCreate.backPlane;
                Plane head = m_context.WebFrontPlateCreate.headPlane;
                Plane tail = m_context.WebFrontPlateCreate.tailPlane;

                plateCreate.SetCtrlPlane(m_context.MainBackWeb, m_context.SubBackWeb);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetCenterPlane(m_context.MainNearCenterPlane, m_context.SubNearCenterPlane);
                plateCreate.SetPlateProperties(m_webProps, ubPlateProperties);
                if (webPlatePlaceMode == WebOpenMode.RIGHT)
                    plateCreate.thickness = 0;
                plateCreate.Execute();
            }
            // 下翼缘
            {
                CommonPlateCreate plateCreate = m_context.BottomBottomPlateCreate;

                Plane front = m_context.TopTopPlateCreate.frontPlane;
                Plane back = m_context.TopTopPlateCreate.backPlane;
                Plane head = m_context.TopTopPlateCreate.headPlane;
                Plane tail = m_context.TopTopPlateCreate.tailPlane;

                plateCreate.SetCtrlPlane(m_context.MainBottomFlange, m_context.SubBottomFlange);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetCenterPlane(m_context.MainNearCenterPlane, m_context.SubNearCenterPlane);
                plateCreate.SetPlateProperties(m_flangeProps, ubPlateProperties);
                plateCreate.Execute();
            }
            //下侧前侧内翼缘
            {
                CommonPlateCreate plateCreate = m_context.BottomFrontPlateCreate;

                Plane front = m_context.MainFrontWeb.Move(m_bottomFlangeInsideProps.Height);
                Plane back = m_context.MainFrontWeb.Invert();
                Plane head = m_context.MainNearCenterPlane.Move(m_context.Gap / 2 + m_bottomFlangeInsideProps.Width / 2);//远离主零件
                Plane tail = m_context.SubNearCenterPlane.Move(m_context.Gap / 2 + m_bottomFlangeInsideProps.Width / 2);
                plateCreate.SetCtrlPlane(m_context.MainBottomFlangeInside, m_context.SubBottomFlangeInside);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetCenterPlane(m_context.MainNearCenterPlane, m_context.SubNearCenterPlane);
                plateCreate.SetPlateProperties(m_bottomFlangeInsideProps, ubPlateProperties);
                plateCreate.Execute();
            }
            //下侧后侧内翼缘
            {
                CommonPlateCreate plateCreate = m_context.BottomBackPlateCreate;

                Plane front = Util.CreatePlaneBySymmetry(m_context.BottomFrontPlateCreate.backPlane, m_context.MainWebCenter);
                Plane back = Util.CreatePlaneBySymmetry(m_context.BottomFrontPlateCreate.frontPlane, m_context.MainWebCenter);
                Plane head = m_context.MainNearCenterPlane.Move(m_context.Gap / 2 + m_bottomFlangeInsideProps.Width / 2);//远离主零件
                Plane tail = m_context.SubNearCenterPlane.Move(m_context.Gap / 2 + m_bottomFlangeInsideProps.Width / 2);

                plateCreate.SetCtrlPlane(m_context.MainBottomFlangeInside, m_context.SubBottomFlangeInside);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetCenterPlane(m_context.MainNearCenterPlane, m_context.SubNearCenterPlane);
                plateCreate.SetPlateProperties(m_bottomFlangeInsideProps, ubPlateProperties);
                plateCreate.Execute();
            }
        }

    }
}
