﻿using LoongCAD.Geometry;
using LoongCAD.STC.Plugins;
using LoongCAD.STC.Object3D;
using LoongCAD.Util;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Drawing;

namespace ShortColumn133.ColumnSplice42
{
    public class SpliceContext
    {
        public static Plane DefaultPlane = new Plane(Vec3.ZERO + int.MaxValue * Vec3.AXIS_X, Vec3.AXIS_Z);
        private ConnectionComponent m_comp;

        public double gap; //梁中间间距

        //连接板长宽
        private double m_webWidth = 0; //此值不为0（b参数未设置时）时，作为腹连接板宽度
        private double m_webHeight = 0;
        private double m_flangeWidth = 0;
        private double m_flangeHeight = 0;
        private double m_mainWebMargin = 0;
        private double m_subWebMargin = 0;
        private double m_mainFlangeMargin = 0;
        private double m_subFlangeMargin = 0;


        // 主零件
        public SteelPlate m_mainTopFlangePlate;
        public SteelPlate m_mainBottomFlangePlate;
        public SteelPlate m_mainWebPlate;

        private Plane m_mainNearCenterPlane; //主零件靠近中心的面
        private Plane m_mainAwayCenterPlane; //主零件远离中心的面
        private Plane m_mainFront; //主零件的前侧面
        private Plane m_mainBack; //主零件的后侧面

        private Plane m_mainTopFlange = DefaultPlane; //顶翼板的上表面
        private Plane m_mainTopFlangeInside = DefaultPlane; //顶翼板的下表面

        private Plane m_mainBottomFlange = DefaultPlane; //底翼板的外侧面
        private Plane m_mainBottomFlangeInside = DefaultPlane; //底翼板的内侧面

        private Plane m_mainFrontWeb = DefaultPlane; //前腹板面
        private Plane m_mainWebCenter = DefaultPlane; //腹板中心面(法线为front方向)
        private Plane m_mainHCenter = DefaultPlane; //梁中心面（法线为Top方向）
        private Plane m_mainBackWeb = DefaultPlane; //后腹板面

        public double m_main_b1; //翼缘宽度
        public double m_main_s; //腹板厚度
        public double m_main_h; //高度
        public double m_main_t1; //上翼缘厚度
        public double m_main_t2; //下翼缘厚度


        //次零件
        private Beam m_subPart = null;

        private Plane m_subNearCenterPlane = null;
        private Plane m_subAwayCenterPlane = null;
        private Plane m_subFront = null; //次零件的前侧面
        private Plane m_subBack = null; //次零件的后侧面

        private Plane m_subTopFlange = DefaultPlane; //顶翼板的上表面
        private Plane m_subTopFlangeInside = DefaultPlane; //顶翼板的下表面

        private Plane m_subBottomFlange = DefaultPlane; //底翼板的外侧表面
        private Plane m_subBottomFlangeInside = DefaultPlane; //底翼板的内侧表面

        private Plane m_subFrontWeb = DefaultPlane; //前腹板面
        private Plane m_subWebCenter = DefaultPlane; //腹板中心面
        private Plane m_subHCenter = DefaultPlane;
        private Plane m_subBackWeb = DefaultPlane; //后腹板面

        public double m_sub_b1;
        public double m_sub_s;
        public double m_sub_h;
        public double m_sub_t = 0; //翼缘厚度

        //
        private CommonBoltGroup m_topBoltGroup = new CommonBoltGroup();
        private CommonBoltGroup m_webBoltGroup = new CommonBoltGroup();
        private CommonBoltGroup m_bottomBoltGroup = new CommonBoltGroup();

        //
        private BoltParameter m_flangeBoltParameter = new FlangeBoltParameter();
        private BoltParameter m_webBoltParameter = new WebBoltParameter();

        //
        private CommonPlateCreate m_topTopPlateCreate;
        private CommonPlateCreate m_topFrontPlateCreate;
        private CommonPlateCreate m_topBackPlateCreate;
        private CommonPlateCreate m_webFrontPlateCreate;
        private CommonPlateCreate m_webBackPlateCreate;
        private CommonPlateCreate m_bottomBottomPlateCreate;
        private CommonPlateCreate m_bottomFrontPlateCreate;
        private CommonPlateCreate m_bottomBackPlateCreate;

        public bool Initialize()
        {
            //初始化板
            m_topTopPlateCreate = new CommonPlateCreate(this);
            m_topFrontPlateCreate = new CommonPlateCreate(this);
            m_topBackPlateCreate = new CommonPlateCreate(this);
            m_webFrontPlateCreate = new CommonPlateCreate(this);
            m_webBackPlateCreate = new CommonPlateCreate(this);
            m_bottomBottomPlateCreate = new CommonPlateCreate(this);
            m_bottomFrontPlateCreate = new CommonPlateCreate(this);
            m_bottomBackPlateCreate = new CommonPlateCreate(this);

            m_main_b1 = m_mainFront.Dist(m_mainBack.Origin);

            m_sub_b1 = m_subFront.Dist(m_subBack.Origin);

            m_main_s = m_mainFrontWeb.Dist(m_mainBackWeb.Origin);
            m_sub_s = m_subFrontWeb.Dist(m_subBackWeb.Origin);

            m_main_h = m_mainTopFlange.Dist(m_mainBottomFlange.Origin);

            m_sub_h = m_subTopFlange.Dist(m_subBottomFlange.Origin);

            m_main_t1 = m_mainTopFlange.Dist(m_mainTopFlangeInside.Origin);
            m_main_t2 = m_mainBottomFlange.Dist(m_mainBottomFlangeInside.Origin);

            if (m_subTopFlangeInside != null)
                m_sub_t = m_subTopFlange.Dist(m_subTopFlangeInside.Origin);
            if (m_subBottomFlangeInside != null)
                m_sub_t = m_subBottomFlange.Dist(m_subBottomFlangeInside.Origin);

            return true;
        }

        public void test(Plane p)
        {
            Comp.CreateQuad(p, 4000, 4000, true);
        }
        public void test(Vec3 p)
        {
            Comp.CreatePoint(p, true);
        }
        public void test(Vec3 v1, Vec3 v2)
        {
            Comp.CreateLine(v1, v2, 0, Color.Red, true);
        }
        public void test(Mat43 mat)
        {
            double len = 1000;
            Comp.CreatePoint(mat.Pos, true);
            Comp.CreateLine(mat.Pos, mat.Pos + len / 2 * mat.X.Normalize(), 0, Color.Red, true);
            Comp.CreateLine(mat.Pos, mat.Pos + len / 2 * mat.Y.Normalize(), 0, Color.Green, true);
            Comp.CreateLine(mat.Pos, mat.Pos + len / 2 * mat.Z.Normalize(), 0, Color.Purple, true);
        }



        public ConnectionComponent Comp { get => m_comp; set => m_comp = value; }
        public SteelPlate MainTopFlangePlate { get => m_mainTopFlangePlate; set => m_mainTopFlangePlate = value; }
        public SteelPlate MainBottomFlangePlate { get => m_mainBottomFlangePlate; set => m_mainBottomFlangePlate = value; }
        public SteelPlate MainWebPlate { get => m_mainWebPlate; set => m_mainWebPlate = value; }
        public Beam SubPart { get => m_subPart; set => m_subPart = value; }
        public Plane MainNearCenterPlane { get => m_mainNearCenterPlane; set => m_mainNearCenterPlane = value; }
        public Plane MainAwayCenterPlane { get => m_mainAwayCenterPlane; set => m_mainAwayCenterPlane = value; }
        public Plane SubNearCenterPlane { get => m_subNearCenterPlane; set => m_subNearCenterPlane = value; }
        public Plane SubAwayCenterPlane { get => m_subAwayCenterPlane; set => m_subAwayCenterPlane = value; }
        public Plane MainTopFlange { get => m_mainTopFlange; set => m_mainTopFlange = value; }
        public Plane MainFront { get => m_mainFront; set => m_mainFront = value; }
        public Plane MainBack { get => m_mainBack; set => m_mainBack = value; }
        public Plane SubTopFlange { get => m_subTopFlange; set => m_subTopFlange = value; }
        public Plane MainTopFlangeInside { get => m_mainTopFlangeInside; set => m_mainTopFlangeInside = value; }
        public Plane SubTopFlangeInside { get => m_subTopFlangeInside; set => m_subTopFlangeInside = value; }
        public Plane MainFrontWeb { get => m_mainFrontWeb; set => m_mainFrontWeb = value; }
        public Plane MainWebCenter { get => m_mainWebCenter; set => m_mainWebCenter = value; }
        public Plane MainBackWeb { get => m_mainBackWeb; set => m_mainBackWeb = value; }
        public Plane SubFrontWeb { get => m_subFrontWeb; set => m_subFrontWeb = value; }
        public Plane SubBackWeb { get => m_subBackWeb; set => m_subBackWeb = value; }
        public Plane MainHCenter { get => m_mainHCenter; set => m_mainHCenter = value; }
        public Plane MainBottomFlange { get => m_mainBottomFlange; set => m_mainBottomFlange = value; }
        public Plane SubBottomFlange { get => m_subBottomFlange; set => m_subBottomFlange = value; }
        public Plane MainBottomFlangeInside { get => m_mainBottomFlangeInside; set => m_mainBottomFlangeInside = value; }
        public Plane SubBottomFlangeInside { get => m_subBottomFlangeInside; set => m_subBottomFlangeInside = value; }
        public CommonBoltGroup TopBoltGroup { get => m_topBoltGroup; set => m_topBoltGroup = value; }
        public CommonBoltGroup WebBoltGroup { get => m_webBoltGroup; set => m_webBoltGroup = value; }
        public CommonBoltGroup BottomBoltGroup { get => m_bottomBoltGroup; set => m_bottomBoltGroup = value; }
        public BoltParameter FlangeBoltParameter { get => m_flangeBoltParameter; set => m_flangeBoltParameter = value; }
        public BoltParameter WebBoltParameter { get => m_webBoltParameter; set => m_webBoltParameter = value; }
        public Plane SubFront { get => m_subFront; set => m_subFront = value; }
        public Plane SubBack { get => m_subBack; set => m_subBack = value; }
        public Plane SubHCenter { get => m_subHCenter; set => m_subHCenter = value; }
        public double WebWidth { get => m_webWidth; set => m_webWidth = value; }
        public double WebHeight { get => m_webHeight; set => m_webHeight = value; }
        public double FlangeWidth { get => m_flangeWidth; set => m_flangeWidth = value; }
        public double FlangeHeight { get => m_flangeHeight; set => m_flangeHeight = value; }
        public double MainWebMargin { get => m_mainWebMargin; set => m_mainWebMargin = value; }
        public double SubWebMargin { get => m_subWebMargin; set => m_subWebMargin = value; }
        public double MainFlangeMargin { get => m_mainFlangeMargin; set => m_mainFlangeMargin = value; }
        public double SubFlangeMargin { get => m_subFlangeMargin; set => m_subFlangeMargin = value; }
        public double Gap { get => gap; set => gap = value; }
        public CommonPlateCreate TopTopPlateCreate { get => m_topTopPlateCreate; set => m_topTopPlateCreate = value; }
        public CommonPlateCreate TopFrontPlateCreate { get => m_topFrontPlateCreate; set => m_topFrontPlateCreate = value; }
        public CommonPlateCreate TopBackPlateCreate { get => m_topBackPlateCreate; set => m_topBackPlateCreate = value; }
        public CommonPlateCreate WebFrontPlateCreate { get => m_webFrontPlateCreate; set => m_webFrontPlateCreate = value; }
        public CommonPlateCreate WebBackPlateCreate { get => m_webBackPlateCreate; set => m_webBackPlateCreate = value; }
        public CommonPlateCreate BottomBottomPlateCreate { get => m_bottomBottomPlateCreate; set => m_bottomBottomPlateCreate = value; }
        public CommonPlateCreate BottomFrontPlateCreate { get => m_bottomFrontPlateCreate; set => m_bottomFrontPlateCreate = value; }
        public CommonPlateCreate BottomBackPlateCreate { get => m_bottomBackPlateCreate; set => m_bottomBackPlateCreate = value; }
    }
}
