﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ShortColumn133
{
    public class BoltLocation
    {
        //默认值
        double MAX = int.MaxValue - 1; //数值小于MAX看作有效值
        public double length = double.MaxValue;
        public double margin1 = double.MaxValue; //边距1
        public double margin2 = double.MaxValue; //边距2
        public int count = 0; //该方向的螺栓数量
        public List<double> stride = new List<double>(); //螺栓间距数组
        public List<double> defaultStride = new List<double>(60);
        public bool centerMode = false; //无center无side时是否居中

        public BoltLocation() { }

        /// <summary>
        /// 设置默认的间距
        /// </summary>
        /// <param name="margin1"></param>
        /// <param name="margin2"></param>
        /// <param name="count"></param>
        /// <param name="stride"></param>
        public BoltLocation(double margin1, double margin2, int count, double[] stride)
        {
            this.margin1 = margin1;
            this.margin2 = margin2;
            this.count = count;
            this.defaultStride = new List<double>(stride); //这里的stride必须有值
            this.stride = ParseStrideInput(stride);
        }

        /// <summary>
        /// 处理输入的螺栓间距
        /// </summary>
        /// <param name="count">螺栓数量</param>
        /// <param name="originArray">从UI中获取的DoubleArray</param>
        /// <returns></returns>
        public List<double> ParseStrideInput(double[] originArray)
        {
            if (count > MAX)
                return new List<double>();
            if (originArray == null || originArray.Length == 0)
                originArray = this.defaultStride.ToArray();
            List<double> list = new List<double>();
            for (int i = 0; i < count - 1; ++i)
            {
                list.Add(originArray[Math.Min(i, originArray.Length - 1)]);
            }
            return list;
        }

        /// <summary>
        /// 设置间距
        /// </summary>
        /// <param name="count"></param>
        /// <param name="stride"></param>
        public List<double> SetStride(int count, double[] stride)
        {
            if (count < MAX) //一个有效的输入(count可以是0)
            {
                this.count = count;
                this.stride = ParseStrideInput(stride);
            }
            return this.stride;
        }

        /// <summary>
        /// 设置默认间距
        /// </summary>
        /// <param name="count"></param>
        /// <param name="stride"></param>
        public void SetDefaultStride(int count, double[] stride)
        {
            this.count = count;
            defaultStride = new List<double>();
            for(int i = 0; i < count; i++)
            {
                defaultStride.Add(stride[Math.Min(stride.Length - 1, i)]);
            }
        }


        /// <summary>
        /// 输入UI中获取到的值，获得位置参数
        /// </summary>
        public void SetBoltLocation(double margin1, double margin2, int count, double[] stride, double length)
        {
            SetStride(count, stride);
            if (length > MAX) //未设置板参数，定位按默认值和设置的值来
            {
                if (margin1 < MAX)
                    this.margin1 = margin1;
                if (margin2 < MAX)
                    this.margin2 = margin2;
                this.length = this.margin1 + this.stride.Sum() + this.margin2;
                return;
            }
            this.length = length; //输入了一个有效length
            if (margin2 < MAX) //有side往side靠
            {
                if (margin1 < MAX && this.stride.Count > 0 && stride.Length == 0) //有center也有side，但是不输入间距
                {
                    double boltGap = (length - margin1 - margin2) / this.stride.Count; //自动布置螺栓间距
                    for (int i = 0; i < this.stride.Count; i++)
                        this.stride[i] = boltGap;
                    this.margin1 = margin1;
                    this.margin2 = length - this.stride.Sum() - margin1;
                    return;
                }
                else if(margin1 < MAX && this.stride.Count > 0 && stride.Length != 0 && centerMode) //全都输入了
                {
                    this.margin1 = length / 2 - this.stride.Sum() / 2; //居中
                    this.margin2 = this.margin1;
                    return;
                }
                this.margin1 = length - this.stride.Sum() - margin2; //有side无center，center由side决定
                this.margin2 = margin2;
                return;
            }
            if (margin1 < MAX && margin2 > MAX) //有center无side往center方向靠
            {
                this.margin1 = margin1;
                this.margin2 = length - this.stride.Sum() - margin1; //有center无side，side由center决定
                return;
            }
            if (margin1 > MAX && margin2 > MAX) //无center无side
            {
                if (centerMode)
                {
                    this.margin1 = length / 2 - this.stride.Sum() / 2; //居中
                    this.margin2 = this.margin1;
                }
                else
                    this.margin2 = length - this.stride.Sum() - this.margin1; //往center方向靠
                return;
            }
        }

        /// <summary>
        /// 用于自适应以中心定位的螺栓，用SetStride设置默认间距
        /// </summary>
        /// <param name="margin1"></param>
        /// <param name="margin2"></param>
        /// <param name="count"></param>
        /// <param name="stride"></param>
        /// <param name="centerOffset">螺栓中心点的偏移</param>
        /// <param name="beamHeight">梁高度</param>
        /// <param name="plateMarginTop">板上端高于梁的距离</param>
        /// <param name="plateMarginTop">板下端低于梁的距离</param>
        public void SetCenterBoltLoaction(double margin1, double margin2, int count, double[] stride, 
            double centerOffset, double beamHeight, double plateMarginTop, double plateMarginBottom, double defaultMargin
            )
        {
            if (count < MAX) //一个有效的输入(count可以是0)
            {
                this.count = count;
                this.stride = ParseStrideInput(stride);
            }
            /*
             * 未设置板高度，定位按默认值和设置的值来
             */
            bool isFillPlateTop = plateMarginTop < MAX;
            bool isFillPlateBottom = plateMarginBottom < MAX;
            SetDefaultValue(ref plateMarginTop, defaultMargin);
            SetDefaultValue(ref plateMarginBottom, defaultMargin);
            double defaultMargin1 = plateMarginBottom + centerOffset - this.stride.Sum() / 2;
            double defaultMargin2 = beamHeight - centerOffset + plateMarginTop - this.stride.Sum() / 2;
            //上下边距都没填
            if (!isFillPlateBottom && !isFillPlateTop)
            {
                SetDefaultValue(ref this.margin1, margin1, defaultMargin1);
                SetDefaultValue(ref this.margin2, margin2, defaultMargin2);
                CountLength();
            }
            //只填了上边距
            else if(isFillPlateTop && !isFillPlateBottom)
            {
                if (margin2 < MAX)
                    defaultMargin1 += defaultMargin2 - margin2;
                SetDefaultValue(ref this.margin1, margin1, defaultMargin1);
                SetDefaultValue(ref this.margin2, margin2, defaultMargin2);
                CountLength();
            }
            //只填了下边距
            else if (!isFillPlateTop && isFillPlateBottom)
            {
                if (margin1 < MAX)
                    defaultMargin2 += defaultMargin1 - margin1;
                SetDefaultValue(ref this.margin1, margin1, defaultMargin1);
                SetDefaultValue(ref this.margin2, margin2, defaultMargin2);
                CountLength();
            }
            /*
             * 确定了板高度
             */
            else
            {
                if(margin1 > MAX && margin2 > MAX) //未设置边距，居中（梁中间）
                {
                    this.margin1 = plateMarginBottom + centerOffset - this.stride.Sum() / 2;
                    this.margin2 = beamHeight - centerOffset + plateMarginTop - this.stride.Sum() / 2;
                    CountLength();
                }
                else //centerOffset无效，居中（板中间）
                {
                    //centerMode = true;
                    double plateLength = beamHeight + plateMarginTop + plateMarginBottom;
                    SetBoltLocation(margin1, margin2, count, stride, plateLength);
                }
            }
        }
        

        public double CountLength()
        {
            length = this.margin1 + this.stride.Sum() + this.margin2;
            return length;
        }


        /// <summary>
        /// 设置多个默认值，排在前面的优先赋值
        /// </summary>
        /// <param name="value"></param>
        /// <param name="setValueList"></param>
        public static void SetDefaultValue(ref double value, params double[] setValueList)
        {
            for (int i = 0; i < setValueList.Length; i++)
            {
                if (value > -1e10 && value < 1e10) //一个正常的区间
                    break;
                value = setValueList[i];
            }
        }
    }
}
