﻿using System;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace ShortColumn133
{
    public class PluginMain : ComponentPlugin
    {
        private readonly Guid m_nameGuid = new Guid(System.Reflection.Assembly.GetExecutingAssembly().GetName().Name);
        private Form_PluginMain m_propsForm = null;

        public PluginMain()
        {
        }

        public override Guid Guid => m_nameGuid;

        public override bool OnInstanceRun(ComponentBase compInst, long defaultId)
        {
            if (compInst == null || !(compInst is ConnectionComponent))
                return false;

            //System.Diagnostics.Debug.WriteLine(System.Reflection.Assembly.GetExecutingAssembly().GetName().Name);

            // 1. 获取组件实例的属性
            DObject compProps = compInst.UIProps;
            if (compProps == null)
            {
                compProps = new DObject();
            }

            // 2. 初始化参数处理器，此过程包含以下操作:
            // * 初始化参数值
            // * 初始化参数空值表
            // * 初始化参数内置默认值表
            // * 组合参数默认值
            // 其他组件可直接复制此段代码
            ParamsHandler paramsHandler = new ParamsHandler();
            paramsHandler.ParamValues = compProps;
            paramsHandler.InitializeDefaultValues(this.GetDefaultsData(defaultId));
             
            // 3. 初始化组件上下文，此过程执行以下操作
            var compContext = ComponentInstanceContext.CreateInstance(compInst as ConnectionComponent);
            if (compContext == null)
                return false;

            {
                ColumnSplice42Handler connHandler = new ColumnSplice42Handler(compContext, paramsHandler);

                PlateHandler plateHandler = new PlateHandler(compContext, paramsHandler);
                plateHandler.Execute();
                connHandler.Execute();

                WeldHandler weldHandler = new WeldHandler(compContext, paramsHandler);
                weldHandler.Execute();
            }

            return true;
        }

        public override bool OnShowInstancePropsForm(ComponentBase instObj, long defId, Form ownerForm)
        {
            if (instObj == null)
                return false;
            if (m_propsForm == null)
            {
                m_propsForm = new Form_PluginMain(this);
                m_propsForm.Owner = ownerForm;
            }

            m_propsForm.ShowInstanceProps(instObj,
                new ParamsHandler().InitializeDefaultValues(this.GetDefaultsData(defId)));
            return true;
        }

        /// <summary>
        /// 显示组件的属性窗，必须实现此方法
        /// </summary>
        /// <param name="ownerForm">组件窗口所属主窗口</param>
        /// <returns>是否显示成功</returns>
        /// <remarks><b>其他组件可直接复制此方法</b></remarks>
        public override bool OnShowPluginPropsForm(Form ownerForm)
        {
            if (m_propsForm == null)
            {
                m_propsForm = new Form_PluginMain(this);
                m_propsForm.Owner = ownerForm;
            }

            m_propsForm.ShowPluginProps();
            return true;
        }
    }
}
