﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;
using LoongCAD.Localization;

namespace ShortColumn133
{
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {
        private DObject m_rootUIProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;
        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostIntialize();
        }

        public override ComponentPlugin ReferencePlugin => m_refPlugin;

        private void OnPostIntialize()
        {
            // 设置窗口标题
            this.Text = "带托梁的拼接节点（108）";

            foreach (TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }

            this.tb_flange_plate_outside_chamfer_mode.ClearItems();
            this.tb_flange_plate_outside_chamfer_mode.AddIntegerItems(0, 0, 4);
            this.tb_flange_plate_outside_chamfer_mode.UpdateItems();
            this.tb_flange_plate_outside_chamfer_mode.SelectedKey = 0;

            this.tb_flange_plate_inside_chamfer_mode.ClearItems();
            this.tb_flange_plate_inside_chamfer_mode.AddIntegerItems(0, 0, 5);
            this.tb_flange_plate_inside_chamfer_mode.UpdateItems();
            this.tb_flange_plate_inside_chamfer_mode.SelectedKey = 0;

            this.tb_web_conn_plate_open_mode.ClearItems();
            this.tb_web_conn_plate_open_mode.AddIntegerItems(0, 0, 4);
            this.tb_web_conn_plate_open_mode.UpdateItems();
            this.tb_web_conn_plate_open_mode.SelectedKey = 0;

            this.tb_flange_plate_shape_mode.ClearItems();
            this.tb_flange_plate_shape_mode.AddIntegerItems(0, 0, 3);
            this.tb_flange_plate_shape_mode.UpdateItems();
            this.tb_flange_plate_shape_mode.SelectedKey = 0;

            this.tb_lifting_hole_open_mode.ClearItems();
            this.tb_lifting_hole_open_mode.AddIntegerItems(0, 0, 3);
            this.tb_lifting_hole_open_mode.UpdateItems();
            this.tb_lifting_hole_open_mode.SelectedKey = 0;

            this.tb_rat_hole_shape.ClearItems();
            this.tb_rat_hole_shape.AddIntegerItems(0, 0, 4);
            this.tb_rat_hole_shape.UpdateItems();
            this.tb_rat_hole_shape.SelectedKey = 0;

            //工厂螺栓选项
            this.tb_web_bolt_right_made_mode.ClearItems();
            this.tb_web_bolt_right_made_mode.AddItem(0, "默认");
            this.tb_web_bolt_right_made_mode.AddItem(1, "工地");
            this.tb_web_bolt_right_made_mode.AddItem(2, "车间");
            this.tb_web_bolt_right_made_mode.UpdateItems();

            this.tb_web_bolt_left_made_mode.ClearItems();
            this.tb_web_bolt_left_made_mode.AddItem(0, "默认");
            this.tb_web_bolt_left_made_mode.AddItem(1, "工地");
            this.tb_web_bolt_left_made_mode.AddItem(2, "车间");
            this.tb_web_bolt_left_made_mode.UpdateItems();

            this.tb_backing_bar_top_place_mode.ClearItems();
            this.tb_backing_bar_top_place_mode.AddIntegerItems(0, 0, 4);//上翼缘衬垫板位置
            this.tb_backing_bar_top_place_mode.UpdateItems();
            this.tb_backing_bar_top_place_mode.SelectedKey = 0;

            this.tb_backing_bar_bottom_place_mode.ClearItems();
            this.tb_backing_bar_bottom_place_mode.AddIntegerItems(0, 0, 4);//
            this.tb_backing_bar_bottom_place_mode.UpdateItems();
            this.tb_backing_bar_bottom_place_mode.SelectedKey = 0;

            this.tb_backing_bar_length_mode.ClearItems();
            this.tb_backing_bar_length_mode.AddIntegerItems(0, 0, 3);
            this.tb_backing_bar_length_mode.UpdateItems();
            this.tb_backing_bar_length_mode.SelectedKey = 0;
        } 

        public override void ClearUI(Control root = null)
        {
            // 刷新螺栓基础面板
            this.tb_web_bolt_basic_props.RefreshControls();
            this.tb_flange_bolt_basic_props.RefreshControls();
            base.ClearUI(root);
        }

        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            this.ClearPlaceholders();
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }


        public void SetEnable(LoongCAD.UI.DoubleInput tb_cutx, LoongCAD.UI.DoubleInput tb_cuty,
            LoongCAD.UI.CheckBox act_cutx, LoongCAD.UI.CheckBox act_cuty, int selected, int defaultSelect = 2) //默认选Line
        {
            if (selected == 0)
                selected = defaultSelect;
            switch (selected)
            {
                case 1:
                    tb_cutx.Enabled = false; //NONE
                    act_cutx.Enabled = false;
                    tb_cuty.Enabled = false;
                    act_cuty.Enabled = false;
                    break;
                case 2:
                    tb_cutx.Enabled = true; //Line
                    act_cutx.Enabled = true;
                    tb_cuty.Enabled = true;
                    act_cuty.Enabled = true;
                    break;
                case 3:
                    tb_cutx.Enabled = true; //Round
                    act_cutx.Enabled = true;
                    tb_cuty.Enabled = false;
                    act_cuty.Enabled = false;
                    break;
                case 4:
                    tb_cutx.Enabled = true; //Inv_round
                    act_cutx.Enabled = true;
                    tb_cuty.Enabled = false;
                    act_cuty.Enabled = false;
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_rootUIProps.CopyFrom(srcProps);
        }

        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }

        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_rootUIProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_rootUIProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_rootUIProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_rootUIProps;

            return comp.UIProps;
        }
        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null)
            {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }

        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked)
            {
                retProps.CopyFrom(m_rootUIProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }

        private void btn_weld_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_rootUIProps, null);
        }

        private void tb_flange_plate_inside_chamfer_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_flange_plate_inside_chamfer_mode.SelectedIntKey;
            SetEnable(tb_flange_plate_inside_chamfer_cutx, tb_flange_plate_inside_chamfer_cuty,
                act_flange_plate_inside_chamfer_cutx, act_flange_plate_inside_chamfer_cuty, select);
        }

        private void tb_flange_plate_outside_chamfer_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_flange_plate_outside_chamfer_mode.SelectedIntKey;
            SetEnable(tb_flange_plate_outside_chamfer_cutx, tb_flange_plate_outside_chamfer_cuty,
                act_flange_plate_outside_chamfer_cutx, act_flange_plate_outside_chamfer_cuty, select, 1);
        }

        private void tb_flange_plate_shape_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_flange_plate_shape_mode.SelectedIntKey;
            if (select == 2)
            {
                SetOutsideChamferEnable(true);
                tb_flange_plate_outside_chamfer_mode_SelectedIndexChanged(null, null);
            }
            else
            {
                SetOutsideChamferEnable(false);
            }
        }

        private void SetOutsideChamferEnable(bool b)
        {
            tb_flange_plate_outside_chamfer_mode.Enabled = b;
            act_flange_plate_outside_chamfer_mode.Enabled = b;
            tb_flange_plate_outside_chamfer_cutx.Enabled = b;
            act_flange_plate_outside_chamfer_cutx.Enabled = b;
            tb_flange_plate_outside_chamfer_cuty.Enabled = b;
            act_flange_plate_outside_chamfer_cuty.Enabled = b;

        }
    }
}
