﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace ShortColumn133
{
    class PlateHandler
    {
        private ComponentInstanceContext m_context = null;
        private ParamsHandler m_paramsHandler = null;
        //
        Define.CornerCutType insideChamferType, outsideChamferType;
        FlangeShapeMode flangeShape;
        double insideChamferCutX, insideChamferCutY, outsideChamferCutX, outsideChamferCutY;

        public PlateHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            AlignBeam();
            double flangePlateMarginSide = m_paramsHandler.GetDoubleValue(ParamNames.FLANGE_PLATE_MARGIN_TOP);
            double flangePlateMarginOutside = m_paramsHandler.GetDoubleValue(ParamNames.FLANGE_PLATE_MARGIN_OUTSIDE);
            Plane outsideTemPlane = m_context.MainNearBoundPlane.Invert().Move(flangePlateMarginOutside); //用于确定斜面位置
            flangeShape = (FlangeShapeMode)m_paramsHandler.GetDoubleValue(ParamNames.FLANGE_PLATE_SHAPE_MODE);

            /*
             * 上下翼板
             */
            //6（8）个边面
            Plane[] flangePlateSidePlanes;
            {
                //在柱内侧的上边面和下边面
                Plane topInsidePlane = m_context.MainTopFlangeInside.Move(flangePlateMarginSide);
                Plane bottomInsidePlane = m_context.MainBottomFlangeInside.Move(flangePlateMarginSide);
                //柱的腹板面作为翼板左边面
                Plane insidePlane = m_context.MainNearWebPlane.Clone();
                //次梁的的近心面作为翼板右边面
                Plane outsidePlane = m_context.SubNearCenterPlane.Clone();
                //翼板侧边为斜边
                if (flangeShape == FlangeShapeMode.LINE)
                {
                    //上侧斜面
                    Vec3 topInsidePoint = outsideTemPlane.Intersect(m_context.MainTopFlangeInside, m_context.SubTopBound);
                    Vec3 topOutsidePoint1 = m_context.SubBackBound.Intersect(m_context.SubNearCenterPlane, m_context.SubTopBound);
                    Vec3 topOutsidePoint2 = m_context.SubBackBound.Intersect(m_context.SubNearCenterPlane, m_context.SubBottomBound);
                    Plane topPlane = Util.CreatePlaneBy3Point(topInsidePoint, topOutsidePoint1, topOutsidePoint2);
                    //下侧斜面
                    Vec3 bottomInsidePoint = outsideTemPlane.Intersect(m_context.MainBottomFlangeInside, m_context.SubTopBound);
                    Vec3 bottomOutsidePoint1 = m_context.SubFrontBound.Intersect(m_context.SubNearCenterPlane, m_context.SubTopBound);
                    Vec3 bottomOutsidePoint2 = m_context.SubFrontBound.Intersect(m_context.SubNearCenterPlane, m_context.SubBottomBound);
                    Plane bottomPlane = Util.CreatePlaneBy3Point(bottomInsidePoint, bottomOutsidePoint1, bottomOutsidePoint2);
                    flangePlateSidePlanes = new Plane[] { insidePlane, topInsidePlane, topPlane, outsidePlane, bottomPlane, bottomInsidePlane };
                }
                //翼板侧边为两个直角边
                else
                {
                    flangePlateSidePlanes = new Plane[] { insidePlane, topInsidePlane, outsideTemPlane, m_context.SubBackPlane, outsidePlane, m_context.SubFrontPlane, outsideTemPlane, bottomInsidePlane };
                }
                //若是旋转了，直接创建四边的板
                if (m_context.IsSubRotate)
                {
                    flangePlateSidePlanes = new Plane[] { insidePlane, m_context.SubFrontBound, outsidePlane, m_context.SubBackBound };
                }
                if (m_context.IsMainRotate)
                {
                    insidePlane = m_context.MainNearBoundPlane.Clone();
                    flangePlateSidePlanes = new Plane[] { insidePlane, m_context.SubFrontBound, outsidePlane, m_context.SubBackBound };
                }

            }
            //板参数
            PlateProperties topPlateProps = new PlateProperties();
            topPlateProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.TOP_FLANGE_PLATE_PROPS));
            PlateProperties bottomPlateProps = new PlateProperties();
            bottomPlateProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.BOTTOM_FLANGE_PLATE_PROPS));
            //折角参数
            insideChamferType = ConvertToCutType(m_paramsHandler.GetIntValue(ParamNames.FLANGE_PLATE_INSIDE_CHAMFER_MODE));
            insideChamferCutX = m_paramsHandler.GetDoubleValue(ParamNames.FLANGE_PLATE_INSIDE_CHAMFER_CUTX);
            insideChamferCutY = m_paramsHandler.GetDoubleValue(ParamNames.FLANGE_PLATE_INSIDE_CHAMFER_CUTY);
            if (insideChamferType == Define.CornerCutType.CORNER_CUT_ROUND || insideChamferType == Define.CornerCutType.CORNER_CUT_INV_ROUND)
                insideChamferCutY = insideChamferCutX;
            outsideChamferType = ConvertToCutType(m_paramsHandler.GetIntValue(ParamNames.FLANGE_PLATE_OUTSIDE_CHAMFER_MODE));
            outsideChamferCutX = m_paramsHandler.GetDoubleValue(ParamNames.FLANGE_PLATE_OUTSIDE_CHAMFER_CUTX);
            outsideChamferCutY = m_paramsHandler.GetDoubleValue(ParamNames.FLANGE_PLATE_OUTSIDE_CHAMFER_CUTY);
            if (outsideChamferType == Define.CornerCutType.CORNER_CUT_ROUND || outsideChamferType == Define.CornerCutType.CORNER_CUT_INV_ROUND)
                outsideChamferCutY = outsideChamferCutX;
            //下上翼板的创建
            SteelPlate topPlate = CreateFlangePlate(topPlateProps, flangePlateSidePlanes, m_context.SubTopBound.Invert());
            m_context.spliceContext.MainTopFlangePlate = topPlate;
            m_context.TopPlate = topPlate;
            m_context.spliceContext.MainTopFlangeInside = m_context.SubTopBound.Invert().Move(topPlateProps.Thickness);
            SteelPlate bottomPlate = CreateFlangePlate(bottomPlateProps, flangePlateSidePlanes, m_context.SubBottomBound.Invert());
            m_context.spliceContext.MainBottomFlangePlate = bottomPlate;
            m_context.BottomPlate = bottomPlate;
            m_context.spliceContext.MainBottomFlangeInside = m_context.SubBottomBound.Invert().Move(bottomPlateProps.Thickness);

            /*
             * 腹板
             */
            PlateProperties webPlateProps = new PlateProperties();
            do {
                //
                Plane insidePlane = m_context.MainNearWebPlane.Clone();
                if (m_context.IsMainRotate)
                    insidePlane = m_context.MainNearBoundPlane.Clone();
                Plane outsidePlane = m_context.SubNearCenterPlane.Clone();
                Plane topPlane = m_context.SubTopBound.Move(-topPlateProps.Thickness);
                Plane bottomPlane = m_context.SubBottomBound.Move(-bottomPlateProps.Thickness);
                Plane[] webPlateSidePlanes = new Plane[] { insidePlane, topPlane, outsidePlane, bottomPlane };
                //
                Plane ctrlPlane = Util.CenterPlane(m_context.SubFrontWeb, m_context.SubBackWeb);
                //
                webPlateProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.WEB_PLATE_PROPS));
                m_context.spliceContext.MainFrontWeb = ctrlPlane.Move(webPlateProps.Thickness / 2);
                m_context.spliceContext.MainBackWeb = ctrlPlane.Invert().Move(webPlateProps.Thickness / 2);
                if (webPlateProps.Thickness <= 0)
                    break;
                ctrlPlane.MoveThis(-webPlateProps.Thickness / 2);
                SteelPlate sp = m_context.Comp.CreateSteelPlate(ctrlPlane, webPlateProps.Thickness, webPlateSidePlanes, true);
                if (sp == null)
                    break;
                m_context.spliceContext.MainWebPlate = sp;
                m_context.WebPlate = sp; 
                bottomPlateProps.ApplyToPart(sp);
                sp.Flush();
            } while (false);


            /*
             * 焊接准备（与柱）
             */
            //梁参数，把三个板看作一个整体（一个I型梁）
            Plane topFlangeInside = m_context.spliceContext.MainTopFlangeInside; 
            Plane bottomFlangeInside = m_context.spliceContext.MainBottomFlangeInside;
            Plane flangeFrontPlane = flangePlateSidePlanes[1]; //梁的前面
            Plane flangeBackPlane = flangePlateSidePlanes[flangePlateSidePlanes.Length - 1]; //后面
            Plane webCenterPlane = Util.CenterPlane(flangeFrontPlane, flangeBackPlane); //假想梁的腹板中心面
            double beamWidth = flangeFrontPlane.Dist(flangeBackPlane.Origin);
            //鼠洞
            double ratHoleRadius = m_paramsHandler.GetDoubleValue(ParamNames.BACKING_BAR_RADIUS);
            Plane cutCtrlPlane = Util.CenterPlane(m_context.SubFrontBound, m_context.SubBackBound);
            RatHoleShapeMode ratHoleShape = (RatHoleShapeMode)m_paramsHandler.GetDoubleValue(ParamNames.RAT_HOLE_SHAPE_MODE);
            CreateBeamHoleCut(topFlangeInside, m_context.MainNearWebPlane, cutCtrlPlane, ratHoleRadius, webPlateProps.Thickness, m_context.WebPlate, ratHoleShape);
            CreateBeamHoleCut(bottomFlangeInside, m_context.MainNearWebPlane, cutCtrlPlane, ratHoleRadius, webPlateProps.Thickness, m_context.WebPlate, ratHoleShape);
            //焊接衬板
            do
            {
                //
                PlateProperties backingBarProps = new PlateProperties();
                backingBarProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.BACKING_BAR_PROPS));
                if (!Util.PlatePropIsValid(backingBarProps))
                    break;
                //四个边面
                BackingBarLengthMode lengthMode = (BackingBarLengthMode)m_paramsHandler.GetIntValue(ParamNames.BACKING_BAR_LENGTH_MODE);
                double barLength = m_paramsHandler.GetDoubleValue(ParamNames.BACKING_BAR_LENGTH);//焊接衬板长度
                if (lengthMode == BackingBarLengthMode.MARGIN_FLANGE)
                {
                    Util.SetDefaultValue(ref barLength, 0);
                    barLength = beamWidth + 2 * barLength;//获得的是焊接衬板相对于假想梁的偏移，重新计算长度
                }
                else
                    Util.SetDefaultValue(ref barLength, beamWidth);
                double barHoriOffset = m_paramsHandler.GetDoubleValue(ParamNames.BACKING_BAR_MARGIN);
                Plane leftPlane = m_context.MainNearWebPlane.Move(barHoriOffset);
                Plane rightPlane = leftPlane.Move(backingBarProps.Width);
                Plane frontPlane = webCenterPlane.Move(barLength / 2);
                Plane backPlane = webCenterPlane.Move(-barLength / 2);
                Plane[] sidePlanes = { leftPlane, frontPlane, rightPlane, backPlane };
                //上
                do {
                    BackingBarPlaceMode topPlace = (BackingBarPlaceMode)m_paramsHandler.GetIntValue(ParamNames.BACKING_BAR_TOP_PLACE_MODE);
                    if (topPlace == BackingBarPlaceMode.NONE)
                        break;
                    Plane topBarCtrlPlane = topPlace == BackingBarPlaceMode.INSIDE ? topFlangeInside : m_context.SubTopBound;
                    SteelPlate sp = m_context.Comp.CreateSteelPlate(topBarCtrlPlane, backingBarProps.Thickness, sidePlanes, true);
                    if (sp == null)
                        break;
                    backingBarProps.ApplyToPart(sp);
                    sp.Flush();
                    m_context.TopBackingBar = sp;
                } while (false);
                //下
                do {
                    BackingBarPlaceMode bottomPlace = (BackingBarPlaceMode)m_paramsHandler.GetIntValue(ParamNames.BACKING_BAR_BOTTOM_PLACE_MODE);
                    if (bottomPlace == BackingBarPlaceMode.NONE)
                        break;
                    Plane bottomBarCtrlPlane = bottomPlace == BackingBarPlaceMode.INSIDE ? bottomFlangeInside : m_context.SubBottomBound;
                    SteelPlate sp = m_context.Comp.CreateSteelPlate(bottomBarCtrlPlane, backingBarProps.Thickness, sidePlanes, true);
                    if (sp == null)
                        break;
                    backingBarProps.ApplyToPart(sp);
                    sp.Flush();
                    m_context.BottomBackingBar = sp;
                } while (false);

            } while (false);

        }

        public SteelPlate CreateFlangePlate(PlateProperties props, Plane[] sidePlanes, Plane ctrlPlane)
        {
            if (props.Thickness <= 0)
                return null;
            SteelPlate sp = m_context.Comp.CreateSteelPlate(ctrlPlane, props.Thickness, sidePlanes, true);
            if (sp == null)
                return null;
            props.ApplyToPart(sp);
            sp.Flush();
            ChamferControl chamferControl = new ChamferControl(sp, sidePlanes);
            chamferControl.ChamferOnePoint(insideChamferCutX, insideChamferCutY, insideChamferType, chamferControl.comparePointList[0]);
            chamferControl.ChamferOnePoint(insideChamferCutY, insideChamferCutX, insideChamferType, chamferControl.comparePointList[1]);
            if (flangeShape == FlangeShapeMode.ORTHO && !m_context.IsSubRotate)
            {
                chamferControl.ChamferOnePoint(outsideChamferCutY, outsideChamferCutX, outsideChamferType, chamferControl.comparePointList[3]);
                chamferControl.ChamferOnePoint(outsideChamferCutX, outsideChamferCutY, outsideChamferType, chamferControl.comparePointList[6]);
            }
            return sp;
        }

        public Define.CornerCutType ConvertToCutType(int i)
        {
            ChamferMode chamferMode = (ChamferMode)i;
            switch (chamferMode)
            {
                case ChamferMode.NONE:
                    return Define.CornerCutType.CORNER_CUT_NONE;
                case ChamferMode.LINE:
                    return Define.CornerCutType.CORNER_CUT_LINE;
                case ChamferMode.ROUND:
                    return Define.CornerCutType.CORNER_CUT_ROUND;
                case ChamferMode.INV_ROUND:
                    return Define.CornerCutType.CORNER_CUT_INV_ROUND;
                default:
                    return Define.CornerCutType.CORNER_CUT_NONE;
            }
        }

        public void AlignBeam()
        {
            double gap = m_paramsHandler.GetDoubleValue(ParamNames.COLUMN_BEAM_DISTANCE);
            if (gap != double.MaxValue)
            {
                Plane alignPlane = Util.VerticalPlane(m_context.SubTopPlane, m_context.MainWebCenterPlane);
                alignPlane = Util.VerticalPlane(m_context.SubFrontPlane, alignPlane);
                //Plane alignPlane = m_context.MainWebCenterPlane.Clone();
                alignPlane.MoveThis(gap);
                m_context.Comp.CreateAlignPlane(m_context.SubPart, alignPlane, true);
                m_context.AlignPlane = alignPlane;
                m_context.SubNearCenterPlane = alignPlane.Invert();
            }
            m_context.spliceContext.MainNearCenterPlane = m_context.SubNearCenterPlane.Invert();
            m_context.spliceContext.SubNearCenterPlane = m_context.SubNearCenterPlane.Clone();
        }

        public void CreateBeamHoleCut(Plane topPlane, Plane leftPlane, Plane ctrlPlane, double radius, double thickness, PartBase part, RatHoleShapeMode shape)
        {
            if (part == null || radius <= 0)
                return;
            Plane bottomPlane = topPlane.Move(radius);
            Plane rightPlane = leftPlane.Move(radius);
            Plane[] sidePlans = { topPlane, rightPlane, bottomPlane, leftPlane };
            thickness += 0.2; //切割体完全重合时无法切割
            Plane cutBottomPlane = ctrlPlane.Move(thickness / 2);
            Plane cutTopPlane = ctrlPlane.Move(-thickness / 2);
            PartBase sp = m_context.Comp.CreatePlateCutter(part, cutBottomPlane, cutTopPlane, sidePlans, true);
            ChamferControl chamferControl = new ChamferControl(sp, sidePlans);
            switch (shape)
            {
                case RatHoleShapeMode.ROUND:
                    chamferControl.ChamferOnePoint(radius, radius, Define.CornerCutType.CORNER_CUT_ROUND, chamferControl.comparePointList[2]);
                    break;
                case RatHoleShapeMode.TRIANGLE:
                    chamferControl.ChamferOnePoint(radius, radius, Define.CornerCutType.CORNER_CUT_LINE, chamferControl.comparePointList[2]);
                    break;
                default:
                    break;
            }
            
            sp.Flush();
        }

    }

}
