﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.UI.Component;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;
using LoongCAD.STC;

namespace ShortColumn133
{
    class WeldHandler
    {
        private ComponentInstanceContext m_context = null;
        private ParamsHandler m_paramsHandler = null;
        private List<WeldProperties> m_weldParams = new List<WeldProperties>();
        private Mat43 workAxis;

        public WeldHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }


        public void Execute()
        {
            if (!ParseParameter())
                return;
            //焊缝1：上翼板（原名上翼缘）与柱的焊接（1条）
            {
                //设置切割参数
                double thickness = m_context.spliceContext.m_main_t1;
                double horiSideUp = thickness / 2;
                double vertSideUp = thickness / 2;
                double horiSideDown = thickness / 2;
                double vertSideDown = thickness / 2;
                TriCutParameters(m_weldParams[0], ref horiSideUp, ref vertSideUp, ref horiSideDown, ref vertSideDown);
                //线上部分的切割
                Plane triCutCtrlPlane = m_context.IsSubRotate ? m_context.SubHoriCenterPlane : m_context.SubVertCenterPlane;
                double triCutLength = m_context.MainHeight;
                CreateTriCut(m_context.MainNearWebPlane.Invert(), m_context.spliceContext.MainTopFlange, m_context.TopPlate, horiSideUp, vertSideUp, triCutCtrlPlane, triCutLength);
                //线下部分的切割
                CreateTriCut(m_context.MainNearWebPlane.Invert(), m_context.spliceContext.MainTopFlangeInside, m_context.TopPlate, horiSideDown, vertSideDown, triCutCtrlPlane, triCutLength);
                //焊接
                CreateWeld(m_context.MainPart, m_context.TopPlate, m_weldParams[0], LoongCAD.STC.Define.SpaceAxis.AXIS_Z);
            }
            //焊缝2：下翼板（原名较低翼缘）与柱的焊接（1条）
            {
                //设置切割参数
                double thickness = m_context.spliceContext.m_main_t2;
                double horiSideUp = thickness / 2;
                double vertSideUp = thickness / 2;
                double horiSideDown = thickness / 2;
                double vertSideDown = thickness / 2;
                TriCutParameters(m_weldParams[1], ref horiSideUp, ref vertSideUp, ref horiSideDown, ref vertSideDown);
                //线上部分的切割
                Plane triCutCtrlPlane = m_context.IsSubRotate ? m_context.SubHoriCenterPlane : m_context.SubVertCenterPlane;
                double triCutLength = m_context.MainHeight;
                CreateTriCut(m_context.MainNearWebPlane.Invert(), m_context.spliceContext.MainBottomFlange, m_context.BottomPlate, horiSideDown, vertSideDown, triCutCtrlPlane, triCutLength);
                //线下部分的切割
                CreateTriCut(m_context.MainNearWebPlane.Invert(), m_context.spliceContext.MainBottomFlangeInside, m_context.BottomPlate, horiSideUp, vertSideUp, triCutCtrlPlane, triCutLength);
                //焊接
                CreateWeld(m_context.MainPart, m_context.BottomPlate, m_weldParams[1], LoongCAD.STC.Define.SpaceAxis.AXIS_NZ);
            }
            //焊缝3：腹板与柱的焊接（2条）
            {
                //切割
                double thickness = m_context.spliceContext.m_main_s;
                double horiSideUp = thickness / 2;
                double vertSideUp = thickness / 2;
                double horiSideDown = thickness / 2;
                double vertSideDown = thickness / 2;
                TriCutParameters(m_weldParams[2], ref horiSideUp, ref vertSideUp, ref horiSideDown, ref vertSideDown);
                Plane triCutCtrlPlane = m_context.IsSubRotate ? m_context.SubVertCenterPlane : m_context.SubHoriCenterPlane;
                double triCutLength = m_context.SubTopBound.Dist(m_context.SubBottomBound.Origin);
                CreateTriCut(m_context.MainNearWebPlane.Invert(), m_context.spliceContext.MainFrontWeb, m_context.WebPlate, horiSideDown, vertSideDown, triCutCtrlPlane, triCutLength);
                CreateTriCut(m_context.MainNearWebPlane.Invert(), m_context.spliceContext.MainBackWeb, m_context.WebPlate, horiSideUp, vertSideUp, triCutCtrlPlane, triCutLength);
                //焊接
                CreateWeld(m_context.MainPart, m_context.WebPlate, m_weldParams[2], LoongCAD.STC.Define.SpaceAxis.AXIS_X);
                //CreateWeld(m_context.MainPart, m_context.WebPlate, m_weldParams[2], LoongCAD.STC.Define.SpaceAxis.AXIS_NX);
            }
            //焊缝4：腹板与上翼缘板的焊接（1条）
            {
                CreateWeld(m_context.TopPlate, m_context.WebPlate, m_weldParams[3], LoongCAD.STC.Define.SpaceAxis.AXIS_Z);
            }
            //焊缝5：腹板与下翼缘板的焊接（1条）
            {
                CreateWeld(m_context.BottomPlate, m_context.WebPlate, m_weldParams[4], LoongCAD.STC.Define.SpaceAxis.AXIS_NZ);
            }
            //背杆焊缝
            {
                BackingBarPlaceMode topPlace = (BackingBarPlaceMode)m_paramsHandler.GetIntValue(ParamNames.BACKING_BAR_TOP_PLACE_MODE);
                BackingBarPlaceMode bottomPlace = (BackingBarPlaceMode)m_paramsHandler.GetIntValue(ParamNames.BACKING_BAR_BOTTOM_PLACE_MODE);
                if (m_context.TopBackingBar != null && m_context.TopPlate != null)
                {
                    if (topPlace == BackingBarPlaceMode.INSIDE)
                        CreateWeld(m_context.TopPlate, m_context.TopBackingBar, m_weldParams[5], LoongCAD.STC.Define.SpaceAxis.AXIS_Z);
                    else if (topPlace == BackingBarPlaceMode.OUTSIDE)
                        CreateWeld(m_context.TopPlate, m_context.TopBackingBar, m_weldParams[5], LoongCAD.STC.Define.SpaceAxis.AXIS_NZ);
                }
                if (m_context.BottomBackingBar != null && m_context.BottomPlate != null)
                {
                    if (topPlace == BackingBarPlaceMode.INSIDE)
                        CreateWeld(m_context.BottomPlate, m_context.BottomBackingBar, m_weldParams[5], LoongCAD.STC.Define.SpaceAxis.AXIS_NZ);
                    else if (topPlace == BackingBarPlaceMode.OUTSIDE)
                        CreateWeld(m_context.BottomPlate, m_context.BottomBackingBar, m_weldParams[5], LoongCAD.STC.Define.SpaceAxis.AXIS_Z);
                }
            }

            //构件关系
            PartBase prePart = m_context.WebPlate;
            if (prePart == null)
                prePart = m_context.TopPlate;
            else if (m_context.TopPlate == null)
                prePart = m_context.BottomPlate;
            if (prePart != null)
            {
                prePart.PreBuilt.Main = m_context.MainPart;
                prePart.PreBuilt.Refresh();
            }
        }

        public bool ParseParameter()
        {
            //确定方向，为自动焊缝的工作坐标系做准备
            Vec3 x = m_context.SubBackBound.Normal;
            if (m_context.IsSubRotate)
                x = m_context.TopOrient;
            Vec3 y = m_context.MainNearBoundPlane.Normal;
            //Vec3 z = m_context.TopOrient;
            workAxis = new Mat43(x, y, x.Cross(y), Vec3.ZERO);

            //焊缝属性
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                DObject weldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                WeldProperties weldProperties = new WeldProperties();
                weldProperties.SetProps(weldProps);
                m_weldParams.Add(weldProperties);
            }

            return true;
        }

        public void CreateWeld(PartBase mainPart, PartBase subPart, WeldProperties weldProps, LoongCAD.STC.Define.SpaceAxis spaceAxis)
        {
            if (mainPart == null || subPart == null)
                return;
            Weld weld = m_context.Comp.CreateAutoWeld(mainPart, subPart, LoongCAD.STC.Define.SpaceAxis.AXIS_X, true);
            weldProps.ApplyToWeld(weld);
            weld.WorkAxis = workAxis;
            weld.AutoWeldPosition = spaceAxis;
            weld.Refresh();
        }

        /// <summary>
        /// 设置焊接准备中三角形切割的参数
        /// </summary>
        private void TriCutParameters(WeldProperties weldProps, ref double horiSideUp, ref double vertSideUp, ref double horiSideDown, ref double vertSideDown)
        {
            Define.WeldType typeUp = weldProps.TypeUp;
            Define.WeldType typeDown = weldProps.TypeDown;
            Define.WeldContour contourUp = weldProps.ContourUp;
            Define.WeldContour contourDown = weldProps.ContourDown;
            //有效判断
            bool upVaild = ((typeUp != Define.WeldType.WELD_TYPE_FILLET && typeUp != Define.WeldType.WELD_TYPE_NONE) || contourUp != Define.WeldContour.WELD_CONTOUR_NONE);
            bool downVaild = ((typeDown != Define.WeldType.WELD_TYPE_FILLET && typeDown != Define.WeldType.WELD_TYPE_NONE) || contourDown != Define.WeldContour.WELD_CONTOUR_NONE);
            if (!upVaild)
                vertSideUp = 0;
            if (!downVaild)
                vertSideDown = 0;
            if (upVaild && typeDown == Define.WeldType.WELD_TYPE_NONE && contourDown == Define.WeldContour.WELD_CONTOUR_NONE)
                vertSideUp *= 2;
            if (!upVaild && downVaild && weldProps.SizeDown != double.MaxValue)
                vertSideDown *= 2;
            //尺寸处理
            if (weldProps.SizeUp != double.MaxValue)
                vertSideUp = weldProps.SizeUp;
            else
                weldProps.SizeUp = 6;
            if (weldProps.SizeDown != double.MaxValue)
                vertSideDown = weldProps.SizeDown;
            else
                weldProps.SizeDown = 6;
            //角度处理
            if (weldProps.GrooveAngleUp > 0 && weldProps.GrooveAngleUp < 90)
                horiSideUp = Math.Tan(weldProps.GrooveAngleUp * Math.PI / 180) * vertSideUp;
            else
                horiSideUp = vertSideUp;
            if (weldProps.GrooveAngleDown > 0 && weldProps.GrooveAngleDown < 90)
                horiSideDown = Math.Tan(weldProps.GrooveAngleDown * Math.PI / 180) * vertSideDown;
            else
                horiSideDown = vertSideDown;
        }

        /// <summary>
        /// 创建三角形的切割
        /// </summary>
        private void CreateTriCut(Plane sidePlane1, Plane sidePlane2, PartBase part, double side1, double side2, Plane triCutCtrlPlane, double cutLength)
        {
            if (side1 == 0 || side2 == 0)
                return;
            Vec3 tempVec = sidePlane1.Normal.Cross(sidePlane2.Normal);
            Vec3 acute1MoveVector = tempVec.Cross(sidePlane2.Normal).Normalize();
            Vec3 acute2MoveVector = sidePlane1.Normal.Cross(tempVec).Normalize();

            Vec3 rightPoint = triCutCtrlPlane.Intersect(sidePlane1, sidePlane2);
            Vec3 acutePoint1 = rightPoint + acute1MoveVector * side1; //side是直角三角形的直角边长
            Vec3 acutePoint2 = rightPoint + acute2MoveVector * side2;

            Plane sidePlane3 = new Plane(acutePoint1, acutePoint2 - acutePoint1, triCutCtrlPlane.Normal);
            Plane ctrlBottomPlane = triCutCtrlPlane.Move(-cutLength / 2);
            Plane ctrlTopPlane = triCutCtrlPlane.Move(cutLength / 2);
            PartBase cutPartBase = m_context.Comp.CreatePlateCutter(part, ctrlBottomPlane, ctrlTopPlane, new Plane[] { sidePlane1, sidePlane2, sidePlane3 }, true);
            cutPartBase.Flush();
        }
    }
}
