﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;

namespace SpliceConnection77
{ 
    public class ComponentInstanceContext
    {
        public static Plane DefaultPlane = new Plane(Vec3.ZERO + int.MaxValue * Vec3.AXIS_X, Vec3.AXIS_Z);
        private ConnectionComponent m_comp;

        public double gap; //梁中间间距
        private double m_webWidth = 0; //此值不为0（b参数未设置时）时，作为腹连接板宽度
        private double m_webHeight = 0; 
        public double webLocation = double.MaxValue; //作为腹连接板距主梁上翼缘距离
        //连接板长宽
        private double m_topFlangeWidth = 0;
        private double m_topFlangeHeight = 0;
        private double m_topFlangeInsideWidth = 0;
        private double m_bottomFlangeWidth = 0;
        private double m_bottomFlangeHeight = 0;
        private double m_bottomFlangeInsideWidth = 0;
        public double flangeInsideLocation = double.MaxValue; //翼缘内侧连接板默认与外缘对齐

        //public double web_bolt_width_edge_gap = 40.0;  //更改了定位规则



        // 主零件
        public Beam m_mainPart = null;
        private BeamPlanes m_mainPlanes;

        private Plane m_mainAlignPlane = null; //主零件切割面
        private Plane m_mainNearCenterPlane = null; //主零件靠近中心的面
        private Plane m_mainAwayCenterPlane = null; //主零件远离中心的面
        private Plane m_mainFront = null; //主零件的前侧面
        private Plane m_mainBack = null; //主零件的后侧面

        private Plane m_mainTopFlange = DefaultPlane; //顶翼板的上表面
        private Plane m_mainTopFlangeInside = DefaultPlane; //顶翼板的下表面

        private Plane m_mainBottomFlange = DefaultPlane; //底翼板的外侧面
        private Plane m_mainBottomFlangeInside = DefaultPlane; //底翼板的内侧面

        private Plane m_mainFrontWeb = DefaultPlane; //前腹板面
        private Plane m_mainWebCenter = DefaultPlane; //腹板中心面(法线为front方向)
        private Plane m_mainHCenter = DefaultPlane; //梁中心面（法线为Top方向）
        private Plane m_mainBackWeb = DefaultPlane; //后腹板面

        private double m_mainHeadTailDistance; //梁长度
        public double m_main_b1; //翼缘宽度
        public double m_main_s; //腹板厚度
        public double m_main_h; //高度
        public double m_main_t; //翼缘厚度


        //次零件
        private Beam m_subPart = null;
        private BeamPlanes m_subPlanes;

        private Plane m_subAlignPlane = null; 
        private Plane m_subNearCenterPlane = null; 
        private Plane m_subAwayCenterPlane = null;
        private Plane m_subFront = null; //次零件的前侧面
        private Plane m_subBack = null; //次零件的后侧面

        private Plane m_subTopFlange = DefaultPlane; //顶翼板的上表面
        private Plane m_subTopFlangeInside = DefaultPlane; //顶翼板的下表面

        private Plane m_subBottomFlange = DefaultPlane; //底翼板的外侧表面
        private Plane m_subBottomFlangeInside = DefaultPlane; //底翼板的内侧表面

        private Plane m_subFrontWeb = DefaultPlane; //前腹板面
        private Plane m_subWebCenter = DefaultPlane; //腹板中心面
        private Plane m_subHCenter = DefaultPlane;
        private Plane m_subBackWeb = DefaultPlane; //后腹板面

        public double m_subHeadTailDistance; //梁长度
        public double m_sub_b1;
        public double m_sub_s;
        public double m_sub_h;
        public double m_sub_t; //翼缘厚度

        //
        private CommonBoltGroup m_topBoltGroup = new CommonBoltGroup();
        private CommonBoltGroup m_webBoltGroup = new CommonBoltGroup();
        private CommonBoltGroup m_bottomBoltGroup = new CommonBoltGroup();

        //
        private BoltParameter m_topFlangeBoltParameter = new TopFlangeBoltParameter();
        private BoltParameter m_bottomFlangeBoltParameter = new BottomFlangeBoltParameter();
        private BoltParameter m_webBoltParameter = new WebBoltParameter();

        //
        private CommonPlateCreate m_topTopPlateCreate;
        private CommonPlateCreate m_topFrontPlateCreate;
        private CommonPlateCreate m_topBackPlateCreate;
        private CommonPlateCreate m_webFrontPlateCreate;
        private CommonPlateCreate m_webBackPlateCreate;
        private CommonPlateCreate m_bottomBottomPlateCreate;
        private CommonPlateCreate m_bottomFrontPlateCreate;
        private CommonPlateCreate m_bottomBackPlateCreate;

        //10处螺栓的开关
        private int m_boltMode = 0b1111111111;

        //螺栓是否在工厂安装
        private bool m_mainBoltFactoryMode = false;
        private bool m_subBoltFactoryMode = false;


        ComponentInstanceContext()
        {

        }

        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (!compContext.Initialize(comp))
            {
                compContext.Clear();
                return null;
            }
            return compContext;
        }

        public bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;
            Beam mainPart = comp.Main as Beam;
            Beam subPart = comp.Subs.Length > 0 ? comp.Subs.First<TFObject>() as Beam : null;
            if (mainPart == null || subPart == null)
                return false;
            m_comp = comp;
            m_mainPart = mainPart;
            m_subPart = subPart;

            //初始化板
            m_topTopPlateCreate = new CommonPlateCreate(this);
            m_topFrontPlateCreate = new CommonPlateCreate(this);
            m_topBackPlateCreate = new CommonPlateCreate(this);
            m_webFrontPlateCreate = new CommonPlateCreate(this);
            m_webBackPlateCreate = new CommonPlateCreate(this);
            m_bottomBottomPlateCreate = new CommonPlateCreate(this);
            m_bottomFrontPlateCreate = new CommonPlateCreate(this);
            m_bottomBackPlateCreate = new CommonPlateCreate(this);

            m_mainPlanes = BeamPlanes.CreateFromBeam(m_mainPart);
            m_subPlanes = BeamPlanes.CreateFromBeam(m_subPart);

            /*
             * 设置主次梁靠近中心的面
             */
            Plane mainPlane1 = MainPlanes.HeadBound;
            Plane mainPlane2 = MainPlanes.TailBound;
            Plane subPlane1 = SubPlanes.HeadBound;
            Plane subPlane2 = SubPlanes.TailBound;
            MainHeadTailDistance = mainPlane1.Dist(mainPlane2.Origin);//计算长度
            SubHeadTailDistance = subPlane1.Dist(subPlane2.Origin);
            //主梁近心面
            if (mainPlane1.Dist(subPlane1.Origin) <= mainPlane2.Dist(subPlane1.Origin))
            {//距离subPlane1距离小的较近
                MainNearCenterPlane = mainPlane1;
                MainAwayCenterPlane = mainPlane2;
            }
            else
            {
                MainNearCenterPlane = mainPlane2;
                MainAwayCenterPlane = mainPlane1;
            }
            //主梁近心面
            if (subPlane1.Dist(mainPlane1.Origin) <= subPlane2.Dist(mainPlane1.Origin))
            {//距离mainPlane1距离小的较近
                SubNearCenterPlane = subPlane1;
                SubAwayCenterPlane = subPlane2;
            }
            else
            {
                SubNearCenterPlane = subPlane2;
                SubAwayCenterPlane = subPlane1;
            }
            MainAlignPlane = MainAwayCenterPlane.Clone();
            SubAlignPlane = SubAwayCenterPlane.Clone();


            /*
             * 截面平行校验
             */
            Vec3 parallelChecked = mainPlane1.Normal.Cross(subPlane1.Normal);
            if (parallelChecked.Dist(Vec3.ZERO) > 1e-6)
                return false;


            /*
             * 设置一些基础面
             */

            //前后面
            m_mainFront = m_mainPlanes.FrontBound;
            m_mainBack = m_mainPlanes.BackBound;
            m_subFront = m_subPlanes.FrontBound;
            m_subBack = m_subPlanes.BackBound;

            //腹板面
            if(m_mainPlanes.FrontWeb != null)
                m_mainFrontWeb = m_mainPlanes.FrontWeb; //前腹板面
            if (m_mainPlanes.BackWeb != null)
                m_mainBackWeb = m_mainPlanes.BackWeb; //后腹板面
            if (m_subPlanes.FrontWeb != null)
                m_subFrontWeb = m_subPlanes.FrontWeb;
            if (m_subPlanes.BackWeb != null)
                m_subBackWeb = m_subPlanes.BackWeb;

            //上翼缘面
            if (m_mainPlanes.TopFlange != null)
                m_mainTopFlange = m_mainPlanes.TopFlange;  //主
            if (m_mainPlanes.TopFlangeInside != null)
                m_mainTopFlangeInside = m_mainPlanes.TopFlangeInside;
            if (m_subPlanes.TopFlange != null)
                m_subTopFlange = m_subPlanes.TopFlange;  //次
            if (m_subPlanes.TopFlangeInside != null)
                m_subTopFlangeInside = m_subPlanes.TopFlangeInside;

            //下翼缘面
            if (m_mainPlanes.BottomFlange != null)
                m_mainBottomFlange = m_mainPlanes.BottomFlange;
            if (m_mainPlanes.BottomFlangeInside != null)
                m_mainBottomFlangeInside = m_mainPlanes.BottomFlangeInside;
            if (m_subPlanes.BottomFlange != null)
                m_subBottomFlange = m_subPlanes.BottomFlange;
            if (m_subPlanes.BottomFlangeInside != null)
                m_subBottomFlangeInside = m_subPlanes.BottomFlangeInside;

            //相对角度校验
            Vec3 angleChecked = m_mainTopFlange.Normal.Cross(m_subTopFlange.Normal);
            if (angleChecked.Dist(Vec3.ZERO) > 1e-6)
                return false;

            //根据主次的点击顺序，调整前后腹板面
            Vec3 frontOrient = m_mainNearCenterPlane.Normal.Cross(m_mainTopFlange.Normal);
            if (frontOrient.Dot(m_mainFront.Normal) < 0)
            {
                Swap(ref m_mainFront, ref m_mainBack);
                Swap(ref m_mainFrontWeb, ref m_mainBackWeb);
            }

            //调整次零件朝向，确保主次零件相同部位朝向一样
            if (!IsSameOrient(m_mainTopFlange, m_subTopFlange)) //Top-Bottom方向不同，互换值
            {
                Swap(ref m_subTopFlange, ref m_subBottomFlange);
                Swap(ref m_subTopFlangeInside, ref m_subBottomFlangeInside);
            }
            if (!IsSameOrient(m_mainFront, m_subFront)) //Front-Back方向不同，互换值
            {
                Swap(ref m_subFront, ref m_subBack);
                Swap(ref m_subFrontWeb, ref m_subBackWeb);
            }


            m_main_b1 = m_mainFront.Dist(m_mainBack.Origin);
            m_mainWebCenter = m_mainFront.Move(-m_main_b1 / 2);

            m_sub_b1 = m_subFront.Dist(m_subBack.Origin);
            m_subWebCenter = m_subFront.Move(-m_sub_b1 / 2);

            m_main_s = m_mainFrontWeb.Dist(m_mainBackWeb.Origin);
            m_sub_s = m_subFrontWeb.Dist(m_subBackWeb.Origin);

            m_main_h = m_mainTopFlange.Dist(m_mainBottomFlange.Origin);
            m_mainHCenter = m_mainTopFlange.Move(-m_main_h / 2);

            m_sub_h = m_subTopFlange.Dist(m_subBottomFlange.Origin);
            m_subHCenter = m_subTopFlange.Move(-m_sub_h / 2);

            m_main_t = m_mainTopFlange.Dist(m_mainTopFlangeInside.Origin);
            m_sub_t = m_subTopFlange.Dist(m_subTopFlangeInside.Origin);

            return true;
        }


        public bool IsSameOrient(Plane p1, Plane p2)
        {
            return p1.Normal.Dot(p2.Normal) > 0;
        }

        public void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_subPart = null;
            m_mainPlanes = null;
            m_subPlanes = null;

            m_mainNearCenterPlane = null; 
            m_mainAwayCenterPlane = null; 
            m_subNearCenterPlane = null; 
            m_subAwayCenterPlane = null;
            m_mainAlignPlane = null; 
            m_subAlignPlane = null;
        }

        public static void Swap(ref Plane p1, ref Plane p2)
        {
            Plane p = p1;
            p1 = p2;
            p2 = p;
        }

        public ConnectionComponent Comp { get => m_comp; set => m_comp = value; }
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public Beam SubPart { get => m_subPart; set => m_subPart = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public BeamPlanes SubPlanes { get => m_subPlanes; set => m_subPlanes = value; }
        public Plane MainAlignPlane { get => m_mainAlignPlane; set => m_mainAlignPlane = value; }
        public Plane SubAlignPlane { get => m_subAlignPlane; set => m_subAlignPlane = value; }
        public Plane MainNearCenterPlane { get => m_mainNearCenterPlane; set => m_mainNearCenterPlane = value; }
        public Plane MainAwayCenterPlane { get => m_mainAwayCenterPlane; set => m_mainAwayCenterPlane = value; }
        public Plane SubNearCenterPlane { get => m_subNearCenterPlane; set => m_subNearCenterPlane = value; }
        public Plane SubAwayCenterPlane { get => m_subAwayCenterPlane; set => m_subAwayCenterPlane = value; }
        public Plane MainTopFlange { get => m_mainTopFlange; set => m_mainTopFlange = value; }
        public Plane MainFront { get => m_mainFront; set => m_mainFront = value; }
        public Plane MainBack { get => m_mainBack; set => m_mainBack = value; }
        public Plane SubTopFlange { get => m_subTopFlange; set => m_subTopFlange = value; }
        public Plane MainTopFlangeInside { get => m_mainTopFlangeInside; set => m_mainTopFlangeInside = value; }
        public Plane SubTopFlangeInside { get => m_subTopFlangeInside; set => m_subTopFlangeInside = value; }
        public Plane MainFrontWeb { get => m_mainFrontWeb; set => m_mainFrontWeb = value; }
        public Plane MainWebCenter { get => m_mainWebCenter; set => m_mainWebCenter = value; }
        public Plane MainBackWeb { get => m_mainBackWeb; set => m_mainBackWeb = value; }
        public Plane SubFrontWeb { get => m_subFrontWeb; set => m_subFrontWeb = value; }
        public Plane SubBackWeb { get => m_subBackWeb; set => m_subBackWeb = value; }
        public Plane MainHCenter { get => m_mainHCenter; set => m_mainHCenter = value; }
        public Plane MainBottomFlange { get => m_mainBottomFlange; set => m_mainBottomFlange = value; }
        public Plane SubBottomFlange { get => m_subBottomFlange; set => m_subBottomFlange = value; }
        public Plane MainBottomFlangeInside { get => m_mainBottomFlangeInside; set => m_mainBottomFlangeInside = value; }
        public Plane SubBottomFlangeInside { get => m_subBottomFlangeInside; set => m_subBottomFlangeInside = value; }
        public CommonBoltGroup TopBoltGroup { get => m_topBoltGroup; set => m_topBoltGroup = value; }
        public CommonBoltGroup WebBoltGroup { get => m_webBoltGroup; set => m_webBoltGroup = value; }
        public CommonBoltGroup BottomBoltGroup { get => m_bottomBoltGroup; set => m_bottomBoltGroup = value; }
        public BoltParameter TopFlangeBoltParameter { get => m_topFlangeBoltParameter; set => m_topFlangeBoltParameter = value; }
        public BoltParameter BottomFlangeBoltParameter { get => m_bottomFlangeBoltParameter; set => m_bottomFlangeBoltParameter = value; }
        public BoltParameter WebBoltParameter { get => m_webBoltParameter; set => m_webBoltParameter = value; }
        public CommonPlateCreate TopTopPlateCreate { get => m_topTopPlateCreate; set => m_topTopPlateCreate = value; }
        public CommonPlateCreate TopFrontPlateCreate { get => m_topFrontPlateCreate; set => m_topFrontPlateCreate = value; }
        public CommonPlateCreate TopBackPlateCreate { get => m_topBackPlateCreate; set => m_topBackPlateCreate = value; }
        public CommonPlateCreate WebFrontPlateCreate { get => m_webFrontPlateCreate; set => m_webFrontPlateCreate = value; }
        public CommonPlateCreate WebBackPlateCreate { get => m_webBackPlateCreate; set => m_webBackPlateCreate = value; }
        public CommonPlateCreate BottomBottomPlateCreate { get => m_bottomBottomPlateCreate; set => m_bottomBottomPlateCreate = value; }
        public CommonPlateCreate BottomFrontPlateCreate { get => m_bottomFrontPlateCreate; set => m_bottomFrontPlateCreate = value; }
        public CommonPlateCreate BottomBackPlateCreate { get => m_bottomBackPlateCreate; set => m_bottomBackPlateCreate = value; }
        public Plane SubFront { get => m_subFront; set => m_subFront = value; }
        public Plane SubBack { get => m_subBack; set => m_subBack = value; }
        public Plane SubHCenter { get => m_subHCenter; set => m_subHCenter = value; }
        public int BoltMode { get => m_boltMode; set => m_boltMode = value; }
        public bool MainBoltFactoryMode { get => m_mainBoltFactoryMode; set => m_mainBoltFactoryMode = value; }
        public bool SubBoltFactoryMode { get => m_subBoltFactoryMode; set => m_subBoltFactoryMode = value; }
        public double MainHeadTailDistance { get => m_mainHeadTailDistance; set => m_mainHeadTailDistance = value; }
        public double SubHeadTailDistance { get => m_subHeadTailDistance; set => m_subHeadTailDistance = value; }
        public double Gap { get => gap; set => gap = value; }
        public double WebWidth { get => m_webWidth; set => m_webWidth = value; }
        public double WebHeight { get => m_webHeight; set => m_webHeight = value; }
        public double TopFlangeWidth { get => m_topFlangeWidth; set => m_topFlangeWidth = value; }
        public double TopFlangeHeight { get => m_topFlangeHeight; set => m_topFlangeHeight = value; }
        public double BottomFlangeWidth { get => m_bottomFlangeWidth; set => m_bottomFlangeWidth = value; }
        public double BottomFlangeHeight { get => m_bottomFlangeHeight; set => m_bottomFlangeHeight = value; }
        public double TopFlangeInsideWidth { get => m_topFlangeInsideWidth; set => m_topFlangeInsideWidth = value; }
        public double BottomFlangeInsideWidth { get => m_bottomFlangeInsideWidth; set => m_bottomFlangeInsideWidth = value; }

    }
}
