﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.Util;
using System.Diagnostics;
using System.IO;
using System.Linq.Expressions;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Threading.Tasks;


namespace SpliceConnection77
{
    public class BottomFlangeBoltParameter : BoltParameter
    {
        private ParamsHandler paramsHandler;
        private ComponentInstanceContext m_context;
        public BottomFlangeBoltParameter() { }


        public override BoltParameter Clone()
        {
            BoltParameter boltParameter = new BottomFlangeBoltParameter();
            boltParameter.BoltStandard = this.BoltStandard;
            boltParameter.BoltSize = this.BoltSize;
            boltParameter.HoleTol = this.HoleTol;
            boltParameter.ThreadInMaterial = this.ThreadInMaterial;
            boltParameter.MadeType = this.MadeType;
            boltParameter.HoleX = this.HoleX;
            boltParameter.HoleY = this.HoleY;
            boltParameter.HoleType = this.HoleType;
            boltParameter.RotateSlot = this.RotateSlot;
            boltParameter.SectionFlags = this.SectionFlags;
            boltParameter.CutLength = this.CutLength;
            boltParameter.ExtLength = this.ExtLength;
            boltParameter.RotationSide = this.RotationSide;
            boltParameter.BoltShape = this.BoltShape;
            return boltParameter;
        }



        /*
         * 基础属性
         */
        public override void InitialBasic(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            DObject basicProps = paramsHandler.GetDObjectValue(ParamNames.BOLT_FLANGE_BASIC_PROPS);
            CopyFromBasicProps(basicProps);

            DObject holeProps = paramsHandler.GetDObjectValue(ParamNames.BOLT_FLANGE_HOLE_PROPS);
            CopyFromHoleProps(holeProps);

            DObject contentProps = paramsHandler.GetDObjectValue(ParamNames.BOLT_FLANGE_CONTENT_PROPS);
            CopyFromContentProps(contentProps);

            DObject extendProps = paramsHandler.GetDObjectValue(ParamNames.BOLT_FLANGE_EXTEND_PROPS);
            CopyFromExtendProps(extendProps);

            int shapeIndex = paramsHandler.GetIntValue(ParamNames.BOLT_FLANGE_SHAPE);
            SetShape(shapeIndex);

            int orientIndex = paramsHandler.GetIntValue(ParamNames.BOLT_FLANGE_ORIENTATION);
            SetOrient(orientIndex);

            this.paramsHandler = paramsHandler;
            this.m_context = compContext;
        }

        /*
        * 位置属性
        */
        public override void InitalLocation(CommonBoltGroup bg)
        {
            bg.SetHoriPlane(m_context.SubNearCenterPlane.Invert(), m_context.MainNearCenterPlane.Invert());
            bg.SetVertPlane(m_context.MainWebCenter.Invert(), m_context.MainWebCenter);
            //处理水平方向上的螺栓分布
            CommonBoltLocation horiBoltLocation;
            {
                //边距
                double centerGap = paramsHandler.GetDoubleValue(ParamNames.BOLT_FLANGE_HORI_CENTER_MARGIN);
                double sideGap = paramsHandler.GetDoubleValue(ParamNames.BOLT_FLANGE_HORI_EDGE_MARGIN);
                //螺栓之间的间距
                int count = paramsHandler.GetIntValue(ParamNames.BOLT_FLANGE_HORI_COUNT);
                double[] originStride = paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_FLANGE_HORI_STRIDE);
                //自动布置螺栓
                horiBoltLocation = new CommonBoltLocation(40, 40, 3, new double[] { 75.0 });
                horiBoltLocation.SetBoltLocation(centerGap, sideGap, count, originStride, (m_context.BottomFlangeWidth - m_context.Gap) / 2); //自动排列螺栓
                bg.SetHoriLocationProps(horiBoltLocation.gap1, horiBoltLocation.stride, horiBoltLocation.gap2, CommonBoltLocation.BoltLocationMode.MODE1);
                if (m_context.BottomFlangeInsideWidth == double.MaxValue) //内侧连接板未设置width时，才用默认值
                {
                    if (m_context.BottomFlangeWidth == double.MaxValue)
                        m_context.BottomFlangeInsideWidth = bg.TotalLengthHori; //外侧的连接板没有设置width时，用默认
                    else
                        m_context.BottomFlangeInsideWidth = 460.0 + m_context.Gap;
                }
                m_context.BottomFlangeWidth = bg.TotalLengthHori;
            }
            //处理垂直方向上的螺栓分布
            CommonBoltLocation vertBoltLocation;
            {
                //内侧连接板的定位
                double flangeInsideLocation = paramsHandler.GetDoubleValue(ParamNames.BOLT_FLANGE_VERT_INSIDE_MARGIN);
                m_context.flangeInsideLocation = flangeInsideLocation;
                //
                double centerGap = paramsHandler.GetDoubleValue(ParamNames.BOLT_FLANGE_VERT_CENTER_GAP);
                double sideGap = paramsHandler.GetDoubleValue(ParamNames.BOLT_FLANGE_VERT_EDGE_MARGIN);
                int count = paramsHandler.GetIntValue(ParamNames.BOLT_FLANGE_VERT_COUNT);
                double[] originStride = paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_FLANGE_VERT_STRIDE);
                vertBoltLocation = new CommonBoltLocation(59, 35, 1, new double[] { 50.0 });
                vertBoltLocation.SetBoltLocation(centerGap / 2, sideGap, count, originStride, m_context.BottomFlangeHeight / 2); //自动排列螺栓
                bg.SetVertLocationProps(vertBoltLocation.gap1 * 2, vertBoltLocation.stride, vertBoltLocation.gap2, CommonBoltLocation.BoltLocationMode.MODE2);
                m_context.BottomFlangeHeight = bg.TotalLengthVert;
            }
            //
            if (horiBoltLocation.count == 0 || vertBoltLocation.count == 0)
            {
                bg.boltFlags[0] = false;
                bg.boltFlags[1] = false;
                bg.boltFlags[2] = false;
                bg.boltFlags[3] = false;
            }

        }

    }
}
