﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.Util;

namespace SpliceConnection77
{
    public class WebBoltParameter : BoltParameter
    {
        public WebBoltParameter() { }

        private ParamsHandler paramsHandler;
        private ComponentInstanceContext m_context;


        public override BoltParameter Clone()
        {
            BoltParameter boltParameter = new WebBoltParameter();
            boltParameter.BoltStandard = this.BoltStandard;
            boltParameter.BoltSize = this.BoltSize;
            boltParameter.HoleTol = this.HoleTol;
            boltParameter.ThreadInMaterial = this.ThreadInMaterial;
            boltParameter.MadeType = this.MadeType;
            boltParameter.HoleX = this.HoleX;
            boltParameter.HoleY = this.HoleY;
            boltParameter.HoleType = this.HoleType;
            boltParameter.RotateSlot = this.RotateSlot;
            boltParameter.SectionFlags = this.SectionFlags;
            boltParameter.CutLength = this.CutLength;
            boltParameter.ExtLength = this.ExtLength;
            boltParameter.RotationSide = this.RotationSide;
            boltParameter.BoltShape = this.BoltShape;
            return boltParameter;
        }

        /*
         * 基础属性
         */
        public override void InitialBasic(ComponentInstanceContext compContext, ParamsHandler paramsHandler) 
        {
            DObject basicProps = paramsHandler.GetDObjectValue(ParamNames.BOLT_WEB_BASIC_PROPS);
            CopyFromBasicProps(basicProps);

            DObject holeProps = paramsHandler.GetDObjectValue(ParamNames.BOLT_WEB_HOLE_PROPS);
            CopyFromHoleProps(holeProps);

            DObject contentProps = paramsHandler.GetDObjectValue(ParamNames.BOLT_WEB_CONTENT_PROPS);
            CopyFromContentProps(contentProps);

            DObject extendProps = paramsHandler.GetDObjectValue(ParamNames.BOLT_WEB_EXTEND_PROPS);
            CopyFromExtendProps(extendProps);

            int shapeIndex = paramsHandler.GetIntValue(ParamNames.BOLT_WEB_SHAPE);
            SetShape(shapeIndex);

            int orientIndex = paramsHandler.GetIntValue(ParamNames.BOLT_WEB_ORIENTATION);
            SetOrient(orientIndex);


            this.paramsHandler = paramsHandler;
            this.m_context = compContext;
        }


        public override void InitalLocation(CommonBoltGroup bg)
        {
            bg.SetHoriPlane(m_context.SubNearCenterPlane.Invert(), m_context.MainNearCenterPlane.Invert());
            //水平方向
            CommonBoltLocation horiBoltLocation;
            {
                double centerGap = paramsHandler.GetDoubleValue(ParamNames.BOLT_WEB_HORI_CENTER_MARGIN);
                double sideGap = paramsHandler.GetDoubleValue(ParamNames.BOLT_WEB_HORI_EDGE_MARGIN);
                int count = paramsHandler.GetIntValue(ParamNames.BOLT_WEB_HORI_COUNT);
                double[] originStride = paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_WEB_HORI_STRIDE);
                //
                horiBoltLocation = new CommonBoltLocation(40, 40, 2, new double[] { 75.0 });
                horiBoltLocation.SetBoltLocation(centerGap, sideGap, count, originStride, (m_context.WebWidth - m_context.Gap) / 2); //自动排列螺栓
                bg.SetHoriLocationProps(horiBoltLocation.gap1, horiBoltLocation.stride, horiBoltLocation.gap2, CommonBoltLocation.BoltLocationMode.MODE1);
                m_context.WebWidth = bg.TotalLengthHori;
            }
            //垂直方向
            CommonBoltLocation vertBoltLocation;
            {
                //手动设置腹板位置
                double webLocation = paramsHandler.GetDoubleValue(ParamNames.BOLT_WEB_VERT_FLANGE_MARGIN);
                m_context.webLocation = webLocation;
                //
                double sideGap = paramsHandler.GetDoubleValue(ParamNames.BOLT_WEB_VERT_EDGE_MARGIN);
                int count = paramsHandler.GetIntValue(ParamNames.BOLT_WEB_VERT_COUNT);
                double[] originStride = paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_WEB_VERT_STRIDE);
                //TODO 这里对冲突的情况做特殊处理，后续可以设置控件为灰色时移除
                if(sideGap != double.MaxValue && originStride.Length != 0 && count != int.MaxValue && m_context.WebHeight != double.MaxValue)
                {
                    sideGap = double.MaxValue;
                }
                //
                vertBoltLocation = new CommonBoltLocation(40, 40, 2, new double[] { 75.0 });
                vertBoltLocation.centerMode = true;
                vertBoltLocation.SetBoltLocation(sideGap, sideGap, count, originStride, m_context.WebHeight); //自动排列螺栓
                bg.SetVertLocationProps(vertBoltLocation.gap1, vertBoltLocation.stride, vertBoltLocation.gap2, CommonBoltLocation.BoltLocationMode.MODE3);
                m_context.WebHeight = bg.TotalLengthVert;
            }
            if (m_context.webLocation != double.MaxValue) //手动定位腹连接板
            {
                bg.SetVertPlane(m_context.MainTopFlange.Move(-m_context.webLocation - m_context.WebHeight), null);
            }
            else
            {
                bg.SetVertPlane(m_context.MainHCenter.Move(-m_context.WebHeight / 2), null);
            }
            //
            if (horiBoltLocation.count == 0 || vertBoltLocation.count == 0)
            {
                bg.boltFlags[0] = false;
                bg.boltFlags[1] = false;
                bg.boltFlags[2] = false;
                bg.boltFlags[3] = false;
            }
        }

    }
}
