﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace SpliceConnection77
{
    class BoltsHandler
    {
        private ComponentInstanceContext m_context = null; //公共参数、方法
        private ParamsHandler m_paramsHandler = null; //包含UI里的参数

        public BoltsHandler(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }
        public void Execute()
        {
            //螺栓组主零件
            PartBase mainPart = m_context.MainPart;
            PartBase subPart = m_context.SubPart;
            PartBase[] main = { mainPart, subPart }; //分别以主梁和次梁作为螺栓的主零件

            //是否启用螺栓
            int mode1 = m_context.BoltMode >> 6;
            int mode2 = m_context.BoltMode >> 2 & 0b1100;
            int mode3 = m_context.BoltMode & 0b1111;

            //设置切割长度
            m_context.TopFlangeBoltParameter.CutLength = m_context.TopTopPlateCreate.outerPlane.Dist(m_context.TopFrontPlateCreate.outerPlane.Origin) * 2 + 1;
            m_context.WebBoltParameter.CutLength = m_context.WebFrontPlateCreate.outerPlane.Dist(m_context.WebBackPlateCreate.outerPlane.Origin) * 2 + 1;
            m_context.BottomFlangeBoltParameter.CutLength = m_context.BottomBottomPlateCreate.outerPlane.Dist(m_context.BottomFrontPlateCreate.outerPlane.Origin) * 2 + 1;


            //处理车间螺栓选项
            BoltParameter topMainBoltParameter = m_context.TopFlangeBoltParameter.Clone();
            BoltParameter topSubBoltParameter = m_context.TopFlangeBoltParameter.Clone();
            BoltParameter webMainBoltParameter = m_context.WebBoltParameter.Clone();
            BoltParameter webSubBoltParameter = m_context.WebBoltParameter.Clone();
            BoltParameter bottomMainBoltParameter = m_context.BottomFlangeBoltParameter.Clone();
            BoltParameter bottomSubBoltParameter = m_context.BottomFlangeBoltParameter.Clone();
            if(m_context.MainBoltFactoryMode)
            {
                topMainBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
                webMainBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
                bottomMainBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
            }
            if (m_context.SubBoltFactoryMode)
            {
                topSubBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
                webSubBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
                bottomSubBoltParameter.MadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
            }

            //上翼缘螺栓
            m_context.TopBoltGroup.Initial(m_context.Comp, m_context.TopTopPlateCreate.outerPlane);
            m_context.TopBoltGroup.SetBoltParameters(topMainBoltParameter, topSubBoltParameter);
            m_context.TopBoltGroup.AddMainParts(main);
            m_context.TopBoltGroup.AddSubParts(m_context.TopFrontPlateCreate.mainParts, m_context.TopBackPlateCreate.mainParts, m_context.TopFrontPlateCreate.subParts, m_context.TopBackPlateCreate.subParts);
            m_context.TopBoltGroup.AddSubParts(m_context.TopTopPlateCreate.mainParts, m_context.TopTopPlateCreate.subParts);
            m_context.TopBoltGroup.Execute(mode1);

            //腹板螺栓
            m_context.WebBoltGroup.Initial(m_context.Comp, m_context.WebFrontPlateCreate.outerPlane);
            m_context.WebBoltGroup.SetBoltParameters(webMainBoltParameter, webSubBoltParameter);
            m_context.WebBoltGroup.AddMainParts(main);
            m_context.WebBoltGroup.AddSubParts(m_context.WebFrontPlateCreate.mainParts, m_context.WebFrontPlateCreate.subParts); 
            m_context.WebBoltGroup.AddSubParts(m_context.WebBackPlateCreate.mainParts, m_context.WebBackPlateCreate.subParts);
            m_context.WebBoltGroup.Execute(mode2);

            //下翼缘螺栓
            m_context.BottomBoltGroup.Initial(m_context.Comp, m_context.BottomBottomPlateCreate.outerPlane);
            m_context.BottomBoltGroup.SetBoltParameters(bottomMainBoltParameter, bottomSubBoltParameter);
            m_context.BottomBoltGroup.AddMainParts(main);
            m_context.BottomBoltGroup.AddSubParts(m_context.BottomFrontPlateCreate.mainParts, m_context.BottomBackPlateCreate.mainParts, m_context.BottomFrontPlateCreate.subParts, m_context.BottomBackPlateCreate.subParts);
            m_context.BottomBoltGroup.AddSubParts(m_context.BottomBottomPlateCreate.mainParts, m_context.BottomBottomPlateCreate.subParts);
            m_context.BottomBoltGroup.Execute(mode3);
        }

    }
}
