﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace SpliceConnection77
{
    class CommonPlateWeld
    {
        private Plane mainBeamWeldCreatePlane, subBeamWeldCreatePlane; //比较了梁和连接板的前后
        private Plane mainConnWeldCreatePlane, subConnWeldCreatePlane; //不比较前后，在指定面上创建焊缝（frontPlane或backPlane）

        //PlateCreate类的属性
        private ConnectionComponent cmp;

        private PartBase mainPart;
        private PartBase subPart;
        private List<PartBase> mainParts;
        private List<PartBase> subParts;

        private Plane mainCtrlPlane, subCtrlPlane; //主次梁表面
        public Plane frontPlane, backPlane, headPlane, tailPlane;
        private Plane connCtrlPlane; //连接板的创建面 
        private Plane ubCtrlPlane; //垫板的创建面
        private Plane ubHead;
        private Plane ubTail;

        private double distance;
        private double length;
        private double thickness;
        private double underBoardThickNess;

        private WeldProperties weldProperties;

        //为焊缝做准备
        public List<Plane> underPlanes = new List<Plane>();

        public CommonPlateWeld(CommonPlateCreate plateCreate, WeldProperties weldProperties)
        {
            mainParts = plateCreate.mainParts; //parts的第一个是连接板,所以plane的的个数少一
            subParts = plateCreate.subParts;
            cmp = plateCreate.cmp;
            mainCtrlPlane = plateCreate.mainCtrlPlane;
            subCtrlPlane = plateCreate.subCtrlPlane;
            frontPlane = plateCreate.frontPlane;
            backPlane = plateCreate.backPlane;
            headPlane = plateCreate.headPlane;
            tailPlane = plateCreate.tailPlane;
            underBoardThickNess = plateCreate.underBoardThickness;
            mainPart = plateCreate.mainPart;
            subPart = plateCreate.subPart;
            connCtrlPlane = plateCreate.connCtrlPlane;
            ubCtrlPlane = plateCreate.ubCtrlPlane;
            ubHead = plateCreate.ubHead;
            ubTail = plateCreate.ubTail;
            distance = plateCreate.distance;
            length = plateCreate.length;
            underPlanes = plateCreate.underPlanes;
            thickness = plateCreate.thickness;
            underBoardThickNess = plateCreate.underBoardThickness;

            //
            mainBeamWeldCreatePlane = frontPlane;
            subBeamWeldCreatePlane = frontPlane;
            mainConnWeldCreatePlane = frontPlane;
            subConnWeldCreatePlane = frontPlane;

            //
            this.weldProperties = weldProperties;
        }

        /// <summary>
        /// 根据板的height设置焊接面
        /// </summary>
        public void SetBoundPlane(Plane mainFrontPlane, Plane subFrontPlane, Plane mainBackPlane, Plane subBackPlane)
        {
            if(mainFrontPlane != null)
            {
                if (mainFrontPlane.SignedDist(frontPlane.Origin) > 0)
                    mainBeamWeldCreatePlane = mainFrontPlane;
                else
                    mainBeamWeldCreatePlane = frontPlane;
                mainConnWeldCreatePlane = frontPlane;
            }
            else if(mainBackPlane != null)
            {
                if (mainBackPlane.SignedDist(backPlane.Origin) > 0)
                    mainBeamWeldCreatePlane = mainBackPlane;
                else
                    mainBeamWeldCreatePlane = backPlane;
                mainConnWeldCreatePlane = backPlane;
            }
            if (subFrontPlane != null)
            {
                if (subFrontPlane.SignedDist(frontPlane.Origin) > 0)
                    subBeamWeldCreatePlane = subFrontPlane;
                else
                    subBeamWeldCreatePlane = frontPlane;
                subConnWeldCreatePlane = frontPlane;
            }
            else if (subBackPlane != null)
            {
                if (subBackPlane.SignedDist(backPlane.Origin) > 0)
                    subBeamWeldCreatePlane = subBackPlane;
                else
                    subBeamWeldCreatePlane = backPlane;
                subConnWeldCreatePlane = backPlane;
            }

        }

        public void AddWelds(bool isWeldMainConn, bool isWeldSubConn, bool isWeldMainUnder, bool isWeldSubUnder, bool isMainWeld, bool isSubWeld)
        {
            if (isWeldMainConn)
            {
                if (isMainWeld)
                {
                    bool isCreate = false;
                    CreateMainBeamConnWeld(ref isCreate); //焊缝有效就创建梁和连接板
                    if(!isCreate) //如果创建失败，创建连接板和垫板
                        CreateMainUnderConnWeld(true);
                }
                else
                {
                    bool ignoreDist = true;
                    CreateMainBeamConnWeld(ref ignoreDist);
                }
            }
            if (isWeldSubConn)
            {
                if (isSubWeld)
                {
                    bool isCreate = false;
                    CreateSubBeamConnWeld(ref isCreate); //焊缝有效就创建梁和连接板
                    if (!isCreate) //如果创建失败，创建连接板和垫板
                        CreateSubUnderConnWeld(true);
                }
                else
                {
                    bool ignoreDist = true;
                    CreateSubBeamConnWeld(ref ignoreDist);
                }
            }
            if (isWeldMainUnder)
            {
                if (isMainWeld) //焊缝
                {
                    CreateMainBeamUnderWeld(false);
                    CreateMainUnderUnderWeld();
                }
                else //螺栓
                    CreateMainBeamUnderWeld(true);
            }
            if (isWeldSubUnder)
            {
                if (isSubWeld) //焊缝
                {
                    CreateSubBeamUnderWeld(false);
                    CreateSubUnderUnderWeld();
                }
                else //螺栓
                    CreateSubBeamUnderWeld(true);
            }
        }

        public void CreateWeld(PartBase partBase1, PartBase partBase2, Vec3[] v)
        {
            Weld weld = cmp.CreatePolyWeld(partBase1, partBase2, v, true);
            weldProperties.ApplyToWeld(weld);
            weld.Flush();
        }

        /// <summary>
        /// 创建衬板和连接板之间的焊缝
        /// </summary>
        /// <param name="ignoreDist">忽略距离过远造成无效焊缝</param>
        public void CreateMainUnderConnWeld(bool ignoreDist)
        {
            if (thickness == 0 || underBoardThickNess == 0 || underBoardThickNess == double.MaxValue)
                return;

            //主梁在后，在主梁垫板和连接板上创建焊缝
            if (distance > 0 || ignoreDist)
            {
                Vec3 v1 = connCtrlPlane.Intersect(mainConnWeldCreatePlane, tailPlane.Move(-length));
                Vec3 v2 = connCtrlPlane.Intersect(mainConnWeldCreatePlane, tailPlane);
                CreateWeld(mainParts[mainParts.Count - 1], mainParts[0], new Vec3[] { v1, v2 });
            }

            if (distance == 0)
            {
                Vec3 v1 = connCtrlPlane.Intersect(mainConnWeldCreatePlane, ubTail);
                Vec3 v2 = connCtrlPlane.Intersect(mainConnWeldCreatePlane, tailPlane);
                CreateWeld(mainParts[1], mainParts[0], new Vec3[] { v1, v2 });
            }
        }
        /// <summary>
        /// 创建衬板和连接板之间的焊缝
        /// </summary>
        /// <param name="ignoreDist">忽略距离过远造成无效焊缝</param>
        public void CreateSubUnderConnWeld(bool ignoreDist)
        { 
            if (thickness == 0 || underBoardThickNess == 0 || underBoardThickNess == double.MaxValue)
                return;

            //次梁在后
            if (distance < 0 || ignoreDist)
            {
                Vec3 v1 = connCtrlPlane.Intersect(subConnWeldCreatePlane, headPlane);
                Vec3 v2 = connCtrlPlane.Intersect(subConnWeldCreatePlane, headPlane.Move(-length));
                CreateWeld(subParts[subParts.Count - 1], subParts[0], new Vec3[] { v1, v2 });

            }

            if (distance == 0)
            {
                Vec3 v1 = connCtrlPlane.Intersect(subConnWeldCreatePlane, ubHead);
                Vec3 v2 = connCtrlPlane.Intersect(subConnWeldCreatePlane, headPlane);
                CreateWeld(subParts[1], subParts[0], new Vec3[] { v1, v2 });
            }
        }



        /// <summary>
        /// 创建衬板和衬板之间的焊缝
        /// </summary>
        public void CreateMainUnderUnderWeld()
        {
            if (underBoardThickNess == 0 || underBoardThickNess == double.MaxValue)
                return;
            if (mainParts.Count > 2)
            {
                Plane[] ubPlane = underPlanes.ToArray(); //这是焊缝的创建面
                for (int i = 1; i < ubPlane.Length; i++)
                {
                    Vec3 v1 = ubPlane[i].Intersect(mainConnWeldCreatePlane, ubTail);
                    Vec3 v2 = ubPlane[i].Intersect(mainConnWeldCreatePlane, ubHead);
                    CreateWeld(mainParts[i], mainParts[i + 1], new Vec3[] { v1, v2 });
                }
            }
        }
        /// <summary>
        /// 创建衬板和衬板之间的焊缝
        /// </summary>
        public void CreateSubUnderUnderWeld()
        {
            if (underBoardThickNess == 0 || underBoardThickNess == double.MaxValue)
                return;
            if (subParts.Count > 2)
            {
                Plane[] ubPlane = underPlanes.ToArray();
                for (int i = 1; i < ubPlane.Length; i++)
                {
                    Vec3 v1 = ubPlane[i].Intersect(subConnWeldCreatePlane, ubHead);
                    Vec3 v2 = ubPlane[i].Intersect(subConnWeldCreatePlane, ubTail);
                    CreateWeld(subParts[i], subParts[i + 1], new Vec3[] { v1,v2 });
                }
            }
        }



        /// <summary>
        /// 创建梁和衬板之间的焊缝
        /// </summary>
        /// <param name="ignoreDist">忽略距离过远造成无效焊缝</param>
        public void CreateMainBeamUnderWeld(bool ignoreDist)
        {
            if (underBoardThickNess == 0 || underBoardThickNess == double.MaxValue)
                return;
            //主梁在后，在主梁上创建垫板焊缝
            if (distance > 0)
            {
                int count = 1; //要创建焊缝的衬板的数量 
                if(ignoreDist)
                    count = underPlanes.Count;
                for (int i = 0; i < count; i++)
                {
                    if (!ignoreDist)
                    {
                        Vec3 v1 = underPlanes[i].Intersect(mainBeamWeldCreatePlane, tailPlane.Move(-length));
                        Vec3 v2 = underPlanes[i].Intersect(mainBeamWeldCreatePlane, tailPlane);
                        CreateWeld(mainPart, mainParts[i + 1], new Vec3[] { v1, v2 });
                    }
                    else
                    {
                        Plane createPlane = mainConnWeldCreatePlane;
                        if (i == 0)
                            createPlane = mainBeamWeldCreatePlane;
                        Vec3 v1 = underPlanes[i].Intersect(createPlane, tailPlane.Move(-length));
                        Vec3 v2 = underPlanes[i].Intersect(createPlane, tailPlane);
                        CreateWeld(mainPart, mainParts[i + 1], new Vec3[] { CenterPoint(v1, v2) });
                    }
                }
            }
            if (distance == 0)
            {
                Vec3 v1 = mainCtrlPlane.Intersect(mainBeamWeldCreatePlane, ubTail);
                Vec3 v2 = mainCtrlPlane.Intersect(mainBeamWeldCreatePlane, tailPlane);
                CreateWeld(mainPart, mainParts[0], new Vec3[] { v1, v2 });
            }
        }
        /// <summary>
        /// 创建梁和衬板之间的焊缝
        /// </summary>
        /// <param name="ignoreDist">忽略距离过远造成无效焊缝</param>
        public void CreateSubBeamUnderWeld(bool ignoreDist)
        {
            if (underBoardThickNess == 0 || underBoardThickNess == double.MaxValue)
                return;
            if (distance < 0)
            {
                int count = 1; //要创建焊缝的衬板的数量 
                if (ignoreDist)
                    count = underPlanes.Count;
                for (int i = 0; i < count; i++)
                {
                    if (!ignoreDist)
                    {
                        Vec3 v1 = underPlanes[i].Intersect(subBeamWeldCreatePlane, headPlane);
                        Vec3 v2 = underPlanes[i].Intersect(subBeamWeldCreatePlane, headPlane.Move(-length));
                        CreateWeld(subPart, subParts[i + 1], new Vec3[] { v1, v2 });
                    }
                    else
                    {
                        Plane createPlane = subConnWeldCreatePlane;
                        if (i == 0)
                            createPlane = subBeamWeldCreatePlane;
                        Vec3 v1 = underPlanes[i].Intersect(createPlane, headPlane);
                        Vec3 v2 = underPlanes[i].Intersect(createPlane, headPlane.Move(-length));
                        CreateWeld(subPart, subParts[i + 1], new Vec3[] { CenterPoint(v1, v2) });
                    }
                }
            }
            if (distance == 0)
            {
                Vec3 v1 = mainCtrlPlane.Intersect(subBeamWeldCreatePlane, ubHead);
                Vec3 v2 = mainCtrlPlane.Intersect(subBeamWeldCreatePlane, headPlane);
                CreateWeld(subPart, subParts[0], new Vec3[] { v1, v2 });
            }
        }



        /// <summary>
        /// 创建梁和连接板之间的焊缝，ignoreDist为false时返回是否创建成功
        /// </summary>
        /// <param name="ignoreDist">忽略距离过远造成无效焊缝</param>
        public void CreateMainBeamConnWeld(ref bool ignoreDist)
        {
            if (thickness == 0)
                return;
            if(distance < 0)
            {
                Vec3 v1 = connCtrlPlane.Intersect(mainBeamWeldCreatePlane, tailPlane.Move(-length));
                Vec3 v2 = connCtrlPlane.Intersect(mainBeamWeldCreatePlane, tailPlane);
                CreateWeld(mainPart, mainParts[0], new Vec3[] { v1, v2 });
                ignoreDist = true; //表示创建成功
            }
            else if (distance > 0 && ignoreDist)
            {
                Vec3 v1 = connCtrlPlane.Intersect(mainConnWeldCreatePlane, tailPlane.Move(-length));
                //Vec3 v2 = connCtrlPlane.Intersect(mainConnWeldCreatePlane, tailPlane);
                CreateWeld(mainPart, mainParts[0], new Vec3[] { v1 });
            }
            if(distance == 0 && (underBoardThickNess == 0 || underBoardThickNess == double.MaxValue))
            {
                Vec3 v1 = mainCtrlPlane.Intersect(mainBeamWeldCreatePlane, ubTail);
                Vec3 v2 = mainCtrlPlane.Intersect(mainBeamWeldCreatePlane, tailPlane);
                CreateWeld(mainPart, mainParts[0], new Vec3[] { v1, v2 });
                
            }

        }
        /// <summary>
        /// 创建梁和连接板之间的焊缝
        /// </summary>
        /// <param name="ignoreDist">忽略距离过远造成无效焊缝</param>
        public void CreateSubBeamConnWeld(ref bool ignoreDist)
        {
            if (thickness == 0)
                return;
            if (distance > 0 )
            {
                Vec3 v1 = connCtrlPlane.Intersect(subBeamWeldCreatePlane, headPlane);
                Vec3 v2 = connCtrlPlane.Intersect(subBeamWeldCreatePlane, headPlane.Move(-length));
                CreateWeld(subPart, subParts[0], new Vec3[] { v1, v2 });
                ignoreDist = true;
            }else if(distance < 0 && ignoreDist)
            {
                Vec3 v1 = connCtrlPlane.Intersect(subConnWeldCreatePlane, headPlane);
                //Vec3 v2 = connCtrlPlane.Intersect(subBeamWeldCreatePlane, headPlane.Move(-length));
                CreateWeld(subPart, subParts[0], new Vec3[] { v1 });
            }
            if (distance == 0 && (underBoardThickNess == 0 || underBoardThickNess == double.MaxValue))
            {
                Vec3 v1 = mainCtrlPlane.Intersect(subBeamWeldCreatePlane, ubHead);
                Vec3 v2 = mainCtrlPlane.Intersect(subBeamWeldCreatePlane, headPlane);
                CreateWeld(subPart, subParts[0], new Vec3[] { v1, v2 });
            }

        }

        public Vec3 CenterPoint(Vec3 v1, Vec3 v2)
        {
            Vec3 v = (v2 - v1).Normalize();
            return v1 + v * v1.Dist(v2) / 2;
        }
    }
}
