﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Plugins;

namespace SpliceConnection77
{
    public class CutHandler
    {

        private ComponentInstanceContext m_context = null; //公共参数、方法

        private ParamsHandler m_paramsHandler = null; //包含UI里的参数
        public CutHandler(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }
        public void Execute()
        {
            double gap = m_paramsHandler.GetDoubleValue(ParamNames.BEAM_GAP);
            int alignMode = m_paramsHandler.GetIntValue(ParamNames.BEAM_GAP_TYPE);
            double gap1 = 0, gap2 = 0;
            var comp = m_context.Comp;
            //设置对齐参数
            double originGap = m_context.MainNearCenterPlane.Dist(m_context.SubNearCenterPlane.Origin);
            if(originGap == 0)  //紧挨着的情况
            {
                if(alignMode == 1) //居中，默认
                {
                    gap1 = -gap / 2;
                    gap2 = -gap / 2;
                }
                if (alignMode == 2) //主梁向中间对齐
                {
                    gap1 = 0;
                    gap2 = -gap;
                }
                if (alignMode == 3) //次梁向中间对齐
                {
                    gap1 = -gap;
                    gap2 = 0;
                }
            }else  //梁中间有间距时，中心定为主梁边缘
            {
                if (alignMode == 1) //居中，默认
                {
                    gap1 = -gap / 2;
                    gap2 = originGap - gap / 2;
                }
                if (alignMode == 2) //主梁不动，次梁对齐
                {
                    gap1 = 0;
                    gap2 = originGap - gap;
                }
                if (alignMode == 3) //次梁不动，主梁对齐
                {
                    gap1 = originGap / 2 - gap;
                    gap2 = originGap / 2;
                }
            }
            //对齐主零件
            m_context.MainAlignPlane.MoveThis(-gap1 - m_context.MainHeadTailDistance);
            comp.CreateAlignPlane(m_context.MainPart, m_context.MainAlignPlane, true);

            
            //对齐次零件
            m_context.SubAlignPlane.MoveThis(-gap2 - m_context.SubHeadTailDistance);
            comp.CreateAlignPlane(m_context.SubPart, m_context.SubAlignPlane, true);
            comp.Flush();

            //切割之后重新计算各个面和参数
            m_context.MainNearCenterPlane = m_context.MainAlignPlane.Invert(); //重新设置近心面
            m_context.SubNearCenterPlane = m_context.SubAlignPlane.Invert();
            m_context.MainHeadTailDistance = m_context.MainNearCenterPlane.Dist(m_context.MainAwayCenterPlane.Origin);//计算长度
            m_context.SubHeadTailDistance = m_context.SubNearCenterPlane.Dist(m_context.SubAwayCenterPlane.Origin);

            m_context.Gap = gap;
        }

    }
}
