﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace SpliceConnection77
{
    class LiftingHoleHandler
    {
        private ComponentInstanceContext m_context = null; //公共参数、方法
        private ParamsHandler m_paramsHandler = null; //包含UI里的参数
        private double radius = 10.0;

        public LiftingHoleHandler(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            //0,1,2,3——不创建、不创建、次、主
            int placeMode = m_paramsHandler.GetIntValue(ParamNames.LIFTING_HOLE_PLACE);
            radius = m_paramsHandler.GetDoubleValue(ParamNames.LIFTING_HOLE_RADIUS);
            double vertMargin = m_paramsHandler.GetDoubleValue(ParamNames.LIFTING_HOLE_VERT_MARGIN);
            double horiMargin = m_paramsHandler.GetDoubleValue(ParamNames.LIFTING_HOLE_HORI_MARGIN);

            if (placeMode == 0 || placeMode == 1)
                return;

            Plane frontPlane = m_context.WebFrontPlateCreate.frontPlane;
            Plane backPlane = m_context.WebFrontPlateCreate.backPlane;
            double dist = frontPlane.SignedDist(backPlane.Origin);
            Plane connCenterPlane = frontPlane.Move(dist / 2);
            if (connCenterPlane.Normal.Dot(m_context.MainHCenter.Normal) < 0)
                connCenterPlane.InvertThis();

            if(placeMode == 3) {
                List<PartBase> subParts1 = new List<PartBase>();
                subParts1.AddRange(m_context.WebFrontPlateCreate.mainParts);
                subParts1.AddRange(m_context.WebBackPlateCreate.mainParts);
                Vec3 o1 = m_context.MainFrontWeb.Intersect(connCenterPlane.Move(vertMargin), m_context.MainNearCenterPlane.Move(-horiMargin));
                CreateBoltHole(m_context.MainFrontWeb.Invert(), o1, m_context.MainPart, subParts1.ToArray());
            }

            if(placeMode == 2)
            {
                List<PartBase> subParts2 = new List<PartBase>();
                subParts2.AddRange(m_context.WebFrontPlateCreate.subParts);
                subParts2.AddRange(m_context.WebBackPlateCreate.subParts);
                Vec3 o2 = m_context.SubFrontWeb.Intersect(connCenterPlane.Move(vertMargin), m_context.SubNearCenterPlane.Move(-horiMargin));
                CreateBoltHole(m_context.SubFrontWeb.Invert(), o2, m_context.SubPart, subParts2.ToArray());
            }
        }

        public void CreateBoltHole(Plane p, Vec3 o, PartBase mainPart, PartBase[] subParts)
        {
            Vec3 z = p.Normal;
            Vec3 x = z.Cross(Vec3.AXIS_X);
            if (x.Equals(Vec3.ZERO))
                x = z.Cross(Vec3.AXIS_Y);
            Vec3 y = x.Cross(z);

            Mat43 workAxis = new Mat43(x, y, z, o);
            Vec3 posStart = o;
            Vec3 posEnd = posStart + x;

            BoltGroup boltGroup;
            if(subParts == null || subParts.Length == 0)
                boltGroup = m_context.Comp.CreateBoltHoles(null, mainPart, posStart, posEnd, true);
            else
                boltGroup = m_context.Comp.CreateBoltHoles(mainPart, subParts, posStart, posEnd, true);
            boltGroup.WorkAxis = workAxis;
            boltGroup.ArrayStrideX = new double[] { }; //X方向间距
            boltGroup.ArrayStrideY = new double[] { };

            //基础属性
            boltGroup.BoltStandard = "TS10.9";
            boltGroup.BoltSize = radius;
            boltGroup.HoleTol = 0; //孔容差范围
            boltGroup.ThreadInMtrl = false; //是否包含螺纹 
            boltGroup.MadeType = Define.PartMadeType.PART_MADE_SITE; //螺栓制作类型(工厂/工地) 

            //螺栓孔设置
            boltGroup.HoleX = 0; //孔X方向扩大尺寸
            boltGroup.HoleY = 0;
            boltGroup.HoleType = Define.BoltHoleType.BOLT_HOLE_OVERSIZED; //孔类型
            boltGroup.RotateSlot = Define.BoltRotateSlot.BOLT_SLOT_PARALLEL; //孔开槽旋转方式 

            //螺栓垫片(螺母)设置
            //boltGroup.SectionFlags = 196742; //螺栓部件开关标志位

            //其它
            boltGroup.CutLength = 200; //切割长度
            boltGroup.ExtLength = 0; // 附加长度 
            boltGroup.RotationSide = Define.ObjectSide.SIDE_TOP; //螺栓朝向
            boltGroup.Shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ARRAY; //螺栓形状(排列方式) 

            //解决旋转后位置错误的问题
            boltGroup.Rotation = 0.0;
            boltGroup.HeadOffsetX = 0.0;
            boltGroup.HeadOffsetY = 0.0;
            boltGroup.HeadOffsetZ = 0.0;
            boltGroup.TailOffsetX = 0.0;
            boltGroup.TailOffsetY = 0.0;
            boltGroup.TailOffsetZ = 0.0;
            boltGroup.VerticalOffset = 0.0;
            boltGroup.DepthOffset = 0.0;

            boltGroup.Refresh();
        }

    }
}
