﻿using System;
using System.Linq;
using System.Text;
using System.Collections.Generic;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace SpliceConnection77
{
    class PlateHandler
    {
        private ComponentInstanceContext m_context = null; //公共参数、方法
        private ParamsHandler m_paramsHandler = null; //包含UI里的参数

        private PlateProperties m_topFlangeOutSide = new PlateProperties();
        private PlateProperties m_topFlangeInSide = new PlateProperties();
        private PlateProperties m_webProp = new PlateProperties();
        private PlateProperties m_bottomFlangeOutSide = new PlateProperties();
        private PlateProperties m_bottomFlangeInSide = new PlateProperties();

        private PlateProperties m_topFlangeOutSideGap = new PlateProperties();
        private PlateProperties m_topFlangeInSideGap = new PlateProperties();
        private PlateProperties m_webGap = new PlateProperties();
        private PlateProperties m_bottomFlangeOutSideGap = new PlateProperties();
        private PlateProperties m_bottomFlangeInSideGap = new PlateProperties();

        public PlateHandler(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            ParsePlateProps();
            UpdatePlateProps();

            //创建连接板和垫板
            double frontWebDistance = m_context.MainFront.Dist(m_context.MainFrontWeb.Origin);
            //腹板开关
            int webPlatePlaceMode = m_paramsHandler.GetIntValue(ParamNames.WEB_PLATE_PLACE);

            // CreateTopToplates
            {
                CommonPlateCreate plateCreate = m_context.TopTopPlateCreate;

                Plane front = m_context.MainWebCenter.Move(m_context.TopFlangeHeight / 2);
                Plane back = m_context.MainWebCenter.Move(-m_context.TopFlangeHeight / 2).Invert();
                Plane head = m_context.MainNearCenterPlane.Move(m_context.Gap / 2 + m_context.TopFlangeWidth / 2);//远离主零件
                Plane tail = m_context.SubNearCenterPlane.Move(m_context.Gap / 2 + m_context.TopFlangeWidth / 2);

                plateCreate.SetCtrlPlane(m_context.MainTopFlange, m_context.SubTopFlange);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetPlateProperties(m_topFlangeOutSide, m_topFlangeOutSideGap);
                plateCreate.SetDistance(m_context.gap);
                plateCreate.Execute();

            }
            // CreateTopFrontPlate
            {
                CommonPlateCreate plateCreate = m_context.TopFrontPlateCreate;

                Plane front = m_context.MainFront.Move(-frontWebDistance / 2 + m_topFlangeInSide.Height / 2);
                Plane back = front.Move(-m_topFlangeInSide.Height).Invert();
                Plane head = m_context.MainNearCenterPlane.Move(m_context.Gap / 2 + m_topFlangeInSide.Width / 2);//远离主零件
                Plane tail = m_context.SubNearCenterPlane.Move(m_context.Gap / 2 + m_topFlangeInSide.Width / 2);

                //连接板的重新定位（相对于外侧的位置）
                if (m_context.flangeInsideLocation != double.MaxValue)
                {
                    //m_topFlangeInside1Front = m_context.MainWebCenter.Move(m_context.TopFlangeHeight / 2 - m_context.flangeInsideLocation - m_context.web_bolt_width_edge_gap);
                    front = m_context.MainFront.Move(-m_context.flangeInsideLocation);
                    back = front.Move(-m_topFlangeInSide.Height).Invert();
                }

                plateCreate.SetCtrlPlane(m_context.MainTopFlangeInside, m_context.SubTopFlangeInside);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetPlateProperties(m_topFlangeInSide, m_topFlangeInSideGap);
                plateCreate.SetDistance(m_context.gap);
                plateCreate.Execute();
            }
            // CreateTopBackPlate
            {
                CommonPlateCreate plateCreate = m_context.TopBackPlateCreate;

                Plane front = Util.CreatePlaneBySymmetry(m_context.TopFrontPlateCreate.backPlane, m_context.MainWebCenter);
                Plane back = Util.CreatePlaneBySymmetry(m_context.TopFrontPlateCreate.frontPlane, m_context.MainWebCenter);
                Plane head = m_context.MainNearCenterPlane.Move(m_context.Gap / 2 + m_topFlangeInSide.Width / 2);//远离主零件
                Plane tail = m_context.SubNearCenterPlane.Move(m_context.Gap / 2 + m_topFlangeInSide.Width / 2);

                plateCreate.SetCtrlPlane(m_context.MainTopFlangeInside, m_context.SubTopFlangeInside);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetPlateProperties(m_topFlangeInSide, m_topFlangeInSideGap);
                plateCreate.SetDistance(m_context.gap);
                plateCreate.Execute();
            }
            // CreateWebFrontPlate
            {
                CommonPlateCreate plateCreate = m_context.WebFrontPlateCreate;

                Plane front = m_context.MainHCenter.Move(m_context.WebHeight / 2);
                Plane back = m_context.MainHCenter.Move(-m_context.WebHeight / 2).Invert();
                Plane head = m_context.MainNearCenterPlane.Move(m_context.Gap / 2 + m_context.WebWidth / 2);//远离主零件
                Plane tail = m_context.SubNearCenterPlane.Move(m_context.Gap / 2 + m_context.WebWidth / 2);

                if (m_context.webLocation != double.MaxValue)
                {
                    front = m_context.MainTopFlange.Move(-m_context.webLocation);
                    back = front.Move(-m_context.WebHeight).Invert();
                }
                plateCreate.SetCtrlPlane(m_context.MainFrontWeb, m_context.SubFrontWeb);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetPlateProperties(m_webProp, m_webGap);
                plateCreate.SetDistance(m_context.gap);
                if (webPlatePlaceMode == 2)
                    plateCreate.thickness = 0;
                plateCreate.Execute();
            }
            // CreateWebBackPlate
            {
                CommonPlateCreate plateCreate = m_context.WebBackPlateCreate;

                Plane front = m_context.WebFrontPlateCreate.frontPlane;
                Plane back = m_context.WebFrontPlateCreate.backPlane;
                Plane head = m_context.WebFrontPlateCreate.headPlane;
                Plane tail = m_context.WebFrontPlateCreate.tailPlane;

                plateCreate.SetCtrlPlane(m_context.MainBackWeb, m_context.SubBackWeb);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetPlateProperties(m_webProp, m_webGap);
                plateCreate.SetDistance(m_context.gap);
                if (webPlatePlaceMode == 3)
                    plateCreate.thickness = 0;
                plateCreate.Execute();
            }
            // CreateBottomBottomPlates
            {
                CommonPlateCreate plateCreate = m_context.BottomBottomPlateCreate;

                Plane front = m_context.MainWebCenter.Move(m_context.BottomFlangeHeight / 2);
                Plane back = m_context.MainWebCenter.Move(-m_context.BottomFlangeHeight / 2).Invert();
                Plane head = m_context.MainNearCenterPlane.Move(m_context.Gap / 2 + m_context.BottomFlangeWidth / 2);//远离主零件
                Plane tail = m_context.SubNearCenterPlane.Move(m_context.Gap / 2 + m_context.BottomFlangeWidth / 2);

                plateCreate.SetCtrlPlane(m_context.MainBottomFlange, m_context.SubBottomFlange);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetPlateProperties(m_bottomFlangeOutSide, m_bottomFlangeOutSideGap);
                plateCreate.SetDistance(m_context.gap);
                plateCreate.Execute();
            }
            // CreateBottomFrontPlate
            {
                CommonPlateCreate plateCreate = m_context.BottomFrontPlateCreate;

                Plane front = m_context.MainFront.Move(-frontWebDistance / 2 + m_bottomFlangeInSide.Height / 2);
                Plane back = front.Move(-m_bottomFlangeInSide.Height).Invert();
                Plane head = m_context.MainNearCenterPlane.Move(m_context.Gap / 2 + m_bottomFlangeInSide.Width / 2);//远离主零件
                Plane tail = m_context.SubNearCenterPlane.Move(m_context.Gap / 2 + m_bottomFlangeInSide.Width / 2);

                if (m_context.flangeInsideLocation != double.MaxValue)
                {
                    front = m_context.MainFront.Move(-m_context.flangeInsideLocation);
                    back = front.Move(-m_bottomFlangeInSide.Height).Invert();
                }

                plateCreate.SetCtrlPlane(m_context.MainBottomFlangeInside, m_context.SubBottomFlangeInside);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetPlateProperties(m_bottomFlangeInSide, m_bottomFlangeInSideGap);
                plateCreate.SetDistance(m_context.gap);
                plateCreate.Execute();
            }
            // CreateBottomBackPlate
            {
                CommonPlateCreate plateCreate = m_context.BottomBackPlateCreate;

                Plane front = Util.CreatePlaneBySymmetry(m_context.BottomFrontPlateCreate.backPlane, m_context.MainWebCenter);
                Plane back = Util.CreatePlaneBySymmetry(m_context.BottomFrontPlateCreate.frontPlane, m_context.MainWebCenter);
                Plane head = m_context.MainNearCenterPlane.Move(m_context.Gap / 2 + m_bottomFlangeInSide.Width / 2);//远离主零件
                Plane tail = m_context.SubNearCenterPlane.Move(m_context.Gap / 2 + m_bottomFlangeInSide.Width / 2);

                plateCreate.SetCtrlPlane(m_context.MainBottomFlangeInside, m_context.SubBottomFlangeInside);
                plateCreate.SetSidePlane(front, back, head, tail);
                plateCreate.SetPlateProperties(m_bottomFlangeInSide, m_bottomFlangeInSideGap);
                plateCreate.SetDistance(m_context.gap);
                plateCreate.Execute();
            }

        }

        private void ParsePlateProps()
        {
            //顶端外翼缘衬板
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.TOP_FLANGE_OUTSIDE_PLATE_GAP);
                m_topFlangeOutSideGap.SetProps(plateProps);
            }
            //顶端内翼缘衬板
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.TOP_FLANGE_INSIDE_PLATE_GAP);
                m_topFlangeInSideGap.SetProps(plateProps);
            }
            //底端外翼缘衬板
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.BOTTOM_FLANGE_OUTSIDE_PLATE_GAP);
                m_bottomFlangeOutSideGap.SetProps(plateProps);
            }
            //底端内翼缘衬板
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.BOTTOM_FLANGE_INSIDE_PLATE_GAP);
                m_bottomFlangeInSideGap.SetProps(plateProps);
            }
            //腹板衬板
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.WEB_PLATE_GAP);
                m_webGap.SetProps(plateProps);
            }
            //顶部连接板（外侧）x1
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.TOP_FLANGE_OUTSIDE_PLATE_PROPS);
                m_topFlangeOutSide.SetProps(plateProps);
                SwapWidthHeight(m_topFlangeOutSide);
                if (m_topFlangeOutSide.Thickness == double.MaxValue)
                    m_topFlangeOutSide.Thickness = 10.0;
                m_context.TopFlangeWidth = m_topFlangeOutSide.Width;
                m_context.TopFlangeHeight = m_topFlangeOutSide.Height;
            }
            //顶部连接板（内侧）x2
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.TOP_FLANGE_INSIDE_PLATE_PROPS);
                m_topFlangeInSide.SetProps(plateProps);
                SwapWidthHeight(m_topFlangeInSide);
                if (m_topFlangeInSide.Thickness == double.MaxValue)
                    m_topFlangeInSide.Thickness = 10.0;
                if (m_topFlangeInSide.Height == double.MaxValue)
                    m_topFlangeInSide.Height = 70.0;
                m_context.TopFlangeInsideWidth = m_topFlangeInSide.Width;
            }
            //底部连接板（外侧）x1
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.BOTTOM_FLANGE_OUTSIDE_PLATE_PROPS);
                m_bottomFlangeOutSide.SetProps(plateProps);
                SwapWidthHeight(m_bottomFlangeOutSide);
                if (m_bottomFlangeOutSide.Thickness == double.MaxValue)
                    m_bottomFlangeOutSide.Thickness = 10.0;
                m_context.BottomFlangeWidth = m_bottomFlangeOutSide.Width;
                m_context.BottomFlangeHeight = m_bottomFlangeOutSide.Height;
            }
            //底部连接板（内侧）x2
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.BOTTOM_FLANGE_INSIDE_PLATE_PROPS);
                m_bottomFlangeInSide.SetProps(plateProps);
                SwapWidthHeight(m_bottomFlangeInSide);
                if (m_bottomFlangeInSide.Thickness == double.MaxValue)
                    m_bottomFlangeInSide.Thickness = 10.0;
                if (m_bottomFlangeInSide.Height == double.MaxValue)
                    m_bottomFlangeInSide.Height = 70.0;
                m_context.BottomFlangeInsideWidth = m_bottomFlangeInSide.Width;
            }
            //腹板的连接板x2
            {
                DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.WEB_PLATE_PROPS);
                m_webProp.SetProps(plateProps);
                SwapWidthHeight(m_webProp);
                if (m_webProp.Thickness == double.MaxValue)
                    m_webProp.Thickness = 10.0;
                m_context.WebWidth = m_webProp.Width;
                m_context.WebHeight = m_webProp.Height;
            }
        }

        private void SwapWidthHeight(PlateProperties plateProperties)
        {
            //解决宽度和高度和实际不一致的问题
            double width = plateProperties.Width;
            double height = plateProperties.Height;
            plateProperties.Width = height;
            plateProperties.Height = width;
        }

        public void UpdatePlateProps()
        {
            /*
             * 初始化螺栓参数
             */
            m_context.TopFlangeBoltParameter.InitialBasic(m_context, m_paramsHandler);
            m_context.BottomFlangeBoltParameter.InitialBasic(m_context, m_paramsHandler);
            m_context.WebBoltParameter.InitialBasic(m_context, m_paramsHandler);


            /*
             * 计算连接板的长宽(由螺栓得出)
             */
            //上翼缘
            m_context.TopFlangeBoltParameter.InitalLocation(m_context.TopBoltGroup);
            if (m_topFlangeInSide.Width == double.MaxValue)
                m_topFlangeInSide.Width = m_context.TopFlangeInsideWidth;
            //腹板
            m_context.WebBoltParameter.InitalLocation(m_context.WebBoltGroup);
            //下翼缘
            m_context.BottomFlangeBoltParameter.InitalLocation(m_context.BottomBoltGroup);
            if (m_bottomFlangeInSide.Width == double.MaxValue)
                m_bottomFlangeInSide.Width = m_context.BottomFlangeInsideWidth;

        }
    }
}
