﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace SpliceConnection77
{
    //参数选项卡
    class PlateWeldHandler
    {
        private ComponentInstanceContext m_context = null; //公共参数、方法
        private ParamsHandler m_paramsHandler = null; //包含UI里的参数
        private bool mainTop = true;
        private bool mainBottom = true;
        private bool subTop = true;
        private bool subBottom = true;
        private bool mainWeb = true;
        private bool subWeb = true;

        public PlateWeldHandler(ComponentInstanceContext compContext, ParamsHandler paramsHandler)
        {
            m_context = compContext;
            m_paramsHandler = paramsHandler;
        }

        public void ReadComponentParameter()
        {

            //螺栓还是焊接
            mainTop = m_paramsHandler.GetIntValue(ParamNames.BOLT_WELD_SELECT_MAIN_TOP) == 2; //是否开启焊缝
            mainBottom = m_paramsHandler.GetIntValue(ParamNames.BOLT_WELD_SELECT_MAIN_BOTTOM) == 2;
            subTop = m_paramsHandler.GetIntValue(ParamNames.BOLT_WELD_SELECT_SUB_TOP) == 2;
            subBottom = m_paramsHandler.GetIntValue(ParamNames.BOLT_WELD_SELECT_SUB_BOTTOM) == 2;
            mainWeb = m_paramsHandler.GetIntValue(ParamNames.BOLT_WELD_SELECT_WEB_MAIN) == 2;
            subWeb = m_paramsHandler.GetIntValue(ParamNames.BOLT_WELD_SELECT_WEB_SUB) == 2;

            if (mainTop)
                m_context.BoltMode &= 0b0101111111;
            if (mainBottom)
                m_context.BoltMode &= 0b1111110101;
            if (subTop)
                m_context.BoltMode &= 0b1010111111;
            if (subBottom)
                m_context.BoltMode &= 0b1111111010;
            if (mainWeb)
                m_context.BoltMode &= 0b1111011111;
            if (subWeb)
                m_context.BoltMode &= 0b1111101111;

            //车间螺栓设置
            int factoryBolt = m_paramsHandler.GetIntValue(ParamNames.FACTORY_BOLT);
            switch (factoryBolt)
            {
                case 1:
                    m_context.MainBoltFactoryMode = true; //主梁螺栓
                    break;
                case 2:
                    m_context.SubBoltFactoryMode = true; //次梁螺栓
                    break;
                default:
                    break;
            }
        }

        //添加焊接
        public void Execute()
        {
            //连接板焊接——0,1,2,3代表无、主零件、次零件、两者
            int plateWeldMode = m_paramsHandler.GetIntValue(ParamNames.CONNECT_PLATE_COMPONENT);
            bool isWeldMainConn = (plateWeldMode == 1 || plateWeldMode == 3);
            bool isWeldSubConn = (plateWeldMode == 2 || plateWeldMode == 3);
            //垫板焊接
            int underMode = m_paramsHandler.GetIntValue(ParamNames.UNDER_BOARD_WELD);
            bool isWeldMainUnder = (underMode == 1 || underMode == 3);
            bool isWeldSubUnder = (underMode == 2 || underMode == 3);
            //焊接属性
            List<WeldProperties> weldProperties = new List<WeldProperties>();
            for(int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; i++)
            {
                DObject dweldProps = m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                dweldProps = WeldPropertyPanel.ParseNativeProps(dweldProps);
                WeldProperties weldProperty = new WeldProperties();
                weldProperty.SetProps(dweldProps);
                if(weldProperty.SizeUp == double.MaxValue)
                    weldProperty.SizeUp = 6;
                if (weldProperty.SizeDown == double.MaxValue)
                    weldProperty.SizeDown = 6;
                weldProperties.Add(weldProperty);
            }
            //上翼缘焊接
            {
                CommonPlateWeld plateWeld = new CommonPlateWeld(m_context.TopTopPlateCreate, weldProperties[0]);
                plateWeld.SetBoundPlane(m_context.MainFront, m_context.SubFront, null, null);
                plateWeld.AddWelds(isWeldMainConn, isWeldSubConn, isWeldMainUnder, isWeldSubUnder, mainTop, subTop);
            }
            {
                CommonPlateWeld plateWeld = new CommonPlateWeld(m_context.TopFrontPlateCreate, weldProperties[0]);
                plateWeld.SetBoundPlane(m_context.MainFront, m_context.SubFront, null, null);
                plateWeld.AddWelds(isWeldMainConn, isWeldSubConn, isWeldMainUnder, isWeldSubUnder, mainTop, subTop);
            }
            {
                CommonPlateWeld plateWeld = new CommonPlateWeld(m_context.TopBackPlateCreate, weldProperties[0]);
                plateWeld.SetBoundPlane(null, null, m_context.MainBack, m_context.SubBack);
                plateWeld.AddWelds(isWeldMainConn, isWeldSubConn, isWeldMainUnder, isWeldSubUnder, mainTop, subTop);
            }
            //腹板焊接
            {
                CommonPlateWeld plateWeld = new CommonPlateWeld(m_context.WebFrontPlateCreate, weldProperties[2]);
                //plateWeld.SetWeldFront(m_context.MainFront, m_context.SubFront);
                plateWeld.AddWelds(isWeldMainConn, isWeldSubConn, isWeldMainUnder, isWeldSubUnder, mainWeb, subWeb);
            }
            {
                CommonPlateWeld plateWeld = new CommonPlateWeld(m_context.WebBackPlateCreate, weldProperties[2]);
                //plateWeld.SetWeldFront(m_context.MainFront, m_context.SubFront);
                plateWeld.AddWelds(isWeldMainConn, isWeldSubConn, isWeldMainUnder, isWeldSubUnder, mainWeb, subWeb);
            }
            //下翼缘焊接
            {
                CommonPlateWeld plateWeld = new CommonPlateWeld(m_context.BottomBottomPlateCreate, weldProperties[3]);
                plateWeld.SetBoundPlane(m_context.MainFront, m_context.SubFront, null, null);
                plateWeld.AddWelds(isWeldMainConn, isWeldSubConn, isWeldMainUnder, isWeldSubUnder, mainBottom, subBottom);
            }
            {
                CommonPlateWeld plateWeld = new CommonPlateWeld(m_context.BottomFrontPlateCreate, weldProperties[3]);
                plateWeld.SetBoundPlane(m_context.MainFront, m_context.SubFront, null, null);
                plateWeld.AddWelds(isWeldMainConn, isWeldSubConn, isWeldMainUnder, isWeldSubUnder, mainBottom, subBottom);
            }
            {
                CommonPlateWeld plateWeld = new CommonPlateWeld(m_context.BottomBackPlateCreate, weldProperties[3]);
                plateWeld.SetBoundPlane(null, null, m_context.MainBack, m_context.SubBack);
                plateWeld.AddWelds(isWeldMainConn, isWeldSubConn, isWeldMainUnder, isWeldSubUnder, mainBottom, subBottom);
            }
        }


    }
}
