﻿using LoongCAD.STC;
using LoongCAD.Util;
using LoongCAD.UI.Component;
using LoongCAD.STC.Object3D;

namespace StiffenedShearPlate17
{
    public class BoltsConfig
    {
        /// <summary>
        /// 螺栓的基本属性值
        /// </summary>
        private class BoltsBasicProps
        {
            /// <summary>
            /// 螺栓标准 
            /// </summary>
            private string BoltStandard;
            /// <summary>
            /// 螺栓尺寸
            /// </summary>
            private double BoltSize;
            /// <summary>
            /// 螺栓孔容差
            /// </summary>
            private double HoleTol;
            /// <summary>
            /// 孔是否包含螺纹
            /// </summary>
            private bool ThreadInMaterial;
            /// <summary>
            /// 第一个工厂/工地属性
            /// </summary>
            private Define.PartMadeType MadeType0;
            /// <summary>
            /// 第二个工厂/工地属性
            /// </summary>
            private Define.PartMadeType MadeType1;
            /// <summary>
            /// 处理螺栓的基本属性值
            /// </summary>
            /// <param name="basicProps"></param>
            public void CopyFromBasicProps(DObject basicProps)
            {
                if (basicProps == null)
                    return;
                BoltStandard = basicProps.GetString(BoltBasicPropertyPanel.ParamNames.BoltStandard);
                BoltSize = basicProps.GetDouble(BoltBasicPropertyPanel.ParamNames.BoltSize);
                HoleTol = basicProps.GetDouble(BoltBasicPropertyPanel.ParamNames.HoleTol);
                ThreadInMaterial = basicProps.GetInt(BoltBasicPropertyPanel.ParamNames.ThreadInMtrl) == 2;
                MadeType0 = ParseMadeType(basicProps.GetInt(BoltBasicPropertyPanel.ParamNames.MadeType0));
                MadeType1 = ParseMadeType(basicProps.GetInt(BoltBasicPropertyPanel.ParamNames.MadeType1));
                if (!Equals(HoleTol, double.MaxValue)) return;
                if (BoltSize >= 27.0d)
                    HoleTol = 3.0d;
                else if (BoltSize >= 16.0d)
                    HoleTol = 2.0d;
                else
                    HoleTol = 1.5;
                //处理螺栓构建地点：工地、工厂
                Define.PartMadeType ParseMadeType(int comboIndex)
                {
                    switch (comboIndex)
                    {
                        case 2:
                            return Define.PartMadeType.PART_MADE_WORKSHOP;
                        default:
                            return Define.PartMadeType.PART_MADE_SITE;
                    }
                }
            }
            /// <summary>
            /// 填充给螺栓组
            /// </summary>
            /// <param name="boltGroup"></param>
            /// <param name="fillMadeType0">为真则用MadeType0进行填充</param>
            public void Fill(BoltGroup boltGroup, bool fillMadeType0 = true)
            {
                if (boltGroup == null)
                    return;
                boltGroup.BoltStandard = BoltStandard;
                boltGroup.BoltSize = BoltSize;
                boltGroup.HoleTol = HoleTol;
                boltGroup.ThreadInMtrl = ThreadInMaterial;
                boltGroup.MadeType = fillMadeType0 ? MadeType0 : MadeType1;
            }
        }
        /// <summary>
        /// 螺栓的孔相关配置
        /// </summary>
        private class BoltsHoleProps
        {
            private enum SlotInMode
            {
                None = 0, Beam = 1, Plate = 2, Both = 3,
            }
            private double HoleX;
            private double HoleY;
            private Define.BoltHoleType HoleType;
            private Define.BoltRotateSlot RotSlot;
            //NOTE 处理SlotIn长洞配置项,由于参数分离，在拿到的数据上操作
            private SlotInMode SlotIn;
            /// <summary>
            /// 解析孔类型相关数据
            /// </summary>
            /// <param name="holeProps"></param>
            public void CopyFromHoleProps(DObject holeProps)
            {
                if (holeProps == null)
                    return;
                HoleX = holeProps.GetDouble(BoltHolePanel.ParamNames.HoleX);
                HoleY = holeProps.GetDouble(BoltHolePanel.ParamNames.HoleY);
                var comboIndex = holeProps.GetInt(BoltHolePanel.ParamNames.HoleType);
                switch (comboIndex)
                {
                    case 1:
                        HoleType = Define.BoltHoleType.BOLT_HOLE_SLOTTED;
                        break;
                    default:
                        HoleType = Define.BoltHoleType.BOLT_HOLE_OVERSIZED;
                        break;
                }
                comboIndex = holeProps.GetInt(BoltHolePanel.ParamNames.RotSlot);
                switch (comboIndex)
                {
                    case 1:
                        RotSlot = Define.BoltRotateSlot.BOLT_SLOT_ODD;
                        break;
                    case 2:
                        RotSlot = Define.BoltRotateSlot.BOLT_SLOT_EVEN;
                        break;
                    case 0:
                    default:
                        RotSlot = Define.BoltRotateSlot.BOLT_SLOT_PARALLEL;
                        break;
                }
                comboIndex = holeProps.GetInt(BoltHolePanel.ParamNames.SlotIn);
                switch (comboIndex)
                {
                    case 0:
                        SlotIn = SlotInMode.None;
                        break;
                    case 1:
                        SlotIn = SlotInMode.Beam;
                        break;
                    case 2:
                        SlotIn = SlotInMode.Plate;
                        break;
                    default:
                        SlotIn = SlotInMode.Both;
                        break;
                }
            }
            public void Fill(BoltGroup boltGroup)
            {
                if (boltGroup == null)
                    return;
                boltGroup.HoleX = HoleX;
                boltGroup.HoleY = HoleY;
                boltGroup.HoleType = HoleType;
                boltGroup.RotateSlot = RotSlot;
                //boltGroup
                var sectFlags = boltGroup.SectionFlags;
                switch (SlotIn)
                {
                    case SlotInMode.Beam:
                        sectFlags |= Define.BOLT_SECT_OVERSIZED_HOLE0;
                        break;
                    case SlotInMode.Plate:
                        {
                            sectFlags |= Define.BOLT_SECT_OVERSIZED_HOLE1;
                            sectFlags |= Define.BOLT_SECT_OVERSIZED_HOLE2;
                        }
                        break;
                    case SlotInMode.Both:
                        {
                            sectFlags |= Define.BOLT_SECT_OVERSIZED_HOLE0;
                            sectFlags |= Define.BOLT_SECT_OVERSIZED_HOLE1;
                            sectFlags |= Define.BOLT_SECT_OVERSIZED_HOLE2;
                        }
                        break;
                    case SlotInMode.None:
                    default:
                        break;
                }
                boltGroup.SectionFlags = sectFlags;
            }
        }
        /// <summary>
        /// 螺栓组件的启用配置
        /// </summary>
        private class BoltsContentProps
        {
            private bool PartWasherUp;
            private bool PartWasherDown0;
            private bool PartWasherDown1;
            private bool PartNut0;
            private bool PartNut1;
            public void CopyFromContentProps(DObject contentProps)
            {
                if (contentProps == null)
                    return;
                PartWasherUp = contentProps.GetBool(BoltContentPanel.ParamNames.PartWasherUp);
                PartWasherDown0 = contentProps.GetBool(BoltContentPanel.ParamNames.PartWasherDown0);
                PartWasherDown1 = contentProps.GetBool(BoltContentPanel.ParamNames.PartWasherDown1);
                PartNut0 = contentProps.GetBool(BoltContentPanel.ParamNames.PartNut0);
                PartNut1 = contentProps.GetBool(BoltContentPanel.ParamNames.PartNut1);
            }
            public void Fill(BoltGroup boltGroup)
            {
                if (boltGroup == null)
                    return;
                var sectFlags = boltGroup.SectionFlags;
                if (PartWasherUp)
                    sectFlags |= Define.BOLT_SECT_WASHER_UP;
                if (PartWasherDown0)
                    sectFlags |= Define.BOLT_SECT_WASHER_DOWN0;
                if (PartWasherDown1)
                    sectFlags |= Define.BOLT_SECT_WASHER_DOWN1;
                if (PartNut0)
                    sectFlags |= Define.BOLT_SECT_NUT0;
                if (PartNut1)
                    sectFlags |= Define.BOLT_SECT_NUT1;
                boltGroup.SectionFlags = sectFlags;
                boltGroup.WasherUpEnable = PartWasherUp;
                boltGroup.WasherDownEnable0 = PartWasherDown0;
                boltGroup.WasherDownEnable1 = PartWasherDown1;
                boltGroup.NutEnable0 = PartNut0;
                boltGroup.NutEnable1 = PartNut1;
            }
        }
        /// <summary>
        /// 螺栓的基本属性值
        /// </summary>
        private BoltsBasicProps m_basicProps = new BoltsBasicProps();
        /// <summary>
        /// 螺栓的孔相关配置
        /// </summary>
        private BoltsHoleProps m_holeProps = new BoltsHoleProps();
        /// <summary>
        /// 螺栓组件的启用配置
        /// </summary>
        private BoltsContentProps m_contentProps = new BoltsContentProps();
        /// <summary>
        /// 螺栓组件的附加长度
        /// </summary>
        private double m_extendLength = 0.0;
        /// <summary>
        /// 螺栓组的形状
        /// </summary>
        private Define.BoltGroupShape m_shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ARRAY;

        /// <summary>
        /// 处理螺栓的基本属性值
        /// </summary>
        /// <param name="basicProps"></param>
        public void CopyFromBasicProps(DObject basicProps)
        {
            m_basicProps?.CopyFromBasicProps(basicProps);
        }
        /// <summary>
        /// 解析孔类型、槽孔相关配置
        /// </summary>
        /// <param name="holeProps"></param>
        public void CopyFromHoleProps(DObject holeProps)
        {
            m_holeProps.CopyFromHoleProps(holeProps);
        }
        /// <summary>
        /// 解析螺栓组件的装配相关配置
        /// </summary>
        /// <param name="contentProps"></param>
        public void CopyFromContentProps(DObject contentProps)
        {
            m_contentProps.CopyFromContentProps(contentProps);
        }
        /// <summary>
        /// 解析螺栓的附加长度配置
        /// </summary>
        /// <param name="extendProps"></param>
        public void CopyFromExtendProps(DObject extendProps)
        {
            if (extendProps == null)
                return;
            m_extendLength = extendProps.GetDouble(BoltExtendPanel.ParamNames.ExtLength);
        }
        /// <summary>
        /// 解析螺栓的排列方式配置
        /// </summary>
        /// <param name="shapeProps"></param>
        public void CopyFromShapeProps(int shapeProps)
        {
            switch (shapeProps)
            {
                case 2:
                    m_shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_X_ODD;
                    break;
                case 3:
                    m_shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_X_EVEN;
                    break;
                case 4:
                    m_shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_Y_ODD;
                    break;
                case 5:
                    m_shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ALTER_Y_EVEN;
                    break;
                default:
                    m_shape = Define.BoltGroupShape.BOLT_GROUPSHAPE_ARRAY;
                    break;
            }
        }
        /// <summary>
        /// 填充螺栓组的螺栓基本属性
        /// </summary>
        /// <param name="boltGroup"></param>
        /// <param name="fillMadeType0"></param>
        public void FillBasicProps(BoltGroup boltGroup, bool fillMadeType0 = true)
        {
            m_basicProps?.Fill(boltGroup, fillMadeType0);
        }
        /// <summary>
        /// 填充螺栓组的螺栓形状
        /// </summary>
        /// <param name="boltGroup"></param>
        /// <param name="shapeIndex"></param>
        public void FillShape(BoltGroup boltGroup)
        {
            boltGroup.Shape = m_shape;
        }
        /// <summary>
        /// 填充螺栓组的孔配置
        /// </summary>
        /// <param name="boltGroup"></param>
        public void FillHoleProps(BoltGroup boltGroup)
        {
            m_holeProps?.Fill(boltGroup);
        }
        /// <summary>
        /// 填充螺栓组的启用配置
        /// </summary>
        /// <param name="boltGroup"></param>
        public void FillContentProps(BoltGroup boltGroup)
        {
            m_contentProps?.Fill(boltGroup);
        }
        /// <summary>
        /// 填充螺栓组的附加长度
        /// </summary>
        /// <param name="boltGroup"></param>
        public void FillExtendProps(BoltGroup boltGroup)
        {
            if (boltGroup != null)
            {
                boltGroup.ExtLength = m_extendLength;
            }
        }
    }
}
