﻿using System;
using LoongCAD.Util;
using LoongCAD.UI.Component;

namespace StiffenedShearPlate17
{
    public class ParamsHandler : LoongCAD.STC.Plugins.ParamsHandlerBase
    {
        /// <summary>
        /// 属性空值全局总表
        /// </summary>
        protected static DObject g_emptyValues = null;

        /// <summary>
        /// 属性内置默认值全局总表
        /// </summary>
        protected static DObject g_builtinDefaults = null;

        /// <summary>
        /// 默认构造函数
        /// </summary>
        /// <remarks><b>此方法需要复制到其他组件代码中使用</b></remarks>
        public ParamsHandler()
            : base(ParamsHandler.EmptyValues, ParamsHandler.BuiltinDefautls)
        {

        }
        /// <summary>
        /// 组件公用属性空值表
        /// </summary>
        public static DObject EmptyValues
        {
            get
            {
                if (g_emptyValues == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_emptyValues;
            }
        }
        /// <summary>
        /// 组件公用内置默认值接口
        /// </summary>
        public static DObject BuiltinDefautls
        {
            get
            {
                if (g_builtinDefaults == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_builtinDefaults;
            }
        }
        /// <summary>
        /// 初始化默认值
        /// </summary>
        /// <param name="defaultConfig">从默认值配置加载的默认值</param>
        public override LoongCAD.Util.DObject InitializeDefaultValues(DObject defaultConfig)
        {
            m_defaultValues = new DObject();
            m_defaultValues.CopyFrom(defaultConfig);

            var emptyValues = ParamsHandler.EmptyValues;
            var builtinDefaults = ParamsHandler.BuiltinDefautls;
            if (emptyValues == null || builtinDefaults == null)
            {
                Console.WriteLine("ParamsHandler.InitializeDefaultValues: [Warning] EmptyValue/BuiltinDefaults is null!");
                return m_defaultValues;
            }

            this.TryCompleteDObjectValues(m_defaultValues, builtinDefaults, emptyValues);
            return m_defaultValues;
        }
        /// <summary>
        /// 初始化组件所有实例公用的属性空值、属性内置默认值
        /// </summary>
        public static void IntializeEmptyDefaults()
        {
            g_emptyValues = new DObject();
            g_builtinDefaults = new DObject();

            //图形页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_SPLINT_UP_GAP, double.MaxValue, double.MaxValue); //auto
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_SPLINT_DOWN_GAP, double.MaxValue, double.MaxValue); //auto
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_SPLINT_WEB_GAP, double.MaxValue, 10.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_CONNECT_SECONDARY_GAP, double.MaxValue, 10.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_CONNECT_FLANGE_GAP, double.MaxValue, double.MaxValue); //auto
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_CONNECT_BORDER_GAP, double.MaxValue, double.MaxValue); //auto
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_STIFF_FLANGE_GAP, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_STIFF_BORDER_GAP, double.MaxValue, 0.0);

            //板页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_SPLINT_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 16.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CONNECT_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, double.MaxValue,
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_PAD_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, double.MaxValue,
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_PAD_MEMBER, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CONNECT_MEMBER, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_PAD_UP_GAP, double.MaxValue, double.MaxValue); //auto
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_PAD_DOWN_GAP, double.MaxValue, double.MaxValue); //auto
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_PAD_MARGIN, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_PAD_POSITION, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CONNECT_TYPE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CONNECT_CHAMFER_TYPE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CONNECT_CHAMFER_X, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CONNECT_CHAMFER_Y, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CONNECT_CUT_X, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CONNECT_CUT_Y, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CONNECT_VERT_CUT, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PLATE_CONNECT_HORI_REMAIN_CUT, double.MaxValue, double.MaxValue);

            //加劲肋页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 16.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "P",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "P",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "钢板"
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_ENABLED, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_INSIDE_CHAMFER, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_INSIDE_CHAMFER_X, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_INSIDE_CHAMFER_Y, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CHAMFER, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CHAMFER_X, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_CHAMFER_Y, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_DIR, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.STIFF_VERT_OFFSET, double.MaxValue, 0.0);

            //螺栓页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_BASIC_PROPS, BoltBasicPropertyPanel.EmptyValue, (DObject)new object[] {
                BoltBasicPropertyPanel.ParamNames.BoltStandard, "TS10.9",
                BoltBasicPropertyPanel.ParamNames.BoltSize, 20.0,
                BoltBasicPropertyPanel.ParamNames.ThreadInMtrl, 2, // true
                BoltBasicPropertyPanel.ParamNames.HoleTol, double.MaxValue,
                BoltBasicPropertyPanel.ParamNames.MadeType0, 0, // Workshop
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_HOLE_PROPS, BoltHolePanel.EmptyValue, (DObject)new object[] {
                BoltHolePanel.ParamNames.HoleX, 0.0,
                BoltHolePanel.ParamNames.HoleY, 0.0,
                BoltHolePanel.ParamNames.HoleType, 1,
                BoltHolePanel.ParamNames.RotSlot, 3,
                BoltHolePanel.ParamNames.SlotIn, 4
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_CONTENT_PROPS, null, (DObject)new object[]
            {
                BoltContentPanel.ParamNames.PartWasherUp, false,
                BoltContentPanel.ParamNames.PartWasherDown0, false,
                BoltContentPanel.ParamNames.PartWasherDown1, true,
                BoltContentPanel.ParamNames.PartNut0, true,
                BoltContentPanel.ParamNames.PartNut1, false,
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_EXTEND_PROPS, BoltExtendPanel.EmptyValue, (DObject)new object[]
            {
                BoltExtendPanel.ParamNames.ExtLength, 0.0,
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_OFFSET_MODE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_OFFSET, double.MaxValue, 70.5);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_UP_PLATE_GAP, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_DOWN_PLATE_GAP, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_COUNT, int.MaxValue, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_VERT_STRIDE, new double[] { }, new double[] { 60.0 });

            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SPLINT_HORI_COUNT, int.MaxValue, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SPLINT_HORI_STRIDE, new double[] { }, new double[] { 60.0 });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SPLINT_LEFT_GAP, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SPLINT_RIGHT_GAP, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_SPLINT_MADE_TYPE, 0, 0);

            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_CONNECT_HORI_COUNT, int.MaxValue, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_CONNECT_HORI_STRIDE, new double[] { }, new double[] { 60.0 });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_CONNECT_LEFT_GAP, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_CONNECT_RIGHT_GAP, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT_CONNECT_MADE_TYPE, 0, 0);

            // 槽口页
            g_emptyValues.CopyFrom(BeamNotchPage.EmptyValues);
            g_builtinDefaults.CopyFrom(BeamNotchPage.BuiltinDefaults);

            // 焊缝
            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WELD_PROPS_LIST[i], WeldPropertyPanel.EmptyValue, new DObject {
                    { WeldPropertyPanel.ParamNames.SizeUp, 8.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeUp, 11 }, // Fillet
                    { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                    { WeldPropertyPanel.ParamNames.SizeDown, 8.0 },
                    { WeldPropertyPanel.ParamNames.WeldTypeDown, 11 }, // Fillet
                    { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                    { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                    { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                    { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                    { WeldPropertyPanel.ParamNames.EdgeAround, 1 }, // Edge
                    { WeldPropertyPanel.ParamNames.MadeType, 2 }, // Workshop
                    { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                    { WeldPropertyPanel.ParamNames.RefText, "" }
                });
            }
        }
    }
}
