﻿using System;
using System.Windows.Forms;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;
using System.Collections.Generic;

namespace StiffenedShearPlate17 
{
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {
        private DObject m_rootUIProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;
        private bool m_stiffEnabled = true;
        private bool m_stiffInsideX = true;
        private bool m_stiffInsideY = true;
        private bool m_stiffX = true;
        private bool m_stiffY = true;

        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostIntialize();
        }

        public override ComponentPlugin ReferencePlugin => m_refPlugin;

        private void OnPostIntialize()
        {
            // 设置窗口标题
            this.Text = "梁梁双夹板(207)";

            // 螺栓标准变化之后调用螺栓内容的自适应方法
            this.panel_bolt_basic_property.BoltStandardChanged += this.boltContentPanel.AdaptBoltContents;

            //初始化文字选项卡
            this.cbo_pad_member.ClearItems();
            this.cbo_pad_member.AddItem(0, "默认");
            this.cbo_pad_member.AddItem(1, "否");
            this.cbo_pad_member.AddItem(2, "是");
            this.cbo_pad_member.UpdateItems();
            this.cbo_pad_member.SelectedKey = 0;

            this.cbo_connect_member.ClearItems();
            this.cbo_connect_member.AddItem(0, "默认");
            this.cbo_connect_member.AddItem(1, "无");
            this.cbo_connect_member.AddItem(2, "主要成员");
            this.cbo_connect_member.AddItem(3, "次成员");
            this.cbo_connect_member.AddItem(4, "两者");
            this.cbo_connect_member.UpdateItems();
            this.cbo_connect_member.SelectedKey = 0;

            this.cbo_bolt_vert_offset_mode.ClearItems();
            this.cbo_bolt_vert_offset_mode.AddItem(0, "默认");
            this.cbo_bolt_vert_offset_mode.AddItem(1, "顶部");
            this.cbo_bolt_vert_offset_mode.AddItem(2, "中间");
            this.cbo_bolt_vert_offset_mode.AddItem(3, "底部");
            this.cbo_bolt_vert_offset_mode.UpdateItems();
            this.cbo_bolt_vert_offset_mode.SelectedKey = 0;

            this.cbo_bolt_connect_made_type.ClearItems();
            this.cbo_bolt_connect_made_type.AddItem(0, "默认");
            this.cbo_bolt_connect_made_type.AddItem(1, "工地");
            this.cbo_bolt_connect_made_type.AddItem(2, "工厂");
            this.cbo_bolt_connect_made_type.UpdateItems();
            this.cbo_bolt_connect_made_type.SelectedKey = 0;

            this.cbo_bolt_splint_made_type.ClearItems();
            this.cbo_bolt_splint_made_type.AddItem(0, "默认");
            this.cbo_bolt_splint_made_type.AddItem(1, "工地");
            this.cbo_bolt_splint_made_type.AddItem(2, "工厂");
            this.cbo_bolt_splint_made_type.UpdateItems();
            this.cbo_bolt_splint_made_type.SelectedKey = 0;

            //初始化图片选项卡
            this.cbo_pad_position.ClearItems();
            this.cbo_pad_position.AddIntegerItems(0, 0, 3);
            this.cbo_pad_position.UpdateItems();

            this.cbo_connect_type.ClearItems();
            this.cbo_connect_type.AddIntegerItems(0, 0, 4);
            this.cbo_connect_type.UpdateItems();

            this.cbo_stiff_enabled.ClearItems();
            this.cbo_stiff_enabled.AddIntegerItems(0, 0, 2);
            this.cbo_stiff_enabled.UpdateItems();

            this.cbo_stiff_dir.ClearItems();
            this.cbo_stiff_dir.AddIntegerItems(0, 0, 3);
            this.cbo_stiff_dir.UpdateItems();

            foreach (TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }
        }

        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null)
            {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }

        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked)
            {
                retProps.CopyFrom(m_rootUIProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }

        /// <summary>
        /// 当前组件需要重写此方法，并刷新螺栓基础面板
        /// </summary>
        /// <param name="root"></param>
        public override void ClearUI(Control root = null)
        {
            // 刷新螺栓基础面板
            this.panel_bolt_basic_property.RefreshControls();
            base.ClearUI(root);
        }

        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }

        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_rootUIProps.CopyFrom(srcProps);
        }

        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }

        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_rootUIProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_rootUIProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_rootUIProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_rootUIProps;
            return comp.UIProps;
        }

        private void btn_weld_props_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_rootUIProps, null);
        }

        private void cbo_connect_chamfer_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            bool enabledX = true;
            bool enabledY = false;
            if(cbo_connect_chamfer_type.SelectedIndex == 1)
                enabledX = false;
            if (cbo_connect_chamfer_type.SelectedIndex == 0 || cbo_connect_chamfer_type.SelectedIndex == 2)
                enabledY = true;

            tb_connect_chamfer_x.Enabled = enabledX;
            act_connect_chamfer_x.Enabled = enabledX;
            tb_connect_chamfer_y.Enabled = enabledY;
            act_connect_chamfer_y.Enabled = enabledY;
        }

        private void cbo_stiff_enabled_SelectedIndexChanged(object sender, EventArgs e)
        {
            m_stiffEnabled = true;
            if (cbo_stiff_enabled.SelectedIndex == 1)
                m_stiffEnabled = false;
            cbo_stiff_dir.Enabled = m_stiffEnabled;
            act_stiff_dir.Enabled = m_stiffEnabled;
            tb_stiff_vert_offset.Enabled = m_stiffEnabled;
            act_stiff_vert_offset.Enabled = m_stiffEnabled;
            tb_stiff_flange_gap.Enabled = m_stiffEnabled;
            act_stiff_flange_gap.Enabled = m_stiffEnabled;
            tb_stiff_border_gap.Enabled = m_stiffEnabled;
            act_stiff_border_gap.Enabled = m_stiffEnabled;

            cbo_stiff_chamfer.Enabled = m_stiffEnabled;
            act_stiff_chamfer.Enabled = m_stiffEnabled;
            cbo_stiff_inside_chamfer.Enabled = m_stiffEnabled;
            act_stiff_inside_chamfer.Enabled = m_stiffEnabled;
            AdjustStiffInsideChamfer();
            SetInsideChamferEnabled();
            AdjustStiffChamfer();
            SetChamferEnabled();
        }

        private void cbo_stiff_inside_chamfer_SelectedIndexChanged(object sender, EventArgs e)
        {
            AdjustStiffInsideChamfer();
            SetInsideChamferEnabled();
        }

        private void AdjustStiffInsideChamfer()
        {
            if (!m_stiffEnabled)
            {
                m_stiffInsideX = false;
                m_stiffInsideY = false;
                return;
            }
            m_stiffInsideX = true;
            m_stiffInsideY = true;
            if (cbo_stiff_inside_chamfer.SelectedIndex == 1)
            {
                m_stiffInsideX = false;
                m_stiffInsideY = false;
            }
            if (cbo_stiff_inside_chamfer.SelectedIndex > 2)
                m_stiffInsideY = false;
        }

        private void SetInsideChamferEnabled()
        {
            tb_stiff_inside_chamfer_x.Enabled = m_stiffInsideX;
            act_stiff_inside_chamfer_x.Enabled = m_stiffInsideX;
            tb_stiff_inside_chamfer_y.Enabled = m_stiffInsideY;
            act_stiff_inside_chamfer_y.Enabled = m_stiffInsideY;
        }

        private void abo_stiff_chamfer_SelectedIndexChanged(object sender, EventArgs e)
        {
            AdjustStiffChamfer();
            SetChamferEnabled();
        }

        private void AdjustStiffChamfer()
        {
            if (!m_stiffEnabled)
            {
                m_stiffX = false;
                m_stiffY = false;
                return;
            }
            m_stiffX = true;
            m_stiffY = true;
            if (cbo_stiff_chamfer.SelectedIndex == 1)
            {
                m_stiffX = false;
                m_stiffY = false;
            }  
            if (cbo_stiff_chamfer.SelectedIndex > 2)
                m_stiffY = false;
        }

        private void SetChamferEnabled()
        {
            tb_stiff_chamfer_x.Enabled = m_stiffX;
            act_stiff_chamfer_x.Enabled = m_stiffX;
            tb_stiff_chamfer_y.Enabled = m_stiffY;
            act_stiff_chamfer_y.Enabled = m_stiffY;
        }

        private void cbo_connect_type_SelectedIndexChanged(object sender, EventArgs e)
        {
            bool flag = false;
            if (cbo_connect_type.SelectedIndex == 3)
                flag = true;

            tb_connect_hori_remain_cut.Enabled = flag;
            act_connect_hori_remain_cut.Enabled = flag;
            tb_connect_vert_cut.Enabled = flag;
            act_connect_vert_cut.Enabled = flag;
        }
    }
}
