﻿using System;
using LoongCAD.UI;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;

namespace StiffenedShearPlate17
{
    public partial class Form_WeldParameter : ParamFormBase
    {
        public delegate void OnApplyPropsDelegate(DObject srcProps);
        public delegate void OnModifyPropsDelegate(DObject srcProps);
        public delegate DObject OnGetPropsDelegate();

        private OnApplyPropsDelegate m_applyHandler = null;
        private OnGetPropsDelegate m_getHandler = null;
        private OnModifyPropsDelegate m_modifyHandler = null;
        public Form_WeldParameter()
        {
            InitializeComponent();
            this.OnPostIntialize();
        }
        private void OnPostIntialize()
        {
            // 窗口关闭时不释放实例，只是隐藏窗口
            // 此行代码很重要
            this.DisposeOnClose = false;
            this.AutoScroll = true;

            this.bar_interactive.OkButton.Click += this.OnClickOk;
            this.bar_interactive.ApplyButton.Click += this.OnClickApply;
            this.bar_interactive.ModifyButton.Click += this.OnClickModify;
            this.bar_interactive.GetButton.Click += this.OnClickGet;
            this.bar_interactive.CancelButton.Click += this.OnClickCancel;
        }
        public void Show(DObject srcProps, DObject defaultValues)
        {
            this.ClearUI();
            this.SetParameters(srcProps);
            this.EnableControlChecks(true);
#if false
            this.UpdatePlaceholders(srcProps, defaultValues);
#else
            this.ClearPlaceholders();
#endif
            this.Show();
        }
        public OnApplyPropsDelegate ApplyHandler
        {
            set => m_applyHandler = value;
            get => m_applyHandler;
        }
        public OnGetPropsDelegate GetHandler
        {
            set => m_getHandler = value;
            get => m_getHandler;
        }
        public OnModifyPropsDelegate ModifyHandler
        {
            set => m_modifyHandler = value;
            get => m_modifyHandler;
        }
        private void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            DObject placeholders = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.ClearPlaceholders();
            this.SetPlaceholders(placeholders);
        }
        private void OnClickOk(object sender, EventArgs e)
        {
            if (m_applyHandler != null)
            {
                m_applyHandler(this.GetParameters());
            }
            this.Hide();
        }
        private void OnClickApply(object sender, EventArgs e)
        {
            if (m_applyHandler != null)
            {
                m_applyHandler(this.GetParameters());
            }
        }
        private void OnClickModify(object sender, EventArgs e)
        {
            if (m_modifyHandler != null)
            {
                m_modifyHandler(this.GetParameters());
            }
        }
        private void OnClickGet(object sender, EventArgs e)
        {
            if (m_getHandler != null)
            {
                var srcProps = m_getHandler();
                this.SetParameters(srcProps);
            }
        }
        private void OnClickCheckBox(object sender, EventArgs e)
        {
            this.SwitchControlChecks();
        }
        private void OnClickCancel(object sender, EventArgs e)
        {
            this.Hide();
        }
    }
}
