﻿using System.Collections.Generic;
using System.Linq;
using LoongCAD.STC;

namespace StiffenedShearPlate17
{
    public class BoltData
    {
        public int VertOffsetMode { get; set; }
        public double VertOffset { get; set; }
        public double UpPlateGap { get; set; }
        public double DownPlateGap { get; set; }
        public double[] VertStride { get; set; }

        public double[] SplintHoriStride { get; set; }
        public double SplintLeftGap { get; set; }
        public double SplintRightGap { get; set; }
        public Define.PartMadeType SplintBoltMadeType { get; set; }

        public double[] ConnectHoriStride { get; set; }
        public double ConnectLeftGap { get; set; }
        public double ConnectRightGap { get; set; }
        public Define.PartMadeType ConnectBoltMadeType { get; set; }
        public BoltsConfig BoltsCfg { get; set; } = new BoltsConfig();
    }
    public class BoltParse
    {
        private ParamsHandler m_paramsHandler = null;
        private BoltData m_boltData = new BoltData();
        public BoltParse(ParamsHandler paramsHandler)
        {
            m_paramsHandler = paramsHandler;
        }
        public BoltData GetBoltParams()
        {
            if (!ParseProps())
                return null;
            return m_boltData;
        }
        private bool ParseProps()
        {
            m_boltData.BoltsCfg.CopyFromBasicProps
               (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_BASIC_PROPS));
            m_boltData.BoltsCfg.CopyFromHoleProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_HOLE_PROPS));
            m_boltData.BoltsCfg.CopyFromContentProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_CONTENT_PROPS));
            m_boltData.BoltsCfg.CopyFromExtendProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT_EXTEND_PROPS));

            m_boltData.VertOffsetMode = m_paramsHandler.GetIntValue(ParamNames.BOLT_VERT_OFFSET_MODE);
            m_boltData.VertOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_VERT_OFFSET);
            m_boltData.UpPlateGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_UP_PLATE_GAP);
            m_boltData.DownPlateGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_DOWN_PLATE_GAP);
            var vertNum = m_paramsHandler.GetIntValue(ParamNames.BOLT_VERT_COUNT);
            var arrX = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_VERT_STRIDE);
            m_boltData.VertStride = HandlerStride(arrX, vertNum);

            var splintHoriNum = m_paramsHandler.GetIntValue(ParamNames.BOLT_SPLINT_HORI_COUNT);
            var splintArrY = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_SPLINT_HORI_STRIDE);
            m_boltData.SplintHoriStride = HandlerStride(splintArrY, splintHoriNum);
            m_boltData.SplintLeftGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_SPLINT_LEFT_GAP);
            m_boltData.SplintRightGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_SPLINT_RIGHT_GAP);
            var madeType0 = m_paramsHandler.GetIntValue(ParamNames.BOLT_SPLINT_MADE_TYPE);
            if (madeType0 == 2)
                m_boltData.SplintBoltMadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
            else
                m_boltData.SplintBoltMadeType = Define.PartMadeType.PART_MADE_SITE;

            var connectHoriNum = m_paramsHandler.GetIntValue(ParamNames.BOLT_CONNECT_HORI_COUNT);
            var connectArrY = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_CONNECT_HORI_STRIDE);
            m_boltData.ConnectHoriStride = HandlerStride(connectArrY, connectHoriNum);
            m_boltData.ConnectLeftGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_CONNECT_LEFT_GAP);
            m_boltData.ConnectRightGap = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_CONNECT_RIGHT_GAP);
            var madeType1 = m_paramsHandler.GetIntValue(ParamNames.BOLT_CONNECT_MADE_TYPE);
            if (madeType1 == 2)
                m_boltData.ConnectBoltMadeType = Define.PartMadeType.PART_MADE_WORKSHOP;
            else
                m_boltData.ConnectBoltMadeType = Define.PartMadeType.PART_MADE_SITE;
            return true;
        }
        /// <summary>
        /// 根据螺栓数目调整螺栓间距的设置
        /// </summary>
        private double[] HandlerStride(double[] arr, int num)
        {
            if (num <= 0)
                return null;
            if (num == 1)
                return new double[] { 0 };
            var ret = new List<double>();
            if (arr.Length == 0)
                ret.AddRange(Enumerable.Repeat(60.0, num - 1));
            else
            {
                for (int i = 0; i < num - 1; i++)
                {
                    if (i > arr.Length - 1)
                        ret.Add(arr[arr.Length - 1]);
                    else
                        ret.Add(arr[i]);
                }
            }
            return ret.ToArray();
        }
    }
}
