﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;
using LoongCAD.Localization;

namespace StubConnection119
{
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {
        private DObject m_rootUIProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;
        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostIntialize();
        }
        public override ComponentPlugin ReferencePlugin => m_refPlugin;

        private void OnPostIntialize()
        {
            // 设置窗口标题
            this.Text = "梁柱带托梁端板节点（109）";

            foreach (TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }

            this.tb_side_stiffener_chamfer_mode.ClearItems();
            this.tb_side_stiffener_chamfer_mode.AddIntegerItems(0, 0, 5);
            this.tb_side_stiffener_chamfer_mode.UpdateItems();
            this.tb_side_stiffener_chamfer_mode.SelectedKey = 0;

            this.tb_middle_stiffener_chamfer_mode.ClearItems();
            this.tb_middle_stiffener_chamfer_mode.AddIntegerItems(0, 0, 5);
            this.tb_middle_stiffener_chamfer_mode.UpdateItems();
            this.tb_middle_stiffener_chamfer_mode.SelectedKey = 0;

            this.tb_left_stiffener_open_mode.ClearItems();
            this.tb_left_stiffener_open_mode.AddIntegerItems(0, 0, 3);
            this.tb_left_stiffener_open_mode.UpdateItems();
            this.tb_left_stiffener_open_mode.SelectedKey = 0;

            this.tb_main_top_chamfer_mode.ClearItems();
            this.tb_main_top_chamfer_mode.AddIntegerItems(0, 0, 5);
            this.tb_main_top_chamfer_mode.UpdateItems();
            this.tb_main_top_chamfer_mode.SelectedKey = 0;

            this.tb_main_bottom_chamfer_mode.ClearItems();
            this.tb_main_bottom_chamfer_mode.AddIntegerItems(0, 0, 5);
            this.tb_main_bottom_chamfer_mode.UpdateItems();
            this.tb_main_bottom_chamfer_mode.SelectedKey = 0;

            this.tb_sub_top_chamfer_mode.ClearItems();
            this.tb_sub_top_chamfer_mode.AddIntegerItems(0, 0, 5);
            this.tb_sub_top_chamfer_mode.UpdateItems();
            this.tb_sub_top_chamfer_mode.SelectedKey = 0;

            this.tb_sub_bottom_chamfer_mode.ClearItems();
            this.tb_sub_bottom_chamfer_mode.AddIntegerItems(0, 0, 5);
            this.tb_sub_bottom_chamfer_mode.UpdateItems();
            this.tb_sub_bottom_chamfer_mode.SelectedKey = 0;

            this.tb_sub_haunch_shape.ClearItems();
            this.tb_sub_haunch_shape.AddIntegerItems(0, 0, 3);
            this.tb_sub_haunch_shape.UpdateItems();
            this.tb_sub_haunch_shape.SelectedKey = 0;

            this.tb_main_haunch_shape.ClearItems();
            this.tb_main_haunch_shape.AddIntegerItems(0, 0, 3);
            this.tb_main_haunch_shape.UpdateItems();
            this.tb_main_haunch_shape.SelectedKey = 0;
            //
            this.bolt_basic_standard_props.BoltStandardChanged += this.bolt_content_props.AdaptBoltContents;

            
        }

        public override void ClearUI(Control root = null)
        {
            // 刷新螺栓基础面板
            this.bolt_basic_standard_props.RefreshControls();
            base.ClearUI(root);
        }

        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            this.ClearPlaceholders();
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }

        private void btn_weld_props_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_rootUIProps, null);
        }

        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_rootUIProps.CopyFrom(srcProps);
        }

        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }

        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_rootUIProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_rootUIProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_rootUIProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_rootUIProps;

            return comp.UIProps;
        }
        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null)
            {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }

        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked)
            {
                retProps.CopyFrom(m_rootUIProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }


        private void tb_side_stiffener_chamfer_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_side_stiffener_chamfer_mode.SelectedIntKey;
            SetEnable(tb_side_stiffener_chamfer_cutx, tb_side_stiffener_chamfer_cuty,
                act_side_stiffener_chamfer_cutx, act_side_stiffener_chamfer_cuty, select);
        }


        private void tb_middle_stiffener_chamfer_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_middle_stiffener_chamfer_mode.SelectedIntKey;
            SetEnable(tb_middle_stiffener_chamfer_cutx, tb_middle_stiffener_chamfer_cuty,
                act_middle_stiffener_chamfer_cutx, act_middle_stiffener_chamfer_cuty, select);
        }


        public void SetEnable(LoongCAD.UI.DoubleInput tb_cutx, LoongCAD.UI.DoubleInput tb_cuty, 
            LoongCAD.UI.CheckBox act_cutx, LoongCAD.UI.CheckBox act_cuty, int selected)
        {
            switch (selected)
            {
                case 1:
                    tb_cutx.Enabled = false;
                    act_cutx.Enabled = false;
                    tb_cuty.Enabled = false;
                    act_cuty.Enabled = false;
                    break;
                case 3:
                    tb_cutx.Enabled = true;
                    act_cutx.Enabled = true;
                    tb_cuty.Enabled = false;
                    act_cuty.Enabled = false;
                    break;
                case 4:
                    tb_cutx.Enabled = true;
                    act_cutx.Enabled = true;
                    tb_cuty.Enabled = false;
                    act_cuty.Enabled = false;
                    break;
                default:
                    tb_cutx.Enabled = true;
                    act_cutx.Enabled = true;
                    tb_cuty.Enabled = true;
                    act_cuty.Enabled = true;
                    break;
            }
        }

        private void tb_main_top_chamfer_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_main_top_chamfer_mode.SelectedIntKey;
            SetEnable(tb_main_top_chamfer_cutx, tb_main_top_chamfer_cuty,
                act_main_top_chamfer_cutx, act_main_top_chamfer_cuty, select);
        }

        private void tb_sub_top_chamfer_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_sub_top_chamfer_mode.SelectedIntKey;
            SetEnable(tb_sub_top_chamfer_cutx, tb_sub_top_chamfer_cuty,
                act_sub_top_chamfer_cutx, act_sub_top_chamfer_cuty, select);
        }

        private void tb_main_bottom_chamfer_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_main_bottom_chamfer_mode.SelectedIntKey;
            SetEnable(tb_main_bottom_chamfer_cutx, tb_main_bottom_chamfer_cuty,
                act_main_bottom_chamfer_cutx, act_main_bottom_chamfer_cuty, select);
        }

        private void tb_sub_bottom_chamfer_mode_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_sub_bottom_chamfer_mode.SelectedIntKey;
            SetEnable(tb_sub_bottom_chamfer_cutx, tb_sub_bottom_chamfer_cuty,
                act_sub_bottom_chamfer_cutx, act_sub_bottom_chamfer_cuty, select);
        }

        private void tb_main_haunch_shape_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_main_haunch_shape.SelectedIntKey;
            if (select == 1)
            {
                this.tb_main_haunch_outside_chamfer_cutx.Enabled = true;
                this.act_main_haunch_outside_chamfer_cutx.Enabled = true;
                this.act_main_haunch_outside_chamfer_cuty.Enabled = true;
                this.tb_main_haunch_outside_chamfer_cuty.Enabled = true;
            }
            else
            {
                this.tb_main_haunch_outside_chamfer_cutx.Enabled = false;
                this.act_main_haunch_outside_chamfer_cutx.Enabled = false;
                this.act_main_haunch_outside_chamfer_cuty.Enabled = false;
                this.tb_main_haunch_outside_chamfer_cuty.Enabled = false;
            }
        }

        private void tb_sub_haunch_shape_SelectedIndexChanged(object sender, EventArgs e)
        {
            int select = tb_sub_haunch_shape.SelectedIntKey;
            if (select == 1)
            {
                this.tb_sub_haunch_outside_chamfer_cutx.Enabled = true;
                this.act_sub_haunch_outside_chamfer_cutx.Enabled = true;
                this.act_sub_haunch_outside_chamfer_cuty.Enabled = true;
                this.tb_sub_haunch_outside_chamfer_cuty.Enabled = true;
            }
            else
            {
                this.tb_sub_haunch_outside_chamfer_cutx.Enabled = false;
                this.act_sub_haunch_outside_chamfer_cutx.Enabled = false;
                this.act_sub_haunch_outside_chamfer_cuty.Enabled = false;
                this.tb_sub_haunch_outside_chamfer_cuty.Enabled = false;
            }
        }

    }
}
