﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace StubConnection119
{
    class BoltHandler
    {
        private ComponentInstanceContext m_context = null;
        private ParamsHandler m_paramsHandler = null;

        public BoltHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {
            //主次端板
            if (!(m_context.PartsDict.ContainsKey("main_end_plate") && m_context.PartsDict.ContainsKey("sub_end_plate")))
                return;
            PartBase mainEndPart = m_context.PartsDict["main_end_plate"];
            PartBase subEndPart = m_context.PartsDict["sub_end_plate"];

            //螺栓边距
            BoltLocation horiLoction = m_context.EndPlateBoltLocationHori;
            BoltLocation vertLoction = m_context.EndPlateBoltLocationVert;
            if (horiLoction.count <= 0 || vertLoction.count <= 0)
                return;

            //端板边面
            Plane frontPlane = m_context.EndPlateSidePlanes[0];
            Plane backPlane = m_context.EndPlateSidePlanes[2];
            Plane topPlane = m_context.EndPlateSidePlanes[1];
            Plane bottomPlane = m_context.EndPlateSidePlanes[3];

            //起始点
            double horiOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_HORI_OFFSET);
            Plane boltCenterPlane = topPlane.Move(-vertLoction.margin2 - vertLoction.stride.Sum() / 2);
            Vec3 start = backPlane.Move(-horiLoction.margin1).Intersect(m_context.SubEndPlateCtrlPlane, boltCenterPlane);
            Vec3 end = frontPlane.Move(horiOffset).Intersect(m_context.SubEndPlateCtrlPlane, boltCenterPlane);

            //
            Vec3 x = m_context.MainBottomBound.Normal;
            Vec3 y = m_context.TopOrient;
            Mat43 workAxis = new Mat43(x, y, x.Cross(y), Vec3.ZERO);

            //螺栓位置设置
            BoltGroup boltGroup = m_context.Comp.CreateBoltGroup(mainEndPart, subEndPart, start, end, true);
            boltGroup.WorkAxis = workAxis;

            //
            boltGroup.ArrayStrideX = horiLoction.stride.ToArray();
            boltGroup.ArrayStrideY = vertLoction.stride.ToArray();

            //要删除的螺栓
            int[] deleteBolts = m_paramsHandler.GetIntArrayValue(ParamNames.IGNORE_BOLT_INDEX);
            boltGroup.IgnoreIndices = deleteBolts;

            //参数解析
            DObject basicProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_BASIC_PROPS);
            DObject holeProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_HOLE_PROPS);
            DObject contentProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_CONTENT_PROPS);
            DObject extendProps = m_paramsHandler.GetDObjectValue(ParamNames.BOLT_EXTEND_PROPS);
            BoltParameter boltParameter = new BoltParameter();
            boltParameter.CopyFromBasicProps(basicProps);
            boltParameter.CopyFromHoleProps(holeProps);
            boltParameter.CopyFromContentProps(contentProps);
            boltParameter.CopyFromExtendProps(extendProps);

            //基础属性
            boltGroup.BoltStandard = boltParameter.BoltStandard;
            boltGroup.BoltSize = boltParameter.BoltSize;
            boltGroup.HoleTol = boltParameter.HoleTol; //孔容差范围
            boltGroup.ThreadInMtrl = boltParameter.ThreadInMaterial; //是否包含螺纹 
            boltGroup.MadeType = boltParameter.MadeType; //螺栓制作类型(工厂/工地) 

            //螺栓孔设置
            boltGroup.HoleX = boltParameter.HoleX; //孔X方向扩大尺寸
            boltGroup.HoleY = boltParameter.HoleY;
            boltGroup.HoleType = boltParameter.HoleType; //孔类型
            boltGroup.RotateSlot = boltParameter.RotateSlot; //孔开槽旋转方式 

            //螺栓垫片(螺母)设置
            boltGroup.SectionFlags = boltParameter.SectionFlags; //螺栓部件开关标志位

            //其它
            boltGroup.CutLength = 200; //切割长度
            boltGroup.ExtLength = boltParameter.ExtLength; // 附加长度 
            boltGroup.RotationSide = boltParameter.RotationSide; //螺栓朝向

            //解决旋转后位置错误的问题
            boltGroup.Rotation = 0.0;
            boltGroup.HeadOffsetX = 0.0;
            boltGroup.HeadOffsetY = 0.0;
            boltGroup.HeadOffsetZ = 0.0;
            boltGroup.TailOffsetX = 0.0;
            boltGroup.TailOffsetY = 0.0;
            boltGroup.TailOffsetZ = 0.0;
            boltGroup.VerticalOffset = 0.0;
            boltGroup.DepthOffset = 0.0;

            boltGroup.Refresh();
        }

    }
}
