﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.UI.Component;
using LoongCAD.Util;

namespace StubConnection119
{
    class PlateHandler
    {
        private ComponentInstanceContext m_context = null; 
        private ParamsHandler m_paramsHandler = null;
        //板参数
        PlateProperties mainEndPlateProps = new PlateProperties(); //主端板
        PlateProperties subEndPlateProps = new PlateProperties();
        BoltLocation horiLocation = new BoltLocation();
        BoltLocation vertLocation = new BoltLocation();
        Plane[] endPlateSidePlanes;
        //
        Plane ctrlPlane; //主端板朝向柱一侧的面（平行于控制面），腹板和上下翼板都焊接在这个面上
        Plane rightStiffTopBound, rightStiffBottomBound; //右侧顶板、底板的边界（平行于顶、底板控制面）
        Plane webStiffFrontPlane, webStiffBackPlane; //右侧中间加劲板（已改名为腹板）的上下（MainTop-MainBottom方向）面
        double gap;
        double endPlateMarginTop, endPlateMarginBottom;
        double vertOffset, horiOffset;
        Define.CornerCutType sideChamferMode, middleChamferMode;
        double sideChamferCutX, sideChamferCutY, middleChamferCutX, middleChamferCutY;


        public PlateHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public void Execute()
        {

            ParsePlateParameter();

            CreateEndPlate();

            CreateRightStiff();

            CreateLeftStiff();

            CreateHaunchPlates();

        }

        //创建端板
        public void CreateEndPlate()
        {
            //m_context.test(m_context.EndPlateBottomPlane);
            //Vec3 subTopCenterPoint = m_context.SubTopBound.Intersect(m_context.SubNearCenterPlane, m_context.SubWebCenterPlane);
            //Plane vertCenterPlane = new Plane(subTopCenterPoint, m_context.TopOrient).Move(-horiOffset); //端板定位面（法线朝上）
            ctrlPlane = m_context.SubEndPlateCtrlPlane.Clone();
            Plane vertCenterPlane = m_context.SubVertCenterPlane.Move(horiOffset);
            Plane frontPlane = vertCenterPlane.Move(horiLocation.stride.Sum() / 2 + horiLocation.margin2);
            Plane backPlane = vertCenterPlane.Move(-horiLocation.stride.Sum() / 2 - horiLocation.margin1).Invert();
            Plane topPlane = Util.VerticalPlane(ctrlPlane, ctrlPlane.Intersect(m_context.SubTopPlane), m_context.TopOrient);
            Plane bottomPlane = Util.VerticalPlane(ctrlPlane, ctrlPlane.Intersect(m_context.SubBottomPlane), -m_context.TopOrient);
            topPlane.MoveThis(endPlateMarginTop);
            bottomPlane.MoveThis(endPlateMarginBottom);
            endPlateSidePlanes = new Plane[] { frontPlane, topPlane, backPlane, bottomPlane };
            //次
            do
            {
                if (!Util.PlatePropIsValid(subEndPlateProps))
                    break;
                SteelPlate subEndPlate = m_context.Comp.CreateSteelPlate(ctrlPlane, subEndPlateProps.Thickness, endPlateSidePlanes, true);
                subEndPlateProps.ApplyToPart(subEndPlate);
                subEndPlate.Flush();
                m_context.PartsDict.Add("sub_end_plate", subEndPlate);
            } while (false);
            ctrlPlane.MoveThis(subEndPlateProps.Thickness + gap);
            //主
            do
            {
                if (!Util.PlatePropIsValid(mainEndPlateProps))
                    break;
                SteelPlate mainEndPlate = m_context.Comp.CreateSteelPlate(ctrlPlane, mainEndPlateProps.Thickness, endPlateSidePlanes, true);
                mainEndPlateProps.ApplyToPart(mainEndPlate);
                mainEndPlate.Flush();
                m_context.PartsDict.Add("main_end_plate", mainEndPlate);
            } while (false);
            ctrlPlane.MoveThis(mainEndPlateProps.Thickness);
            //
            m_context.EndPlateSidePlanes = endPlateSidePlanes;
        }

        //创建右侧加劲肋（顶板、底板、中间板）——（已改名为上、下翼板和腹板）
        public void CreateRightStiff()
        {
            Plane topStiffPlane, bottomStiffPlane = null;
            Plane bottomStiffCtrlPlane = null;
            /*
             * 顶底板，各有6个边面
             */
            {
                //靠近内侧腹板的面
                Plane insidePlane = m_context.MainNearWebPlane.Clone();
                //靠近上翼缘的在柱内部的面
                double marginTop = m_paramsHandler.GetDoubleValue(ParamNames.RIGHT_STIFFENER_MARGIN_FRONT);
                Plane topInsidePlane = m_context.MainTopFlangeInside.Move(marginTop);
                //上侧斜面
                Line topInsideLine = topInsidePlane.Intersect(m_context.MainNearBoundPlane);
                Line topOutsideLine = endPlateSidePlanes[2].Intersect(ctrlPlane);
                Plane topPlane = Util.CreatePlaneBy2Line(topInsideLine, topOutsideLine);
                if (topInsidePlane.Normal.Dot(topPlane.Normal) < 0)
                    topPlane.InvertThis();
                //外侧面
                Plane outsidePlane = ctrlPlane.Clone(); //ctrlPlane是端板创建完之后，端板靠近柱的边界面
                //靠近下翼缘的在柱内部的面
                double marginBottom = m_paramsHandler.GetDoubleValue(ParamNames.RIGHT_STIFFENER_MARGIN_BACK);
                Plane bottomInsidePlane = m_context.MainBottomFlangeInside.Move(marginBottom);
                //下侧斜面
                Line bottomInsideLine = bottomInsidePlane.Intersect(m_context.MainNearBoundPlane);
                Line bottomOutsideLine = endPlateSidePlanes[0].Intersect(ctrlPlane);
                Plane bottomPlane = Util.CreatePlaneBy2Line(bottomInsideLine, bottomOutsideLine);
                if (topInsidePlane.Normal.Dot(topPlane.Normal) < 0)
                    topPlane.InvertThis();
                //
                Plane[] sidePlanes = { insidePlane, topInsidePlane, topPlane, outsidePlane, bottomPlane, bottomInsidePlane };

                //上翼板（顶板），节点上方向的右侧加劲肋
                double ctrlPlaneOffsetTop = m_paramsHandler.GetDoubleValue(ParamNames.RIGHT_STIFFENER_MARGIN_TOP);
                double topThickness;
                do {
                    //板属性
                    PlateProperties stiffProps = new PlateProperties();
                    stiffProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.RIGHT_TOP_STIFFENER_PROPS));
                    topThickness = stiffProps.Thickness;
                    //创建面获取
                    Plane stiffCtrlPlane = Util.VerticalPlane(ctrlPlane, m_context.SubEndPlateCtrlPlane.Intersect(m_context.SubTopPlane), -m_context.TopOrient);
                    stiffCtrlPlane.MoveThis(-ctrlPlaneOffsetTop);
                    topStiffPlane = stiffCtrlPlane.Move(topThickness); //中间板的上侧边面(旋转之前)
                    rightStiffTopBound = stiffCtrlPlane.Invert();
                    //
                    if (topThickness <= 0 || insidePlane.Dist(outsidePlane.Origin) < 1e-3)
                        break;
                    //创建，添加属性
                    SteelPlate sp = m_context.Comp.CreateSteelPlate(stiffCtrlPlane, topThickness, sidePlanes, true);
                    m_context.PartsDict.Add("top_plate", sp);
                    if (sp == null)
                        break;
                    stiffProps.ApplyToPart(sp);
                    sp.Flush();
                    //折角
                    ChamferControl chamferControl = new ChamferControl(sp, sidePlanes);
                    chamferControl.ChamferOnePoint(sideChamferCutY, sideChamferCutX, sideChamferMode, chamferControl.comparePointList[1]);
                    chamferControl.ChamferOnePoint(sideChamferCutX, sideChamferCutY, sideChamferMode, chamferControl.comparePointList[0]);
                } while (false);

                //下翼板（底板），节点下方向的右侧加劲肋
                double ctrlPlaneOffsetBottom = m_paramsHandler.GetDoubleValue(ParamNames.RIGHT_STIFFENER_MARGIN_BOTTOM);
                double bottomThickness;
                do
                {
                    //
                    PlateProperties stiffProps = new PlateProperties();
                    stiffProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.RIGHT_BOTTOM_STIFFENER_PROPS));
                    bottomThickness = stiffProps.Thickness;
                    if (bottomThickness <= 0 || insidePlane.Dist(outsidePlane.Origin) < 1e-3)
                        break;
                    //
                    Plane stiffCtrlPlane = Util.VerticalPlane(ctrlPlane, m_context.SubEndPlateCtrlPlane.Intersect(m_context.SubBottomPlane), m_context.TopOrient);
                    stiffCtrlPlane.MoveThis(-ctrlPlaneOffsetBottom);//底板的创建面(旋转之前的创建面)
                    bottomStiffPlane = stiffCtrlPlane.Move(bottomThickness); //中间板的下侧边面(旋转之前)
                    //计算旋转角度(sin值)
                    double topBottomGap = m_paramsHandler.GetDoubleValue(ParamNames.RIGHT_STIFFENER_GAP);
                    double topBottomGapOutside = m_context.SubTopPlane.Dist(m_context.SubBottomPlane.Origin) + ctrlPlaneOffsetTop + ctrlPlaneOffsetBottom;
                    Util.SetDefaultValue(ref topBottomGap, topBottomGapOutside);
                    double distWeb = m_context.MainNearWebPlane.Dist(bottomOutsideLine.V0);
                    double rotateAngle = Math.Atan((topBottomGapOutside - topBottomGap) / distWeb);
                   /* double a = topBottomGapOutside - topBottomGap; //直角边a
                    double b = distWeb; //直角边b
                    double _sin = a / Math.Sqrt(a * a + b * b);*/
                    //对底板的创建面进行旋转,top和bottom是旋转中心线(主端板下侧靠近柱的一条棱)上的两个点
                    Vec3 top = endPlateSidePlanes[2].Intersect(stiffCtrlPlane, ctrlPlane);
                    Vec3 bottom = endPlateSidePlanes[0].Intersect(stiffCtrlPlane, ctrlPlane);
                    Util.RotatePlane(ref stiffCtrlPlane, top, bottom, rotateAngle);
                    //计算底板创建面位置
                  /* Line outsideLine = stiffCtrlPlane.Intersect(ctrlPlane);
                    Line insideLine = topStiffPlane.Move(topBottomGap - topThickness).Intersect(m_context.MainNearWebPlane);
                    m_context.test(topStiffPlane);
                    stiffCtrlPlane = Util.CreatePlaneBy2Line(outsideLine, insideLine);
                    Util.Along(ref stiffCtrlPlane, m_context.TopOrient);*/
                    bottomStiffCtrlPlane = stiffCtrlPlane.Move(bottomThickness); //中间板的下侧边面（旋转之后）
                    rightStiffBottomBound = stiffCtrlPlane.Invert(); 
                    //移动底板的内侧边面和外侧边面，防止嵌入到柱里面
                    double planeMove = bottomThickness * Math.Sin(rotateAngle);
                    if (rotateAngle >= 0) //TODO 中间折线部分错误
                        Util.MovePlane(ref sidePlanes[3], sidePlanes[3].Normal, planeMove);
                    else
                        Util.MovePlane(ref sidePlanes[0], sidePlanes[0].Normal, -planeMove);
                    //
                    SteelPlate sp = m_context.Comp.CreateSteelPlate(stiffCtrlPlane, bottomThickness, sidePlanes, true);
                    m_context.PartsDict.Add("bottom_plate", sp);
                    if (sp == null)
                        break;
                    stiffProps.ApplyToPart(sp);
                    sp.Flush();
                    //折角
                    ChamferControl chamferControl = new ChamferControl(sp, sidePlanes);
                    chamferControl.ChamferOnePoint(sideChamferCutY, sideChamferCutX, sideChamferMode, chamferControl.comparePointList[1]);
                    chamferControl.ChamferOnePoint(sideChamferCutX, sideChamferCutY, sideChamferMode, chamferControl.comparePointList[0]);
                } while (false) ;

            }

            /*
             * 腹板（中间板）
             */
            do {
                //
                PlateProperties stiffProps = new PlateProperties();
                stiffProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.RIGHT_MIDDLE_STIFFENER_PROPS));
                if (stiffProps.Thickness <= 0)
                    break;
                //次梁中心面
                Plane vertCenterPlane = m_context.SubVertCenterPlane.Clone(); //平行于节点top方向
                Plane horiCenterPlane = m_context.SubHoriCenterPlane.Clone(); //垂直于节点top方向
                //顶底板上下边界，没有创建面就用次梁中心面代替
                if (topStiffPlane == null)
                    topStiffPlane = horiCenterPlane;
                if (bottomStiffPlane == null)
                    bottomStiffPlane = horiCenterPlane;
                if (bottomStiffCtrlPlane != null)
                { 
                    Vec3 p = bottomStiffCtrlPlane.Intersect(m_context.MainNearWebPlane, m_context.MainBottomBound); //判断底边界面远近
                    Plane bottomStiffPlane2 = new Plane(p, bottomStiffPlane.Normal);
                    if (topStiffPlane.Dist(bottomStiffPlane2.Origin) > topStiffPlane.Dist(bottomStiffPlane.Origin))
                        bottomStiffPlane = bottomStiffPlane2;
                } 
                //宽度由顶、底板的间距确定
                double width = topStiffPlane.Dist(bottomStiffPlane.Origin);
                if (width < 1e-4)
                    break;
                //中间板的TOP-BOTTOM方向（也是节点的top-bottom方向）的中心面，中间板的3个控制点都在这个面上
                Plane centerPlateVPlane = Util.CenterPlane(topStiffPlane, bottomStiffPlane);
                //中间板的MainTop-MainBottom方向中心面
                Vec3 intersectPoint = vertCenterPlane.Intersect(m_context.MainNearBoundPlane, centerPlateVPlane);
                Plane centerPlateHPlane = new Plane(intersectPoint, m_context.MainTopBound.Normal);
                //控制点列表
                Vec3 point1 = centerPlateVPlane.Intersect(centerPlateHPlane, m_context.MainNearWebPlane);
                Vec3 point2 = centerPlateVPlane.Intersect(centerPlateHPlane, m_context.MainNearBoundPlane);
                Vec3 point3 = centerPlateVPlane.Intersect(vertCenterPlane, ctrlPlane);
                Vec3[] ctrlPoints = { point1, point2, point3 };
                if (point1.Dist(point2) < 1e-3)
                    break;
                //
                Vec3 x = m_context.MainNearBoundPlane.Normal;
                Vec3 y = m_context.MainTopBound.Normal;
                Mat43 workAxis = new Mat43(x, y, x.Cross(y), Vec3.ZERO);
                String profile = "PL" + stiffProps.Thickness + "*" + width;
                SteelPolyBeam plate = m_context.Comp.CreateSteelPolyBeam(profile, ctrlPoints, true);
                m_context.PartsDict.Add("web_plate", plate);
                //
                if (plate == null)
                    break;
                plate.WorkAxis = workAxis;
                plate.RotationSide = Define.ObjectSide.SIDE_TOP;
                stiffProps.ApplyToPart(plate);
                plate.Flush();
                //让下侧对齐到底板上（仅设置角度时有效）
                if (bottomStiffCtrlPlane != null && !bottomStiffCtrlPlane.Normal.IsParallelTo(bottomStiffPlane.Normal))
                    m_context.Comp.CreateCutPlane(plate, bottomStiffCtrlPlane, true);
                //折角
                {
                    //靠近柱的上侧折角
                    Define.CornerCutType mainTopCutMode = ConvertToCutType((StiffChamferMode)m_paramsHandler.GetIntValue(ParamNames.MAIN_TOP_CHAMFER_MODE));
                    double mainTopCutX = m_paramsHandler.GetDoubleValue(ParamNames.MAIN_TOP_CHAMFER_CUTX);
                    double mainTopCutY = m_paramsHandler.GetDoubleValue(ParamNames.MAIN_TOP_CHAMFER_CUTY);
                    SetCutYEqualCutX(ref mainTopCutY, mainTopCutX, mainTopCutMode);
                    CreateCutterChamfer(topStiffPlane, m_context.MainNearWebPlane, centerPlateHPlane, plate, mainTopCutX, mainTopCutY, mainTopCutMode);
                    //靠近端板的上侧折角
                    Define.CornerCutType subTopCutMode = ConvertToCutType((StiffChamferMode)m_paramsHandler.GetIntValue(ParamNames.SUB_TOP_CHAMFER_MODE));
                    double subTopCutX = m_paramsHandler.GetDoubleValue(ParamNames.SUB_TOP_CHAMFER_CUTX);
                    double subTopCutY = m_paramsHandler.GetDoubleValue(ParamNames.SUB_TOP_CHAMFER_CUTY);
                    SetCutYEqualCutX(ref subTopCutY, subTopCutX, subTopCutMode);
                    CreateCutterChamfer(topStiffPlane, ctrlPlane, vertCenterPlane, plate, subTopCutX, subTopCutY, subTopCutMode);
                    //靠近柱的下侧折角
                    Define.CornerCutType mainBottomCutMode = ConvertToCutType((StiffChamferMode)m_paramsHandler.GetIntValue(ParamNames.MAIN_BOTTOM_CHAMFER_MODE));
                    double mainBottomCutX = m_paramsHandler.GetDoubleValue(ParamNames.MAIN_BOTTOM_CHAMFER_CUTX);
                    double mainBottomCutY = m_paramsHandler.GetDoubleValue(ParamNames.MAIN_BOTTOM_CHAMFER_CUTY);
                    SetCutYEqualCutX(ref mainBottomCutY, mainBottomCutX, mainBottomCutMode);
                    CreateCutterChamfer(bottomStiffCtrlPlane, m_context.MainNearWebPlane, centerPlateHPlane, plate, mainBottomCutX, mainBottomCutY, mainBottomCutMode);
                    //靠近端板的下侧折角
                    Define.CornerCutType subBottomCutMode = ConvertToCutType((StiffChamferMode)m_paramsHandler.GetIntValue(ParamNames.SUB_BOTTOM_CHAMFER_MODE));
                    double subBottomCutX = m_paramsHandler.GetDoubleValue(ParamNames.SUB_BOTTOM_CHAMFER_CUTX);
                    double subBottomCutY = m_paramsHandler.GetDoubleValue(ParamNames.SUB_BOTTOM_CHAMFER_CUTY);
                    SetCutYEqualCutX(ref subBottomCutY, subBottomCutX, subBottomCutMode);
                    CreateCutterChamfer(bottomStiffCtrlPlane, ctrlPlane, vertCenterPlane, plate, subBottomCutX, subBottomCutY, subBottomCutMode);
                }
                //上下面（为腹板加劲板做准备）
                Plane centerPlateHPlane1 = new Plane(point3, m_context.SubFrontPlane.Normal);
                webStiffFrontPlane = centerPlateHPlane1.Move(stiffProps.Thickness / 2);
                webStiffBackPlane = centerPlateHPlane1.Invert().Move(stiffProps.Thickness / 2);
            } while (false);


        }

        //创建左侧加劲肋（顶部加劲肋、底部加劲肋、中加劲肋）——（已改名为柱加劲板）
        public void CreateLeftStiff()
        {
            //是否开启柱左侧的3个加劲肋
            LeftStiffOpenMode openMode = (LeftStiffOpenMode)m_paramsHandler.GetIntValue(ParamNames.LEFT_STIFFENER_OPEN_MODE);
            if (openMode == LeftStiffOpenMode.CLOSE)
                return;

            /*
             * 板属性
             */
            PlateProperties topStiffProps = new PlateProperties();
            PlateProperties centerStiffProps = new PlateProperties();
            PlateProperties bottomStiffProps = new PlateProperties();
            topStiffProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.LEFT_TOP_STIFFENER_PROPS));
            centerStiffProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.LEFT_TOP_STIFFENER_PROPS));
            bottomStiffProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.LEFT_TOP_STIFFENER_PROPS));

            /*
             * 顶、底加劲肋
             */
            Plane topCtrlPlane, bottomCtrlPlane; //顶、底加劲肋的控制面
            Plane insidePlane, outsidePlane; //内、外边面
            {
                //边面
                double vertMarginTopInside = m_paramsHandler.GetDoubleValue(ParamNames.LEFT_STIFFENER_VERT_MARGIN_SIDE);
                double vertMarginBottomInside = m_paramsHandler.GetDoubleValue(ParamNames.LEFT_STIFFENER_VERT_MARGIN_SIDE);
                double horiMarginCenter = m_paramsHandler.GetDoubleValue(ParamNames.LEFT_STIFFENER_HORI_MARGIN_CENTER);
                double horiMarginSide = m_paramsHandler.GetDoubleValue(ParamNames.LEFT_STIFFENER_HORI_MARGIN_SIDE);
                Plane topPlane = m_context.MainBottomFlangeInside.Move(vertMarginTopInside);
                Plane bottomPlane = m_context.MainTopFlangeInside.Move(vertMarginBottomInside);
                insidePlane = m_context.MainAwayWebPlane.Move(horiMarginCenter);
                outsidePlane = m_context.MainAwayBoundPlane.Move(-horiMarginSide);
                Plane[] sidePlanes = { topPlane, insidePlane, bottomPlane, outsidePlane};
                //上
                do {
                    //计算控制面
                    double marginTop = m_paramsHandler.GetDoubleValue(ParamNames.SIDE_STIFFENER_MARGIN_TOP);
                    Line line = m_context.SubTopPlane.Intersect(m_context.SubNearCenterPlane);
                    topCtrlPlane = new Plane(line.V0, -m_context.TopOrient).Move(-marginTop);
                    //板属性
                    if (topStiffProps.Thickness <= 0)
                        break;
                    //
                    SteelPlate sp = m_context.Comp.CreateSteelPlate(topCtrlPlane, topStiffProps.Thickness, sidePlanes, true);
                    m_context.PartsDict.Add("column_top_stiff", sp);
                    if (sp == null)
                        break;
                    topStiffProps.ApplyToPart(sp);
                    sp.Flush();
                    //折角
                    ChamferControl chamferControl = new ChamferControl(sp, sidePlanes);
                    chamferControl.ChamferOnePoint(sideChamferCutX, sideChamferCutY, sideChamferMode, chamferControl.comparePointList[1]);
                    chamferControl.ChamferOnePoint(sideChamferCutY, sideChamferCutX, sideChamferMode, chamferControl.comparePointList[2]);
                } while (false);
                //下
                do {
                    //计算控制面
                    double marginBottom = m_paramsHandler.GetDoubleValue(ParamNames.SIDE_STIFFENER_MARGIN_BOTTOM);
                    Line line = m_context.SubBottomPlane.Intersect(m_context.SubNearCenterPlane);
                    bottomCtrlPlane = new Plane(line.V0, m_context.TopOrient).Move(-marginBottom);
                    //板属性
                    if (bottomStiffProps.Thickness <= 0)
                        break;
                    //
                    SteelPlate sp = m_context.Comp.CreateSteelPlate(bottomCtrlPlane, bottomStiffProps.Thickness, sidePlanes, true);
                    m_context.PartsDict.Add("column_bottom_stiff", sp);
                    if (sp == null)
                        break;
                    bottomStiffProps.ApplyToPart(sp);
                    sp.Flush();
                    //折角
                    ChamferControl chamferControl = new ChamferControl(sp, sidePlanes);
                    chamferControl.ChamferOnePoint(sideChamferCutX, sideChamferCutY, sideChamferMode, chamferControl.comparePointList[1]);
                    chamferControl.ChamferOnePoint(sideChamferCutY, sideChamferCutX, sideChamferMode, chamferControl.comparePointList[2]);
                } while (false) ;
            }

            /*
             * 中间加劲肋
             */
            do {
                if (centerStiffProps.Thickness <= 0)
                    break;
                Vec3 intersectPoint = m_context.SubVertCenterPlane.Intersect(m_context.MainAwayWebPlane, Util.CenterPlane(topCtrlPlane, bottomCtrlPlane));
                Plane ctrlPlane = new Plane(intersectPoint, m_context.MainTopBound.Normal).Move(-centerStiffProps.Thickness / 2);
                Plane[] sidePlanes = { topCtrlPlane.Move(topStiffProps.Thickness), insidePlane, bottomCtrlPlane.Move(bottomStiffProps.Thickness), outsidePlane };
                SteelPlate sp = m_context.Comp.CreateSteelPlate(ctrlPlane, centerStiffProps.Thickness, sidePlanes, true);
                m_context.PartsDict.Add("column_center_stiff", sp);
                if (sp == null)
                    break;
                bottomStiffProps.ApplyToPart(sp);
                sp.Flush();
                //折角
                ChamferControl chamferControl = new ChamferControl(sp, sidePlanes);
                chamferControl.ChamferOnePoint(middleChamferCutX, middleChamferCutY, middleChamferMode, chamferControl.comparePointList[1]);
                chamferControl.ChamferOnePoint(middleChamferCutY, middleChamferCutX, middleChamferMode, chamferControl.comparePointList[2]);
            } while (false);
        }

        //创建腋板——（已改名为上、下加劲板）
        public void CreateHaunchPlates()
        {
            //
            m_context.HaunchDict.Add("sub_haunch", new List<HaunchHelper>());
            m_context.HaunchDict.Add("main_top_haunch", new List<HaunchHelper>());
            m_context.HaunchDict.Add("main_web_haunch", new List<HaunchHelper>());
            m_context.HaunchDict.Add("main_bottom_haunch", new List<HaunchHelper>());
            Plane subVertCenterPlane = Util.CenterPlane(m_context.SubFrontBound, m_context.SubBackBound);

            /*
             * 加劲板属性
             */
            PlateProperties topStiffProps = new PlateProperties(); 
            PlateProperties bottomStiffProps = new PlateProperties();
            PlateProperties webStiffProps = new PlateProperties();
            WeldProperties weldProps = new WeldProperties();
            topStiffProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.TOP_HAUNCH_PLATES_PROPS));
            bottomStiffProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.BOTTOM_HAUNCH_PLATES_PROPS));
            webStiffProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.WEB_STIFF_PROPS));
            SetPropsFollow(bottomStiffProps, topStiffProps);
            weldProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[7]));
            double outsideCutX, outsideCutY, insideCutX ,insideCutY; //折角参数
            StiffShape stiffShape;
            int webStiffCount = m_paramsHandler.GetIntValue(ParamNames.WEB_STIFF_COUNT); //次梁腋板属性
            double[] webStiffStride = (double[])m_paramsHandler.GetDoubleArrayValue(ParamNames.WEB_STIFF_STRIDE).Clone();
            double webStiffMarginTop = m_paramsHandler.GetDoubleValue(ParamNames.WEB_STIFF_MARGIN_TOP);
            double subHeight = m_context.SubTopBound.Dist(m_context.SubBottomBound.Origin);
            Util.SetDefaultValue(ref webStiffMarginTop, 60); //默认2个腋板，翼缘边距60，stride自适应
            Util.SetDefaultValue(ref webStiffStride[0], subHeight - 2 * webStiffMarginTop);
            List<double> stiffStride = BoltParameter.ParseStrideInput(webStiffCount, webStiffStride);

            /*
             * 主加劲板（靠近柱一侧）
             */
            //折角参数
            outsideCutX = m_paramsHandler.GetDoubleValue(ParamNames.MAIN_HAUNCH_OUTSIDE_CHAMFER_CUTX);
            outsideCutY = m_paramsHandler.GetDoubleValue(ParamNames.MAIN_HAUNCH_OUTSIDE_CHAMFER_CUTY);
            insideCutX = m_paramsHandler.GetDoubleValue(ParamNames.MAIN_HAUNCH_INSIDE_CHAMFER_CUTX);
            insideCutY = m_paramsHandler.GetDoubleValue(ParamNames.MAIN_HAUNCH_INSIDE_CHAMFER_CUTY);
            stiffShape = (StiffShape)m_paramsHandler.GetIntValue(ParamNames.MAIN_HAUNCH_SHAPE);
            //上加劲板
            {
                HaunchHelper haunch = new HaunchHelper(ctrlPlane, rightStiffTopBound, m_context.SubVertCenterPlane, Util.Copy(topStiffProps));
                haunch.SetFollowPlane(endPlateSidePlanes[1]);
                haunch.SetOutsideChamfer(outsideCutX, outsideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                haunch.SetInsideChamfer(insideCutX, insideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                if (stiffShape == StiffShape.TRIANGLE)
                    haunch.isTrianlge = true;
                haunch.CreateHaunch(m_context.Comp);
                m_context.HaunchDict["main_top_haunch"].Add(haunch);
            }
            //下加劲板
            {
                HaunchHelper haunch = new HaunchHelper(ctrlPlane, rightStiffBottomBound, m_context.SubVertCenterPlane, Util.Copy(bottomStiffProps));
                haunch.SetFollowPlane(endPlateSidePlanes[3]);
                haunch.SetOutsideChamfer(outsideCutX, outsideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                haunch.SetInsideChamfer(insideCutX, insideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                if (stiffShape == StiffShape.TRIANGLE)
                    haunch.isTrianlge = true;
                haunch.CreateHaunch(m_context.Comp);
                m_context.HaunchDict["main_bottom_haunch"].Add(haunch);
            }
            //腹板加劲板
            do
            {
                int mainWebStiffCount = m_paramsHandler.GetIntValue(ParamNames.MAIN_WEB_STIFF_COUNT);
                Util.SetDefaultValue(ref mainWebStiffCount, webStiffCount); //跟随右侧数目
                double[] mainWebStiffStride = (double[])m_paramsHandler.GetDoubleArrayValue(ParamNames.MAIN_WEB_STIFF_STRIDE).Clone();
                double mainWebStiffMarginTop = m_paramsHandler.GetDoubleValue(ParamNames.MAIN_WEB_STIFF_MARGIN_TOP);
                Plane webStiffCtrlPlane;
                List<double> mainWebStiffStrideList;
                //未旋转时可跟随右侧
                if (!m_context.IsSubRotate)
                {
                    //跟随右侧间距
                    Util.SetDefaultValue(ref mainWebStiffStride[0], stiffStride[0]); 
                    mainWebStiffStrideList = BoltParameter.ParseStrideInput(mainWebStiffCount, mainWebStiffStride);
                    //边距设置
                    if (mainWebStiffMarginTop != double.MaxValue)//设置了边距
                        webStiffCtrlPlane = rightStiffTopBound.Invert().Move(mainWebStiffMarginTop);
                    else  //左侧未设置边距，默认跟随右侧加劲板
                        webStiffCtrlPlane = Util.VerticalPlane(ctrlPlane, m_context.SubEndPlateCtrlPlane.Intersect(m_context.SubTopPlane), -m_context.TopOrient).Move(webStiffMarginTop);
                }
                //旋转了需自适应
                else
                {
                    //自适应边距
                    Util.SetDefaultValue(ref mainWebStiffMarginTop, 60);
                    webStiffCtrlPlane = rightStiffTopBound.Invert().Move(mainWebStiffMarginTop);
                    //自适应间距
                    subHeight = m_context.SubTopPlane.Dist(m_context.SubBottomPlane.Origin);
                    Util.SetDefaultValue(ref mainWebStiffStride[0], subHeight - 2 * mainWebStiffMarginTop);
                    mainWebStiffStrideList = BoltParameter.ParseStrideInput(mainWebStiffCount, mainWebStiffStride);
                }
                if (!Util.PlatePropIsValid(webStiffProps) || webStiffCount <= 0)
                    break;
                //添加多跟腹板加劲板
                for (int i = 0; ; i++)
                {
                    //前侧
                    HaunchHelper haunch1 = new HaunchHelper(ctrlPlane, webStiffFrontPlane, webStiffCtrlPlane, Util.Copy(webStiffProps));
                    haunch1.SetFollowPlane(endPlateSidePlanes[0]);
                    haunch1.SetOutsideChamfer(outsideCutX, outsideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    haunch1.SetInsideChamfer(insideCutX, insideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch1.isTrianlge = true;
                    haunch1.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["main_web_haunch"].Add(haunch1);
                    //后侧
                    HaunchHelper haunch2 = new HaunchHelper(ctrlPlane, webStiffBackPlane, webStiffCtrlPlane, Util.Copy(webStiffProps));
                    haunch2.SetFollowPlane(endPlateSidePlanes[2]);
                    haunch2.SetOutsideChamfer(outsideCutX, outsideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    haunch2.SetInsideChamfer(insideCutX, insideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch2.isTrianlge = true;
                    haunch2.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["main_web_haunch"].Add(haunch2);
                    //
                    if (i == mainWebStiffStrideList.Count)
                        break;
                    webStiffCtrlPlane.MoveThis(mainWebStiffStrideList[i]);
                }
            } while (false);

            /*
             * 次加劲板（靠近梁一侧）
             */
            //折角参数
            outsideCutX = m_paramsHandler.GetDoubleValue(ParamNames.SUB_HAUNCH_OUTSIDE_CHAMFER_CUTX);
            outsideCutY = m_paramsHandler.GetDoubleValue(ParamNames.SUB_HAUNCH_OUTSIDE_CHAMFER_CUTY);
            insideCutX = m_paramsHandler.GetDoubleValue(ParamNames.SUB_HAUNCH_INSIDE_CHAMFER_CUTX);
            insideCutY = m_paramsHandler.GetDoubleValue(ParamNames.SUB_HAUNCH_INSIDE_CHAMFER_CUTY);
            stiffShape = (StiffShape)m_paramsHandler.GetIntValue(ParamNames.SUB_HAUNCH_SHAPE);
            //上加劲板
            {
                HaunchHelper haunch = new HaunchHelper(m_context.SubEndPlateCtrlPlane.Invert(), m_context.SubTopBound, subVertCenterPlane, Util.Copy(topStiffProps));
                Plane topPlane = m_context.IsSubRotate ? endPlateSidePlanes[2] : endPlateSidePlanes[1];
                haunch.SetFollowPlane(topPlane);
                haunch.SetOutsideChamfer(outsideCutX, outsideCutY, Define.CornerCutType.CORNER_CUT_LINE); //TODO 梁和板斜放的情况
                haunch.SetInsideChamfer(insideCutX, insideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                if (stiffShape == StiffShape.TRIANGLE)
                    haunch.isTrianlge = true;
                haunch.CreateHaunch(m_context.Comp);
                m_context.HaunchDict["sub_haunch"].Add(haunch);
            }
            //下加劲板
            {
                HaunchHelper haunch = new HaunchHelper(m_context.SubEndPlateCtrlPlane.Invert(), m_context.SubBottomBound, subVertCenterPlane, Util.Copy(bottomStiffProps));
                Plane bottomPlane = m_context.IsSubRotate ? endPlateSidePlanes[0] : endPlateSidePlanes[3];
                haunch.SetFollowPlane(bottomPlane);
                haunch.SetOutsideChamfer(outsideCutX, outsideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                haunch.SetInsideChamfer(insideCutX, insideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                if (stiffShape == StiffShape.TRIANGLE)
                    haunch.isTrianlge = true;
                haunch.CreateHaunch(m_context.Comp);
                m_context.HaunchDict["sub_haunch"].Add(haunch);
            }
            //腹板加劲板
            do {
                Plane webStiffCtrlPlane = Util.VerticalPlane(ctrlPlane, m_context.SubEndPlateCtrlPlane.Intersect(m_context.SubTopBound), -m_context.TopOrient).Move(webStiffMarginTop);
                if (!Util.PlatePropIsValid(webStiffProps) || webStiffCount <= 0)
                    break;
                for (int i = 0;; i++)
                {
                    //前侧
                    HaunchHelper haunch1 = new HaunchHelper(m_context.SubEndPlateCtrlPlane.Invert(), m_context.SubFrontWeb, webStiffCtrlPlane, Util.Copy(webStiffProps));
                    Plane bottomPlane = m_context.IsSubRotate ? endPlateSidePlanes[1] : endPlateSidePlanes[0];
                    haunch1.SetFollowPlane(bottomPlane);
                    haunch1.SetOutsideChamfer(outsideCutX, outsideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    haunch1.SetInsideChamfer(insideCutX, insideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch1.isTrianlge = true;
                    haunch1.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["sub_haunch"].Add(haunch1);
                    //后侧
                    HaunchHelper haunch2 = new HaunchHelper(m_context.SubEndPlateCtrlPlane.Invert(), m_context.SubBackWeb, webStiffCtrlPlane, webStiffProps);
                    bottomPlane = m_context.IsSubRotate ? endPlateSidePlanes[3] : endPlateSidePlanes[2];
                    haunch2.SetFollowPlane(bottomPlane);
                    haunch2.SetOutsideChamfer(outsideCutX, outsideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    haunch2.SetInsideChamfer(insideCutX, insideCutY, Define.CornerCutType.CORNER_CUT_LINE);
                    if (stiffShape == StiffShape.TRIANGLE)
                        haunch2.isTrianlge = true;
                    haunch2.CreateHaunch(m_context.Comp);
                    m_context.HaunchDict["sub_haunch"].Add(haunch2);
                    //
                    if (i == stiffStride.Count)
                        break;
                    webStiffCtrlPlane.MoveThis(stiffStride[i]);
                }
            } while (false);

        }

        //解析板参数、螺栓参数
        public void ParsePlateParameter()
        {
            //端板属性
            mainEndPlateProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.END_PLATE1_PROPS));
            subEndPlateProps.SetProps(m_paramsHandler.GetDObjectValue(ParamNames.END_PLATE2_PROPS));
            //梁间距
            gap = m_paramsHandler.GetDoubleValue(ParamNames.ENDPLATE_DISTANCE);
            //垂直螺栓参数
            double boltMarginBottom = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_BOTTOM_MARGIN); //底部螺栓与端板边缘的距离
            double boltMarginTop = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_TOP_MARGIN);
            double[] vertStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_VERT_STRIDE);
            int vertCount = m_paramsHandler.GetIntValue(ParamNames.BOLT_VERT_COUNT);
            vertOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_VERT_OFFSET); //螺栓的垂直偏移
            //水平螺栓参数
            double boltMarginLeft = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_LEFT_MARGIN);
            double boltMarginRight = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_RIGHT_MARGIN);
            double[] horiStride = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT_HORI_STRIDE);
            int horiCount = m_paramsHandler.GetIntValue(ParamNames.BOLT_HORI_COUNT);
            horiOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT_HORI_OFFSET); //板的水平偏移（由螺栓中心偏移引起）
            //另一个方向，保证螺栓的参数是与节点方向一致 
            if (m_context.IsSubRotate)
            {
                Util.Swap(ref boltMarginTop, ref boltMarginLeft);
                Util.Swap(ref boltMarginBottom, ref boltMarginRight);
                Util.Swap(ref vertStride, ref horiStride);
                Util.Swap(ref vertCount, ref horiCount);
                Util.Swap(ref vertOffset, ref horiOffset);
                Util.Swap(ref mainEndPlateProps.Height, ref mainEndPlateProps.Width);
            }
            //端板和螺栓的Top-Bottom方向的定位
            {
                double beamHeight = m_context.SubTopPlane.Dist(m_context.SubBottomPlane.Origin) ;
                //端板参数
                endPlateMarginTop = m_paramsHandler.GetDoubleValue(ParamNames.ENDPLATE_MARGIN_TOP);
                endPlateMarginBottom = m_paramsHandler.GetDoubleValue(ParamNames.ENDPLATE_MARGIN_BOTTOM);
                double endPlateHeight = mainEndPlateProps.Height;
                //默认值
                vertLocation.SetDefaultStride(2, new double[] { 60.0 });
                //螺栓参数
                //若设置板高度，重新定位
                if (endPlateHeight != double.MaxValue)
                {
                    //两个边距没填，上下边距由板高度决定
                    if (endPlateMarginTop == double.MaxValue && endPlateMarginBottom == double.MaxValue)
                    {
                        endPlateMarginBottom = (endPlateHeight - beamHeight) / 2;
                        endPlateMarginTop = (endPlateHeight - beamHeight) / 2;
                    }
                    //填了上边距，就由上边距定位（不管下边距是否填写）
                    else if (endPlateMarginTop != double.MaxValue)
                    {
                        endPlateMarginBottom = endPlateHeight - beamHeight - endPlateMarginTop;
                    }
                    //只填了下边距，按下边距定位
                    else
                    {
                        endPlateMarginTop = endPlateHeight - beamHeight - endPlateMarginBottom;
                    }
                }
                //计算螺栓边距
                vertLocation.SetCenterBoltLoaction(boltMarginBottom, boltMarginTop, vertCount, vertStride, beamHeight / 2 + vertOffset,
                    beamHeight, endPlateMarginTop, endPlateMarginBottom, 30);
                //两个边距没填，由螺栓(垂直方向上居中次梁)确定高度
                if (endPlateMarginTop == double.MaxValue && endPlateMarginBottom == double.MaxValue)
                {
                    endPlateMarginBottom = vertLocation.margin1 + vertLocation.stride.Sum() / 2 - (beamHeight / 2 + vertOffset);
                    endPlateMarginTop = vertLocation.margin2 + vertLocation.stride.Sum() / 2 - (beamHeight / 2 - vertOffset);
                }
                //填了端板距上翼缘的距离，下边距由螺栓参数确定
                else if (endPlateMarginTop != double.MaxValue && endPlateMarginBottom == double.MaxValue)
                {
                    endPlateMarginBottom = vertLocation.length - beamHeight - endPlateMarginTop;
                }
                //填了端板距下翼缘的距离，上边距由螺栓参数确定
                else if (endPlateMarginTop == double.MaxValue && endPlateMarginBottom != double.MaxValue)
                {
                    endPlateMarginTop = vertLocation.length - beamHeight - endPlateMarginBottom;
                }
                //
                vertLocation.stride.Reverse();
                m_context.EndPlateBoltLocationVert = vertLocation;
            }
            //端板和螺栓的Front-Back方向定位
            {
                //默认值
                double subWidth = m_context.SubFrontPlane.Dist(m_context.SubBackPlane.Origin);
                if (subWidth <= 100)
                    subWidth = 100;
                double defaultBoltMargin = (subWidth - 60 ) / 2;
                double[] defaultBoltStride = new double[] { 80.0 };
                int defaultBoltCount = 2;
                //未设置板宽度
                horiLocation = new BoltLocation(defaultBoltMargin, defaultBoltMargin, defaultBoltCount, defaultBoltStride);
                horiLocation.centerMode = true;
                horiLocation.SetBoltLocation(boltMarginRight, boltMarginLeft, horiCount, horiStride, mainEndPlateProps.Width);
                m_context.EndPlateBoltLocationHori = horiLocation;
            }
            //折角参数
            sideChamferMode = ConvertToCutType((StiffChamferMode)m_paramsHandler.GetIntValue(ParamNames.SIDE_STIFFENER_CHAMFER_MODE));
            middleChamferMode = ConvertToCutType((StiffChamferMode)m_paramsHandler.GetIntValue(ParamNames.MIDDLE_STIFFENER_CHAMFER_MODE));
            sideChamferCutX = m_paramsHandler.GetDoubleValue(ParamNames.SIDE_STIFFENER_CHAMFER_CUTX);
            sideChamferCutY = m_paramsHandler.GetDoubleValue(ParamNames.SIDE_STIFFENER_CHAMFER_CUTY);
            SetCutYEqualCutX(ref sideChamferCutY, sideChamferCutX, sideChamferMode);
            middleChamferCutX = m_paramsHandler.GetDoubleValue(ParamNames.MIDDLE_STIFFENER_CHAMFER_CUTX);
            middleChamferCutY = m_paramsHandler.GetDoubleValue(ParamNames.MIDDLE_STIFFENER_CHAMFER_CUTY);
            SetCutYEqualCutX(ref middleChamferCutY, middleChamferCutX, middleChamferMode);
        }

        public void CreateCutterChamfer(Plane sidePlane1, Plane sidePlane2, Plane ctrlPlane, PartBase cutPlate,
            double cutX, double cutY, Define.CornerCutType cutType, double thickness = 100)
        {
            InvertCutType(ref cutType); //对板的的折角类型取反（round<->inv_round），得到切割体的折角
            if (cutType == Define.CornerCutType.CORNER_CUT_NONE)
                return;
            double _sin = Math.Sin(Util.VectorAngle(sidePlane1.Normal, sidePlane2.Normal));
            Plane sidePlane3 = sidePlane1.Move(cutY * _sin);
            Plane sidePlane4 = sidePlane2.Move(cutX * _sin);
            Plane topPlane = ctrlPlane.Move(thickness / 2);
            Plane bottomPlane = ctrlPlane.Move(-thickness / 2);
            Plane[] sidePlanes = { sidePlane1, sidePlane2, sidePlane3, sidePlane4 };
            PartBase cutObject = m_context.Comp.CreatePlateCutter(cutPlate, bottomPlane, topPlane, sidePlanes, true);
            if (cutObject == null)
                return;
            ChamferControl chamferControl = new ChamferControl(cutObject, sidePlanes);
            chamferControl.ChamferOnePoint(cutX, cutY, cutType, chamferControl.comparePointList[3]);
            cutObject.Flush();
        }

        public void InvertCutType(ref Define.CornerCutType cutType)
        {
            if (cutType == Define.CornerCutType.CORNER_CUT_ROUND)
            {
                cutType = Define.CornerCutType.CORNER_CUT_INV_ROUND;
                return;
            }
            if (cutType == Define.CornerCutType.CORNER_CUT_INV_ROUND)
            {
                cutType = Define.CornerCutType.CORNER_CUT_ROUND;
                return;
            }
        }

        public void SetCutYEqualCutX(ref double cuty, double cutx, Define.CornerCutType cutType)
        {
            if (cutType == Define.CornerCutType.CORNER_CUT_INV_ROUND || cutType == Define.CornerCutType.CORNER_CUT_ROUND)
                cuty = cutx;
        }

        public Define.CornerCutType ConvertToCutType(StiffChamferMode chamferMode)
        {
            switch (chamferMode)
            {
                case StiffChamferMode.CORNER_CUT_NONE:
                    return Define.CornerCutType.CORNER_CUT_NONE;
                case StiffChamferMode.CORNER_CUT_LINE:
                    return Define.CornerCutType.CORNER_CUT_LINE;
                case StiffChamferMode.CORNER_CUT_ROUND:
                    return Define.CornerCutType.CORNER_CUT_ROUND;
                case StiffChamferMode.CORNER_CUT_INV_ROUND:
                    return Define.CornerCutType.CORNER_CUT_INV_ROUND;
            }
            return Define.CornerCutType.CORNER_CUT_NONE;
        }

        private void SetPropsFollow(PlateProperties props1, PlateProperties props2)
        {
            Util.SetDefaultValue(ref props1.Thickness, props2.Thickness);
            Util.SetDefaultValue(ref props1.Width, props2.Width);
            Util.SetDefaultValue(ref props1.Height, props2.Height);
        }
    }
}
