﻿using System;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using TubeGusset_20_.Running;
using TargetComponentType = LoongCAD.STC.Object3D.ConnectionComponent;

namespace TubeGusset_20_.Main
{
    public class ComponentInstanceContext
    {
        public TargetComponentType TargetComp { get; private set; }

        public Beam MainPart { get; private set; } = null;
        public BeamPlanes MainPlanes { get; private set; } = null;
        public ParamsHandler ParamsHandler { get; set; } = null;

        #region 组件内部自定义
        /// <summary>
        /// 公用ParamNames的数据
        /// </summary>
        public PublicParamData PublicParamData;
        /// <summary>
        /// 焊缝处理
        /// </summary>
        public BuildWeld BuildWeld; 
        #endregion

        /// <summary>
        /// 利用传过来的TargetComponent初始化上下文
        /// 失败时返回null
        /// </summary>
        /// <param name="comp"></param>
        /// <returns></returns>
        public static ComponentInstanceContext CreatInstance(TargetComponentType comp)
        {
            var compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp))
                return compContext;
            compContext.Clear();
            return null;
        }

        private void Clear()
        {
            MainPart = null;
            MainPlanes = null;
        }

        /// <summary>
        /// 上下文的初始化，处理TargetComponent的信息，解析数据。
        /// 如果处理、解析失败，则返回false
        /// </summary>
        /// <param name="comp"></param>
        /// <returns></returns>
        private bool Initialize(TargetComponentType comp)
        {
            if (comp == null)
            {
                Console.WriteLine(@"comp is null");
                return false;
            }
            //把传过来的组件，放到上下文
            TargetComp = comp;
            
            //把目标组件的主构件尝试转化为梁
            if (!(comp.Main is Beam mainPart))
            {
                Console.WriteLine(@"comp.Main Conversion Beam failed");
                return false;
            }
            //赋值给组件的主要部分（一般情况是梁）
            MainPart = mainPart;
            //解析梁
            MainPlanes = BeamPlanes.CreateFromBeam(MainPart);

            return true;
        }


    }
}