﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using LoongCAD.STC.Plugins;
using LoongCAD.UI;
using LoongCAD.Util;

namespace TubeGusset_20_.Main
{
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {
        private PluginMain m_plugin; 
        public PluginMain Plugin { get => m_plugin; set => m_plugin = value; }
        private DObject m_weldPropsBatch = new DObject();
        private DObject m_rootUIProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;
        public override ComponentPlugin ReferencePlugin => m_refPlugin;

        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            OnPostInitialize();

        }

        private void OnPostInitialize()
        {
            #region 额外的配置
            gusset_bolts_bolt_basic_properties.BoltStandardChanged += gusset_bolts_bolt_assembly.AdaptBoltContents;
            brace_bolts_1_bolt_basic_properties.BoltStandardChanged += brace_bolts_1_bolt_assembly.AdaptBoltContents;
            brace_bolts_2_bolt_basic_properties.BoltStandardChanged += brace_bolts_2_bolt_assembly.AdaptBoltContents;
            brace_bolts_3_bolt_basic_properties.BoltStandardChanged += brace_bolts_3_bolt_assembly.AdaptBoltContents;
            #endregion
            #region 图形选项卡
            AddItemForImageComboBox(pic_select_gusset_plate_dimensions,3);
            #endregion
            #region 支柱螺栓下来选项卡配置
            AddItemForImageComboBox(brace_conn_select_connection_part_types, 5);
            AddItemForImageComboBox(brace_conn_select_number_of_connection_plate, 3);
            AddItemForImageComboBox(brace_conn_select_connection_plate_chamfer, 3);
            AddItemForImageComboBox(brace_conn_select_brace_is_notched_cut, 3);
            AddItemForImageComboBox(brace_conn_select_end_plate_type, 3);
            AddItemForKevValve(brace_conn_select_end_plate_chamfer_type,new List<string>() 
            {"默认","否","线","割挖","弧" });
            #endregion
            #region 节点板相关的下拉选项卡配置
            AddItemForImageComboBox(gusset_gusset_plate_shape_select,4);
            AddItemForImageComboBox(gusset_gusset_plate_position_on_brace_select,4);
            AddItemForImageComboBox(gusset_select_connection_or_clip_angles,7);
            AddItemForImageComboBox(gusset_select_clip_angle_orientation,3);
            #endregion
            #region 节点板螺栓的下拉选项卡配置
            var madeType = new List<string>() {"默认","工地","车间"};
            AddItemForKevValve(gusset_bolts_select_main_part_bolts_made_type,madeType);
            AddItemForKevValve(gusset_bolts_select_linked_part_bolts_made_type,madeType);
            AddItemForImageComboBox(gusset_bolts_select_clip_angle_attachment_type,5);
            AddItemForImageComboBox(gusset_bolts_select_bolts_on_gusset_plate,3);
            #endregion
            #region 支柱螺栓页相关的下拉选项卡配置
            AddItemForBolts(brace_bolts_1_vertical_bolts_select_measure_method);
            AddItemForBolts(brace_bolts_2_vertical_bolts_select_measure_method);
            AddItemForBolts(brace_bolts_3_vertical_bolts_select_measure_method);
            #endregion

            foreach(TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }
        }

        #region UI控件的工具函数
        private void AddItemForKevValve(KeyValueComboBox ui,List<string> cfg)
        {
            ui.ClearItems();
            for (int i = 0; i < cfg.Count; i++) {
                ui.AddItem(i, cfg[i]);
            }
            ui.UpdateItems();
        }

        private void AddItemForImageComboBox(ImageComboBox imageComboBox,int num)
        {
            imageComboBox.ClearItems();
            for(var i = 0; i< num; ++i) {
                imageComboBox.AddItem(i,i);
            }
            imageComboBox.UpdateItems();
        }

        private void AddItemForBolts(KeyValueComboBox ui)
        {
            ui.ClearItems();
            ui.AddItem(0,"默认");
            ui.AddItem(1,"顶部");
            ui.AddItem(2,"中间");
            ui.AddItem(3,"底部");
            ui.UpdateItems();
        }
        #endregion

        public override void ClearUI(Control root = null)
        {
            //相关螺栓刷新
            brace_bolts_1_bolt_basic_properties.RefreshControls();
            brace_bolts_2_bolt_basic_properties.RefreshControls();
            brace_bolts_3_bolt_basic_properties.RefreshControls();
            gusset_bolts_bolt_basic_properties.RefreshControls();
            base.ClearUI(root);
        }
        
        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps,defaultValues,ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }

        #region 焊缝相关
        /// <summary>
        /// 焊缝开关控件的回调
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_weld_props_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_rootUIProps, null);
        }


        /// <summary>
        /// 设置焊缝值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if(root == null)
            {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }
        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked)
            {
                retProps.CopyFrom(m_rootUIProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }


        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_rootUIProps.CopyFrom(srcProps);
        }

        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }

        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_rootUIProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_rootUIProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_rootUIProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_rootUIProps;

            return comp.UIProps;
        }
        #endregion
        #region 20号组件内部UI控件变换代码
        private void brace_conn_select_number_of_connection_plate_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(brace_conn_select_number_of_connection_plate.SelectedIndex == 2) {
                pictureBox_conn_num1.Visible = false;
                pictureBox_conn_num2.Visible = true;
                brace_conn_width_of_middle_end_plate.Enabled = true;
                brace_conn_middle_end_plate.Enabled = true;
            }
            else {
                pictureBox_conn_num2.Visible = false;
                pictureBox_conn_num1.Visible = true;
                brace_conn_width_of_middle_end_plate.Enabled = false;
                brace_conn_middle_end_plate.Enabled = false;
            }

        }

        private void imageComboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(brace_conn_select_connection_plate_chamfer.SelectedIndex == 2) {
                brace_conn_chamfer_horizontal_dimension.Enabled = true;
                brace_conn_chamfer_vertical_dimension.Enabled = true;
                checkBoxbrace_conn_chamfer_horizontal_dimension.Enabled = true;
                checkBoxbrace_conn_chamfer_vertical_dimension.Enabled = true;
            }
            else {
                brace_conn_chamfer_horizontal_dimension.Enabled = false;
                brace_conn_chamfer_vertical_dimension.Enabled = false;
                checkBoxbrace_conn_chamfer_horizontal_dimension.Enabled = false;
                checkBoxbrace_conn_chamfer_vertical_dimension.Enabled = false;
            }

        }

        private void brace_conn_select_connection_part_types_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(brace_conn_select_connection_part_types.SelectedIndex == 4)
            {
                brace_conn_tongue_plate.Enabled = true;
                brace_conn_cover_plate.Enabled = true;
            }
            else
            {
                brace_conn_tongue_plate.Enabled = false;
                brace_conn_cover_plate.Enabled = false;
            }
        }

        private void brace_conn_select_brace_is_notched_cut_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(brace_conn_select_brace_is_notched_cut.SelectedIndex == 2)
            {
                brace_conn_size_gap_between_brace_and_connection_plate.Enabled = true;
                checkBoxbrace_conn_size_gap_between_brace_and_connection_plate.Enabled = true;
                brace_conn_cut_in_bracing_length.Enabled = false;
                checkBoxbrace_conn_cut_in_bracing_length.Enabled = false;
                brace_conn_cut_in_bracing_width.Enabled = false;
                checkBoxbrace_conn_cut_in_bracing_width.Enabled = false;
            }
            else
            {
                brace_conn_size_gap_between_brace_and_connection_plate.Enabled = false;
                checkBoxbrace_conn_size_gap_between_brace_and_connection_plate.Enabled = false;
                brace_conn_cut_in_bracing_length.Enabled = true;
                checkBoxbrace_conn_cut_in_bracing_length.Enabled = true;
                brace_conn_cut_in_bracing_width.Enabled = true;
                checkBoxbrace_conn_cut_in_bracing_width.Enabled = true;
            }

        }

        private void brace_conn_select_end_plate_type_SelectedIndexChanged(object sender, EventArgs e)
        {

            if(brace_conn_select_end_plate_type.SelectedIndex == 2)
            {
                brace_conn_circular_end_plate_edge_distance_from_brace_outer_edge.Enabled = true;

                brace_conn_square_end_plate_left_edge_distance_from_brace_outer_edge.Enabled = false;
                brace_conn_square_end_plate_right_edge_distance_from_brace_outer_edge.Enabled = false;
                brace_conn_square_end_plate_top_edge_distance_from_brace_outer_edge.Enabled = false;
                checkBoxbrace_conn_square_end_plate_left_edge_distance_from_brace_outer_edge.Enabled = false;
                checkBoxbrace_conn_square_end_plate_right_edge_distance_from_brace_outer_edge.Enabled = false;
                checkBoxbrace_conn_square_end_plate_top_edge_distance_from_brace_outer_edge.Enabled = false;

                brace_conn_select_end_plate_chamfer_type.Enabled = false;
                brace_conn_end_plate_chamfer_horizontal_dist.Enabled = false;
                brace_conn_end_plate_chamfer_vertical_dist.Enabled = false;
                checkBoxbrace_conn_select_end_plate_chamfer_type.Enabled = false;
                checkBoxbrace_conn_end_plate_chamfer_horizontal_dist.Enabled = false;
                checkBoxbrace_conn_end_plate_chamfer_vertical_dist.Enabled = false;
            }
            else
            {
                brace_conn_circular_end_plate_edge_distance_from_brace_outer_edge.Enabled = false;
                brace_conn_square_end_plate_left_edge_distance_from_brace_outer_edge.Enabled = true;
                brace_conn_square_end_plate_right_edge_distance_from_brace_outer_edge.Enabled = true;
                brace_conn_square_end_plate_top_edge_distance_from_brace_outer_edge.Enabled = true;
                checkBoxbrace_conn_square_end_plate_left_edge_distance_from_brace_outer_edge.Enabled = true;
                checkBoxbrace_conn_square_end_plate_right_edge_distance_from_brace_outer_edge.Enabled = true;
                checkBoxbrace_conn_square_end_plate_top_edge_distance_from_brace_outer_edge.Enabled = true;

                brace_conn_select_end_plate_chamfer_type.Enabled = true;
                brace_conn_end_plate_chamfer_horizontal_dist.Enabled = true;
                brace_conn_end_plate_chamfer_vertical_dist.Enabled = true;
                checkBoxbrace_conn_select_end_plate_chamfer_type.Enabled = true;
                checkBoxbrace_conn_end_plate_chamfer_horizontal_dist.Enabled = true;
                checkBoxbrace_conn_end_plate_chamfer_vertical_dist.Enabled = true;
            }
        }

        private void pic_select_gusset_plate_dimensions_SelectedIndexChanged(object sender, EventArgs e)
        {
            //需要切割
            if(pic_select_gusset_plate_dimensions.SelectedIndex == 2)
            {
                pic_gusset_plate_cut_dimensions.Visible = true;
                checkBoxpic_gusset_plate_cut_dimensions.Visible = true;
                pic_gap_between_gusset_plate_and_main_part.Visible = false;
                checkBoxpic_gap_between_gusset_plate_and_main_part.Visible = false;
                gusset_horizontal_size_of_cut.Enabled = true;
                checkBoxgusset_horizontal_size_of_cut.Enabled = true;
                gusset_vertical_size_of_cut.Enabled = true;
                checkBoxgusset_vertical_size_of_cut.Enabled = true;
            }
            else
            {
                pic_gusset_plate_cut_dimensions.Visible = false;
                checkBoxpic_gusset_plate_cut_dimensions.Visible = false;
                pic_gap_between_gusset_plate_and_main_part.Visible = true;
                checkBoxpic_gap_between_gusset_plate_and_main_part.Visible = true;
                gusset_horizontal_size_of_cut.Enabled = false;
                checkBoxgusset_horizontal_size_of_cut.Enabled = false;
                gusset_vertical_size_of_cut.Enabled = false;
                checkBoxgusset_vertical_size_of_cut.Enabled = false;
            }
        }
        #endregion
    }
}
