﻿using System;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.STC.Object3D;
using TubeGusset_20_.Main;
using TubeGusset_20_.Tool;

namespace TubeGusset_20_.Running
{
    
    public class BuildBraceBolts : RunningBaseClass
    {
        public BuildBraceBolts(ComponentInstanceContext context, ParserSubPart subPart, ParserBraceBolts braceBolts, BuildGussetPlate gussetPlate, BuildConnPart connPart) : base(context)
        {
            SubPart = subPart;
            BraceBolts = braceBolts;
            GussetPlate = gussetPlate;
            ConnPart = connPart;
        }
        public List<BoltGroup> BoltGroups { get; set; } = new List<BoltGroup>();

        private readonly ParserSubPart SubPart;
        private ParserBraceBolts BraceBolts;
        private BuildGussetPlate GussetPlate;
        private BuildConnPart ConnPart;

        public override bool Execute()
        {
            
            var subs = SubPart.SubData;
            for (int i = 0; i < subs.Count; ++i)
            {
                var index = i % 3;
                BuildBoltsWith(subs[i],index,i);
            }
            return true;
        }


        /// <summary>
        /// 创建螺栓组
        /// </summary>
        /// <param name="subData"></param>
        /// <param name="index"></param>
        /// <param name="subIndex"></param>
        private void BuildBoltsWith(SubData subData, int index,int subIndex)
        {
            //var plate = m_context.BuildGussetPlate.GussetPlate;
            var plate = GussetPlate.GussetPlate;
            if (plate == null) {
                return;
            }
            var param = BraceBolts.ParamDataList[index];
            //数目为0时，不创建支撑处的螺栓
            if (param.HorBoltsNum == 0 || param.VerBoltsNum == 0) {
                return ;
            }
            var firstCtrlPoint = PointHelper.Midpoint(subData.TopConnFarAlignPoint, subData.BtmConnFarAlignPoint);
            var lastCtrlPoint = PointHelper.Move(firstCtrlPoint, subData.NearToFarLine, 100);
            var target = m_context.TargetComp;
            var subList= new List<PartBase>();
            var connFront = ConnPart.ConnPlatesFront.ElementAtOrDefault(subIndex);
            var connBack= ConnPart.ConnPlatesBack.ElementAtOrDefault(subIndex);
            if (connFront != null) {
                subList.Add(connFront);
            }
            if (connBack!= null) {
                subList.Add(connBack);
            }

            if (subList.Count == 0)
            {
                return;
            }
            var boltGroup = target.CreateBoltGroup
                (plate, subList.ToArray(), firstCtrlPoint, lastCtrlPoint, true);
            var cfg = BraceBolts;
            cfg.SetUpBoltGroup(boltGroup,index);
            var maxThickness = Math.Max(m_pub.CoverPlate.Thickness, m_pub.ConnPartConnPlate.Thickness);
            //boltGroup.CutLength = (m_pub.GussetPlatePro.Thickness + maxThickness*subList.Count) * 2 + 1;
            boltGroup.CutLength = 120;
            boltGroup.Refresh();
            BoltGroups.Add(boltGroup);
            if (m_pub.SelectConnPartType == 2)
            {
                var boltGroup2 = target.CreateBoltGroup
                (subData.Beam, subList.ToArray(), firstCtrlPoint, lastCtrlPoint, true);
                cfg.SetUpBoltGroup(boltGroup2,index,false);
                boltGroup2.CutLength = 120;
                boltGroup2.HeadOffsetX = m_pub.pic_distance_between_the_gusset_plate_and_brace_part + 40.0d;
                boltGroup2.Refresh();
                BoltGroups.Add(boltGroup2);
            }else if (m_pub.SelectConnPartType == 4)
            {
                var tonguePlate = ConnPart.TonguePlate[subIndex];
                var boltGroup3 = target.CreateBoltGroup
                    (tonguePlate,subList.ToArray(),firstCtrlPoint,lastCtrlPoint,true);
                cfg.SetUpBoltGroup(boltGroup3,index,false);
                boltGroup3.CutLength = 120;
                boltGroup3.HeadOffsetX = m_pub.pic_distance_between_the_gusset_plate_and_brace_part + 40.0d;
                boltGroup3.Refresh();
                BoltGroups.Add(boltGroup3);
            }
        }
    }
}