﻿using System.Collections.Generic;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using TubeGusset_20_.Main;
using TubeGusset_20_.Tool;

namespace TubeGusset_20_.Running
{
    /// <summary>
    /// 处理螺栓开槽
    /// </summary>
    public class BuildBraceNotch : RunningBaseClass
    {
        public BuildBraceNotch(ComponentInstanceContext context, ParserSubPart subPart) : base(context)
        {
            SubPart = subPart;
        }

        private readonly ParserSubPart SubPart;

        private Plane LandingPl;

        public override bool Execute()
        {
            var off = m_pub.DistOfConnAndGusset + m_pub.ConnPartConnPlate.Thickness;
            LandingPl = SubPart.NearBuildPlane.Move(off);
            ExecuteAlignment(); 
            return true;
        }
        
        /// <summary>
        /// 执行支撑对齐操作
        /// </summary>
        private void ExecuteAlignment()
        {
            foreach (var sub in SubPart.SubData)
            {
                if (m_pub.SelectConnPartType == 3)
                {
                    m_context.TargetComp.CreateAlignPlane(sub.Beam, sub.GstNearAlignPlane, true);
                    HandlerBackCut(sub);
                    HandlerFrontCut(sub);
                }
                else if (m_pub.SelectConnPartType == 4)
                {
                    m_context.TargetComp.CreateAlignPlane
                        (sub.Beam, sub.FinalAlignPlane, true);
                }
                else
                {
                    m_context.TargetComp.CreateAlignPlane(sub.Beam, sub.FinalAlignPlane, true);
                }
            }
        }

        private void HandlerFrontCut(SubData sub)
        {
            var topPl = new Plane(LandingPl.Origin,-LandingPl.Normal);
            var btmPl = PlaneHelper.GetSameNormalPlane(sub.BeamPlanes, PlaneHelper.BOUND_PLANE_TBFB, -topPl.Normal);
            if (btmPl == null) { return; }
            var midPl = PlaneHelper.GetMidPlane(sub.TopPlane, sub.BottomPlane);
            var sidePl = new List<Plane>();
            sidePl.Add(sub.GstNearAlignPlane.Move(-1));
            sidePl.Add(midPl.Move(sub.BoltGroupVerLength/2));
            sidePl.Add(sub.GstNearAlignPlane.Move(sub.BoltGroupHorLength));
            sidePl.Add(midPl.Move(-sub.BoltGroupVerLength/2));
            
            var cut= m_context.TargetComp.CreatePlateCutter(sub.Beam,topPl,btmPl.Move(10),sidePl.ToArray(),true);
            if (cut is SteelPlate cutPlate)
            {
                var p1 = cutPlate.CtrlPoints[1];
                var p2 = cutPlate.CtrlPoints[2];
                HandlerCtrlPoint(p1);
                HandlerCtrlPoint(p2);
            }

            void HandlerCtrlPoint(CtrlPoint p1)
            {
                if (p1 is ChamferCtrlPoint p)
                {
                    p.CutType = Define.CornerCutType.CORNER_CUT_ROUND;
                    p.CutX = sub.BoltSize;
                    p.CutY = sub.BoltSize;
                    p.Refresh();
                }
            }
        }

        private void HandlerBackCut(SubData sub)
        {
            var topPl = new Plane(LandingPl);
            var btmPl = PlaneHelper.GetSameNormalPlane(sub.BeamPlanes, PlaneHelper.BOUND_PLANE_TBFB, -topPl.Normal);
            if(btmPl == null){return;}
            var sidePl = new List<Plane>();
            sidePl.Add(sub.TopPlane.Move(1));
            sidePl.Add(sub.GstNearAlignPlane.Move(-1));
            sidePl.Add(sub.BottomPlane.Move(1));
            sidePl.Add(sub.FinalAlignPlane);
            m_context.TargetComp.CreatePlateCutter(sub.Beam, topPl, btmPl.Move(1),sidePl.ToArray(), true);
        }
    }
}