﻿using System.Collections.Generic;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using TubeGusset_20_.Main;
using TubeGusset_20_.Tool;
using TargetComponentType = LoongCAD.STC.Object3D.ConnectionComponent;

namespace TubeGusset_20_.Running
{
    public class BuildConnPart:RunningBaseClass
    {
        public BuildConnPart(ComponentInstanceContext context, ParserSubPart subPart) : base(context)
        {
            SubPart = subPart;
        }

        private readonly ParserSubPart SubPart;

        private readonly ParamData AllParamData = new ParamData();

        private class ParamData
        {
            /// <summary>
            /// 支撑在连接部分距离
            /// </summary>
            public double BraceLengthOnConnPart;
            /// <summary>
            /// 中间端板的宽度
            /// </summary>
            public double MidEndPlateWight;
            /// <summary>
            /// 支撑是否被切割
            /// </summary>
            public int SelectBraceIsCut;
            /// <summary>
            /// 连接部分和支撑部分的切割间隙
            /// </summary>
            public double CutGapOfConnPartBrace;
            /// <summary>
            /// 支撑切割的额外宽度
            /// </summary>
            public double CutInBraceWidth;
            /// <summary>
            /// 支撑切割的额外长度
            /// </summary>
            public double CutInBraceLength;
            

            /// <summary>
            /// 是否创建折角
            /// </summary>
            public int SelectCanChamfer;
            /// <summary>
            /// 折角的垂直距离
            /// </summary>
            public double ChamferVerticalDist;
            /// <summary>
            /// 折角的水平距离
            /// </summary>
            public double ChamferHorizontalDist;

            /// <summary>
            /// 节点板的顶侧外边缘缩减值
            /// </summary>
            public double EndPlateTopOutEdgeDist;
            /// <summary>
            /// 节点板的底侧外边缘缩减值
            /// </summary>
            public double EndPlateBtmOutEdgeDist;
            /// <summary>
            /// 节点板的外边缘缩减值
            /// </summary>
            public double EndPlateOutEdgeDist;
            /// <summary>
            /// 圆形端板的边缘缩减值
            /// </summary>
            public double EndPlateCircularDist;

            /// <summary>
            /// 端板形状
            /// </summary>
            public int SelectEndPlateType;
            /// <summary>
            /// 端板的折角类型
            /// </summary>
            public int SelectEndPlateChamferType;
            /// <summary>
            /// 端板折角的水平值
            /// </summary>
            public double EndPlateChamferHorDist;
            /// <summary>
            /// 端板折角的垂直值
            /// </summary>
            public double EndPlateChamferVerDist;
        }

        /// <summary>
        /// 连接板边缘部分到gstFar的距离
        /// </summary>
        private double DistToGstFar;
        /// <summary>
        /// 连接板是否需要创建倒角
        /// </summary>
        private bool CanChamferConn;
        /// <summary>
        /// 连接板数组，下标对应支撑
        /// </summary>
        public readonly List<SteelPlate> ConnPlatesFront = new List<SteelPlate>();
        /// <summary>
        /// 连接板数组，后侧
        /// </summary>
        public readonly List<SteelPlate> ConnPlatesBack= new List<SteelPlate>();

        private readonly List<SteelPlate> EndPlatesFront = new List<SteelPlate>();
        private readonly List<SteelPlate> EndPlatesBack = new List<SteelPlate>();
        private readonly List<SteelPlate> MidEndPlates = new List<SteelPlate>();
        public readonly List<SteelPlate> TonguePlate = new List<SteelPlate>();
        /// <summary>
        /// 是否有输入width，为真为有输入
        /// </summary>
        private bool m_hasWidth;
        /// <summary>
        /// 是否有输入height,为真为有输入
        /// </summary>
        private bool m_hasHeight;

        /// <summary>
        /// 宽度偏移值
        /// </summary>
        private double m_widthOff;
        /// <summary>
        /// 高度偏移值
        /// </summary>
        private double m_heightOff;

        /// <summary>
        /// 前侧端板的构建面
        /// </summary>
        private Plane m_buildFrontPlane;
        /// <summary>
        /// 后侧端板的构建面
        /// </summary>
        private Plane m_buildBackPlane;


        public override bool Init()
        {
            var handler = m_context.ParamsHandler;
            AllParamData.BraceLengthOnConnPart = handler.GetDoubleValue
                (ParamNames.PIC_BRACE_LENGTH_ON_THE_CONNECTION_PART);
            AllParamData.SelectBraceIsCut = handler.GetIntValue(ParamNames.BRACE_CONN_SELECT_BRACE_IS_NOTCHED_CUT);
            AllParamData.CutGapOfConnPartBrace = handler.GetDoubleValue(ParamNames.BRACE_CONN_SIZE_GAP_BETWEEN_BRACE_AND_CONNECTION_PLATE);
            AllParamData.CutInBraceWidth = handler.GetDoubleValue(ParamNames.BRACE_CONN_CUT_IN_BRACING_WIDTH);
            AllParamData.CutInBraceLength = handler.GetDoubleValue(ParamNames.BRACE_CONN_CUT_IN_BRACING_LENGTH);
            AllParamData.SelectCanChamfer = handler.GetIntValue(ParamNames.BRACE_CONN_SELECT_CONNECTION_PLATE_CHAMFER);
            CanChamferConn = AllParamData.SelectCanChamfer == 2;
            AllParamData.ChamferVerticalDist = handler.GetDoubleValue(ParamNames.BRACE_CONN_CHAMFER_VERTICAL_DIMENSION);
            AllParamData.ChamferHorizontalDist= handler.GetDoubleValue(ParamNames.BRACE_CONN_CHAMFER_HORIZONTAL_DIMENSION);

            #region 端板相关
            AllParamData.SelectEndPlateType = handler.GetIntValue(ParamNames.BRACE_CONN_SELECT_END_PLATE_TYPE);
            AllParamData.SelectEndPlateChamferType = handler.GetIntValue(ParamNames.BRACE_CONN_SELECT_END_PLATE_CHAMFER_TYPE);
            
            AllParamData.EndPlateOutEdgeDist = handler.GetDoubleValue(ParamNames.BRACE_CONN_SQUARE_END_PLATE_TOP_EDGE_DISTANCE_FROM_BRACE_OUTER_EDGE);
            AllParamData.EndPlateTopOutEdgeDist = handler.GetDoubleValue(ParamNames.BRACE_CONN_SQUARE_END_PLATE_LEFT_EDGE_DISTANCE_FROM_BRACE_OUTER_EDGE);
            AllParamData.EndPlateBtmOutEdgeDist = handler.GetDoubleValue(ParamNames.BRACE_CONN_SQUARE_END_PLATE_RIGHT_EDGE_DISTANCE_FROM_BRACE_OUTER_EDGE);
            
            AllParamData.EndPlateChamferHorDist = handler.GetDoubleValue(ParamNames.BRACE_CONN_END_PLATE_CHAMFER_HORIZONTAL_DIST);
            AllParamData.EndPlateChamferVerDist = handler.GetDoubleValue(ParamNames.BRACE_CONN_END_PLATE_CHAMFER_VERTICAL_DIST);
            
            AllParamData.EndPlateCircularDist = handler.GetDoubleValue(ParamNames.BRACE_CONN_CIRCULAR_END_PLATE_EDGE_DISTANCE_FROM_BRACE_OUTER_EDGE);

            AllParamData.MidEndPlateWight = handler.GetDoubleValue(ParamNames.BRACE_CONN_WIDTH_OF_MIDDLE_END_PLATE);

            if (AllParamData.SelectEndPlateType == 2) {//圆型节点板
                AllParamData.EndPlateOutEdgeDist = 0.0d;
                AllParamData.EndPlateTopOutEdgeDist = 0.0d;
                AllParamData.EndPlateBtmOutEdgeDist = 0.0d;
                AllParamData.EndPlateChamferHorDist = 0.0d;
                AllParamData.EndPlateChamferVerDist = 0.0d;
            }
            #endregion

            if (AllParamData.SelectBraceIsCut == 2)
            {
                AllParamData.CutInBraceWidth = 0.0d;
                AllParamData.CutInBraceLength = 0.0d;
            }
            SetDistToGstFar();
            InitEndPlateWightHeight();
            return true;
        }

        private void SetDistToGstFar()
        {
            DistToGstFar = AllParamData.BraceLengthOnConnPart +
                           m_pub.pic_distance_between_the_gusset_plate_and_brace_part+
                           m_pub.EndPlate.Thickness;
        }

        public override bool Prepare()
        {
            PrepareEndPlateFrontBackPlane();
            return true;
        }

        public override bool Execute()
        {
            if (m_pub.SelectConnPartType != 4)
            {
                CreateConnPartNormal();
            }
            else
            {
                CreateConnPartType4();
            }
            return true;
        }

        private void CreateConnPartType4() {
            var subData = SubPart.SubData;
            var landingPl = SubPart.CentreBuildPlane;
            foreach (var sub in subData)
            {
                CreateCoverPlate(sub,landingPl);
                CreateTonguePlate(sub,landingPl);
                HandlerEndPlate(sub);
            }
        }

        /// <summary>
        /// 创建舌柱
        /// </summary>
        /// <param name="sub"></param>
        /// <param name="landingPl"></param>
        private void CreateTonguePlate(SubData sub, Plane landingPl)
        {
            var planes = new List<Plane>();
            var wight = sub.GstPartAreaLength + DistToGstFar;
            var height = sub.TopConnNearAlignPoint.Dist(sub.BtmConnNearAlignPoint);
            if (!Equals(m_pub.TonguePlate.Height, double.MaxValue))
            {
                height = m_pub.TonguePlate.Height;
            }
            var midPl = PlaneHelper.GetMidPlane(sub.TopConnPartPlane, sub.BtmConnPartPlane);
            var buildRefPl = sub.GstFarAlignPlane.Move(m_pub.pic_distance_between_the_gusset_plate_and_brace_part);
            var buildFarRefPl = buildRefPl.Move(wight);
            planes.Add(buildRefPl);
            planes.Add(midPl.Move(height/2));
            if (AllParamData.SelectBraceIsCut == 2)
            {
                planes.Add(buildFarRefPl);
                planes.Add(sub.TopPlane.Move(AllParamData.CutGapOfConnPartBrace));
                planes.Add(sub.FinalAlignPlane);
                planes.Add(sub.BottomPlane.Move(AllParamData.CutGapOfConnPartBrace));
                planes.Add(buildFarRefPl);
            }
            else
            {
                planes.Add(buildFarRefPl);
            }
            planes.Add(midPl.Move(-height/2));

            var tonguePlate = m_context.TargetComp.CreateSteelPlate(PlaneHelper.GetLandingPoint(landingPl, planes).ToArray(),
                m_pub.TonguePlate.Thickness, Define.ObjectSide.SIDE_MIDDLE, true);
            TonguePlate.Add(tonguePlate);
            m_pub.TonguePlate.ApplyToPart(tonguePlate);
            m_context.BuildWeld.BuildWeldForTonguePlate(SubPart,tonguePlate,sub,buildFarRefPl);

        }

        /// <summary>
        /// 创建盖板
        /// </summary>
        /// <param name="sub"></param>
        /// <param name="landingPl"></param>
        private void CreateCoverPlate(SubData sub,Plane landingPl) {
            var planes = new List<Plane>();
            var wight = sub.GstPartAreaLength*2 + m_pub.pic_distance_between_the_gusset_plate_and_brace_part;
            if (!Equals(m_pub.CoverPlate.Width, double.MaxValue))
            {
                wight = m_pub.CoverPlate.Width;
            }
            var height = sub.TopConnNearAlignPoint.Dist(sub.BtmConnNearAlignPoint);
            if (!Equals(m_pub.CoverPlate.Height, double.MaxValue))
            {
                height = m_pub.CoverPlate.Height;
            }
            
            var midPl = PlaneHelper.GetMidPlane(sub.TopConnPartPlane, sub.BtmConnPartPlane);
            planes.Add(sub.GstNearAlignPlane);
            planes.Add(midPl.Move(height/2));
            planes.Add(sub.GstNearAlignPlane.Move(wight));
            planes.Add(midPl.Move(-height/2));
            if (CanChamferConn) {
                var topChamferPl = CalculateTopChamferPlane(sub);
                planes.Insert(1,topChamferPl);
                var btmChamferPl = CalculateBtmChamferPlane(sub);
                planes.Add(btmChamferPl);
            }

            var cover1 =  m_context.TargetComp.CreateSteelPlate(PlaneHelper.GetLandingPoint(landingPl, planes).ToArray(),
                m_pub.CoverPlate.Thickness, Define.ObjectSide.SIDE_MIDDLE, true);
            ConnPlatesFront.Add(cover1);
            HandlerPlate1(cover1);
            m_pub.CoverPlate.ApplyToPart(cover1);
            if (m_pub.SelectConnPlateNum == 2)
            {
                var cover2 =  m_context.TargetComp.CreateSteelPlate(PlaneHelper.GetLandingPoint(landingPl, planes).ToArray(),
                    m_pub.CoverPlate.Thickness, Define.ObjectSide.SIDE_MIDDLE, true);
                ConnPlatesBack.Add(cover2);
                HandlerPlate2(cover2);
                m_pub.CoverPlate.ApplyToPart(cover2);
            }
        }

        private void CreateConnPartNormal()
        {
            var subData = SubPart.SubData;
            var landingPl = SubPart.CentreBuildPlane;
            foreach (var sub in subData)
            {
                var planes = new List<Plane>();
                HandlerPlaneCut(planes, sub);
                var points = PlaneHelper.GetLandingPoint(landingPl,planes);
                var plate1 =m_context.TargetComp.CreateSteelPlate(points.ToArray(), m_pub.ConnPartConnPlate.Thickness, Define.ObjectSide.SIDE_MIDDLE, true);
                m_pub.ConnPartConnPlate.ApplyToPart(plate1);
                ConnPlatesFront.Add(plate1);
                HandlerPlate1(plate1);
                SteelPlate plate2 = null;
                if (m_pub.SelectConnPlateNum == 2)
                {
                    plate2 = m_context.TargetComp.CreateSteelPlate(points.ToArray(), m_pub.ConnPartConnPlate.Thickness, Define.ObjectSide.SIDE_MIDDLE, true);
                    m_pub.ConnPartConnPlate.ApplyToPart(plate2);
                    ConnPlatesBack.Add(plate2);
                    HandlerPlate2(plate2);
                }

                var farBuild = GetFarBuildPl(sub);
                var farDist = sub.FinalAlignPlane.SignedDist(farBuild.Origin);
                if (farDist > 0) {
                    m_context.BuildWeld.BuildWeldForConnPartInBraceNoCut(SubPart,plate1,plate2,sub,farDist);
                } 
                HandlerBraceCut(sub);
                HandlerEndPlate(sub);
                if (m_pub.SelectConnPartType == 0 || m_pub.SelectConnPartType == 1 || m_pub.SelectConnPartType == 2)
                {
                    HandlerMidEndPlate(sub);
                }
                
                
            }
            PlateApplyToPart();
        }

        /// <summary>
        /// 处理中间端板
        /// </summary>
        /// <param name="sub"></param>
        private void HandlerMidEndPlate(SubData sub)
        {
            if (m_pub.SelectConnPlateNum != 2)
                return;
            var planes = new List<Plane>();
            //var landingPl = sub.GstFarAlignPlane.Move(DistToGstFar +m_pub.MiddleEndPlate.Thickness/2);
            var off = AllParamData.BraceLengthOnConnPart + m_pub.MiddleEndPlate.Thickness/2;
            var landingPl = sub.FinalAlignPlane.Move(off);
            
            var offWight = m_pub.DistOfConnAndGusset + AllParamData.MidEndPlateWight ;
            var offHeight = 0.0d;
            if (!Equals(m_pub.MiddleEndPlate.Height, double.MaxValue)) {
                offHeight = m_pub.MiddleEndPlate.Height/2 - sub.TopBottomDist/2;
            }
            planes.Add(sub.TopPlane.Move(offHeight));
            planes.Add(SubPart.NearBuildPlane.Move(offWight));
            planes.Add(sub.BottomPlane.Move(offHeight));
            planes.Add(SubPart.FarBuildPlane.Move(offWight));
            var points = PlaneHelper.GetLandingPoint(landingPl, planes);
            var midEndPlate = m_context.TargetComp.CreateSteelPlate(points.ToArray(),m_pub.MiddleEndPlate.Thickness,Define.ObjectSide.SIDE_MIDDLE,true);
            MidEndPlates.Add(midEndPlate);
            m_context.BuildWeld.BuildWeldForMidEndPlate(midEndPlate, sub);
        }


        /// <summary>
        /// 处理端板
        /// </summary>
        /// <param name="sub"></param>
        private void HandlerEndPlate(SubData sub)
        {
            var target = m_context.TargetComp;
            var landingPl = sub.FinalAlignPlane.Move(-m_pub.EndPlate.Thickness/2);
            var midPl = PlaneHelper.GetMidPlane(sub.TopPlane, sub.BottomPlane);
            //拷贝准备阶段的两侧端板的前后参考面
            var buildFrontPlane = new Plane(m_buildFrontPlane);
            var buildBackPlane = new Plane(m_buildBackPlane);
            //构建端板的前后out面
            var subOutPlane1 = PlaneHelper.GetSameNormalPlane(sub.BeamPlanes, PlaneHelper.BOUND_PLANE_TBFB, buildFrontPlane.Normal);
            var buildOutPlane1 = (m_hasWidth && AllParamData.SelectEndPlateType !=2)
                ? buildFrontPlane.Move(m_widthOff)
                : subOutPlane1;
            var subOutPlane2 = PlaneHelper.GetSameNormalPlane(sub.BeamPlanes, PlaneHelper.BOUND_PLANE_TBFB, buildBackPlane.Normal);
            var buildOutPlane2 = (m_hasWidth && AllParamData.SelectEndPlateType !=2)
                ? buildBackPlane.Move(m_widthOff)
                : subOutPlane2;

            if (buildOutPlane1 == null || buildOutPlane2 == null) {return;}
            
            //构建端板的顶底边界面
            var buildTopPlane = m_hasHeight ? midPl.Move(m_heightOff) : sub.TopPlane;
            var buildBtmPlane = m_hasHeight ? midPl.Move(-m_heightOff) : sub.BottomPlane;
            //NOTE 处理由于连接板没有插入次零件时，端板应该只创建一个的情况
            var thickness = m_pub.EndPlate.Thickness;
            var bOneEndPlate = AllParamData.BraceLengthOnConnPart <= -thickness;
            
            //处理圆型端板
            if (AllParamData.SelectEndPlateType == 2)
            {
                var cirOff = AllParamData.EndPlateCircularDist;
                var horR = PlaneHelper.GetTwoPlaneDist(buildOutPlane1, buildOutPlane2)/2 - cirOff;
                var verR = PlaneHelper.GetTwoPlaneDist(sub.TopPlane, sub.BottomPlane)/2 - cirOff;
                var planes= new List<Plane>();
                planes.Add(buildOutPlane1.Move(-cirOff));
                planes.Add(sub.TopPlane.Move(-cirOff));
                planes.Add(buildOutPlane2.Move(-cirOff));
                planes.Add(sub.BottomPlane.Move(-cirOff));
                var pts = PlaneHelper.GetLandingPoint(landingPl, planes);
                
                //TODO 存在被切割面完全切割导致不存在的端板,不考虑圆弧形焊缝
                if (bOneEndPlate)
                {
                    CreateCircularEndPlate(target, subOutPlane1, buildFrontPlane, pts, 
                        thickness, horR, verR, EndPlatesFront,false);
                }
                else
                {
                    CreateCircularEndPlate(target, subOutPlane1, buildFrontPlane, pts, 
                        thickness, horR, verR, EndPlatesFront);
                    CreateCircularEndPlate(target, subOutPlane2, buildBackPlane, pts, 
                        thickness, horR, verR, EndPlatesBack);
                }
                return;
            }
            
            //处理边缘的额外缩减值
            buildTopPlane.MoveThis(-AllParamData.EndPlateTopOutEdgeDist);
            buildBtmPlane.MoveThis(-AllParamData.EndPlateBtmOutEdgeDist);
            buildOutPlane1.MoveThis(-AllParamData.EndPlateOutEdgeDist);
            buildOutPlane2.MoveThis(-AllParamData.EndPlateOutEdgeDist);
            

            SteelPlate endFront = null;
            SteelPlate endBack = null;
            if (bOneEndPlate)
            {
                var endPlatesFront = EndPlatesFront;
                endFront = CreateEndPlate(buildTopPlane, buildOutPlane2, buildBtmPlane, buildOutPlane1, subOutPlane1,landingPl, thickness, endPlatesFront);
                HandlerOneEndPlatePoint(endFront);
            }
            else
            {
                var endPlatesFront = EndPlatesFront;
                endFront = CreateEndPlate(buildTopPlane, buildFrontPlane, buildBtmPlane, buildOutPlane1, subOutPlane1,landingPl, thickness, endPlatesFront);
                var endPlatesBack = EndPlatesBack;
                endBack = CreateEndPlate(buildTopPlane, buildBackPlane, buildBtmPlane, buildOutPlane2, subOutPlane2, landingPl, thickness, endPlatesBack);
                HandlerTwoEndPlatePoint(endFront,endBack);
            }
            //NOTE 处理焊缝位置,直接那板的实际控制点来创建焊缝，暂不考虑带弧形的焊缝
            m_context.BuildWeld.BuildWeldForEndPlate(endFront,endBack,sub,buildFrontPlane,buildBackPlane,landingPl);
        }

        private static void CreateCircularEndPlate(TargetComponentType target, Plane subOutPlane, Plane cutPlane,
            List<Vec3> pts,
            double thickness, double horR, double verR, List<SteelPlate> endPlates, bool canCut = true)
        {
            bool bCanBuild = subOutPlane.SignedDist(cutPlane.Origin) < 0.0d;
            if (bCanBuild)
            {
                var endPlateFront = CreateCircularEndPlate(target, pts, thickness, horR, verR);
                if (canCut)
                {
                    target.CreateCutPlane(endPlateFront,cutPlane, true).Flush();
                }
                endPlates.Add(endPlateFront);
            }
        }

        /// <summary>
        /// 创建圆形端板
        /// </summary>
        /// <param name="target"></param>
        /// <param name="pts"></param>
        /// <param name="thickness"></param>
        /// <param name="horR"></param>
        /// <param name="verR"></param>
        /// <returns></returns>
        private static SteelPlate CreateCircularEndPlate(TargetComponentType target, List<Vec3> pts, double thickness, double horR,
            double verR)
        {
            var endPlate = target.CreateSteelPlate(pts.ToArray(), thickness,Define.ObjectSide.SIDE_MIDDLE, true);
            foreach (var point in endPlate.CtrlPoints)
            {
                if (!(point is ChamferCtrlPoint pt)) continue;
                pt.CutType = Define.CornerCutType.CORNER_CUT_ROUND;
                pt.CutX = horR;
                pt.CutY = verR;
                pt.Flush();
            }
            return endPlate;
        }

        /// <summary>
        /// 根据方形控制点创建端板,
        /// 当构建端板的内侧面位于，次零件相对外侧面的外侧时，不创建。
        /// (端板应只处于次零件内部)
        /// </summary>
        /// <param name="buildTopPlane">相对于当前次零件，构建端板时的顶面</param>
        /// <param name="buildInPlane">相对于当前次零件，构建端板靠内侧的面</param>
        /// <param name="buildBtmPlane">相对于当前次零件，构建端板时的底面</param>
        /// <param name="buildOutPlane">相对于当前次零件，构建端板靠外侧的面</param>
        /// <param name="subOutPlane">相对于当前端板，次零件的外侧面</param>
        /// <param name="landingPl">控制点的构建平面</param>
        /// <param name="thickness">端板厚度</param>
        /// <param name="endPlates">端板的汇总列表</param>
        /// <returns>不创建或者创建失败时返回null</returns>
        private SteelPlate CreateEndPlate(Plane buildTopPlane, Plane buildInPlane, Plane buildBtmPlane, Plane buildOutPlane,
            Plane subOutPlane,Plane landingPl, double thickness, List<SteelPlate> endPlates)
        {
            var pls = new List<Plane> 
                {buildTopPlane,buildInPlane,buildBtmPlane,buildOutPlane};
            bool bCanBuild = subOutPlane.SignedDist(buildInPlane.Origin) < 0.0d;
            SteelPlate endPlate = null;
            if (bCanBuild) {
                endPlate = m_context.TargetComp.CreateSteelPlate(PlaneHelper.GetLandingPoint(landingPl, pls).ToArray(), 
                    thickness,Define.ObjectSide.SIDE_MIDDLE, true);
                endPlates.Add(endPlate);
            }
            return endPlate;
        }

        /// <summary>
        /// 准备前后端板的构建面
        /// </summary>
        private void PrepareEndPlateFrontBackPlane()
        {
            m_buildFrontPlane = SubPart.NearBuildPlane.Move(m_pub.ConnPartConnPlate.Thickness + m_pub.DistOfConnAndGusset);
            m_buildBackPlane = m_buildFrontPlane.Move(-m_pub.ConnPartConnPlate.Thickness);
            m_buildBackPlane = new Plane(m_buildBackPlane.Origin, -m_buildBackPlane.Normal);
            if (m_pub.SelectConnPlateNum == 2) {
                m_buildBackPlane.MoveThis(m_pub.DistOfConnAndGusset*2 +m_pub.ConnPartConnPlate.Thickness + m_pub.GussetPlatePro.Thickness);   
            }
            if (m_pub.SelectConnPartType == 4) {
                m_buildFrontPlane = SubPart.CentreBuildPlane.Move(m_pub.TonguePlate.Thickness / 2);
                m_buildBackPlane = SubPart.CentreBuildPlane.Move(-m_pub.TonguePlate.Thickness / 2);
                m_buildBackPlane = new Plane(m_buildBackPlane.Origin, -m_buildBackPlane.Normal);
            }
        }

        private void PlateApplyToPart()
        {
            foreach (var front in EndPlatesFront) {
                m_pub.EndPlate.ApplyToPart(front);
            }
            foreach (var back in EndPlatesBack) {
                m_pub.EndPlate.ApplyToPart(back);
            }
            foreach (var mid in MidEndPlates) {
                m_pub.MiddleEndPlate.ApplyToPart(mid);
            }
        }

        private void InitEndPlateWightHeight()
        {
            m_widthOff = 0.0d;
            m_heightOff = 0.0d;
            m_hasWidth = !Equals(m_pub.EndPlate.Width, double.MaxValue);
            m_hasHeight= !Equals(m_pub.EndPlate.Height, double.MaxValue);
            if (m_hasWidth) {
                m_widthOff = m_pub.EndPlate.Width;
            }
            if (m_hasHeight) {
                m_heightOff = m_pub.EndPlate.Height/2;
            }
        }

        /// <summary>
        /// 处理单个端板的控制点配置
        /// </summary>
        /// <param name="endPlate"></param>
        private void HandlerOneEndPlatePoint(SteelPlate endPlate)
        {
            if (AllParamData.SelectEndPlateType == 2)//不处理圆形端板
            {
                return;
            }
            if (endPlate != null)
            {
                if(endPlate.CtrlPoints == null){return;}
                if(endPlate.CtrlPoints.Length < 4){return;}
                var horDist = AllParamData.EndPlateChamferHorDist;
                var verDist = AllParamData.EndPlateChamferVerDist;
                HandlerChamferPoint(endPlate.CtrlPoints[0],horDist,verDist);
                HandlerChamferPoint(endPlate.CtrlPoints[1],verDist,horDist);
                HandlerChamferPoint(endPlate.CtrlPoints[2],horDist,verDist);
                HandlerChamferPoint(endPlate.CtrlPoints[3],verDist,horDist);
                endPlate.Flush();
            }
        }
        
        /// <summary>
        /// 处理两个端板的控制点配置,针对前后端板进行设计
        /// </summary>
        private void HandlerTwoEndPlatePoint(SteelPlate front,SteelPlate back)
        {
            if (AllParamData.SelectEndPlateType == 2)//不处理圆形端板
            {
                return;
            }
            CtrlPoint point1;
            CtrlPoint point2;
            var horDist = AllParamData.EndPlateChamferHorDist;
            var verDist = AllParamData.EndPlateChamferVerDist;
            if (front?.CtrlPoints != null && front.CtrlPoints.Length >3)
            {
                point1 = front.CtrlPoints[2];
                point2 = front.CtrlPoints[3];
                HandlerChamferPoint(point1,horDist,verDist);
                HandlerChamferPoint(point2,verDist,horDist);
                front.Flush();
            }
            if (back?.CtrlPoints != null && back.CtrlPoints.Length >3)
            {
                point1 = back.CtrlPoints[2];
                point2 = back.CtrlPoints[3];
                HandlerChamferPoint(point1,horDist,verDist);
                HandlerChamferPoint(point2,verDist,horDist);
                back.Flush();
            }
            
        }

        private void HandlerChamferPoint(CtrlPoint point1,double cutX,double cutY)
        {
            if (point1 is ChamferCtrlPoint point)
            {
                point.CutType = GetCutType();
                point.CutX = cutX;
                point.CutY= cutY;
                point.Flush();
            }
        }

        private Define.CornerCutType GetCutType()
        {
            var type = AllParamData.SelectEndPlateChamferType;
            if (type == 0 || type == 1) {
                return Define.CornerCutType.CORNER_CUT_NONE;
            }
            if (type == 2) {
                return Define.CornerCutType.CORNER_CUT_LINE;
            }
            if (type == 3)
            {
                return Define.CornerCutType.CORNER_CUT_PARA_ROUND;
            }
            return Define.CornerCutType.CORNER_CUT_ROUND;
        }

        /// <summary>
        /// 处理对支撑的切割
        /// </summary>
        /// <param name="sub"></param>
        private void HandlerBraceCut(SubData sub)
        {
            if (AllParamData.SelectBraceIsCut != 2)
            {
                var pt1BackPl = SubPart.NearBuildPlane.Move(m_pub.DistOfConnAndGusset);
                var pt1FrontPl = pt1BackPl.Move(m_pub.ConnPartConnPlate.Thickness);
                pt1BackPl = new Plane(pt1BackPl.Origin, -pt1BackPl.Normal);
                
                
                var lengthOffset = AllParamData.BraceLengthOnConnPart + AllParamData.CutInBraceLength;
                
                var cutTop = pt1FrontPl.Move(AllParamData.CutInBraceWidth/2);
                var cutBtm = pt1BackPl.Move(AllParamData.CutInBraceWidth/2);
                
                if (m_pub.SelectConnPlateNum == 2)
                {
                    var widthOffset = m_pub.ConnPartConnPlate.Thickness + AllParamData.CutInBraceWidth + m_pub.DistOfConnAndGusset;
                    cutBtm = SubPart.FarBuildPlane.Move(widthOffset);
                    cutTop = SubPart.NearBuildPlane.Move(widthOffset);
                    lengthOffset += m_pub.MiddleEndPlate.Thickness;
                }
                var cutSides = new List<Plane>()
                {
                    sub.FinalAlignPlane,sub.TopPlane.Move(1.0d),
                    sub.FinalAlignPlane.Move(lengthOffset),sub.BottomPlane.Move(1.0d),
                };
                m_context.TargetComp.CreatePlateCutter(sub.Beam,cutBtm,cutTop,cutSides.ToArray(),true);
            }
        }

        private void HandlerPlaneCut(List<Plane> planes, SubData sub)
        {
            if (AllParamData.SelectBraceIsCut == 2) {
                HandlerPlaneInCut(planes,sub);
            }
            else {
                HandlerPlaneNoCut(planes, sub);
            }
        }

        private Plane CalculateTopChamferPlane(SubData sub)
        {
            var p1 = PointHelper.Move(sub.TopConnNearAlignPoint, -sub.TopConnPartPlane.Normal,
                AllParamData.ChamferVerticalDist);
            var p2 = PointHelper.Move(sub.TopConnNearAlignPoint, sub.GstNearAlignPlane.Normal,
                AllParamData.ChamferHorizontalDist);
            var axisX = p1 - p2;
            var normal = axisX.Cross(SubPart.NearBuildPlane.Normal);
            return new Plane(p1,normal);
        }
        private Plane CalculateBtmChamferPlane(SubData sub)
        {
            var p1 = PointHelper.Move(sub.BtmConnNearAlignPoint, -sub.BtmConnPartPlane.Normal,
                AllParamData.ChamferVerticalDist);
            var p2 = PointHelper.Move(sub.BtmConnNearAlignPoint, sub.GstNearAlignPlane.Normal,
                AllParamData.ChamferHorizontalDist);
            var axisX = p1 - p2;
            var normal = axisX.Cross(SubPart.NearBuildPlane.Normal);
            return new Plane(p1,normal);
        }

        private void HandlerPlaneNoCut(List<Plane> planes, SubData sub)
        {
            var farBuildPl = GetFarBuildPl(sub);
            planes.Add(sub.GstNearAlignPlane);
            planes.Add(sub.TopConnPartPlane);
            planes.Add(farBuildPl);
            planes.Add(sub.BtmConnPartPlane);
            if (CanChamferConn) {
                var topChamferPl = CalculateTopChamferPlane(sub);
                planes.Insert(1,topChamferPl);
                var btmChamferPl = CalculateBtmChamferPlane(sub);
                planes.Add(btmChamferPl);
            }
        }



        private void HandlerPlaneInCut(List<Plane> planes, SubData sub)
        {
            var farBuildPl = GetFarBuildPl(sub);

            bool canCut = !(sub.FinalAlignPlane.SignedDist(farBuildPl.Origin) < 0);

            if (!canCut) {
                HandlerPlaneNoCut(planes,sub);
                return;
            }
            
            planes.Add(sub.GstNearAlignPlane);
            planes.Add(sub.TopConnPartPlane);
            planes.Add(farBuildPl);
            planes.Add(sub.TopPlane.Move(AllParamData.CutGapOfConnPartBrace));
            planes.Add(sub.FinalAlignPlane);
            planes.Add(sub.BottomPlane.Move(AllParamData.CutGapOfConnPartBrace));
            planes.Add(farBuildPl);
            planes.Add(sub.BtmConnPartPlane);
            if (CanChamferConn) {
                var topChamferPl = CalculateTopChamferPlane(sub);
                planes.Insert(1,topChamferPl);
                var btmChamferPl = CalculateBtmChamferPlane(sub);
                planes.Add(btmChamferPl);
            }
        }

        private Plane GetFarBuildPl(SubData sub)
        {
            var farBuildDist = DistToGstFar;
            if (!Equals(m_pub.ConnPartConnPlate.Width, double.MaxValue)) {
                farBuildDist = m_pub.ConnPartConnPlate.Width- sub.GstPartAreaLength;
            }
            var farBuildPl = sub.GstFarAlignPlane.Move(farBuildDist);
            return farBuildPl;
        }

        /// <summary>
        /// 连接板的后处理
        /// </summary>
        /// <param name="plate"></param>
        private void HandlerPlate1(SteelPlate plate)
        {
            if (plate == null)
            {
                return;
            }
            plate.WorkAxis = new Mat43(SubPart.AxisX, SubPart.AxisY, SubPart.RefSubBuildPlane.Normal, Vec3.ZERO);
            var normal = plate.WorldMatrix.Z;
            var dot = normal.Dot(SubPart.RefSubBuildPlane.Normal);
            plate.OffsetSide = dot > 0 ? Define.ObjectSide.SIDE_LEFT : Define.ObjectSide.SIDE_RIGHT;
            plate.Offset = m_pub.GussetPlatePro.Thickness/2 + m_pub.DistOfConnAndGusset;
            plate.Flush();
        }
        
        /// <summary>
        /// 连接板的后处理
        /// </summary>
        /// <param name="plate"></param>
        private void HandlerPlate2(SteelPlate plate)
        {
            if (plate == null)
            {
                return;
            }
            plate.WorkAxis = new Mat43(SubPart.AxisX, SubPart.AxisY, SubPart.RefSubBuildPlane.Normal, Vec3.ZERO);
            var normal = plate.WorldMatrix.Z;
            var dot = normal.Dot(SubPart.RefSubBuildPlane.Normal);
            plate.OffsetSide = dot < 0 ? Define.ObjectSide.SIDE_LEFT : Define.ObjectSide.SIDE_RIGHT;
            plate.Offset = m_pub.GussetPlatePro.Thickness/2 + m_pub.DistOfConnAndGusset;
            plate.Flush();
        }
    }
}