﻿using System;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using TubeGusset_20_.Main;
using TubeGusset_20_.Tool;

namespace TubeGusset_20_.Running
{
    public class BuildLProfilePlate:RunningBaseClass
    {
        public BuildLProfilePlate(ComponentInstanceContext context, ParserSubPart parserSubPart, BuildConnPlate buildConnPlate, BuildGussetPlate buildGussetPlate) : base(context)
        {
            ParserSubPart = parserSubPart;
            BuildConnPlate = buildConnPlate;
            BuildGussetPlate = buildGussetPlate;
        }

        private readonly ParserSubPart ParserSubPart;
        private readonly BuildConnPlate BuildConnPlate;
        private readonly BuildGussetPlate BuildGussetPlate;
        
        internal ParamData AllParamData { get;} = new ParamData();

        internal class ParamData 
        {
            public BeamProperties ConnectionPlatePro { get; set; } = new BeamProperties();
        }
        
        internal Vec3 m_topBuildP = Vec3.ZERO;
        internal Vec3 m_btmBuildP = Vec3.ZERO;
        public SteelBeam LProfilePlate1 { get; set; }
        public SteelBeam LProfilePlate2 { get; set; }
        public bool m_bCanBuild1 = false;
        public bool m_bCanBuild2 = false;

        internal bool m_bSwitchLongShortSide = false;
        

        public override bool Init()
        {
            if (!InitParamData()) return false;
            return base.Init();
        }

        private bool InitParamData()
        {
            var handler = m_context.ParamsHandler;
            AllParamData.ConnectionPlatePro.SetProps
                (handler.GetDObjectValue(ParamNames.GUSSET_L_PROFILE_PLATE));
           
            
            return true;
        }

        public override bool Prepare()
        {
            var select = m_pub.SelectConnectionPlateOrLProfile;
            m_bCanBuild1 = select == 4 || select == 6;
            m_bCanBuild2 = select == 5 || select == 6;
            //select = m_context.BuildGussetPlate.AllParamData.SelectClipAngleOrientation;
            select = m_pub.SelectClipAngleOrientation;
            m_bSwitchLongShortSide = select == 2;
            return base.Prepare();
        }

        public override bool Execute()
        {
            var connPlate = BuildConnPlate;
            if (!m_bCanBuild1 && !m_bCanBuild2)
            {
                return true;
            }
            
            m_topBuildP = connPlate.m_topBuildP;
            m_btmBuildP = connPlate.m_btmBuildP;

            PointHelper.MoveToRef(ref m_topBuildP,ParserSubPart.FarBuildPlane);
            PointHelper.MoveToRef(ref m_btmBuildP,ParserSubPart.FarBuildPlane);
            PointHelper.MoveToRef(ref m_topBuildP,ParserSubPart.FaceBuildPlane);
            PointHelper.MoveToRef(ref m_btmBuildP,ParserSubPart.FaceBuildPlane);
            
            var startP = m_btmBuildP;
            var endP = m_topBuildP;
            if (m_bSwitchLongShortSide) {
                var tmp = endP;
                endP = startP;
                startP = tmp;
            }

            var target = m_context.TargetComp;

            if (m_bCanBuild1)
            {
                LProfilePlate1 = target.CreateSteelBeam (AllParamData.ConnectionPlatePro.Profile, startP, endP, true);
                if (LProfilePlate1 == null) {
                    return false;
                }
                HandlerPlate1();
                var dir = ParserSubPart.FarBuildPlane.Normal;
                m_context.BuildWeld.BuildWeldForLProfile(ParserSubPart,BuildGussetPlate,LProfilePlate1,m_topBuildP,m_btmBuildP,dir);
            }

            if (m_bCanBuild2)
            {
                LProfilePlate2 = target.CreateSteelBeam (AllParamData.ConnectionPlatePro.Profile,endP,startP,true);
                if (LProfilePlate2 == null) {
                    return false;
                }
                HandlerPlate2();
                var dir = -ParserSubPart.FarBuildPlane.Normal;
                m_context.BuildWeld.BuildWeldForLProfile(ParserSubPart,BuildGussetPlate,LProfilePlate2,m_topBuildP,m_btmBuildP,dir);
            }

            return true;
        }

        private bool CheckAxisY()
        {
            var axisY = ParserSubPart.AxisY;
            var dot = axisY.Dot(Vec3.AXIS_Z);
            return dot< 0;
        }


        private void HandlerPlate1()
        {
            var subPart = ParserSubPart;
            LProfilePlate1.WorkAxis = new Mat43(subPart.AxisX, subPart.AxisY, subPart.FarBuildPlane.Normal.Normalize(), Vec3.ZERO);
            LProfilePlate1.RotationSide = Define.ObjectSide.SIDE_FRONT;
            LProfilePlate1.HorzOffsetSide = Define.ObjectSide.SIDE_LEFT;
            LProfilePlate1.VertOffsetSide= Define.ObjectSide.SIDE_FRONT;
            
            if (m_bSwitchLongShortSide) {
                LProfilePlate1.RotationSide = Define.ObjectSide.SIDE_TOP;
                LProfilePlate1.HorzOffsetSide = Define.ObjectSide.SIDE_RIGHT;
            }
            AllParamData.ConnectionPlatePro.ApplyToPart(LProfilePlate1);
            LProfilePlate1.Refresh();
        }
        
        private void HandlerPlate2()
        {
            var subPart = ParserSubPart;
            LProfilePlate2.WorkAxis = new Mat43(subPart.AxisX, subPart.AxisY, subPart.FarBuildPlane.Normal.Normalize(), Vec3.ZERO);
            //LProfilePlate2.WorkAxis = new Mat43(subPart.AxisX, subPart.AxisY, subPart.AxisX.Cross(subPart.AxisY), Vec3.ZERO);
            LProfilePlate2.RotationSide = Define.ObjectSide.SIDE_BACK;
            LProfilePlate2.HorzOffsetSide = Define.ObjectSide.SIDE_RIGHT;
            LProfilePlate2.VertOffsetSide = Define.ObjectSide.SIDE_BACK;
            LProfilePlate2.VerticalOffset = m_pub.GussetPlatePro.Thickness;
            
            if (m_bSwitchLongShortSide) {
                LProfilePlate2.RotationSide = Define.ObjectSide.SIDE_BOTTOM;
                LProfilePlate2.HorzOffsetSide = Define.ObjectSide.SIDE_LEFT;
            }
            AllParamData.ConnectionPlatePro.ApplyToPart(LProfilePlate2);
            LProfilePlate2.Refresh();
        }
    }
}