﻿using System;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;
using TubeGusset_20_.Main;
using TubeGusset_20_.Tool;

namespace TubeGusset_20_.Running
{
    public class BuildWeld : RunningBaseClass
    {
        public BuildWeld(ComponentInstanceContext context) : base(context)
        {
            m_context.BuildWeld = this;
        }

        /// <summary>
        /// 存储焊缝数据
        /// </summary>
        public List<WeldProperties> m_weldProps = new List<WeldProperties>();
        
        public override bool Init()
        {
            if (!InitWeld()) return true;
            return base.Init();
        }

        private bool InitWeld()
        {
            for(int i=0;i<ParamNames.WELD_PROPS_LIST.Length;++i)
            {
                m_weldProps.Add(new WeldProperties());
            }

            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                var weldProps = m_context.ParamsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }
            return true;
        }

        public void BuildWeldForEndPlate(SteelPlate endFront, SteelPlate endBack, SubData sub,
            Plane frontPl,Plane backPl,Plane landingPl)
        {
            landingPl = new Plane(landingPl.Move(m_pub.EndPlate.Thickness / 2));
            var thickness = 0.0d;
            if (sub.Profile is CircularHollowProfile cir) {
                thickness = cir.PlateThickness;
            }
            if (sub.Profile is RectHollowProfile rct) {
                thickness = rct.PlateThickness;
            }
            if (endFront != null) {
#if false
                var frontP = PlaneHelper.GetLandingPoint(landingPl, new List<Plane>()
                {
                    frontPl,sub.TopPlane.Move(-thickness),   
                    PlaneHelper.GetSameNormalPlane(sub.BeamPlanes,PlaneHelper.BOUND_PLANE_TBFB,frontPl.Normal).Move(-thickness),
                    sub.BottomPlane.Move(-thickness)
                });
                var weldFront = m_context.TargetComp.CreatePolyWeld(sub.Beam, endFront, frontP.ToArray(), true);
                m_weldProps[2].ApplyToWeld(weldFront);
#endif
#if true
                var pts = new List<Vec3>();
                if (endFront.CtrlPoints != null && endFront.CtrlPoints.Length > 0)
                {
                    pts.AddRange(endFront.CtrlPoints.Select(ctrlPt => ctrlPt.WorldPos));
                    pts.Add(endFront.CtrlPoints.First().WorldPos);
                }
                for (var i = 0; i < pts.Count; i++)
                {
                    pts[i] = pts[i].ProjectTo(landingPl);
                }
                var weld = m_context.TargetComp.CreatePolyWeld(sub.Beam, endFront, pts.ToArray(), true);
                m_weldProps[2].ApplyToWeld(weld);
#endif
            }
            if (endBack != null) {
#if false 
                var backP = PlaneHelper.GetLandingPoint(landingPl, new List<Plane>()
                {
                    backPl,sub.TopPlane.Move(-thickness),   
                    PlaneHelper.GetSameNormalPlane(sub.BeamPlanes,PlaneHelper.BOUND_PLANE_TBFB,backPl.Normal).Move(-thickness),
                    sub.BottomPlane.Move(-thickness)
                });
                var weldBack = m_context.TargetComp.CreatePolyWeld(sub.Beam, endBack, backP.ToArray(), true);
                m_weldProps[2].ApplyToWeld(weldBack);
#endif
#if true
                var pts = new List<Vec3>();
                if (endBack.CtrlPoints != null && endBack.CtrlPoints.Length > 0)
                {
                    pts.AddRange(endBack.CtrlPoints.Select(ctrlPt => ctrlPt.WorldPos));
                    pts.Add(endBack.CtrlPoints.First().WorldPos);
                }
                for (var i = 0; i < pts.Count; i++)
                {
                    pts[i] = pts[i].ProjectTo(landingPl);
                }
                var weld = m_context.TargetComp.CreatePolyWeld(sub.Beam, endFront, pts.ToArray(), true);
                m_weldProps[2].ApplyToWeld(weld);
#endif
            }
        }
        public void BuildWeldForConnPlate(ParserSubPart subPart,BuildGussetPlate buildGussetPlate,
            SteelBeam connectionPlate, Vec3 btmBuildP, Vec3 topBuildP
            , double thickness, double width, double height)
        {
            if (connectionPlate == null) {
                return;
            }
            var target = m_context.TargetComp;
            var select = m_pub.SelectClipAngleAttachmentType;
            var nearPlane = subPart.NearBuildPlane.Dist(topBuildP) > subPart.FarBuildPlane.Dist(topBuildP)
                ? subPart.FarBuildPlane: subPart.NearBuildPlane;
            var faceOffPlane = subPart.FaceBuildPlane.Move(subPart.AllParamData.OffSetForFaceBuildPlane);
            //连接板和节点板之间的焊缝
            var pointsL1 = new List<Vec3>();
            var topNearFaceP = PointHelper.MoveTo(topBuildP, new List<Plane>() { nearPlane, subPart.FaceBuildPlane });
            var btmNearFaceP = PointHelper.MoveTo(btmBuildP, new List<Plane>() { nearPlane, subPart.FaceBuildPlane });
            pointsL1.Add(topNearFaceP);
            pointsL1.Add(PointHelper.Move(topNearFaceP,nearPlane.Normal,thickness));
            pointsL1.Add(PointHelper.Move(btmNearFaceP,nearPlane.Normal,thickness));
            pointsL1.Add(btmNearFaceP);
            pointsL1.Add(topNearFaceP);
            var weld1 = target.CreatePolyWeld(m_context.MainPart,
                connectionPlate,pointsL1.ToArray(),true);
            m_weldProps[0].ApplyToWeld(weld1);
            
            if(select == 0 || select == 1 || select == 3) 
                return;
            
            var topNearFaceOffP= PointHelper.MoveTo(topBuildP, new List<Plane>(){nearPlane,faceOffPlane});
            //连接板和主零件之间的焊缝
            var pointsL2 = new List<Vec3>();
            pointsL2.Add(topNearFaceOffP);
            var tmpPoint = PointHelper.MoveTo(topBuildP, nearPlane);
            PointHelper.MoveRef(ref tmpPoint,faceOffPlane.Normal,width/2);
            pointsL2.Add(tmpPoint);
            tmpPoint = PointHelper.MoveTo(btmBuildP, nearPlane);
            PointHelper.MoveRef(ref tmpPoint,faceOffPlane.Normal,width/2);
            pointsL2.Add(tmpPoint);
            var btmNearFaceOffP= PointHelper.MoveTo(btmBuildP, new List<Plane>(){nearPlane,faceOffPlane});
            pointsL2.Add(btmNearFaceOffP);
            pointsL2.Add(topNearFaceOffP);
            
            var weld2 = target.CreatePolyWeld(buildGussetPlate.GussetPlate,connectionPlate,pointsL2.ToArray(),true);
            m_weldProps[3].ApplyToWeld(weld2);
        }

        /// <summary>
        /// 给L型截面创建焊缝
        /// </summary>
        /// <param name="lProfilePlate"></param>
        /// <param name="topBuildP"></param>
        /// <param name="btmBuildP"></param>
        /// <param name="dir"></param>
        public void BuildWeldForLProfile(ParserSubPart parserSubPart,BuildGussetPlate buildGussetPlate,
            SteelBeam lProfilePlate, Vec3 topBuildP, Vec3 btmBuildP, Vec3 dir)
        {
            if (lProfilePlate == null) {
                return;
            }
            var select =  m_pub.SelectClipAngleAttachmentType;
            var bGstPlate = select == 2 || select == 4;
            var bMain = select == 3 || select == 4;
            if(!bGstPlate && !bMain) return;
            if (lProfilePlate.Profile.GetType() == typeof(LProfile))
            {
                if(!(lProfilePlate.Profile is LProfile lProfile)) return;
                var beamPlanes = BeamPlanes.CreateFromBeam(lProfilePlate);
                if(beamPlanes == null) return;
                //以BLL200*100*5为例
                var target = m_context.TargetComp;
                var subPart = parserSubPart;
                var gstPart = buildGussetPlate;

                var gstNearBuildPl = new Plane(gstPart.m_buildPointsInFacePlane.First(),subPart.AxisX);
                var typeList = new List<EBeamPlaneType>() {
                    EBeamPlaneType.FrontBound, EBeamPlaneType.BackBound,
                    EBeamPlaneType.TopBound, EBeamPlaneType.BottomBound,
                };
                if (!PlaneHelper.CheckTypeList(beamPlanes, typeList)) {
                    return;
                }
                var gstFarBuildPl = PlaneHelper.GetSameNormalPlane(beamPlanes,typeList,subPart.AxisX);
                var gstLandingPl = PlaneHelper.GetSameNormalPlane(beamPlanes, typeList, -dir);
                if(gstFarBuildPl == null || gstLandingPl == null){return;}
                var gstTopPlane = new Plane(topBuildP, subPart.AxisY);
                var gstBtmPlane = new Plane(btmBuildP, subPart.AxisY);

                var mainLandingPl = new Plane(subPart.FaceBuildPlane);
                var mainTopPlane = new Plane(topBuildP, subPart.AxisY);
                var mainBtmPlane = new Plane(btmBuildP, subPart.AxisY);
                var mainNearBuildPl = new Plane(gstLandingPl);
                var mainFarBuildPl = PlaneHelper.GetSameNormalPlane(beamPlanes,typeList,-gstLandingPl.Normal);
                if(mainFarBuildPl == null){return;}
                

                if (bGstPlate)
                {
                    //连接板和节点板之间的焊缝
                    var gstPointL = PlaneHelper.GetLandingPoint(gstLandingPl, new List<Plane>()
                    {
                        gstNearBuildPl,gstTopPlane,gstFarBuildPl,gstBtmPlane,
                    });
                    gstPointL.Add(gstPointL.First());
                    var weldGst = target.CreatePolyWeld(gstPart.GussetPlate,lProfilePlate,gstPointL.ToArray(),true);
                    m_weldProps[3].ApplyToWeld(weldGst);
                }

                if (bMain)
                {
                    //连接件和主零件之间的焊缝
                    var mainPointL = PlaneHelper.GetLandingPoint(mainLandingPl, new List<Plane>()
                    {
                        mainNearBuildPl,mainTopPlane,mainFarBuildPl,mainBtmPlane,
                    });
                    mainPointL.Add(mainPointL.First());
                    var weldMain = target.CreatePolyWeld(m_context.MainPart,lProfilePlate,mainPointL.ToArray(),true);
                    m_weldProps[0].ApplyToWeld(weldMain);
                }
            }
        }

        public void BuildWeldForMidEndPlate(SteelPlate midEndPlate, SubData sub)
        {
            if (midEndPlate == null) {
                return;
            }
            var points = midEndPlate.CtrlPoints.Select(p => p.WorldPos).ToList();
            for (var i = 0; i < points.Count; i++)
            {
                var p = points[i];
                points[i] = PointHelper.Move(p, sub.FarPlane.Normal, m_pub.MiddleEndPlate.Thickness / 2);
            }

            var weld1 = m_context.TargetComp.CreatePolyWeld(sub.Beam,midEndPlate,new Vec3[]{points[0],points[3]},true);
            var weld2 = m_context.TargetComp.CreatePolyWeld(sub.Beam,midEndPlate,new Vec3[]{points[1],points[2]},true);
            m_weldProps[2].ApplyToWeld(weld1);
            m_weldProps[2].ApplyToWeld(weld2);
        }

        public void BuildWeldForGussetPlate(Plane topMainPlane, Plane btmMainPlane, SteelPlate gussetPlate,
            ParserSubPart subPart)
        {
            if (gussetPlate == null) {
                return;
            }
            if (m_pub.SelectConnectionPlateOrLProfile == 0 || m_pub.SelectConnectionPlateOrLProfile == 1)
            {
                if(subPart.m_bExtensionMode)
                    return;
                var landingPl = subPart.FaceBuildPlane;
                var points = PlaneHelper.GetLandingPoint(landingPl, new List<Plane>()
                {
                    topMainPlane,subPart.NearBuildPlane,
                    btmMainPlane,subPart.FarBuildPlane,
                });
                points.Add(points.First());

                var weld = m_context.TargetComp.CreatePolyWeld(m_context.MainPart, gussetPlate, points.ToArray(), true);
                m_weldProps[0].ApplyToWeld(weld);
                //NOTE 检查是否有延长，且获得延长穿过主零件的位置是否能产生焊缝。
                var dist = m_pub.GstPassThroughTheMainPartDist;
                landingPl.MoveThis(-dist);
                var dir = subPart.FaceBuildPlane.Normal;
                var pls = new List<Tuple<Plane,double>>();
                var types = new List<EBeamPlaneType>()
                {
                    EBeamPlaneType.FrontWeb,EBeamPlaneType.BackWeb,
                    EBeamPlaneType.TopFlange,EBeamPlaneType.TopFlangeInside,
                    EBeamPlaneType.BottomFlange,EBeamPlaneType.BottomFlangeInside,
                };
                foreach (var type in types)
                {
                    var pl = PlaneHelper.GetBeamPlane(type,m_context.MainPlanes);
                    if (pl != null)
                    {
                        if(dir.IsVerticalTo(pl.Normal)){continue;}
                        if (dir.IsParallelTo(pl.Normal)) 
                        {
                            dist = landingPl.SignedDist(pl.Origin);
                            pls.Add(new Tuple<Plane, double>(pl,dist));
                        }
                    }
                }
                //拿到法线平行的面与参考面的有符号距离
                var pt = points.First();
                pt.ProjectToThis(landingPl);
                var pts = pls.Select(pl => pt.ProjectTo(pl.Item1)).ToList();
                pts.Sort((p1,p2)=> PointHelper.ComparePointsByProjection(p1,p2,-dir));
                if (pts.Count > 2)
                {
                    var ret = PointHelper.MaxPointsByProjection(pt, pts.Last(), dir);
                    if (!ret.Item2)
                    {
                        var refPl = new Plane(ret.Item1,landingPl.Normal);
                        points.Clear();
                        points = PlaneHelper.GetLandingPoint(refPl, new List<Plane>()
                        {
                            topMainPlane,subPart.NearBuildPlane,
                            btmMainPlane,subPart.FarBuildPlane,
                        });
                        points.Add(points.First());

                        weld = m_context.TargetComp.CreatePolyWeld(m_context.MainPart, gussetPlate, points.ToArray(), true);
                        m_weldProps[0].ApplyToWeld(weld);
                    }
                }
            }
        }

        public void BuildWeldForTonguePlate(ParserSubPart parserSubPart,SteelPlate tonguePlate, SubData sub, Plane buildFarRefPl)
        {
            if (tonguePlate == null) {
                return;
            }
            var landingPl = sub.TopPlane;
            var points = PlaneHelper.GetLandingPoint(landingPl, new List<Plane>()
            {
                sub.FinalAlignPlane,parserSubPart.CentreBuildPlane.Move(tonguePlate.Thickness/2),
                buildFarRefPl,parserSubPart.CentreBuildPlane.Move(-tonguePlate.Thickness/2),
            });
            var weld = m_context.TargetComp.CreatePolyWeld(sub.Beam, tonguePlate, points.ToArray(), true);
            m_weldProps[1].ApplyToWeld(weld);
            points.Clear();
            landingPl = sub.BottomPlane;
            points = PlaneHelper.GetLandingPoint(landingPl, new List<Plane>()
            {
                sub.FinalAlignPlane,parserSubPart.CentreBuildPlane.Move(tonguePlate.Thickness/2),
                buildFarRefPl,parserSubPart.CentreBuildPlane.Move(-tonguePlate.Thickness/2),
            });
            weld = m_context.TargetComp.CreatePolyWeld(sub.Beam, tonguePlate, points.ToArray(), true);
            m_weldProps[1].ApplyToWeld(weld);
        }

        public void BuildWeldForConnPartInBraceNoCut(ParserSubPart parserSubPart,SteelPlate plate1, SteelPlate plate2 ,SubData sub, double farDist)
        {
            var farBuildPl = sub.FinalAlignPlane.Move(farDist);
            //var nearBuildPl = sub.GstFarAlignPlane.Move(m_pub.EndPlate.Thickness + m_pub.pic_distance_between_the_gusset_plate_and_brace_part);
            var nearBuildPl = new Plane(sub.FinalAlignPlane);
            
            var landingPl = sub.TopPlane;
            WeldBuild(parserSubPart,plate1, plate2, sub, landingPl, nearBuildPl, farBuildPl);
            landingPl = sub.BottomPlane;
            WeldBuild(parserSubPart,plate1, plate2, sub, landingPl, nearBuildPl, farBuildPl);
        }

        private void WeldBuild(ParserSubPart parserSubPart,SteelPlate plate1, SteelPlate plate2, SubData sub, Plane landingPl, Plane nearBuildPl,
            Plane farBuildPl)
        {
            if(plate1 == null){return;}
            var num = 1 + (plate2 != null ? 1 : 0);

            if (num == 1)
            {
                var points = PlaneHelper.GetLandingPoint(landingPl, new List<Plane>()
                {
                    nearBuildPl,parserSubPart.NearBuildPlane.Move(m_pub.DistOfConnAndGusset),
                    farBuildPl,parserSubPart.NearBuildPlane.Move(m_pub.DistOfConnAndGusset+plate1.Thickness),
                });
                var weld1 = m_context.TargetComp.CreatePolyWeld(sub.Beam, plate1, points.ToArray(), true);
                m_weldProps[1].ApplyToWeld(weld1);
            }
            if (num == 2)
            {
                var points = PlaneHelper.GetLandingPoint(landingPl, new List<Plane>()
                {
                    nearBuildPl,parserSubPart.NearBuildPlane.Move(m_pub.DistOfConnAndGusset+plate1.Thickness), 
                    farBuildPl,parserSubPart.NearBuildPlane.Move(m_pub.DistOfConnAndGusset),
                });
                if (points.Count > 0) { points.RemoveAt(points.Count - 1); }
                var weld1 = m_context.TargetComp.CreatePolyWeld(sub.Beam, plate1, points.ToArray(), true);
                m_weldProps[1].ApplyToWeld(weld1);
                points.Clear();
                points = PlaneHelper.GetLandingPoint(landingPl, new List<Plane>()
                {
                    nearBuildPl,parserSubPart.FarBuildPlane.Move(m_pub.DistOfConnAndGusset+plate2.Thickness),
                    farBuildPl,parserSubPart.FarBuildPlane.Move(m_pub.DistOfConnAndGusset),
                });
                if (points.Count > 0) { points.RemoveAt(points.Count - 1); }
                var weld2 = m_context.TargetComp.CreatePolyWeld(sub.Beam, plate2, points.ToArray(), true);
                m_weldProps[1].ApplyToWeld(weld2);
            }
        }
    }
}