﻿using System.Collections.Generic;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using TubeGusset_20_.Main;
using TubeGusset_20_.Tool;

namespace TubeGusset_20_.Running
{
    
    public class ParserBraceBolts:RunningBaseClass
    {
        public ParserBraceBolts(ComponentInstanceContext context) : base(context)
        {
        }

        /// <summary>
        /// 所有与面板相关的值
        /// </summary>
        internal List<ParamData> ParamDataList { get; } = new List<ParamData>();
        /// <summary>
        /// 螺栓数据
        /// </summary>
        public List<BoltsData> BoltsData { get; set; } = new List<BoltsData>();

        internal class ParamData
        {
            public BoltsConfig BoltsCfg { get; set; } = new BoltsConfig();
            public int HorBoltsNum { get; set; }
            public double[] HorBoltsSpace { get; set;}
            public int VerBoltsNum { get; set; }
            public double[] VerBoltsSpace { get; set; }
            public double VerNearEdgeDist { get; set; }
            public double VerFarEdgeDist { get; set; }
            public int VerMeasureMethod { get; set; }
            public double VerMeasureOffset { get; set; }
            /// <summary>
            /// 支撑的边缘面到连接部分边缘的距离
            /// </summary>
            public double EdgeDistBraceAndConnPart { get; set; }
        }

        /// <summary>
        /// 将属性值配置给螺栓组
        /// </summary>
        /// <param name="boltGroup"></param>
        /// <param name="index"></param>
        /// <param name="fillMadeType0"></param>
        public void SetUpBoltGroup(BoltGroup boltGroup,int index,bool fillMadeType0 = true)
        {
            var paramCfg = ParamDataList[index];
            paramCfg.BoltsCfg.FillShape(boltGroup);
            paramCfg.BoltsCfg.FillBasicProps(boltGroup,fillMadeType0);
            paramCfg.BoltsCfg.FillContentProps(boltGroup);
            paramCfg.BoltsCfg.FillExtendProps(boltGroup);
            paramCfg.BoltsCfg.FillHoleProps(boltGroup);
            var boltCfg = BoltsData[index];
            boltCfg.SetUpWith(boltGroup);
        }
        
        public override bool Init()
        {
            if (!InitParamData()) return false;
            return true;
        }

        public override bool Prepare()
        {
            return true;
        }

        /// <summary>
        /// 处理面板数据
        /// </summary>
        /// <returns></returns>
        private bool InitParamData()
        {
            var handler = m_context.ParamsHandler;
            #region 支柱螺栓1 
            var boltGroup1 = new ParamData {
                HorBoltsNum = handler.GetIntValue(ParamNames.BRACE_BOLTS_1_HORIZONTAL_BOLTS_NUMBER),
                HorBoltsSpace = handler.GetDoubleArrayValue(ParamNames.BRACE_BOLTS_1_HORIZONTAL_BOLTS_SPACING),
                VerBoltsNum = handler.GetIntValue(ParamNames.BRACE_BOLTS_1_VERTICAL_BOLTS_NUMBER),
                VerBoltsSpace = handler.GetDoubleArrayValue(ParamNames.BRACE_BOLTS_1_VERTICAL_BOLTS_SPACING),
                VerNearEdgeDist = handler.GetDoubleValue(ParamNames.BRACE_BOLTS_1_VERTICAL_BOLTS_NEAR_EDGE_DIST),
                VerFarEdgeDist = handler.GetDoubleValue(ParamNames.BRACE_BOLTS_1_VERTICAL_BOLTS_FAR_EDGE_DIST),
                VerMeasureMethod = handler.GetIntValue(ParamNames.BRACE_BOLTS_1_VERTICAL_BOLTS_SELECT_MEASURE_METHOD),
                VerMeasureOffset = handler.GetDoubleValue(ParamNames.BRACE_BOLTS_1_VERTICAL_BOLTS_MEASURE_OFFSET),
                EdgeDistBraceAndConnPart = handler.GetDoubleValue(ParamNames.BRACE_BOLTS_1_DISTANCE_BETWEEN_BRACE_AND_CONNECTION_PLATE_EDGE)
            };
            boltGroup1.BoltsCfg.CopyFromBasicProps
                (handler.GetDObjectValue(ParamNames.BRACE_BOLTS_1_BOLT_BASIC_PROPERTIES));
            boltGroup1.BoltsCfg.CopyFromHoleProps(
                handler.GetDObjectValue(ParamNames.BRACE_BOLTS_1_SLOTTED_HOLES));
            boltGroup1.BoltsCfg.CopyFromContentProps
                (handler.GetDObjectValue(ParamNames.BRACE_BOLTS_1_BOLT_ASSEMBLY));
            boltGroup1.BoltsCfg.CopyFromExtendProps
                (handler.GetDObjectValue(ParamNames.BRACE_BOLTS_1_BOLT_LENGTH_INCREASE));
            boltGroup1.BoltsCfg.CopyFromShapeProps
                (handler.GetIntValue(ParamNames.BRACE_BOLTS_1_STAGGERING_OF_BOLTS));
            ParamDataList.Add(boltGroup1);
            #endregion
            #region 支柱螺栓2 
            var boltGroup2 = new ParamData {
                HorBoltsNum = handler.GetIntValue(ParamNames.BRACE_BOLTS_2_HORIZONTAL_BOLTS_NUMBER),
                HorBoltsSpace = handler.GetDoubleArrayValue(ParamNames.BRACE_BOLTS_2_HORIZONTAL_BOLTS_SPACING),
                VerBoltsNum = handler.GetIntValue(ParamNames.BRACE_BOLTS_2_VERTICAL_BOLTS_NUMBER),
                VerBoltsSpace = handler.GetDoubleArrayValue(ParamNames.BRACE_BOLTS_2_VERTICAL_BOLTS_SPACING),
                VerNearEdgeDist = handler.GetDoubleValue(ParamNames.BRACE_BOLTS_2_VERTICAL_BOLTS_NEAR_EDGE_DIST),
                VerFarEdgeDist = handler.GetDoubleValue(ParamNames.BRACE_BOLTS_2_VERTICAL_BOLTS_FAR_EDGE_DIST),
                VerMeasureMethod = handler.GetIntValue(ParamNames.BRACE_BOLTS_2_VERTICAL_BOLTS_SELECT_MEASURE_METHOD),
                VerMeasureOffset = handler.GetDoubleValue(ParamNames.BRACE_BOLTS_2_VERTICAL_BOLTS_MEASURE_OFFSET),
                EdgeDistBraceAndConnPart = handler.GetDoubleValue(ParamNames.BRACE_BOLTS_2_DISTANCE_BETWEEN_BRACE_AND_CONNECTION_PLATE_EDGE)
            };
            boltGroup2.BoltsCfg.CopyFromBasicProps
                (handler.GetDObjectValue(ParamNames.BRACE_BOLTS_2_BOLT_BASIC_PROPERTIES));
            boltGroup2.BoltsCfg.CopyFromHoleProps(
                handler.GetDObjectValue(ParamNames.BRACE_BOLTS_2_SLOTTED_HOLES));
            boltGroup2.BoltsCfg.CopyFromContentProps
                (handler.GetDObjectValue(ParamNames.BRACE_BOLTS_2_BOLT_ASSEMBLY));
            boltGroup2.BoltsCfg.CopyFromExtendProps
                (handler.GetDObjectValue(ParamNames.BRACE_BOLTS_2_BOLT_LENGTH_INCREASE));
            boltGroup2.BoltsCfg.CopyFromShapeProps
                (handler.GetIntValue(ParamNames.BRACE_BOLTS_2_STAGGERING_OF_BOLTS));
            ParamDataList.Add(boltGroup2);
            #endregion
            #region 支柱螺栓3 
            var boltGroup3 = new ParamData {
                HorBoltsNum = handler.GetIntValue(ParamNames.BRACE_BOLTS_3_HORIZONTAL_BOLTS_NUMBER),
                HorBoltsSpace = handler.GetDoubleArrayValue(ParamNames.BRACE_BOLTS_3_HORIZONTAL_BOLTS_SPACING),
                VerBoltsNum = handler.GetIntValue(ParamNames.BRACE_BOLTS_3_VERTICAL_BOLTS_NUMBER),
                VerBoltsSpace = handler.GetDoubleArrayValue(ParamNames.BRACE_BOLTS_3_VERTICAL_BOLTS_SPACING),
                VerNearEdgeDist = handler.GetDoubleValue(ParamNames.BRACE_BOLTS_3_VERTICAL_BOLTS_NEAR_EDGE_DIST),
                VerFarEdgeDist = handler.GetDoubleValue(ParamNames.BRACE_BOLTS_3_VERTICAL_BOLTS_FAR_EDGE_DIST),
                VerMeasureMethod = handler.GetIntValue(ParamNames.BRACE_BOLTS_3_VERTICAL_BOLTS_SELECT_MEASURE_METHOD),
                VerMeasureOffset = handler.GetDoubleValue(ParamNames.BRACE_BOLTS_3_VERTICAL_BOLTS_MEASURE_OFFSET),
                EdgeDistBraceAndConnPart = handler.GetDoubleValue(ParamNames.BRACE_BOLTS_3_DISTANCE_BETWEEN_BRACE_AND_CONNECTION_PLATE_EDGE)
            };
            boltGroup3.BoltsCfg.CopyFromBasicProps
                (handler.GetDObjectValue(ParamNames.BRACE_BOLTS_3_BOLT_BASIC_PROPERTIES));
            boltGroup3.BoltsCfg.CopyFromHoleProps(
                handler.GetDObjectValue(ParamNames.BRACE_BOLTS_3_SLOTTED_HOLES));
            boltGroup3.BoltsCfg.CopyFromContentProps
                (handler.GetDObjectValue(ParamNames.BRACE_BOLTS_3_BOLT_ASSEMBLY));
            boltGroup3.BoltsCfg.CopyFromExtendProps
                (handler.GetDObjectValue(ParamNames.BRACE_BOLTS_3_BOLT_LENGTH_INCREASE));
            boltGroup3.BoltsCfg.CopyFromShapeProps
                (handler.GetIntValue(ParamNames.BRACE_BOLTS_3_STAGGERING_OF_BOLTS));
            ParamDataList.Add(boltGroup3);
            #endregion
            return true;
        }
        

        /// <summary>
        /// 给支撑准备螺栓数据
        /// </summary>
        /// <param name="subData"></param>
        /// <param name="index"></param>
        public void PrepareBoltsData(SubData subData, int index)
        {
            PrepareBoltsData(subData.NearToFarLine,subData.TopPlane.Normal,subData.TopBottomDist,index);
        }
        
        
        /// <summary>
        /// 准备螺栓生成相关的配置属性：工作坐标系、方向、偏移等。
        /// </summary>
        /// <param name="axisX"></param>
        /// <param name="axisY"></param>
        /// <param name="dist"></param>
        /// <param name="index"></param>
        public void PrepareBoltsData(Vec3 axisX,Vec3 axisY,double dist, int index)
        {
            var paramData = ParamDataList[index];
            var boltsData = new BoltsData();
            //var axisX = subData.NearToFarLine;
            //var axisY = subData.TopPlane.Normal;
            //确定工作平面
            boltsData.m_workAxis = new Mat43(axisX,axisY,axisX.Cross(axisY),Vec3.ZERO);
            boltsData.ArrayStrideY = AdaptBoltSpacingY(paramData);
            boltsData.ArrayStrideX = AdaptBoltSpacingX(paramData);
            boltsData.HeadOffsetX = -(boltsData.ArrayStrideX.Sum() + paramData.VerFarEdgeDist);
            
            //处理测量方式
            var select = paramData.VerMeasureMethod;
            var offset = paramData.VerMeasureOffset;
            //var dist = subData.TopBottomDist;
            var sum = boltsData.ArrayStrideY.Sum();
            switch (select)
            {
                case 0:
                case 2:
                    break;
                case 1:
                    offset += (sum - dist) / 2;
                    break;
                case 3:
                    offset += - (sum - dist) / 2;
                    break;
            }
            boltsData.VerticalOffset = offset; 

            //NOTE 配置CutLength 每个支撑的cutLength都不相同，不能存放在boltsData中
            //var cutL = m_context.BuildGussetPlate.AllParamData.Thickness * 2;
            //var subL = subData.Thickness * 2;
            //boltsData.CutLength = cutL + subL + 1; 
            
            BoltsData.Add(boltsData);

        }

        /// <summary>
        /// 给SubData设置AreaLength(螺栓区域总长度)
        /// </summary>
        /// <param name="subData"></param>
        /// <param name="index"></param>
        public void SetUpGstPartAreaLength(SubData subData, int index)
        {
            // NOTE 在ParserSubPart中调用
            var paramData = ParamDataList[index];
            subData.GstPartAreaLength = AdaptBoltSpacingX(paramData).Sum() + 
                                     paramData.VerFarEdgeDist + paramData.VerNearEdgeDist;
            var boltSize = paramData.BoltsCfg.m_basicProps.BoltSize;
            subData.BoltGroupHorLength = subData.GstPartAreaLength - paramData.VerFarEdgeDist + boltSize;
            subData.BoltGroupVerLength = AdaptBoltSpacingY(paramData).Sum() + boltSize*2.5;
            subData.BoltSize = boltSize;
        }

        /// <summary>
        /// 设置SubData的连接参照顶底面
        /// </summary>
        /// <param name="subData"></param>
        /// <param name="index"></param>
        public void SetUpConnPartPlane(SubData subData, int index)
        {
            var paramData = ParamDataList[index];
            //NOTE 处理ConnPart的板属性宽度
            var dist = paramData.EdgeDistBraceAndConnPart;
            var height = m_pub.ConnPartConnPlate.Height;
            if (!Equals(height, double.MaxValue))
            {
                dist = height - subData.TopBottomDist;
            }
            subData.TopConnPartPlane = subData.TopPlane.Move(dist);
            subData.BtmConnPartPlane = subData.BottomPlane.Move(dist);
        }
        
        private double[] AdaptBoltSpacingX(ParamData cfg)
        {
            return BoltsHelp.AdaptBoltSpacing(cfg.VerBoltsNum, cfg.VerBoltsSpace, 60.0d);
        }

        private double[] AdaptBoltSpacingY(ParamData cfg)
        {
            return BoltsHelp.AdaptBoltSpacing(cfg.HorBoltsNum, cfg.HorBoltsSpace, 60.0d);
        }
    }
}