﻿using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC;
using TubeGusset_20_.Main;
using TubeGusset_20_.Tool;

namespace TubeGusset_20_.Running
{
    public class ParserGussetBolts : RunningBaseClass
    {

        public ParserGussetBolts(ComponentInstanceContext context, ParserSubPart parserSubPart, BuildGussetPlate buildGussetPlate) : base(context)
        {
            ParserSubPart = parserSubPart;
            BuildGussetPlate = buildGussetPlate;
        }

        private readonly ParserSubPart ParserSubPart;
        private readonly BuildGussetPlate BuildGussetPlate;

        internal bool m_bWightHasFill;
        internal BoltsData ConnPlateBoltsData { get; } = new BoltsData();
        internal BoltsData LProfileBoltsData { get; } = new BoltsData();
        internal ParamData AllParamData { get; } = new ParamData();
        
        internal Vec3 TopBoltBuild = Vec3.ZERO;
        internal Vec3 BtmBoltBuild = Vec3.ZERO;

        public class ParamData
        {
            public BoltsConfig BoltsConfig = new BoltsConfig();
            public int HorBoltsNum { get; set; }
            public double[] HorBoltsSpace { get; set;}
            public double HorTopEdgeDist { get; set; }
            public double HorBtmEdgeDist { get; set; }
            
            public int VerBoltsNum { get; set; }
            public double[] VerBoltsSpace { get; set; }
            public double VerNearEdgeDist { get; set; }
            
            public int MainPartBoltsNum { get; set; }
            public double[] MainPartBoltsSpace { get; set; }
            public double MainPartCenterEdgeDist { get; set; }
            
            //螺栓制作配置
            public int LinkedPartMadeType { get; set; }
            public int MainPartMadeType { get; set; }
            public int SelectBoltsOnGussetPlate { get; set; }
        }
        
        public override bool Init()
        {
            if (!InitParamData()) return false;
            if (!InitBoltsData()) return false;
            return base.Init();
        }

        private bool InitBoltsData()
        {
            ConnPlateBoltsData.ArrayStrideX = BoltsHelp.AdaptBoltSpacing
                (AllParamData.HorBoltsNum,AllParamData.HorBoltsSpace,60.0d);
            ConnPlateBoltsData.ArrayStrideY = BoltsHelp.AdaptBoltSpacing
                (AllParamData.VerBoltsNum,AllParamData.VerBoltsSpace,60.0d);
            return true;
        }

        private bool InitParamData()
        {
            var handler = m_context.ParamsHandler;
            var config = AllParamData.BoltsConfig;
            config.CopyFromBasicProps (handler.GetDObjectValue
                (ParamNames.GUSSET_BOLTS_BOLT_BASIC_PROPERTIES));
            config.CopyFromHoleProps( handler.GetDObjectValue
                (ParamNames.GUSSET_BOLTS_SLOTTED_HOLES));
            config.CopyFromContentProps (handler.GetDObjectValue
                (ParamNames.GUSSET_BOLTS_BOLT_ASSEMBLY));
            config.CopyFromExtendProps (handler.GetDObjectValue
            (ParamNames.GUSSET_BOLTS_BOLT_LENGTH_INCREASE));
            config.CopyFromShapeProps (handler.GetIntValue
            (ParamNames.GUSSET_BOLTS_STAGGERING_OF_BOLTS));
            
            AllParamData.HorBoltsNum = handler.GetIntValue
            (ParamNames.GUSSET_BOLTS_HORIZONTAL_BOLTS_NUMBER);
            AllParamData.HorBoltsSpace = handler.GetDoubleArrayValue
            (ParamNames.GUSSET_BOLTS_HORIZONTAL_BOLTS_SPACING);
            AllParamData.HorTopEdgeDist = handler.GetDoubleValue
            (ParamNames.GUSSET_BOLTS_HORIZONTAL_BOLTS_TOP_EDGE_DIST);
            AllParamData.HorBtmEdgeDist = handler.GetDoubleValue
            (ParamNames.GUSSET_BOLTS_HORIZONTAL_BOLTS_BOTTOM_EDGE_DIST);
                
            AllParamData.VerBoltsNum = handler.GetIntValue
            (ParamNames.GUSSET_BOLTS_VERTICAL_BOLTS_NUMBER);
            AllParamData.VerBoltsSpace = handler.GetDoubleArrayValue
            (ParamNames.GUSSET_BOLTS_VERTICAL_BOLTS_SPACING);
            AllParamData.VerNearEdgeDist = handler.GetDoubleValue
            (ParamNames.GUSSET_BOLTS_VERTICAL_BOLTS_NEAR_EDGE_DIST);

            AllParamData.MainPartBoltsNum = handler.GetIntValue
                (ParamNames.GUSSET_BOLTS_MAIN_PART_BOLTS_NUMBER);
            AllParamData.MainPartBoltsSpace = handler.GetDoubleArrayValue
                (ParamNames.GUSSET_BOLTS_MAIN_PART_BOLTS_SPACING);
            AllParamData.MainPartCenterEdgeDist = handler.GetDoubleValue
                (ParamNames.GUSSET_BOLTS_MAIN_PART_BOLTS_CENTER_EDGE_DIST);
            
            AllParamData.MainPartMadeType = handler.GetIntValue
                (ParamNames.GUSSET_BOLTS_SELECT_MAIN_PART_BOLTS_MADE_TYPE);
            AllParamData.LinkedPartMadeType = handler.GetIntValue
                (ParamNames.GUSSET_BOLTS_SELECT_LINKED_PART_BOLTS_MADE_TYPE);
            AllParamData.SelectBoltsOnGussetPlate = handler.GetIntValue
                (ParamNames.GUSSET_BOLTS_SELECT_BOLTS_ON_GUSSET_PLATE);
            config.m_basicProps.MadeType0 = AllParamData.LinkedPartMadeType == 2 
                ? Define.PartMadeType.PART_MADE_WORKSHOP : Define.PartMadeType.PART_MADE_SITE;
            config.m_basicProps.MadeType1 = AllParamData.MainPartMadeType== 2 
                ? Define.PartMadeType.PART_MADE_WORKSHOP : Define.PartMadeType.PART_MADE_SITE;
            return true;
        }

        public override bool Prepare()
        {
            if (!PrepareBoltData()) return false;
            return base.Prepare();
        }

        private bool PrepareBoltData()
        {
            var subPart = ParserSubPart;
            var axisX = subPart.AxisY;
            var axisY = subPart.AxisX;
            ConnPlateBoltsData.m_workAxis = new Mat43
                (axisX,axisY,axisX.Cross(axisY),Vec3.ZERO);
            ConnPlateBoltsData.RotationSide = Define.ObjectSide.SIDE_TOP;
            return true;
        }

        public override bool Execute()
        {
            return true;
        }

        /// <summary>
        /// 计算螺栓的构建点，由于流程控制吗，在外部调用。
        /// </summary>
        /// <param name="d"></param>
        public void CalculateBoltBuildPoint(double d)
        {
            var width = d;
            var height = GetConnectionPlateHeight();
            var gusset = BuildGussetPlate;
            BtmBoltBuild = gusset.m_buildPointsInFacePlane.First();
            TopBoltBuild = gusset.m_buildPointsInFacePlane.Last();
            var subPart = ParserSubPart;
            var distTopBtm = TopBoltBuild.Dist(BtmBoltBuild);
            var distX = width / 2 - subPart.AllParamData.OffSetForFaceBuildPlane;
            PointHelper.MoveRef(ref TopBoltBuild, subPart.AxisX, distX);
            PointHelper.MoveRef(ref BtmBoltBuild, subPart.AxisX, distX);
            var distY = distTopBtm - height;
            var axisY = subPart.AxisY;
            PointHelper.MoveRef(ref BtmBoltBuild, axisY, distY);

            //var distZ = m_context.BuildConnPlate.AllParamData.Thickness / 2;
            var distZ = m_pub.ConnectionPlatePro.Thickness / 2;
            var axisZ = -subPart.RefSubBuildPlane.Normal;
            PointHelper.MoveRef(ref TopBoltBuild, axisZ, distZ);
            PointHelper.MoveRef(ref BtmBoltBuild, axisZ, distZ);
        }

        /// <summary>
        /// 当连接板宽度未填写时的自适应宽度
        /// </summary>
        /// <returns></returns>
        public double GetConnectionPlateWidth()
        {
            //TODO 垂直方向不存在FarEdgeDist
            var ret = ConnPlateBoltsData.ArrayStrideY.Sum();
            ret += AllParamData.VerNearEdgeDist*2 ;
            return ret;
        }

        /// <summary>
        /// 处理当节点宽度有填写的时候。
        /// </summary>
        public void HandlerWightHasFill()
        {
            m_bWightHasFill = true;

            //var halfWidth= m_context.BuildConnPlate.AllParamData.Width/2;
            var halfWidth= m_pub.ConnectionPlatePro.Width/2;
            var halfSpacing = ConnPlateBoltsData.ArrayStrideY.Sum() / 2;
            var off = AllParamData.VerNearEdgeDist;
            if (off <= 2 * halfWidth)
            {
                ConnPlateBoltsData.VerticalOffset = -(halfWidth - halfSpacing - off);
            }
            else
            {
                ConnPlateBoltsData.VerticalOffset = 0.0d;
            }
        }

        public void HandlerWightNoFill()
        {
            m_bWightHasFill = false;
        }

        public double GetConnectionPlateHeight()
        {
            return ConnPlateBoltsData.ArrayStrideX.Sum() +
                   AllParamData.HorBtmEdgeDist + AllParamData.HorTopEdgeDist;
        }
        
    }
}