﻿using LoongCAD.STC.Plugins;
using TubeGusset_20_.Main;

namespace TubeGusset_20_.Running
{
    public class PublicParamData:RunningBaseClass
    {
        public PublicParamData(ComponentInstanceContext context) : base(context)
        {
            m_context.PublicParamData = this;
        }

        
        /// <summary>
        ///支撑部分与节点板边缘的距离（有封板时，封板算作支撑部分）
        /// </summary>
        public double pic_distance_between_the_gusset_plate_and_brace_part;
        /// <summary>
        /// 端板数据
        /// </summary>
        public readonly PlateProperties EndPlate = new PlateProperties();
        /// <summary>
        /// 中间端板的数据
        /// </summary>
        public readonly PlateProperties MiddleEndPlate = new PlateProperties();
        /// <summary>
        /// 连接板的板属性
        /// </summary>
        public readonly PlateProperties ConnectionPlatePro = new PlateProperties();
        /// <summary>
        /// 连接部分连接板的板属性
        /// </summary>
        public readonly PlateProperties ConnPartConnPlate = new PlateProperties();
        /// <summary>
        /// 连接部分盖板的板属性
        /// </summary>
        public readonly PlateProperties CoverPlate = new PlateProperties();
        /// <summary>
        /// 连接部分舌板的板属性
        /// </summary>
        public readonly PlateProperties TonguePlate = new PlateProperties();
        /// <summary>
        /// 节点板的形状选择
        /// </summary>
        public int ShapeSelect { get; private set; }
        /// <summary>
        /// 节点板的解析器
        /// </summary>
        internal PlateProperties GussetPlatePro { get; set; } = new PlateProperties();
        /// <summary>
        /// 构建连接板和L截面板的选择
        /// </summary>
        public int SelectConnectionPlateOrLProfile { get; private set; }
        /// <summary>
        /// 选择L截面的对齐方式
        /// </summary>
        public int SelectClipAngleOrientation { get; private set; }
        /// <summary>
        /// 节点板是否穿过主零件
        /// </summary>
        public int SelectGstPassThroughTheMainPart{ get; private set; }
        /// <summary>
        /// 节点板穿过著零件的距离
        /// </summary>
        public double GstPassThroughTheMainPartDist { get; private set; }
        /// <summary>
        /// 选择支柱连接部分的类型
        /// </summary>
        public int SelectConnPartType { get; private set; }
        /// <summary>
        /// 选择连接板的数目
        /// </summary>
        public int SelectConnPlateNum;
        /// <summary>
        /// 连接部分与节点板的距离,Z轴方向
        /// </summary>
        public double DistOfConnAndGusset;
        /// <summary>
        /// 角钢连接类型，同时会影响不使用角钢时的螺栓和焊接选择
        /// </summary>
        public int SelectClipAngleAttachmentType { get; set; }
        public override bool Init()
        {
            var handler = m_context.ParamsHandler;
            pic_distance_between_the_gusset_plate_and_brace_part = 
                handler.GetDoubleValue(ParamNames.PIC_DISTANCE_BETWEEN_THE_GUSSET_PLATE_AND_BRACE_PART);
            EndPlate.SetProps(handler.GetDObjectValue(ParamNames.BRACE_CONN_END_PLATE));
            ShapeSelect = handler.GetIntValue(ParamNames.GUSSET_GUSSET_PLATE_SHAPE_SELECT);
            GussetPlatePro.SetProps(handler.GetDObjectValue(ParamNames.GUSSET_GUSSET_PLATE));
            MiddleEndPlate.SetProps(handler.GetDObjectValue(ParamNames.BRACE_CONN_MIDDLE_END_PLATE));
            SelectConnectionPlateOrLProfile = handler.GetIntValue(ParamNames.GUSSET_SELECT_CONNECTION_OR_CLIP_ANGLES);
            SelectClipAngleOrientation = handler.GetIntValue(ParamNames.GUSSET_SELECT_CLIP_ANGLE_ORIENTATION);
            ConnectionPlatePro.SetProps (handler.GetDObjectValue(ParamNames.GUSSET_CONNECTION_PLATE));
            SelectGstPassThroughTheMainPart = handler.GetIntValue(ParamNames.PIC_SELECT_GUSSET_PLATE_DIMENSIONS);
            GstPassThroughTheMainPartDist = handler.GetDoubleValue(ParamNames.PIC_GUSSET_PLATE_CUT_DIMENSIONS);
            if (SelectGstPassThroughTheMainPart != 2) {
                GstPassThroughTheMainPartDist = 0.0d;
            }
            SelectConnPartType = handler.GetIntValue(ParamNames.BRACE_CONN_SELECT_CONNECTION_PART_TYPES);
            DistOfConnAndGusset = handler.GetDoubleValue(ParamNames.BRACE_CONN_GUSSET_AND_CONNECTION_PLATE_DISTANCE);
            ConnPartConnPlate.SetProps(handler.GetDObjectValue(ParamNames.BRACE_CONN_CONNECTION_PLATE));
            CoverPlate.SetProps(handler.GetDObjectValue(ParamNames.BRACE_CONN_COVER_PLATE));
            TonguePlate.SetProps(handler.GetDObjectValue(ParamNames.BRACE_CONN_TONGUE_PLATE));
            SelectClipAngleAttachmentType = handler.GetIntValue (ParamNames.GUSSET_BOLTS_SELECT_CLIP_ANGLE_ATTACHMENT_TYPE);
            SelectConnPlateNum = handler.GetIntValue(ParamNames.BRACE_CONN_SELECT_NUMBER_OF_CONNECTION_PLATE);
            return true;
        }
    }
}