﻿using System;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using U.S.Base_plate_1047_.Main;
using U.S.Base_plate_1047_.Main.Running;
using U.S.Base_plate_1047_.Tool;

namespace U.S.Base_plate_1047_.Running
{
    public class BuildAnchorRod:RunningBaseClass
    {
        public BuildAnchorRod(ComponentInstanceContext context) : base(context)
        {
            context.AnchorRod = this;
        }
        /// <summary>
        /// 附加板
        /// </summary>
        private BuildExtraPlates m_extraPlates;

        /// <summary>
        /// 上垫板的板信息
        /// </summary>
        public PlateData m_upperWasherPlate;
        /// <summary>
        /// 下垫板的板信息
        /// </summary>
        public PlateData m_lowerWasherPlate = new PlateData(10,100,100);
        /// <summary>
        /// 锚钉杆的直径
        /// </summary>
        public double m_anchorBoltSize = 0.0d;
        /// <summary>
        /// 螺母的截面类型
        /// </summary>
        public string m_nutProFile = "";
        /// <summary>
        /// 螺母的厚度
        /// </summary>
        public double m_nutThickness = 0.0d;
        /// <summary>
        /// 垫片的截面类型
        /// </summary>
        public string m_washerProFile = "";
        /// <summary>
        /// 垫片的厚度
        /// </summary>
        public double m_washerThickness= 0.0d;
        /// <summary>
        /// 垫片板的厚度
        /// </summary>
        public double m_washerPlateThickness = 0.0d;
        /// <summary>
        /// 能否创建锚栓
        /// </summary>
        public bool m_bCanBuildAnchorRod;

        /// <summary>
        /// 上垫板的孔的建造者
        /// </summary>
        private BoltGroupBuilder m_upperPlateBoltsBuilder;
        /// <summary>
        /// 锚钉杆类型
        /// </summary>
        public int m_anchorType = 1;
        /// <summary>
        /// 锚钉杆数据
        /// </summary>
        public AnchorRodData m_anchorRodData = new AnchorRodData();
        /// <summary>
        /// 选择窗口的bool集合
        /// </summary>
        public Dictionary<int, bool> m_selectOption = new Dictionary<int, bool>()
        {
            {1,false}, {2,false}, {3,false}, {4,false}, {5,false},
            {6,false}, {7,false}, {8,false}, {9,false}, {10,false},
        };
        /// <summary>
        /// 选择窗口的每一项的厚度值
        /// </summary>
        public Dictionary<int, double> m_selectThickness = new Dictionary<int, double>()
        {
            {1,0.0d}, {2,0.0d}, {3,0.0d}, {4,0.0d}, {5,0.0d},
            {6,0.0d}, {7,0.0d}, {8,0.0d}, {9,0.0d}, {10,0.0d},
        };

        /// <summary>
        /// 实例化的锚钉杆
        /// </summary>
        private List<SteelPolyBeam> m_anchorRodList = new List<SteelPolyBeam>();

        /// <summary>
        /// 锚钉杆上端螺纹
        /// </summary>
        private double m_threadUpLength = 0.0d;
        /// <summary>
        /// 锚钉杆下端螺纹
        /// </summary>
        private double m_threadDownLength = 0.0d;

        public BeamProperties m_washerBeamHelp;
        public BeamProperties m_nutBeamHelp;
        public PlateProperties m_upperPlateHelp;
        private BeamProperties m_anchorRodBeamHelp;
        public BuildAnchorRodSelect m_buildSelect;

        /// <summary>
        /// 执行函数
        /// </summary>
        public void Running()
        {
            if (!m_pub.m_buildAnchor)
                return;
            PrepareData();
            PrepareAnchorRodData();
            PrepareSelectData();
            //刷新基板上螺栓组的孔径
            var boltGroup = m_context.Bolts.m_boltGroup;
            if(boltGroup == null){return;}
            boltGroup.BoltSize = m_anchorBoltSize;
            boltGroup.Refresh();
            var bolts= boltGroup.Bolts;
            m_buildSelect = new BuildAnchorRodSelect(m_context, this);
            m_buildSelect.PrepareOnce();
            var anchorRodHelper = new AnchorRodHelper();
            foreach (var bolt in bolts)
            {
                SteelPolyBeam beam = null;
                if (m_bCanBuildAnchorRod)
                {
                    beam = CreateAnchorRod(bolt, anchorRodHelper);
                }
                m_buildSelect.Running(bolt, beam);
            }
            HandleAllHook();
            m_buildSelect.CreateCastPlate();
            m_extraPlates?.Running();
            HandleCompRelationships();
        }

        /// <summary>
        /// 处理锚钉杆的构件关系
        /// </summary>
        private void HandleCompRelationships()
        {
            var handler = m_context.ParamsHandler;
            var select = handler.GetIntValue(ParamNames.ANCHOR_ROD_ASSEMBLY_INCLUDED_SELECT);
            if(m_anchorRodList.Count == 0){return;}
            var main = m_anchorRodList.First();
            if((select == 2 || select == 3) && main != null)
            {
                List<long> idList = new List<long>();
                for (int i = 1; i < m_anchorRodList.Count; i++)
                {
                    m_anchorRodList[i].Flush();
                    var a = m_anchorRodList[i];
                    var b = a.PreBuilt;
                    var c = b.Id;
                    idList.Add(c);
                }
                main.PreBuilt.AddSubObjects(idList.ToArray());
                if (select != 3) return;
                var levelPlate = m_context.LevelingPlate.Plate;
                if (levelPlate != null)
                    main.PreBuilt.AddSubObject(levelPlate.PreBuilt.Id);
            }
        }

        /// <summary>
        /// 准备选择项
        /// </summary>
        private void PrepareSelectData()
        {
            var handler = m_context.ParamsHandler;
            bool CheckBool(int i)
            {
                return i == 2;
            }
            m_selectOption[1] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_CREATE_NUT_1));
            m_selectOption[2] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_CREATE_WASHER_2));
            m_selectOption[3] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_CREATE_WASHER_PLATE_3));
            m_selectOption[4] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_CREATE_NUT_4));
            m_selectOption[5] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_CREATE_WASHER_5));
            m_selectOption[6] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_CREATE_NUT_6));
            m_selectOption[7] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_CREATE_NUT_7));
            m_selectOption[8] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_CREATE_NUT_8));
            m_selectOption[9] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_CREATE_WASHER_PLATE_9));
            //特殊处理
            var select = handler.GetIntValue(ParamNames.ANCHOR_SELECT_CREATE_WASHER_PLATE_9);
            if (select == 0 && m_anchorType == 4)
            {
                m_selectOption[9] = true;
            }
            if (m_extraPlates != null)
            {
                m_extraPlates.Plate1CanBuild = m_selectOption[9];
            }
            
            m_selectOption[10] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_CREATE_NUT_10));
            //准备螺母和垫片数据
            PrepareNutData();
            PrepareWasherData();

            m_upperPlateHelp = new PlateProperties();
            m_upperPlateHelp.SetProps(handler.GetDObjectValue(ParamNames.ANCHOR_WASHER_PLATE));
            m_washerPlateThickness = m_upperPlateHelp.Thickness;
            m_upperWasherPlate = new PlateData(m_upperPlateHelp.Thickness,m_upperPlateHelp.Width,m_upperPlateHelp.Height);
            //处理螺纹
            m_threadUpLength = handler.GetDoubleValue(ParamNames.ANCHOR_LENGTH_OF_THE_UPPER_THREAD);
            m_threadDownLength = handler.GetDoubleValue(ParamNames.ANCHOR_LENGTH_OF_THE_LOWER_THREAD);
            if (Equals(m_threadDownLength, double.MaxValue))
                m_threadDownLength = 4 * m_anchorBoltSize + m_upperWasherPlate.Thickness;

            m_selectThickness[1] = m_nutThickness*Coefficient(m_selectOption[1]);
            m_selectThickness[2] = m_washerThickness*Coefficient(m_selectOption[2]);
            m_selectThickness[3] = m_washerPlateThickness*Coefficient(m_selectOption[3]);
            m_selectThickness[4] = m_washerThickness*Coefficient(m_selectOption[4]);
            m_selectThickness[5] = m_nutThickness*Coefficient(m_selectOption[5]);
            m_selectThickness[6] = m_nutThickness*Coefficient(m_selectOption[6]);
            m_selectThickness[7] = m_nutThickness*Coefficient(m_selectOption[7]);
            m_selectThickness[8] = m_nutThickness*Coefficient(m_selectOption[8]);
            m_selectThickness[9] = m_lowerWasherPlate.Thickness;
            m_selectThickness[10] = m_nutThickness*Coefficient(m_selectOption[10]);
            return;
            int Coefficient(bool b)
            {
                return b ? 1 : 0;
            }
        }

        /// <summary>
        /// 准备垫片数据
        /// </summary>
        private void PrepareWasherData()
        {
            var handler = m_context.ParamsHandler;
            m_washerBeamHelp = new BeamProperties();
            m_washerBeamHelp.SetProps(handler.GetDObjectValue(ParamNames.ANCHOR_WASHER_PROFILE));
            if (m_washerBeamHelp.Profile == "")
            {
                m_washerThickness= m_anchorBoltSize / 2; //未填写需要自适应
                m_washerProFile = "PD" + m_washerThickness * 4 + "*" + m_washerThickness;
            }
            else
            {
                double tmp = 1.0d;
                TryToAnalyzeTheBeamSection(ref tmp, m_washerBeamHelp.Profile);
                m_washerProFile = "PD" + tmp + "*" + tmp/4;
            }
            
            var thickness = handler.GetDoubleValue(ParamNames.ANCHOR_SIZE_OR_THE_THICKNESS_OF_THE_WASHER);
            m_washerThickness = thickness; 
            if (Math.Abs(thickness - double.MaxValue) < 0.01)
                m_washerThickness= m_anchorBoltSize / 2;//未填写需要自适应
            m_washerBeamHelp.Profile = m_washerProFile;
        }

        /// <summary>
        /// 准备螺栓螺母的数据
        /// </summary>
        private void PrepareNutData()
        {
            var handler = m_context.ParamsHandler;
            m_nutBeamHelp = new BeamProperties();
            m_nutBeamHelp.SetProps(handler.GetDObjectValue(ParamNames.ANCHOR_NUT_PROFILE));
            if (m_nutBeamHelp.Profile == "")
            {
                m_nutThickness = m_anchorBoltSize; //自适应   
                //m_nutProFile = "PD" + m_nutThickness * 2 + "*" + m_nutThickness / 2;
                m_nutProFile = "NUT" + m_nutThickness * 2;
            }
            else
            {
                double tmp = 1.0d;
                TryToAnalyzeTheBeamSection(ref tmp, m_nutBeamHelp.Profile);
                //m_nutProFile = "PD" + tmp+ "*" + tmp / 4;
                m_nutProFile = "NUT" + tmp;
            }
            var thickness = handler.GetDoubleValue(ParamNames.ANCHOR_SIZE_OR_LENGTH_OF_THE_NUT);
            m_nutThickness = thickness;
            if (Math.Abs(thickness - double.MaxValue) < 0.01)
                m_nutThickness = m_anchorBoltSize; //自适应   
            m_nutBeamHelp.Profile = m_nutProFile;
        }


        /// <summary>
        /// 根据基板上的螺栓组位置，创建锚钉杆
        /// </summary>
        /// <param name="bolt"></param>
        /// <param name="anchorRodHelper"></param>
        private SteelPolyBeam CreateAnchorRod(Bolt bolt, AnchorRodHelper anchorRodHelper)
        {
            var pos = new Vec3(bolt.WorldMatrix.Pos);
            var refPl = new Plane(m_context.BasePlate.m_buildTopPoint, m_context.AxisZ);
            PointHelper.MoveToRef(ref pos,refPl);
            var hookDir = HandleTheHookDir(bolt);
            //anchorRodHelper.CalculateControlPoint(-bolt.WorkAxis.Z, hookDir, m_anchorRodData, pos);
            anchorRodHelper.CalculateControlPoint(-m_context.AxisZ, hookDir, m_anchorRodData, pos);
            //创建锚钉杆
            var beam = m_context.TargetComp.CreateSteelPolyBeam(m_anchorRodData.RodProFile,
                anchorRodHelper.ReturnAllControlPointsOfAnchorRod(), true);
            m_anchorRodBeamHelp.ApplyToPart(beam);
            var pt = anchorRodHelper.ControlPoint1.Move(m_context.AxisZ, m_pub.LowerPlateOffsetD);
            m_extraPlates?.AddAnchorRod(m_anchorRodList.Count,beam,pt);
            m_anchorRodList.Add(beam);
            HandleThread(beam,anchorRodHelper);
            return beam;
        }

        /// <summary>
        /// 处理螺纹
        /// </summary>
        /// <param name="beam"></param>
        /// <param name="anchorRodHelper"></param>
        private void HandleThread(SteelPolyBeam beam, AnchorRodHelper anchorRodHelper)
        {

            string proFile = "PD" + (m_anchorBoltSize+1) + "*2";
            var point1 = new Vec3(anchorRodHelper.ControlPoint0);
            var point2 = PointHelper.Move(point1, -m_context.AxisZ, m_threadUpLength);
            if(m_threadUpLength > 0)
            {
                var cutter1 = m_context.TargetComp.CreateBeamCutter(beam, proFile, point1, point2, true);
                cutter1.ExactCutMode = true;
                cutter1.Refresh();
            }
            //处理下部切割
            if (m_anchorType != 4 )
                return;
            point1 = new Vec3(anchorRodHelper.ControlPoint1);
            point2 = PointHelper.Move(point1, m_context.AxisZ, m_threadDownLength);
            var cutter2 = m_context.TargetComp.CreateBeamCutter(beam,proFile,point2,point1,true);
            cutter2.ExactCutMode = true;
            cutter2.Refresh();
        }


        /// <summary>
        /// 处理弯钩的方向
        /// </summary>
        /// <param name="bolt"></param>
        /// <returns></returns>
        private Vec3 HandleTheHookDir(Bolt bolt)
        {
            var pos = bolt.WorldMatrix.Pos;
            var dir = new Vec3(Vec3.ZERO);
            var type = m_anchorRodData.HookDirType;
            if (type == 0 || type == 1 || type == 2)
            {
                HandleTheHookDirReferenceFlange(ref dir, pos);
                if (type == 2)
                    dir = -dir;
            }else if (type == 3 || type == 4)
            {
                HandleTheHookDirReferenceWeb(ref dir, pos);
                if (type == 4)
                    dir = -dir;
            }
            return dir;
        }

        private void HandleTheHookDirReferenceWeb(ref Vec3 dir, Vec3 pos)
        {
            if (!dir.IsValid()) throw new ArgumentException(@"Value should be valid.", nameof(dir));
            //本质上用topFlange移动获得的
            var plane = BeamPlanesHelper.GetControlPlaneInWeb(m_context.MainPlanes);
            var dist = plane.SignedDist(pos);
            dir = dist >= 0 ? new Vec3(plane.Normal) : new Vec3(-plane.Normal);
        }

        /// <summary>
        /// 处理锚钉杆的弯钩方向，参考翼缘
        /// </summary>
        /// <param name="dir"></param>
        /// <param name="pos"></param>
        private void HandleTheHookDirReferenceFlange(ref Vec3 dir, Vec3 pos)
        {
            if (!dir.IsValid()) throw new ArgumentException(@"Value should be valid.", nameof(dir));
            //本质上用topFlange移动获得的
            var plane = BeamPlanesHelper.GetControlPlaneTopToBottom(m_context.MainPlanes);
            var dist = plane.SignedDist(pos);
            dir = dist >= 0 ? new Vec3(plane.Normal) : new Vec3(-plane.Normal);
        }

        /// <summary>
        /// 处理所有的锚钉杆的弯钩
        /// </summary>
        private void HandleAllHook()
        {
            //var handler = m_context.ParamsHandler;
            //var a = handler.GetDoubleValue(ParamNames.ANCHOR_RADIUS_OF_THE_HOOK);
            var a = m_pub.m_anchorRadius;
            if (Equals(a, double.MaxValue))
                a = m_anchorBoltSize * 2;
            foreach (var beam in m_anchorRodList)
            {
                if (m_anchorType == 2)
                    HandleTheHookType2(beam,a);
                if(m_anchorType ==3)
                    HandleTheHookType3(beam,a);
            }
        }

        /// <summary>
        /// 处理锚钉杆的弯钩，锚钉杆类型为2
        /// </summary>
        /// <param name="beam"></param>
        /// <param name="a"></param>
        private void HandleTheHookType2(SteelPolyBeam beam,double a)
        {
            if (beam.CtrlPoints[1] is ChamferCtrlPoint point) //配置锚钉杆的弯曲
            {
                point.CutType = Define.CornerCutType.CORNER_CUT_ROUND;
                point.CutR = a;
                point.Flush();
            }
            beam.Flush();
        }

        /// <summary>
        /// 处理锚钉杆的弯钩，锚钉杆类型为3
        /// </summary>
        /// <param name="beam"></param>
        /// <param name="a"></param>
        private void HandleTheHookType3(SteelPolyBeam beam,double a)
        {
            if (beam.CtrlPoints[1] is ChamferCtrlPoint point1) //配置锚钉杆的弯曲
            {
                point1.CutType = Define.CornerCutType.CORNER_CUT_ROUND;
                point1.CutR = a;
                point1.Flush();
            }
            if (beam.CtrlPoints[2] is ChamferCtrlPoint point2) //配置锚钉杆的弯曲
            {
                point2.CutType = Define.CornerCutType.CORNER_CUT_ROUND;
                point2.CutR = a;
                point2.Flush();
            }
            beam.Flush();
        }
        /// <summary>
        /// 准备锚钉杆数据
        /// </summary>
        private void PrepareAnchorRodData()
        {
            var handler = m_context.ParamsHandler;
#if false
            var rodProFile = handler.GetDObjectValue(ParamNames.ANCHOR_ROD_PROFILE);
            m_anchorRodBeamHelp = new BeamProperties();
            m_anchorRodBeamHelp.SetProps(rodProFile);
            if (m_anchorRodBeamHelp.Profile == "")
                m_anchorBoltSize= 20.0d;
            else
                TryToAnalyzeTheBeamSection(ref m_anchorBoltSize, m_anchorRodBeamHelp.Profile);
#endif
            m_anchorRodBeamHelp = m_pub.AnchorRodBeamPro;
            m_anchorBoltSize = m_pub.AnchorRodSize;
            m_bCanBuildAnchorRod = m_anchorBoltSize > 0.0d;
            m_anchorRodData.RodProFile = "D" + m_anchorBoltSize;
            m_anchorRodBeamHelp.Profile = m_anchorRodData.RodProFile;
            m_anchorRodData.LengthOfTheAnchorRod = handler.GetDoubleValue(ParamNames.ANCHOR_LENGTH_OF_THE_ANCHOR_ROD);
            m_anchorRodData.LengthOfTheAnchorRodAboveTheBasePlate= 
                handler.GetDoubleValue(ParamNames.ANCHOR_LENGTH_OF_THE_ANCHOR_ROD_ABOVE_THE_BASE_PLATE);
            //准备锚钉杆类型
            m_anchorType = handler.GetIntValue(ParamNames.ANCHOR_ROD_TYPES);
            m_anchorRodData.RodType = m_anchorType;
            if (m_anchorType == 4)
            {
                m_extraPlates = new BuildExtraPlates(m_context);
            }
            //准备b,c
            var b = handler.GetDoubleValue(ParamNames.ANCHOR_WIDTH_OF_THE_HOOK);
            if (Equals(b, double.MaxValue))
                b = m_anchorRodData.LengthOfTheAnchorRod/ 5;
            m_anchorRodData.HorizontalBendingDistance = b;
            var c = handler.GetDoubleValue(ParamNames.ANCHOR_HEIGHT_OF_THE_HOOK);
            if (Equals(c, double.MaxValue))
                c = m_anchorRodData.LengthOfTheAnchorRod/ 5;
            m_anchorRodData.VerticalBendingDistance= c;
            //准备弯钩类型
            m_anchorRodData.HookDirType = handler.GetIntValue(ParamNames.ANCHOR_HOOK_DIRECTION);
            //增加弯钩半径判断。
            //仅处理双弯钩的情况
            if (Equals(m_pub.m_anchorRadius, double.MaxValue))
            {
                m_pub.m_anchorRadius = 2*m_anchorBoltSize;
            }
            if (m_anchorRodData.RodType == 3)
            {
                if (m_anchorRodData.HorizontalBendingDistance < m_pub.m_anchorRadius * 2)
                {
                    m_anchorRodData.HorizontalBendingDistance = m_pub.m_anchorRadius * 2;
                }
            }
        }
        /// <summary>
        /// 尝试解析梁截面
        /// </summary>
        /// <param name="anchorBoltSize"></param>
        /// <param name="str"></param>
        /// <exception cref="ArgumentOutOfRangeException"></exception>
        private void TryToAnalyzeTheBeamSection(ref double anchorBoltSize, string str)
        {
            if (anchorBoltSize < 0.0d) throw new ArgumentOutOfRangeException(nameof(anchorBoltSize));
            // 提取第一个数字组
            var numberGroup = new string(str.SkipWhile(c => !char.IsDigit(c)).TakeWhile(char.IsDigit).ToArray());
            anchorBoltSize = !string.IsNullOrEmpty(numberGroup) ? double.Parse(numberGroup) : 0.0d;
        }

        public static double TryToAnalyzeTheBeamSection(string str)
        {
            var numberGroup = new string(str.SkipWhile(c => !char.IsDigit(c)).TakeWhile(char.IsDigit).ToArray());
            var anchorBoltSize = !string.IsNullOrEmpty(numberGroup) ? double.Parse(numberGroup) : 0.0d;
            return anchorBoltSize;
        }

        /// <summary>
        /// 准备数据
        /// </summary>
        private void PrepareData()
        {
            //句柄
            m_upperPlateBoltsBuilder = new BoltGroupBuilder();
            //处理孔径
        }
    }

}