﻿using System;
using System.Collections.Generic;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using U.S.Base_plate_1047_.Running;
using U.S.Base_plate_1047_.Tool;

namespace U.S.Base_plate_1047_.Main.Running
{
    public class BuildAnchorRodSelect:RunningBaseClass
    {
        /// <summary>
        /// 锚钉杆
        /// </summary>
        private readonly BuildAnchorRod m_buildAnchorRod;
        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="context"></param>
        /// <param name="buildAnchorRod"></param>
        public BuildAnchorRodSelect(ComponentInstanceContext context,BuildAnchorRod buildAnchorRod) : base(context)
        {
            m_buildAnchorRod = buildAnchorRod;
        }
        /// <summary>
        /// 铸造板的板属性
        /// </summary>
        private PlateData m_castPlateData = PlateData.ZERO;
        /// <summary>
        /// 是否能构建铸造板
        /// </summary>
        private bool m_bCanBuildCastPlate;
        /// <summary>
        /// 创建出来的铸造板的实例
        /// </summary>
        public SteelBeam m_castPlate;
        /// <summary>
        /// 铸造板的偏移
        /// </summary>
        private double m_castPlateOffset;
        /// <summary>
        /// 下方垫片板的偏移值e
        /// </summary>
        private double m_lowerPlateOffsetE = 0.0d;
        /// <summary>
        /// 下方垫片板的偏移值d
        /// </summary>
        private double m_lowerPlateOffsetD;
        /// <summary>
        /// 是否能创建垫片的孔
        /// </summary>
        private bool m_bCanBuildWasherHole;
        /// <summary>
        /// 是否能创建8-9-10
        /// </summary>
        private bool m_bCanBuildNum_8_9_10;

        /// <summary>
        /// 焊缝的bool集合
        /// </summary>
        private readonly Dictionary<int, bool> m_selectWeld = new Dictionary<int, bool>
        {
            {1,false}, {2,false}, {3,false}, {4,false}, {5,false},
            {6,false}, {7,false}, {8,false}, {9,false}, {10,false},
        };

        /// <summary>
        /// 当前处理的锚钉杆
        /// </summary>
        private SteelPolyBeam m_anchorRodNow;

        /// <summary>
        /// 铸造板属性
        /// </summary>
        private readonly PlateProperties m_castPlatePro = new PlateProperties();


        public override bool Prepare()
        {
            return base.Prepare();
        }

        public void PrepareOnce()
        {
            PrepareDate();
            PrepareSelectWeldData();
        }

        /// <summary>
        /// 创建
        /// </summary>
        /// <param name="bolt"></param>
        /// <param name="anchorRod"></param>
        public void Running(Bolt bolt, SteelPolyBeam anchorRod)
        {
            m_anchorRodNow = anchorRod;
            if (m_anchorRodNow == null) {
                CreateSelect3(bolt);
            }
            else {
                CreateSelect1(bolt);
                CreateSelect2(bolt);
                CreateSelect3(bolt);
                CreateSelect4(bolt);
                CreateSelect5(bolt);
                CreateSelect6(bolt);
                CreateSelect7(bolt);
                CreateSelect8(bolt);
                //CreateSelect9(bolt);
                CreateSelect10(bolt);
            }
        }

        private void PrepareSelectWeldData()
        {
            var handler = m_context.ParamsHandler;
            bool CheckBool(int i)
            {
                return i == 2;
            }
            m_selectWeld[1] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_WELD_NUT_1));
            m_selectWeld[2] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_WELD_WASHER_2));
            m_selectWeld[3] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_WELD_WASHER_PLATE_3));
            m_selectWeld[4] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_WELD_NUT_4));
            m_selectWeld[5] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_WELD_WASHER_5));
            m_selectWeld[6] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_WELD_NUT_6));
            m_selectWeld[7] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_WELD_NUT_7));
            m_selectWeld[8] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_WELD_NUT_8));
            m_selectWeld[9] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_WELD_WASHER_PLATE_9));
            m_selectWeld[10] = CheckBool(handler.GetIntValue(ParamNames.ANCHOR_SELECT_WELD_NUT_10));
        }

        /// <summary>
        /// 准备铸造板的数据
        /// </summary>
        private void PrepareDate()
        {
            var handler = m_context.ParamsHandler;
            m_castPlatePro.SetProps(handler.GetDObjectValue(ParamNames.ANCHOR_CAST_PLATE));
            m_castPlateData= new PlateData(m_castPlatePro.Thickness, m_castPlatePro.Width, m_castPlatePro.Height);
            //处理bool值
            m_bCanBuildCastPlate = m_castPlateData.Thickness > 0.0d && m_buildAnchorRod.m_bCanBuildAnchorRod;
            m_bCanBuildWasherHole= handler.GetIntValue(ParamNames.ANCHOR_SELECT_CREATE_HOLE_IN_WASHER) == 2;
            m_bCanBuildNum_8_9_10 = m_buildAnchorRod.m_anchorType == 4;
            
            if (Equals(m_castPlateData.Width,double.MaxValue))
                //m_castPlateData.Width = m_context.BasePlate.m_width;
                m_castPlateData.Width = m_pub.BasePlateWidth;
            if (Equals(m_castPlateData.Height,double.MaxValue))
                //m_castPlateData.Height = m_context.BasePlate.m_height;
                m_castPlateData.Height = m_pub.BasePlateHeight;
            m_castPlateOffset = handler.GetDoubleValue(ParamNames.
                ANCHOR_DISTANCE_BETWEEN_THE_CAST_PLATE_AND_THE_BASE_PLATE);
            m_lowerPlateOffsetD = m_pub.LowerPlateOffsetD;
            WasherBoltTolerance = GetWasherAndWasherPlateBoltTolerance();
        }

        /// <summary>
        /// 创建铸造板
        /// </summary>
        public void CreateCastPlate()
        {
            if (!m_bCanBuildCastPlate)
                return;
            var axisX = m_context.BasePlate.BuildAxisX;
            var axisY = m_context.BasePlate.BuildAxisY;
            var topToBottomH = BeamPlanesHelper.GetColumnCrossSectionTopToBottom_H(m_context.MainPlanes);
            if (Equals(topToBottomH, double.MinValue)) {
                topToBottomH = m_context.MainPartHeight;
            }
            var point = new Vec3(m_context.BasePlate.m_buildBasePlatePoint);
            var length = topToBottomH/ 2;
            var pointStart = PointHelper.Move(point,axisY,length);
            var pointEnd = PointHelper.Move(point, axisY, -length);
            PointHelper.MoveRef(ref pointStart,-m_context.AxisZ,m_castPlateOffset);
            PointHelper.MoveRef(ref pointEnd,-m_context.AxisZ,m_castPlateOffset);

            var proFile = GetCastPlateProFile(m_castPlateData);
            m_castPlate =  m_context.TargetComp.CreateSteelBeam(proFile, pointStart, pointEnd, true);
            m_castPlatePro.ApplyToPart(m_castPlate);
            if (m_castPlate == null){
                Console.WriteLine(@"create cast plate failed");
                return;
            }
            m_castPlate.WorkAxis = new Mat43(m_context.WorkMat43);
            m_castPlate.Rotation = 0.0d;
            m_castPlate.RotationSide = Define.ObjectSide.SIDE_FRONT;
            m_castPlate.VertOffsetSide= Define.ObjectSide.SIDE_FRONT;

            //var offL = m_context.BasePlate.m_height - topToBottomH;
            var offL = m_pub.BasePlateHeight - topToBottomH;
            m_castPlate.HeadOffsetZ= offL / 2;
            m_castPlate.TailOffsetZ = -(offL / 2);
            m_castPlate.Flush();

            //CreateCastPlateBolt(m_castPlateOffset-m_castPlateData.Thickness+m_context.BasePlate.m_thickness);
            CreateCastPlateBolt(m_castPlateOffset-m_castPlateData.Thickness+m_pub.BasePlateThickness);
        }

        /// <summary>
        /// 创建铸造板的螺栓组
        /// </summary>
        /// <param name="offset"></param>
        private void CreateCastPlateBolt(double offset)
        {
            var builder = new BoltGroupBuilder();
            builder.CopyPointStartAndEndFromBoltGroup(m_context.Bolts.m_boltGroup);
            PointHelper.MoveRef(ref builder.posStart,m_context.AxisZ,-offset);
            PointHelper.MoveRef(ref builder.posEnd,m_context.AxisZ,-offset);
            builder.AddSubPart(m_castPlate);
            builder.BuildHolesGroup(m_context);
            if(builder.m_boltGroup == null){return;}
            builder.CopyFromBoltGroup(m_context.Bolts.m_boltGroup);
            builder.m_boltGroup.HoleTol = GetCastPlateBoltHoleTol();
            builder.m_boltGroup.Refresh();
        }

        /// <summary>
        /// 获取铸造板孔隙
        /// </summary>
        /// <returns></returns>
        private double GetCastPlateBoltHoleTol()
        {
            var tol = m_context.ParamsHandler.GetDoubleValue(ParamNames.ANCHOR_CAST_PLATE_HOLES_TOLERANCE);
            if (Equals(tol, double.MaxValue))
                tol = m_context.Bolts.m_boltGroup.HoleTol;
            return tol;
        }

        /// <summary>
        /// 获取铸造板的截面类型
        /// </summary>
        /// <param name="castPlate"></param>
        /// <returns></returns>
        private string GetCastPlateProFile(PlateData castPlate)
        {
            //以PL50*500为例
            var a = castPlate.Thickness;
            var b = castPlate.Width;
            var ret = "PL" + a + "*" + b;
            return ret;
        }
        
        private object ExtractFirstNumber(string str)
        {
            string number = "";
            bool numberFound = false;
            foreach (var c in str)
            {
                if (char.IsDigit(c))
                {
                    number += c;
                    numberFound = true;
                }else if (numberFound)
                {
                    break;
                }
            }
            return number;
        }

        /// <summary>
        /// 创建编号1,基板上方螺母
        /// </summary>
        /// <param name="bolt"></param>
        private void CreateSelect1(Bolt bolt)
        {
            if (m_buildAnchorRod.m_selectOption[1])
            {
                var offset = m_buildAnchorRod.m_selectThickness[1] +
                             m_buildAnchorRod.m_selectThickness[2] +
                             m_buildAnchorRod.m_selectThickness[3];
                var sub = CreateNutOrWasher(m_buildAnchorRod.m_nutProFile,bolt,m_buildAnchorRod.m_selectThickness[1],offset);
                m_buildAnchorRod.m_nutBeamHelp.ApplyToPart(sub);//填充数据
                if (m_selectWeld[1])
                    m_context.Weld.BuildWeldSeamForNutOrWasher(m_anchorRodNow,sub);
            }
        }

        /// <summary>
        /// 创建编号二，基板上方垫片
        /// </summary>
        /// <param name="bolt"></param>
        private void CreateSelect2(Bolt bolt)
        {
            if (m_buildAnchorRod.m_selectOption[2])
            {
                var thickness = m_buildAnchorRod.m_selectThickness[2];
                var builder = new BoltGroupBuilder();
                var offset = thickness +
                             m_buildAnchorRod.m_selectThickness[3];
                
                var proFile = m_buildAnchorRod.m_washerProFile;
                var proNum = ExtractFirstNumber(proFile);
                if (m_bCanBuildWasherHole) {
                    proFile = "D" + proNum;
                }
                var sub =CreateNutOrWasher(proFile,bolt,
                    thickness,offset,builder);
                m_buildAnchorRod.m_washerBeamHelp.ApplyToPart(sub);//填充其他属性
                sub.ProfileName = proFile;
                sub.Flush();
                if (m_selectWeld[2])
                    m_context.Weld.BuildWeldSeamForNutOrWasher(m_anchorRodNow,sub);
                if(!m_bCanBuildWasherHole)
                    return;
                //在bolt的位置创建一个螺栓
                //builder.posStart =  PointHelper.Move( bolt.WorldMatrix.Pos,bolt.WorkAxis.Z,offset);
                //builder.posEnd=  PointHelper.Move (builder.posStart,bolt.WorkAxis.X,10);
                builder.posStart =  PointHelper.Move( bolt.WorldMatrix.Pos,m_context.AxisZ,offset);
                builder.posEnd=  PointHelper.Move (builder.posStart,m_context.AxisX,10);
                builder.BuildHolesGroup(m_context);
                if(builder.m_boltGroup == null){return;}
                builder.m_boltGroup.WorkAxis = new Mat43(m_context.AxisX,m_context.AxisY,m_context.AxisZ,Vec3.ZERO);
                builder.SetAllPosAndOffsetZero();
                builder.m_boltGroup.RotationSide = Define.ObjectSide.SIDE_TOP;
                builder.m_boltGroup.BoltSize = m_context.Bolts.m_boltGroup.BoltSize;
                builder.m_boltGroup.CutLength = thickness* 2 +1;//留冗余
                //builder.m_boltGroup.HoleTol = GetWasherAndWasherPlateBoltTolerance();
                builder.m_boltGroup.HoleTol = WasherBoltTolerance;
                builder.m_boltGroup.Refresh();
            }
        }


        /// <summary>
        /// 创建编号3,基板上方垫片板
        /// </summary>
        /// <param name="bolt"></param>
        private void CreateSelect3(Bolt bolt)
        {
            if (m_buildAnchorRod.m_selectOption[3])
            {
                var thickness = m_buildAnchorRod.m_selectThickness[3];
                var builder = new BoltGroupBuilder();
                var sub = CreateWasherPlate(bolt,thickness/2,m_buildAnchorRod.m_upperWasherPlate,builder);
                m_buildAnchorRod.m_upperPlateHelp.ApplyToPart(sub);
                if (m_selectWeld[3]) //构建焊缝
                {
                    if (m_anchorRodNow == null)
                    {
                        m_context.Weld.BuildWeldSeamForWasherPlate(m_context.BasePlate.m_basePlate,sub);
                    }
                    else
                    {
                        m_context.Weld.BuildWeldSeamForWasherPlate(m_anchorRodNow,sub);
                    }
                }
                var pos = new Vec3(bolt.WorldMatrix.Pos);
                var refPl = new Plane(m_context.BasePlate.m_buildTopPoint, m_context.AxisZ);
                PointHelper.MoveToRef(ref pos,refPl);
                builder.posStart =  PointHelper.Move( pos,m_context.AxisZ,thickness);
                builder.posEnd=  PointHelper.Move (builder.posStart,m_context.AxisX,10);
                builder.BuildHolesGroup(m_context);
                if (builder.m_boltGroup == null) { return; }
                builder.m_boltGroup.WorkAxis = new Mat43(m_context.AxisX,m_context.AxisY,m_context.AxisZ,Vec3.ZERO);
                builder.SetAllPosAndOffsetZero();
                builder.m_boltGroup.RotationSide = Define.ObjectSide.SIDE_TOP;
                builder.m_boltGroup.BoltSize = GetWasherPlateBoltSize();
                builder.m_boltGroup.CutLength = thickness* 2 +1;//留冗余
                //builder.m_boltGroup.HoleTol = GetWasherAndWasherPlateBoltTolerance();
                builder.m_boltGroup.HoleTol = WasherBoltTolerance;
                builder.m_boltGroup.Refresh();
            }
        }

        /// <summary>
        /// 创建编号4,基板下方垫片
        /// </summary>
        /// <param name="bolt"></param>
        private void CreateSelect4(Bolt bolt)
        {
            if (m_buildAnchorRod.m_selectOption[4])
            {
                var thickness = m_buildAnchorRod.m_selectThickness[4];
                var builder = new BoltGroupBuilder();
                //var offset = m_context.BasePlate.m_thickness; 
                var offset = m_pub.BasePlateThickness; 
                var proFile = m_buildAnchorRod.m_washerProFile;
                var proNum = ExtractFirstNumber(proFile);
                if (m_bCanBuildWasherHole) {
                    proFile = "D" + proNum;
                }
                var sub = CreateNutOrWasher(proFile,bolt,
                    thickness,-offset,builder);
                m_buildAnchorRod.m_washerBeamHelp.ApplyToPart(sub);//填充其他属性
                sub.ProfileName = proFile;
                sub.Flush();
                if (m_selectWeld[4])
                    m_context.Weld.BuildWeldSeamForNutOrWasher(m_anchorRodNow,sub);
                if(!m_bCanBuildWasherHole)
                    return;
                //在bolt的位置创建一个螺栓
                //builder.posStart =  PointHelper.Move( bolt.WorldMatrix.Pos,bolt.WorkAxis.Z,-offset);
                //builder.posEnd=  PointHelper.Move (builder.posStart,bolt.WorkAxis.X,10);
                builder.posStart =  PointHelper.Move( bolt.WorldMatrix.Pos,m_context.AxisZ,-offset);
                builder.posEnd=  PointHelper.Move (builder.posStart,m_context.AxisX,10);
                builder.BuildHolesGroup(m_context);
                if (builder.m_boltGroup == null) { return; }
                builder.m_boltGroup.WorkAxis = new Mat43(m_context.AxisX,m_context.AxisY,m_context.AxisZ,Vec3.ZERO);
                builder.SetAllPosAndOffsetZero();
                builder.m_boltGroup.RotationSide = Define.ObjectSide.SIDE_TOP;
                builder.m_boltGroup.BoltSize = GetWasherBoltSize();
                builder.m_boltGroup.CutLength = thickness* 2 +1;//留冗余
                //builder.m_boltGroup.HoleTol = GetWasherAndWasherPlateBoltTolerance();
                builder.m_boltGroup.HoleTol = WasherBoltTolerance;
                builder.m_boltGroup.Refresh();
            }    
        }

        /// <summary>
        /// 创建编号5，基板下方螺母
        /// </summary>
        /// <param name="bolt"></param>
        private void CreateSelect5(Bolt bolt)
        {
            if (m_buildAnchorRod.m_selectOption[5])
            {
                var thickness = m_buildAnchorRod.m_selectThickness[5];
                //var offset = m_buildAnchorRod.m_selectThickness[4] + m_context.BasePlate.m_thickness;
                var offset = m_buildAnchorRod.m_selectThickness[4] + m_pub.BasePlateThickness;
                var sub = CreateNutOrWasher(m_buildAnchorRod.m_nutProFile,bolt,thickness,-offset);
                m_buildAnchorRod.m_nutBeamHelp.ApplyToPart(sub);//填充数据
                if (m_selectWeld[5])
                    m_context.Weld.BuildWeldSeamForNutOrWasher(m_anchorRodNow,sub);
            }
        }

        /// <summary>
        /// 创建编号6，铸造板上方螺母
        /// </summary>
        /// <param name="bolt"></param>
        private void CreateSelect6(Bolt bolt)
        {
            if (m_buildAnchorRod.m_selectOption[6] && m_bCanBuildCastPlate)
            {
                var thickness = m_buildAnchorRod.m_selectThickness[6];
                //var offset = m_context.BasePlate.m_thickness + m_castPlateOffset - thickness -
                var offset = m_pub.BasePlateThickness + m_castPlateOffset - thickness -
                             m_castPlateData.Thickness;
                var sub = CreateNutOrWasher(m_buildAnchorRod.m_nutProFile,bolt,thickness,-offset);
                m_buildAnchorRod.m_nutBeamHelp.ApplyToPart(sub);//填充数据
                if (m_selectWeld[6])
                    m_context.Weld.BuildWeldSeamForNutOrWasher(m_anchorRodNow,sub);
            }
        }
        
        /// <summary>
        /// 创建编号7，铸造板下方螺母
        /// </summary>
        /// <param name="bolt"></param>
        private void CreateSelect7(Bolt bolt)
        {
            if (m_buildAnchorRod.m_selectOption[7] && m_bCanBuildCastPlate)
            {
                var thickness = m_buildAnchorRod.m_selectThickness[7];
                //var offset = m_context.BasePlate.m_thickness + m_castPlateOffset; 
                var offset = m_pub.BasePlateThickness + m_castPlateOffset; 
                var sub = CreateNutOrWasher(m_buildAnchorRod.m_nutProFile,bolt,thickness,-offset);
                m_buildAnchorRod.m_nutBeamHelp.ApplyToPart(sub);//填充数据
                if (m_selectWeld[7])
                    m_context.Weld.BuildWeldSeamForNutOrWasher(m_anchorRodNow,sub);
            }
        }

        /// <summary>
        /// 创建编号8，下部垫片板上方螺母
        /// </summary>
        /// <param name="bolt"></param>
        private void CreateSelect8(Bolt bolt)
        {
            if (m_buildAnchorRod.m_selectOption[8] && m_bCanBuildNum_8_9_10)
            {
                var thickness = m_buildAnchorRod.m_selectThickness[8];
                var offset = m_buildAnchorRod.m_anchorRodData.LengthOfTheAnchorRodBelowTheBasePlate
                    -m_lowerPlateOffsetD-m_buildAnchorRod.m_lowerWasherPlate.Thickness-thickness; 
                var sub = CreateNutOrWasher(m_buildAnchorRod.m_nutProFile,bolt,thickness,-offset);
                m_buildAnchorRod.m_nutBeamHelp.ApplyToPart(sub);//填充数据
                if (m_selectWeld[8])
                    m_context.Weld.BuildWeldSeamForNutOrWasher(m_anchorRodNow,sub);
            }
        }

        /// <summary>
        /// 创建编号9，下部垫片板
        /// </summary>
        /// <param name="bolt"></param>
        private void CreateSelect9(Bolt bolt)
        {
            if (m_buildAnchorRod.m_selectOption[9] && m_bCanBuildNum_8_9_10)
            {
                //NOTE 附加板替代
#if false
                var builder = new BoltGroupBuilder();
                var thickness = m_buildAnchorRod.m_selectThickness[9];
                var offset = m_buildAnchorRod.m_anchorRodData.LengthOfTheAnchorRodBelowTheBasePlate -
                             m_lowerPlateOffsetD - thickness / 2;
                var sub = CreateWasherPlateForLower(bolt,-offset,m_buildAnchorRod.m_lowerWasherPlate,builder);
                sub.Name = "锚栓下方板垫片";
                sub.Material = "Q235B";
                sub.NumberStart = 1001;
                sub.PrebuiltNumberStart = 1;
                sub.PrebuiltNumberPrefix = "A";
                sub.Flush();
                if (m_selectWeld[9])//构建焊缝
                    m_context.Weld.BuildWeldSeamForWasherPlate(m_anchorRodNow,sub);
                var pos = new Vec3(bolt.WorldMatrix.Pos);
                var refPl = new Plane(m_context.BasePlate.m_buildTopPoint, m_context.AxisZ);
                pos.ProjectToThis(refPl);
                builder.posStart = pos.Move(m_context.AxisZ, -offset);
                builder.posEnd = builder.posStart.Move(m_context.AxisX, 10);
                builder.BuildHolesGroup(m_context);
                if (builder.m_boltGroup == null) { return; }
                builder.m_boltGroup.WorkAxis = new Mat43(m_context.WorkMat43);
                builder.SetAllPosAndOffsetZero();
                builder.m_boltGroup.RotationSide = Define.ObjectSide.SIDE_TOP;
                builder.m_boltGroup.BoltSize = m_buildAnchorRod.m_anchorBoltSize + 1; 
                builder.m_boltGroup.CutLength = thickness* 2 +1;//留冗余
                //builder.m_boltGroup.HoleTol = GetWasherAndWasherPlateBoltTolerance();
                builder.m_boltGroup.HoleTol = WasherBoltTolerance;
                builder.m_boltGroup.Refresh();
#endif 
            }
        }

        /// <summary>
        /// 创建编号10，下部垫片板下方螺母
        /// </summary>
        /// <param name="bolt"></param>
        private void CreateSelect10(Bolt bolt)
        {
            if (m_buildAnchorRod.m_selectOption[10] && m_bCanBuildNum_8_9_10)
            {
                var thickness = m_buildAnchorRod.m_selectThickness[10];
                var offset = m_buildAnchorRod.m_anchorRodData.LengthOfTheAnchorRodBelowTheBasePlate
                    -m_lowerPlateOffsetD; 
                var sub = CreateNutOrWasher(m_buildAnchorRod.m_nutProFile,bolt,thickness,-offset);
                m_buildAnchorRod.m_nutBeamHelp.ApplyToPart(sub);//填充数据
                if (m_selectWeld[10])
                    m_context.Weld.BuildWeldSeamForNutOrWasher(m_anchorRodNow,sub);
            }
        }

        /// <summary>
        /// 返回垫圈的孔径
        /// </summary>
        /// <returns></returns>
        private double GetWasherBoltSize()
        {
            return m_context.Bolts.m_boltGroup.BoltSize;
        }

        /// <summary>
        /// 返回垫圈的孔的容许值
        /// </summary>
        /// <returns></returns>
        private double GetWasherAndWasherPlateBoltTolerance()
        {
            var ret = m_pub.m_anchorWasherHoleTolerance;
            if (Equals(ret, double.MaxValue))
                ret = m_context.Bolts.m_boltGroup.HoleTol;
            return ret;
        }
        
        private double WasherBoltTolerance { get;set; }

        /// <summary>
        /// 返回垫圈板的孔径
        /// </summary>
        /// <returns></returns>
        private double GetWasherPlateBoltSize()
        {
            return m_context.Bolts.m_boltGroup.BoltSize;
        }

        /// <summary>
        /// 在螺栓参考点创建螺母
        /// </summary>
        /// <param name="proFile"></param>
        /// <param name="bolt">参考的螺栓</param>
        /// <param name="thickness">螺母的厚度</param>
        /// <param name="offset">螺母的构建点相对于参考螺栓位置+Z的偏移距离</param>
        /// <param name="boltGroupBuilder">螺栓组的构建器</param>
        /// <returns></returns>
        private SteelBeam CreateNutOrWasher(string proFile, Bolt bolt,
            double thickness, double offset, BoltGroupBuilder boltGroupBuilder = null)
        {
            if (thickness == 0)
                thickness = m_buildAnchorRod.m_anchorBoltSize;
            var pos = bolt.WorldMatrix.Pos;
            var refPl = new Plane(m_context.BasePlate.m_buildTopPoint, m_context.AxisZ);
            PointHelper.MoveToRef(ref pos,refPl);
            //var posA = PointHelper.Move(pos, bolt.WorkAxis.Z,offset); 
            //var posB = PointHelper.Move(posA, -bolt.WorkAxis.Z,thickness); 
            var posA = PointHelper.Move(pos, m_context.AxisZ,offset); 
            var posB = PointHelper.Move(posA, -m_context.AxisZ,thickness); 
            var beam=m_context.TargetComp.CreateSteelBeam(proFile,posA,posB,true);
            boltGroupBuilder?.AddSubPart(beam);
            return beam;
        }

        /// <summary>
        /// 创建垫片板
        /// </summary>
        /// <param name="bolt">参考的螺栓</param>
        /// <param name="offset">x相对螺栓+Z轴的偏移</param>
        /// <param name="plateData">板的数据</param>
        /// <param name="boltGroupBuilder">螺栓组构建器</param>
        private SteelPlate CreateWasherPlate(Bolt bolt, double offset,
            PlateData plateData, BoltGroupBuilder boltGroupBuilder)
        {
            if (!plateData.IsValid())
                return null;
            var pos = new Vec3(bolt.WorldMatrix.Pos);
            var refPl = new Plane(m_context.BasePlate.m_buildTopPoint, m_context.AxisZ);
            PointHelper.MoveToRef(ref pos,refPl);
            PointHelper.MoveRef(ref pos,bolt.WorkAxis.Z,offset);
            var plateHelper = new RectangularPlateHelper();
            plateHelper.CalculateControlPoint(m_context.BasePlate.BuildAxisX,m_context.BasePlate.BuildAxisY,plateData,pos);
            var plate = m_context.TargetComp.CreateSteelPlate
                (plateHelper.ReturnAllControlPoints(),plateData.Thickness,
                    Define.ObjectSide.SIDE_MIDDLE,true);
            boltGroupBuilder?.AddSubPart(plate);
            return plate;
        }
        
        /// <summary>
        /// 创建垫片板
        /// </summary>
        /// <param name="bolt">参考的螺栓</param>
        /// <param name="offset">x相对螺栓+Z轴的偏移</param>
        /// <param name="plateData">板的数据</param>
        /// <param name="boltGroupBuilder">螺栓组构建器</param>
        private SteelBeam CreateWasherPlateForLower(Bolt bolt, double offset,
            PlateData plateData, BoltGroupBuilder boltGroupBuilder)
        {
            if (!plateData.IsValid())
                return null;
            var pos = new Vec3(bolt.WorldMatrix.Pos);
            var refPl = new Plane(m_context.BasePlate.m_buildTopPoint, m_context.AxisZ);
            PointHelper.MoveToRef(ref pos,refPl);
            //PointHelper.MoveRef(ref pos,bolt.WorkAxis.Z,offset);
            PointHelper.MoveRef(ref pos,m_context.AxisZ,offset);
            var point1 = PointHelper.Move(pos, m_context.BasePlate.BuildAxisX,plateData.Height/2);
            var point2 = PointHelper.Move(pos, m_context.BasePlate.BuildAxisX,-plateData.Height/2);
            string proFile = "PL" + plateData.Thickness + "*" + plateData.Width;
            var plate = m_context.TargetComp.CreateSteelBeam(proFile, point1, point2, true);
            plate.WorkAxis = new Mat43(m_context.AxisX, m_context.AxisY, m_context.AxisZ, Vec3.ZERO);
            plate.Rotation = 0.0d;
            plate.RotationSide = Define.ObjectSide.SIDE_FRONT;
            plate.Flush();
            boltGroupBuilder?.AddSubPart(plate);
            return plate;
        }
    }
}