﻿using System;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using U.S.Base_plate_1047_.Main;
using U.S.Base_plate_1047_.Main.Running;
using U.S.Base_plate_1047_.Tool;

namespace U.S.Base_plate_1047_.Running
{
    /// <summary>
    /// 构建螺栓
    /// </summary>
    public class BuildBolts : RunningBaseClass
    {
        public BuildBolts(ComponentInstanceContext context) : base(context)
        {
            m_context.Bolts = this;
        }

        /// <summary>
        /// 构建点A，位于x轴下方，y轴左方 
        /// </summary>
        private Vec3 m_buildPointA = Vec3.ZERO;
        /// <summary>
        /// 构建点B，位于x轴下方，y轴右方
        /// </summary>
        private Vec3 m_buildPointB = Vec3.ZERO;

        /// <summary>
        /// A、B两点的距离
        /// </summary>
        private double m_distABH = 0;
        private double m_distABW = 0;
        
        /// <summary>
        /// 螺栓工作平面的x轴
        /// </summary>
        private Vec3 m_axisX = Vec3.ZERO;
        /// <summary>
        /// 螺栓工作平面的y轴
        /// </summary>
        private Vec3 m_axisY = Vec3.ZERO;
        
        /// <summary>
        /// 螺栓边距,最左侧的螺栓距离基板左边缘的距离
        /// </summary>
        private double m_horizontalOffset_left;
        /// <summary>
        /// 螺栓边距,最右侧的螺栓距离基板右边缘的距离
        /// </summary>
        private double m_horizontalOffset_right;
        
        /// <summary>
        /// 螺栓边距,最上侧的螺栓距离基板上边缘的距离
        /// </summary>
        private double m_verticalOffset_top = 0.0d;
        /// <summary>
        /// 螺栓边距,最下侧的螺栓距离基板下边缘的距离
        /// </summary>
        private double m_verticalOffset_bottom = 0.0d;
        
        

        /// <summary>
        /// 螺栓组的X轴方向的间隔
        /// </summary>
        private double[] m_arrayStrideX = null;

        /// <summary>
        /// 螺栓组的头部偏移X
        /// </summary>
        private double m_boltsHeadOffsetX = 0.0;
        
        /// <summary>
        /// 螺栓 组的头部偏移Y
        /// </summary>
        private double m_boltsVerticalOffset = 0.0;
        
        /// <summary>
        /// 螺栓组的Y轴方向的间隔
        /// </summary>
        private double[] m_arrayStrideY = null;

        /// <summary>
        /// 水平螺栓组的数量
        /// </summary>
        private int m_horizontalBoltsNum = 2;
        /// <summary>
        /// 垂直螺栓组的数量
        /// </summary>
        private int m_verticalBoltsNum = 2;

        /// <summary>
        /// 螺栓组
        /// </summary>
        public BoltGroup m_boltGroup = null;
        private bool m_bHasInputArrX;
        private bool m_bHasInputArrY;

        public void Running()
        {
            PrepareData();
            var target = m_context.TargetComp;
            
            //创建板实例
            m_context.BasePlate.CreateBasePlate();
            if (m_context.BasePlate.m_basePlate == null)
            {
                return;
            }
            var buildLevelingPlate =new BuildLevelingPlate(m_context);
            buildLevelingPlate.Running();
            var buildFittingPlate =new BuildFittingPlate(m_context);
            buildFittingPlate.Running();
            var buildShearKey =new BuildShearKey(m_context);
            buildShearKey.Running();
            
            //拿去创建螺栓还是创建孔
            var select = m_context.ParamsHandler.GetIntValue(ParamNames.PARAMETERS_BASE_PLATE_WITH_HOLES_OR_BOLTS);
            bool buildBolts = !(select == 0 || select == 1);
            select = m_context.ParamsHandler.GetIntValue(ParamNames.ANCHOR_SELECT_CAN_BUILD);
            bool buildAnchor = !(select == 0 || select == 1);

            bool bCanBuild = m_verticalBoltsNum > 0 && m_horizontalBoltsNum > 0;
            if (!bCanBuild) { return; }
            #region 创建螺栓 or 孔 
            //需要板创建好的实例，但是板需要在螺栓这边处理好数据后，再确立好自己的位置进行创建
            if (m_context.GetBoltsAllSubPartBases().Count == 0)
            {

                if (buildBolts && !buildAnchor)
                {
                    m_boltGroup = target.CreateBoltGroup
                    (null,m_context.BasePlate.m_basePlate,
                        m_buildPointA,m_buildPointB,true);
                    buildBolts = true;
                }
                else
                {
                    m_boltGroup = target.CreateBoltHoles(null, m_context.BasePlate.m_basePlate,
                        m_buildPointA, m_buildPointB, true);
                    buildBolts = false;
                }
            }
            else
            {
                if (buildBolts && !buildAnchor)
                {
                    m_boltGroup = target.CreateBoltGroup
                    (m_context.BasePlate.m_basePlate,m_context.GetBoltsAllSubPartBases().ToArray(),
                        m_buildPointA,m_buildPointB,true);
                    buildBolts = true;
                }
                else
                {
                    m_boltGroup = target.CreateBoltHoles
                    (m_context.BasePlate.m_basePlate,m_context.GetBoltsAllSubPartBases().ToArray(),
                        m_buildPointA,m_buildPointB,true);
                    buildBolts = false;
                }
            }
            #endregion
            if(m_boltGroup == null){return;}
            //m_boltGroup.WorkAxis = new Mat43(m_axisX, m_axisY, m_axisX.Cross(m_axisY), Vec3.ZERO);//确定工作平面
            m_boltGroup.WorkAxis = new Mat43(m_axisX, m_axisY, m_context.AxisZ, Vec3.ZERO);//确定工作平面
            
            m_boltGroup.Rotation = 0.0d;
            m_boltGroup.VerticalOffset = 0.0d;
            m_boltGroup.DepthOffset = 0.0d;
            m_boltGroup.HeadOffsetX = 0.0d;
            m_boltGroup.HeadOffsetY = 0.0d;
            m_boltGroup.HeadOffsetZ = 0.0d;
            m_boltGroup.TailOffsetX = 0.0d;
            m_boltGroup.TailOffsetY = 0.0d;
            m_boltGroup.TailOffsetZ = 0.0d;
            
            //m_boltGroup.ArrayStrideX = m_arrayStrideX ?? AdaptiveBoltSpacing(m_verticalBoltsNum - 1, m_distABH-(m_verticalOffset_top+ m_verticalOffset_bottom));
            //m_boltGroup.ArrayStrideY = m_arrayStrideY ?? AdaptiveBoltSpacing(m_horizontalBoltsNum - 1, m_distABW-(m_horizontalOffset_left+m_horizontalOffset_right));
            m_boltGroup.ArrayStrideX = m_arrayStrideX ;
            m_boltGroup.ArrayStrideY = m_arrayStrideY ;
            var handler = m_context.ParamsHandler;
            select=handler.GetIntValue(ParamNames.BOLTS_BOLTING_DIRECTION);
            m_boltGroup.RotationSide = Define.ObjectSide.SIDE_TOP;
            if (select == 2)
                m_boltGroup.RotationSide = Define.ObjectSide.SIDE_BOTTOM;
            //自适应值x轴方向的偏移值
            m_boltGroup.HeadOffsetX = m_boltsHeadOffsetX;
            m_boltGroup.VerticalOffset = m_boltsVerticalOffset;
            m_boltGroup.CutLength = m_context.GetBoltsCutLength();  
            //配置孔
            var boltCfg = new BoltsConfig();
            boltCfg.CopyFromBasicProps(handler.GetDObjectValue(ParamNames.BOLTS_BOLT_BASIC_PROPERTIES));
            
            boltCfg.CopyFromShapeProps(m_pub.SelectBoltShape);
            
            boltCfg.CopyFromContentProps(handler.GetDObjectValue(ParamNames.BOLTS_BOLT_ASSEMBLY));
            boltCfg.CopyFromExtendProps(handler.GetDObjectValue(ParamNames.BOLTS_BOLT_LENGTH_INCREASE));
            boltCfg.CopyFromHoleProps(handler.GetDObjectValue(ParamNames.BOLTS_SLOTTED_HOLES));
            boltCfg.FillBasicProps(m_boltGroup);
            boltCfg.FillShape(m_boltGroup);
            boltCfg.FillExtendProps(m_boltGroup);
            boltCfg.FillHoleProps(m_boltGroup);
            
            //NOTE 处理螺栓忽略
            var ignoreHand = new HandlerBoltIgnore(m_context, m_boltGroup);
            ignoreHand.Run();
            
            if (!buildBolts) {
                if(buildAnchor)
                    m_boltGroup.RotationSide = Define.ObjectSide.SIDE_TOP;//避免对齐问题
                m_boltGroup.Refresh();
                return;
            } 
            boltCfg.FillContentProps(m_boltGroup);
            m_boltGroup.Refresh();
        }

        public void PrepareData()
        {
            //先拿到底部点
            var basePlate = m_context.BasePlate;
            var buildPos = basePlate.m_buildBasePlatePoint;
            m_axisX = new Vec3(-basePlate.BuildAxisY);
            m_axisY = new Vec3(basePlate.BuildAxisX);
            //配置控制点
            //第一步，在处理螺栓其他输入之前，确立控制点位于正中放置的板的边缘
            m_buildPointA = PointHelper.Move(buildPos, -basePlate.BuildAxisY, m_pub.BasePlateHeight/ 2);
            m_buildPointB = PointHelper.Move(buildPos, basePlate.BuildAxisY, m_pub.BasePlateHeight/ 2);
#if DEBUG
            m_context.TargetComp.CreatePoint(buildPos,true);
            m_context.TargetComp.CreatePoint(m_buildPointA,true);
            m_context.TargetComp.CreatePoint(m_buildPointB,true);
#endif
            
        }

        public override bool Init()
        {
            InitData();
            return true;
        }

        private void InitData()
        {
            var handler = m_context.ParamsHandler;
            m_horizontalOffset_left = handler.GetDoubleValue(ParamNames.BOLTS_BOLT_EDGE_LEFT_DISTANCE);
            m_horizontalOffset_right = handler.GetDoubleValue(ParamNames.BOLTS_BOLT_EDGE_RIGHT_DISTANCE);
            m_verticalOffset_top = handler.GetDoubleValue(ParamNames.BOLTS_BOLT_EDGE_TOP_VERTICAL_DISTANCE);
            m_verticalOffset_bottom = handler.GetDoubleValue(ParamNames.BOLTS_BOLT_EDGE_BOTTOM_VERTICAL_DISTANCE);
            m_distABH= m_pub.BasePlateHeight;
            m_distABW = m_pub.BasePlateWidth;
            //处理边距和数目
            InitArrayStride();
            InitHorSide();
            InitVerSide();

#if false
            //处理HeadOffset,保障螺栓的居中距离
            //仅处理X方向，因为X方向是从控制点
            //这里会和偏移相关,即BoltsHeadOffset
            //因为y轴的创建，默认是居中的，所以在居中选项下，不用处理
            if (m_arrayStrideX != null) {
                var sum = m_arrayStrideX.Sum();
                m_boltsHeadOffsetX= (m_distABH-sum)/2;
                //m_boltsHeadOffsetX = m_verticalOffset_top;
            }
            else {
                m_boltsHeadOffsetX = m_verticalOffset_top;
            }
            //获取螺栓边距
            InitOffset();
#endif
        }
        private void InitHorSide()
        {
            var left = m_horizontalOffset_left;
            var right = m_horizontalOffset_right;
            var boltArea = 0.0d;
            var baseWidth = m_pub.BasePlateWidth;
            m_arrayStrideY = m_arrayStrideY ??AdaptiveBoltSpacing(m_horizontalBoltsNum - 1,baseWidth-(left + right)); 
            if (m_arrayStrideY != null)
            {
                boltArea += m_arrayStrideY.Sum();
            }
            var boltOff = baseWidth / 2 - boltArea / 2 - left;
            //处理基板的偏移
            var mainWidth = m_context.MainPartWidth;
            var handler = m_context.ParamsHandler;
            var select = handler.GetIntValue(ParamNames.BOLTS_SELECT_HOW_TO_MEASURE_THE_DIMENSIONS_FOR_HORIZONTAL_BOLT_GROUP_POSITION);
            var offset = handler.GetDoubleValue(ParamNames.BOLTS_DIMENSION_FOR_HORIZONTAL_BOLT_GROUP_POSITION);
            if (select == 0 || select == 2)
            {
                m_pub.MoveBuildPoint_X(offset);
            }else if (select == 1)
            {
                var plateOff = left + mainWidth / 2 - baseWidth / 2;
                m_pub.MoveBuildPoint_X(-plateOff);
                m_pub.MoveBuildPoint_X(offset);
                boltOff = baseWidth / 2 - boltArea / 2 - left;
            }else if (select == 3)
            {
                var plateOff = right + mainWidth / 2 - baseWidth / 2;
                m_pub.MoveBuildPoint_X(plateOff);
                m_pub.MoveBuildPoint_X(offset);
                boltOff = baseWidth / 2 - boltArea / 2 - right;
                boltOff = -boltOff;
            }
            m_boltsVerticalOffset = boltOff;
        }

        private void InitVerSide()
        {
            var top = m_verticalOffset_top;
            var bottom = m_verticalOffset_bottom;
            var boltArea = 0.0d;
            var baseHeight = m_pub.BasePlateHeight;
            m_arrayStrideX = m_arrayStrideX ?? AdaptiveBoltSpacing(m_verticalBoltsNum - 1, baseHeight - (top + bottom));
            if (m_arrayStrideX != null)
            {
                boltArea += m_arrayStrideX.Sum();
            }
            var boltOff = bottom;
            var mainHeight = m_context.MainPartHeight;
            var handler = m_context.ParamsHandler;
            var select = handler.GetIntValue(ParamNames.BOLTS_SELECT_HOW_TO_MEASURE_THE_DIMENSIONS_FOR_VERTICAL_BOLT_GROUP_POSITION);
            var offset = handler.GetDoubleValue(ParamNames.BOLTS_DIMENSION_FOR_VERTICAL_BOLT_GROUP_POSITION);
            if (select == 0 || select == 2)
            {
                m_pub.MoveBuildPoint_Y(offset);
            }else if (select == 1)
            {
                var plateOff = top + mainHeight / 2 - baseHeight / 2;
                m_pub.MoveBuildPoint_Y(plateOff);
                m_pub.MoveBuildPoint_Y(offset);
            }else if (select == 3)
            {
                var plateOff = bottom + mainHeight / 2 - baseHeight / 2;
                m_pub.MoveBuildPoint_Y(-plateOff);
                m_pub.MoveBuildPoint_Y(offset);
            }
            m_boltsHeadOffsetX = boltOff;
        }

        /// <summary>
        /// 处理螺栓数目和边距输入的自适应
        /// 给最终创建螺栓组时候的间距数组赋值
        /// </summary>
        private void InitArrayStride()
        {
            var handler = m_context.ParamsHandler;
            //获取螺栓数目
            m_horizontalBoltsNum = handler.GetIntValue(ParamNames.BOLTS_NUMBER_OF_HORIZONTAL_BOLTS);
            m_verticalBoltsNum = handler.GetIntValue(ParamNames.BOLTS_NUMBER_OF_VERTICAL_BOLTS);
            //获取螺栓间距值
            var tmpArrY = handler.GetDoubleArrayValue(ParamNames.BOLTS_VERTICAL_BOLT_SPACING);
            var tmpArrX = handler.GetDoubleArrayValue(ParamNames.BOLTS_HORIZONTAL_BOLT_SPACING);
            m_bHasInputArrX = tmpArrX?.Length > 0;
            m_bHasInputArrY = tmpArrY?.Length > 0;
            //处理螺栓数目和间距数目不统一的情况
            //生成螺栓的最终数目以螺栓数目为准
            //而间距如果不够的话，以最后的间距为准
            if (m_horizontalBoltsNum > 1 && tmpArrX!= null && tmpArrX.Length > 0)
            {
                List<double> tmp = new List<double>();
                for (var i = 1; i < m_horizontalBoltsNum; ++i)
                {
                   tmp.Add(tmpArrX[Math.Min(i-1,tmpArrX.Length-1)]); 
                }
                m_arrayStrideY = tmp.ToArray();
            }
            if (m_verticalBoltsNum > 1 && tmpArrY != null && tmpArrY.Length > 0)
            {
                List<double> tmp = new List<double>();
                for (var i = 1; i < m_verticalBoltsNum; ++i)
                {
                    tmp.Add(tmpArrY[Math.Min(i-1,tmpArrY.Length-1)]);
                }
                m_arrayStrideX = tmp.ToArray();
            }
        }

        /// <summary>
        /// 指定数目和总长，返回间距数组
        /// 当num为0时，返回一个空间隔数组
        /// </summary>
        /// <param name="num">间距数目</param>
        /// <param name="dist">总长度</param>
        /// <returns></returns>
        private double[] AdaptiveBoltSpacing(int num, double dist)
        {
            List<double> ret = new List<double>();
            if (num <= 0)
            {
                return ret.ToArray();
            }
            var step = dist / num;
            for (var i = 0; i < num; ++i)
            {
                ret.Add(step);
            }
            return ret.ToArray();
        }
    }

    public class HandlerBoltIgnore:RunningBaseClass
    {
        public HandlerBoltIgnore(ComponentInstanceContext context, BoltGroup boltGroup) : base(context)
        {
            BoltGroup = boltGroup;
        }
        private BoltGroup BoltGroup { get; set; }
        private int[] IgnoreNums { get; set; }
        /// <summary>
        /// 螺栓的键值对：忽略的下标:具体螺栓位置
        /// </summary>
        private Dictionary<int,Vec3> BoltMap { get; set; }

        public void Run()
        {
            if(BoltGroup?.Bolts == null){return;}
            BoltGroup.Refresh();
            if(BoltGroup.Bolts.Length == 0){return;}
            var handler = m_context.ParamsHandler;
            IgnoreNums = handler.GetIntArrayValue(ParamNames.BOLTS_IGNORE_BOLT_INDEX_ARRAY);

            var bolts = BoltGroup.Bolts;
            BoltMap = new Dictionary<int, Vec3>();
            foreach (var bolt in bolts)
            {
                BoltMap.Add(bolt.ShapeIndex+1,bolt.WorldMatrix.Pos);
            }

            BoltGroup.IgnoreIndices = IgnoreNums;
            BoltGroup.Refresh();

            for (var i = 0; i < IgnoreNums.Length; i++)
            {
                var ignoreNum = IgnoreNums[i];
                if (BoltMap.TryGetValue(ignoreNum, out var pt))
                {
#if DEBUG
                    m_context.TargetComp.CreatePoint(pt,true);
#endif
                }
            }

            var select1 = handler.GetIntValue(ParamNames.BOLTS_SELECT_CAN_IGNORE_CENTRE_BOLT);
            var shape = m_pub.SelectBoltShape;
            var bShapeCan = shape == 0 || shape == 1;
            if (!bShapeCan) { return; }
            
            if (select1 != 2) { return; }
            
            var xNum = BoltGroup.ArrayStrideX?.Length;
            if(xNum == null){return;}
            var x = xNum.Value + 1;
            var yNum = BoltGroup.ArrayStrideY?.Length;
            if(yNum == null){return;}
            var y = yNum.Value + 1;
            if(x< 3 || y< 3){return;}

            
            var ignoreList = new HashSet<int>();

            for (int i = 2; i < y; i++)
            {
                for (int j = 2; j < x ; j++)
                {
                    ignoreList.Add((i-1)*x+ j);
                }
            }

            foreach (var ignoreNum in IgnoreNums)
            {
                ignoreList.Add(ignoreNum);
            }

            BoltGroup.IgnoreIndices = ignoreList.ToArray();
        }
    }
    
}