﻿using System;
using System.Collections.Generic;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using U.S.Base_plate_1047_.Main;
using U.S.Base_plate_1047_.Tool;

namespace U.S.Base_plate_1047_.Running
{
    public class BuildFittingPlate:RunningBaseClass
    {
        public BuildFittingPlate(ComponentInstanceContext context) : base(context)
        {
            m_context.FittingPlate = this;
        }

        private const int FittingNum = 3;

        /// <summary>
        /// 存储1、2、3号衬板的板属性
        /// 对应的下标为0，1，2
        /// </summary>
        public readonly PlateData[] m_allFittingPlateData= new PlateData[3];

        /// <summary>
        /// 存储1、2、3号衬板的板数量
        /// </summary>
        public readonly int[] m_allFittingPlateNum = new int[FittingNum];

        /// <summary>
        /// 存储1、2、3号衬板的板对象
        /// </summary>
        public readonly List<Plate>[] m_allFittingPlate = new List<Plate>[FittingNum];

        private readonly List<PlateProperties> m_fittingPlateHelp = new List<PlateProperties>();
        
        public void Running()
        {
            PrepareData();

            var help = new RectangularPlateHelper();
            var main = m_context.MainPart;
            var axisX = new Vec3(m_context.BasePlate.BuildAxisX);
            var axisY = new Vec3(m_context.BasePlate.BuildAxisY);

            for (var i = 0; i < FittingNum; i++)
            {
                m_allFittingPlate[i] = new List<Plate>();
            }
            
            Vec3 buildPoint;
            //如果层板存在,构建点应该从层板下方开始
            if (m_context.LevelingPlate.CanBuild)
            {
                buildPoint = m_context.LevelingPlate.m_buildPoint;
                //再往下移动厚度的一半
                buildPoint = PointHelper.Move(buildPoint, -m_context.AxisZ,
                    m_context.LevelingPlate.m_plateData.Thickness / 2);
            }
            else
            {
                buildPoint = m_context.BasePlate.m_buildBasePlatePoint;
                buildPoint = PointHelper.Move(buildPoint, -m_context.AxisZ,
                    m_pub.BasePlateThickness);
            }
            for (var i = 0; i < 3; i++)
            {
                var num = m_allFittingPlateNum[i];
                var plateData = m_allFittingPlateData[i];
                if (!plateData.IsValid())
                   continue;
                for (var k = 0; k < num; k++)
                {
                    if (Equals(plateData.Thickness, 0.0d)) {
                        continue;
                    }
                    //移动到构建几何中心
                    buildPoint = PointHelper.Move (buildPoint, -m_context.AxisZ, plateData.Thickness / 2);
                    help.CalculateControlPoint(axisX,axisY,plateData,buildPoint);
                    var plate = m_context.TargetComp.CreateSteelPlate(help.ReturnAllControlPoints(), 
                        plateData.Thickness, Define.ObjectSide.SIDE_MIDDLE, true);
                    m_allFittingPlate[i].Add(plate);
                    m_fittingPlateHelp[i].ApplyToPart(plate);
                    if (i == 1 || i == 2)
                    {
                        //衬板2、3的其余属性和1相同。
                        var fph1 = m_fittingPlateHelp[0];
                        SpecialApplyToPart(fph1, plate);
                    }

                    plate?.Flush();
                    //移动到底面
                    buildPoint = PointHelper.Move (buildPoint, -m_context.AxisZ, plateData.Thickness / 2);
                }
            }

        }

        private void SpecialApplyToPart(PlateProperties plateProperties, SteelPlate plate)
        {
            if (plateProperties == null) {
                return;
            }
            if (plate == null) {
                return;
            }
            if (plateProperties.NoPrefix != null) {
                plate.NumberPrefix = plateProperties.NoPrefix;
            }
            if (plateProperties.NoStart >= 0) {
                plate.NumberStart = plateProperties.NoStart;
            }
            if (plateProperties.NoPostfix != null) {
                plate.NumberPostfix = plateProperties.NoPostfix;
            }
            if (plateProperties.AssNoPrefix != null) {
                plate.PrebuiltNumberPrefix = plateProperties.AssNoPrefix;
            }
            if (plateProperties.AssNoStart >= 0) {
                plate.PrebuiltNumberStart = plateProperties.AssNoStart;
            }
            if (plateProperties.AssNoPostfix != null) {
                plate.PrebuiltNumberPostfix = plateProperties.AssNoPostfix;
            }
        }

        /// <summary>
        /// 准备数据
        /// </summary>
        private void PrepareData()
        {
            var handler = m_context.ParamsHandler;

            var inputData = new DObject[]
            {
                handler.GetDObjectValue(ParamNames.PARTS_FITTING_PLATE_1),
                handler.GetDObjectValue(ParamNames.PARTS_FITTING_PLATE_2),
                handler.GetDObjectValue(ParamNames.PARTS_FITTING_PLATE_3),
            };
            
            for (var i = 0; i < inputData.Length; i++)
            {
                var plate = new PlateProperties();
                var data = inputData[i];
                plate.SetProps(data);
                //NOTE 额外处理高宽
                if (Equals(plate.Width, double.MaxValue)) {
                    //plate.Width = m_context.BasePlate.m_width;
                    plate.Width = m_pub.BasePlateHeight;
                }

                if (Equals(plate.Height, double.MaxValue))
                {
                    //plate.Height = m_context.BasePlate.m_height;
                    plate.Height = m_pub.BasePlateWidth;
                }
                var plateData = new PlateData(plate.Thickness, plate.Width, plate.Height);
                m_allFittingPlateData[i] = plateData;
                m_fittingPlateHelp.Add(plate);
            }

            var intArray = new int[]
            {
                handler.GetIntValue(ParamNames.PARTS_FITTING_PLATE_1_NUM),
                handler.GetIntValue(ParamNames.PARTS_FITTING_PLATE_2_NUM),
                handler.GetIntValue(ParamNames.PARTS_FITTING_PLATE_3_NUM),
            };
            Array.Copy(intArray,m_allFittingPlateNum,3);
        }

    }
}