﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using U.S.Base_plate_1047_.Main;
using U.S.Base_plate_1047_.Tool;

namespace U.S.Base_plate_1047_.Running
{
    public class BuildGroutHole : RunningBaseClass
    {
        public BuildGroutHole(ComponentInstanceContext context) : base(context)
        {
            m_context.GroutHole = this;
        }

        /// <summary>
        /// 构建灌浆孔的对象
        /// </summary>
        public BoltGroup m_groutHole = null; 

        public void Running()
        {
            if (m_context.BasePlate.m_basePlate == null)
            {
                return;
            }
            var worldMatrix = m_context.MainPart.WorldMatrix;
            var target = m_context.TargetComp;
            var holder = m_context.ParamsHandler;
            //灌浆孔的数据
            //var buildPoint = m_pub.m_mainCenterPoint;
            var buildPoint = PointHelper.MoveTo(m_context.DetailCentrePt,m_pub.BasePlateTopPl);
            
            var horizontalOffset =
                holder.GetDoubleValue(ParamNames.PIC_GROUT_HOLE_HORIZONTAL_DISTANCE_FROM_COLUMN_CENTER);
            var verticalOffset=
                holder.GetDoubleValue(ParamNames.PIC_GROUT_HOLE_VERTICAL_DISTANCE_FROM_COLUMN_CENTER);
            var diameter = holder.GetDoubleValue(ParamNames.PIC_DIAMETER_OF_THE_GROUT_HOLE);
            //偏移
            //孔大小不存在时，不创建孔
            if (diameter <= 0.0d){
                return;
            }
            buildPoint = PointHelper.Move(buildPoint, worldMatrix.X, horizontalOffset); 
            buildPoint = PointHelper.Move(buildPoint, -worldMatrix.Y, verticalOffset);

            var point = PointHelper.Move(buildPoint, worldMatrix.X, 50.0d);
            
            //创建灌浆孔
            m_groutHole = target.CreateBoltHoles(null,m_context.BasePlate.m_basePlate, buildPoint, point, true);
            if (m_groutHole == null) {
                return;
            }
            m_groutHole.BoltSize = diameter;//螺栓尺寸，这里作为孔径
            var axisZ = m_context.MainCtlFarPt - m_context.MainCtlNearPt;
            axisZ.NormalizeThis();
            var axisX = worldMatrix.X.Normalize();
            var axisY = axisX.Cross(axisZ).Normalize();
            m_groutHole.WorkAxis = new Mat43(axisX,axisY,axisZ,Vec3.ZERO);//右手正交坐标系
            m_groutHole.Rotation = 0.0d;
            m_groutHole.RotationSide = Define.ObjectSide.SIDE_TOP;
            //m_groutHole.ListX = new double[];//间距，做额外处理
            //孔的长度问题，以及贯穿问题
            m_groutHole.Refresh();
        }

    }
}