﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using U.S.Base_plate_1047_.Main;
using U.S.Base_plate_1047_.Tool;

namespace U.S.Base_plate_1047_.Running
{
    /// <summary>
    /// 构件灌浆层(砂浆层)
    /// </summary>
    public class BuildGrouting : RunningBaseClass
    {
        public BuildGrouting(ComponentInstanceContext context) : base(context)
        {
            m_context.Grouting = this;
        }

        /// <summary>
        /// 灌浆厚度
        /// </summary>
        public double m_thickness;

        private string m_part_prefix; 
        private string m_part_num; 
        private string m_comp_prefix; 
        private string m_comp_num;

        public override bool Init()
        {
            InitData();
            return base.Init();
        }

        /// <summary>
        /// 准备数据
        /// </summary>
        private void InitData()
        {
            var handler = m_context.ParamsHandler;
            //m_thickness = handler.GetDoubleValue(ParamNames.ANCHOR_GROUTING_THICKNESS);
            m_part_prefix = handler.GetStringValue(ParamNames.ANCHOR_GROUTING_PART_PREFIX);
            m_part_num= handler.GetStringValue(ParamNames.ANCHOR_GROUTING_PART_NUMBER);
            m_comp_prefix = handler.GetStringValue(ParamNames.ANCHOR_GROUTING_COMPONENT_PREFIX);
            m_comp_num= handler.GetStringValue(ParamNames.ANCHOR_GROUTING_COMPONENT_NUMBER);
        }

        public override bool Prepare()
        {
            m_thickness = m_pub.m_groutingThickness;
            return true;
        }

        public void Running()
        {
            if (m_thickness <= 0)
                return;
            var basePlate = m_context.BasePlate;
            var height = m_pub.BasePlateHeight + 60;
            var width = m_pub.BasePlateWidth + 60;
            var buildPoint = new Vec3(basePlate.m_buildBasePlatePoint);
            //偏移自身厚度
            PointHelper.MoveRef(ref buildPoint,-m_context.AxisZ,m_thickness);
            var helper = new RectangularPlateHelper();
            var plateData = new PlateData(m_thickness, height, width);
            helper.CalculateControlPoint(basePlate.BuildAxisX,basePlate.BuildAxisY,plateData,buildPoint);
            var grouting = m_context.TargetComp.CreateSteelPlate(helper.ReturnAllControlPoints(), m_thickness,
                Define.ObjectSide.SIDE_MIDDLE, true);
            if(grouting == null){return;}
            grouting.WorkAxis = new Mat43(helper.AxisX,helper.AxisY,helper.AxisX.Cross(helper.AxisY).Normalize(),Vec3.ZERO);
            grouting.OffsetSide = Define.ObjectSide.SIDE_RIGHT;
            grouting.Name = "GROUT灌浆";
            grouting.Material = "GROUT";
            grouting.NumberPrefix = m_part_prefix;
            grouting.PrebuiltNumberPrefix= m_comp_prefix;
            //grouting.NumberStart = m_part_num;
            PartProperties.ParseNoStartPostFix(m_part_num,out var numberStart,out var numberPostfix);
            grouting.NumberStart = numberStart;
            grouting.NumberPostfix = numberPostfix;
            //grouting.PrebuiltNumberStart= m_comp_num;
            PartProperties.ParseNoStartPostFix(m_comp_num,out numberStart,out numberPostfix);
            grouting.PrebuiltNumberStart = numberStart;
            grouting.PrebuiltNumberPostfix = numberPostfix;
            grouting.Flush();
            CuttingGrouting(grouting,helper,plateData.Thickness);
        }

        private void CuttingGrouting(SteelPlate grouting, RectangularPlateHelper helper, double thickness)
        {
            var point0 = new Vec3(helper.ControlPoint0);
            var point1 = new Vec3(helper.ControlPoint1);
            var point2 = new Vec3(helper.ControlPoint2);
            var point3 = new Vec3(helper.ControlPoint3);
            Cut(grouting, helper, thickness, point0,helper.AxisY,helper.AxisX);
            Cut(grouting, helper, thickness, point1,helper.AxisX,-helper.AxisY);
            Cut(grouting, helper, thickness, point2,-helper.AxisY,-helper.AxisX);
            Cut(grouting, helper, thickness, point3,-helper.AxisX,helper.AxisY);
        }

        private void Cut(SteelPlate grouting, RectangularPlateHelper helper, double thickness, Vec3 point0,Vec3 dir1,Vec3 dir2)
        {
            var point1 = PointHelper.Move(point0, m_context.AxisZ, thickness);
            var point2 = PointHelper.Move(point1, dir1, 30);
            var line12 = point0 - point2;
            var cutPlane = new Plane(point2, line12, dir2);
            cutPlane.InvertThis();
            var cut = m_context.TargetComp.CreateCutPlane(grouting, cutPlane, true);
        }
    }
}