﻿using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using U.S.Base_plate_1047_.Main;
using U.S.Base_plate_1047_.Tool;

namespace U.S.Base_plate_1047_.Running
{
    /// <summary>
    /// 层板构建
    /// </summary>
    public class BuildLevelingPlate:RunningBaseClass
    {
        public BuildLevelingPlate(ComponentInstanceContext context) : base(context)
        {
            m_context.LevelingPlate = this;
        }

        /// <summary>
        /// 层板的构建点,实际板的几何中心点，在内部
        /// </summary>
        public Vec3 m_buildPoint = Vec3.ZERO;

        /// <summary>
        /// 层板的属性
        /// </summary>
        public PlateData m_plateData = null;

        private PlateProperties m_levelingPlateHelper;

        /// <summary>
        /// 创建好之后的板对象
        /// </summary>
        public SteelPlate Plate { get; set; } = null;

        /// <summary>
        /// 能否被创建
        /// </summary>
        public  bool CanBuild { get; private set; } = false;

        public void Running()
        {
            PrepareData();
            if (!CanBuild)
            {
                return;
            }
            //m_context.TargetComp.CreateSteelPlate();
            var help = new RectangularPlateHelper();
            var main = m_context.BasePlate;
            var axisX = main.BuildAxisX;
            var axisY = main.BuildAxisY;
            help.CalculateControlPoint(axisX,axisY,m_plateData,m_buildPoint,4);

            Plate = m_context.TargetComp.CreateSteelPlate(
                help.ReturnAllControlPoints(),
                m_plateData.Thickness,
                Define.ObjectSide.SIDE_MIDDLE,
                true);
            m_levelingPlateHelper.ApplyToPart(Plate);
        }

        /// <summary>
        /// 准备数据
        /// </summary>
        private void PrepareData()
        {
            var handler = m_context.ParamsHandler;
            //处理板数据
            m_levelingPlateHelper = new PlateProperties();
            var data = handler.GetDObjectValue(ParamNames.PARTS_LEVELING_PLATE);
            m_levelingPlateHelper.SetProps(data);
            if (m_levelingPlateHelper.Thickness == 0 || m_levelingPlateHelper.Width == 0 || m_levelingPlateHelper.Height == 0)
            {
                CanBuild = false;
                return;
            }

            if (Equals(m_levelingPlateHelper.Width, double.MaxValue))
            {
                m_levelingPlateHelper.Width = m_pub.BasePlateWidth;
            }

            if (Equals(m_levelingPlateHelper.Height, double.MaxValue))
            {
                m_levelingPlateHelper.Height = m_pub.BasePlateHeight;
            }
            
            CanBuild = true;

            m_plateData = new PlateData(m_levelingPlateHelper.Thickness, m_levelingPlateHelper.Height, m_levelingPlateHelper.Width);
            m_buildPoint = m_context.BasePlate.m_buildTopPoint;
            m_buildPoint = PointHelper.Move
                (m_buildPoint, -m_context.AxisZ, 
                    m_plateData.Thickness / 2+m_pub.BasePlateThickness);
                    //m_plateData.Thickness / 2+m_context.BasePlate.m_thickness);
        }
    }
}