﻿using System;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using U.S.Base_plate_1047_.Main;
using U.S.Base_plate_1047_.Tool;

namespace U.S.Base_plate_1047_.Running
{
    /// <summary>
    /// 构建抗剪键
    /// </summary>
    public class BuildShearKey:RunningBaseClass
    {
        public BuildShearKey(ComponentInstanceContext context) : base(context)
        {
            m_context.ShearKey = this;
        }

        /// <summary>
        /// 抗剪键的界面
        /// </summary>
        private string m_interfaceType;
        /// <summary>
        /// 抗剪键的构造起始点
        /// </summary>
        private Vec3 m_buildPoint;
        /// <summary>
        /// 抗剪键的长度
        /// </summary>
        private double m_length = 0.0d;
        /// <summary>
        /// 抗剪键的旋转角度
        /// </summary>
        private double m_rotation = 0.0d;
        /// <summary>
        /// 是否可以构建
        /// </summary>
        public bool m_canBuild = false;
        /// <summary>
        /// 抗剪键
        /// </summary>
        public SteelBeam m_shearKey = null;

        private BeamProperties m_shearKeyHelp;


        public void Running()
        {
            PrepareData();
            if(!m_canBuild){return;}
            var buildPoint1 = m_buildPoint;
            var buildPoint2 = PointHelper.Move(buildPoint1,-m_context.AxisZ,m_length);
            m_shearKey =  m_context.TargetComp.CreateSteelBeam(m_interfaceType, buildPoint1, buildPoint2, true);
            if(m_shearKey == null){return;}
            m_shearKey.WorkAxis = new Mat43(m_context.WorkMat43);
            m_shearKey.RotationSide = Define.ObjectSide.SIDE_TOP;
            m_shearKey.Rotation = m_rotation%360;
            m_shearKey.Flush();
            m_shearKeyHelp.ApplyToPart(m_shearKey);
            m_context.Weld.BuildWeldSeamForShearKey(m_shearKey,buildPoint1);
        }

        private void PrepareData()
        {
            var handler = m_context.ParamsHandler;
            m_shearKeyHelp = m_pub.ShearKeyPro;
            m_canBuild = m_pub.ShearKeyCanBuild;
            
            m_interfaceType = m_shearKeyHelp.Profile;
            m_length = handler.GetDoubleValue(ParamNames.PARAMETERS_SHEAR_KEY_LENGTH);
            if (Equals(m_length, double.MaxValue))
            {
                m_length = BeamPlanesHelper.GetColumnCrossSection_Min(m_context.MainPlanes);
                if (Equals(m_length, double.MinValue))
                {
                    var proFile = m_context.MainPart.Profile;
                    m_length = proFile == null ? 0.0d : Math.Min(proFile.ShapeHeight, proFile.ShapeWidth);
                }
            }
                
            //为偏移做处理
            var horizontalOffset = handler.GetDoubleValue(ParamNames.PARAMETERS_SHEAR_KEY_HORIZONTAL_OFFSET);
            var verticalOffset =handler.GetDoubleValue(ParamNames.PARAMETERS_SHEAR_KEY_VERTICAL_OFFSET);
            m_rotation=handler.GetDoubleValue(ParamNames.PARAMETERS_SHEAR_KEY_ROTATION_ANGLE);

            //确定抗剪键生成位置
            var index = m_pub.parts_shear_key_profile_welded;
            m_buildPoint = m_context.DetailCentrePt;
            //选择在层板下生成抗剪键，且层板要存在
            if (index == 2 && m_context.LevelingPlate.CanBuild)
            {
                //m_buildPoint = m_context.LevelingPlate.m_buildPoint;
                PointHelper.MoveToRef(ref m_buildPoint, new Plane(m_context.LevelingPlate.m_buildPoint,m_context.AxisZ));
                m_buildPoint = PointHelper.Move
                    (m_buildPoint, -m_context.AxisZ, m_context.LevelingPlate.m_plateData.Thickness/2);
            }
            else//否则在基板下生成抗剪键
            {
                //m_buildPoint = m_context.BasePlate.m_buildTopPoint;
                //m_buildTopPoint = PointHelper.Move (m_buildTopPoint, -m_context.AxisZ, m_context.BasePlate.m_thickness);
                //m_buildPoint = PointHelper.Move (m_buildPoint, -m_context.AxisZ, m_pub.BasePlateThickness);
                PointHelper.MoveToRef(ref m_buildPoint,m_pub.BasePlateBtmPl);
            }
            //尚未偏移
            var axisX = m_context.MainPart.WorldMatrix.X;
            var axisY = m_context.MainPart.WorldMatrix.Y;
            PointHelper.MoveRef(ref m_buildPoint, axisX, horizontalOffset);
            PointHelper.MoveRef(ref m_buildPoint, -axisY, verticalOffset);

        }
    }
}