﻿using System.Collections.Generic;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;
using U.S.Base_plate_1047_.Main;
using U.S.Base_plate_1047_.Main.Running;
using U.S.Base_plate_1047_.Tool;


namespace U.S.Base_plate_1047_.Running
{
    public class BuildWeld:RunningBaseClass
    {
        public BuildWeld(ComponentInstanceContext context) : base(context)
        {
            m_context.Weld = this;
            PrepareWeld();
        }

        /// <summary>
        /// 存储焊缝数据
        /// </summary>
        private readonly List<WeldProperties> m_weldProps = new List<WeldProperties>();

        public void Running()
        {
            BuildWeldSeamForMain();
        }

        private void PrepareWeld()
        {
            for(int i=0;i<ParamNames.WELD_PROPS_LIST.Length;++i)
            {
                m_weldProps.Add(new WeldProperties());
            }

            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                var weldProps = m_context.ParamsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }
        }

        /// <summary>
        /// 为主零件(柱子)创建焊缝
        /// </summary>
        private void BuildWeldSeamForMain()
        {
            if (m_pub.m_weldGap > 0.0d) {
                return;
            }
            var bottomPlane= new Plane(m_context.BasePlate.m_buildTopPoint, m_context.AxisZ);
            var mainPlanes = m_context.MainPlanes;
            //TopFlange
            var pointA = bottomPlane.Intersect(mainPlanes.TopFlange, mainPlanes.TopFlangeBack);
            var pointB = bottomPlane.Intersect(mainPlanes.TopFlange, mainPlanes.TopFlangeFront);
            var weld =  m_context.TargetComp.CreatePolyWeld(m_context.MainPart, m_context.BasePlate.m_basePlate,new Vec3[]{pointA,pointB}, true);
            m_weldProps[0].ApplyToWeld(weld);
            //BottomFlange
            pointA = bottomPlane.Intersect(mainPlanes.BottomFlange, mainPlanes.BottomFlangeBack);
            pointB = bottomPlane.Intersect(mainPlanes.BottomFlange, mainPlanes.BottomFlangeFront);
            weld = m_context.TargetComp.CreatePolyWeld(m_context.MainPart, m_context.BasePlate.m_basePlate,new Vec3[]{pointA,pointB}, true);
            m_weldProps[2].ApplyToWeld(weld);
            //web
            pointA = bottomPlane.Intersect(mainPlanes.BackWeb, mainPlanes.TopFlangeInside);
            pointB = bottomPlane.Intersect(mainPlanes.BackWeb, mainPlanes.BottomFlangeInside);
            weld = m_context.TargetComp.CreatePolyWeld(m_context.MainPart, m_context.BasePlate.m_basePlate,new Vec3[]{pointA,pointB}, true);
            m_weldProps[1].ApplyToWeld(weld);
        }

        /// <summary>
        /// 构建加劲肋的焊缝
        /// </summary>
        public void BuildWeldSeamForStiffener(SteelPlate steelPlate, StiffenerHelper stiffenerHelper)
        {
            List<Vec3> points = new List<Vec3>();
            //与主零件的焊缝
#if false 
            var dir = stiffenerHelper.RightHandZAxis();
            points.Add(new Vec3(stiffenerHelper.ControlPoint0));
            points.Add(new Vec3(stiffenerHelper.ControlPoint5));
            for (var index = 0; index < points.Count; ++index)
            {
                points[index] = PointHelper.Move(points[index], dir, steelPlate.Thickness / 2);
            }
            //NOTE 处理5号点于主零件不相接的情况，目前处理了切割主零件导致的不相交
            var refPl = new Plane(m_context.BasePlate.m_buildTopPoint, m_context.AxisZ);
            refPl.MoveThis(m_pub.m_weldGap);
            var dist = refPl.SignedDist(points.Last());
            if (dist < 0)
            {
                points[points.Count - 1] = PointHelper.MoveTo(points.Last(), refPl);
            }
            
            var weld = m_context.TargetComp.CreatePolyWeld(m_context.MainPart, steelPlate,points.ToArray(), true);
            m_weldProps[3].ApplyToWeld(weld);
            points.Clear();
#endif
#if true
            var autoWeld0 = m_context.TargetComp.CreateAutoWeld(m_context.MainPart, steelPlate, Define.SpaceAxis.AXIS_X, true);
            var mat0 = new Mat43(m_context.WorkMat43);
            mat0.Pos = stiffenerHelper.ControlPoint3;
            mat0.Z = -stiffenerHelper.AxisDirectionY;
            autoWeld0.WorkAxis = mat0;
            autoWeld0.AutoWeldPosition = Define.SpaceAxis.AXIS_Z;
            m_weldProps[3].ApplyToWeld(autoWeld0);
#endif
            
            //与基板的焊缝
#if false 
            points.Add(new Vec3(stiffenerHelper.ControlPoint3));
            points.Add(new Vec3(stiffenerHelper.ControlPoint4));
            for (var index = 0; index < points.Count; ++index) {
                points[index] = PointHelper.Move(points[index], dir, steelPlate.Thickness / 2);
            }
            weld = m_context.TargetComp.CreatePolyWeld(m_context.BasePlate.m_basePlate, steelPlate,points.ToArray(), true);
            m_weldProps[3].ApplyToWeld(weld);
#endif
#if true 
            //var autoWeld = m_context.TargetComp.CreateAutoWeld(m_context.BasePlate.m_basePlate, steelPlate,Define.SpaceAxis.AXIS_X, true);
            var autoWeld = m_context.TargetComp.CreateAutoWeld(steelPlate, m_context.BasePlate.m_basePlate,Define.SpaceAxis.AXIS_X, true);
            
            var mat = new Mat43(m_context.WorkMat43);
            mat.Pos = stiffenerHelper.ControlPoint3;
            mat.Z = steelPlate.WorldMatrix.Z;
            autoWeld.WorkAxis = mat;
            autoWeld.AutoWeldPosition = Define.SpaceAxis.AXIS_Z;
            m_weldProps[3].ApplyToWeld(autoWeld);
#endif

        }

        /// <summary>
        /// 为螺母或者垫片构建焊缝
        /// </summary>
        /// <param name="main"></param>
        /// <param name="sub"></param>
        public void BuildWeldSeamForNutOrWasher(SteelPolyBeam main, SteelBeam sub)
        {
            var point0 = sub.CtrlPoints.First().WorldPos;
            var point1 = sub.CtrlPoints.Last().WorldPos;
            //可以考虑偏移
            var moveOff = m_context.AnchorRod.m_anchorBoltSize / 2;
            PointHelper.MoveRef(ref point0,m_context.AxisX,moveOff);
            PointHelper.MoveRef(ref point1,m_context.AxisX,moveOff);
            
            var weld=m_context.TargetComp.CreatePolyWeld(main,sub,new Vec3[]{point0,point1},true);
            m_weldProps[1].ApplyToWeld(weld);
        }

        /// <summary>
        /// 为垫片板构建焊缝
        /// </summary>
        /// <param name="main"></param>
        /// <param name="sub"></param>
        public void BuildWeldSeamForWasherPlate(PartBase main, SteelPlate sub)
        {
            if (sub == null) { 
                return;
            }
            var point0 = sub.CtrlPoints.First().WorldPos;
            var point1 = sub.CtrlPoints.Last().WorldPos;
            PointHelper.MoveRef(ref point0,-m_context.AxisZ,sub.Thickness/2 );
            PointHelper.MoveRef(ref point1,-m_context.AxisZ,sub.Thickness/2 );
            var point = PointHelper.Midpoint(point0, point1);
            //可以考虑偏移
            var moveOff = m_context.AnchorRod.m_anchorBoltSize / 2;
            PointHelper.MoveRef(ref point,m_context.AxisX,moveOff);
            
            var weld = m_context.TargetComp.CreatePolyWeld(main,sub,new Vec3[]{point},true);
            m_weldProps[1].ApplyToWeld(weld);
        }
        /// <summary>
        /// 为垫片板构建焊缝
        /// </summary>
        /// <param name="main"></param>
        /// <param name="sub"></param>
        public void BuildWeldSeamForWasherPlate(SteelPolyBeam main, SteelBeam sub)
        {
            var point0 = sub.CtrlPoints.First().WorldPos;
            var point1 = sub.CtrlPoints.Last().WorldPos;
            //可以考虑偏移
            var moveOff = m_context.AnchorRod.m_anchorBoltSize / 2;
            PointHelper.MoveRef(ref point1,m_context.AxisX,moveOff);
            
            
            var weld = m_context.TargetComp.CreatePolyWeld(main,sub,new Vec3[]{point1},true);
            m_weldProps[1].ApplyToWeld(weld);
        }

        /// <summary>
        /// 为抗剪键盘构建焊缝
        /// </summary>
        /// <param name="shearKey"></param>
        /// <param name="buildPoint1"></param>
        public void BuildWeldSeamForShearKey(SteelBeam shearKey, Vec3 buildPoint1)
        {
            if(shearKey == null){return;}
            PartBase main = null;
            var points = new List<Vec3>();
            var select = m_pub.parts_shear_key_profile_welded;
            if (select == 2 && m_context.LevelingPlate.CanBuild) 
            { main = m_context.LevelingPlate.Plate; }
            else { main = m_context.MainPart; }
#if false
            var beamPlanes = BeamPlanes.CreateFromBeam(shearKey);
            HandlerShearKeyPoint(beamPlanes, buildPoint1, points);
            var weld = m_context.TargetComp.CreatePolyWeld(main,shearKey,points.ToArray(),true);
            m_weldProps[1].ApplyToWeld(weld);
#endif
            var weld = m_context.TargetComp.CreateAutoWeld(main, shearKey, Define.SpaceAxis.AXIS_X, true);
            var mat = new Mat43(m_context.WorkMat43) {
                Pos = buildPoint1
            };
            weld.WorkAxis = mat;
            weld.AutoWeldPosition = Define.SpaceAxis.AXIS_Z;
            m_weldProps[1].ApplyToWeld(weld);
        }

#if false 
        private void HandlerShearKeyPoint(BeamPlanes beamPlanes, Vec3 buildPoint1, List<Vec3> points)
        {
            //有腹板前后，用腹板前后
            var pl1 = PlaneHelper.GetBeamPlane(EBeamPlaneType.FrontWeb, beamPlanes);
            var pl2 = PlaneHelper.GetBeamPlane(EBeamPlaneType.TopFlangeInside, beamPlanes);
            var pl3 = PlaneHelper.GetBeamPlane(EBeamPlaneType.BottomFlangeInside, beamPlanes);
            if (pl1 != null && pl2 != null && pl3 != null)
            {
                var p1 = PointHelper.MoveTo(buildPoint1, pl2);
                var p2 = PointHelper.MoveTo(buildPoint1, pl3);
                PointHelper.MoveToRef(ref p1,pl1);
                PointHelper.MoveToRef(ref p2,pl1);
                points.Add(p1);
                points.Add(p2);
                return;
            }
            //无腹板用包围面
            var pls = new List<EBeamPlaneType>();
            pls.AddRange(PlaneHelper.BOUND_PLANE_TBFB);
            pls.Sort( (x, y) =>
            {
                var px = PlaneHelper.GetBeamPlane(x,beamPlanes);
                var py = PlaneHelper.GetBeamPlane(y,beamPlanes);
                if (px == null || py == null)
                {
                    return 0;
                }
                var distX = px.Dist(buildPoint1);
                var distY = py.Dist(buildPoint1);
                return distX.CompareTo(distY);
            });
            pl1 = PlaneHelper.GetBeamPlane(pls[0], beamPlanes);
            pl2 = PlaneHelper.GetBeamPlane(pls[1], beamPlanes);
            if (pl1 != null && pl2 != null)
            {
                points.Add(PointHelper.MoveTo(buildPoint1,pl1));
                points.Add(PointHelper.MoveTo(buildPoint1,pl2));
                return;
            }
            //如果都没有就取x轴
            points.Add(PointHelper.Move(buildPoint1,m_context.AxisX,30));
            points.Add(PointHelper.Move(buildPoint1,m_context.AxisX,-30));
        }
#endif
        public void BuildWeldSeamForExtraPlate1(Vec3 ptFirst, Vec3 ptLast, PartBase anchorRod, SteelBeam plate)
        {
            if(anchorRod == null || plate == null){return;}
            var point = PointHelper.Midpoint(ptFirst, ptLast);
            var weld = m_context.TargetComp.CreatePolyWeld(anchorRod,plate,new Vec3[]{point},true);
            m_weldProps[1].ApplyToWeld(weld);
        }
    }
}