﻿using System.Collections.Generic;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using U.S.Base_plate_1047_.Main;
using U.S.Base_plate_1047_.Main.Running;

namespace U.S.Base_plate_1047_.Tool
{
    /// <summary>
    /// 螺栓组构建器
    /// </summary>
    public class BoltGroupBuilder
    {
        public PartBase m_mainPart = null;
        public List<PartBase> m_subPart = new List<PartBase>();
        public Vec3 posStart = Vec3.ZERO;
        public Vec3 posEnd = Vec3.ZERO;
        public BoltGroup m_boltGroup = null;

        /// <summary>
        /// 后续操作的拷贝对象
        /// </summary>
        public BoltGroup CopyFrom = null;

        /// <summary>
        /// 将所有的位置、偏移属性设置为0.0d
        /// </summary>
        public void SetAllPosAndOffsetZero()
        {
            if(m_boltGroup == null){return;}
            m_boltGroup.Rotation = 0.0d;
            m_boltGroup.VerticalOffset = 0.0d;
            m_boltGroup.DepthOffset = 0.0d;
            m_boltGroup.HeadOffsetX = 0.0d;
            m_boltGroup.HeadOffsetY = 0.0d;
            m_boltGroup.HeadOffsetZ = 0.0d;
            m_boltGroup.TailOffsetX = 0.0d;
            m_boltGroup.TailOffsetY = 0.0d;
            m_boltGroup.TailOffsetZ = 0.0d;
        }

        /// <summary>
        /// 创建螺栓组
        /// </summary>
        /// <param name="context"></param>
        public void BuildBoltGroup(ComponentInstanceContext context)
        {
            if (m_subPart == null || m_subPart.Count == 0)
                return;
            if (m_subPart.Count > 1)
            {
                m_boltGroup = context.TargetComp.CreateBoltGroup(m_mainPart, m_subPart.ToArray(), posStart, posEnd, true);
            }
            else
            {
                m_boltGroup = context.TargetComp.CreateBoltGroup(m_mainPart, m_subPart.First(), posStart, posEnd, true);
            }
        }
        /// <summary>
        /// 创建孔组
        /// </summary>
        /// <param name="context"></param>
        public void BuildHolesGroup(ComponentInstanceContext context)
        {
            if (m_subPart == null || m_subPart.Count == 0)
                return;
            if (m_subPart.Count > 1)
            {
                m_boltGroup = context.TargetComp.CreateBoltHoles(m_mainPart, m_subPart.ToArray(), posStart, posEnd, true);
            }
            else
            {
                m_boltGroup = context.TargetComp.CreateBoltHoles(m_mainPart, m_subPart.First(), posStart, posEnd, true);
            }
        }

        /// <summary>
        /// 增加次零件
        /// </summary>
        /// <param name="partBase"></param>
        public void AddSubPart(PartBase partBase)
        {
            if (m_mainPart == null)
            {
                if (m_subPart.Count == 0)
                {
                    m_subPart.Add(partBase);
                }
                else
                {
                    m_mainPart = m_subPart.First();
                    m_subPart.Clear();
                    m_subPart.Add(partBase);
                }
            }
            else
            {
                m_subPart.Add(partBase);
            }
        }

        /// <summary>
        /// 从另一个螺栓组拷贝操作数据，并刷新
        /// </summary>
        /// <param name="boltGroup"></param>
        public void CopyFromBoltGroup(BoltGroup boltGroup)
        {
            if (m_boltGroup == null || boltGroup == null) return;
            m_boltGroup.WorkAxis = boltGroup.WorkAxis;
            m_boltGroup.BoltSize = boltGroup.BoltSize;
            m_boltGroup.ArrayStrideX= boltGroup.ArrayStrideX;
            m_boltGroup.ArrayStrideY= boltGroup.ArrayStrideY;
            m_boltGroup.RotationSide = boltGroup.RotationSide;
            m_boltGroup.HeadOffsetX = boltGroup.HeadOffsetX;
            m_boltGroup.VerticalOffset = boltGroup.VerticalOffset;
            m_boltGroup.CutLength = boltGroup.CutLength;
            m_boltGroup.Refresh();
        }

        /// <summary>
        /// 从另一个螺栓组复制控制点Start和end
        /// </summary>
        /// <param name="boltGroup">要复制的目标螺栓组</param>
        /// <param>往螺栓组的Z轴偏移</param>
        public void CopyPointStartAndEndFromBoltGroup(BoltGroup boltGroup)
        {
            if(boltGroup == null){return;}
            posStart = new Vec3(boltGroup.CtrlPoints[0].WorldPos);
            posEnd= new Vec3(boltGroup.CtrlPoints[1].WorldPos);
            //PointHelper.MoveRef(ref posStart,boltGroup.WorkAxis.Z,offset);
            //PointHelper.MoveRef(ref posEnd,boltGroup.WorkAxis.Z,offset);
        }

        /// <summary>
        /// 清理函数
        /// </summary>
        public void Clear()
        {
            m_mainPart = null;
            m_subPart = null;
            posStart = Vec3.ZERO;
            posEnd = Vec3.ZERO;
        }
    }
}