﻿using System;

namespace U.S.Base_plate_1047_.Tool
{
    public static class CalculationHelper
    {
        /// <summary>
        /// 加劲肋的默认厚度取决于翼缘的厚度
        /// </summary>
        /// <param name="flangeThickness"></param>
        /// <returns></returns>
        public static double CalculationOfStiffenerThickness(double flangeThickness)
        {
            if (flangeThickness < 6.0)
            {
                return 0.0;
            }
            else
            {
                double segment = Math.Floor((flangeThickness - 6) / 3);
                return 3.0 * (segment + 1.0);
            }
        }

         
        /// <summary>
        /// 基板的默认宽度和高度都基于柱子的最大截面尺寸
        /// 一种分段函数
        /// </summary>
        /// <param name="maxColumnCrossSection">柱子的最大截面尺寸</param>
        /// <returns></returns>
        public static double CalculationOfBasePlateWidthAndHeight(double maxColumnCrossSection)
        {
            if (maxColumnCrossSection <= 100)
            {
                return 150;
            }
            //计算出超出100的部分，并且向下取整
            var n1 = (int)maxColumnCrossSection - 100;
            int m1 = (n1-1) / 50;
            var n2 = n1 % 50;
            int m2 = 0;
            if (1 <= n2 && n2 <= 16)
            {
                m2 = 0;
            }else if (17 <= n2 && n2 <= 33)
            {
                m2 = 1;
            }else 
            {
                m2 = 2;
            }
            return  175 + 75 * m1 + m2 * 25;
        }

        /// <summary>
        /// 官方文档写为：数值与基板边缘到柱边缘的距离有关。本质上还是和柱子的最大截面尺寸有关
        /// 其逻辑是，柱子离底板边缘最近的边缘面，即此边缘间距为l，那么加劲板的长度为3*n小于等于l-10
        /// 其中n取整数，得出加劲板的长度
        /// </summary>
        /// <param name="minimumEdgeDistance">柱子离底板边缘最近的边缘面，该边缘面到底板边缘面的距离</param>
        /// <returns></returns>
        public static double CalculateTheWidthOfStiffeners(double minimumEdgeDistance)
        {
            if (minimumEdgeDistance <= 0)
                return 0;
            int n = 0;
            while (true)
            {
                double sum = 3*n;
                if (sum <= minimumEdgeDistance-10)
                {
                    ++n;
                }
                else
                {
                    --n;
                    break;
                }
            }
            return n*3;
        }
    }
}