﻿using LoongCAD.Geometry;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;

namespace WindBraceConnection110.Common
{
    /// <summary>
    /// 支撑截面类型
    /// </summary>
    public enum BraceProfileType
    {
        L,
        T,
        I,
        C,
        U,
        CC,
        RECTHOLLOW,
        BAN,
        HQ,
        WELDBOX,
        OTHER
    }
    public enum BracePosType
    {
        FIRST,
        MIDDLE,
        LAST
    }
    public class BraceData
    {
        public Beam Beam;
        public BeamPlanes BeamPlanes;
        /// <summary>
        /// 支撑的选取序号
        /// </summary>
        public int ClickNumber;
        public BracePosType PosType;
        /// <summary>
        /// 调整后支撑的构建方向
        /// </summary>
        public Vec3 BraceLine;
        /// <summary>
        /// 支撑离主零件较远的控制点
        /// </summary>
        public Vec3 FarPoint;
        /// <summary>
        /// 支撑从主零件构建方向出发，较近的面
        /// </summary>
        public Plane TopPlane;
        /// <summary>
        /// 支撑从主零件构建方向出发，较远的面
        /// </summary>
        public Plane BottomPlane;
        /// <summary>
        /// 支撑沿着Z轴正方向面
        /// 规定Z轴方向为主零件构建方向与组件X轴方向
        /// </summary>
        public Plane FrontPlane;
        /// <summary>
        /// 支撑沿着Z轴负方向面
        /// </summary>
        public Plane BackPlane;
        public BraceProfileType ProfileType;
        /// <summary>
        /// 支撑构件节点板的第一控制点
        /// </summary>
        public Vec3 TopPoint;
        /// <summary>
        /// 支撑构建节点板的第二控制点
        /// </summary>
        public Vec3 BottomPoint;
        /// <summary>
        /// 支撑构建节点板的对齐面
        /// </summary>
        public Plane AlignPlane;
        /// <summary>
        /// 节点板所在支撑的边界面
        /// </summary>
        public Plane GussetBorderPlane;
    }
}
