﻿using System.Linq;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Geometry;
using System.Collections.Generic;

namespace WindBraceConnection110.Common
{
    public class ComponentInstanceContext
    {
        ///<summary>
        /// 目标组件的类型
        ///</summary>
        private ConnectionComponent m_comp;

        private Beam m_mainPart = null;

        private Vec3 m_mainLine;

        private BeamPlanes m_mainPlanes;

        private Vec3 m_axisX = Vec3.ZERO;

        private Vec3 m_axisY = Vec3.ZERO;

        private Vec3 m_axisZ = Vec3.ZERO;

        private List<BraceData> m_braceData = new List<BraceData>();

        private List<BraceData> m_leftBraceData = new List<BraceData>();

        private List<BraceData> m_rightBraceData = new List<BraceData>();

        private List<Vec3> m_leftPoints = new List<Vec3>();

        private List<Vec3> m_rightPoints = new List<Vec3>();

        private List<Vec3> m_upPoints = new List<Vec3>();

        private List<Vec3> m_downPoints = new List<Vec3>();

        private bool m_enableNotch = false;

        private bool m_enableSpecialNotch = false;

        private double m_thickness = double.MaxValue;

        //开发组件需要的上下文
        public ConnectionComponent TargetComp { get => m_comp; set => m_comp = value; }
        public Beam MainPart { get => m_mainPart; set => m_mainPart = value; }
        public Vec3 MainLine { get => m_mainLine; set => m_mainLine = value; }
        public BeamPlanes MainPlanes { get => m_mainPlanes; set => m_mainPlanes = value; }
        public Plane FaceLeftBracePlane { get; set; }
        public Plane FaceRightBracePlane { get; set; }
        public Vec3 X { get => m_axisX; set => m_axisX = value; }
        public Vec3 Y { get => m_axisY; set => m_axisY = value; }
        public Vec3 Z { get => m_axisZ; set => m_axisZ = value; }
        public List<BraceData> BraceData { get => m_braceData; set => m_braceData = value; }
        public List<BraceData> LeftBraceData { get => m_leftBraceData; set => m_leftBraceData = value; }
        public List<BraceData> RightBraceData { get => m_rightBraceData; set => m_rightBraceData = value; }
        public List<Vec3> LeftPoints { get => m_leftPoints; set => m_leftPoints = value; }
        public List<Vec3> RightPoints { get => m_rightPoints; set => m_rightPoints = value; }
        public List<Vec3> UpPoints { get => m_upPoints; set => m_upPoints = value; }
        public List<Vec3> DownPoints { get => m_downPoints; set => m_downPoints = value; }
        public Plane BaseGussetPlane { get; set; }
        public Plane BracePlane { get; set; }
        public Plane OppositeBracePlane { get; set; }
        /// <summary>
        /// T型 I型支撑构建节点板的偏移平面1
        /// </summary>
        public Plane BraceOffsetPlane { get; set; }
        /// <summary>
        /// T型 I型支撑构建节点板的偏移平面2
        /// </summary>
        public Plane OppositeBraceOffsetPlane { get; set; }
        public PartBase RetPlate { get; set; }
        public bool EnableNotch { get => m_enableNotch; set => m_enableNotch = value; }
        public bool EnableSpecialNotch { get => m_enableSpecialNotch; set => m_enableSpecialNotch = value; }
        public double Thickness { get => m_thickness; set => m_thickness = value; }
        public static ComponentInstanceContext CreateInstance(ConnectionComponent comp)
        {
            ComponentInstanceContext compContext = new ComponentInstanceContext();
            if (compContext.Initialize(comp))
                return compContext;
            compContext.Clear();
            return null;
        }
        private bool Initialize(ConnectionComponent comp)
        {
            if (comp == null)
                return false;
            Beam mainPart = comp.Main as Beam;
            if (mainPart == null || comp.Subs == null || comp.Subs.Length < 1)
                return false;
            if (!PrepareMainLine(mainPart))
                return false;
            
            m_comp = comp;
            m_mainPart = mainPart;
            m_mainPlanes = BeamPlanes.CreateFromBeam(mainPart);
            
            return true;
        }
        /// <summary>
        /// 准备主零件由起点到终点的线
        /// </summary>
        private bool PrepareMainLine(Beam mainPart)
        {
            var line = mainPart.CtrlPoints?.Last()?.WorldPos -
                       mainPart.CtrlPoints?.First()?.WorldPos;
            if (!line.HasValue)
                return false;
            m_mainLine = line.Value.Normalize();
            return true;
        }
        private void Clear()
        {
            m_comp = null;
            m_mainPart = null;
            m_mainPlanes = null;
        }
    }
}
