﻿using System;
using LoongCAD.Util;
using LoongCAD.UI.Component;

namespace WindBraceConnection110.Common
{
    public class ParamsHandler : LoongCAD.STC.Plugins.ParamsHandlerBase
    {
        /// <summary>
        /// 属性空值全局总表
        /// </summary>
        protected static DObject g_emptyValues = null;

        /// <summary>
        /// 属性内置默认值全局总表
        /// </summary>
        protected static DObject g_builtinDefaults = null;

        /// <summary>
        /// 默认构造函数
        /// </summary>
        /// <remarks><b>此方法需要复制到其他组件代码中使用</b></remarks>
        public ParamsHandler()
            : base(ParamsHandler.EmptyValues, ParamsHandler.BuiltinDefautls)
        {

        }
        /// <summary>
        /// 组件公用属性空值表
        /// </summary>
        public static DObject EmptyValues
        {
            get
            {
                if (g_emptyValues == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_emptyValues;
            }
        }
        /// <summary>
        /// 组件公用内置默认值接口
        /// </summary>
        public static DObject BuiltinDefautls
        {
            get
            {
                if (g_builtinDefaults == null)
                {
                    IntializeEmptyDefaults();
                }
                return g_builtinDefaults;
            }
        }
        /// <summary>
        /// 初始化默认值
        /// </summary>
        /// <param name="defaultConfig">从默认值配置加载的默认值</param>
        public override LoongCAD.Util.DObject InitializeDefaultValues(DObject defaultConfig)
        {
            m_defaultValues = new DObject();
            m_defaultValues.CopyFrom(defaultConfig);

            var emptyValues = ParamsHandler.EmptyValues;
            var builtinDefaults = ParamsHandler.BuiltinDefautls;
            if (emptyValues == null || builtinDefaults == null)
            {
                Console.WriteLine("ParamsHandler.InitializeDefaultValues: [Warning] EmptyValue/BuiltinDefaults is null!");
                return m_defaultValues;
            }

            this.TryCompleteDObjectValues(m_defaultValues, builtinDefaults, emptyValues);
            return m_defaultValues;
        }
        /// <summary>
        /// 初始化组件所有实例公用的属性空值、属性内置默认值
        /// </summary>
        public static void IntializeEmptyDefaults()
        {
            g_emptyValues = new DObject();
            g_builtinDefaults = new DObject();

            //图形页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_LEFT_BRACE_DISTANCE_TO_PART, double.MaxValue, 20.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_LEFT_LAST_BRACE_GUSSET_CORNER, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_LEFT_LAST_BRACE_GUSSET_LENGTH, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_LEFT_BETWEEN_BRACE_GUSSET_LENGTH, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_LEFT_FIRST_BRACE_GUSSET_LENGTH, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_LEFT_FIRST_BRACE_GUSSET_CORNER, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_RIGHT_BRACE_DISTANCE_TO_PART, double.MaxValue, 20.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_RIGHT_LAST_BRACE_GUSSET_CORNER, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_RIGHT_LAST_BRACE_GUSSET_LENGTH, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_RIGHT_BETWEEN_BRACE_GUSSET_LENGTH, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_RIGHT_FIRST_BRACE_GUSSET_LENGTH, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_RIGHT_FIRST_BRACE_GUSSET_CORNER, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_GUSSET_DISTANCE_TO_PART, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_LEFT_BRACE_POSITION, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_LEFT_DISTANCE_TO_BRACE, double.MaxValue, 20.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_RIGHT_BRACE_POSITION, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.PIC_RIGHT_DISTANCE_TO_BRACE, double.MaxValue, 20.0);

            //节点板页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_PLATE_PROPS, PlatePropertyPanel.EmptyValue, (DObject)new object[] {
                PlatePropertyPanel.ParamNames.Thickness, 8.0,
                PlatePropertyPanel.ParamNames.NoPrefix, "",
                PlatePropertyPanel.ParamNames.NoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.AssNoPrefix, "",
                PlatePropertyPanel.ParamNames.AssNoStartPostfix, "1",
                PlatePropertyPanel.ParamNames.Material, "Q235B",
                PlatePropertyPanel.ParamNames.Name, "板",
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_PLATE_OVERALL_SHAPE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BRACE_ORDER, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_PLATE_UP_SHAPE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_PLATE_DOWN_SHAPE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_PLATE_POSITION, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_PLATE_MOVE_Z, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BRACE_WELD_ENABLED, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_PLATE_HOLE_POSITION, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_PLATE_HOLE_DIAMETER, double.MaxValue, 20.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSER_PLATE_HOLE_EDGE_GAP, double.MaxValue, double.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BRACE_NOTCH, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BRACE_CHAMFER_LENGTH, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BRACE_CHAMFER_ANGLE, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BRACE_VERT_NOTCH_LENGTH, double.MaxValue, 0.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BRACE_HORI_NOTCH_LENGTH, double.MaxValue, 0.0);

            //节点板螺栓页
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BOLT_BASIC_PROPS, BoltBasicPropertyPanel.EmptyValue, (DObject)new object[] {
                    BoltBasicPropertyPanel.ParamNames.BoltStandard, "TS10.9",
                    BoltBasicPropertyPanel.ParamNames.BoltSize, 20.0,
                    BoltBasicPropertyPanel.ParamNames.ThreadInMtrl, 2, // true
                    BoltBasicPropertyPanel.ParamNames.HoleTol, double.MaxValue,
                    BoltBasicPropertyPanel.ParamNames.MadeType0, 0, // Workshop
                });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BOLT_ENABLED, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BOLT_SHAPE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BOLT_VERT_OFFSET_MODE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BOLT_VERT_OFFSET, double.MaxValue, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BOLT_HORI_OFFSET_MODE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BOLT_HORI_OFFSET, double.MaxValue, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BOLT_HORI_COUNT, int.MaxValue, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BOLT_HORI_STRIDE, new double[] { }, new double[] { });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BOLT_VERT_COUNT, int.MaxValue, 2);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.GUSSET_BOLT_VERT_STRIDE, new double[] { }, new double[] { });


            //支柱螺栓1页面
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_BASIC_PROPS, BoltBasicPropertyPanel.EmptyValue, (DObject)new object[] {
                    BoltBasicPropertyPanel.ParamNames.BoltStandard, "TS10.9",
                    BoltBasicPropertyPanel.ParamNames.BoltSize, 20.0,
                    BoltBasicPropertyPanel.ParamNames.ThreadInMtrl, 2, // true
                    BoltBasicPropertyPanel.ParamNames.HoleTol, double.MaxValue,
                    BoltBasicPropertyPanel.ParamNames.MadeType0, 0, // Workshop
                });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_HOLE_PROPS, BoltHolePanel.EmptyValue, (DObject)new object[] {
                BoltHolePanel.ParamNames.HoleX, 0.0,
                BoltHolePanel.ParamNames.HoleY, 0.0,
                BoltHolePanel.ParamNames.HoleType, 1,
                BoltHolePanel.ParamNames.RotSlot, 3,
                BoltHolePanel.ParamNames.SlotIn, 4
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_CONTENT_PROPS, null, (DObject)new object[]
            {
                BoltContentPanel.ParamNames.PartWasherUp, false,
                BoltContentPanel.ParamNames.PartWasherDown0, false,
                BoltContentPanel.ParamNames.PartWasherDown1, true,
                BoltContentPanel.ParamNames.PartNut0, true,
                BoltContentPanel.ParamNames.PartNut1, false,
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_EXTEND_PROPS, BoltExtendPanel.EmptyValue, (DObject)new object[]
            {
                BoltExtendPanel.ParamNames.ExtLength, 0.0,
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_SHAPE, 0, 0); // Array
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_DISTANCE, double.MaxValue, 0); //todo不确定
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_HORI_BRACE_MARGIN, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_HORI_GUSSET_MARGIN, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_VERT_OFFSET_MODE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_VERT_OFFSET, double.MaxValue, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_HORI_COUNT, int.MaxValue, int.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_HORI_STRIDE, new double[] { }, new double[] { });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_VERT_COUNT, int.MaxValue, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT1_VERT_STRIDE, new double[] { }, new double[] { });

            //支柱螺栓2页面
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_BASIC_PROPS, BoltBasicPropertyPanel.EmptyValue, (DObject)new object[] {
                    BoltBasicPropertyPanel.ParamNames.BoltStandard, "TS10.9",
                    BoltBasicPropertyPanel.ParamNames.BoltSize, 20.0,
                    BoltBasicPropertyPanel.ParamNames.ThreadInMtrl, 2, // true
                    BoltBasicPropertyPanel.ParamNames.HoleTol, double.MaxValue,
                    BoltBasicPropertyPanel.ParamNames.MadeType0, 0, // Workshop
                });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_HOLE_PROPS, BoltHolePanel.EmptyValue, (DObject)new object[] {
                BoltHolePanel.ParamNames.HoleX, 0.0,
                BoltHolePanel.ParamNames.HoleY, 0.0,
                BoltHolePanel.ParamNames.HoleType, 1,
                BoltHolePanel.ParamNames.RotSlot, 3,
                BoltHolePanel.ParamNames.SlotIn, 4
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_CONTENT_PROPS, null, (DObject)new object[]
            {
                BoltContentPanel.ParamNames.PartWasherUp, false,
                BoltContentPanel.ParamNames.PartWasherDown0, false,
                BoltContentPanel.ParamNames.PartWasherDown1, true,
                BoltContentPanel.ParamNames.PartNut0, true,
                BoltContentPanel.ParamNames.PartNut1, false,
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_EXTEND_PROPS, BoltExtendPanel.EmptyValue, (DObject)new object[]
            {
                BoltExtendPanel.ParamNames.ExtLength, 0.0,
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_SHAPE, 0, 0); // Array
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_DISTANCE, double.MaxValue, 0); //todo不确定
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_HORI_BRACE_MARGIN, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_HORI_GUSSET_MARGIN, double.MaxValue, 40.0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_VERT_OFFSET_MODE, 0, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_VERT_OFFSET, double.MaxValue, 0);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_HORI_COUNT, int.MaxValue, int.MaxValue);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_HORI_STRIDE, new double[] { }, new double[] { });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_VERT_COUNT, int.MaxValue, 1);
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.BOLT2_VERT_STRIDE, new double[] { }, new double[] { });

            // 焊缝
            
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WELD_PROPS_LIST[0], WeldPropertyPanel.EmptyValue, new DObject {
                { WeldPropertyPanel.ParamNames.SizeUp, 4.0 },
                { WeldPropertyPanel.ParamNames.WeldTypeUp, 11 }, // Fillet
                { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                { WeldPropertyPanel.ParamNames.SizeDown, 0.0 },
                { WeldPropertyPanel.ParamNames.WeldTypeDown, 11 }, // Fillet
                { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                { WeldPropertyPanel.ParamNames.EdgeAround, 1 }, // Edge
                { WeldPropertyPanel.ParamNames.MadeType, 2 }, // Workshop
                { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                { WeldPropertyPanel.ParamNames.RefText, "" }
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WELD_PROPS_LIST[1], WeldPropertyPanel.EmptyValue, new DObject {
                { WeldPropertyPanel.ParamNames.SizeUp, 0.0 },
                { WeldPropertyPanel.ParamNames.WeldTypeUp, 11 }, // Fillet
                { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                { WeldPropertyPanel.ParamNames.SizeDown, 0.0 },
                { WeldPropertyPanel.ParamNames.WeldTypeDown, 11 }, // Fillet
                { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                { WeldPropertyPanel.ParamNames.EdgeAround, 2 }, // Edge
                { WeldPropertyPanel.ParamNames.MadeType, 1 }, // Workshop
                { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                { WeldPropertyPanel.ParamNames.RefText, "" }
            });
            AddEmptyDefaultValue(g_emptyValues, g_builtinDefaults, ParamNames.WELD_PROPS_LIST[2], WeldPropertyPanel.EmptyValue, new DObject {
                { WeldPropertyPanel.ParamNames.SizeUp, 4.0 },
                { WeldPropertyPanel.ParamNames.WeldTypeUp, 11 }, // Fillet
                { WeldPropertyPanel.ParamNames.GrooveAngleUp, 0.0 },
                { WeldPropertyPanel.ParamNames.WeldContourUp,  1}, // None
                { WeldPropertyPanel.ParamNames.WeldFinishUp, 1 }, // None
                { WeldPropertyPanel.ParamNames.LengthUp, 0.0 },
                { WeldPropertyPanel.ParamNames.PitchUp, 0.0 },
                { WeldPropertyPanel.ParamNames.RootFaceUp, 0.0 },
                { WeldPropertyPanel.ParamNames.EffectiveThroatUp, 0.0 },
                { WeldPropertyPanel.ParamNames.RootGapUp, 0.0 },

                { WeldPropertyPanel.ParamNames.SizeDown, 0.0 },
                { WeldPropertyPanel.ParamNames.WeldTypeDown, 11 }, // Fillet
                { WeldPropertyPanel.ParamNames.GrooveAngleDown, 0.0 },
                { WeldPropertyPanel.ParamNames.WeldContourDown,  1}, // None
                { WeldPropertyPanel.ParamNames.WeldFinishDown, 1 }, // None
                { WeldPropertyPanel.ParamNames.LengthDown, 0.0 },
                { WeldPropertyPanel.ParamNames.PitchDown, 0.0 },
                { WeldPropertyPanel.ParamNames.RootFaceDown, 0.0 },
                { WeldPropertyPanel.ParamNames.EffectiveThroatDown, 0.0 },
                { WeldPropertyPanel.ParamNames.RootGapDown, 0.0 },

                { WeldPropertyPanel.ParamNames.EdgeAround, 1 }, // Edge
                { WeldPropertyPanel.ParamNames.MadeType, 2 }, // Workshop
                { WeldPropertyPanel.ParamNames.StitchWeld, 1 }, // false
                { WeldPropertyPanel.ParamNames.RefText, "" }
            });
        }
    }
}
