﻿using System;
using System.Windows.Forms;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;
using WindBraceConnection110.Common;
using System.Collections.Generic;

namespace WindBraceConnection110
{
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_ConnectionComponentProps
    {
        private DObject m_rootUIProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;
        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostIntialize();
        }
        public override ComponentPlugin ReferencePlugin => m_refPlugin;
        private void OnPostIntialize()
        {
            // 设置窗口标题
            this.Text = "双侧支撑连接节点(305)";

            foreach (TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }

            // 螺栓标准变化之后调用螺栓内容的自适应方法
            this.panel_bolt1_basic_property.BoltStandardChanged += this.boltContentPanel1.AdaptBoltContents;
            this.panel_bolt2_basic_property.BoltStandardChanged += this.boltContentPanel2.AdaptBoltContents;

            //初始化图片选项卡
            this.pic_left_brace_position.ClearItems();
            this.pic_left_brace_position.AddIntegerItems(0, 0, 3);
            this.pic_left_brace_position.UpdateItems();

            this.pic_right_brace_position.ClearItems();
            this.pic_right_brace_position.AddIntegerItems(0, 0, 3);
            this.pic_right_brace_position.UpdateItems();

            this.gusset_plate_overall_shape.ClearItems();
            this.gusset_plate_overall_shape.AddIntegerItems(0, 0, 4);
            this.gusset_plate_overall_shape.UpdateItems();

            this.gusset_plate_down_shape.ClearItems();
            this.gusset_plate_down_shape.AddIntegerItems(0, 0, 6);
            this.gusset_plate_down_shape.UpdateItems();

            this.gusset_plate_up_shape.ClearItems();
            this.gusset_plate_up_shape.AddIntegerItems(0, 0, 6);
            this.gusset_plate_up_shape.UpdateItems();

            this.gusset_plate_position.ClearItems();
            this.gusset_plate_position.AddIntegerItems(0, 0, 5);
            this.gusset_plate_position.UpdateItems();

            this.gusset_plate_hole_position.ClearItems();
            this.gusset_plate_hole_position.AddIntegerItems(0, 0, 4);
            this.gusset_plate_hole_position.UpdateItems();

            this.gusset_brace_notch.ClearItems();
            this.gusset_brace_notch.AddIntegerItems(0, 0, 4);
            this.gusset_brace_notch.UpdateItems();

            this.cbo_brace_weld_enabled.ClearItems();
            this.cbo_brace_weld_enabled.AddIntegerItems(0, 0, 3);
            this.cbo_brace_weld_enabled.UpdateItems();

            this.cbo_gusset_bolt_enabled.ClearItems();
            this.cbo_gusset_bolt_enabled.AddIntegerItems(0, 0, 3);
            this.cbo_gusset_bolt_enabled.UpdateItems();

            // 初始化螺栓纵向偏移模式控件
            this.cbo_bolt1_vert_offset_mode.ClearItems();
            this.cbo_bolt1_vert_offset_mode.AddItem(0, "默认");
            this.cbo_bolt1_vert_offset_mode.AddItem(1, "顶部");
            this.cbo_bolt1_vert_offset_mode.AddItem(2, "中间");
            this.cbo_bolt1_vert_offset_mode.AddItem(3, "底部");
            this.cbo_bolt1_vert_offset_mode.UpdateItems();
            this.cbo_bolt1_vert_offset_mode.SelectedKey = 0;
            
            this.cbo_bolt2_vert_offset_mode.ClearItems();
            this.cbo_bolt2_vert_offset_mode.AddItem(0, "默认");
            this.cbo_bolt2_vert_offset_mode.AddItem(1, "顶部");
            this.cbo_bolt2_vert_offset_mode.AddItem(2, "中间");
            this.cbo_bolt2_vert_offset_mode.AddItem(3, "底部");
            this.cbo_bolt2_vert_offset_mode.UpdateItems();
            this.cbo_bolt2_vert_offset_mode.SelectedKey = 0;
            
            this.cbo_gusset_bolt_vert_offset_mode.ClearItems();
            this.cbo_gusset_bolt_vert_offset_mode.AddItem(0, "默认");
            this.cbo_gusset_bolt_vert_offset_mode.AddItem(1, "顶部");
            this.cbo_gusset_bolt_vert_offset_mode.AddItem(2, "中间");
            this.cbo_gusset_bolt_vert_offset_mode.AddItem(3, "底部");
            this.cbo_gusset_bolt_vert_offset_mode.UpdateItems();
            this.cbo_gusset_bolt_vert_offset_mode.SelectedKey = 0;
            
            this.cbo_gusset_bolt_hori_offset_mode.ClearItems();
            this.cbo_gusset_bolt_hori_offset_mode.AddItem(0, "默认");
            this.cbo_gusset_bolt_hori_offset_mode.AddItem(1, "左");
            this.cbo_gusset_bolt_hori_offset_mode.AddItem(2, "中间");
            this.cbo_gusset_bolt_hori_offset_mode.AddItem(3, "右");
            this.cbo_gusset_bolt_hori_offset_mode.UpdateItems();
            this.cbo_gusset_bolt_hori_offset_mode.SelectedKey = 0;
        }
        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null)
            {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }
        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked)
            {
                retProps.CopyFrom(m_rootUIProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }
        /// <summary>
        /// 当前组件需要重写此方法，并刷新螺栓基础面板
        /// </summary>
        /// <param name="root"></param>
        public override void ClearUI(Control root = null)
        {
            // 刷新螺栓基础面板
            this.panel_bolt1_basic_property.RefreshControls();
            this.panel_bolt2_basic_property.RefreshControls();
            this.boltBasicPropertyPanel.RefreshControls();
            base.ClearUI(root);
        }
        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            this.SetPlaceholders(defaultValues);
        }
        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_rootUIProps.CopyFrom(srcProps);
        }
        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }
        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_rootUIProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_rootUIProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_rootUIProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_rootUIProps;
            return comp.UIProps;
        }

        private void btn_weld_props_Click(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_rootUIProps, null);
        }

        private void cbo_gusset_bolt_enabled_SelectedIndexChanged(object sender, EventArgs e)
        {
            bool flag;
            if (cbo_gusset_bolt_enabled.SelectedIndex != 2)
                flag = false;
            else
                flag = true;
            cbo_bolt_shape.Enabled = flag;
            act_gusset_bolt_shape.Enabled = flag;
            act_gusset_bolt_vert_count.Enabled = flag;
            tb_gusset_bolt_vert_count.Enabled = flag;
            act_gusset_bolt_vert_stride.Enabled = flag;
            tb_gusset_bolt_vert_stride.Enabled = flag;
            act_gusset_bolt_hori_count.Enabled = flag;
            tb_gusset_bolt_hori_count.Enabled = flag;
            act_gusset_bolt_hori_stride.Enabled = flag;
            tb_gusset_bolt_hori_stride.Enabled = flag;
            act_gusset_bolt_hori_offset_mode.Enabled = flag;
            cbo_gusset_bolt_hori_offset_mode.Enabled = flag;
            gusset_bolt_hori_offset.Enabled = flag;
            tb_gusset_bolt_hori_offset.Enabled = flag;
            act_gusset_bolt_vert_offset_mode.Enabled = flag;
            cbo_gusset_bolt_vert_offset_mode.Enabled = flag;
            act_gusset_bolt_vert_offset.Enabled = flag;
            tb_gusset_bolt_vert_offset.Enabled = flag;
        }
    }
}
