﻿using WindBraceConnection110.Common;
using System.Collections.Generic;
using System.Linq;

namespace WindBraceConnection110.Handlers
{
    public class BoltParse
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler = null;
        private readonly List<ParamData> m_paramDataMap = new List<ParamData>();

        private List<BraceData> m_subData = new List<BraceData>();
        public BoltParse(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }
        public bool Execute()
        {
            if (!InitParamData()) 
                return false;
            return true;
        }

        /// <summary>
        /// 所有与面板相关的值
        /// </summary>
        internal List<ParamData> ParamDataMap => m_paramDataMap;

        internal class ParamData
        {
            public int HorBoltsNum { get; set; }
            public double[] HorBoltsSpace { get; set; }
            public int VerBoltsNum { get; set; }
            public double[] VerBoltsSpace { get; set; }
            public double VerNearEdgeDist { get; set; }
            public double VerFarEdgeDist { get; set; }
            public int VerMeasureMethod { get; set; }
            public double VerMeasureOffset { get; set; }
            public BoltsConfig BoltsCfg { get; set; } = new BoltsConfig();
        }
        //螺栓属性
        private bool InitParamData()
        {
            double[] arrX,arrY;
            #region 支柱螺栓1 
            var boltGroup1 = new ParamData();
            boltGroup1.BoltsCfg.CopyFromBasicProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT1_BASIC_PROPS));
            boltGroup1.BoltsCfg.CopyFromHoleProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT1_HOLE_PROPS));
            boltGroup1.BoltsCfg.CopyFromContentProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT1_CONTENT_PROPS));
            boltGroup1.BoltsCfg.CopyFromExtendProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT1_EXTEND_PROPS));
            boltGroup1.BoltsCfg.CopyFromShapeProps
                (m_paramsHandler.GetIntValue(ParamNames.BOLT1_SHAPE));

            boltGroup1.HorBoltsNum = m_paramsHandler.GetIntValue(ParamNames.BOLT1_HORI_COUNT);
            if(boltGroup1.HorBoltsNum == int.MaxValue)
            {
                boltGroup1.HorBoltsNum = AdjustHoriBoltCount();
            }
            boltGroup1.VerBoltsNum = m_paramsHandler.GetIntValue(ParamNames.BOLT1_VERT_COUNT);

            arrX = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT1_HORI_STRIDE);
            arrY = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT1_VERT_STRIDE);
            boltGroup1.HorBoltsSpace = HandlerSpacing(arrX, boltGroup1.HorBoltsNum);
            boltGroup1.VerBoltsSpace = HandlerSpacing(arrY, boltGroup1.VerBoltsNum);
            boltGroup1.VerNearEdgeDist = m_paramsHandler.GetDoubleValue(ParamNames.BOLT1_HORI_BRACE_MARGIN);
            boltGroup1.VerFarEdgeDist = m_paramsHandler.GetDoubleValue(ParamNames.BOLT1_HORI_GUSSET_MARGIN);
            boltGroup1.VerMeasureMethod = m_paramsHandler.GetIntValue(ParamNames.BOLT1_VERT_OFFSET_MODE);
            boltGroup1.VerMeasureOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT1_VERT_OFFSET);
            ParamDataMap.Add(boltGroup1);
            #endregion

            #region 支柱螺栓2
            var boltGroup2 = new ParamData();
            boltGroup2.BoltsCfg.CopyFromBasicProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT2_BASIC_PROPS));
            boltGroup2.BoltsCfg.CopyFromHoleProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT2_HOLE_PROPS));
            boltGroup2.BoltsCfg.CopyFromContentProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT2_CONTENT_PROPS));
            boltGroup2.BoltsCfg.CopyFromExtendProps
                (m_paramsHandler.GetDObjectValue(ParamNames.BOLT2_EXTEND_PROPS));
            boltGroup2.BoltsCfg.CopyFromShapeProps
                (m_paramsHandler.GetIntValue(ParamNames.BOLT2_SHAPE));

            boltGroup2.HorBoltsNum = m_paramsHandler.GetIntValue(ParamNames.BOLT2_HORI_COUNT);
            if(boltGroup2.HorBoltsNum == int.MaxValue)
            {
                boltGroup2.HorBoltsNum = AdjustHoriBoltCount();
            }
            boltGroup2.VerBoltsNum = m_paramsHandler.GetIntValue(ParamNames.BOLT2_VERT_COUNT);
            if (boltGroup2.HorBoltsNum < 0 || boltGroup2.VerBoltsNum < 0) return false;

            arrX = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT2_HORI_STRIDE);
            arrY = m_paramsHandler.GetDoubleArrayValue(ParamNames.BOLT2_VERT_STRIDE);
            boltGroup2.HorBoltsSpace = HandlerSpacing(arrX, boltGroup2.HorBoltsNum);
            boltGroup2.VerBoltsSpace = HandlerSpacing(arrY, boltGroup2.VerBoltsNum);
            boltGroup2.VerNearEdgeDist = m_paramsHandler.GetDoubleValue(ParamNames.BOLT2_HORI_BRACE_MARGIN);
            boltGroup2.VerFarEdgeDist = m_paramsHandler.GetDoubleValue(ParamNames.BOLT2_HORI_GUSSET_MARGIN);
            boltGroup2.VerMeasureMethod = m_paramsHandler.GetIntValue(ParamNames.BOLT2_VERT_OFFSET_MODE);
            boltGroup2.VerMeasureOffset = m_paramsHandler.GetDoubleValue(ParamNames.BOLT2_VERT_OFFSET);
            ParamDataMap.Add(boltGroup2);
            #endregion

            return true;
        }
        /// <summary>
        /// 根据节点板位置自适应调整横向螺栓个数
        /// </summary>
        /// <returns></returns>
        private int AdjustHoriBoltCount()
        {
            int count = 2;
            if (m_context.EnableNotch)
                count = 1;
            return count;
        }
        /// <summary>
        /// 根据截面类型和节点板位置自适应调整纵向螺栓个数
        /// </summary>
        private int AdjustVertBoltCount(BraceProfileType type)
        {
            int count;
            switch (type)
            {
                case BraceProfileType.L:
                    count = 1;
                    break;
                case BraceProfileType.T:
                default:
                    count = m_context.EnableNotch ? 1 : 2;
                    break;
            }
            return count;
        }
        /// <summary>
        /// 根据螺栓数目调整螺栓间距的设置
        /// </summary>
        private double[] HandlerSpacing(double[] arr, int num)
        {
            if (num <= 0)
                return null;
            if (num == 1)
                return new double[] { 0 };
            var ret = new List<double>();
            if (arr.Length == 0)
                ret.AddRange(Enumerable.Repeat(60.0, num - 1));
            else
            {
                for (int i = 0; i < num - 1; i++)
                {
                    if (i > arr.Length - 1)
                        ret.Add(arr[arr.Length - 1]);
                    else
                        ret.Add(arr[i]);
                }
            }
            return ret.ToArray();
        }
    }
}
