﻿using System.Collections.Generic;
using LoongCAD.Geometry;
using LoongCAD.Util;
using LoongCAD.STC.Plugins;
using System.Linq;
using System;
using WindBraceConnection110.Common;

namespace WindBraceConnection110.Handlers
{
    public class GussetHandler
    {
        private ComponentInstanceContext m_context;
        private ParamsHandler m_paramsHandler;
        private PlateProperties m_plateProps = new PlateProperties();

        private Plane m_BaseGussetPlane;
        private List<BraceData> leftBraceData = new List<BraceData>();
        private List<BraceData> rightBraceData = new List<BraceData>();
        private Plane m_faceLeftBracePlane;
        private Plane m_faceRightBracePlane;

        //构建节点板的控制点
        private List<Vec3> leftPoints = new List<Vec3>();
        private List<Vec3> rightPoints = new List<Vec3>();
        private List<Vec3> upPoints = new List<Vec3>();
        private List<Vec3> downPoints = new List<Vec3>();

        private Vec3 m_axisY = Vec3.ZERO;
        private Vec3 m_axisZ = Vec3.ZERO;

        //节点板页参数
        private int m_platePosition = int.MaxValue;
        private double m_plateMoveZ = double.MaxValue;
        private int m_plateShape = int.MaxValue;
        private int m_plateUpShape = int.MaxValue;
        private int m_plateDownShape = int.MaxValue;

        //螺栓页参数
        List<BoltParse.ParamData> BoltParams = new List<BoltParse.ParamData>();

        //图形页参数
        private double m_leftFirst = double.MaxValue;
        private double m_leftLast = double.MaxValue;
        private double m_leftFirstCorner = double.MaxValue;
        private double m_leftLastCorner = double.MaxValue;
        private double m_rightFirst = double.MaxValue;
        private double m_rightLast = double.MaxValue;
        private double m_rightFirstCorner = double.MaxValue;
        private double m_rightLastCorner = double.MaxValue;

        private double m_leftLastBracePlateLength = double.MaxValue;
        private double m_leftBetweenBracePlateLength = double.MaxValue;
        private double m_leftFirstBracePlateLength = double.MaxValue;
        private double m_leftLastPlateCorner = double.MaxValue;
        private double m_leftFirstPlateCorner = double.MaxValue;

        private double m_rightLastBracePlateLength = double.MaxValue;
        private double m_rightBetweenBracePlateLength = double.MaxValue;
        private double m_rightFirstBracePlateLength = double.MaxValue;
        private double m_rightLastPlateCorner = double.MaxValue;
        private double m_rightFirstPlateCorner = double.MaxValue;

        private double m_leftPlateDist = double.MaxValue;
        private int m_leftBracePosition = 0;
        private double m_leftBraceDist = double.MaxValue;
        private double m_rightPlateDist = double.MaxValue;
        private int m_rightBracePosition = 0;
        private double m_rightBraceDist = double.MaxValue;

        public GussetHandler(ComponentInstanceContext context, ParamsHandler paramsHandler)
        {
            m_context = context;
            m_paramsHandler = paramsHandler;
        }

        public bool Execute()
        {
            if (!ParseProps())
                return false;
            //建立坐标系
            m_axisY = m_context.Y;
            m_axisZ = m_context.Z;

            //调整节点板在Z轴方向的位置
            AdjustGussetPlanePosition();
            MoveGussetZ();
            m_context.BaseGussetPlane = m_BaseGussetPlane;

            //构建节点板
            m_faceLeftBracePlane = m_context.FaceLeftBracePlane;
            m_faceRightBracePlane = m_context.FaceRightBracePlane;
            if (!GetBoltInfo())
                return false;

            HandlerBraceBetween();
            //左右侧形状控制
            this.ChangeRightPlateShape();
            this.ChangeLeftPlateShape();
            //存储左侧支撑构建节点板的控制点
            for (int i = leftBraceData.Count - 1; i >= 1; i--)
            {
                leftPoints.Add(leftBraceData[i].TopPoint);
                leftPoints.Add(leftBraceData[i - 1].BottomPoint);
            }
            //存储右侧支撑构建节点板的控制点
            for (int i = 0; i < rightBraceData.Count - 1; i++)
            {
                rightPoints.Add(rightBraceData[i].BottomPoint);
                rightPoints.Add(rightBraceData[i + 1].TopPoint);
            }
            //上下侧形状控制
            if (!AdjustPlateUpShape())
                return false;
            if (!AdjustPlateDownShape())
                return false;

            //存储参数信息
            this.DepositData();
            return true;
        }
        private bool ParseProps()
        {
            DObject plateProps = m_paramsHandler.GetDObjectValue(ParamNames.GUSSET_PLATE_PROPS);
            if (plateProps == null)
                return false;
            m_plateProps.SetProps(plateProps);
            if (m_plateProps.Thickness == 0)
                return false;

            //节点板页
            m_platePosition = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_POSITION);
            m_plateMoveZ = m_paramsHandler.GetDoubleValue(ParamNames.GUSSET_PLATE_MOVE_Z);
            m_plateShape = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_OVERALL_SHAPE);
            m_plateUpShape = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_UP_SHAPE);
            m_plateDownShape = m_paramsHandler.GetIntValue(ParamNames.GUSSET_PLATE_DOWN_SHAPE);

            //图形页 
            m_leftPlateDist = m_paramsHandler.GetDoubleValue(ParamNames.PIC_LEFT_BRACE_DISTANCE_TO_PART);
            m_rightPlateDist = m_paramsHandler.GetDoubleValue(ParamNames.PIC_RIGHT_BRACE_DISTANCE_TO_PART);

            m_leftBracePosition = m_paramsHandler.GetIntValue(ParamNames.PIC_LEFT_BRACE_POSITION);
            m_leftBraceDist = m_paramsHandler.GetDoubleValue(ParamNames.PIC_LEFT_DISTANCE_TO_BRACE);
            m_rightBracePosition = m_paramsHandler.GetIntValue(ParamNames.PIC_RIGHT_BRACE_POSITION);
            m_rightBraceDist = m_paramsHandler.GetDoubleValue(ParamNames.PIC_RIGHT_DISTANCE_TO_BRACE);
            //自适应调整支撑附近节点板边缘长度
            m_leftLastBracePlateLength = m_paramsHandler.GetDoubleValue(ParamNames.PIC_LEFT_LAST_BRACE_GUSSET_LENGTH);
            m_leftBetweenBracePlateLength = m_paramsHandler.GetDoubleValue(ParamNames.PIC_LEFT_BETWEEN_BRACE_GUSSET_LENGTH);
            m_leftFirstBracePlateLength = m_paramsHandler.GetDoubleValue(ParamNames.PIC_LEFT_FIRST_BRACE_GUSSET_LENGTH);
            if (m_leftLastBracePlateLength == double.MaxValue)
                m_leftLastBracePlateLength = AutoAdaptParams(true);

            if (m_leftBetweenBracePlateLength == double.MaxValue)
                m_leftBetweenBracePlateLength = AutoAdaptParams(true);

            if (m_leftFirstBracePlateLength == double.MaxValue)
                m_leftFirstBracePlateLength = AutoAdaptParams(true);

            m_rightLastBracePlateLength = m_paramsHandler.GetDoubleValue(ParamNames.PIC_RIGHT_LAST_BRACE_GUSSET_LENGTH);
            m_rightBetweenBracePlateLength = m_paramsHandler.GetDoubleValue(ParamNames.PIC_RIGHT_BETWEEN_BRACE_GUSSET_LENGTH);
            m_rightFirstBracePlateLength = m_paramsHandler.GetDoubleValue(ParamNames.PIC_RIGHT_FIRST_BRACE_GUSSET_LENGTH);
            if (m_rightLastBracePlateLength == double.MaxValue)
                m_rightLastBracePlateLength = AutoAdaptParams(false);

            if (m_rightBetweenBracePlateLength == double.MaxValue)
                m_rightBetweenBracePlateLength = AutoAdaptParams(false);

            if (m_rightFirstBracePlateLength == double.MaxValue)
                m_rightFirstBracePlateLength = AutoAdaptParams(false);

            m_leftLastPlateCorner = m_paramsHandler.GetDoubleValue(ParamNames.PIC_LEFT_LAST_BRACE_GUSSET_CORNER);
            m_leftFirstPlateCorner = m_paramsHandler.GetDoubleValue(ParamNames.PIC_LEFT_FIRST_BRACE_GUSSET_CORNER);
            m_rightLastPlateCorner = m_paramsHandler.GetDoubleValue(ParamNames.PIC_RIGHT_LAST_BRACE_GUSSET_CORNER);
            m_rightFirstPlateCorner = m_paramsHandler.GetDoubleValue(ParamNames.PIC_RIGHT_FIRST_BRACE_GUSSET_CORNER);
            return true;
        }
        /// <summary>
        /// 自适应调整支撑附近节点板边缘长度
        /// <paramref name="flag">true时为左侧，false时右侧</paramref>
        /// </summary>
        private double AutoAdaptParams(bool flag)
        {
            double[] items;
            if (flag)
            {
                items = new double[]
                {
                    m_paramsHandler.GetDoubleValue(ParamNames.PIC_LEFT_LAST_BRACE_GUSSET_LENGTH),
                    m_paramsHandler.GetDoubleValue(ParamNames.PIC_LEFT_BETWEEN_BRACE_GUSSET_LENGTH),
                    m_paramsHandler.GetDoubleValue(ParamNames.PIC_LEFT_FIRST_BRACE_GUSSET_LENGTH)
                };
            }
            else
            {
                items = new double[]
                {
                    m_paramsHandler.GetDoubleValue(ParamNames.PIC_RIGHT_LAST_BRACE_GUSSET_LENGTH),
                    m_paramsHandler.GetDoubleValue(ParamNames.PIC_RIGHT_BETWEEN_BRACE_GUSSET_LENGTH),
                    m_paramsHandler.GetDoubleValue(ParamNames.PIC_RIGHT_FIRST_BRACE_GUSSET_LENGTH)
                };
            }
            int num = 0;
            foreach (var item in items)
            {
                if (item == double.MaxValue)
                    num++;
            }
            double paramValue = double.MaxValue;
            switch (num)
            {
                case 3:
                    //若面板均未填写
                    paramValue = flag ? m_leftBraceDist / 2 : m_rightBraceDist / 2;
                    //自适应支撑之间的距离与节点板边缘距离
                    if (flag && m_leftBracePosition == 2)
                        paramValue = 0.0;
                    if (!flag && m_rightBracePosition == 2)
                        paramValue = 0.0;
                    break;
                case 2:
                    foreach (var value in items)
                    {
                        if (value != double.MaxValue)
                            paramValue = value;
                    }
                    break;
                case 1:
                    paramValue = items[2] == double.MaxValue ? items[0] : items[2];
                    break;
                default:
                    break;
            }
            return paramValue;
        }
        /// <summary>
        /// 调整节点板位置
        /// </summary>
        private void AdjustGussetPlanePosition()
        {
            Plane plane1 = m_context.BracePlane;
            Plane plane2 = m_context.OppositeBracePlane;
            //判断标准
            bool flag = plane1.Normal.Normalize().Dot(m_axisZ) > 0;

            switch (m_platePosition)
            {
                case 1:
                    //向Z轴负方向移动
                    m_BaseGussetPlane = flag ? plane2 : plane1;
                    break;
                case 2:
                    Plane offsetPlane = m_context.BraceOffsetPlane;
                    Plane oppositeOffsetPlane = m_context.OppositeBraceOffsetPlane;
                    if (offsetPlane != null && oppositeOffsetPlane != null)
                        //有腹板截面节点板偏移处理
                        m_BaseGussetPlane = flag ? oppositeOffsetPlane : offsetPlane;
                    else
                    {
                        var dist = plane1.Dist(plane2.Origin);
                        m_BaseGussetPlane = plane1.Move(-dist / 2 - m_plateProps.Thickness / 2);
                    }
                    break;
                case 3:
                    //向Z轴正方向移动
                    m_BaseGussetPlane = flag ? plane1 : plane2;
                    break;
                case 0:
                default:
                    m_BaseGussetPlane = plane1;
                    break;
            }
        }
        /// <summary>
        /// 调整节点板在Z轴方向的距离
        /// </summary>
        private void MoveGussetZ()
        {
            var dot = m_axisZ.Dot(m_BaseGussetPlane.Normal);
            if (dot > 0)
            {
                m_BaseGussetPlane.MoveThis(m_plateMoveZ);
            }
            else
            {
                m_BaseGussetPlane.MoveThis(-m_plateMoveZ);
            }
        }
        /// <summary>
        /// 调节不同侧面点击顺序的节点板边缘值
        /// </summary>
        private void HandlerBraceClickOrder()
        {
            //处理左侧支撑点积顺序
            if (leftBraceData.First().ClickNumber < leftBraceData.Last().ClickNumber)
            {
                m_leftFirst = m_leftFirstBracePlateLength;
                m_leftLast = m_leftLastBracePlateLength;
                m_leftFirstCorner = m_leftFirstPlateCorner;
                m_leftLastCorner = m_leftLastPlateCorner;
            }
            else
            {
                m_leftLast = m_leftFirstBracePlateLength;
                m_leftFirst = m_leftLastBracePlateLength;
                m_leftFirstCorner = m_leftLastPlateCorner;
                m_leftLastCorner = m_leftFirstPlateCorner;
            }
            //处理右侧支撑点积顺序
            if (rightBraceData.First().ClickNumber < rightBraceData.Last().ClickNumber)
            {
                m_rightFirst = m_rightFirstBracePlateLength;
                m_rightLast = m_rightLastBracePlateLength;
                m_rightFirstCorner = m_rightFirstPlateCorner;
                m_rightLastCorner = m_rightLastPlateCorner;
            }
            else
            {
                m_rightLast = m_rightFirstBracePlateLength;
                m_rightFirst = m_rightLastBracePlateLength;
                m_rightFirstCorner = m_rightLastPlateCorner;
                m_rightLastCorner = m_rightFirstPlateCorner;
            }
        }
        /// <summary>
        /// 得到决定板形状的螺栓信息
        /// </summary>
        private bool GetBoltInfo()
        {
            if (m_paramsHandler.GetIntValue(ParamNames.GUSSET_BRACE_NOTCH) == 3)
                m_context.EnableNotch = true;
            BoltParse boltParse = new BoltParse(m_context, m_paramsHandler);
            if (!boltParse.Execute())
                return false;
            BoltParams = boltParse.ParamDataMap;
            return true;
        }
        //通过切割调整支撑之间的距离
        private void HandlerBraceBetween()
        {
            leftBraceData = m_context.LeftBraceData;
            rightBraceData = m_context.RightBraceData;
            //处理支撑点击顺序
            HandlerBraceClickOrder();

            //调整支撑之间
            if (m_rightBracePosition == 2 && rightBraceData.Count > 1)
            {
                var len = rightBraceData.Count - 1;
                CutBrace(rightBraceData[len], rightBraceData[len - 1], m_rightBraceDist);
            }
            if (m_leftBracePosition == 2 && leftBraceData.Count > 1)
            {
                CutBrace(leftBraceData[1], leftBraceData[0], m_leftBraceDist);
            }
        }
        //切割支撑
        private void CutBrace(BraceData topBrace, BraceData bottomBrace, double dist)
        {
            var topNormal = topBrace.TopPlane.Normal.Normalize();
            var bottomNormal = bottomBrace.BottomPlane.Normal.Normalize();
            //计算二面角
            double cosine = topNormal.Dot(-bottomNormal);
            //弧度
            double angle = Math.Acos(cosine) * 180 / Math.PI;
            //计算切割面交点
            var topDist = topBrace.TopPlane.Dist(topBrace.BottomPlane.Origin) / 2;
            var bottomDist = bottomBrace.TopPlane.Dist(bottomBrace.BottomPlane.Origin) / 2;
            var point = m_BaseGussetPlane.Intersect(topBrace.TopPlane.Move(- topDist), 
                bottomBrace.BottomPlane.Move(-bottomDist));

            var topAxisZ = topBrace.BraceLine.Cross(topNormal);
            Plane cutBottomBrace = new Plane(point, 
                BraceTools.RotateVectorInPlane(topNormal, topAxisZ, angle / 2)).Move(dist / 2);
            m_context.TargetComp.CreateCutPlane(bottomBrace.Beam, cutBottomBrace, true);

            var bottomAxisZ = bottomBrace.BraceLine.Cross(bottomNormal);
            Plane cutTopBrace = new Plane(point,
                BraceTools.RotateVectorInPlane(bottomNormal, bottomAxisZ, angle / 2)).Move(dist / 2);
            m_context.TargetComp.CreateCutPlane(topBrace.Beam, cutTopBrace, true);

        }
        /// <summary>
        /// 得到支撑上的板长度
        /// </summary>
        /// <param name="flag">true时为右侧支撑，false时为左侧支撑</param>
        /// <returns></returns>
        private double GetBracePlateLength(bool flag)
        {
            double plateLength;
            var bolts = BoltParams.Last();
            if (flag)
                bolts = BoltParams.First();
            plateLength = bolts.VerNearEdgeDist + bolts.VerFarEdgeDist
                    + BraceTools.SumArray(bolts.HorBoltsSpace);
            return plateLength;
        }
        /// <summary>
        /// 改变节点板右侧形状
        /// </summary>
        private void ChangeRightPlateShape()
        {
            switch (m_plateShape)
            {
                case 3:
                    CreateRightShape6();
                    break;
                case 0:
                default:
                    CreateRightShape6();
                    PrepareBraceBoundary(rightBraceData.ToArray(), true);
                    PrepareConvexHull(rightBraceData.ToArray(), true);
                    HandlerAlign(rightBraceData, true);
                    break;
            }
        }
        /// <summary>
        /// 改变节点板左侧形状
        /// </summary>
        private void ChangeLeftPlateShape()
        {
            switch (m_plateShape)
            {
                case 3:
                    CreateLeftShape6();
                    break;
                case 0:
                default:
                    CreateLeftShape6();
                    PrepareBraceBoundary(leftBraceData.ToArray(), false);
                    PrepareConvexHull(leftBraceData.ToArray(), false);
                    HandlerAlign(leftBraceData, false);
                    break;
            }
        }
        //对齐支撑
        #region 创建形状6
        private void CreateRightShape6()
        {
            AdjustBraceToMain(rightBraceData, m_rightPlateDist, m_faceRightBracePlane);
            AdjustDistBetweenBrace(rightBraceData.ToArray(), m_rightBraceDist, m_rightBracePosition);
            InitGussetBorderPlane(rightBraceData.ToArray(), true);
        }
        private void CreateLeftShape6()
        {
            AdjustBraceToMain(leftBraceData, m_leftPlateDist, m_faceLeftBracePlane);
            AdjustDistBetweenBrace(leftBraceData.ToArray(), m_leftBraceDist, m_leftBracePosition);
            InitGussetBorderPlane(leftBraceData.ToArray(), false);
        }
        #endregion

        #region 创建凸包的形状
        /// <summary>
        /// 处理相邻支撑之间的边界线
        /// </summary>
        /// <param name="flag">true时为右侧支撑，false时为左侧支撑</param>
        private void PrepareBraceBoundary(BraceData[] braceDatas, bool flag)
        {
            double braceGap = flag ? m_rightBetweenBracePlateLength : m_leftBetweenBracePlateLength;

            for (var i = 0; i < braceDatas.Length - 1; ++i)
            {
                var cur = braceDatas[i];
                var next = braceDatas[i + 1];
                var curBoundary = GetCurBorderPlane(cur, braceGap);
                var nextBoundary = GetNextBorderPlane(next, braceGap);
                //处理平行
                if (curBoundary.Normal.IsParallelTo(nextBoundary.Normal))
                    continue;
                //边界的交点，不会改变
                var boundaryP = m_BaseGussetPlane.Intersect(curBoundary, nextBoundary);
                var nextDist = GetConvexPlane(next).SignedDist(boundaryP);
                //边界交点在对齐面的左侧，需要移动
                if (nextDist > 0)
                {
                    UpdateFarAlignPlane(next, boundaryP);
                    UpdateAlignPoint(next, flag);
                }
                var curDist = GetConvexPlane(cur).SignedDist(boundaryP);
                //边界交点在对齐面的左侧，需要移动
                if (curDist > 0)
                {
                    UpdateFarAlignPlane(cur, boundaryP);
                    UpdateAlignPoint(cur, flag);
                }
            }
        }
        /// <summary>
        ///处理凸包 
        /// </summary>
        /// <param name="braceDatas">侧支撑</param>
        /// <param name="flag">true时为右侧支撑，false时为左侧支撑</param>
        private void PrepareConvexHull(BraceData[] braceDatas, bool flag)
        {
            double braceGap = flag ? m_rightBetweenBracePlateLength : m_leftBetweenBracePlateLength;

            //按照排序的braceData检查相邻的支撑的对齐面是不是交于支撑之间。
            for (int i = 0; i < braceDatas.Length - 1; ++i)
            {
                var cur = braceDatas[i];
                var next = braceDatas[i + 1];
                var ret = CheckIntersection(cur, next, braceGap);
                //交点在上边界外
                //可以往后移动next支撑，不需要回溯
                if (ret < 0)
                {
                    //cur的对齐面，cur的边界的交点
                    var adjustP = m_BaseGussetPlane.Intersect(GetConvexPlane(cur), GetCurBorderPlane(cur, braceGap));
                    UpdateFarAlignPlane(next, adjustP);
                    UpdateAlignPoint(next, flag);
                }
                //此时调整过后的交点一定在上边界内，
                //二次检查有没有在下边界外
                ret = CheckIntersection(cur, next, braceGap);
                //交点在下边界外
                //需要往后移动cur支撑，需要回溯
                if (ret > 0)
                {
                    //next的对齐面，next的边界的交点
                    var adjustP = m_BaseGussetPlane.Intersect(GetConvexPlane(next), GetNextBorderPlane(next, braceGap));
                    UpdateFarAlignPlane(cur, adjustP);
                    UpdateAlignPoint(cur, flag);
                    int k = i - 1;//回到上一组
                    //反向循环处理，一直到某一组支撑不需要调整cur支撑
                    while (k >= 0)
                    {
                        cur = braceDatas[k];
                        next = braceDatas[k + 1];
                        ret = CheckIntersection(cur, next, braceGap);
                        if (ret > 0)
                        {
                            //next的对齐面，next的边界的交点
                            adjustP = m_BaseGussetPlane.Intersect(GetConvexPlane(next), GetNextBorderPlane(next, braceGap));
                            UpdateFarAlignPlane(cur, adjustP);
                            UpdateAlignPoint(cur, flag);
                            --k;
                            continue;
                        }
                        break;
                    }
                }
            }
        }
        /// <summary>
        /// 更新对齐面
        /// </summary>
        /// <param name="braceDatas">侧支撑</param>
        /// <param name="flag">true时为右侧支撑，false时为左侧支撑</param>
        private void HandlerAlign(List<BraceData> braceDatas, bool flag)
        {
            foreach (var brace in braceDatas)
            {
                var length = GetBracePlateLength(flag);
                brace.AlignPlane = brace.GussetBorderPlane.Move(-length);
            }
        }
        /// <summary>
        /// 检查cur,next的交点位置。
        /// </summary>
        /// <param name="cur"></param>
        /// <param name="next"></param>
        /// <returns>小于0说明到cur边界的距离小于0，大于0说明到next边界距离小于0</returns>
        private int CheckIntersection(BraceData cur, BraceData next, double braceGap)
        {
            return CheckIntersection(GetConvexPlane(cur), GetConvexPlane(next), GetCurBorderPlane(cur, braceGap),
                GetNextBorderPlane(next, braceGap));
        }
        /// <summary>
        /// 检查交点位于那一侧
        /// </summary>
        /// <param name="curRefPl"></param>
        /// <param name="nextRefPl"></param>
        /// <param name="curBorderPl"></param>
        /// <param name="nextBorderPl"></param>
        /// <returns>小于0说明到cur边界的距离小于0，大于0说明到next边界距离小于0</returns>
        private int CheckIntersection(Plane curRefPl, Plane nextRefPl, Plane curBorderPl, Plane nextBorderPl)
        {
            //得到此时的交点
            var refP = m_BaseGussetPlane.Intersect(curRefPl, nextRefPl);
            //与cur边界的距离
            var curDist = curBorderPl.SignedDist(refP);
            //与next边界的距离
            var nextDist = nextBorderPl.SignedDist(refP);
            if (curDist < 0)
            {
                return -1;
            }
            if (nextDist < 0)
            {
                return 1;
            }
            return 0;
        }
        /// <summary>
        /// 获取当前支撑的边界
        /// </summary>
        private Plane GetCurBorderPlane(BraceData cur, double braceGap)
        {
            return new Plane(cur.BottomPlane).Move(braceGap);
        }
        /// <summary>
        /// 获取下一个支撑的边界
        /// </summary>
        private Plane GetNextBorderPlane(BraceData next, double braceGap)
        {
            return new Plane(next.TopPlane).Move(braceGap);
        }
        /// <summary>
        /// 获取凸面
        /// </summary>
        private Plane GetConvexPlane(BraceData braceData)
        {
            return new Plane(braceData.GussetBorderPlane);
        }
        /// <summary>
        /// 更新节点板边界面
        /// </summary>
        private void UpdateFarAlignPlane(BraceData brace, Vec3 point)
        {
            brace.GussetBorderPlane = new Plane(point, brace.BraceLine);
        }
        /// <summary>
        /// 更新支撑构建节点板的控制点
        /// </summary>
        private void UpdateAlignPoint(BraceData brace, bool flag)
        {
            var first = m_leftFirst;
            var between = m_leftBetweenBracePlateLength;
            var last = m_leftLast;
            if (flag)
            {
                first = m_rightFirst;
                between = m_rightBetweenBracePlateLength;
                last = m_rightLast;
            }

            switch (brace.PosType)
            {
                case BracePosType.FIRST:
                    brace.TopPoint = brace.GussetBorderPlane.Intersect(m_BaseGussetPlane, 
                        brace.TopPlane.Move(first));
                    brace.BottomPoint = brace.GussetBorderPlane.Intersect(m_BaseGussetPlane,
                        brace.BottomPlane.Move(between));
                    break;
                case BracePosType.MIDDLE:
                    brace.TopPoint = brace.GussetBorderPlane.Intersect(m_BaseGussetPlane,
                        brace.TopPlane.Move(between));
                    brace.BottomPoint = brace.GussetBorderPlane.Intersect(m_BaseGussetPlane,
                        brace.BottomPlane.Move(between));
                    break;
                default:
                    brace.TopPoint = brace.GussetBorderPlane.Intersect(m_BaseGussetPlane,
                        brace.TopPlane.Move(between));
                    brace.BottomPoint = brace.GussetBorderPlane.Intersect(m_BaseGussetPlane,
                        brace.BottomPlane.Move(last));
                    break;
            }
        }
        #endregion
        /// <summary>
        /// 调整左右侧支撑到主零件的距离
        /// </summary>
        /// <param name="braces">侧支撑</param>
        /// <param name="dist">距离</param>
        /// <param name="mainPlane">侧面</param>
        private void AdjustBraceToMain(List<BraceData> braces, double dist, Plane mainPlane)
        {
            foreach(var brace in braces)
            {
                var line = brace.BraceLine;
                var point1 = mainPlane.Move(dist).Intersect(brace.TopPlane, m_BaseGussetPlane);
                var point2 = mainPlane.Move(dist).Intersect(brace.BottomPlane, m_BaseGussetPlane);
                brace.AlignPlane = GetDistMinPlane(brace.FarPoint, 
                    new Plane(point1, line), new Plane(point2, line)
                    );
            }
        }
        /// <summary>
        /// 调整左右侧支撑之间的距离
        /// </summary>
        /// <param name="braces">侧支撑</param>
        /// <param name="dist">距离</param>
        /// <param name="type">支撑间位置类型</param>
        private void AdjustDistBetweenBrace(BraceData[] braces, double dist, int type)
        {
            int len = braces.Length;
            if (len == 1)
                return;
            if (type == 2)
                return;

            //调节每侧前面的支撑
            for(int i = 0; i < len - 1; i++)
            {
                var point = braces[i + 1].TopPlane.Move(dist)
                    .Intersect(m_BaseGussetPlane, braces[i].BottomPlane);
                braces[i].AlignPlane = GetDistMinPlane(braces[i].FarPoint,
                    braces[i].AlignPlane, new Plane(point, braces[i].BraceLine));
            }
            //调节每侧后面的支撑
            for(int i = 1; i < len; i++)
            {
                var point = braces[i - 1].BottomPlane.Move(dist)
                    .Intersect(m_BaseGussetPlane, braces[i].TopPlane);
                braces[i].AlignPlane = GetDistMinPlane(braces[i].FarPoint,
                    braces[i].AlignPlane, new Plane(point, braces[i].BraceLine));
            }
        }
        /// <summary>
        /// 初始化节点板边界面
        /// </summary>
        /// <param name="braces">侧支撑</param>
        /// <param name="flag">true时为右侧支撑，false时为左侧支撑</param>
        private void InitGussetBorderPlane(BraceData[] braces, bool flag)
        {
            var length = GetBracePlateLength(flag);
            foreach (var brace in braces)
                brace.GussetBorderPlane = brace.AlignPlane.Move(length);
            var first = m_leftFirst;
            var between = m_leftBetweenBracePlateLength;
            var last = m_leftLast;
            if (flag) 
            {
                first = m_rightFirst;
                between = m_rightBetweenBracePlateLength;
                last = m_rightLast;
            }
            braces[0].TopPoint = braces[0].GussetBorderPlane
                .Intersect(m_BaseGussetPlane, braces[0].TopPlane.Move(first));
            for(int i = 0; i < braces.Length - 1; i++)
            {
                braces[i].BottomPoint = braces[i].GussetBorderPlane
                    .Intersect(m_BaseGussetPlane, braces[i].BottomPlane.Move(between));
                braces[i + 1].TopPoint = braces[i + 1].GussetBorderPlane
                    .Intersect(m_BaseGussetPlane, braces[i + 1].TopPlane.Move(between));
            }
            braces.Last().BottomPoint = braces.Last().GussetBorderPlane
                .Intersect(m_BaseGussetPlane, braces.Last().BottomPlane.Move(last));
        }
        /// <summary>
        /// 改变节点板上侧形状
        /// </summary>
        private bool AdjustPlateUpShape()
        {
            Vec3 leftPoint, rightPoint;
            Vec3[] points;

            if(m_plateShape == 2)
            {
                points = GetSquareCtrlPoints(leftBraceData.First().TopPoint, 
                    rightBraceData.First().TopPoint, - m_axisY);
                if (points == null)
                    return false;
            }
            else
                points = GetUpCtrlPoints();
               
            leftPoint = points[0];
            rightPoint = points[1];

            upPoints.Add(leftPoint);
            switch (m_plateUpShape)
            {
                case 2:
                    {
                        var plane = new Plane(rightPoint, m_axisY);
                        var point = plane.Intersect(m_BaseGussetPlane, m_faceRightBracePlane);
                        
                        //计算与主零件的交点
                        var skewPlane = new Plane(point, leftPoint - point, m_axisZ);
                        upPoints.Add(skewPlane.Intersect(m_BaseGussetPlane, m_faceLeftBracePlane));
                       
                        upPoints.Add(point);
                    }
                    break;
                case 3:
                    {
                        var plane = new Plane(leftPoint, m_axisY);
                        var point = plane.Intersect(m_BaseGussetPlane, m_faceLeftBracePlane);
                        upPoints.Add(point);

                        //计算与主零件的交点
                        var skewPlane = new Plane(point, rightPoint - point, m_axisZ);
                        upPoints.Add(skewPlane.Intersect(m_BaseGussetPlane, m_faceRightBracePlane));
                    }
                    break;
                case 4:
                    {
                        var plane1 = new Plane(leftPoint, m_axisY);
                        upPoints.Add(plane1.Intersect(m_BaseGussetPlane, m_faceLeftBracePlane));
                        var plane2 = new Plane(rightPoint, m_axisY);
                        upPoints.Add(plane2.Intersect(m_BaseGussetPlane, m_faceRightBracePlane));
                    }
                    break;
                case 5:
                    {
                        Plane parallel = new Plane(rightPoint, rightBraceData.First().TopPlane.Normal);
                        var point = parallel.Intersect(m_BaseGussetPlane, m_faceRightBracePlane);
                        var plane = new Plane(point, m_axisY);
                        upPoints.Add(plane.Intersect(m_faceLeftBracePlane, m_BaseGussetPlane));
                        upPoints.Add(point);
                    }
                    break;
                case 0:
                default:
                    break;
            }
            upPoints.Add(rightPoint);

            return true;
        }
        /// <summary>
        /// 改变节点板下侧形状
        /// </summary>
        private bool AdjustPlateDownShape()
        {
            Vec3 leftPoint, rightPoint;
            Vec3[] points;

            if (m_plateShape == 2)
            {
                points = GetSquareCtrlPoints(leftBraceData.Last().BottomPoint,
                    rightBraceData.Last().BottomPoint, m_axisY);
                if (points == null)
                    return false;
            }
            else
                points = GetDownCtrlPoints();
            leftPoint = points[0];
            rightPoint = points[1];

            downPoints.Add(rightPoint);
            switch (m_plateDownShape)
            {
                case 2:
                    {
                        var plane = new Plane(rightPoint, m_axisY);
                        var point = plane.Intersect(m_BaseGussetPlane, m_faceRightBracePlane);
                        downPoints.Add(point);

                        //计算与主零件的交点
                        var skewPlane = new Plane(point, leftPoint - point, m_axisZ);
                        downPoints.Add(skewPlane.Intersect(m_BaseGussetPlane, m_faceLeftBracePlane));
                    }
                    break;
                case 3:
                    {
                        var plane = new Plane(leftPoint, m_axisY);
                        var point = plane.Intersect(m_BaseGussetPlane, m_faceLeftBracePlane);
                        //计算与主零件的交点
                        var skewPlane = new Plane(point, rightPoint - point, m_axisZ);
                        downPoints.Add(skewPlane.Intersect(m_BaseGussetPlane, m_faceRightBracePlane));

                        downPoints.Add(point);
                    }
                    break;
                case 4:
                    {
                        var plane1 = new Plane(rightPoint, m_axisY);
                        downPoints.Add(plane1.Intersect(m_BaseGussetPlane, m_faceRightBracePlane));
                        var plane2 = new Plane(leftPoint, m_axisY);
                        downPoints.Add(plane2.Intersect(m_BaseGussetPlane, m_faceLeftBracePlane));
                    }
                    break;
                case 5:
                    {
                        Plane parallel = new Plane(rightPoint, rightBraceData.Last().BottomPlane.Normal);
                        var point = parallel.Intersect(m_BaseGussetPlane, m_faceRightBracePlane);
                        downPoints.Add(point);
                        var plane = new Plane(point, m_axisY);
                        downPoints.Add(plane.Intersect(m_BaseGussetPlane, m_faceLeftBracePlane));
                    }
                    break;
                case 0:
                default:
                    break;
            }
            downPoints.Add(leftPoint);

            return true;
        }
        private Vec3[] GetSquareCtrlPoints(Vec3 leftPoint, Vec3 rightPoint, Vec3 Y)
        {
            var h = m_plateProps.Height;
            var b = m_plateProps.Width;
            if (b == 0 || h == 0)
                return null;

            var points = new Vec3[2];
            //构建坐标系
            Vec3 lx = m_faceLeftBracePlane.Normal.Normalize();
            Vec3 rx = m_faceRightBracePlane.Normal.Normalize();

            //计算中心点
            double len = m_faceLeftBracePlane.Dist(m_faceRightBracePlane.Origin);
            var plane1 = new Plane(leftBraceData.First().TopPoint, m_axisY);
            var plane2 = new Plane(leftBraceData.Last().BottomPoint, m_axisY);
            var point = plane1.Intersect(m_BaseGussetPlane, m_faceLeftBracePlane) / 2
                + plane2.Intersect(m_BaseGussetPlane, m_faceLeftBracePlane) / 2 + len / 2 * rx;
            //中心面
            Plane center = new Plane(point, m_axisY);

            var leftDist = center.Dist(leftPoint);
            var rightDist = center.Dist(rightPoint);
            
            if(b == double.MaxValue)
            {
                double leftMax = GetMaxpointDist(leftBraceData, m_faceLeftBracePlane);
                double rightMax = GetMaxpointDist(rightBraceData, m_faceRightBracePlane);
                var dist1 = m_faceLeftBracePlane.Dist(leftPoint);
                var dist2 = m_faceRightBracePlane.Dist(rightPoint);
                var min = Math.Min(leftDist, rightDist);
                points[0] = leftPoint + lx * (leftMax - dist1) - Y * min;
                points[1] = rightPoint + rx * (rightMax - dist2) - Y * min;
            }
            else
            {
                points[0] = point + lx * b / 2;
                points[1] = point + rx * b / 2;
            }

            if(h == double.MaxValue)
            {
                points[0] += Y * leftDist;
                points[1] += Y * rightDist;
            }
            else
            {
                points[0] += Y * h / 2;
                points[1] += Y * h / 2;
            }
            return points;
        }
        private Vec3[] GetUpCtrlPoints()
        {
            var left = leftBraceData.First();
            var right = rightBraceData.First();
            Vec3[] points = new Vec3[2];
            //节点板拐角为90度的特殊处理
            if (m_leftFirstCorner == 90.0)
                points[0] = left.BottomPoint;
            else
                points[0] = left.TopPoint;
            if (m_rightFirstCorner == 90.0)
                points[1] = right.BottomPoint;
            else
                points[1] = right.TopPoint;
            return points;
        }
        private Vec3[] GetDownCtrlPoints()
        {
            var left = leftBraceData.Last();
            var right = rightBraceData.Last();
            Vec3[] points = new Vec3[2];
            //节点板拐角为90度的特殊处理
            if (m_leftLastCorner == 90.0)
                points[0] = left.TopPoint;
            else
                points[0] = left.BottomPoint;
            if (m_rightLastCorner == 90.0)
                points[1] = right.TopPoint;
            else
                points[1] = right.BottomPoint;
            return points;
        }
        /// <summary>
        /// 存储节点板数据
        /// </summary>
        private void DepositData()
        {
            m_context.LeftBraceData = leftBraceData;
            m_context.RightBraceData = rightBraceData;

            m_context.LeftPoints = leftPoints;
            m_context.RightPoints = rightPoints;
            m_context.UpPoints = upPoints;
            m_context.DownPoints = downPoints;
        }
        /// <summary>
        /// 得到平行的面集合中距离参照点最近的面
        /// </summary>
        /// <param name="point">参照点</param>
        /// <param name="planes">面集合</param>
        /// <returns></returns>
        private Plane GetDistMinPlane(Vec3 point, params Plane[] planes)
        {
            int num = -1;
            double min = double.MaxValue;
            for(int i = 0; i < planes.Length; i++)
            {
                var temp = planes[i].Dist(point);
                if(temp < min)
                {
                    min = temp;
                    num = i;
                }
            }
            return planes[num];
        }
        /// <summary>
        /// 得到某侧节点板控制点到指定面距离的最大值
        /// </summary>
        /// <param name="braces"></param>
        /// <param name="plane"></param>
        /// <returns></returns>
        private double GetMaxpointDist(List<BraceData> braces, Plane plane)
        {
            double max = -1.0;
            foreach (var brace in braces)
            {
                var dist1 = plane.Dist(brace.TopPoint);
                var dist2 = plane.Dist(brace.BottomPoint);

                var temp = Math.Max(dist1, dist2);
                if (temp > max)
                    max = temp;
            }
            return max;
        }
    }
}
