﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using TargetComponentType = LoongCAD.STC.Object3D.DetailComponent;
namespace stiffeners1030.Main
{
    /// <summary>
    /// 组件主窗口，实现此窗口的要点如下:
    /// <list type="bullet">
    ///     <item>
    ///         <description>节点组件继承自LoongCAD.UI.CompConnectionFormBase_Instance类</description>
    ///     </item>
    ///     <item>
    ///         <description>节点组件继承自LoongCAD.UI.CompDetailFormBase_Instance</description>
    ///     </item>
    /// </list>
    /// </summary>
    public partial class Form_PluginMain : LoongCAD.UI.Component.Form_DetailComponentProps
    {
        private PluginMain m_plugin;
        public PluginMain Plugin { get => m_plugin; set => m_plugin = value; }
        private DObject m_weldPropsBatch = new DObject();
        
        //----------
        private DObject m_rootUIProps = new DObject();
        private ComponentPlugin m_refPlugin = null;
        private Form_WeldParameter m_weldForm = null;
        public override ComponentPlugin ReferencePlugin => m_refPlugin;

        public Form_PluginMain(ComponentPlugin refPlugin)
        {
            InitializeComponent();
            m_refPlugin = refPlugin;
            this.OnPostIntialize();
        }

        private void OnPostIntialize()
        {
            //配置图片选项卡
            parameters_beam_fitting_option.ClearItems();
            parameters_beam_fitting_option.AddItem(0,0);
            parameters_beam_fitting_option.AddItem(1,1);
            parameters_beam_fitting_option.AddItem(2,2);
            parameters_beam_fitting_option.UpdateItems();
            parameters_beam_fitting_option.Size = new Size(160, 160);
            //配置对称锁定选项
            pic_lock_select.ClearItems();
            pic_lock_select.AddItem(0,"默认");
            pic_lock_select.AddItem(1,"对称");
            pic_lock_select.AddItem(2,"不对称");
            pic_lock_select.UpdateItems();

            parts_lock_select.ClearItems();
            parts_lock_select.AddItem(0,"默认");
            parts_lock_select.AddItem(1,"对称");
            parts_lock_select.AddItem(2,"不对称");
            parts_lock_select.UpdateItems();

            foreach(TabPage curPage in this.tab_main.TabPages)
            {
                curPage.AutoScroll = true;
            }
        }


        //焊缝相关
        /// <summary>
        /// 设置当前窗口的属性值
        /// </summary>
        /// <param name="paras"></param>
        /// <param name="root"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        public override void SetParameters(DObject paras, Control root = null)
        {
            base.SetParameters(paras, root);
            if (root == null) {
                m_rootUIProps.Clear();
                m_rootUIProps.CopyFrom(paras);
            }
        }
        
        /// <summary>
        /// 应用焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ApplyWeldProps(DObject srcProps)
        {
            m_rootUIProps.CopyFrom(srcProps);
        }

        /// <summary>
        /// 修改焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected void ModifyWeldProps(DObject srcProps)
        {
            this.ModifyFocusedProps(srcProps);
        }

        /// <summary>
        /// 获取焊缝属性的回调函数
        /// </summary>
        /// <returns></returns>
        /// <remarks><b>包含焊缝子窗口的组件需要复制此方法</b></remarks>
        protected DObject GetWeldProps()
        {
            var refPlugin = this.ReferencePlugin;
            if (refPlugin == null)
                return m_rootUIProps;
            var curScene = LoongCAD.STC.STC.Document.FocusedScene;
            if (curScene == null)
                return m_rootUIProps;
            System.Guid pluginGuid = refPlugin.Guid;
            LoongCAD.STC.Object3D.ObjectBase[] curObjs = curScene.GetSelectedObjects(
                obj => obj is LoongCAD.STC.Object3D.ComponentBase &&
                       ((LoongCAD.STC.Object3D.ComponentBase)obj).CompGuid == pluginGuid);
            if (curObjs == null || curObjs.Length == 0)
                return m_rootUIProps;
            LoongCAD.STC.Object3D.ComponentBase comp = curObjs[0] as LoongCAD.STC.Object3D.ComponentBase;
            if (comp == null)
                return m_rootUIProps;

            return comp.UIProps;
        }

        private void btn_weld_props_Click_1(object sender, EventArgs e)
        {
            if (m_weldForm == null)
            {
                m_weldForm = new Form_WeldParameter();
                m_weldForm.Owner = this.Owner;
                m_weldForm.ApplyHandler += this.ApplyWeldProps;
                m_weldForm.ModifyHandler += this.ModifyWeldProps;
                m_weldForm.GetHandler += this.GetWeldProps;
            }

            m_weldForm.Show(m_rootUIProps, null);
        }

        private void pic_lock_select_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(pic_lock_select.SelectedIndex == 2)
            {
                pic_lower_left_horizontal.Enabled = true;
                pic_lower_left_vertical.Enabled = true;
                pic_lower_right_horizontal.Enabled = true;
                pic_lower_right_vertical.Enabled = true;
                checkBox_pic_lower_left_horizontal.Enabled = true;
                checkBox_pic_lower_left_vertical.Enabled = true;
                checkBox_pic_lower_right_horizontal.Enabled = true;
                checkBox_pic_lower_right_vertical.Enabled = true;

            }
            else
            {
                pic_lower_left_horizontal.Enabled = false;
                pic_lower_left_vertical.Enabled = false;
                pic_lower_right_horizontal.Enabled = false;
                pic_lower_right_vertical.Enabled = false;
                checkBox_pic_lower_left_horizontal.Enabled = false;
                checkBox_pic_lower_left_vertical.Enabled = false;
                checkBox_pic_lower_right_horizontal.Enabled = false;
                checkBox_pic_lower_right_vertical.Enabled = false;
            }
        }

        private void parts_lock_select_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(parts_lock_select.SelectedIndex == 2)
            {
                parts_lower_stiffener.Enabled = true;

            }
            else
            {
                parts_lower_stiffener.Enabled = false;
            }

        }
        
        /// <summary>
        /// 获取当前窗口的属性值
        /// </summary>
        /// <param name="root"></param>
        /// <param name="exclude"></param>
        /// <param name="bConsiderCheck"></param>
        /// <remarks>其他组件若同样包含类似焊缝属性窗的子窗口，则需要复制此方法</remarks>
        /// <returns></returns>
        public override DObject GetParameters(Control root = null, List<Control> exclude = null, bool bConsiderCheck = true)
        {
            DObject retProps = new DObject();
            if (!bConsiderCheck || this.act_weld_props.Checked) {
                retProps.CopyFrom(m_rootUIProps);
            }
            DObject baseProps = base.GetParameters(root, exclude, true);
            retProps.CopyFrom(baseProps);
            return retProps;
        }

        protected override void UpdatePlaceholders(DObject compProps, DObject defaultValues)
        {
            defaultValues = ParamsHandlerBase.SelectDefaultsByEmpty(compProps, defaultValues, ParamsHandler.EmptyValues);
            //base.UpdatePlaceholders(compProps, defaultValues);
            this.SetPlaceholders(defaultValues);
        }
    }
}
