﻿using System;
using System.Windows.Forms;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.Util;
using stiffeners1030.Running;
using stiffeners1030.Tool;
using TargetComponentType = LoongCAD.STC.Object3D.DetailComponent;


namespace stiffeners1030.Main
{
    public class PluginMain : ComponentPlugin
    {
        /// <summary>
        /// 重写 Guid 属性，并返回当前组件的GUID
        /// </summary>
        private readonly Guid m_nameGuid = new Guid(System.Reflection.Assembly.GetExecutingAssembly().GetName().Name);
        public override Guid Guid => m_nameGuid;
        public PluginMain() { }

        /// <summary>
        /// 定义两个成员变量
        /// <list type="bullet">
        ///     <item>
        ///         <term>m_instanceForm</term>
        ///         <description>组件设计中主要操作的面板</description>
        ///     </item>
        ///     <item>
        ///         <term>m_defaultForm</term>
        ///         <description>用于配置默认值的面板</description>
        ///     </item>
        /// </list>
        /// </summary>
        private Form_PluginMain m_propsForm = null;
        //private Form_PluginMain m_defaultForm;

        public override bool OnInstanceRun(ComponentBase compInst, long defaultId)
        {
            if (!(compInst is DetailComponent inst))
                return false;
            
            //1. 获取组件实例的属性
            DObject compProps = inst.UIProps;
            if (compProps == null)
            {
                compProps = new DObject();
            }
            //2. 初始化参数处理器，此过程包含以下操作：
            //* 初始化参数值
            //* 初始化参数空值表
            //* 初始化参数内置默认值表
            //* 组合参数默认值
            //其他代码可以直接复制该段代码
            ParamsHandler paramsHandler = new ParamsHandler();
            paramsHandler.ParamValues = compProps;
            paramsHandler.InitializeDefaultValues(this.GetDefaultsData(defaultId));
            
            var compContext = ComponentInstanceContext.CreatInstance(inst);
            if (compContext == null) {
                return false;
            }
            compContext.ParamsHandler = paramsHandler; 
            
            //具体在世界中的操作
            var runHandler = new RunningHandler();
            runHandler.Running(compContext);
            return true;
        }

        
        public override bool OnShowInstancePropsForm(ComponentBase instObj, long defId, Form ownerForm)
        {
            if (instObj == null)
                return false;
            if (m_propsForm == null) {
                m_propsForm = new Form_PluginMain(this);
                m_propsForm.Owner = ownerForm;
            }

            m_propsForm.ShowInstanceProps(instObj,
                new ParamsHandler().InitializeDefaultValues(this.GetDefaultsData(defId)));
            return true;
        }


        /// <summary>
        /// 显示组件的属性窗，必须实现此方法
        /// </summary>
        /// <param name="ownerForm">组件窗口所属主窗口</param>
        /// <returns>是否显示成功</returns>
        /// <remarks><b>其他组件可直接复制此方法</b></remarks>
        public override bool OnShowPluginPropsForm(Form ownerForm)
        {
            if (m_propsForm == null) {
                m_propsForm = new Form_PluginMain(this);
                m_propsForm.Owner = ownerForm;
            }

            m_propsForm.ShowPluginProps();
            return true;
        }
        
    }
}
