﻿using System;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using stiffeners1030.Main;
using stiffeners1030.Tool;

namespace stiffeners1030.Running
{
    
    public class BuildStiffener:RunningBaseClass
    {
        public BuildStiffener(ComponentInstanceContext context) : base(context)
        {
        }

        private ParamData AllParamData = new ParamData();

        internal class ParamData
        {
            internal double m_upperLeftHorizontal; 
            internal double m_upperLeftVertical; 
            internal double m_upperRightHorizontal; 
            internal double m_upperRightVertical; 
            internal double m_lowerLeftHorizontal; 
            internal double m_lowerLeftVertical; 
            internal double m_lowerRightHorizontal; 
            internal double m_lowerRightVertical;
            internal double m_offsetDistance;
            internal int m_picLockSelect;
            internal int m_partsLockSelect;
            internal readonly PlateProperties m_upperPro = new PlateProperties();
            internal readonly PlateProperties m_lowerPro = new PlateProperties();
            internal int m_stiffenerNum;
            internal double[] m_stiffenerSpacing;
        }

        /// <summary>
        /// 位于TopFlange的中心线上的控制点,未偏移
        /// </summary>
        private Vec3 m_upperBuildPoint;
        /// <summary>
        /// 位于BottomFlange的中心线上的控制点，未偏移
        /// </summary>
        private Vec3 m_lowerBuildPoint;
        
        private Plane m_upperPl;
        private Plane m_lowerPl;

        /// <summary>
        /// 折角数据相同
        /// </summary>
        private bool m_bSameCorner;
        /// <summary>
        /// 板数据相同
        /// </summary>
        private bool m_bSamePro;

        /// <summary>
        /// 构建加劲肋辅助类的工作坐标系X
        /// </summary>
        private Vec3 StiffenerAxisX;

        private List<SteelPlate> m_upperStiffers = new List<SteelPlate>();
        private List<SteelPlate> m_lowerStiffers = new List<SteelPlate>();


        public override bool Init()
        {
            var handler = m_context.ParamsHandler;
            AllParamData.m_upperLeftHorizontal = handler.GetDoubleValue(ParamNames.PIC_UPPER_LEFT_HORIZONTAL_CORNER_DIMENSION);
            AllParamData.m_upperLeftVertical = handler.GetDoubleValue(ParamNames.PIC_UPPER_LEFT_VERTICAL_CORNER_DIMENSION);
            AllParamData.m_upperRightHorizontal = handler.GetDoubleValue(ParamNames.PIC_UPPER_RIGHT_HORIZONTAL_CORNER_DIMENSION);
            AllParamData.m_upperRightVertical = handler.GetDoubleValue(ParamNames.PIC_UPPER_RIGHT_VERTICAL_CORNER_DIMENSION);
            AllParamData.m_lowerLeftHorizontal = handler.GetDoubleValue(ParamNames.PIC_LOWER_LEFT_HORIZONTAL_CORNER_DIMENSION);
            AllParamData.m_lowerLeftVertical = handler.GetDoubleValue(ParamNames.PIC_LOWER_LEFT_VERTICAL_CORNER_DIMENSION);
            AllParamData.m_lowerRightHorizontal = handler.GetDoubleValue(ParamNames.PIC_LOWER_RIGHT_HORIZONTAL_CORNER_DIMENSION);
            AllParamData.m_lowerRightVertical = handler.GetDoubleValue(ParamNames.PIC_LOWER_RIGHT_VERTICAL_CORNER_DIMENSION);
            AllParamData.m_picLockSelect = handler.GetIntValue(ParamNames.PIC_LOCK_SELECT);
            m_bSameCorner = AllParamData.m_picLockSelect != 2;
            AllParamData.m_partsLockSelect = handler.GetIntValue(ParamNames.PARTS_LOCK_SELECT);
            m_bSamePro= AllParamData.m_partsLockSelect != 2;
            AllParamData.m_offsetDistance = handler.GetDoubleValue(ParamNames.PARAMETERS_STIFFENER_OFFSET_DISTANCE);
            AllParamData.m_upperPro.SetProps(handler.GetDObjectValue(ParamNames.PARTS_UPPER_STIFFENER));
            AllParamData.m_lowerPro.SetProps(handler.GetDObjectValue(ParamNames.PARTS_LOWER_STIFFENER));
            AllParamData.m_stiffenerNum = handler.GetIntValue(ParamNames.OTHER_STIFFENER_NUM);
            AllParamData.m_stiffenerSpacing = handler.GetDoubleArrayValue(ParamNames.OTHER_STIFFENER_SPACING);

            AllParamData.m_stiffenerSpacing = AdaptBoltSpacing(AllParamData.m_stiffenerNum, AllParamData.m_stiffenerSpacing, 50.0d);
            //if (AllParamData.m_stiffenerSpacing.Length == 0) { AllParamData.m_stiffenerSpacing = new[] { 0.0d }; }
            return true;
        }

        public override bool Prepare()
        {
            if (!PrepareUpperLower()) return false;

            return true;
        }

        private bool PrepareUpperLower()
        {
            //主控件
            var mainComponent = m_context.TargetComp.Main;
            //A->B 的方向向量
            StiffenerAxisX = -m_context.FarToDetail;
            //用C点和A->B作为法线，确定一个平面C
            Plane planeC = new Plane(m_context.DetailPt,m_context.FarToDetail);
            Plane planeAb = new Plane(m_context.MainCtrlPtA, mainComponent.WorldMatrix.X);
            //拿到顶、底翼板面
            m_upperPl = m_context.MainPlanes.TopFlange ?? m_context.MainPlanes.TopBound;
            m_lowerPl = m_context.MainPlanes.BottomFlange ?? m_context.MainPlanes.BottomBound;
            if (m_upperPl == null) { return false; }
            if (m_lowerPl == null) { return false; }
            m_upperBuildPoint = m_upperPl.Intersect(planeAb, planeC);
            m_lowerBuildPoint = m_lowerPl.Intersect(planeAb, planeC);
            return true;
        }

        public override bool Execute()
        {
            var upperData = new StiffenerData();
            var lowerData = new StiffenerData();
            
            SetStiffenerData(AllParamData.m_upperPro,upperData);
            SetStiffenerData(m_bSamePro ? AllParamData.m_upperPro : AllParamData.m_lowerPro, lowerData);
            SetStiffenerUpperCorner(upperData);
            if (m_bSameCorner) {
                SetStiffenerUpperCorner(lowerData);
            }
            else {
                SetStiffenerLowerCorner(lowerData);
            }

            var help = new StiffenerHelper();
            
            var upperOffDir = m_upperPl.Normal.Cross(StiffenerAxisX);
            PointHelper.MoveRef(ref m_upperBuildPoint,upperOffDir,AllParamData.m_offsetDistance);
            PointHelper.MoveRef(ref m_upperBuildPoint,-StiffenerAxisX,upperData.Height);

            if (upperData.Thickness >= 0 && AllParamData.m_stiffenerNum > 0)
            {
                var sum = AllParamData.m_stiffenerSpacing.Sum();
                var buildPoint = m_upperBuildPoint;
                PointHelper.MoveRef(ref buildPoint,-upperOffDir,sum/2);
                var upperStiffer = m_context.TargetComp.CreateSteelPlate
                (help.RetAllCtrPoints(StiffenerAxisX, m_upperPl.Normal, upperData, buildPoint),
                    upperData.Thickness, Define.ObjectSide.SIDE_MIDDLE, true);
                HandlerStiffer(upperStiffer, help,AllParamData.m_upperPro, upperData.Thickness, upperOffDir,0);
                foreach (var spacing in AllParamData.m_stiffenerSpacing)
                {
                    PointHelper.MoveRef(ref buildPoint,upperOffDir,spacing);
                    upperStiffer = m_context.TargetComp.CreateSteelPlate
                    (help.RetAllCtrPoints(StiffenerAxisX, m_upperPl.Normal, upperData, buildPoint),
                        upperData.Thickness, Define.ObjectSide.SIDE_MIDDLE, true);
                    HandlerStiffer(upperStiffer, help,AllParamData.m_upperPro, upperData.Thickness, upperOffDir,0);
                }
            }

            //var lowerOffDir = m_lowerPl.Normal.Cross(StiffenerAxisX);
            var lowerOffDir = upperOffDir;
            PointHelper.MoveRef(ref m_lowerBuildPoint,lowerOffDir,AllParamData.m_offsetDistance);
            PointHelper.MoveRef(ref m_lowerBuildPoint,-StiffenerAxisX,lowerData.Height);

            var lowerPro = m_bSamePro ? AllParamData.m_upperPro : AllParamData.m_lowerPro;
            if (lowerData.Thickness >= 0 && AllParamData.m_stiffenerNum > 0)
            {
                var sum = AllParamData.m_stiffenerSpacing.Sum();
                var buildPoint = m_lowerBuildPoint;
                PointHelper.MoveRef(ref buildPoint,-lowerOffDir,sum/2);
                var lowerStiffer = m_context.TargetComp.CreateSteelPlate
                (help.RetAllCtrPoints(StiffenerAxisX, m_lowerPl.Normal, lowerData, buildPoint),
                    lowerData.Thickness, Define.ObjectSide.SIDE_MIDDLE, true);
                HandlerStiffer(lowerStiffer,help,lowerPro,lowerData.Thickness,lowerOffDir,1);
                foreach (var spacing in AllParamData.m_stiffenerSpacing)
                {
                    PointHelper.MoveRef(ref buildPoint,lowerOffDir,spacing);
                    lowerStiffer = m_context.TargetComp.CreateSteelPlate
                    (help.RetAllCtrPoints(StiffenerAxisX, m_lowerPl.Normal, lowerData, buildPoint),
                        lowerData.Thickness, Define.ObjectSide.SIDE_MIDDLE, true);
                    HandlerStiffer(lowerStiffer,help,lowerPro,lowerData.Thickness,lowerOffDir,1);
                }
            }

            return true;
        }

        /// <summary>
        /// 处理加劲肋实例
        /// </summary>
        /// <param name="stiffer">要处理的加劲肋实例</param>
        /// <param name="help">加劲肋的辅助类，负责给焊缝类传递焊缝控制点</param>
        /// <param name="platePro">对应需要的ApplyToPart的板属性类</param>
        /// <param name="thickness">厚度值</param>
        /// <param name="offDir">焊缝的偏移方向</param>
        /// <param name="weldIndex">焊缝的属性值的索引</param>
        private void HandlerStiffer(SteelPlate stiffer, StiffenerHelper help,PlateProperties platePro, double thickness, Vec3 offDir,int weldIndex)
        {
            if (stiffer != null)
            {
                platePro.ApplyToPart(stiffer);
                m_context.BuildWeld.BuildWeldForStiffer(m_context.MainPart,stiffer,help,thickness,offDir,weldIndex);
                m_upperStiffers.Add(stiffer);
            }
        }

        private void SetStiffenerLowerCorner(StiffenerData lowerData)
        {
            lowerData.TopHorizontalChamfer = AllParamData.m_lowerLeftHorizontal;
            lowerData.TopVerticalChamfer = AllParamData.m_lowerLeftVertical;
            lowerData.BottomHorizontalChamfer = AllParamData.m_lowerRightHorizontal;
            lowerData.BottomVerticalChamfer = AllParamData.m_lowerRightVertical;
        }

        private void SetStiffenerUpperCorner(StiffenerData upperData)
        {
            upperData.TopHorizontalChamfer = AllParamData.m_upperLeftHorizontal;
            upperData.TopVerticalChamfer = AllParamData.m_upperLeftVertical;
            upperData.BottomHorizontalChamfer = AllParamData.m_upperRightHorizontal;
            upperData.BottomVerticalChamfer = AllParamData.m_upperRightVertical;
        }


        private void SetStiffenerData(PlateProperties pro, StiffenerData data)
        {
            data.Thickness = pro.Thickness;
            data.Width= pro.Width;
            data.Height = pro.Height;
        }
        /// <summary>
        /// 处理间距
        /// </summary>
        /// <param name="array"></param>
        /// <param name="num"></param>
        /// <returns></returns>
        public static List<double> HandlerSpacing(double[] array, int num)
        {
            var ret = new List<double>();
            for (var i = 1; i < num; ++i)
            {
                ret.Add(array[Math.Min(i-1,array.Length-1)]);
            }
            return ret;

        }
        /// <summary>
        /// 自适应间距
        /// </summary>
        /// <param name="num"></param>
        /// <param name="space"></param>
        /// <param name="defaultSpace"></param>
        /// <returns></returns>
        public static double[] AdaptBoltSpacing(int num, double[] space, double defaultSpace)
        {
            if(num <=0)return new double[]{};
            var ret = new List<double>();
            if (space.Length == 0) {
                ret.AddRange(Enumerable.Repeat(defaultSpace,num-1));
                return ret.ToArray();
            }
            ret = HandlerSpacing(space, num);
            return ret.ToArray();
        }
    }
}