﻿using System;
using System.Collections.Generic;
using System.Linq;
using LoongCAD.Geometry;
using LoongCAD.STC;
using LoongCAD.STC.Object3D;
using LoongCAD.STC.Plugins;
using LoongCAD.UI.Component;
using stiffeners1030.Main;
using stiffeners1030.Tool;

namespace stiffeners1030.Running
{
    public class BuildWeld:RunningBaseClass
    {
        public BuildWeld(ComponentInstanceContext context) : base(context)
        {
            m_context.BuildWeld = this;
        }
        /// <summary>
        /// 存储焊缝数据
        /// </summary>
        public readonly List<WeldProperties> m_weldProps = new List<WeldProperties>();
        
        public override bool Init()
        {
            if (!InitWeld()) return true;
            return base.Init();
        }

        private bool InitWeld()
        {
            for(int i=0;i<ParamNames.WELD_PROPS_LIST.Length;++i)
            {
                m_weldProps.Add(new WeldProperties());
            }

            for (int i = 0; i < ParamNames.WELD_PROPS_LIST.Length; ++i)
            {
                var weldProps = m_context.ParamsHandler.GetDObjectValue(ParamNames.WELD_PROPS_LIST[i]);
                weldProps = WeldPropertyPanel.ParseNativeProps(weldProps);
                m_weldProps[i].SetProps(weldProps);
            }
            return true;
        }

        public void BuildWeldForStiffer(Beam mainPart, SteelPlate stiffer, StiffenerHelper help,
            double thickness, Vec3 offDir, int i)
        {
            if(stiffer == null){return;}
            if(mainPart == null){return;}
#if true
            var points = new List<Vec3>();
            var off = thickness / 2;
            //var p = PointHelper.Move(help.ControlPoint0, offDir, off);
            var p = PointHelper.Move(help.ControlPoint0, offDir, off);
            points.Add(p);
            p = PointHelper.Move(stiffer.CtrlPoints.Last().WorldPos, offDir, off);
            points.Add(p);
            if (!CheckInMainPart(points)) {
                return;
            }
            var weld = m_context.TargetComp.CreatePolyWeld(mainPart, stiffer, points.ToArray(), true);
            m_weldProps[i].ApplyToWeld(weld);
#endif
#if false
            var autoWeld = m_context.TargetComp.CreateAutoWeld(mainPart, stiffer,Define.SpaceAxis.AXIS_X, true);
            if (autoWeld == null)
            {
                Console.WriteLine(@"create auto weld fail");
                return;
            }
            var mat = new Mat43(m_context.TargetComp.WorldMatrix);
            mat.Pos = help.ControlPoint0;
            mat.Z = -help.AxisDirectionY;
            autoWeld.WorkAxis = mat;
            autoWeld.AutoWeldPosition = Define.SpaceAxis.AXIS_Z;
            m_weldProps[i].ApplyToWeld(autoWeld);
#endif
        }

        /// <summary>
        /// 检查焊缝点是否在主零件上
        /// </summary>
        /// <param name="points"></param>
        /// <exception cref="NotImplementedException"></exception>
        private bool CheckInMainPart(List<Vec3> points)
        {
            var front = m_context.MainPlanes.FrontBound;
            var back = m_context.MainPlanes.BackBound;
            foreach (var p in points)
            {
                var distF = front.SignedDist(p);
                var distB = back.SignedDist(p);
                if (distF > 0.0d || distB >= 0.0)
                {
                    return false;
                }

            }
            return true;
        }
    }
}